package jp.co.sra.gl4jun;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;

/**
 * GLjImageSource class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/08/19 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: GLjImageSource.java,v 9.9 2010/03/09 09:16:42 nisinaka Exp $
 */
public class GLjImageSource {

	protected int width;
	protected int height;
	protected Color background;
	protected Image image;

	/**
	 * Create a image source for GL rendering.
	 * 
	 * @param aWidth int
	 * @param aHeight int
	 */
	public GLjImageSource(int aWidth, int aHeight) {
		this(aWidth, aHeight, Color.white);
	}

	/**
	 * Create a image source for GL rendering.
	 * 
	 * @param aWidth int
	 * @param aHeight int
	 * @param aBackground java.awt.Color
	 */
	public GLjImageSource(int aWidth, int aHeight, Color aBackground) {
		super();
		width = aWidth;
		height = aHeight;
		background = aBackground;
	}

	/**
	 * Answer my background color.
	 * 
	 * @return java.awt.Color
	 */
	public Color getBackground() {
		return background;
	}

	/**
	 * Answer my image.
	 * 
	 * @return java.awt.Image
	 */
	public Image getImage() {
		return image;
	}

	/**
	 * Answer my size.
	 * 
	 * @return java.awt.Dimension
	 */
	public Dimension getSize() {
		return new Dimension(width, height);
	}

	/**
	 * Set my image.
	 * 
	 * @param anImage java.awt.Image
	 */
	protected void setImage(Image anImage) {
		image = anImage;
	}

}
