package jp.co.sra.jun.collections.linkedlist;

import java.io.IOException;
import java.io.Writer;

import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.system.framework.JunAbstractObject;

/**
 * JunAbstractLink class
 * 
 *  @author    nisinaka
 *  @created   2008/04/24 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAbstractLink.java,v 9.8 2010/03/09 09:16:43 nisinaka Exp $
 */
public abstract class JunAbstractLink<E, L extends JunAbstractLink<E, L>> extends JunAbstractObject {

	/** The entity object. */
	protected E entityObject = null;

	/** The link to the next entity. */
	protected L nextLink = null;

	/**
	 * Create a new instance of JunSingleLink and initialize it.
	 *
	 * @param anObject java.lang.Object
	 * @param aLink jp.co.sra.jun.collections.linkedlist.JunSingleLink
	 * @category Instance creation
	 */
	public JunAbstractLink(E anObject) {
		this.entityObject_(anObject);
	}

	/**
	 * Create a new instance of JunSingleLink and initialize it.
	 *
	 * @param anObject java.lang.Object
	 * @param aLink jp.co.sra.jun.collections.linkedlist.JunSingleLink
	 * @category Instance creation
	 */
	public JunAbstractLink(E anObject, L aLink) {
		this(anObject);
		this.nextLink_(aLink);
	}

	/**
	 * Answer the entity object.
	 * 
	 * @return java.lang.Object
	 * @category accessing
	 */
	public E entityObject() {
		return entityObject;
	}

	/**
	 * Set the entity object.
	 * 
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void entityObject_(E anObject) {
		entityObject = anObject;
	}

	/**
	 * Answer the next link.
	 * 
	 * @return jp.co.sra.jun.collections.linkedlist.JunSingleLink
	 * @category accessing
	 */
	public L nextLink() {
		return nextLink;
	}

	/**
	 * Set the next link.
	 * 
	 * @param aLink jp.co.sra.jun.collections.linkedlist.JunSingleLink
	 * @category accessing
	 */
	public void nextLink_(L aLink) {
		nextLink = aLink;
	}

	/**
	 * Print my string representation on the Writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		if (entityObject instanceof StObject) {
			((StObject) entityObject).printOn_(aWriter);
		} else {
			aWriter.write(entityObject.toString());
		}

		if (nextLink != null) {
			aWriter.write(" ");
			nextLink.printOn_(aWriter);
		}
	}

}
