package jp.co.sra.jun.collections.linkedlist;

/**
 * JunDoubleLink class
 * 
 *  @author    nisinaka
 *  @created   2002/01/21 (by nisinaka)
 *  @updated   2008/04/24 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDoubleLink.java,v 9.9 2010/03/09 09:16:43 nisinaka Exp $
 */
public class JunDoubleLink<T> extends JunAbstractLink<T, JunDoubleLink<T>> {

	/** The link to the previous entity. */
	protected JunDoubleLink<T> backLink = null;

	/**
	 * Create a new instance of JunDoubleLink and initialize it with the entity
	 * object.
	 * 
	 * @param anObject java.lang.Object
	 * @category Instance creation
	 */
	public JunDoubleLink(T anObject) {
		this(anObject, null, null);
	}

	/**
	 * Create a new instance of JunDoubleLink and initialize it with the entity
	 * object and the next link.
	 * 
	 * @param anObject java.lang.Object
	 * @param nextLink jp.co.sra.jun.collections.linkedlist.JunDoubleLink
	 * @category Instance creation
	 */
	public JunDoubleLink(T anObject, JunDoubleLink<T> nextLink) {
		this(anObject, nextLink, null);
	}

	/**
	 * Create a new instance of JunDoubleLink and initialize it with the entity
	 * object, the next link, and the back link.
	 * 
	 * @param anObject java.lang.Object
	 * @param nextLink jp.co.sra.jun.collections.linkedlist.JunDoubleLink
	 * @param backLink jp.co.sra.jun.collections.linkedlist.JunDoubleLink
	 * @category Instance creation
	 */
	public JunDoubleLink(T anObject, JunDoubleLink<T> nextLink, JunDoubleLink<T> backLink) {
		super(anObject);
		this.nextLink_(nextLink);
		this.backLink_(backLink);
	}

	/**
	 * Answer the back link.
	 * 
	 * @return jp.co.sra.jun.collections.linkedlist.JunDoubleLink
	 * @category accessing
	 */
	public JunDoubleLink<T> backLink() {
		return backLink;
	}

	/**
	 * Set the back link.
	 * 
	 * @param aLink jp.co.sra.jun.collections.linkedlist.JunDoubleLink
	 * @category accessing
	 */
	public void backLink_(JunDoubleLink<T> aLink) {
		backLink = aLink;
	}

}
