package jp.co.sra.jun.collections.linkedlist;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;

/**
 * JunDoubleLinkedList class
 * 
 *  @author    nisinaka
 *  @created   2002/01/21 (by nisinaka)
 *  @updated   2008/04/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDoubleLinkedList.java,v 9.10 2010/03/09 09:16:43 nisinaka Exp $
 */
public class JunDoubleLinkedList<E> extends JunAbstractLinkedList<E, JunDoubleLink<E>> {

	/**
	 * Add an object after the key object.
	 * 
	 * @param addObject java.lang.Object
	 * @param keyObject java.lang.Object
	 * @category adding
	 */
	public void add_after_(final E addObject, final E keyObject) {
		if (keyObject == null || firstLink.equals(lastLink) || lastLink.entityObject().equals(keyObject)) {
			this.addLast_(addObject);
			return;
		}

		Object result = this.linksDo_(new StBlockClosure1<Object, JunDoubleLink<E>>() {
			public Object value(JunDoubleLink<E> link) {
				if (link.entityObject().equals(keyObject) == false) {
					return null;
				}

				JunDoubleLink<E> newLink = new JunDoubleLink<E>(addObject, link.nextLink(), link);
				if (link.nextLink() != null) {
					link.nextLink().backLink_(newLink);
				}
				link.nextLink_(newLink);
				return newLink;
			}
		});

		if (result == null) {
			this.addLast_(addObject);
		}
	}

	/**
	 * Add an object before the key object.
	 * 
	 * @param addObject java.lang.Object
	 * @param keyObject java.lang.Object
	 * @category adding
	 */
	public void add_before_(final E addObject, final E keyObject) {
		if ((keyObject == null) || firstLink.equals(lastLink) || firstLink.entityObject().equals(keyObject)) {
			this.addFirst_(addObject);

			return;
		}

		Object result = this.linksDo_(new StBlockClosure1<Object, JunDoubleLink<E>>() {
			public Object value(JunDoubleLink<E> link) {
				if (link.entityObject().equals(keyObject) == false) {
					return null;
				}

				JunDoubleLink<E> newLink = new JunDoubleLink<E>(addObject, link, link.backLink());
				link.backLink().nextLink_(newLink);
				link.backLink_(newLink);

				return newLink;
			}
		});

		if (result == null) {
			this.addFirst_(addObject);
		}
	}

	/**
	 * Add the object as the first entity of the receiver.
	 * 
	 * @param anObject java.lang.Object
	 * @see jp.co.sra.jun.collections.linkedlist.JunAbstractLinkedList#addFirst_(java.lang.Object)
	 * @category adding
	 */
	public void addFirst_(E anObject) {
		JunDoubleLink<E> aLink = this._createLink(anObject);

		if (this.isEmpty()) {
			lastLink = aLink;
		} else {
			aLink.nextLink = firstLink;
			firstLink.backLink_(aLink);
		}

		firstLink = aLink;
	}

	/**
	 * Add the object as the last entity of the receiver.
	 * 
	 * @param anObject java.lang.Object
	 * @see jp.co.sra.jun.collections.linkedlist.JunAbstractLinkedList#addLast_(java.lang.Object)
	 * @category adding
	 */
	public void addLast_(E anObject) {
		JunDoubleLink<E> aLink = this._createLink(anObject);

		if (this.isEmpty()) {
			firstLink = aLink;
		} else {
			lastLink.nextLink_(aLink);
			aLink.backLink_(lastLink);
		}

		lastLink = aLink;
	}

	/**
	 * Remove the first object.
	 * 
	 * @return java.lang.Object
	 * @category removing
	 */
	public E removeFirst() {
		this.emptyCheck();

		JunDoubleLink<E> oldLink = firstLink;

		if (firstLink == lastLink) {
			firstLink = null;
			lastLink = null;
		} else {
			firstLink = oldLink.nextLink();
			oldLink.nextLink().backLink_(null);
		}

		oldLink.nextLink_(null);
		oldLink.backLink_(null);

		return oldLink.entityObject();
	}

	/**
	 * Remove the last object.
	 * 
	 * @return java.lang.Object
	 * @category removing
	 */
	public Object removeLast() {
		this.emptyCheck();

		JunDoubleLink<E> oldLink = lastLink;

		if (firstLink == lastLink) {
			firstLink = null;
			lastLink = null;
		} else {
			lastLink = oldLink.backLink();
			oldLink.backLink().nextLink_(null);
		}

		oldLink.nextLink_(null);
		oldLink.backLink_(null);

		return oldLink.entityObject();
	}

	/**
	 * Remove the specified object. If the object does not exist, the exception
	 * block will be evaluated.
	 * 
	 * @param anObject java.lang.Object
	 * @param exceptionBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category removing
	 */
	public Object remove_ifAbsent_(Object anObject, StBlockClosure0<?> exceptionBlock) {
		if (this.isEmpty()) {
			return exceptionBlock.value();
		}

		JunDoubleLink<E> curLink = null;

		if (firstLink.entityObject().equals(anObject)) {
			curLink = firstLink;
			firstLink = curLink.nextLink();

			if (curLink.nextLink() != null) {
				curLink.nextLink().backLink_(null);
			}
			if (curLink == lastLink) {
				lastLink = null;
			}
		} else {
			JunDoubleLink<E> preLink = firstLink;
			curLink = preLink.nextLink();
			if (curLink == null) {
				return exceptionBlock.value();
			}

			while (curLink.entityObject().equals(anObject) == false) {
				preLink = preLink.nextLink();
				curLink = curLink.nextLink();

				if (curLink == null) {
					return exceptionBlock.value();
				}
			}
			preLink.nextLink_(curLink.nextLink());

			if (curLink.nextLink() != null) {
				curLink.nextLink().backLink_(preLink);
			}
			if (curLink == lastLink) {
				lastLink = preLink;
			}
		}

		curLink.nextLink_(null);
		curLink.backLink_(null);

		return curLink.entityObject();
	}

	/**
	 * Enumerate the objects in reverse order and evaluate with the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public Object reverseDo_(StBlockClosure1<Object, E> aBlock) {
		JunDoubleLink<E> aLink = lastLink;

		while (aLink != null) {
			Object result = aBlock.value(aLink.entityObject());
			if (result != null) {
				return result;
			}

			aLink = aLink.backLink();
		}

		return null;
	}

	/**
	 * Enumerate all the links and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category private
	 */
	protected Object reverseLinksDo_(StBlockClosure1<Object, JunDoubleLink<E>> aBlock) {
		JunDoubleLink<E> aLink = lastLink;

		while (aLink != null) {
			Object result = aBlock.value(aLink);
			if (result != null) {
				return result;
			}

			aLink = aLink.backLink();
		}

		return null;
	}

	/**
	 * Create a link.
	 * 
	 * @param anObject java.lang.Object
	 * @return jp.co.sra.jun.collections.linkedlist.JunAbstractLink
	 * @see jp.co.sra.jun.collections.linkedlist.JunAbstractLinkedList#_createLink(java.lang.Object)
	 * @category private
	 */
	@Override
	protected JunDoubleLink<E> _createLink(E anObject) {
		return new JunDoubleLink<E>(anObject);
	}

}
