package jp.co.sra.jun.collections.sequences;

import java.awt.Color;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StColorValue;
import jp.co.sra.smalltalk.StDisplayable;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;

import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.system.generics.JunTrioObject;

/**
 * JunDoubleMatrix class
 * 
 *  @author    nisinaka
 *  @created   1998/10/09 (by nisinaka)
 *  @updated   2002/11/01 (by nisinaka)
 *  @updated   2007/05/09 (by m-asada)
 *  @updated   2008/04/28 (by nisinaka)
 *  @updated   2009/03/19 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDoubleMatrix.java,v 9.15 2010/03/09 09:16:43 nisinaka Exp $
 */
public class JunDoubleMatrix extends JunMatrix<Double> {

	protected static StSymbol DefalutInverseCalculationMethod = $("GaussElimination");

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param squareSize int
	 * @category Instance creation
	 */
	public JunDoubleMatrix(int squareSize) {
		super(squareSize);
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param squareSize int
	 * @param collection java.lang.Object[]
	 * @category Instance creation
	 */
	public JunDoubleMatrix(int squareSize, double[] collection) {
		super(squareSize);
		this.collection_(collection);
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param rowSize int
	 * @param columnSize int
	 * @category Instance creation
	 */
	public JunDoubleMatrix(int rowSize, int columnSize) {
		super(rowSize, columnSize);
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param rowSize int
	 * @param columnSize int
	 * @param collection double[]
	 * @category Instance creation
	 */
	public JunDoubleMatrix(int rowSize, int columnSize, double[] collection) {
		super(rowSize, columnSize);
		this.collection_(collection);
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param anImage jp.co.sra.smalltalk.StDisplayable
	 * @category Instance creation
	 */
	public JunDoubleMatrix(StDisplayable anImage) {
		this(anImage, new StBlockClosure1<Double, Color>() {
			public Double value(Color color) {
				return StColorValue._GetLuminance(color);
			}
		});
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param anImage jp.co.sra.smalltalk.StDisplayable
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category Instance creation
	 */
	public JunDoubleMatrix(StDisplayable anImage, StBlockClosure1<Double, Color> aBlock) {
		this(anImage.bounds().width, anImage.bounds().height);
		StImage theImage = anImage.asImage();
		int width = theImage.width();
		int height = theImage.height();
		for (int y = 0; y < width; y++) {
			for (int x = 0; x < height; x++) {
				Color aColor = new Color(theImage.getPixel(x, y));
				this.row_column_put_(x, y, aBlock.value(aColor));
			}
		}
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunDoubleMatrix(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of <code>JunDoubleMatrix</code> and initialize it.
	 * 
	 * @param anImage jp.co.sra.smalltalk.StDisplayable
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category Instance creation
	 * @deprecated since Jun701, use Generics.
	 */
	public JunDoubleMatrix(StDisplayable anImage, StBlockClosure<?, ?, ?, ?> aBlock) {
		this(anImage.bounds().width, anImage.bounds().height);
		StImage theImage = anImage.asImage();
		int width = theImage.width();
		int height = theImage.height();
		for (int y = 0; y < width; y++) {
			for (int x = 0; x < height; x++) {
				Color aColor = new Color(theImage.getPixel(x, y));
				Double aValue = (Double) aBlock.value_(aColor);
				this.row_column_put_(x, y, aValue);
			}
		}
	}

	/**
	 * Create a new unit matrix.
	 * 
	 * @param aClass java.lang.Class
	 * @param squareSize int
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @category Instance creation
	 */
	public static JunDoubleMatrix Unit_(Class<JunDoubleMatrix> aClass, int squareSize) {
		JunDoubleMatrix unitMatrix = _New(aClass, squareSize);
		for (int i = 0; i < squareSize; i++) {
			for (int j = 0; j < squareSize; j++) {
				unitMatrix._put(i, j, new Double((i == j) ? 1 : 0));
			}
		}
		return unitMatrix;
	}

	/**
	 * Set the default inverse calculation method to 'Gauss Elimination'.
	 * 
	 * @category Inverse calculation
	 */
	public static void CalculateInverseByGaussElimination() {
		DefaultInverseCalculationMethod_($("GaussElimination"));
	}

	/**
	 * Set the default inverse calculation method to 'LU Decomposition'.
	 * 
	 * @category Inverse calculation
	 */
	public static void CalculateInverseByLuDecomposition() {
		DefaultInverseCalculationMethod_($("LuDecomposition"));
	}

	/**
	 * Answer the current default method for calculating inverse.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category Defaults
	 */
	public static StSymbol DefaultInverseCalculationMethod() {
		return DefalutInverseCalculationMethod;
	}

	/**
	 * Set the new default method for calculating inverse.
	 * 
	 * @param newDefalutInverseCalculationMethod jp.co.sra.smalltalk.StSymbol
	 * @category Defaults
	 */
	protected static void DefaultInverseCalculationMethod_(StSymbol newDefalutInverseCalculationMethod) {
		DefalutInverseCalculationMethod = newDefalutInverseCalculationMethod;
	}

	/**
	 * Answer the value at the specified place as double.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @return double
	 * @category accessing
	 * @deprecated since Jun701, use _at(int, int).
	 */
	public double _doubleValueAt(int rowNumber, int columnNumber) {
		return this._at(rowNumber, columnNumber);
	}

	/**
	 * Put a double value at the specified place.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @param value double
	 * @category accessing
	 * @deprecated since Jun701, use _put(int, int, double).
	 */
	public void _doubleValuePut(int rowNumber, int columnNumber, double value) {
		super._put(rowNumber, columnNumber, value);
	}

	/**
	 * Answer the array of objects in the specified row.
	 * 
	 * @param index int
	 * @return double[]
	 * @category accessing
	 * @deprecated since Jun701, use atRowAsDoubles_(int)
	 */
	public double[] _doubleValuesBasicAt_(int index) {
		Object[] values = this.atRow_(index);
		double[] doubleValues = new double[values.length];
		for (int i = 0; i < values.length; i++) {
			doubleValues[i] = ((Double) values[i]).doubleValue();
		}
		return doubleValues;
	}

	/**
	 * Set the array of objects in the specified row.
	 * 
	 * @param anInteger int
	 * @param aCollection double[]
	 * @category accessing
	 * @deprecated since Jun701, use atRow_put_(int, Double[]) or atRow_put_(int, double[])
	 */
	public void basicAt_put_(int index, double[] aCollection) {
		Double[] doubleValues = new Double[aCollection.length];
		for (int i = 0; i < aCollection.length; i++) {
			doubleValues[i] = new Double(aCollection[i]);
		}
		this.atRow_put_(index, doubleValues);
	}

	/**
	 * Set all elements of the matrix from 'collection'.
	 * 
	 * @param collection double[]
	 * @category accessing
	 */
	public void collection_(double[] collection) {
		int index = 0;
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				this._put(i, j, collection[index]);
				index++;
			}
		}
	}

	/**
	 * Answer the array of double in the specified row.
	 * 
	 * @param anInteger int
	 * @return double[]
	 * @category accessing row
	 */
	public double[] atRowAsDoubles_(int index) {
		Object[] anArray = this.atRow_(index);
		double[] doubles = new double[anArray.length];
		for (int i = 0; i < doubles.length; i++) {
			doubles[i] = (Double) anArray[i];
		}
		return doubles;
	}

	/**
	 * Set the array of double in the specified row.
	 * 
	 * @param anInteger int
	 * @param doubles double[]
	 * @category accessing row
	 */
	public void atRow_put_(int index, double[] doubles) {
		Double[] anArray = new Double[doubles.length];
		for (int i = 0; i < anArray.length; i++) {
			anArray[i] = doubles[i];
		}
		this.atRow_put_(index, anArray);
	}

	/**
	 * Convert this matrix as an array of double.
	 * 
	 * @return double[]
	 * @category converting
	 */
	public double[] asArrayOfDouble() {
		double[] anArray = new double[rowSize * columnSize];
		int index = 0;
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				anArray[index] = this._doubleValueAt(i, j);
				index++;
			}
		}
		return anArray;
	}

	/**
	 * Convert the receiver to an image as <code>StImage</code>.
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 */
	public StImage asImage() {
		return this.asImageColorBlock_(new StBlockClosure1<Color, Double>() {
			public Color value(Double value) {
				return StColorValue.Brightness_(Math.max(0, Math.min(((Number) value).doubleValue(), 1)));
			}
		});
	}

	/**
	 * Convert the receiver to an image as <code>StImage</code> with the specified color block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 */
	public StImage asImageColorBlock_(final StBlockClosure1<Color, Double> aBlock) {
		final StImage anImage = new StImage(this.rowSize(), this.columnSize());
		this.doIJ_(new StBlockClosure3<Object, Double, Integer, Integer>() {
			public Object value(Double value, Integer row, Integer column) {
				Color colorValue = aBlock.value(value);
				anImage.setPixel(row, column, colorValue.getRGB());
				return null;
			}
		});
		return anImage;
	}

	/**
	 * Convert the receiver to an image as <code>StImage</code> with the specified color block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 * @deprecated since Jun701, use Generics.
	 */
	public StImage asImageColorBlock_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		final StImage anImage = new StImage(this.rowSize(), this.columnSize());
		this.doIJ_(new StBlockClosure<Object, Object, Object, Object>() {
			public Object value_value_value_(Object value, Object row, Object column) {
				Color colorValue = (Color) aBlock.value_(value);
				anImage.setPixel(((Number) row).intValue(), ((Number) column).intValue(), colorValue.getRGB());
				return null;
			}
		});
		return anImage;
	}

	/**
	 * Answer the sum of this matrix and aMatrix. This method corresponds to
	 * the '+' method in Smalltalk.
	 * 
	 * @param aMatrix jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @category matrix functions
	 */
	public JunDoubleMatrix _plus(JunDoubleMatrix aMatrix) {
		JunDoubleMatrix newMatrix = (JunDoubleMatrix) this.copy();
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix._put(i, j, (newMatrix._at(i, j) + aMatrix._at(i, j)));
			}
		}
		return newMatrix;
	}

	/**
	 * Calculate a product element at i & j with aMatrix.
	 * 
	 * @param i int
	 * @param j int
	 * @param aMatrix jp.co.sra.jun.collections.sequences.JunMatrix
	 * @return java.lang.Object
	 * @see jp.co.sra.jun.collections.sequences.JunMatrix#_calculateProductElementAt(int, int, jp.co.sra.jun.collections.sequences.JunMatrix)
	 * @category matrix functions
	 */
	protected Double _calculateProductElementAt(int i, final int j, final JunMatrix<Double> aMatrix) {
		final StValueHolder<Double> answerHolder = new StValueHolder<Double>(0.0);
		StBlockClosure2<Void, Double, Integer> aBlock = new StBlockClosure2<Void, Double, Integer>() {
			public Void value(Double value, Integer k) {
				double answer = answerHolder.value();
				double otherValue = aMatrix._at(k, j);
				answer = answer + (value * otherValue);
				answerHolder.value_(answer);
				return null;
			}
		};
		this.rowIndex_doJ_(i, aBlock);
		return answerHolder.value();
	}

	/**
	 * Answer the inverse of this matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @category matrix functions
	 */
	public JunDoubleMatrix inverse() {
		if (this.inverseCalculationMethod() == $("GaussElimination")) {
			return this.inverseByGaussElimination();
		} else if (this.inverseCalculationMethod() == $("LuDecomposition")) {
			return this.inverseByLuDecomposition();
		}
		return this.inverseByGaussElimination();
	}

	/**
	 * Calculate the inverse of this matrix by Gauss Elimination method.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category matrix functions
	 */
	public JunDoubleMatrix inverseByGaussElimination() {
		if (!this.isSquare()) {
			throw SmalltalkException.Error("the matrix is not square.");
		}

		final JunDoubleMatrix copy = (JunDoubleMatrix) this.copy();
		final JunDoubleMatrix inverse = Unit_(JunDoubleMatrix.class, rowSize);

		for (int i = 0; i < columnSize; i++) {
			double value = copy._at(i, i);
			final int _i = i;

			if (value == 0) {
				int x = i;
				while (value == 0 && x < rowSize - 1) {
					x = x + 1;
					value = copy._at(x, i);
				}

				if (value != 0) {
					value = 1 / value;
				}

				final int _x = x;
				final double _value = value;
				copy.rowIndex_doJ_(i, new StBlockClosure2<Void, Double, Integer>() {
					public Void value(Double each, Integer j) {
						double v = each + (copy._at(_x, j) * _value);
						copy._put(_i, j, v);
						return null;
					}
				});

				final JunDoubleMatrix inverse_ = inverse;
				inverse.rowIndex_doJ_(i, new StBlockClosure2<Void, Double, Integer>() {
					public Void value(Double each, Integer j) {
						double v = each + (inverse_._at(_x, j) * _value);
						inverse_._put(_i, j, v);
						return null;
					}
				});
			} else {
				final double _value = value;
				copy.rowIndex_doJ_(i, new StBlockClosure2<Void, Double, Integer>() {
					public Void value(Double each, Integer j) {
						double v = each / _value;
						copy._put(_i, j, v);
						return null;
					}
				});

				inverse.rowIndex_doJ_(i, new StBlockClosure2<Void, Double, Integer>() {
					public Void value(Double each, Integer j) {
						double v = each / _value;
						inverse._put(_i, j, v);
						return null;
					}
				});
			}

			for (int k = 0; k < rowSize; k++) {
				if (k != i) {
					final double _value = copy._at(k, i);
					final int _k = k;
					copy.rowIndex_doJ_(k, new StBlockClosure2<Void, Double, Integer>() {
						public Void value(Double each, Integer j) {
							double v = each - (copy._at(_i, j) * _value);
							copy._put(_k, j, v);
							return null;
						}
					});

					inverse.rowIndex_doJ_(k, new StBlockClosure2<Void, Double, Integer>() {
						public Void value(Double each, Integer j) {
							double v = each - (inverse._at(_i, j) * _value);
							inverse._put(_k, j, v);
							return null;
						}
					});
				}
			}
		}

		return inverse;
	}

	/**
	 * Calculate the inverse of this matrix by LU Decomposition method.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category matrix functions
	 */
	public JunDoubleMatrix inverseByLuDecomposition() {
		if (this.isSquare() == false) {
			throw SmalltalkException.Error("the matrix is not square.");
		}

		JunTrioObject<JunDoubleMatrix, int[], Double> lu = this.luDecompose();
		if (lu == null) {
			throw SmalltalkException.Error("the matrix is singular.");
		}

		JunDoubleMatrix luMatrix = lu.first();
		int[] pivotSequence = lu.second();
		int size = this.rowSize();
		JunDoubleMatrix inverse = new JunDoubleMatrix(size);
		for (int columnIndex = 0; columnIndex < size; columnIndex++) {
			for (int i = 0; i < size; i++) {
				int index = pivotSequence[i];
				double value = (index == columnIndex) ? 1 : 0;
				for (int j = 0; j < i; j++) {
					value -= (luMatrix._at(index, j) * inverse._at(j, columnIndex));
				}
				inverse._put(i, columnIndex, value);
			}
			for (int i = size - 1; i >= 0; i--) {
				int index = pivotSequence[i];
				double value = inverse._at(i, columnIndex);
				for (int j = i + 1; j < size; j++) {
					value -= (luMatrix._at(index, j) * inverse._at(j, columnIndex));
				}
				inverse._put(i, columnIndex, value / luMatrix._at(index, i));
			}
		}

		return inverse;
	}

	/**
	 * Answer the current method for calculating inverse.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category private
	 */
	protected StSymbol inverseCalculationMethod() {
		return DefaultInverseCalculationMethod();
	}

	/**
	 * Calculate the receiver's LU matrix.
	 * 
	 * @return jp.co.sra.jun.system.generics.JunTrioObjectx
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category matrix functions
	 */
	protected JunTrioObject<JunDoubleMatrix, int[], Double> luDecompose() {
		if (this.isSquare() == false) {
			throw SmalltalkException.Error("the matrix is not square.");
		}

		double value;
		double maxValue;
		int size = this.rowSize();
		double[] weights = new double[size];
		int[] pivotSequence = new int[size];
		JunDoubleMatrix luMatrix = (JunDoubleMatrix) this.copy();
		double determinant = 1;
		for (int i = 0; i < size; i++) {
			maxValue = 0;
			for (int j = 0; j < size; j++) {
				value = Math.abs(luMatrix._at(i, j));
				if (maxValue < value) {
					maxValue = value;
				}
			}
			if (maxValue == 0) {
				return null;
			}
			weights[i] = 1 / maxValue;
			pivotSequence[i] = i;
		}

		int maxIndex = -1;
		for (int columnIndex = 0; columnIndex < size; columnIndex++) {
			maxValue = -1;
			for (int i = columnIndex; i < size; i++) {
				int index = pivotSequence[i];
				value = Math.abs(luMatrix._at(index, columnIndex)) * weights[index];
				if (value > maxValue) {
					maxValue = value;
					maxIndex = i;
				}
			}
			int pivotIndex = pivotSequence[maxIndex];
			if (maxIndex != columnIndex) {
				pivotSequence[maxIndex] = pivotSequence[columnIndex];
				pivotSequence[columnIndex] = pivotIndex;
				determinant = -determinant;
			}
			double pivot = luMatrix._at(pivotIndex, columnIndex);
			if (pivot == 0) {
				return null;
			}
			determinant *= pivot;
			for (int i = columnIndex + 1; i < size; i++) {
				value = luMatrix._at(pivotSequence[i], columnIndex);
				value /= pivot;
				luMatrix._put(pivotSequence[i], columnIndex, value);
				for (int j = columnIndex + 1; j < size; j++) {
					luMatrix._put(pivotSequence[i], j, luMatrix._at(pivotSequence[i], j) - (value * luMatrix._at(pivotIndex, j)));
				}
			}
		}

		return new JunTrioObject<JunDoubleMatrix, int[], Double>(luMatrix, pivotSequence, determinant);
	}
}
