package jp.co.sra.jun.collections.sequences;

import java.awt.Point;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.geometry.abstracts.JunGeometry;
import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.system.framework.JunAbstractObject;

/**
 * JunMatrix class
 * 
 *  @author    nisinaka
 *  @created   1998/10/09 (by nisinaka)
 *  @updated   2002/07/18 (by nisinaka)
 *  @updated   2007/05/09 (by m-asada)
 *  @updated   2008/04/28 (by nisinaka)
 *  @updated   2009/03/19 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMatrix.java,v 9.14 2010/03/09 09:16:43 nisinaka Exp $
 */
public class JunMatrix<T> extends JunSequence<T> {

	/** The size of the matrix's row. */
	protected int rowSize;

	/** The size of the matrix's column. */
	protected int columnSize;

	/** The array which holds the matrix's elements. */
	protected ArrayList<T> _matrix;

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMatrix() {
		this(0);
	}

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @param squareSize int
	 * @category Instance creation
	 */
	public JunMatrix(int squareSize) {
		this(squareSize, squareSize);
	}

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @param squareSize int
	 * @param collection java.lang.Object[]
	 * @category Instance creation
	 */
	public JunMatrix(int squareSize, T[] collection) {
		this(squareSize);
		this.collection_(collection);
	}

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @param aRowSize int
	 * @param aColumnSize int
	 * @category Instance creation
	 */
	public JunMatrix(int aRowSize, int aColumnSize) {
		rowSize = aRowSize;
		columnSize = aColumnSize;

		int size = rowSize * columnSize;
		_matrix = new ArrayList<T>(size);
		for (int i = 0; i < size; i++) {
			_matrix.add(null);
		}
	}

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @param rowSize int
	 * @param columnSize int
	 * @param collection java.lang.Object[]
	 * @category Instance creation
	 */
	public JunMatrix(int rowSize, int columnSize, T[] collection) {
		this(rowSize, columnSize);
		this.collection_(collection);
	}

	/**
	 * Create a new instance of <code>JunMatrix</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunMatrix(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer the object at the position specified with rowNumber and columnNumber.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @return java.lang.Object
	 * @category accessing
	 */
	public T _at(int rowNumber, int columnNumber) {
		return _matrix.get(this._index(rowNumber, columnNumber));
	}

	/**
	 * Put 'anObject' at the position specified with 'rowNumber' and 'columnNumber'.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void _put(int rowNumber, int columnNumber, T anObject) {
		_matrix.set(this._index(rowNumber, columnNumber), anObject);
	}

	/**
	 * Answer the object at the position specified with index number.
	 * 
	 * @param indexInteger int
	 * @return java.lang.Object
	 * @category accessing
	 */
	public T at_(int indexInteger) {
		return this.atPoint_(this.computeLocation_(indexInteger));
	}

	/**
	 * Put 'anObject' at the position specified with 'aPoint'.
	 * 
	 * @param indexInteger int
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void at_put_(int indexInteger, T anObject) {
		this.atPoint_put_(this.computeLocation_(indexInteger), anObject);
	}

	/**
	 * Put 'anObject' at the all position.
	 * 
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void atAllPut_(T anObject) {
		for (int rowIndex = 0; rowIndex < this.rowSize(); rowIndex++) {
			for (int columnIndex = 0; columnIndex < this.columnSize(); columnIndex++) {
				this._put(rowIndex, columnIndex, anObject);
			}
		}
	}

	/**
	 * Answer the object at the position specified with point.
	 * 
	 * @param aPoint java.awt.Point
	 * @return java.lang.Object
	 * @category accessing
	 */
	public T atPoint_(Point aPoint) {
		return this._at(aPoint.x, aPoint.y);
	}

	/**
	 * Put 'anObject' at the position specified with 'aPoint'.
	 * 
	 * @param aPoint java.awt.Point
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void atPoint_put_(Point aPoint, T anObject) {
		this._put(aPoint.x, aPoint.y, anObject);
	}

	/**
	 * Answer the array of objects in the specified row.
	 * 
	 * @param index int
	 * @return java.lang.Object[]
	 * @category accessing
	 */
	protected Object[] basicAt_(int index) {
		return this.atRow_(index);
	}

	/**
	 * Set the array of objects in the specified row.
	 * 
	 * @param anInteger int
	 * @param aCollection java.lang.Object[]
	 * @category accessing
	 */
	protected void basicAt_put_(int index, T[] aCollection) {
		this.atRow_put_(index, aCollection);
	}

	/**
	 * Set all elements of the matrix from 'collection'.
	 * 
	 * @param collection java.lang.Object[]
	 * @category accessing
	 */
	public void collection_(T[] collection) {
		int index = 0;
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				this._put(i, j, collection[index]);
				index++;
			}
		}
	}

	/**
	 * Answer a point which represents this matrix size.
	 * 
	 * @return java.awt.Point
	 * @category accessing
	 */
	public Point matrixSize() {
		return new Point(rowSize, columnSize);
	}

	/**
	 * Answer the object at the position specified with rowNumber and columnNumber.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @return java.lang.Object
	 * @category accessing
	 */
	public Object row_column_(int rowNumber, int columnNumber) {
		return this._at(rowNumber, columnNumber);
	}

	/**
	 * Put 'anObject' at the position specified with 'rowNumber' and 'columnNumber'.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @param anObject java.lang.Object
	 * @category accessing
	 */
	public void row_column_put_(int rowNumber, int columnNumber, T anObject) {
		this._put(rowNumber, columnNumber, anObject);
	}

	/**
	 * Answer the number of all elements.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return this.columnSize() * this.rowSize();
	}

	/**
	 * Answer the array of objects in the specified column.
	 * 
	 * @param anInteger int
	 * @return java.lang.Object[]
	 * @category accessing column
	 */
	public Object[] atColumn_(int anInteger) {
		Object[] column = new Object[rowSize];
		for (int i = 0; i < rowSize; i++) {
			column[i] = this._at(i, anInteger);
		}
		return column;
	}

	/**
	 * Set the array of objects in the specified column.
	 * 
	 * @param anInteger int
	 * @param aCollection java.lang.Object[]
	 * @category accessing column
	 */
	public void atColumn_put_(int anInteger, T[] aCollection) {
		for (int rowIndex = 0; rowIndex < this.rowSize(); rowIndex++) {
			this._put(rowIndex, anInteger, aCollection[rowIndex]);
		}
	}

	/**
	 * Answer the size of column.
	 * 
	 * @return int
	 * @category accessing column
	 */
	public int columnSize() {
		return columnSize;
	}

	/**
	 * Answer the array of objects in the specified row.
	 * 
	 * @param anInteger int
	 * @return java.lang.Object[]
	 * @category accessing row
	 */
	public Object[] atRow_(int anInteger) {
		Object[] row = new Object[columnSize];
		for (int j = 0; j < columnSize; j++) {
			row[j] = this._at(anInteger, j);
		}
		return row;
	}

	/**
	 * Set the array of objects in the specified row.
	 * 
	 * @param anInteger int
	 * @param aCollection java.lang.Object[]
	 * @category accessing row
	 */
	public void atRow_put_(int anInteger, T[] aCollection) {
		for (int columnIndex = 0; columnIndex < this.columnSize(); columnIndex++) {
			this._put(anInteger, columnIndex, aCollection[columnIndex]);
		}
	}

	/**
	 * Answer the size of row.
	 * 
	 * @return int
	 * @category accessing row
	 */
	public int rowSize() {
		return rowSize;
	}

	/**
	 * Convert this matrix as an array.
	 * 
	 * @return java.lang.Object[]
	 * @category converting
	 */
	public Object[] asArray() {
		Object[] anArray = new Object[rowSize * columnSize];
		int index = 0;

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				anArray[index] = this._at(i, j);
				index++;
			}
		}

		return anArray;
	}

	/**
	 * Create an entire copy of this matrix.
	 * 
	 * @return jp.co.sra.smalltalk.StObject
	 * @see jp.co.sra.smalltalk.StObject#copy()
	 * @category copying
	 */
	public StObject copy() {
		JunMatrix<T> newMatrix = _New(this.myClass(), rowSize, columnSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix._put(i, j, this._at(i, j));
			}
		}
		return newMatrix;
	}

	/**
	 * Enumerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @see jp.co.sra.jun.collections.sequences.JunSequence#do_(jp.co.sra.smalltalk.StBlockClosure)
	 * @category enumerating
	 */
	public <R> R do_(final StBlockClosure1<R, T> aBlock) {
		return this.doIJ_(new StBlockClosure3<R, T, Integer, Integer>() {
			public R value(T v, Integer i, Integer j) {
				return aBlock.value(v);
			}
		});
	}

	/**
	 * Enumerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @see jp.co.sra.jun.collections.sequences.JunSequence#do_(jp.co.sra.smalltalk.StBlockClosure)
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object do_(final StBlockClosure<Object, Object, Object, Object> aBlock) {
		return this.doIJ_(new StBlockClosure<Object, Object, Object, Object>() {
			public Object value_value_value_(Object v, Object i, Object j) {
				return aBlock.value_(v);
			}
		});
	}

	/**
	 * Enumerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R doIJ_(StBlockClosure3<R, T, Integer, Integer> aBlock) {
		int rowSize = this.rowSize();
		int columnSize = this.columnSize();
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				R result = aBlock.value(this._at(i, j), i, j);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	/**
	 * Enumerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object doIJ_(StBlockClosure<Object, Object, Object, Object> aBlock) {
		int rowSize = this.rowSize();
		int columnSize = this.columnSize();
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				Object result = aBlock.value_value_value_(this._at(i, j), new Integer(i), new Integer(j));
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	/**
	 * Enumerate all the values at the specified row and evaluate the block.
	 * 
	 * @param anInteger int
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public Object rowIndex_doJ_(int anInteger, StBlockClosure2<? extends Object, T, Integer> aBlock) {
		for (int j = 0; j < columnSize; j++) {
			Object result = aBlock.value(this._at(anInteger, j), j);
			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Get the receiver's attributes from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @see jp.co.sra.jun.collections.sequences.JunSequence#fromLispList(jp.co.sra.jun.goodies.lisp.JunLispList)
	 * @category lisp support
	 */
	@Override
	public void fromLispList(JunLispList aList) {
		super.fromLispList(aList);
		this.matrixSizeFromLispList(aList);
		this.matrixFromLispList(aList);
	}

	/**
	 * Convert the receiver's attributes to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#toLispList()
	 * @category lisp support
	 */
	@Override
	public JunLispCons toLispList() {
		JunLispCons aList = super.toLispList();
		aList.add_(this.matrixSizeToLispList());
		aList.add_(this.matrixToLispList());
		return aList;
	}

	/**
	 * Get the receiver's matrix size from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void matrixSizeFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object car) {
				return new Boolean(car instanceof JunLispCons && ((JunLispCons) car).head() == $("matrixSize"));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}

		Point aPoint = new Jun2dPoint((JunLispList) list.tail())._toPoint();
		rowSize = aPoint.x;
		columnSize = aPoint.y;
	}

	/**
	 * Convert the receiver's matrix size to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected JunLispCons matrixSizeToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("matrixSize"));
		aList.tail_(new Jun2dPoint(this.matrixSize()).toLispList());
		return aList;
	}

	/**
	 * Get the receiver's matrix values from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	@SuppressWarnings("unchecked")
	protected void matrixFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object car) {
				return new Boolean(car instanceof JunLispCons && ((JunLispCons) car).head() == $("matrix"));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}

		Object[] items = ((JunLispList) list.tail()).asArray();
		ArrayList<T> anArray = new ArrayList<T>(items.length);
		for (int i = 0; i < items.length; i++) {
			Object each = items[i];
			if (each instanceof Number) {
				anArray.add((T) each);
			} else {
				anArray.add((T) JunGeometry.FromLispList((JunLispList) items[i]));
			}
		}
		_matrix = anArray;
	}

	/**
	 * Convert the receiver's matrix values to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	@SuppressWarnings("unchecked")
	protected JunLispCons matrixToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("matrix"));
		for (T each : _matrix) {
			if (each instanceof JunAbstractObject) {
				list.add_(((JunAbstractObject) each).toLispList());
			} else if (each instanceof JunSequence) {
				list.add_(((JunSequence) each).toLispList());
			} else {
				list.add_(each);
			}
		}
		return list;
	}

	/**
	 * Answer the receiver's horizontal flip matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @category matrix functions
	 */
	public JunMatrix<T> horizontalFlip() {
		JunMatrix<T> newMatrix = _New(this.myClass(), rowSize, columnSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix.row_column_put_(i, columnSize - j - 1, this._at(i, j));
			}
		}
		return newMatrix;
	}

	/**
	 * Answer the receiver's mirror matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @category matrix functions
	 */
	public JunMatrix<T> mirror() {
		return this.horizontalFlip();
	}

	/**
	 * Answer the receiver's transpose matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @category matrix functions
	 */
	public JunMatrix<T> transpose() {
		JunMatrix<T> newMatrix = _New(this.myClass(), columnSize, rowSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix.row_column_put_(j, i, this._at(i, j));
			}
		}
		return newMatrix;
	}

	/**
	 * Answer the receiver's upside down matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @category matrix functions
	 */
	public JunMatrix<T> upsideDown() {
		return this.horizontalFlip();
	}

	/**
	 * Answer the receiver's vertical flip matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @category matrix functions
	 */
	public JunMatrix<T> verticalFlip() {
		JunMatrix<T> newMatrix = _New(this.myClass(), rowSize, columnSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix.row_column_put_(rowSize - i - 1, j, this._at(i, j));
			}
		}
		return newMatrix;
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("Matrix (");
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				T value = this._at(i, j);
				aWriter.write((value != null) ? value.toString() : "null");
				if (j < (columnSize - 1)) {
					aWriter.write(" ");
				}
			}
			if (i < (rowSize - 1)) {
				aWriter.write(" , ");
			}
		}
		aWriter.write(")");
	}

	/**
	 * Answer true if this matrix is square, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isSquare() {
		return this.rowSize() == this.columnSize();
	}

	/**
	 * Calculate the product of this matrix and aMatrix.
	 * 
	 * @param aMatrix jp.co.sra.jun.collections.sequences.JunMatrix
	 * @return jp.co.sra.jun.collections.sequences.JunMatrix
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category matrix functions
	 */
	public JunMatrix<T> product_(JunMatrix<T> aMatrix) {
		if (columnSize != aMatrix.rowSize) {
			throw SmalltalkException.Error("my column size is not equal to your row size.");
		}

		JunMatrix<T> newMatrix = _New(this.myClass(), rowSize, aMatrix.columnSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < aMatrix.columnSize; j++) {
				newMatrix._put(i, j, _calculateProductElementAt(i, j, aMatrix));
			}
		}
		return newMatrix;
	}

	/**
	 * Calculate a product element at i & j with aMatrix.
	 * 
	 * @param i int
	 * @param j int
	 * @param aMatrix jp.co.sra.jun.collections.sequences.JunMatrix
	 * @return java.lang.Object
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category private
	 */
	protected T _calculateProductElementAt(int i, int j, JunMatrix<T> aMatrix) {
		throw new SmalltalkException("don't know how to calculate a product element.");
	}

	/**
	 * Compute location with the specified index.
	 * 
	 * @param anInteger int
	 * @return java.awt.Point
	 * @category private
	 */
	protected Point computeLocation_(int anInteger) {
		return new Point(anInteger / this.columnSize(), anInteger % this.columnSize());
	}

	/**
	 * Answer the array list index of the specified position.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @return int
	 * @category private
	 */
	protected int _index(int rowNumber, int columnNumber) {
		return rowNumber + columnNumber * rowSize;
	}

	/**
	 * Answer my class.
	 * 
	 * @return java.lang.Class
	 * @category private
	 */
	@SuppressWarnings("unchecked")
	private Class<JunMatrix<T>> myClass() {
		return (Class<JunMatrix<T>>) this.getClass();
	}

}
