package jp.co.sra.jun.collections.sequences;

import java.awt.Frame;
import java.awt.TextArea;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StObject;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.goodies.lisp.JunLispNil;

/**
 * JunSequence class
 * 
 *  @author    nisinaka
 *  @created   2002/07/18 (by nisinaka)
 *  @updated   2008/04 24 (by nisinaka)
 *  @updated   2009/03/19 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSequence.java,v 9.11 2010/03/09 09:16:43 nisinaka Exp $
 */
public abstract class JunSequence<T> extends StObject {
	/**
	 * Show a window which displays the specified Lisp List.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Lisp support
	 */
	public static JunLispList ShowLispList_(JunLispList aList) {
		TextArea textArea = new TextArea(5, 40);
		textArea.setEditable(false);
		textArea.append(aList.saveString());

		Frame aFrame = new Frame();
		aFrame.add("Center", textArea);
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
		aFrame.setTitle($String("Lisp List (S expression)"));
		aFrame.pack();
		aFrame.setSize(400, 300);
		aFrame.setVisible(true);

		return aList;
	}

	/**
	 * Create a new instance of <code>JunSphereSurface</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunSequence() {
		super();
	}

	/**
	 * Create a new instance of <code>JunSphereSurface</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunSequence(JunLispList aList) {
		this();
		this.fromLispList(aList);
	}

	/**
	 * Enumerate the objects and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category enumerating
	 */
	public Object detect_(StBlockClosure1<Boolean, T> aBlock) {
		return this.detect_ifNone_(aBlock, new StBlockClosure0<Object>() {
			public Object value() {
				throw SmalltalkException.Error("Element not Found");
			}
		});
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Answer the first element for which aBlock evaluates to true.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param exceptionBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public Object detect_ifNone_(final StBlockClosure1<Boolean, T> aBlock, StBlockClosure0<Object> exceptionBlock) {
		Object result = this.do_(new StBlockClosure1<Object, T>() {
			public Object value(T each) {
				return aBlock.value(each) ? each : null;
			}
		});
		return (result == null) ? exceptionBlock.value() : result;
	}

	/**
	 * Enumerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public abstract <R> R do_(final StBlockClosure1<R, T> aBlock);

	/**
	 * Get the receiver's attributes from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public void fromLispList(JunLispList aList) {
		// Nothing to do by default.
	}

	/**
	 * Answer the kind name of the element.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category lisp support
	 */
	protected StSymbol kindName() {
		return this._className();
	}

	/**
	 * Answer a new lisp cons cell.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons lispCons() {
		return JunLispCons.Cell();
	}

	/**
	 * Create an instance of JunLispNil.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispNil
	 * @category lisp support
	 */
	protected JunLispNil lispNil() {
		return JunLispList.NullList();
	}

	/**
	 * Show a window which displays the Lisp List of the receiver.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList showLispList() {
		return this.showLispList_(this.toLispList());
	}

	/**
	 * Show a window which displays the specified Lisp List.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList showLispList_(JunLispList aList) {
		return ShowLispList_(aList);
	}

	/**
	 * Convert the receiver's attributes to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispCons toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());
		return list;
	}
}
