package jp.co.sra.jun.csg.abstracts;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StInterval;

import jp.co.sra.jun.csg.atoms.JunTransformedCSG;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.boundaries.JunBorderGenerator;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.octree.basic.JunOctree;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.solid.abstracts.JunSolidModel;
import jp.co.sra.jun.topology.elements.JunBody;

/**
 * JunCSG class
 * 
 *  @author    nisinaka
 *  @created   1998/10/18 (by nisinaka)
 *  @updated   2008/04/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCSG.java,v 9.10 2010/03/09 09:16:45 nisinaka Exp $
 */
public abstract class JunCSG extends JunSolidModel {

	/**
	 * Convert to a JunBody.
	 * 
	 * @param xInterval jp.co.sra.smalltalk.StInterval
	 * @param yInterval jp.co.sra.smalltalk.StInterval
	 * @param zInterval jp.co.sra.smalltalk.StInterval
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 */
	public JunBody asJunBodyXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		return JunBorderGenerator.ContourBodyFrom_at_sign_xInterval_yInterval_zInterval_interim_(new StBlockClosure3<Double, Double, Double, Double>() {
			public Double value(Double x, Double y, Double z) {
				return JunCSG.this.valueAt_(new Jun3dPoint(x, y, z));
			}
		}, 0, 1, xInterval, yInterval, zInterval, null);
	}

	/**
	 * Convert as a boxel octree with the specified conditions.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 */
	public JunOctree asJunBoxelOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		final JunCSG self = this;
		octree.block_depth_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint point) {
				return self.valueAt_(point);
			}
		}, anInteger);

		return octree;
	}

	/**
	 * Convert as a b-reps octree with the specified conditions.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 */
	public JunOctree asJunBrepsOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		final JunCSG self = this;
		octree.brepsBlock_depth_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint point) {
				return self.valueAt_(point);
			}
		}, anInteger);

		return octree;
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @param xInterval jp.co.sra.smalltalk.StInterval
	 * @param yInterval jp.co.sra.smalltalk.StInterval
	 * @param zInterval jp.co.sra.smalltalk.StInterval
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		return this.asJunBodyXInterval_yInterval_zInterval_(xInterval, yInterval, zInterval).asJunOpenGL3dObject();
	}

	/**
	 * Answer true if the receiver contains the specified point, otherwise
	 * false.
	 * 
	 * @param a3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return boolean
	 */
	public boolean containsPoint_(Jun3dPoint a3dPoint) {
		return this.valueAt_(a3dPoint) >= 0;
	}

	/**
	 * Answer the JunCSG which is applied the specified transformation.
	 * 
	 * @param aJun3dTransformation
	 *        jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return jp.co.sra.jun.csg.abstracts.JunCSG
	 */
	public JunCSG transform_(Jun3dTransformation aJun3dTransformation) {
		return JunTransformedCSG.On_transform_(this, aJun3dTransformation);
	}

}
