package jp.co.sra.jun.delaunay.twoD;

import java.awt.Graphics;

import jp.co.sra.jun.system.framework.JunAbstractObject;

/**
 * Jun2dDelaunayElement class
 * 
 *  @author    Ryouichi Matsuda
 *  @created   2002/01/14 (by Ryouichi Matsuda)
 *  @updated   2008/04/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun697 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dDelaunayElement.java,v 9.9 2010/03/09 09:16:48 nisinaka Exp $
 */
public abstract class Jun2dDelaunayElement<E extends Jun2dDelaunayElement<E>> extends JunAbstractObject {

	// 1.0d-8
	public static final double DECIMAL_8 = 0.00000001d;
	// 1.0d-12
	public static final double DECIMAL_12 = 0.000000000001d;
	protected E next;
	protected E prev;

	/**
	 * Display the receiver on the specified graphic context.
	 * 
	 * @param aGraphicContext java.awt.Graphics
	 * @category display
	 */
	public void displayOn_(Graphics aGraphicContext) {
		// do nothing.
	}

	/**
	 * display.
	 * 
	 * @category private
	 */
	protected void display() {
		// should not implement...
	}

	/**
	 * Get next element.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category private
	 */
	protected E next() {
		return next;
	}

	/**
	 * Get previous element.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category private
	 */
	protected E prev() {
		return prev;
	}

	/**
	 * Removed from List.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category private
	 */
	protected Jun2dDelaunayElement<E> removedFromList() {
		prev.setNext_(next);
		next.setPrev_(prev);
		next = null;
		prev = null;

		return this;
	}

	/**
	 * Set next element.
	 * 
	 * @param aJun2dDelaunayElement jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category private
	 */
	protected void setNext_(E aJun2dDelaunayElement) {
		next = aJun2dDelaunayElement;
	}

	/**
	 * Set previous element.
	 * 
	 * @param aJun2dDelaunayElement jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category private
	 */
	protected void setPrev_(E aJun2dDelaunayElement) {
		prev = aJun2dDelaunayElement;
	}
}
