package jp.co.sra.jun.delaunay.twoD;

import java.awt.Graphics;
import java.io.IOException;
import java.io.Writer;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.system.framework.JunAbstractObject;

/**
 * Jun2dDelaunayHalfEdge class
 * 
 *  @author    Ryouichi Matsuda
 *  @created   2002/01/14 (by Ryouichi Matsuda)
 *  @updated   2008/04/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun697 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dDelaunayHalfEdge.java,v 9.9 2010/03/09 09:16:48 nisinaka Exp $
 */
public class Jun2dDelaunayHalfEdge extends JunAbstractObject {

	protected Jun2dDelaunayLoop loop;
	protected Jun2dDelaunayVertex vertex;
	protected Jun2dDelaunayEdge edge;
	protected Jun2dDelaunayHalfEdge pair;
	protected Jun2dDelaunayHalfEdge next;
	protected Jun2dDelaunayHalfEdge prev;

	/**
	 * Answer my current edge.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 * @category accessing
	 */
	public Jun2dDelaunayEdge edge() {
		return edge;
	}

	/**
	 * Answer my current loop.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 * @category accessing
	 */
	public Jun2dDelaunayLoop loop() {
		return loop;
	}

	/**
	 * Set my new loop.
	 * 
	 * @param aJun2dDelaunayLoop jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 * @category accessing
	 */
	public void loop_(Jun2dDelaunayLoop aJun2dDelaunayLoop) {
		loop = aJun2dDelaunayLoop;
	}

	/**
	 * Answer my current next half edge.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category accessing
	 */
	public Jun2dDelaunayHalfEdge next() {
		return next;
	}

	/**
	 * Set my new next half edge.
	 * 
	 * @param aJun2dDelaunayHalfEdge jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category accessing
	 */
	public void next_(Jun2dDelaunayHalfEdge aJun2dDelaunayHalfEdge) {
		this.setNext_(aJun2dDelaunayHalfEdge);
		aJun2dDelaunayHalfEdge.setPrev_(this);
	}

	/**
	 * Answer the pair of my half edge.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category accessing
	 */
	public Jun2dDelaunayHalfEdge pair() {
		return pair;
	}

	/**
	 * Answer my current previous half edge.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category accessing
	 */
	public Jun2dDelaunayHalfEdge prev() {
		return prev;
	}

	/**
	 * Answer my current vertex.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @category accessing
	 */
	public Jun2dDelaunayVertex vertex() {
		return vertex;
	}

	/**
	 * Set my new vertex.
	 * 
	 * @param aJun2dDelaunayVertex jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @category accessing
	 */
	public void vertex_(Jun2dDelaunayVertex aJun2dDelaunayVertex) {
		vertex = aJun2dDelaunayVertex;
	}

	/**
	 * Display the receiver on the graphics.
	 * 
	 * @param aGraphicsContext java.awt.Graphics
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphicsContext) {
		aGraphicsContext.drawLine((int) pair.vertex().x(), (int) pair.vertex().y(), (int) vertex.x(), (int) vertex.y());
	}

	/**
	 * Enumerate the half edges and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R do_(StBlockClosure1<R, Jun2dDelaunayHalfEdge> aBlock) {
		Jun2dDelaunayHalfEdge halfEdge;
		Jun2dDelaunayHalfEdge nextHalfEdge;

		halfEdge = this;
		do {
			R returnObject;

			nextHalfEdge = halfEdge.next();
			returnObject = aBlock.value(halfEdge);
			if (returnObject != null) {
				return returnObject;
			}

			halfEdge = nextHalfEdge;
		} while (halfEdge != this);

		return null;
	}

	/**
	 * Enumerate the half edges and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public <R> R do_(final StBlockClosure<R, ?, ?, ?> aBlock) {
		return this.do_(new StBlockClosure1<R, Jun2dDelaunayHalfEdge>() {
			public R value(Jun2dDelaunayHalfEdge edge) {
				return aBlock.value_(edge);
			}
		});
	}

	/**
	 * Calculate the area made with the specified vertex.
	 * 
	 * @param aJun2dDelaunayVertex jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @return double
	 * @category functions
	 */
	public double areaWith_(Jun2dDelaunayVertex aJun2dDelaunayVertex) {
		return this.pair().vertex().areaWith_with_(this.vertex(), aJun2dDelaunayVertex);
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException DOCUMENT ME!
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("HalfEdge(");
		aWriter.write(Double.toString(pair.vertex().x()));
		aWriter.write("@");
		aWriter.write(Double.toString(pair.vertex().y()));
		aWriter.write(" ");
		aWriter.write(Double.toString(this.vertex().x()));
		aWriter.write("@");
		aWriter.write(Double.toString(this.vertex().y()));
		aWriter.write(")");
	}

	/**
	 * Answer true if the receiver intersects with the specified line segment, otherwise false.
	 * 
	 * @param aJun2dDelaunayVertex1 jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param aJun2dDelaunayVertex2 jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @return boolean
	 * @category testing
	 */
	public boolean intersectsWithLineSegmentFrom_to_(Jun2dDelaunayVertex aJun2dDelaunayVertex1, Jun2dDelaunayVertex aJun2dDelaunayVertex2) {
		Jun2dDelaunayVertex myV1;
		Jun2dDelaunayVertex myV2;
		Jun2dDelaunayVertex yourV1;
		Jun2dDelaunayVertex yourV2;

		myV1 = this.vertex();
		myV2 = this.pair().vertex();
		yourV1 = aJun2dDelaunayVertex1;
		yourV2 = aJun2dDelaunayVertex2;

		return (_Sign_(myV1.areaWith_with_(yourV1, yourV2)) != _Sign_(myV2.areaWith_with_(yourV1, yourV2))) && (_Sign_(yourV1.areaWith_with_(myV1, myV2)) != _Sign_(yourV2.areaWith_with_(myV1, myV2)));
	}

	/**
	 * Set my edge.
	 * 
	 * @param aJun2dDelaunayEdge jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 * @category private
	 */
	protected void setEdge_(Jun2dDelaunayEdge aJun2dDelaunayEdge) {
		edge = aJun2dDelaunayEdge;
	}

	/**
	 * Set my next half edge.
	 * 
	 * @param aJun2dDelaunayHalfEdge jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category private
	 */
	protected void setNext_(Jun2dDelaunayHalfEdge aJun2dDelaunayHalfEdge) {
		next = aJun2dDelaunayHalfEdge;
	}

	/**
	 * Set my pair of my half edge.
	 * 
	 * @param aJun2dDelaunayHalfEdge jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category private
	 */
	protected void setPair_(Jun2dDelaunayHalfEdge aJun2dDelaunayHalfEdge) {
		pair = aJun2dDelaunayHalfEdge;
	}

	/**
	 * Set my previous half edge.
	 * 
	 * @param aJun2dDelaunayHalfEdge jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 * @category private
	 */
	protected void setPrev_(Jun2dDelaunayHalfEdge aJun2dDelaunayHalfEdge) {
		prev = aJun2dDelaunayHalfEdge;
	}

	/**
	 * Answer the sign of the value.
	 * 
	 * @param value double
	 * @return int
	 * @category Private
	 */
	protected static int _Sign_(double value) {
		if (value < 0) {
			return -1;
		}

		if (0 < value) {
			return 1;
		}

		return 0;
	}

}
