package jp.co.sra.jun.delaunay.twoD;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;

/**
 * Jun2dDelaunayList class
 * 
 *  @author    Ryouichi Matsuda
 *  @created   2002/01/14 (by Ryouichi Matsuda)
 *  @updated   2008/04 28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dDelaunayList.java,v 9.9 2010/03/09 09:16:48 nisinaka Exp $
 */
public class Jun2dDelaunayList<E extends Jun2dDelaunayElement<E>> extends Jun2dDelaunayElement<E> {

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		next = null;
		prev = null;
	}

	/**
	 * Answer the element at the specified index.
	 * 
	 * @param anInteger int
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category accessing
	 */
	public E at_(int anInteger) {
		E element = null;
		for (int i = 0; i < anInteger; i++) {
			element = (element != null) ? element.next() : this.next();
			if (element == null) {
				throw SmalltalkException.Error("out of bounds");
			}
		}
		return element;
	}

	/**
	 * Set the element at the specified index.
	 * This method should not be called in this class.
	 * 
	 * @param anInteger int
	 * @param aJun2dDelaunayElement jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category accessing
	 */
	public void at_put_(int anInteger, Jun2dDelaunayElement<E> aJun2dDelaunayElement) {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Answer my first element.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayList
	 * @category accessing
	 */
	public E first() {
		if (next == null) {
			SmalltalkException.Error("out of bounds");
		}

		return next;
	}

	/**
	 * Answer my last element.
	 * 
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayList
	 * @category accessing
	 */
	public E last() {
		if (prev == null) {
			SmalltalkException.Error("out of bounds");
		}

		return prev;
	}

	/**
	 * Add the new element to the receiver.
	 * 
	 * @param aJun2dDelaunayElement jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category adding
	 */
	public E add_(E aJun2dDelaunayElement) {
		if (aJun2dDelaunayElement.next() != null) {
			aJun2dDelaunayElement.removedFromList();
		}

		if (prev != null) {
			prev.setNext_(aJun2dDelaunayElement);
			aJun2dDelaunayElement.setPrev_(prev);
		} else {
			this.setNext_(aJun2dDelaunayElement);
			aJun2dDelaunayElement.setPrev_(null);
		}
		aJun2dDelaunayElement.setNext_(null);
		this.setPrev_(aJun2dDelaunayElement);

		return aJun2dDelaunayElement;
	}

	/**
	 * Remove the element from the receiver.
	 * 
	 * @param aJun2dDelaunayElement jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category removing
	 */
	public Jun2dDelaunayElement<E> remove_(E aJun2dDelaunayElement) {
		return aJun2dDelaunayElement.removedFromList();
	}

	/**
	 * Evaluate the Block with each of the receiver's elements and answer the
	 * first element for which the Block evaluates to true.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.delaunay.twoD.Jun2dDelaunayElement
	 * @category enumerating
	 */
	public E detect_ifNone_(final StBlockClosure1<Boolean, E> aBlock, StBlockClosure0<E> errorBlock) {
		E o = this.do_(new StBlockClosure1<E, E>() {
			public E value(E element) {
				if (aBlock.value(element) == Boolean.TRUE) {
					return element;
				}
				return null;
			}
		});
		if (o != null) {
			return o;
		}

		return errorBlock.value();
	}

	/**
	 * Enumerate all elements of the receiver and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R do_(StBlockClosure1<R, E> aBlock) {
		E element = next;
		while (element != null) {
			E nextElement = element.next();
			R o = aBlock.value(element);
			if (o != null) {
				return o;
			}

			element = nextElement;
		}
		return null;
	}

	/**
	 * Enumerate all elements of the receiver and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public <R> R do_(final StBlockClosure<R, ?, ?, ?> aBlock) {
		return this.do_(new StBlockClosure1<R, E>() {
			public R value(E element) {
				return aBlock.value_(element);
			}
		});
	}

}
