package jp.co.sra.jun.geometry.boundaries;

import jp.co.sra.jun.geometry.abstracts.JunGeometry;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunBoundingObject class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2004/10/19 (by Mitsuhiro Asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @updated   2009/03/28 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBoundingObject.java,v 9.10 2010/03/09 09:16:50 nisinaka Exp $
 */
public abstract class JunBoundingObject extends JunGeometry {
	/**
	 * Create a new instance of <code>JunBoundingObject</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunBoundingObject() {
		super();
	}

	/**
	 * Create a new instance of <code>JunBoundingObject</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunBoundingObject(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer true if the receiver is a bounding object, otherwise false.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#isBoundingObject()
	 * @category testing
	 */
	@Override
	public boolean isBoundingObject() {
		return true;
	}

	/**
	 * Answer true if the receiver is empty, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isEmpty() {
		if (this.is3d()) {
			return this.volume() < this.accuracy();
		}
		if (this.is2d()) {
			return this.area() < this.accuracy();
		}
		return false;
	}

}
