package jp.co.sra.jun.geometry.forms;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.goodies.cursors.JunCursors;
import jp.co.sra.jun.goodies.image.framework.JunImageDisplayModel;

/**
 * JunFormTriangulationTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/06/19 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun666 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFormTriangulationTestExamples.java,v 9.9 2010/03/09 09:16:53 nisinaka Exp $
 */
public class JunFormTriangulationTestExamples extends JunForm2dRegionTestExamples {
	/**
	 * Example1: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		Point aPoint = new Point(100, 100);
		Jun2dPoint[][] polylines = ExamplePolylines();
		for (int index = 0; index < polylines.length; index++) {
			final JunFormTriangulation formTriangulation = new JunFormTriangulation(polylines[index]);
			Point extentPoint = formTriangulation.boundingBox().extent()._toPoint();
			BufferedImage aPixmap = new BufferedImage(extentPoint.x, extentPoint.y, BufferedImage.TYPE_INT_ARGB);
			Graphics graphicsContext = aPixmap.getGraphics();
			JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			try {
				cursor._show();

				graphicsContext.setColor(Color.white);
				graphicsContext.fillRect(0, 0, aPixmap.getWidth(), aPixmap.getHeight());
				formTriangulation.triangles();
				formTriangulation.displayOn_(graphicsContext);
			} finally {
				if (graphicsContext != null) {
					graphicsContext.dispose();
					graphicsContext = null;
				}
				cursor._restore();
			}
			JunImageDisplayModel.Show_at_(aPixmap, aPoint);
			aPoint.translate(20, 20);
		}

		return true;
	}

	/**
	 * Example2: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		Jun2dPoint[][] polylines = ExamplePolylines();
		for (int index = 0; index < polylines.length; index++) {
			ArrayList<Class<? extends JunAbstractFormTriangulation>> triangulationClasses = new ArrayList<Class<? extends JunAbstractFormTriangulation>>();
			triangulationClasses.add(JunFormTriangulation1.class);
			triangulationClasses.add(JunFormTriangulation2.class);
			triangulationClasses.add(JunFormTriangulation3.class);

			Iterator<Class<? extends JunAbstractFormTriangulation>> i = triangulationClasses.iterator();
			while (i.hasNext()) {
				Class<? extends JunAbstractFormTriangulation> triangulationClass = i.next();
				final JunFormTriangulation formTriangulation = new JunFormTriangulation(polylines[index], triangulationClass);
				double areaOfTriangles = formTriangulation.area();
				double areaWithSignOfTriangles = formTriangulation.areaWithSign();

				System.out.println(triangulationClass.getName());
				System.out.println("\t total area of triangles: " + areaOfTriangles);
				System.out.println("\t total area of triangles with sign: " + areaWithSignOfTriangles);
			}
			System.out.println();
		}

		return true;
	}

	/**
	 * Answer the example polylines.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @category Examples
	 */
	protected static Jun2dPoint[][] ExamplePolylines() {
		Jun2dPoint[][] examplePolylines = ExampleOriginalPolylines();
		return new Jun2dPoint[][] { examplePolylines[3], examplePolylines[9] };
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunFormTriangulationTestExamples();
	}
}
