package jp.co.sra.jun.geometry.pluralities;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall;
import jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox;
import jp.co.sra.jun.geometry.transformations.Jun2dTransformation;
import jp.co.sra.jun.goodies.lisp.JunLispConverter;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * Jun2dBoundingBalls class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/05/30 (by m-asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun697 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dBoundingBalls.java,v 9.14 2010/03/09 09:16:53 nisinaka Exp $
 */
public class Jun2dBoundingBalls extends JunBoundingBalls<Jun2dBoundingBall> {

	static {
		JunLispConverter.Register(Jun2dBoundingBalls.class);
	}

	/**
	 * Serial Version UID.
	 */
	private static final long serialVersionUID = -8569923528298047393L;

	/**
	 * Create a new instance of <code>Jun2dBoundingBalls</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public Jun2dBoundingBalls() {
		super();
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBalls</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public Jun2dBoundingBalls(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer the geometry objects.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunGeometries#geometries()
	 * @category accessing
	 */
	@Override
	public Jun2dBoundingBall[] geometries() {
		return this._geometries().toArray(new Jun2dBoundingBall[this._geometries().size()]);
	}

	/**
	 * Answer my current bounding ball.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall
	 * @category accessing
	 */
	public Jun2dBoundingBall boundingBall() {
		if (this.isEmpty()) {
			return new Jun2dBoundingBall((Jun2dPoint[]) null);
		}
		Jun2dBoundingBall boundingBall = null;
		Jun2dBoundingBall[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun2dBoundingBall each = objects[i];
			if (boundingBall == null) {
				boundingBall = each.boundingBall();
			} else {
				boundingBall = each.boundingBall().merge_(boundingBall);
			}
		}
		return boundingBall;
	}

	/**
	 * Answer my current bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category accessing
	 */
	public Jun2dBoundingBox boundingBox() {
		if (this.isEmpty()) {
			return new Jun2dBoundingBox();
		}
		Jun2dBoundingBox boundingBox = null;
		Jun2dBoundingBall[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun2dBoundingBall each = objects[i];
			if (boundingBox == null) {
				boundingBox = each.boundingBox();
			} else {
				boundingBox = each.boundingBox().merge_(boundingBox);
			}
		}
		return boundingBox;
	}

	/**
	 * Convert to a <code>Jun2dBoundingBall</code>.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall
	 * @category converting
	 */
	public Jun2dBoundingBall asBoundingBall() {
		return this.boundingBall();
	}

	/**
	 * Convert to a <code>Jun2dBoundingBox</code>.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category converting
	 */
	public Jun2dBoundingBox asBoundingBox() {
		return this.boundingBox();
	}

	/**
	 * Answer <code>true</code> if the receiver is a 2d geometry element.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#is2d()
	 * @category testing
	 */
	public boolean is2d() {
		return true;
	}

	/**
	 * Answer <code>true</code> if the receiver touches the specified bounding object, otherwise <code>false</code>.
	 * 
	 * @param aBoundingObject jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall
	 * @return boolean
	 * @category testing
	 */
	public boolean touches_(final Jun2dBoundingBall aBoundingObject) {
		return this.detect_ifNone_(new StBlockClosure1<Boolean, Jun2dBoundingBall>() {
			public Boolean value(Jun2dBoundingBall each) {
				return each.touches_(aBoundingObject);
			}
		}, null) != null;
	}

	/**
	 * Apply a transformation 'aTransformation' to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBalls
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.Jun2dTransformation
	 * @category transforming
	 */
	public Jun2dBoundingBalls transform_(Jun2dTransformation aTransformation) {
		Jun2dBoundingBalls transformedCopy = new Jun2dBoundingBalls();
		Jun2dBoundingBall[] boundingBalls = this.boundingBalls();
		for (int i = 0; i < boundingBalls.length; i++) {
			this.add_((Jun2dBoundingBall) boundingBalls[i].transform_(aTransformation));
		}
		return transformedCopy;
	}

}
