package jp.co.sra.jun.geometry.pluralities;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StRectangle;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall;
import jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.boundaries.JunBoundingObject;
import jp.co.sra.jun.geometry.transformations.Jun2dTransformation;
import jp.co.sra.jun.goodies.image.support.JunImageAdjuster;
import jp.co.sra.jun.goodies.image.support.JunImageProcessor;
import jp.co.sra.jun.goodies.lisp.JunLispConverter;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dPolygon;

/**
 * Jun2dBoundingBoxes class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2006/04/24 (by Mitsuhiro Asada)
 *  @updated   2007/05/29 (by Mitsuhiro Asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @updated   2009/03/25 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dBoundingBoxes.java,v 9.14 2010/03/09 09:16:54 nisinaka Exp $
 */
public class Jun2dBoundingBoxes extends JunBoundingBoxes<Jun2dBoundingBox> {

	static {
		JunLispConverter.Register(Jun2dBoundingBoxes.class);
	}

	/**
	 * Serial Version UID.
	 */
	private static final long serialVersionUID = -7556860674960888394L;

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public Jun2dBoundingBoxes() {
		super();
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public Jun2dBoundingBoxes(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromObject_(JunOpenGL3dObject anObject) {
		return Jun2dBoundingBoxes.FromObject_maxPartition_(anObject, DefaultPartitionSize());
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromObject_howMany_(JunOpenGL3dObject anObject, Jun2dPoint aPoint) {
		return Jun2dBoundingBoxes.MakeBoxesFromObject_howMany_(anObject, aPoint);
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param partitionSize int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromObject_maxPartition_(JunOpenGL3dObject anObject, int partitionSize) {
		double width = anObject.boundingBox().width();
		double height = anObject.boundingBox().height();
		double denominator = Math.max(width, height);
		Jun2dPoint point = new Jun2dPoint(width / denominator, height / denominator);
		point = point.multipliedBy_(partitionSize).rounded();
		return Jun2dBoundingBoxes.FromObject_howMany_(anObject, point);
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPoints_(Jun2dPoint[] pointCollection) {
		return Jun2dBoundingBoxes.FromPoints_maxPartition_(pointCollection, DefaultPartitionSize());
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPoints_howMany_(Jun2dPoint[] pointCollection, Jun2dPoint aPoint) {
		return Jun2dBoundingBoxes.MakeBoxesFromPoints_howMany_(pointCollection, aPoint);
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @param partitionSize int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPoints_maxPartition_(Jun2dPoint[] pointCollection, int partitionSize) {
		Jun2dBoundingBox box = new Jun2dBoundingBox(pointCollection);
		double width = box.boundingBox().width();
		double height = box.boundingBox().height();
		double denominator = Math.max(width, height);
		Jun2dPoint point = new Jun2dPoint(width / denominator, height / denominator);
		point = point.multipliedBy_(partitionSize).rounded();
		return Jun2dBoundingBoxes.FromPoints_howMany_(pointCollection, point);
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> with polyline and bit size.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPolyline_(Jun2dPoint[] pointCollection) {
		return Jun2dBoundingBoxes.FromPolyline_bitSize_(pointCollection, new Dimension(16, 16));
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> with polyline and bit size.
	 * 
	 * @param pointCollection java.awt.Point[]
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPolyline_(Point[] pointCollection) {
		return Jun2dBoundingBoxes.FromPolyline_bitSize_(pointCollection, new Dimension(16, 16));
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> with polyline and bit size.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @param bitSize java.awt.Dimension
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPolyline_bitSize_(Jun2dPoint[] pointCollection, Dimension bitSize) {
		Jun2dBoundingBoxes boxes = new Jun2dBoundingBoxes();
		if (pointCollection == null || pointCollection.length == 0) {
			return boxes;
		}
		Point[] points = new Point[pointCollection.length];
		int[] xPoints = new int[pointCollection.length];
		int[] yPoints = new int[pointCollection.length];
		for (int i = 0; i < pointCollection.length; i++) {
			points[i] = pointCollection[i]._toPoint();
			xPoints[i] = points[i].x;
			yPoints[i] = points[i].y;
		}
		Dimension dot = new Dimension(Math.max(bitSize.width, 0), Math.max(bitSize.height, 0));
		if (dot.width * dot.height <= 0) {
			return boxes;
		}
		Rectangle box = new Rectangle(points[0].x, points[0].y, 1, 1);
		for (int i = 1; i < points.length; i++) {
			box.add(new Rectangle(points[i].x, points[i].y, 1, 1));
		}
		Point origin = box.getLocation();
		Point corner = new Point(box.x + box.width, box.y + box.height);
		Dimension extent = new Dimension((int) Math.floor(Math.round(box.width / (double) dot.width)), (int) Math.floor(Math.round(box.height / (double) dot.height)));
		extent = new Dimension(Math.max(extent.width, 0), Math.max(extent.height, 0));
		if (extent.width * extent.height <= 0) {
			return boxes;
		}

		StImage pixmap = new StImage(box.width, box.height);
		Graphics2D aGraphics = (Graphics2D) pixmap.image().getGraphics();
		try {
			aGraphics.setColor(Color.white);
			aGraphics.fillRect(0, 0, extent.width, extent.height);
			aGraphics.setColor(Color.black);
			aGraphics.translate(-box.x, -box.y);
			Polygon aPolygon = new Polygon(xPoints, yPoints, xPoints.length);
			aGraphics.fill(aPolygon);
		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
				aGraphics = null;
			}
		}
		StImage image = JunImageAdjuster.Adjust_extent_(pixmap, extent);
		image = image.convertToPalette_(JunImageProcessor._WhiteBlackPalette());

		boxes.add_(new Jun2dBoundingBox(StRectangle.Origin_corner_(origin, corner).toRectangle()));
		for (int y = 0; y < image.height(); y++) {
			for (int x = 0; x < image.width(); x++) {
				if (image.getPixel(x, y) == Color.black.getRGB()) {
					boxes.add_(Jun2dBoundingBox.Origin_extent_(new Jun2dPoint(origin.x + x * dot.width, origin.y + y * dot.height), new Jun2dPoint(dot.width, dot.height)));
				}
			}
		}

		final Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
		boundingBoxes.add_(Jun2dBoundingBox.Origin_extent_(new Jun2dPoint(box.getLocation()), new Jun2dPoint(box.width, box.height)));
		boxes.differenceBoxes().do_(new StBlockClosure1<Void, Jun2dBoundingBox>() {
			public Void value(Jun2dBoundingBox each) {
				boundingBoxes.add_(each);
				return null;
			}
		});
		return boundingBoxes.differenceBoxes();
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> with polyline and bit size.
	 * 
	 * @param pointCollection java.awt.Point[]
	 * @param bitSize java.awt.Dimension
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun2dBoundingBoxes FromPolyline_bitSize_(Point[] pointCollection, Dimension bitSize) {
		Jun2dPoint[] points = new Jun2dPoint[pointCollection.length];
		for (int i = 0; i < pointCollection.length; i++) {
			points[i] = new Jun2dPoint(pointCollection[i].x, pointCollection[i].y);
		}
		return Jun2dBoundingBoxes.FromPolyline_bitSize_(points, bitSize);
	}

	/**
	 * Make a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Private
	 */
	protected static Jun2dBoundingBoxes MakeBoxesFromObject_howMany_(JunOpenGL3dObject aBody, Jun2dPoint aPoint) {
		final Jun2dBoundingBoxes bodyBoxes = new Jun2dBoundingBoxes();
		aBody.polygonsDo_(new StBlockClosure1<Void, JunOpenGL3dPolygon>() {
			public Void value(JunOpenGL3dPolygon aPolygon) {
				Jun3dPoint[] a3dVertexes = aPolygon.vertexes();
				Jun2dPoint[] a2dVertexes = new Jun2dPoint[a3dVertexes.length];
				for (int i = 0; i < a2dVertexes.length; i++) {
					a2dVertexes[i] = a3dVertexes[i].as2dPoint();
				}
				Jun2dBoundingBox boundingBox = new Jun2dBoundingBox(a2dVertexes);
				bodyBoxes.add_(boundingBox);
				return null;
			}
		});
		Jun2dPoint thePoint = Jun2dPoint.Coerce_(aPoint);
		int howManyX = (int) Math.round(thePoint.x());
		int howManyY = (int) Math.round(thePoint.y());
		Jun3dBoundingBox boundingBox = aBody.boundingBox();
		double boxWidth = boundingBox.width() / howManyX;
		double boxHeight = boundingBox.height() / howManyY;

		StImage zBitmapImage = new StImage(howManyX, howManyY, JunImageProcessor._WhiteBlackPalette());

		for (int yIndex = 0; yIndex < howManyY; yIndex++) {
			for (int xIndex = 0; xIndex < howManyX; xIndex++) {
				final Jun2dBoundingBox aBox = Jun2dBoundingBox.Origin_extent_(boundingBox.origin().as2dPoint().plus_(new Jun2dPoint(boxWidth * xIndex, boxHeight * yIndex)), new Jun2dPoint(boxWidth, boxHeight));
				if (bodyBoxes.detect_ifNone_(new StBlockClosure1<Boolean, Jun2dBoundingBox>() {
					public Boolean value(Jun2dBoundingBox each) {
						return (each.intersects_(aBox) || each.touches_(aBox));
					}
				}, new StBlockClosure0<Jun2dBoundingBox>() {
					public Jun2dBoundingBox value() {
						return null;
					}
				}) != null) {
					zBitmapImage.setPixel(xIndex, yIndex, Color.black.getRGB());
				} else {
					zBitmapImage.setPixel(xIndex, yIndex, Color.white.getRGB());
				}
			}
		}

		Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
		for (int yIndex = 0; yIndex < howManyY; yIndex++) {
			for (int xIndex = 0; xIndex < howManyX; xIndex++) {
				Jun2dBoundingBox aBox = Jun2dBoundingBox.Origin_extent_(boundingBox.origin().as2dPoint().plus_(new Jun2dPoint(boxWidth * xIndex, boxHeight * yIndex)), new Jun2dPoint(boxWidth, boxHeight));
				if (zBitmapImage.getPixel(xIndex, yIndex) == Color.black.getRGB()) {
					boundingBoxes.add_(aBox);
				}
			}
		}

		return boundingBoxes;
	}

	/**
	 * Create a new instance of <code>Jun2dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category Private
	 */
	protected static Jun2dBoundingBoxes MakeBoxesFromPoints_howMany_(Jun2dPoint[] pointCollection, Jun2dPoint aPoint) {
		Jun3dPoint thePoint = Jun3dPoint.Coerce_(aPoint);
		int howManyX = (int) Math.round(thePoint.x());
		int howManyY = (int) Math.round(thePoint.y());
		Jun2dBoundingBox boundingBox = new Jun2dBoundingBox(pointCollection);
		double boxWidth = boundingBox.width() / howManyX;
		double boxHeight = boundingBox.height() / howManyY;

		StImage zBitmapImage = new StImage(howManyX, howManyY, JunImageProcessor._WhiteBlackPalette());

		for (int yIndex = 0; yIndex < howManyY; yIndex++) {
			for (int xIndex = 0; xIndex < howManyX; xIndex++) {
				final Jun2dBoundingBox aBox = Jun2dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun2dPoint(boxWidth * xIndex, boxHeight * yIndex)), new Jun2dPoint(boxWidth, boxHeight));
				Jun2dPoint detect = null;
				for (int i = 0; i < pointCollection.length; i++) {
					if (aBox.containsPoint_(pointCollection[i])) {
						detect = pointCollection[i];
						break;
					}
				}
				if (detect != null) {
					zBitmapImage.setPixel(xIndex, yIndex, Color.black.getRGB());
				} else {
					zBitmapImage.setPixel(xIndex, yIndex, Color.white.getRGB());
				}
			}
		}

		Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
		for (int yIndex = 0; yIndex < howManyY; yIndex++) {
			for (int xIndex = 0; xIndex < howManyX; xIndex++) {
				Jun2dBoundingBox aBox = Jun2dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun2dPoint(boxWidth * xIndex, boxHeight * yIndex)), new Jun2dPoint(boxWidth, boxHeight));
				if (zBitmapImage.getPixel(xIndex, yIndex) == Color.black.getRGB()) {
					boundingBoxes.add_(aBox);
				}
			}
		}

		return boundingBoxes;
	}

	/**
	 * Answer the geometry objects.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunGeometries#geometries()
	 * @category accessing
	 */
	@Override
	public Jun2dBoundingBox[] geometries() {
		return this._geometries().toArray(new Jun2dBoundingBox[this._geometries().size()]);
	}

	/**
	 * Answer the this area size
	 * 
	 * @return double
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#area()
	 * @category accessing
	 */
	public double area() {
		double area = 0.0d;
		Jun2dBoundingBox[] boundingBoxes = this.boundingObjects();
		for (int i = 0; i < boundingBoxes.length; i++) {
			area = area + boundingBoxes[i].area();
		}
		return area;
	}

	/**
	 * Answer my current bounding ball.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall
	 * @category accessing
	 */
	public Jun2dBoundingBall boundingBall() {
		if (this.isEmpty()) {
			return new Jun2dBoundingBall((Jun2dPoint[]) null);
		}
		Jun2dBoundingBall boundingBall = null;
		Jun2dBoundingBox[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun2dBoundingBox each = objects[i];
			if (boundingBall == null) {
				boundingBall = each.boundingBall();
			} else {
				boundingBall = each.boundingBall().merge_(boundingBall);
			}
		}
		return boundingBall;
	}

	/**
	 * Answer my current bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category accessing
	 */
	public Jun2dBoundingBox boundingBox() {
		if (this.isEmpty()) {
			return new Jun2dBoundingBox();
		}
		Jun2dBoundingBox boundingBox = null;
		Jun2dBoundingBox[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun2dBoundingBox each = objects[i];
			if (boundingBox == null) {
				boundingBox = each;
			} else {
				boundingBox = each.merge_(boundingBox);
			}
		}
		return boundingBox;
	}

	/**
	 * Convert to a Jun2dBoundingBall.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBall
	 * @category converting
	 */
	public Jun2dBoundingBall asBoundingBall() {
		return this.boundingBall();
	}

	/**
	 * Convert to a Jun2dBoundingBox.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category converting
	 */
	public Jun2dBoundingBox asBoundingBox() {
		return this.boundingBox();
	}

	/**
	 * Convert to a Rectangle.
	 * 
	 * @return java.awt.Rectangle
	 * @see jp.co.sra.jun.geometry.pluralities.JunBoundingBoxes#asRectangle()
	 * @category converting
	 */
	public Rectangle asRectangle() {
		return this.asBoundingBox().asRectangle();
	}

	/**
	 * Convert to an array of Rectangle.
	 * 
	 * @return java.awt.Rectangle[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunBoundingBoxes#asRectangles()
	 * @category converting
	 */
	public Rectangle[] asRectangles() {
		final Collection<Rectangle> aCollection = new ArrayList<Rectangle>();
		this.do_(new StBlockClosure1<Void, Jun2dBoundingBox>() {
			public Void value(Jun2dBoundingBox boundingBox) {
				aCollection.add(boundingBox.asRectangle());
				return null;
			}
		});
		return aCollection.toArray(new Rectangle[aCollection.size()]);
	}

	/**
	 * Answer the receiver's complement boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category functions
	 */
	public Jun2dBoundingBoxes complementBoxes() {
		Jun2dBoundingBoxes complementBoxes = this.unionBoxes();
		Jun2dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
			return complementBoxes;
		}

		Jun2dBoundingBox[] boundingObjects = complementBoxes.boundingBoxes();
		for (int i = 0; i < boundingObjects.length; i++) {
			if (boundingObjects[i].equals(intersectionBox)) {
				complementBoxes._geometries().remove(boundingObjects[i]);
			}
		}
		complementBoxes = (Jun2dBoundingBoxes) complementBoxes.reject_(new StBlockClosure1<Boolean, Jun2dBoundingBox>() {
			public Boolean value(Jun2dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return complementBoxes;
	}

	/**
	 * Answer the difference boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category functions
	 */
	public Jun2dBoundingBoxes differenceBoxes() {
		if (this.isEmpty()) {
			return (Jun2dBoundingBoxes) this.copy();
		}

		Jun2dBoundingBox[] rectangles = this.boundingBoxes();
		Jun2dBoundingBox firstRectangle = rectangles[0];
		Jun2dBoundingBoxes differenceBoxes = new Jun2dBoundingBoxes();
		differenceBoxes.add_(firstRectangle);
		for (int i = 1; i < rectangles.length; i++) {
			Jun2dBoundingBox aRectangle = rectangles[i];
			Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
			JunBoundingObject[] myComponents = differenceBoxes.boundingObjects();
			for (int j = 0; j < myComponents.length; j++) {
				Jun2dBoundingBox each = (Jun2dBoundingBox) myComponents[j];
				if (each.intersects_(firstRectangle)) {
					boundingBoxes.addAll_(each.boxesOutside_(aRectangle));
				} else {
					boundingBoxes.add_(each);
				}
			}

			differenceBoxes = boundingBoxes;
		}
		differenceBoxes = (Jun2dBoundingBoxes) differenceBoxes.reject_(new StBlockClosure1<Boolean, Jun2dBoundingBox>() {
			public Boolean value(Jun2dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return differenceBoxes;
	}

	/**
	 * Answer the intersection box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox intersectionBox() {
		if (this.isEmpty()) {
			return null;
		}
		Jun2dBoundingBox intersectionBox = null;
		JunBoundingObject[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun2dBoundingBox each = (Jun2dBoundingBox) objects[i];
			if (intersectionBox == null) {
				intersectionBox = each;
			} else {
				if (intersectionBox.intersects_(each)) {
					intersectionBox = intersectionBox.intersect_(each);
				}
			}
		}
		if (intersectionBox.isEmpty()) {
			return null;
		}
		return intersectionBox;
	}

	/**
	 * Answer the intersection boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category functions
	 */
	public Jun2dBoundingBoxes intersectionBoxes() {
		Jun2dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
			return new Jun2dBoundingBoxes();
		}

		Jun2dBoundingBoxes intersectionBoxes = new Jun2dBoundingBoxes();
		intersectionBoxes.add_(intersectionBox);
		return intersectionBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes with specified level.
	 * 
	 * @param anInteger int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category functions
	 */
	public Jun2dBoundingBoxes subdivide2Level_(int anInteger) {
		Jun2dBoundingBoxes boundingBoxes = this;
		for (int i = 0; i < anInteger; i++) {
			boundingBoxes = boundingBoxes.subdivide2();
		}
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's unify.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox unify() {
		return this.asBoundingBox();
	}

	/**
	 * Answer the receiver's union boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category functions
	 */
	public Jun2dBoundingBoxes unionBoxes() {
		Jun2dBoundingBoxes unionBoxes = new Jun2dBoundingBoxes();
		Jun2dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
		} else {
			Jun2dBoundingBox[] tmp = this.boundingBoxes();
			Jun2dBoundingBox[] boxes = new Jun2dBoundingBox[tmp.length + 1];
			for (int i = 0; i < tmp.length; i++) {
				boxes[i] = tmp[i];
			}
			boxes[boxes.length - 1] = intersectionBox;

			for (int i = 0; i < boxes.length; i++) {
				Jun2dBoundingBox aBox = boxes[i];
				Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
				boundingBoxes.add_(aBox);
				Jun2dBoundingBox[] myComponents = unionBoxes.boundingBoxes();
				for (int j = 0; j < myComponents.length; j++) {
					Jun2dBoundingBox each = myComponents[j];
					if (each.intersects_(aBox)) {
						boundingBoxes.addAll_(each.boxesOutside_(aBox));
					} else {
						boundingBoxes.add_(each);
					}
				}
				unionBoxes = boundingBoxes;
			}
		}
		unionBoxes = (Jun2dBoundingBoxes) unionBoxes.reject_(new StBlockClosure1<Boolean, Jun2dBoundingBox>() {
			public Boolean value(Jun2dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return unionBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category subdividing
	 */
	public Jun2dBoundingBoxes subdivide() {
		return this.subdivide4();
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category subdividing
	 */
	public Jun2dBoundingBoxes subdivide2() {
		Collection<Jun2dBoundingBox> boundingBoxArray = new ArrayList<Jun2dBoundingBox>(this.size() * 2);
		for (int n = 0; n < this.size(); n++) {
			Jun2dBoundingBox[] boundingBoxes = this.boundingBox().subdivide2().boundingBoxes();
			boundingBoxArray.add(boundingBoxes[0]);
			boundingBoxArray.add(boundingBoxes[1]);
		}
		Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
		boundingBoxes.boundingBoxes_(boundingBoxArray);
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category subdividing
	 */
	public Jun2dBoundingBoxes subdivide4() {
		Collection<Jun2dBoundingBox> boundingBoxArray = new ArrayList<Jun2dBoundingBox>(this.size() * 4);
		for (int n = 0; n < this.size(); n++) {
			Jun2dBoundingBox[] boundingBoxes = this.boundingBox().subdivide4().boundingBoxes();
			boundingBoxArray.add(boundingBoxes[0]);
			boundingBoxArray.add(boundingBoxes[1]);
			boundingBoxArray.add(boundingBoxes[2]);
			boundingBoxArray.add(boundingBoxes[3]);
		}
		Jun2dBoundingBoxes boundingBoxes = new Jun2dBoundingBoxes();
		boundingBoxes.boundingBoxes_(boundingBoxArray);
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes with specified level.
	 * 
	 * @param anInteger int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @category subdividing
	 */
	public Jun2dBoundingBoxes subdivideLevel_(int anInteger) {
		Jun2dBoundingBoxes boundingBoxes = this;
		for (int i = 0; i < anInteger; i++) {
			boundingBoxes = boundingBoxes.subdivide();
		}
		return boundingBoxes;
	}

	/**
	 * Answer true if the receiver is a 2d geometry element.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#is2d()
	 * @category testing
	 */
	public boolean is2d() {
		return true;
	}

	/**
	 * Answer <code>true</code> if the receiver touches the specified bounding object, otherwise <code>false</code>.
	 * 
	 * @param aBoundingObject jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox
	 * @return boolean
	 * @category testing
	 */
	public boolean touches_(final Jun2dBoundingBox aBoundingObject) {
		return this.detect_ifNone_(new StBlockClosure1<Boolean, Jun2dBoundingBox>() {
			public Boolean value(Jun2dBoundingBox each) {
				return each.touches_(aBoundingObject);
			}
		}, null) != null;
	}

	/**
	 * Apply a transformation 'aTransformation' to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun2dBoundingBoxes
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.Jun2dTransformation
	 * @category transforming
	 */
	public Jun2dBoundingBoxes transform_(Jun2dTransformation aTransformation) {
		Jun2dBoundingBoxes transformedCopy = new Jun2dBoundingBoxes();
		Jun2dBoundingBox[] boundingObjects = this.boundingObjects();
		for (int i = 0; i < boundingObjects.length; i++) {
			this.add_(boundingObjects[i].transform_(aTransformation));
		}
		return transformedCopy;
	}

}
