package jp.co.sra.jun.geometry.pluralities;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.lisp.JunLispConverter;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * Jun3dBoundingBalls class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/05/30 (by m-asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun697 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun3dBoundingBalls.java,v 9.14 2010/03/09 09:16:54 nisinaka Exp $
 */
public class Jun3dBoundingBalls extends JunBoundingBalls<Jun3dBoundingBall> {

	static {
		JunLispConverter.Register(Jun3dBoundingBalls.class);
	}

	/**
	 * Serial Version UID.
	 */
	private static final long serialVersionUID = -3587301156189085986L;

	/**
	 * Create a new instance of <code>Jun3dBoundingBalls</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public Jun3dBoundingBalls() {
		super();
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBalls</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public Jun3dBoundingBalls(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer the geometry objects.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunGeometries#geometries()
	 * @category accessing
	 */
	@Override
	public Jun3dBoundingBall[] geometries() {
		return this._geometries().toArray(new Jun3dBoundingBall[this._geometries().size()]);
	}

	/**
	 * Answer my current bounding ball.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall
	 * @category accessing
	 */
	public Jun3dBoundingBall boundingBall() {
		if (this.isEmpty()) {
			return new Jun3dBoundingBall((Jun3dPoint[]) null);
		}
		Jun3dBoundingBall boundingBall = null;
		Jun3dBoundingBall[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun3dBoundingBall each = objects[i];
			if (boundingBall == null) {
				boundingBall = each.boundingBall();
			} else {
				boundingBall = each.boundingBall().merge_(boundingBall);
			}
		}
		return boundingBall;
	}

	/**
	 * Answer my current bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category accessing
	 */
	public Jun3dBoundingBox boundingBox() {
		if (this.isEmpty()) {
			return new Jun3dBoundingBox();
		}
		Jun3dBoundingBox boundingBox = null;
		Jun3dBoundingBall[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun3dBoundingBall each = objects[i];
			if (boundingBox == null) {
				boundingBox = each.boundingBox();
			} else {
				boundingBox = each.boundingBox().merge_(boundingBox);
			}
		}
		return boundingBox;
	}

	/**
	 * Convert to a <code>Jun2dBoundingBall</code>.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall
	 * @category converting
	 */
	public Jun3dBoundingBall asBoundingBall() {
		return this.boundingBall();
	}

	/**
	 * Convert to a <code>Jun2dBoundingBox</code>.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category converting
	 */
	public Jun3dBoundingBox asBoundingBox() {
		return this.boundingBox();
	}

	/**
	 * Answer <code>true</code> if the receiver is a 3d geometry element.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#is3d()
	 * @category testing
	 */
	public boolean is3d() {
		return true;
	}

	/**
	 * Answer <code>true</code> if the receiver touches the specified bounding object, otherwise <code>false</code>.
	 * 
	 * @param aBoundingObject jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall
	 * @return boolean
	 * @category testing
	 */
	public boolean touches_(final Jun3dBoundingBall aBoundingObject) {
		return this.detect_ifNone_(new StBlockClosure1<Boolean, Jun3dBoundingBall>() {
			public Boolean value(Jun3dBoundingBall each) {
				return each.touches_(aBoundingObject);
			}
		}, null) != null;
	}

	/**
	 * Apply a transformation 'aTransformation' to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBalls
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @category transforming
	 */
	public Jun3dBoundingBalls transform_(Jun3dTransformation aTransformation) {
		Jun3dBoundingBalls transformedCopy = new Jun3dBoundingBalls();
		Jun3dBoundingBall[] boundingBalls = this.boundingBalls();
		for (int i = 0; i < boundingBalls.length; i++) {
			this.add_((Jun3dBoundingBall) boundingBalls[i].transform_(aTransformation));
		}
		return transformedCopy;
	}

}
