package jp.co.sra.jun.geometry.pluralities;

import java.awt.Color;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StImage;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.boundaries.JunBoundingBox;
import jp.co.sra.jun.geometry.boundaries.JunBoundingObject;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.image.support.JunImageProcessor;
import jp.co.sra.jun.goodies.lisp.JunLispConverter;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dPolygon;

/**
 * Jun3dBoundingBoxes class
 * 
 *  @author    m-asada
 *  @created   2006/04/24 (by m-asada)
 *  @updated   2007/05/29 (by Mitsuhiro Asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @updated   2009/03/25 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun3dBoundingBoxes.java,v 9.14 2010/03/09 09:16:54 nisinaka Exp $
 */
public class Jun3dBoundingBoxes extends JunBoundingBoxes<Jun3dBoundingBox> {

	static {
		JunLispConverter.Register(Jun3dBoundingBoxes.class);
	}

	/**
	 * Serial Version UID.
	 */
	private static final long serialVersionUID = 331347700249998864L;

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public Jun3dBoundingBoxes() {
		super();
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public Jun3dBoundingBoxes(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromObject_(JunOpenGL3dObject anObject) {
		return Jun3dBoundingBoxes.FromObject_maxPartition_(anObject, DefaultPartitionSize());
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromObject_howMany_(JunOpenGL3dObject anObject, Jun3dPoint aPoint) {
		return Jun3dBoundingBoxes.MakeBoxesFromObject_howMany_(anObject, aPoint);
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param partitionSize int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromObject_maxPartition_(JunOpenGL3dObject anObject, int partitionSize) {
		double width = anObject.boundingBox().width();
		double height = anObject.boundingBox().height();
		double depth = anObject.boundingBox().depth();
		double denominator = Math.max(width, Math.max(height, depth));
		Jun3dPoint point = new Jun3dPoint(width / denominator, height / denominator, depth / denominator);
		point = point.multipliedBy_(partitionSize).rounded();
		return Jun3dBoundingBoxes.FromObject_howMany_(anObject, point);
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromPoints_(Jun3dPoint[] pointCollection) {
		return Jun3dBoundingBoxes.FromPoints_maxPartition_(pointCollection, DefaultPartitionSize());
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromPoints_howMany_(Jun3dPoint[] pointCollection, Jun3dPoint aPoint) {
		return Jun3dBoundingBoxes.MakeBoxesFromPoints_howMany_(pointCollection, aPoint);
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @param partitionSize int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Instance creation
	 */
	public static Jun3dBoundingBoxes FromPoints_maxPartition_(Jun3dPoint[] pointCollection, int partitionSize) {
		Jun3dBoundingBox box = new Jun3dBoundingBox(pointCollection);
		double width = box.boundingBox().width();
		double height = box.boundingBox().height();
		double depth = box.boundingBox().depth();
		double denominator = Math.max(width, Math.max(height, depth));
		Jun3dPoint point = new Jun3dPoint(width / denominator, height / denominator, depth / denominator);
		point = point.multipliedBy_(partitionSize).rounded();
		return Jun3dBoundingBoxes.FromPoints_howMany_(pointCollection, point);
	}

	/**
	 * Make a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Private
	 */
	protected static Jun3dBoundingBoxes MakeBoxesFromObject_howMany_(JunOpenGL3dObject aBody, Jun3dPoint aPoint) {
		final Jun3dBoundingBoxes bodyBoxes = new Jun3dBoundingBoxes();
		aBody.polygonsDo_(new StBlockClosure1<Void, JunOpenGL3dPolygon>() {
			public Void value(JunOpenGL3dPolygon aPolygon) {
				Jun3dBoundingBox boundingBox = new Jun3dBoundingBox(aPolygon.vertexes());
				bodyBoxes.add_(boundingBox);
				return null;
			}
		});
		Jun3dPoint thePoint = Jun3dPoint.Coerce_(aPoint);
		int howManyX = (int) Math.round(thePoint.x());
		int howManyY = (int) Math.round(thePoint.y());
		int howManyZ = (int) Math.round(thePoint.z());
		Jun3dBoundingBox boundingBox = aBody.boundingBox();
		double boxWidth = boundingBox.width() / howManyX;
		double boxHeight = boundingBox.height() / howManyY;
		double boxDepth = boundingBox.depth() / howManyZ;

		StImage[] zBitmapArray = new StImage[howManyZ];
		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = new StImage(howManyX, howManyY, JunImageProcessor._WhiteBlackPalette());
			zBitmapArray[zIndex] = zBitmapImage;
		}
		StImage[] xBitmapArray = new StImage[howManyX];
		for (int xIndex = 0; xIndex < howManyX; xIndex++) {
			StImage zBitmapImage = new StImage(howManyY, howManyZ, JunImageProcessor._WhiteBlackPalette());
			xBitmapArray[xIndex] = zBitmapImage;
		}
		StImage[] yBitmapArray = new StImage[howManyY];
		for (int yIndex = 0; yIndex < howManyY; yIndex++) {
			StImage yBitmapImage = new StImage(howManyZ, howManyX, JunImageProcessor._WhiteBlackPalette());
			yBitmapArray[yIndex] = yBitmapImage;
		}

		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = zBitmapArray[zIndex];
			for (int yIndex = 0; yIndex < howManyY; yIndex++) {
				StImage yBitmapImage = yBitmapArray[yIndex];
				for (int xIndex = 0; xIndex < howManyX; xIndex++) {
					StImage xBitmapImage = xBitmapArray[xIndex];
					final Jun3dBoundingBox aBox = Jun3dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun3dPoint(boxWidth * xIndex, boxHeight * yIndex, boxDepth * zIndex)), new Jun3dPoint(boxWidth, boxHeight, boxDepth));
					if (bodyBoxes.detect_ifNone_(new StBlockClosure1<Boolean, Jun3dBoundingBox>() {
						public Boolean value(Jun3dBoundingBox each) {
							return (each.intersects_(aBox) || each.touches_(aBox));
						}
					}, new StBlockClosure0<Jun3dBoundingBox>() {
						public Jun3dBoundingBox value() {
							return null;
						}
					}) != null) {
						zBitmapImage.setPixel(xIndex, yIndex, Color.black.getRGB());
						xBitmapImage.setPixel(yIndex, zIndex, Color.black.getRGB());
						yBitmapImage.setPixel(zIndex, xIndex, Color.black.getRGB());
					} else {
						zBitmapImage.setPixel(xIndex, yIndex, Color.white.getRGB());
						xBitmapImage.setPixel(yIndex, zIndex, Color.white.getRGB());
						yBitmapImage.setPixel(zIndex, xIndex, Color.white.getRGB());
					}
				}
			}
		}

		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = MakeBitmap_(zBitmapArray[zIndex]);
			zBitmapArray[zIndex] = zBitmapImage;
		}
		for (int xIndex = 0; xIndex < howManyX; xIndex++) {
			StImage xBitmapImage = MakeBitmap_(xBitmapArray[xIndex]);
			xBitmapArray[xIndex] = xBitmapImage;
		}
		for (int yIndex = 0; yIndex < howManyZ; yIndex++) {
			StImage yBitmapImage = MakeBitmap_(yBitmapArray[yIndex]);
			yBitmapArray[yIndex] = yBitmapImage;
		}

		Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = zBitmapArray[zIndex];
			for (int yIndex = 0; yIndex < howManyY; yIndex++) {
				StImage yBitmapImage = yBitmapArray[yIndex];
				for (int xIndex = 0; xIndex < howManyX; xIndex++) {
					StImage xBitmapImage = xBitmapArray[xIndex];
					Jun3dBoundingBox aBox = Jun3dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun3dPoint(boxWidth * xIndex, boxHeight * yIndex, boxDepth * zIndex)), new Jun3dPoint(boxWidth, boxHeight, boxDepth));
					if (zBitmapImage.getPixel(xIndex, yIndex) == Color.black.getRGB() && xBitmapImage.getPixel(yIndex, zIndex) == Color.black.getRGB() && yBitmapImage.getPixel(zIndex, xIndex) == Color.black.getRGB()) {
						boundingBoxes.add_(aBox);
					}
				}
			}
		}

		return boundingBoxes;
	}

	/**
	 * Create a new instance of <code>Jun3dBoundingBoxes</code> and initialize it.
	 * 
	 * @param pointCollection jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Private
	 */
	protected static Jun3dBoundingBoxes MakeBoxesFromPoints_howMany_(Jun3dPoint[] pointCollection, Jun3dPoint aPoint) {
		Jun3dPoint thePoint = Jun3dPoint.Coerce_(aPoint);
		int howManyX = (int) Math.round(thePoint.x());
		int howManyY = (int) Math.round(thePoint.y());
		int howManyZ = (int) Math.round(thePoint.z());
		Jun3dBoundingBox boundingBox = new Jun3dBoundingBox(pointCollection);
		double boxWidth = boundingBox.width() / howManyX;
		double boxHeight = boundingBox.height() / howManyY;
		double boxDepth = boundingBox.depth() / howManyZ;

		StImage[] zBitmapArray = new StImage[howManyZ];
		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = new StImage(howManyX, howManyY, JunImageProcessor._WhiteBlackPalette());
			zBitmapArray[zIndex] = zBitmapImage;
		}

		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = zBitmapArray[zIndex];
			for (int yIndex = 0; yIndex < howManyY; yIndex++) {
				for (int xIndex = 0; xIndex < howManyX; xIndex++) {
					final Jun3dBoundingBox aBox = Jun3dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun3dPoint(boxWidth * xIndex, boxHeight * yIndex, boxDepth * zIndex)), new Jun3dPoint(boxWidth, boxHeight, boxDepth));
					Jun3dPoint detect = null;
					for (int i = 0; i < pointCollection.length; i++) {
						if (aBox.containsPoint_(pointCollection[i])) {
							detect = pointCollection[i];
							break;
						}
					}
					if (detect != null) {
						zBitmapImage.setPixel(xIndex, yIndex, Color.black.getRGB());
					} else {
						zBitmapImage.setPixel(xIndex, yIndex, Color.white.getRGB());
					}
				}
			}
		}

		Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
		for (int zIndex = 0; zIndex < howManyZ; zIndex++) {
			StImage zBitmapImage = zBitmapArray[zIndex];
			for (int yIndex = 0; yIndex < howManyY; yIndex++) {
				for (int xIndex = 0; xIndex < howManyX; xIndex++) {
					Jun3dBoundingBox aBox = Jun3dBoundingBox.Origin_extent_(boundingBox.origin().plus_(new Jun3dPoint(boxWidth * xIndex, boxHeight * yIndex, boxDepth * zIndex)), new Jun3dPoint(boxWidth, boxHeight, boxDepth));
					if (zBitmapImage.getPixel(xIndex, yIndex) == Color.black.getRGB()) {
						boundingBoxes.add_(aBox);
					}
				}
			}
		}

		return boundingBoxes;
	}

	/**
	 * Answer the geometry objects.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunGeometries#geometries()
	 * @category accessing
	 */
	@Override
	public Jun3dBoundingBox[] geometries() {
		return this._geometries().toArray(new Jun3dBoundingBox[this._geometries().size()]);
	}

	/**
	 * Answer my current bounding ball.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall
	 * @category accessing
	 */
	public Jun3dBoundingBall boundingBall() {
		if (this.isEmpty()) {
			return new Jun3dBoundingBall((Jun3dPoint[]) null);
		}
		Jun3dBoundingBall boundingBall = null;
		Jun3dBoundingBox[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun3dBoundingBox each = objects[i];
			if (boundingBall == null) {
				boundingBall = each.boundingBall();
			} else {
				boundingBall = each.boundingBall().merge_(boundingBall);
			}
		}
		return boundingBall;
	}

	/**
	 * Answer my current bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category accessing
	 */
	public Jun3dBoundingBox boundingBox() {
		if (this.isEmpty()) {
			return new Jun3dBoundingBox();
		}
		Jun3dBoundingBox boundingBox = null;
		Jun3dBoundingBox[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun3dBoundingBox each = objects[i];
			if (boundingBox == null) {
				boundingBox = each;
			} else {
				boundingBox = each.merge_(boundingBox);
			}
		}
		return boundingBox;
	}

	/**
	 * Answer the this volume value.
	 * 
	 * @return double
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#volume()
	 * @category accessing
	 */
	public double volume() {
		double volume = 0.0d;
		Jun3dBoundingBox[] boundingBoxes = this.boundingObjects();
		for (int i = 0; i < boundingBoxes.length; i++) {
			volume = volume + boundingBoxes[i].volume();
		}
		return volume;
	}

	/**
	 * Convert to a Jun3dBoundingBall.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBall
	 * @category converting
	 */
	public Jun3dBoundingBall asBoundingBall() {
		return this.boundingBall();
	}

	/**
	 * Convert to a Jun3dBoundingBox.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category converting
	 */
	public Jun3dBoundingBox asBoundingBox() {
		return this.boundingBox();
	}

	/**
	 * Convert to a Rectangle.
	 * 
	 * @return java.awt.Rectangle
	 * @see jp.co.sra.jun.geometry.pluralities.JunBoundingBoxes#asRectangle()
	 * @category converting
	 */
	public Rectangle asRectangle() {
		return this.asBoundingBox().asRectangle();
	}

	/**
	 * Convert to an array of Rectangle.
	 * 
	 * @return java.awt.Rectangle[]
	 * @see jp.co.sra.jun.geometry.pluralities.JunBoundingBoxes#asRectangles()
	 * @category converting
	 */
	public Rectangle[] asRectangles() {
		final Collection<Rectangle> aCollection = new ArrayList<Rectangle>();
		this.do_(new StBlockClosure1<Void, Jun3dBoundingBox>() {
			public Void value(Jun3dBoundingBox boundingBox) {
				aCollection.add(boundingBox.asRectangle());
				return null;
			}
		});
		return aCollection.toArray(new Rectangle[aCollection.size()]);
	}

	/**
	 * Answer the receiver's complement boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category functions
	 */
	public Jun3dBoundingBoxes complementBoxes() {
		Jun3dBoundingBoxes complementBoxes = this.unionBoxes();
		Jun3dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
			return complementBoxes;
		}

		Jun3dBoundingBox[] boundingObjects = complementBoxes.boundingBoxes();
		for (int i = 0; i < boundingObjects.length; i++) {
			if (boundingObjects[i].equals(intersectionBox)) {
				complementBoxes._geometries().remove(boundingObjects[i]);
			}
		}
		complementBoxes = (Jun3dBoundingBoxes) complementBoxes.reject_(new StBlockClosure1<Boolean, Jun3dBoundingBox>() {
			public Boolean value(Jun3dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return complementBoxes;
	}

	/**
	 * Answer the difference boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category functions
	 */
	public Jun3dBoundingBoxes differenceBoxes() {
		if (this.isEmpty()) {
			return (Jun3dBoundingBoxes) this.copy();
		}

		Jun3dBoundingBox[] rectangles = this.boundingBoxes();
		Jun3dBoundingBox firstRectangle = rectangles[0];
		Jun3dBoundingBoxes differenceBoxes = new Jun3dBoundingBoxes();
		differenceBoxes.add_(firstRectangle);
		for (int i = 1; i < rectangles.length; i++) {
			Jun3dBoundingBox aRectangle = rectangles[i];
			Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
			Jun3dBoundingBox[] myComponents = differenceBoxes.boundingObjects();
			for (int j = 0; j < myComponents.length; j++) {
				Jun3dBoundingBox each = myComponents[j];
				if (each.intersects_(firstRectangle)) {
					boundingBoxes.addAll_(each.boxesOutside_(aRectangle));
				} else {
					boundingBoxes.add_(each);
				}
			}

			differenceBoxes = boundingBoxes;
		}
		differenceBoxes = (Jun3dBoundingBoxes) differenceBoxes.reject_(new StBlockClosure1<Boolean, Jun3dBoundingBox>() {
			public Boolean value(Jun3dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return differenceBoxes;
	}

	/**
	 * Answer the intersection box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category functions
	 */
	public Jun3dBoundingBox intersectionBox() {
		if (this.isEmpty()) {
			return null;
		}
		Jun3dBoundingBox intersectionBox = null;
		JunBoundingObject[] objects = this.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			Jun3dBoundingBox each = (Jun3dBoundingBox) objects[i];
			if (intersectionBox == null) {
				intersectionBox = each;
			} else {
				if (intersectionBox.intersects_(each)) {
					intersectionBox = intersectionBox.intersect_(each);
				} else {
					return null;
				}
			}
		}
		if (intersectionBox.isEmpty()) {
			return null;
		}
		return intersectionBox;
	}

	/**
	 * Answer the intersection boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category functions
	 */
	public Jun3dBoundingBoxes intersectionBoxes() {
		Jun3dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
			return new Jun3dBoundingBoxes();
		}

		Jun3dBoundingBoxes intersectionBoxes = new Jun3dBoundingBoxes();
		intersectionBoxes.add_(intersectionBox);
		return intersectionBoxes;
	}

	/**
	 * Answer the receiver's unify.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category functions
	 */
	public Jun3dBoundingBox unify() {
		return this.asBoundingBox();
	}

	/**
	 * Answer the receiver's union boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category functions
	 */
	public Jun3dBoundingBoxes unionBoxes() {
		Jun3dBoundingBoxes unionBoxes = new Jun3dBoundingBoxes();
		Jun3dBoundingBox intersectionBox = this.intersectionBox();
		if (intersectionBox == null) {
		} else {
			Jun3dBoundingBox[] tmp = this.boundingBoxes();
			Jun3dBoundingBox[] boxes = new Jun3dBoundingBox[tmp.length + 1];
			for (int i = 0; i < tmp.length; i++) {
				boxes[i] = tmp[i];
			}
			boxes[boxes.length - 1] = intersectionBox;

			for (int i = 0; i < boxes.length; i++) {
				Jun3dBoundingBox aBox = boxes[i];
				Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
				boundingBoxes.add_(aBox);
				JunBoundingBox[] myComponents = unionBoxes.boundingBoxes();
				for (int j = 0; j < myComponents.length; j++) {
					Jun3dBoundingBox each = (Jun3dBoundingBox) myComponents[j];
					if (each.intersects_(aBox)) {
						boundingBoxes.addAll_(each.boxesOutside_(aBox));
					} else {
						boundingBoxes.add_(each);
					}
				}
				unionBoxes = boundingBoxes;
			}
		}
		unionBoxes = (Jun3dBoundingBoxes) unionBoxes.reject_(new StBlockClosure1<Boolean, Jun3dBoundingBox>() {
			public Boolean value(Jun3dBoundingBox aBox) {
				return aBox.isEmpty();
			}
		});
		return unionBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category subdividing
	 */
	public Jun3dBoundingBoxes subdivide() {
		return this.subdivide8();
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category subdividing
	 */
	public Jun3dBoundingBoxes subdivide2() {
		Collection<Jun3dBoundingBox> boundingBoxArray = new ArrayList<Jun3dBoundingBox>(this.size() * 2);
		for (int n = 0; n < this.size(); n++) {
			Jun3dBoundingBox[] boundingBoxes = this.boundingBox().subdivide2().boundingBoxes();
			boundingBoxArray.add(boundingBoxes[0]);
			boundingBoxArray.add(boundingBoxes[1]);
		}
		Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
		boundingBoxes.boundingBoxes_(boundingBoxArray);
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category subdividing
	 */
	public Jun3dBoundingBoxes subdivide8() {
		Collection<Jun3dBoundingBox> boundingBoxArray = new ArrayList<Jun3dBoundingBox>(this.size() * 8);
		for (int n = 0; n < this.size(); n++) {
			Jun3dBoundingBox[] boundingBoxes = this.boundingBox().subdivide8().boundingBoxes();
			boundingBoxArray.add(boundingBoxes[0]);
			boundingBoxArray.add(boundingBoxes[1]);
			boundingBoxArray.add(boundingBoxes[2]);
			boundingBoxArray.add(boundingBoxes[3]);
			boundingBoxArray.add(boundingBoxes[4]);
			boundingBoxArray.add(boundingBoxes[5]);
			boundingBoxArray.add(boundingBoxes[6]);
			boundingBoxArray.add(boundingBoxes[7]);
		}
		Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
		boundingBoxes.boundingBoxes_(boundingBoxArray);
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes with specified level.
	 * 
	 * @param anInteger int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category subdividing
	 */
	public Jun3dBoundingBoxes subdivideLevel_(int anInteger) {
		Jun3dBoundingBoxes boundingBoxes = this;
		for (int i = 0; i < anInteger; i++) {
			boundingBoxes = boundingBoxes.subdivide();
		}
		return boundingBoxes;
	}

	/**
	 * Answer the receiver's subdivide bounding boxes with specified level.
	 * 
	 * @param anInteger int
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category subdividing
	 */
	public Jun3dBoundingBoxes subdivide2Level_(int anInteger) {
		Jun3dBoundingBoxes boundingBoxes = this;
		for (int i = 0; i < anInteger; i++) {
			boundingBoxes = boundingBoxes.subdivide2();
		}
		return boundingBoxes;
	}

	/**
	 * Answer true if the receiver is a 3d geometry element.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#is3d()
	 * @category testing
	 */
	public boolean is3d() {
		return true;
	}

	/**
	 * Answer <code>true</code> if the receiver touches the specified bounding object, otherwise <code>false</code>.
	 * 
	 * @param aBoundingObject jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @return boolean
	 * @category testing
	 */
	public boolean touches_(final Jun3dBoundingBox aBoundingObject) {
		return this.detect_ifNone_(new StBlockClosure1<Boolean, Jun3dBoundingBox>() {
			public Boolean value(Jun3dBoundingBox each) {
				return each.touches_(aBoundingObject);
			}
		}, null) != null;
	}

	/**
	 * Apply a transformation 'aTransformation' to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @category transforming
	 */
	public Jun3dBoundingBoxes transform_(Jun3dTransformation aTransformation) {
		Jun3dBoundingBoxes transformedCopy = new Jun3dBoundingBoxes();
		Jun3dBoundingBox[] boundingObjects = this.boundingObjects();
		for (int i = 0; i < boundingObjects.length; i++) {
			this.add_(boundingObjects[i].transform_(aTransformation));
		}
		return transformedCopy;
	}

}
