package jp.co.sra.jun.geometry.pluralities;

import java.awt.Color;
import java.awt.Rectangle;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dVertex;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * Jun3dBoundingBoxesTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/05/28 (by m-asada)
 *  @updated   2009/03/25 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun3dBoundingBoxesTestExamples.java,v 9.10 2010/03/09 09:16:54 nisinaka Exp $
 */
public class Jun3dBoundingBoxesTestExamples extends JunTestExamples {
	/**
	 * Example1: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		Jun3dBoundingBoxes boundingBoxes = Jun3dBoundingBoxesTestExamples.ExampleBoundingBoxes();
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		compoundObject.add_(boundingBoxes.asJunOpenGL3dObjectColor_alpha_(Color.cyan, 0.5f));
		compoundObject.add_(boundingBoxes.intersectionBoxes().asJunOpenGL3dObjectColor_alpha_(Color.magenta, 0.25f).translatedBy_(new Jun3dPoint(0, 0, 200)));
		compoundObject.add_(boundingBoxes.unionBoxes().asJunOpenGL3dObjectColor_alpha_(Color.magenta, 0.25f).translatedBy_(new Jun3dPoint(0, 0, 400)));
		compoundObject.add_(boundingBoxes.complementBoxes().asJunOpenGL3dObjectColor_alpha_(Color.magenta, 0.25f).translatedBy_(new Jun3dPoint(0, 0, 600)));
		compoundObject.add_(boundingBoxes.differenceBoxes().asJunOpenGL3dObjectColor_alpha_(Color.magenta, 0.25f).translatedBy_(new Jun3dPoint(0, 0, 800)));
		compoundObject.showIn_(new Rectangle(200, 200, 280, 500)).selectAll();

		return true;
	}

	/**
	 * Example2: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		JunOpenGL3dObject anObject = JunOpenGL3dObject.Torus();
		Jun3dBoundingBoxes boundingBoxes = Jun3dBoundingBoxes.FromObject_maxPartition_(anObject, 16);
		anObject.paint_alpha_(Color.magenta, 0.75f);
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(new JunOpenGL3dObject[] { anObject, boundingBoxes.asJunOpenGL3dObjectColor_alpha_(Color.cyan, 0.5f) });
		compoundObject.show();

		return true;
	}

	/**
	 * Example3: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		Jun3dPoint[] pointCollection = new Jun3dPoint[100];
		for (int i = 0; i < pointCollection.length; i++) {
			pointCollection[i] = new Jun3dPoint(Math.random(), Math.random(), Math.random());
		}
		Jun3dBoundingBoxes boundingBoxes = Jun3dBoundingBoxes.FromPoints_maxPartition_(pointCollection, 10);
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		for (int i = 0; i < pointCollection.length; i++) {
			JunOpenGL3dVertex anOpenGL3dObject = new JunOpenGL3dVertex(pointCollection[i]);
			anOpenGL3dObject.size_(4);
			anOpenGL3dObject.paint_(Color.blue);
			compoundObject.add_(anOpenGL3dObject);
		}
		compoundObject.add_(boundingBoxes.asJunOpenGL3dObjectColor_alpha_(Color.cyan, 0.5f));
		compoundObject.show();

		return true;
	}

	/**
	 * Example: for subdivide.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean _ExampleSubdivide() {
		new Jun3dPoint(0, 0, 0).box_(new Jun3dPoint(1, 1, 1)).asBoundingBoxes().subdivide().showWithUnitArrows().wireframePresentation();

		return true;
	}

	/**
	 * Answer the example bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.Jun3dBoundingBoxes
	 * @category Examples
	 */
	protected static Jun3dBoundingBoxes ExampleBoundingBoxes() {
		Jun3dBoundingBoxes boundingBoxes = new Jun3dBoundingBoxes();
		boundingBoxes.add_(new Jun3dPoint(51, 33, 0).corner_(new Jun3dPoint(195, 147, 80)));
		boundingBoxes.add_(new Jun3dPoint(93, 45, 20).corner_(new Jun3dPoint(237, 159, 100)));
		boundingBoxes.add_(new Jun3dPoint(129, 69, 40).corner_(new Jun3dPoint(273, 183, 120)));
		boundingBoxes.add_(new Jun3dPoint(81, 111, 60).corner_(new Jun3dPoint(225, 225, 140)));
		return boundingBoxes;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) throws Exception {
		new Jun3dBoundingBoxesTestExamples();
	}
}
