package jp.co.sra.jun.geometry.pluralities;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Collection;

import jp.co.sra.smalltalk.StImage;

import jp.co.sra.jun.geometry.boundaries.JunBoundingBox;
import jp.co.sra.jun.goodies.image.support.JunImageProcessor;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunBoundingBoxes class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2006/04/24 (by Mitsuhiro Asada)
 *  @updated   2007/05/29 (by Mitsuhiro Asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun667 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBoundingBoxes.java,v 9.10 2010/03/09 09:16:54 nisinaka Exp $
 */
public abstract class JunBoundingBoxes<G extends JunBoundingBox> extends JunBoundingObjects<G> {
	/**
	 * Answer the default partition size;
	 * 
	 * @return int
	 * @category Defaults
	 */
	public static final int DefaultPartitionSize() {
		return 10;
	}

	/**
	 * Make bitmap image with the specified source image.
	 * 
	 * @param sourceImage jp.co.sra.smalltalk.StImage
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Private
	 */
	protected static StImage MakeBitmap_(StImage sourceImage) {
		StImage expandedImage = new StImage(sourceImage.width() + 2, sourceImage.height() + 2, JunImageProcessor._WhiteBlackPalette());
		JunImageProcessor.Fill_color_(expandedImage, Color.white);
		Rectangle inset = expandedImage.bounds();
		inset.grow(-1, -1);
		expandedImage.copy_from_in_rule_(inset, new Point(0, 0), sourceImage, StImage.Over);
		JunImageProcessor.Fill_seed_(expandedImage, new Point(0, 0));
		StImage bitmapImage = new StImage(sourceImage.width(), sourceImage.height(), JunImageProcessor._WhiteBlackPalette());
		bitmapImage.copy_from_in_rule_(bitmapImage.bounds(), new Point(1, 1), expandedImage, StImage.Over);
		for (int y = 0; y < bitmapImage.height(); y++) {
			for (int x = 0; x < bitmapImage.width(); x++) {
				int v;
				if (bitmapImage.getPixel(x, y) == Color.white.getRGB()) {
					v = Color.black.getRGB();
				} else {
					v = Color.white.getRGB();
					if (sourceImage.getPixel(x, y) == Color.black.getRGB()) {
						v = Color.black.getRGB();
					}
				}
				bitmapImage.setPixel(x, y, v);
			}
		}
		return bitmapImage;
	}

	/**
	 * Create a new instance of JunBoundingBoxes and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunBoundingBoxes() {
		super();
	}

	/**
	 * Create a new instance of <code>JunBoundingBoxes</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunBoundingBoxes(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer my current bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.JunBoundingBox[]
	 * @category accessing
	 */
	public G[] boundingBoxes() {
		return this.boundingObjects();
	}

	/**
	 * Set my new bounding boxes.
	 * 
	 * @param boundingBoxCollection java.util.Collection
	 * @category accessing
	 */
	public void boundingBoxes_(Collection<G> boundingBoxCollection) {
		this.boundingObjects_(boundingBoxCollection);
	}

	/**
	 * Convert to a bounding boxes.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunBoundingBoxes
	 * @category converting
	 */
	public JunBoundingBoxes<G> asBoundingBoxes() {
		return this;
	}

	/**
	 * Convert to a Rectangle.
	 * 
	 * @return java.awt.Rectangle
	 * @category converting
	 */
	public abstract Rectangle asRectangle();

	/**
	 * Convert to an array of Rectangle.
	 * 
	 * @return java.awt.Rectangle[]
	 * @category converting
	 */
	public abstract Rectangle[] asRectangles();

	/**
	 * Answer true if the receiver is a bounding boxes, otherwise false.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#isBoundingBoxes()
	 * @category testing
	 */
	public boolean isBoundingBoxes() {
		return true;
	}

}
