package jp.co.sra.jun.geometry.pluralities;

import java.util.Collection;

import jp.co.sra.jun.geometry.boundaries.JunBoundingObject;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunBoundingObjects class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2004/10/19 (by Mitsuhiro Asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun667 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBoundingObjects.java,v 9.10 2010/03/09 09:16:54 nisinaka Exp $
 */
public abstract class JunBoundingObjects<G extends JunBoundingObject> extends JunGeometries<G> {
	/**
	 * Create a new instance of <code>JunBoundingObjects</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunBoundingObjects() {
		super();
	}

	/**
	 * Create a new instance of <code>JunBoundingObjects</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunBoundingObjects(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of JunGeometry and initialize it.
	 * 
	 * @param sizeInteger int
	 * @category Instance creation
	 */
	public JunBoundingObjects(int sizeInteger) {
		super();
	}

	/**
	 * Answer the bounding objects.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.JunBoundingObject[]
	 * @category accessing
	 */
	public G[] boundingObjects() {
		return this.geometries();
	}

	/**
	 * Set the bounding objects.
	 * 
	 * @param boundingObjectCollection java.util.Collection
	 * @category accessing
	 */
	public void boundingObjects_(Collection<G> boundingObjectCollection) {
		this.geometries_(boundingObjectCollection);
	}

	/**
	 * Add all of the bounding objects.
	 * 
	 * @param boundingObjects jp.co.sra.jun.geometry.pluralities.JunBoundingObjects
	 * @category adding
	 */
	public void addAll_(JunBoundingObjects<G> boundingObjects) {
		if (boundingObjects == null) {
			return;
		}
		G[] objects = boundingObjects.boundingObjects();
		for (int i = 0; i < objects.length; i++) {
			this.add_(objects[i]);
		}
	}

	/**
	 * Answer true if the receiver is a bounding objects, otherwise false.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#isBoundingObjects()
	 * @category testing
	 */
	public boolean isBoundingObjects() {
		return true;
	}

	/**
	 * Answer true if the receiver is empty, otherwise false.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.pluralities.JunGeometries#isEmpty()
	 * @category testing
	 */
	public boolean isEmpty() {
		if (this.size() > 0) {
			JunBoundingObject[] objects = this.boundingObjects();
			for (int i = 0; i < objects.length; i++) {
				if (objects[i].isEmpty() == false) {
					return false;
				}
			}
		}
		return true;
	}

}
