package jp.co.sra.jun.geometry.pluralities;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.abstracts.JunGeometry;
import jp.co.sra.jun.geometry.transformations.JunTransformation;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunGeometries class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/05/29 (by m-asada)
 *  @updated   2009/03/25 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunGeometries.java,v 9.12 2010/03/09 09:16:54 nisinaka Exp $
 */
public abstract class JunGeometries<G extends JunGeometry> extends JunGeometry {

	protected List<G> geometries;

	/**
	 * Create a new instance of <code>JunGeometries</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunGeometries() {
		super();
	}

	/**
	 * Create a new instance of <code>JunGeometries</code> and initialize it.
	 * 
	 * @category Instance creation
	 * @param sizeInteger int
	 */
	public JunGeometries(int sizeInteger) {
		super();
		this.initialize_(sizeInteger);
	}

	/**
	 * Create a new instance of <code>JunAbstractObject</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunGeometries(JunLispList aList) {
		super(aList);
	}

	/**
	 * Initialize this object.
	 * 
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		geometries = new ArrayList<G>();
	}

	/**
	 * Initialize this object with array size.
	 * 
	 * @param sizeInteger int
	 * @category initialize-release
	 */
	public void initialize_(int sizeInteger) {
		super.initialize();
		geometries = new ArrayList<G>(sizeInteger);
	}

	/**
	 * Answer the geometry objects as <code>List</code>.
	 * 
	 * @return java.util.List
	 * @category accessing
	 */
	protected List<G> _geometries() {
		return geometries;
	}

	/**
	 * Returns the element at the specified position in this list.
	 *
	 * @param index int
	 * @return jp.co.sra.jun.geometry.abstracts.JunGeometry
	 * @throws java.lang.IndexOutOfBoundsException
	 * @category accessing
	 */
	public G at_(int index) {
		return this._geometries().get(index);
	}

	/**
	 * Replaces the element at the specified position in this list with the specified element.
	 *
	 * @param index int
	 * @param geometry jp.co.sra.jun.geometry.abstracts.JunGeometry
	 * @throws java.lang.IndexOutOfBoundsException
	 * @category accessing
	 */
	public void at_put_(int index, G geometry) {
		if (geometry == null) {
			return;
		}
		if (this.size() == index) {
			this._geometries().add(geometry);
		} else {
			this._geometries().set(index, geometry);
		}
	}

	/**
	 * Answer the geometry objects.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunGeometry[]
	 * @category accessing
	 */
	public abstract G[] geometries();

	/**
	 * Set the geometry objects.
	 * 
	 * @param geometryCollection java.util.Collection
	 * @category accessing
	 */
	public void geometries_(Collection<G> geometryCollection) {
		if (geometryCollection instanceof List) {
			geometries = (List<G>) geometryCollection;
		} else {
			geometries = new ArrayList<G>(geometryCollection);
		}
	}

	/**
	 * Set the geometry objects.
	 * 
	 * @param geometryCollection jp.co.sra.jun.geometry.abstracts.JunGeometry
	 * @category accessing
	 */
	public void geometries_(G[] geometryCollection) {
		Collection<G> aCollection = new ArrayList<G>();
		for (int i = 0; i < geometryCollection.length; i++) {
			aCollection.add(geometryCollection[i]);
		}
		this.geometries_(aCollection);
	}

	/**
	 * Answer the size of geometry objects.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return this._geometries().size();
	}

	/**
	 * Add a <code>JunGeometry</code> to the geometries.
	 * 
	 * @param aGeometry jp.co.sra.jun.geometry.abstracts.JunGeometry
	 * @category adding
	 */
	public void add_(G aGeometry) {
		if (aGeometry == null) {
			return;
		}

		this._geometries().add(aGeometry);
	}

	/**
	 * Add all of the array elements.
	 * 
	 * @param anArray jp.co.sra.jun.geometry.abstracts.JunGeometry[]
	 * @category adding
	 */
	public void addAll_(G[] anArray) {
		if (anArray == null) {
			return;
		}

		for (int i = 0; i < anArray.length; i++) {
			this.add_(anArray[i]);
		}
	}

	/**
	 * Add all of the collection elements.
	 * 
	 * @param aCollection java.util.Collection
	 * @category adding
	 */
	public void addAll_(Collection<G> aCollection) {
		if (aCollection == null) {
			return;
		}

		this._geometries().addAll(aCollection);
	}

	/**
	 * Answer true if the receiver is equal to the object while concerning an accuracy.
	 * 
	 * @param anObject java.lang.Object
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#equal_(java.lang.Object)
	 * @category comparing
	 */
	@Override
	public boolean equal_(Object anObject) {
		if (anObject == null || this.getClass() != anObject.getClass()) {
			return false;
		}

		JunGeometries<?> another = (JunGeometries<?>) anObject;
		int size = this.size();
		if (size != another.size()) {
			return false;
		}

		JunGeometry[] myGeometries = this.geometries();
		JunGeometry[] anotherGeometries = another.geometries();
		for (int i = 0; i < size; i++) {
			if (myGeometries[i].equal_(anotherGeometries[i]) == false) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Answer true if the Object is equal to the receiver, otherwise false.
	 * 
	 * @param anObject java.lang.Object
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#equals(java.lang.Object)
	 * @category comparing
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject == null || this.getClass() != anObject.getClass()) {
			return false;
		}

		JunGeometries<?> another = (JunGeometries<?>) anObject;
		int size = this.size();
		if (size != another.size()) {
			return false;
		}

		JunGeometry[] myGeometries = this.geometries();
		JunGeometry[] anotherGeometries = another.geometries();
		for (int i = 0; i < size; i++) {
			if (myGeometries[i].equals(anotherGeometries[i]) == false) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Convert to an array of <code>JunOpenGL3dObject<code>.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunGeometry[]
	 * @category converting
	 */
	public G[] asArray() {
		return this.geometries();
	}

	/**
	 * Convert to a <code>JunOpenGL3dObject<code>.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#asJunOpenGL3dObject()
	 * @category converting
	 */
	@Override
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(this.size());
		G[] geometries = this.geometries();
		for (int i = 0; i < geometries.length; i++) {
			compoundObject.add_(geometries[i].asJunOpenGL3dObject());
		}
		return compoundObject;
	}

	/**
	 * Evaluate aBlock with each of the values of the receiver as the argument.
	 * Collect the resulting values into a collection that is like the receiver.
	 * Answer the new collection.
	 * 
	 * @return java.util.Collection
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public Collection<G> collect_(StBlockClosure1<G, G> aBlock) {
		ArrayList<G> newBoundingObjects = new ArrayList<G>();
		G[] myGeometries = this.geometries();
		for (int i = 0; i < myGeometries.length; i++) {
			newBoundingObjects.add(aBlock.value(myGeometries[i]));
		}
		return newBoundingObjects;
	}

	/**
	 * Evaluate aBlock with each of the values of the receiver as the argument.
	 * Collect the resulting values into a collection that is like the receiver.
	 * Answer the new collection.
	 * 
	 * @return java.util.Collection
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Collection<G> collect_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.collect_(new StBlockClosure1<G, G>() {
			@SuppressWarnings("unchecked")
			public G value(G g) {
				return (G) aBlock.value_(g);
			}
		});
	}

	/**
	 * Enumerate each objects and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating 
	 */
	public <R> R do_(StBlockClosure1<R, G> aBlock) {
		G[] myGeometries = this.geometries();
		for (int i = 0; i < myGeometries.length; i++) {
			R result = aBlock.value(myGeometries[i]);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/**
	 * Enumerate each objects and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object do_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.do_(new StBlockClosure1<Object, G>() {
			public Object value(G g) {
				return aBlock.value_(g);
			}
		});
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Answer the first element for which aBlock evaluates to true.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating 
	 */
	public G detect_(StBlockClosure1<Boolean, G> aBlock) {
		return this.detect_ifNone_(aBlock, null);
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Answer the first element for which aBlock evaluates to true.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object detect_(StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.detect_ifNone_(aBlock, null);
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Answer the first element for which aBlock evaluates to true.
	 * 
	 * @return java.lang.Object
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param exceptionBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public G detect_ifNone_(StBlockClosure1<Boolean, G> aBlock, StBlockClosure0<G> exceptionBlock) {
		G[] myGeometries = this.geometries();
		for (int i = 0; i < myGeometries.length; i++) {
			Boolean result = aBlock.value(myGeometries[i]);
			if (result != null) {
				if (result.booleanValue()) {
					return myGeometries[i];
				}
			}
		}
		return (exceptionBlock == null) ? null : exceptionBlock.value();
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Answer the first element for which aBlock evaluates to true.
	 * 
	 * @return java.lang.Object
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param exceptionBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object detect_ifNone_(final StBlockClosure<?, ?, ?, ?> aBlock, final StBlockClosure<?, ?, ?, ?> exceptionBlock) {
		StBlockClosure0<G> myExceptionBlock = null;
		if (exceptionBlock != null) {
			myExceptionBlock = new StBlockClosure0<G>() {
				@SuppressWarnings("unchecked")
				public G value() {
					return (G) exceptionBlock.value();
				}
			};
		}

		return this.detect_ifNone_(new StBlockClosure1<Boolean, G>() {
			public Boolean value(G g) {
				return (Boolean) aBlock.value_(g);
			}
		}, myExceptionBlock);
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Collect into a new collection like the receiver, only those elements
	 * for which aBlock evaluates to false.  Answer the new collection.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunGeometries
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public JunGeometries<G> reject_(final StBlockClosure1<Boolean, G> aBlock) {
		return this.select_(new StBlockClosure1<Boolean, G>() {
			public Boolean value(G g) {
				return aBlock.value(g) == false;
			}
		});
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument.
	 * Collect into a new collection like the receiver, only those elements
	 * for which aBlock evaluates to false.  Answer the new collection.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunGeometries
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunGeometries<G> reject_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.reject_(new StBlockClosure1<Boolean, G>() {
			public Boolean value(G g) {
				return (Boolean) aBlock.value_(g);
			}
		});
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument. 
	 * Collect into a new collection like the receiver, only those elements
	 * for which aBlock evaluates to true.  Answer the new collection.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunGeometries
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	@SuppressWarnings("unchecked")
	public JunGeometries<G> select_(final StBlockClosure1<Boolean, G> aBlock) {
		Class<JunGeometries<G>> aClass = (Class<JunGeometries<G>>) this.getClass();
		final JunGeometries<G> copy = _New(aClass);
		this.do_(new StBlockClosure1<Void, G>() {
			public Void value(G g) {
				if (aBlock.value(g)) {
					copy.add_(g);
				}
				return null;
			}
		});
		return copy;
	}

	/**
	 * Evaluate aBlock with each of the receiver's elements as the argument. 
	 * Collect into a new collection like the receiver, only those elements
	 * for which aBlock evaluates to true.  Answer the new collection.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunGeometries
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunGeometries<G> select_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.select_(new StBlockClosure1<Boolean, G>() {
			public Boolean value(G g) {
				return (Boolean) aBlock.value_(g);
			}
		});
	}

	/**
	 * Get the receiver's attributes from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#fromLispList(jp.co.sra.jun.goodies.lisp.JunLispList)
	 * @category lisp support
	 */
	@Override
	public void fromLispList(JunLispList aList) {
		super.fromLispList(aList);
		this.geometriesFromLispList(aList);
	}

	/**
	 * Convert the receiver's attributes to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#toLispList()
	 * @category lisp support
	 */
	@Override
	public JunLispCons toLispList() {
		JunLispCons list = super.toLispList();
		list.add_(this.geometriesToLispList());
		return list;
	}

	/**
	 * Get the receiver's geometries from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	@SuppressWarnings("unchecked")
	protected void geometriesFromLispList(JunLispList aList) {
		final JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object car) {
				return new Boolean(car instanceof JunLispCons && ((JunLispCons) car).head() == $("geometries"));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}

		Object[] objects = ((JunLispList) list.tail()).asArray();
		for (int i = 0; i < objects.length; i++) {
			this.add_((G) JunGeometry.FromLispList((JunLispList) objects[i]));
		}
	}

	/**
	 * Convert the receiver's geometries to the lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons geometriesToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("geometries"));
		G[] geometries = this.geometries();
		for (int i = 0; i < geometries.length; i++) {
			list.add_(geometries[i].toLispList());
		}
		return list;
	}

	/**
	 * Print my storable string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.lang.IOException if failed.
	 * @see jp.co.sra.smalltalk.StObject#storeOn_(java.io.Writer)
	 * @category printing
	 */
	@Override
	public void storeOn_(Writer aWriter) throws IOException {
		aWriter.write("((");
		aWriter.write(this.getClass().getName());
		aWriter.write(" new)");
		aWriter.write("\n");
		aWriter.write("\t");
		aWriter.write("addAll: ((");
		aWriter.write(this._geometries().getClass().getName());
		aWriter.write(" new: ");
		aWriter.write(Integer.toString(this._geometries().size()));
		aWriter.write(")");

		JunGeometry[] geometries = this.geometries();
		for (int i = 0; i < geometries.length; i++) {
			aWriter.write("\n");
			aWriter.write("\t");
			aWriter.write("\t");
			aWriter.write("add: [");
			geometries[i].storeOn_(aWriter);
			aWriter.write("] value;");
		}
		aWriter.write("\n");
		aWriter.write("))");
	}

	/**
	 * Answer true if the receiver is empty, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isEmpty() {
		return this._geometries().isEmpty();
	}

	/**
	 * Apply a transformation 'aTransformation' to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.pluralities.JunGeometries
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.JunTransformation
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#transform_(jp.co.sra.jun.geometry.transformations.JunTransformation)
	 * @category transforming
	 */
	@SuppressWarnings("unchecked")
	@Override
	public JunGeometries<G> transform_(JunTransformation aTransformation) {
		Class<JunGeometries<G>> aClass = (Class<JunGeometries<G>>) this.getClass();
		JunGeometries<G> transformedCopy = _New(aClass);
		G[] boundingObjects = this.geometries();
		for (int i = 0; i < boundingObjects.length; i++) {
			this.add_((G) boundingObjects[i].transform_(aTransformation));
		}
		return transformedCopy;
	}

}
