package jp.co.sra.jun.geometry.pluralities;

import java.util.Collection;

import jp.co.sra.jun.geometry.surfaces.JunTriangle;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunTriangles class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/06/05 (by m-asada)
 *  @updated   2008/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun667 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTriangles.java,v 9.10 2010/03/09 09:16:54 nisinaka Exp $
 */
public abstract class JunTriangles<G extends JunTriangle> extends JunGeometries<G> {

	/**
	 * Create a new instance of <code>JunTriangles</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunTriangles() {
		super();
	}

	/**
	 * Create a new instance of <code>JunTriangles</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunTriangles(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of <code>JunTriangles</code> and initialize it.
	 * 
	 * @param triangleCollection jp.co.sra.jun.geometry.surfaces.JunTriangle[]
	 * @category Instance creation
	 */
	public JunTriangles(G[] triangleCollection) {
		super();
		this.triangles_(triangleCollection);
	}

	/**
	 * Create a new instance of <code>JunTriangles</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunTriangles(Collection<G> triangleCollection) {
		super();
		this.triangles_(triangleCollection);
	}

	/**
	 * Answer my current triangles.
	 * 
	 * @return jp.co.sra.jun.geometry.surfaces.JunTriangle[]
	 * @category accessing
	 */
	public G[] triangles() {
		return this.geometries();
	}

	/**
	 * Set my new triangles triangles.
	 * 
	 * @param triangleCollection jp.co.sra.jun.geometry.surfaces.JunTriangle[]
	 * @category accessing
	 */
	public void triangles_(G[] triangleCollection) {
		this.geometries_(triangleCollection);
	}

	/**
	 * Set the receiver's triangles.
	 * 
	 * @param triangleCollection java.util.Collection
	 * @category accessing
	 */
	public void triangles_(Collection<G> triangleCollection) {
		this.geometries_(triangleCollection);
	}

	/**
	 * Answer true if the receiver is a triangles, otherwise false.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.geometry.abstracts.JunGeometry#isTriangles()
	 * @category testing
	 */
	public boolean isTriangles() {
		return true;
	}

}
