package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateAndTimeDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateAndTimeDialog extends JunAbstractDialog<JunCalendarDateAndTimeModel> {

	/**
	 * Open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel Request() {
		return JunCalendarDateAndTimeDialog.Request_(null);
	}

	/**
	 * Open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 * @return JunCalendarDateAndTimeModel
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel Request_(String messageString) {
		Calendar[] dateAndTime = { Calendar.getInstance(), Calendar.getInstance() };
		return JunCalendarDateAndTimeDialog.Request_initialDateAndTime_(messageString, dateAndTime);
	}

	/**
	 * Initialize date and time , open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 
	 * @return JunCalendarDateAndTimeModel
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel Request_initialDateAndTime_(String messageString, Calendar[] dateAndTime) {
		return JunCalendarDateAndTimeDialog.Request_initialDateAndTime_onAccept_onCancel_(messageString, dateAndTime, null, null);
	}

	/**
	 * Initialize date and time , set accept button and cancel button,
	 * open a calendarDateAndTime dialog and return selected date and time.
	 *
	 * @param messageString java.lang.String
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return JunCalendarDateAndTimeModel
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel Request_initialDateAndTime_onAccept_onCancel_(String messageString, Calendar[] dateAndTime, StBlockClosure1<Boolean, JunCalendarDateAndTimeModel> acceptBlock,
			StBlockClosure1<Boolean, JunCalendarDateAndTimeModel> cancelBlock) {
		JunCalendarDateAndTimeDialog calendarDateAndTimeDialog = new JunCalendarDateAndTimeDialog();
		if (dateAndTime != null) {
			calendarDateAndTimeDialog.calendarDateAndTimeModel_(JunCalendarDateAndTimeModel.DateAndTime_(dateAndTime));
		}
		Dialog dialog = calendarDateAndTimeDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			calendarDateAndTimeDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			calendarDateAndTimeDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateAndTimeDialog.accept().value()) {
			return calendarDateAndTimeDialog.calendarDateAndTimeModel();
		} else {
			return null;
		}
	}

	/**
	 * Return aCalendarDateAndTimeModel.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeModel calendarDateAndTimeModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarDateAndTimeModel());
		}
		return this.model();
	}

	/**
	 * Set aCalendarDateAndTimeModel.
	 *
	 * @param aCalendarDateAndTimeModel JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public void calendarDateAndTimeModel_(JunCalendarDateAndTimeModel aCalendarDateAndTimeModel) {
		this.model_(aCalendarDateAndTimeModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateAndTimeDialogViewAwt(this);
		} else {
			return new JunCalendarDateAndTimeDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Date and Time Dialog");
	}

}
