package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateAndTimeIntervalDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeIntervalDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateAndTimeIntervalDialog extends JunAbstractDialog<JunCalendarDateAndTimeIntervalModel> {

	/**
	 * Create a new instance of <code>JunCalendarDateAndTimeIntervalDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarDateAndTimeIntervalDialog() {
		super();
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request()
	 * 
	 * @return JunCalendarDateAndTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request() {
		return JunCalendarDateAndTimeIntervalDialog.Request_(null);
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_("Select date.")
	 * 
	 * @param messageString java.lang.String
	 * @return JunCalendarDateAndTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_(String messageString) {
		Calendar[][] dateArray = { { Calendar.getInstance(), Calendar.getInstance() }, { Calendar.getInstance(), Calendar.getInstance() } };
		return JunCalendarDateAndTimeIntervalDialog.Request_initialDatesAndTimes_(messageString, dateArray);
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_initialDate_("Select date.", Calendar.getInstance());
	 * 
	 * @param messageString java.lang.String
	 * @param dateArray java.util.Calendar[][]
	 * @return JunCalendarDateAndTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_initialDatesAndTimes_(String messageString, Calendar[][] dateArray) {
		return JunCalendarDateAndTimeIntervalDialog.Request_initialDatesAndTimes_onAccept_onCancel_(messageString, dateArray, null, null);
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_initialDate_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 *
	 * @param messageString java.lang.String
	 * @param dateArray java.util.Calendar[][]
	 * @param anAcceptBlock StBlockClosure
	 * @param aCancelBlock StBlockClosure
	 * @return JunCalendarDateAndTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_initialDatesAndTimes_onAccept_onCancel_(String messageString, Calendar[][] dateArray, StBlockClosure1<Boolean, JunCalendarDateAndTimeIntervalModel> anAcceptBlock,
			StBlockClosure1<Boolean, JunCalendarDateAndTimeIntervalModel> aCancelBlock) {
		JunCalendarDateAndTimeIntervalDialog calendarDateAndTimeIntervalDialog = new JunCalendarDateAndTimeIntervalDialog();
		if (!(dateArray == null)) {
			if (dateArray.length > 1) {
				calendarDateAndTimeIntervalDialog.calendarDateAndTimeIntervalModel_(JunCalendarDateAndTimeIntervalModel.From_to_(dateArray[0], dateArray[dateArray.length - 1]));
			}
		}
		Dialog dialog = calendarDateAndTimeIntervalDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarDateAndTimeIntervalDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarDateAndTimeIntervalDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateAndTimeIntervalDialog.accept().value()) {
			return calendarDateAndTimeIntervalDialog.calendarDateAndTimeIntervalModel().dateAndTimes();
		} else {
			return null;
		}
	}

	/**
	 * Return a JunCalendarDateAndTimeIntervalModel instance.
	 * 
	 * @return JunCalendarDateAndTimeIntervalModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeIntervalModel calendarDateAndTimeIntervalModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarDateAndTimeIntervalModel());
		}
		return this.model();
	}

	/**
	 * Set new instance of JunCalendarDateAndTimeIntervalModel.
	 * @param aCalendarDateAndTimeIntervalModel JunCalendarDateAndTimeIntervalModel
	 * @category accessing
	 */
	public void calendarDateAndTimeIntervalModel_(JunCalendarDateAndTimeIntervalModel aCalendarDateAndTimeIntervalModel) {
		this.model_(aCalendarDateAndTimeIntervalModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateAndTimeIntervalDialogViewAwt(this);
		} else {
			return new JunCalendarDateAndTimeIntervalDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Period (Date and Time) Dialog");
	}

}
