package jp.co.sra.jun.goodies.calendar;

import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateAndTimeIntervalModel class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeIntervalModel.java,v 9.11 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateAndTimeIntervalModel extends JunCalendarModel {
	protected JunCalendarDateAndTimeModel fromCalendarDateAndTimeModel;
	protected JunCalendarDateAndTimeModel toCalendarDateAndTimeModel;

	/**
	 * Create a new instance of <code>JunCalendarDateAndTimeIntervalModel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarDateAndTimeIntervalModel() {
		Calendar[] fromDateAndTime = { Calendar.getInstance(), Calendar.getInstance() };
		Calendar[] toDateAndTime = { Calendar.getInstance(), Calendar.getInstance() };
		this.fromCalendarDateAndTimeModel().dateAndTime_(fromDateAndTime);
		this.toCalendarDateAndTimeModel().dateAndTime_(toDateAndTime);
	}

	/**
	 * Create instance and set dateAndTime from fromDateAndTime to toDateAndTime.
	 *
	 * @param fromDateAndTime java.util.Calendar
	 * @param toDateAndTime java.util.Calendar
	 * @category Utilities
	 */
	public static JunCalendarDateAndTimeIntervalModel From_to_(Calendar[] fromDateAndTime, Calendar[] toDateAndTime) {
		JunCalendarDateAndTimeIntervalModel calendarDateAndTimeIntervalModel = new JunCalendarDateAndTimeIntervalModel();
		calendarDateAndTimeIntervalModel.fromCalendarDateAndTimeModel().dateAndTime_(fromDateAndTime);
		calendarDateAndTimeIntervalModel.toCalendarDateAndTimeModel().dateAndTime_(toDateAndTime);
		return calendarDateAndTimeIntervalModel;
	}

	/**
	 * Answer fromCalendarDateAndTimeModel.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeModel fromCalendarDateAndTimeModel() {
		if (fromCalendarDateAndTimeModel == null) {
			fromCalendarDateAndTimeModel = new JunCalendarDateAndTimeModel();
			final JunCalendarDateAndTimeIntervalModel this_ = this;
			fromCalendarDateAndTimeModel.validateBlock_(new StBlockClosure2<Calendar[], Calendar[], Calendar[]>() {
				public Calendar[] value(Calendar[] oldDateAndTime, Calendar[] newDateAndTime) {
					return this_.validateFromDateAndTime_with_(oldDateAndTime, newDateAndTime);
				}
			});
		}
		return fromCalendarDateAndTimeModel;
	}

	/**
	 * Answer fromCaalendarDateAndTimeModel and toCalendarDateAndTimeModel.
	 *
	 * @return Object[]
	 * @category accessing
	 */
	public JunCalendarDateAndTimeModel[] dateAndTimes() {
		JunCalendarDateAndTimeModel[] array = new JunCalendarDateAndTimeModel[2];
		array[0] = this.fromCalendarDateAndTimeModel();
		array[1] = this.toCalendarDateAndTimeModel();
		return array;
	}

	/**
	 * Answer toCalendarDateAndTimeModel.
	 *
	 * @return JunCalendarDateAndTimeModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeModel toCalendarDateAndTimeModel() {
		if (toCalendarDateAndTimeModel == null) {
			toCalendarDateAndTimeModel = new JunCalendarDateAndTimeModel();
			final JunCalendarDateAndTimeIntervalModel this_ = this;
			toCalendarDateAndTimeModel.validateBlock_(new StBlockClosure2<Calendar[], Calendar[], Calendar[]>() {
				public Calendar[] value(Calendar[] oldDateAndTime, Calendar[] newDateAndTime) {
					return this_.validateToDateAndTime_with_(oldDateAndTime, newDateAndTime);
				}
			});
		}
		return toCalendarDateAndTimeModel;
	}

	/**
	 * Default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		JunCalendarDateAndTimeIntervalView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new JunCalendarDateAndTimeIntervalViewAwt(this);
		} else {
			aView = new JunCalendarDateAndTimeIntervalViewSwing(this);
		}
		return aView;
	}

	/**
	 * Answer old dateAndTime.
	 *
	 * @param oldDateAndTime Calendar[]
	 * @param newDateAndTime Calendar[]
	 * @return Calendar[]
	 * @category private
	 */
	private Calendar[] validateFromDateAndTime_with_(Calendar[] oldDateAndTime, Calendar[] newDateAndTime) {
		if (newDateAndTime[0].before(this.toCalendarDateAndTimeModel().dateAndTime()[0])) {
			return newDateAndTime;
		}
		if (newDateAndTime[0].equals(this.toCalendarDateAndTimeModel().dateAndTime()[0]) && (newDateAndTime[1].before(this.toCalendarDateAndTimeModel().dateAndTime()[1]) || newDateAndTime[1].equals(this.toCalendarDateAndTimeModel().dateAndTime()[1]))) {
			return newDateAndTime;
		}
		return this.toCalendarDateAndTimeModel().dateAndTime();
	}

	/**
	 * Answer latest dateAndTime.
	 *
	 * @param oldDateAndTime Calendar[]
	 * @param newDateAndTime Calendar[]
	 * @return Calendar[]
	 * @category private
	 */
	private Calendar[] validateToDateAndTime_with_(Calendar[] oldDateAndTime, Calendar[] newDateAndTime) {
		if (newDateAndTime[0].after(this.fromCalendarDateAndTimeModel().dateAndTime()[0])) {
			return newDateAndTime;
		}
		if (newDateAndTime[0].equals(this.fromCalendarDateAndTimeModel().dateAndTime()[0]) && (newDateAndTime[1].after(this.fromCalendarDateAndTimeModel().dateAndTime()[1]) || newDateAndTime[1].equals(this.fromCalendarDateAndTimeModel().dateAndTime()[1]))) {
			return newDateAndTime;
		}
		return this.fromCalendarDateAndTimeModel().dateAndTime();
	}

	/**
	 * Print current date and time.
	 * 
	 * @return java.lang.String
	 * @category printing
	 */
	public java.lang.String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("(");
		buf.append(this.dateAndTimes()[0]);
		buf.append(") (");
		buf.append(this.dateAndTimes()[1]);
		buf.append(")");
		return buf.toString();
	}

	/**
	 * Return Window title.
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Period (Date and Time)");
	}
}
