package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateDialog extends JunAbstractDialog<JunCalendarDateModel> {

	/**
	 * Create a new instance of <code>JunCalendarDateDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarDateDialog() {
		super();
	}

	/**
	 * JunCalendarDateDialog.Request()
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request() {
		return JunCalendarDateDialog.Request_(null);
	}

	/**
	 * JunCalendarDateDialog.Request_("Select date.")
	 * @param messageString java.lang.String
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request_(String messageString) {
		return JunCalendarDateDialog.Request_initialDate_(messageString, Calendar.getInstance());
	}

	/**
	 * JunCalendarDateDialog.Request_initialDate_("Select date.", Calendar.getInstance());
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request_initialDate_(String messageString, Calendar aDate) {
		return JunCalendarDateDialog.Request_initialDate_onAccept_onCancel_(messageString, aDate, null, null);
	}

	/**
	 * JunCalendarDateDialog.Request_initialDate_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @param acceptBlock StBlockClosure
	 * @param cancelBlock StBlockClosure
	 * @return java.util.Calendar
	 * @category Utilities
	 */
	public static Calendar Request_initialDate_onAccept_onCancel_(String messageString, Calendar aDate, StBlockClosure1<Boolean, JunCalendarDateModel> acceptBlock, StBlockClosure1<Boolean, JunCalendarDateModel> cancelBlock) {
		JunCalendarDateDialog calendarDateDialog = new JunCalendarDateDialog();
		if (!(aDate == null)) {
			calendarDateDialog.calendarDateModel_(JunCalendarDateModel.Date_(aDate));
		}
		Dialog dialog = calendarDateDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(acceptBlock == null)) {
			calendarDateDialog.acceptBlock_(acceptBlock);
		}
		if (!(cancelBlock == null)) {
			calendarDateDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateDialog.accept().value()) {
			return calendarDateDialog.calendarDateModel().date();
		} else {
			return null;
		}
	}

	/**
	 * Return a JunCalendarDateModel instance.
	 * @return JunCalendarDateModel
	 * @category accessing
	 */
	public JunCalendarDateModel calendarDateModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarDateModel());
		}
		return this.model();
	}

	/**
	 * Set new instance of JunCalendarDateModel.
	 * @param aCalendarDateModel JunCalendarDateModel
	 * @category accessing
	 */
	public void calendarDateModel_(JunCalendarDateModel aCalendarDateModel) {
		this.model_(aCalendarDateModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateDialogViewAwt(this);
		} else {
			return new JunCalendarDateDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StSimpleDialog#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Date Dialog");
	}

}
