package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarDateIntervalDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateIntervalDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateIntervalDialog extends JunAbstractDialog<JunCalendarDateIntervalModel> {

	/**
	 * Create a new instance of <code>JunCalendarDateIntervalDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarDateIntervalDialog() {
		super();
	}

	/**
	 * JunCalendarDateIntervalDialog.Request()
	 * 
	 * @return JunCalendarDateModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateModel[] Request() {
		return JunCalendarDateIntervalDialog.Request_(null);
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_("Select date.")
	 * 
	 * @param messageString java.lang.String
	 * @return JunCalendarDateModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateModel[] Request_(String messageString) {
		Calendar[] dateArray = { Calendar.getInstance(), Calendar.getInstance() };
		return JunCalendarDateIntervalDialog.Request_initialDates_(messageString, dateArray);
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_initialDate_("Select date.", Calendar.getInstance());
	 * 
	 * @param messageString java.lang.String
	 * @param dateArray java.util.Calendar[]
	 * @return JunCalendarDateModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateModel[] Request_initialDates_(String messageString, Calendar[] dateArray) {
		return JunCalendarDateIntervalDialog.Request_initialDates_onAccept_onCancel_(messageString, dateArray, null, null);
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_initialDate_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 *
	 * @param messageString java.lang.String
	 * @param dateArray java.util.Calendar[]
	 * @param anAcceptBlock StBlockClosure
	 * @param aCancelBlock StBlockClosure
	 * @return JunCalendarDateModel[]
	 * @category Utilities
	 */
	public static JunCalendarDateModel[] Request_initialDates_onAccept_onCancel_(String messageString, Calendar[] dateArray, StBlockClosure1<Boolean, JunCalendarDateIntervalModel> anAcceptBlock,
			StBlockClosure1<Boolean, JunCalendarDateIntervalModel> aCancelBlock) {
		JunCalendarDateIntervalDialog calendarDateIntervalDialog = new JunCalendarDateIntervalDialog();
		if (!(dateArray == null)) {
			if (dateArray.length > 1) {
				calendarDateIntervalDialog.calendarDateIntervalModel_(JunCalendarDateIntervalModel.From_to_(dateArray[0], dateArray[dateArray.length - 1]));
			}
		}
		Dialog dialog = calendarDateIntervalDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarDateIntervalDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarDateIntervalDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateIntervalDialog.accept().value()) {
			return calendarDateIntervalDialog.calendarDateIntervalModel().dates();
		} else {
			return null;
		}
	}

	/**
	 * Return a JunCalendarDateIntervalModel instance.
	 * @return JunCalendarDateIntervalModel
	 * @category accessing
	 */
	public JunCalendarDateIntervalModel calendarDateIntervalModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarDateIntervalModel());
		}
		return this.model();
	}

	/**
	 * Set new instance of JunCalendarDateIntervalModel.
	 * @param aCalendarDateIntervalModel JunCalendarDateIntervalModel
	 * @category accessing
	 */
	public void calendarDateIntervalModel_(JunCalendarDateIntervalModel aCalendarDateIntervalModel) {
		this.model_(aCalendarDateIntervalModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateIntervalDialogViewAwt(this);
		} else {
			return new JunCalendarDateIntervalDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Period (Date) Dialog");
	}

}
