package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;

import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.dialog.JunAbstractDialogViewAwt;

/**
 * JunCalendarDateIntervalDialogViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateIntervalDialogViewAwt.java,v 9.10 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarDateIntervalDialogViewAwt extends JunAbstractDialogViewAwt implements JunCalendarDateIntervalDialogView {

	protected JunCalendarDateIntervalViewAwt calendarDateIntervalView;
	protected JunCalendarDateViewAwt fromView;
	protected JunCalendarDateViewAwt toView;

	/**
	 * Create a new instance of <code>JunCalendarDateIntervalDialogViewAwt</code> and initialize it.
	 * 
	 * @param newCalendarDateIntervalDialog JunCalendarDateIntervalDialog
	 * @category Instance creation
	 */
	public JunCalendarDateIntervalDialogViewAwt(JunCalendarDateIntervalDialog newCalendarDateIntervalDialog) {
		super(newCalendarDateIntervalDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog = new Dialog(new Frame(), this.getCalendarDateIntervalDialog().windowTitle(), true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateIntervalDialog.
	 * 
	 * @return JunCalendarDateIntervalDialog
	 * @category accessing
	 */
	public JunCalendarDateIntervalDialog getCalendarDateIntervalDialog() {
		return (JunCalendarDateIntervalDialog) this.model();
	}

	/**
	 * Build this component.
	 * 
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fromView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -102, 0.5f, -1));
		this.add(this.toView(), WindowSpecLayout.Position(0f, 1, 0.5f, 1, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setPreferredSize(new Dimension(400, 400));
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarDateIntervalViewAwt
	 * @category accessing
	 */
	protected JunCalendarDateIntervalViewAwt calendarDateIntervalView() {
		if (calendarDateIntervalView == null) {
			calendarDateIntervalView = new JunCalendarDateIntervalViewAwt(this.getCalendarDateIntervalDialog().calendarDateIntervalModel());
		}
		return calendarDateIntervalView;
	}

	/**
	 * Answer from date view.
	 *
	 * @return JunCalendarDateViewAwt
	 * @category accessing
	 */
	protected JunCalendarDateViewAwt fromView() {
		if (fromView == null) {
			fromView = new JunCalendarDateViewAwt(this.getCalendarDateIntervalDialog().calendarDateIntervalModel().fromCalendarDateModel());
		}
		return fromView;
	}

	/**
	 * Answer to date view.
	 *
	 * @return JunCalendarDateViewAwt
	 * @category accessing
	 */
	protected JunCalendarDateViewAwt toView() {
		if (toView == null) {
			toView = new JunCalendarDateViewAwt(this.getCalendarDateIntervalDialog().calendarDateIntervalModel().toCalendarDateModel());
		}
		return toView;
	}

}
