package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarTimeDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarTimeDialog extends JunAbstractDialog<JunCalendarTimeModel> {

	/**
	 * Create a new instance of <code>JunCalendarTimeDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarTimeDialog() {
		super();
	}

	/**
	 * JunCalendarTimeDialog.Request()
	 * 
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request() {
		return JunCalendarTimeDialog.Request_(null);
	}

	/**
	 * JunCalendarTimeDialog.Request_("Select date.")
	 * 
	 * @param messageString java.lang.String
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request_(String messageString) {
		return JunCalendarTimeDialog.Request_initialTime_(messageString, Calendar.getInstance());
	}

	/**
	 * JunCalendarTimeDialog.Request_initialTime_("Select date.", Calendar.getInstance());
	 * 
	 * @param messageString java.lang.String
	 * @param aTime java.util.Calendar
	 * @return Calendar
	 * @category Utilities
	 */
	public static Calendar Request_initialTime_(String messageString, Calendar aTime) {
		return JunCalendarTimeDialog.Request_initialTime_onAccept_onCancel_(messageString, aTime, null, null);
	}

	/**
	 * JunCalendarTimeDialog.Request_initialTime_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 *
	 * @param messageString java.lang.String
	 * @param aTime java.util.Calendar
	 * @param anAcceptBlock StBlockClosure
	 * @param aCancelBlock StBlockClosure
	 * @return java.util.Calendar
	 * @category Utilities
	 */
	public static Calendar Request_initialTime_onAccept_onCancel_(String messageString, Calendar aTime, StBlockClosure1<Boolean, JunCalendarTimeModel> anAcceptBlock, StBlockClosure1<Boolean, JunCalendarTimeModel> aCancelBlock) {
		JunCalendarTimeDialog calendarTimeDialog = new JunCalendarTimeDialog();
		if (!(aTime == null)) {
			calendarTimeDialog.calendarTimeModel_(JunCalendarTimeModel.Time_(aTime));
		}
		Dialog dialog = calendarTimeDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarTimeDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarTimeDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarTimeDialog.accept().value()) {
			return calendarTimeDialog.calendarTimeModel().time();
		} else {
			return null;
		}
	}

	/**
	 * Return a JunCalendarTimeModel instance.
	 * 
	 * @return JunCalendarTimeModel
	 * @category accessing
	 */
	public JunCalendarTimeModel calendarTimeModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarTimeModel());
		}
		return this.model();
	}

	/**
	 * Set new instance of JunCalendarTimeModel.
	 * 
	 * @param aCalendarTimeModel JunCalendarTimeModel
	 * @category accessing
	 */
	public void calendarTimeModel_(JunCalendarTimeModel aCalendarTimeModel) {
		this.model_(aCalendarTimeModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category accessing
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarTimeDialogViewAwt(this);
		} else {
			return new JunCalendarTimeDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Time Dialog");
	}

}
