package jp.co.sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarTimeIntervalDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeIntervalDialog.java,v 9.12 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarTimeIntervalDialog extends JunAbstractDialog<JunCalendarTimeIntervalModel> {

	/**
	 * Create a new instance of <code>JunCalendarTimeIntervalDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunCalendarTimeIntervalDialog() {
		super();
	}

	/**
	 * JunCalendarTimeIntervalDialog.Request()
	 * 
	 * @return JunCalendarTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarTimeModel[] Request() {
		return JunCalendarTimeIntervalDialog.Request_(null);
	}

	/**
	 * JunCalendarTimeIntervalDialog.Request_("Select date.")
	 * 
	 * @param messageString java.lang.String
	 * @return JunCalendarTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarTimeModel[] Request_(String messageString) {
		Calendar[] timeArray = { Calendar.getInstance(), Calendar.getInstance() };
		return JunCalendarTimeIntervalDialog.Request_initialTime_(messageString, timeArray);
	}

	/**
	 * JunCalendarTimeIntervalDialog.Request_initialTime_("Select date.", Calendar.getInstance());
	 * 
	 * @param messageString java.lang.String
	 * @param timeArray java.util.Calendar[]
	 * @return JunCalendarTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarTimeModel[] Request_initialTime_(String messageString, Calendar[] timeArray) {
		return JunCalendarTimeIntervalDialog.Request_initialTime_onAccept_onCancel_(messageString, timeArray, null, null);
	}

	/**
	 * JunCalendarTimeIntervalDialog.Request_initialTime_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 *
	 * @param messageString java.lang.String
	 * @param timeArray java.util.Calendar[]
	 * @param anAcceptBlock StBlockClosure
	 * @param aCancelBlock StBlockClosure
	 * @return JunCalendarTimeModel[]
	 * @category Utilities
	 */
	public static JunCalendarTimeModel[] Request_initialTime_onAccept_onCancel_(String messageString, Calendar[] timeArray, StBlockClosure1<Boolean, JunCalendarTimeIntervalModel> anAcceptBlock,
			StBlockClosure1<Boolean, JunCalendarTimeIntervalModel> aCancelBlock) {
		JunCalendarTimeIntervalDialog calendarTimeIntervalDialog = new JunCalendarTimeIntervalDialog();
		if (!(timeArray == null)) {
			if (timeArray.length > 1) {
				calendarTimeIntervalDialog.calendarTimeIntervalModel_(JunCalendarTimeIntervalModel.From_to_(timeArray[0], timeArray[timeArray.length - 1]));
			}
		}
		Dialog dialog = calendarTimeIntervalDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarTimeIntervalDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarTimeIntervalDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarTimeIntervalDialog.accept().value()) {
			return calendarTimeIntervalDialog.calendarTimeIntervalModel().times();
		} else {
			return null;
		}
	}

	/**
	 * Return a JunCalendarTimeIntervalModel instance.
	 * 
	 * @return JunCalendarTimeIntervalModel
	 * @category accessing
	 */
	public JunCalendarTimeIntervalModel calendarTimeIntervalModel() {
		if (this.model() == null) {
			this.model_(new JunCalendarTimeIntervalModel());
		}
		return this.model();
	}

	/**
	 * Set new instance of JunCalendarTimeIntervalModel.
	 * 
	 * @param aCalendarTimeIntervalModel JunCalendarTimeIntervalModel
	 * @category accessing
	 */
	public void calendarTimeIntervalModel_(JunCalendarTimeIntervalModel aCalendarTimeIntervalModel) {
		this.model_(aCalendarTimeIntervalModel);
	}

	/**
	 * Answer a default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category accessing
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarTimeIntervalDialogViewAwt(this);
		} else {
			return new JunCalendarTimeIntervalDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Period (Time) Dialog");
	}

}
