package jp.co.sra.jun.goodies.calendar;

import java.util.Calendar;

import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunCalendarTimeIntervalModel class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   2008/06/09 (by Nobuto Matsubara)
 *  @version   758 (with StPL9.7) based on Jun472 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeIntervalModel.java,v 9.11 2010/03/09 09:16:59 nisinaka Exp $
 */
public class JunCalendarTimeIntervalModel extends JunCalendarModel {
	protected JunCalendarTimeModel fromCalendarTimeModel;
	protected JunCalendarTimeModel toCalendarTimeModel;

	/**
	 * Create instance and set time from fromTime to toTime.
	 *
	 * @param fromTime java.util.Calendar
	 * @param toTime java.util.Calendar
	 * @category Utilities
	 */
	public static JunCalendarTimeIntervalModel From_to_(Calendar fromTime, Calendar toTime) {
		JunCalendarTimeIntervalModel calendarTimeIntervalModel = new JunCalendarTimeIntervalModel();
		calendarTimeIntervalModel.fromCalendarTimeModel().time_(fromTime);
		calendarTimeIntervalModel.toCalendarTimeModel().time_(toTime);
		return calendarTimeIntervalModel;
	}

	/**
	 * Answer fromCalendarTimeModel.
	 *
	 * @return JunCalendarTimeModel
	 * @category accessing
	 */
	public JunCalendarTimeModel fromCalendarTimeModel() {
		if (fromCalendarTimeModel == null) {
			fromCalendarTimeModel = new JunCalendarTimeModel();
			final JunCalendarTimeIntervalModel this_ = this;
			fromCalendarTimeModel.validateBlock_(new StBlockClosure2<Calendar, Calendar, Calendar>() {
				public Calendar value(Calendar oldTime, Calendar newTime) {
					return this_.validateFromTime_with_(oldTime, newTime);
				}
			});
		}
		return fromCalendarTimeModel;
	}

	/**
	 * Answer fromCaalendarTimeModel and toCalendarTimeModel.
	 *
	 * @return JunCalendarTimeModel[]
	 * @category accessing
	 */
	public JunCalendarTimeModel[] times() {
		JunCalendarTimeModel[] array = new JunCalendarTimeModel[2];
		array[0] = this.fromCalendarTimeModel();
		array[1] = this.toCalendarTimeModel();
		return array;
	}

	/**
	 * Answer toCalendarTimeModel.
	 *
	 * @return JunCalendarTimeModel
	 * @category accessing
	 */
	public JunCalendarTimeModel toCalendarTimeModel() {
		if (toCalendarTimeModel == null) {
			toCalendarTimeModel = new JunCalendarTimeModel();
			final JunCalendarTimeIntervalModel this_ = this;
			toCalendarTimeModel.validateBlock_(new StBlockClosure2<Calendar, Calendar, Calendar>() {
				public Calendar value(Calendar oldTime, Calendar newTime) {
					return this_.validateToTime_with_(oldTime, newTime);
				}
			});
		}
		return toCalendarTimeModel;
	}

	/**
	 * Default view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		JunCalendarTimeIntervalView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new JunCalendarTimeIntervalViewAwt(this);
		} else {
			aView = new JunCalendarTimeIntervalViewSwing(this);
		}
		return aView;
	}

	/**
	 * Answer old time.
	 *
	 * @param oldTime java.util.Calendar
	 * @param newTime java.util.Calendar
	 * @return java.util.Calendar
	 * @category private
	 */
	private Calendar validateFromTime_with_(Calendar oldTime, Calendar newTime) {
		if (newTime.before(this.toCalendarTimeModel().time())) {
			return newTime;
		}
		return this.toCalendarTimeModel().time();
	}

	/**
	 * Answer latest time.
	 *
	 * @param oldTime java.util.Calendar
	 * @param newTime java.util.Calendar
	 * @return java.util.Calendar
	 * @category private
	 */
	private Calendar validateToTime_with_(Calendar oldTime, Calendar newTime) {
		if (newTime.after(this.fromCalendarTimeModel().time())) {
			return newTime;
		}
		return this.fromCalendarTimeModel().time();
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Period (Time)");
	}
}
