package jp.co.sra.jun.goodies.colors;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StColorValue;
import jp.co.sra.smalltalk.StSymbol;

/**
 * JunColorBarCollection class
 * 
 *  @author    nisinaka
 *  @created   2004/06/14 (by nisinaka)
 *  @updated   2008/04/16 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun452 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorBarCollection.java,v 9.11 2010/03/09 09:17:00 nisinaka Exp $
 */
public class JunColorBarCollection extends JunColorBarModel {

	protected List<Color> colorCollection;

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		colorCollection = null;
	}

	/**
	 * Answer my current color collection.
	 * 
	 * @return java.util.ArrayList
	 * @category color accessing
	 */
	public List<Color> colorCollection() {
		if (colorCollection == null) {
			colorCollection = this.defaultColorCollection();
		}
		return colorCollection;
	}

	/**
	 * Set my new color collection.
	 * 
	 * @param newCollection java.util.ArrayList&lt;java.awt.Color&gt;
	 * @category color accessing
	 */
	public void colorCollection_(ArrayList<Color> newCollection) {
		colorCollection = newCollection;
		this.changed_($("color"));
	}

	/**
	 * Set my new color collection.
	 * 
	 * @param newCollection java.awt.Color[]
	 * @category color accessing
	 */
	public void colorCollection_(Color[] newCollection) {
		ArrayList<Color> anArrayList = null;
		if (newCollection != null) {
			anArrayList = new ArrayList<Color>(newCollection.length);
			for (int i = 0; i < newCollection.length; i++) {
				anArrayList.add(newCollection[i]);
			}
		}
		this.colorCollection_(anArrayList);
	}

	/**
	 * Add a color to the collection.
	 * 
	 * @param aColor java.awt.Color
	 * @category color accessing
	 */
	public void addColor_(Color aColor) {
		this.colorCollection().add(aColor);
		this.changed_($("color"));
	}

	/**
	 * Answer my default color collection.
	 * 
	 * @return java.util.ArrayList&lt;java.awt.Color&gt;
	 * @category defaults
	 */
	protected ArrayList<Color> defaultColorCollection() {
		List<StSymbol> colorNames = StColorValue.ConstantNames();
		ArrayList<Color> aCollection = new ArrayList<Color>(colorNames.size());
		for (StSymbol colorName : colorNames) {
			aCollection.add(StColorValue._GetColorByName_(colorName.toString()));
		}
		return aCollection;
	}

	/**
	 * Answer the color which correspond to the normalized value.
	 * 
	 * @param normalizedValue double
	 * @return java.awt.Color
	 * @see jp.co.sra.jun.goodies.colors.JunColorBarModel#getColor_(double)
	 * @category private
	 */
	protected Color getColor_(double normalizedValue) {
		List<Color> aCollection = this.colorCollection();
		if (aCollection.isEmpty()) {
			return Color.white;
		}

		int index = (int) (aCollection.size() * normalizedValue);
		if (index >= aCollection.size()) {
			index = aCollection.size() - 1;
		}
		return aCollection.get(index);
	}

}
