package jp.co.sra.jun.goodies.colors;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunColorBarDialog class
 * 
 *  @author    nisinaka
 *  @created   2004/06/14 (by nisinaka)
 *  @updated   2008/05/14 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun651 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorBarDialog.java,v 9.12 2010/03/09 09:17:00 nisinaka Exp $
 */
public class JunColorBarDialog extends JunAbstractDialog<JunColorBarModel> {

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request() {
		return Request($String("Color Bar Dialog"));
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString) {
		return Request(messageString, 0.5);
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @param aValue double
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, double aValue) {
		return Request(messageString, aValue, 0, 1);
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @param aValue double
	 * @param firstMarker double
	 * @param lastMarker double
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, double aValue, double firstMarker, double lastMarker) {
		JunColorBarHSB colorBar = new JunColorBarHSB();
		colorBar.value_(aValue);
		colorBar.firstMarker_(firstMarker);
		colorBar.lastMarker_(lastMarker);
		return Request(messageString, colorBar, null, null);
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @param colorBarModel jp.co.sra.jun.goodies.colors.JunColorBarModel
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunColorBarModel colorBarModel) {
		return Request(messageString, colorBarModel, null, null);
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @param colorBarModel jp.co.sra.jun.goodies.colors.JunColorBarModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object[]
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunColorBarModel colorBarModel, StBlockClosure<Boolean, JunColorBarModel, ?, ?> acceptBlock, StBlockClosure<Boolean, JunColorBarModel, ?, ?> cancelBlock) {
		JunColorBarDialog colorBarDialog = new JunColorBarDialog();
		if (colorBarModel != null) {
			colorBarDialog.colorBarModel_(colorBarModel);
		}
		Dialog dialog = colorBarDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			colorBarDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			colorBarDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		Object[] answer = null;
		if (colorBarDialog.accept().value()) {
			answer = new Object[] { colorBarDialog.colorBarModel().color(), colorBarDialog.colorBarModel().value(), colorBarDialog.colorBarModel().interval() };
		}
		return answer;
	}

	/**
	 * Open a dialog to request a color information with a color bar model.
	 * 
	 * @param messageString java.lang.String
	 * @param colorBarModel jp.co.sra.jun.goodies.colors.JunColorBarModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunColorBarModel colorBarModel, StBlockClosure1<Boolean, JunColorBarModel> acceptBlock, StBlockClosure1<Boolean, JunColorBarModel> cancelBlock) {
		JunColorBarDialog colorBarDialog = new JunColorBarDialog();
		if (colorBarModel != null) {
			colorBarDialog.colorBarModel_(colorBarModel);
		}
		Dialog dialog = colorBarDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			colorBarDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			colorBarDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		Object[] answer = null;
		if (colorBarDialog.accept().value()) {
			answer = new Object[] { colorBarDialog.colorBarModel().color(), colorBarDialog.colorBarModel().value(), colorBarDialog.colorBarModel().interval() };
		}
		return answer;
	}

	/**
	 * Answer my current color bar model.
	 * 
	 * @return jp.co.sra.jun.goodies.colors.JunColorBarModel
	 * @category accessing
	 */
	public JunColorBarModel colorBarModel() {
		if (this.model() == null) {
			this.model_(new JunColorBarHSB());
		}
		return this.model();
	}

	/**
	 * Set my new color bar model.
	 * 
	 * @param newColorBarModel jp.co.sra.jun.goodies.colors.JunColorBarModel
	 * @category accessing
	 */
	public void colorBarModel_(JunColorBarModel newColorBarModel) {
		this.model_(newColorBarModel);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StSimpleDialog#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunColorBarDialogViewAwt(this);
		} else {
			return new JunColorBarDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Color Bar Dialog");
	}
}
