package jp.co.sra.jun.goodies.colors;

import java.awt.Color;
import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunColorChoiceDialog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/04/02 (by Mitsuhiro Asada)
 *  @updated   2008/05/14 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorChoiceDialog.java,v 9.12 2010/03/09 09:17:01 nisinaka Exp $
 */
public class JunColorChoiceDialog extends JunAbstractDialog<JunColorChoiceModel> {

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @category Utilities
	 */
	public static Color Request() {
		return Request_(null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @category Utilities
	 */
	public static Color Request_(String messageString) {
		return Request_colorChoice_(messageString, new JunColorChoiceSBH());
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorValue java.lang.Color
	 * @category Utilities
	 */
	public static Color Request_color_(String messageString, Color colorValue) {
		JunColorChoiceModel colorChoiceModel = new JunColorChoiceSBH();
		colorChoiceModel.color_(colorValue);
		return Request_colorChoice_onAccept_onCancel_(messageString, colorChoiceModel, (StBlockClosure1<Boolean, JunColorChoiceModel>) null, (StBlockClosure1<Boolean, JunColorChoiceModel>) null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorChoiceModel jp.co.sra.jun.goodies.colors.JunColorChoiceModel
	 * @category Utilities
	 */
	public static Color Request_colorChoice_(String messageString, JunColorChoiceModel colorChoiceModel) {
		return Request_colorChoice_onAccept_onCancel_(messageString, colorChoiceModel, (StBlockClosure1<Boolean, JunColorChoiceModel>) null, (StBlockClosure1<Boolean, JunColorChoiceModel>) null);
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorChoiceModel jp.co.sra.jun.goodies.colors.JunColorChoiceModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Color Request_colorChoice_onAccept_onCancel_(String messageString, JunColorChoiceModel colorChoiceModel, StBlockClosure<Boolean, JunColorChoiceModel, ?, ?> acceptBlock, StBlockClosure<Boolean, JunColorChoiceModel, ?, ?> cancelBlock) {
		JunColorChoiceDialog colorChoiceDialog = new JunColorChoiceDialog();
		if (colorChoiceModel != null) {
			colorChoiceDialog.colorChoiceModel_(colorChoiceModel);
		}
		Dialog dialog = colorChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			colorChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			colorChoiceDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(dialog);
		if (colorChoiceDialog.accept().value()) {
			return colorChoiceDialog.colorChoiceModel().color();
		} else {
			return null;
		}
	}

	/**
	 * Open a color choice dialog and let a user to choose.
	 * 
	 * @return java.awt.Color
	 * @param messageString java.lang.String
	 * @param colorChoiceModel jp.co.sra.jun.goodies.colors.JunColorChoiceModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category Utilities
	 */
	public static Color Request_colorChoice_onAccept_onCancel_(String messageString, JunColorChoiceModel colorChoiceModel, StBlockClosure1<Boolean, JunColorChoiceModel> acceptBlock, StBlockClosure1<Boolean, JunColorChoiceModel> cancelBlock) {
		JunColorChoiceDialog colorChoiceDialog = new JunColorChoiceDialog();
		if (colorChoiceModel != null) {
			colorChoiceDialog.colorChoiceModel_(colorChoiceModel);
		}
		Dialog dialog = colorChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			colorChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			colorChoiceDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(dialog);
		if (colorChoiceDialog.accept().value()) {
			return colorChoiceDialog.colorChoiceModel().color();
		} else {
			return null;
		}
	}

	/**
	 * Answer the current color choice model.
	 *
	 * @return jp.co.sra.jun.goodies.colors.JunColorChoiceModel
	 * @category accessing
	 */
	public JunColorChoiceModel colorChoiceModel() {
		if (this.model() == null) {
			this.model_(new JunColorChoiceSBH());
		}
		return this.model();
	}

	/**
	 * Set the current color choice model.
	 *
	 * @param choiceModel jp.co.sra.jun.goodies.colors.JunColorChoiceModel
	 * @category accessing
	 */
	public void colorChoiceModel_(JunColorChoiceModel choiceModel) {
		this.model_(choiceModel);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunColorChoiceDialogViewAwt(this);
		} else {
			return new JunColorChoiceDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Color Choice Dialog");
	}
}
