package jp.co.sra.jun.goodies.colors;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JPanel;

import jp.co.sra.jun.system.dialog.JunAbstractDialogViewSwing;

/**
 * JunColorChoiceDialogViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/04/02 (by Mitsuhiro Asada)
 *  @updated   2008/05/14 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorChoiceDialogViewSwing.java,v 9.11 2010/03/09 09:17:01 nisinaka Exp $
 */
public class JunColorChoiceDialogViewSwing extends JunAbstractDialogViewSwing implements JunColorChoiceDialogView {
	protected JunColorChoiceViewSwing colorChoiceView;

	/**
	 * Create a new instance of JunColorChoiceDialogViewSwing and initialize it with the JunColorChoiceDialog.
	 * 
	 * @param newColorChoiceDialog jp.co.sra.jun.goodies.colors.JunColorChoiceDialog
	 * @category Instance creation
	 */
	public JunColorChoiceDialogViewSwing(JunColorChoiceDialog newColorChoiceDialog) {
		super(newColorChoiceDialog);
	}

	/**
	 * Answer my model as a JunColorChoiceDialog.
	 * 
	 * @return jp.co.sra.jun.goodies.colors.JunColorChoiceDialog
	 * @see jp.co.sra.jun.goodies.colors.JunColorChoiceDialogView#getColorChoiceDialog()
	 * @category model accessing
	 */
	public JunColorChoiceDialog getColorChoiceDialog() {
		return (JunColorChoiceDialog) this.model();
	}

	/**
	 * Answer the panel on accept/cancel buttons.
	 * 
	 * @return javax.swing.JPanel
	 * @category interface opening
	 */
	protected JPanel _acceptCancelButtons() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		panel.add(this.acceptButton());
		panel.add(this.cancelButton());

		return panel;
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());
		this.add(this.colorChoiceView(), BorderLayout.CENTER);
		this.add(this._acceptCancelButtons(), BorderLayout.SOUTH);

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my color choice view.
	 *
	 * @return jp.co.sra.jun.goodies.colors.JunColorChoiceViewSwing
	 * @category interface opening
	 */
	protected JunColorChoiceViewSwing colorChoiceView() {
		if (colorChoiceView == null) {
			colorChoiceView = new JunColorChoiceViewSwing(this.getColorChoiceDialog().colorChoiceModel());
		}
		return colorChoiceView;
	}
}
