package jp.co.sra.jun.goodies.files;

import java.io.File;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunFileModelTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2003/03/05 (by nisinaka)
 *  @updated   2008/04/22 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun433 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileModelTestExamples.java,v 9.9 2010/03/09 09:17:07 nisinaka Exp $
 */
public class JunFileModelTestExamples extends JunTestExamples {

	/**
	 * Example1: Open a JunFileModel.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunFileModel fileModel = new JunFileModel();
		fileModel.open();
		return true;
	}

	/**
	 * Example2: Create a FileModel with computing a new value and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		JunFileModel fileModel = new JunFileModel();
		fileModel.compute_(new StBlockClosure1<File, File>() {
			public File value(File newValue) {
				System.out.println(newValue);
				return null;
			}
		});
		fileModel.open();
		return true;
	}

	/**
	 * Example3: Create a FileModel with computing a new value, assign an initial value, and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		JunFileModel fileModel = new JunFileModel();
		fileModel.compute_(new StBlockClosure1<File, File>() {
			public File value(File newValue) {
				System.out.println(newValue);
				return null;
			}
		});
		fileModel.value_(new File(JunFileModel.DefaultDirectory(), "zzz.st"));
		fileModel.open();
		return true;
	}

	/**
	 * Example4: Create a FileModel with file type specified, computing a new value, assign an initial value, and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] {
				JunFileModel.FileType.All("all files (*.*)"),
				new JunFileModel.FileType("java files (*.java)", new String[] { "*.java", "*.JAVA" }),
				new JunFileModel.FileType("text files (*.txt)", new String[] { "*.txt", "*.TXT" }),
				new JunFileModel.FileType("html files (*.html)", new String[] { "*.html", "*.HTML", "*.htm", "*.HTM" }) };
		JunFileModel fileModel = new JunFileModel();
		fileModel.fileTypes_(fileTypes);
		fileModel.fileType().value_(fileTypes[3]);
		fileModel.compute_(new StBlockClosure1<File, File>() {
			public File value(File newValue) {
				System.out.println(newValue);
				return null;
			}
		});
		fileModel.open();
		return true;
	}

	/**
	 * Example5: Create a FileModel with file type specified, computing a new value, assign an initial value, and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] {
				JunFileModel.FileType.All("all files (*.*)"),
				new JunFileModel.FileType("java files (*.java)", new String[] { "*.java", "*.JAVA" }),
				new JunFileModel.FileType("text files (*.txt)", new String[] { "*.txt", "*.TXT" }),
				new JunFileModel.FileType("html files (*.html)", new String[] { "*.html", "*.HTML", "*.htm", "*.HTM" }) };
		JunFileModel fileModel = new JunFileModel();
		fileModel.value_(new File(JunFileModel.DefaultDirectory(), "zzz.html"));
		fileModel.fileTypes_(fileTypes);
		fileModel.fileType().value_(fileTypes[3]);
		fileModel.compute_(new StBlockClosure1<File, File>() {
			public File value(File newValue) {
				System.out.println(newValue);
				return null;
			}
		});
		fileModel.open();
		return true;
	}

	/**
	 * Execute all examples.
	 *
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunFileModelTestExamples();
	}
}
