package jp.co.sra.jun.goodies.gauge;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunGaugeDialog class
 * 
 *  @author    nisinaka
 *  @created   2004/04/14 (by nisinaka)
 *  @updated   2008/05/14 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun492 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunGaugeDialog.java,v 9.11 2010/03/09 09:17:10 nisinaka Exp $
 */
public class JunGaugeDialog extends JunAbstractDialog<JunGaugeModel> {

	/**
	 * Open JunGaugeDialog to request a value.
	 * 
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request() {
		return Request($String("Select a value."));
	}

	/**
	 * Open a JunGaugeDialog to request a value.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString) {
		return Request(messageString, new JunGaugeModel());
	}

	/**
	 * Open a JunGaugeDialog to request a value with the speicified gauge model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString 
	 * @param aGaugeModel
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, JunGaugeModel aGaugeModel) {
		return Request(messageString, aGaugeModel, null, null);
	}

	/**
	 * Open a JunGaugeDialog to request a value with the specified gauge model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param aGaugeModel jp.co.sra.jun.goodies.gauge.JunGaugeModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Object Request(String messageString, JunGaugeModel aGaugeModel, StBlockClosure<Boolean, JunGaugeModel, ?, ?> acceptBlock, StBlockClosure<Boolean, JunGaugeModel, ?, ?> cancelBlock) {
		JunGaugeDialog gaugeDialog = new JunGaugeDialog();
		if (aGaugeModel != null) {
			gaugeDialog.gaugeModel_(aGaugeModel);
		}
		Dialog aDialog = gaugeDialog.allButOpenDialog();
		if (messageString != null) {
			aDialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			gaugeDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			gaugeDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(aDialog);

		return gaugeDialog.accept().value() ? gaugeDialog.gaugeModel().value() : null;
	}

	/**
	 * Open a JunGaugeDialog to request a value with the specified gauge model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param aGaugeModel jp.co.sra.jun.goodies.gauge.JunGaugeModel
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, JunGaugeModel aGaugeModel, StBlockClosure1<Boolean, JunGaugeModel> acceptBlock, StBlockClosure1<Boolean, JunGaugeModel> cancelBlock) {
		JunGaugeDialog gaugeDialog = new JunGaugeDialog();
		if (aGaugeModel != null) {
			gaugeDialog.gaugeModel_(aGaugeModel);
		}
		Dialog aDialog = gaugeDialog.allButOpenDialog();
		if (messageString != null) {
			aDialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			gaugeDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			gaugeDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(aDialog);

		return gaugeDialog.accept().value() ? gaugeDialog.gaugeModel().value() : null;
	}

	/**
	 * Open a JunGaugeDialog to request a value with the specified initial value.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param value double
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, double value) {
		return Request(messageString, new JunGaugeModel(value));
	}

	/**
	 * Open a JunGaugeDialog to request a value with the specified initial value.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param value double
	 * @param fromValue double
	 * @param toValue double
	 * @param roundValue double
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static Object Request(String messageString, double value, double fromValue, double toValue, double roundValue) {
		return Request(messageString, new JunGaugeModel(value, fromValue, toValue, roundValue));
	}

	/**
	 * Answer my current gauge model.
	 * 
	 * @return jp.co.sra.jun.goodies.gauge.JunGaugeModel
	 * @category accessing
	 */
	public JunGaugeModel gaugeModel() {
		if (this.model() == null) {
			this.model_(new JunGaugeModel());
		}
		return this.model();
	}

	/**
	 * Set my new gauge model.
	 * 
	 * @param newGaugeModel jp.co.sra.jun.goodies.gauge.JunGaugeModel
	 * @category accessing
	 */
	public void gaugeModel_(JunGaugeModel newGaugeModel) {
		this.model_(newGaugeModel);
	}

	/**
	 * Answer my default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunGaugeDialogViewAwt(this);
		} else {
			return new JunGaugeDialogViewSwing(this);
		}
	}

}
