package jp.co.sra.jun.goodies.huffman;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;
import java.util.TreeMap;

import jp.co.sra.smalltalk.StObject;

/**
 * JunHuffmanArray class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/10/17 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun707 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunHuffmanArray.java,v 9.7 2010/03/09 09:17:10 nisinaka Exp $
 */
public class JunHuffmanArray<E> extends StObject {
	protected Map<Integer, E> array;

	/**
	 * Create a new instance of <code>JunHuffmanArray</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunHuffmanArray() {
		this(0);
	}

	/**
	 * Create a new instance of <code>JunHuffmanArray</code> and initialize it.
	 * 
	 * @param size int
	 * @category Instance creation
	 */
	public JunHuffmanArray(int size) {
		super();
		this.initialize_(size);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @param size int
	 * @category initialize-release
	 */
	protected void initialize_(int size) {
		// array = new TreeMap<Integer, E>(size);
		array = new TreeMap<Integer, E>();
	}

	/**
	 * Answer the element at the specified position in the receiver.
	 * 
	 * @param index
	 * @return java.lang.Object
	 * @throws IndexOutOfBoundsException if index is out of range <tt>(index &lt; 0 || index &gt;= size())</tt>.
	 * @category accessing
	 */
	public E at_(int index) {
		return array.get(index);
	}

	/**
	 * Replaces the element at the specified position in the receiver with the specified element.
	 * 
	 * @param index int
	 * @param value java.lang.Object
	 * @category accessing
	 */
	public void at_put_(int index, E value) {
		array.put(index, value);
	}

	/**
	 * Answer the number of elements in the receiver.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return array.size();
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException if failed.
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	@Override
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("#(");
		aWriter.write("\n");
		for (int i = 0; i < this.size(); i++) {
			aWriter.write(Integer.toString(i));
			aWriter.write(": ");
			E element = this.at_(i);
			if (element instanceof StObject) {
				((StObject) element).printOn_(aWriter);
			} else {
				aWriter.write(element.toString());
			}
			aWriter.write("\n");
		}
		aWriter.write(")");
		aWriter.write("\n");
	}
}
