package jp.co.sra.jun.goodies.huffman;

import java.awt.Rectangle;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import jp.co.sra.smalltalk.StImage;

import jp.co.sra.jun.goodies.utilities.JunImageUtility;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.support.JunSystem;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunHuffmanCoderTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/10/24 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun707 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunHuffmanCoderTestExamples.java,v 9.8 2010/03/09 09:17:10 nisinaka Exp $
 */
public class JunHuffmanCoderTestExamples extends JunTestExamples {
	/**
	 * Example1: Compress and expand with "AAAAAAAAAAAAAAABBBBBBBCCCCCCDDDDDDEEEEEE".
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		byte[] inputBytes = null;
		ByteArrayOutputStream aStream = null;
		try {
			aStream = new ByteArrayOutputStream();
			for (int i = 0; i < 15; i++) {
				aStream.write('A');
			}
			for (int i = 0; i < 7; i++) {
				aStream.write('B');
			}
			for (int i = 0; i < 6; i++) {
				aStream.write('C');
			}
			for (int i = 0; i < 6; i++) {
				aStream.write('D');
			}
			for (int i = 0; i < 5; i++) {
				aStream.write('E');
			}
			aStream.flush();
			aStream.close();
			inputBytes = aStream.toByteArray();
			aStream = null;
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.flush();
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(inputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Compress_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		System.out.print(Integer.toString(inputBytes.length));
		System.out.print(" --> ");
		System.out.println(Integer.toString(outputBytes.length));

		try {
			inputStream = new ByteArrayInputStream(outputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Expand_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		for (int i = 0; i < outputBytes.length; i++) {
			System.out.print((char) outputBytes[i]);
		}
		System.out.println();

		return true;
	}

	/**
	 * Example2: Compress and expand with <code>JunSystem.Copyright()</code>.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		byte[] inputBytes = null;
		ByteArrayOutputStream aStream = null;
		try {
			aStream = new ByteArrayOutputStream();
			String copyrightString = JunSystem.Copyright();
			for (int i = 0; i < copyrightString.length(); i++) {
				aStream.write(copyrightString.charAt(i));
			}
			aStream.flush();
			aStream.close();
			inputBytes = aStream.toByteArray();
			aStream = null;
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.flush();
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(inputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Compress_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		System.out.print(Integer.toString(inputBytes.length));
		System.out.print(" --> ");
		System.out.println(Integer.toString(outputBytes.length));

		try {
			inputStream = new ByteArrayInputStream(outputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Expand_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		for (int i = 0; i < outputBytes.length; i++) {
			System.out.print((char) outputBytes[i]);
		}
		System.out.println();

		return true;
	}

	/**
	 * Example3: Compress and expand with <code>StImage</code>.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		StImage anImage = new StImage(JunImageUtility.FromDisplay_(new Rectangle(0, 0, 512, 512)));

		byte[] inputBytes = null;
		ObjectOutputStream aStream = null;
		try {
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			aStream = new ObjectOutputStream(byteArrayStream);
			aStream.writeObject(anImage);
			aStream.flush();
			aStream.close();
			aStream = null;
			inputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.flush();
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(inputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Compress_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		System.out.print(Integer.toString(inputBytes.length));
		System.out.print(" --> ");
		System.out.println(Integer.toString(outputBytes.length));

		try {
			inputStream = new ByteArrayInputStream(outputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Expand_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		ObjectInputStream objectInputStream = null;
		try {
			objectInputStream = new ObjectInputStream(new ByteArrayInputStream(outputBytes));
			StImage newImage = (StImage) objectInputStream.readObject();
			newImage._display();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return true;
	}

	/**
	 * Example4: Compress and expand with <code>JunOpenGL3dObject.Smalltalk()</code>.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		JunOpenGL3dObject aBody = JunOpenGL3dObject.Smalltalk();

		byte[] inputBytes = null;
		ObjectOutputStream aStream = null;
		try {
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			aStream = new ObjectOutputStream(byteArrayStream);
			aStream.writeObject(aBody);
			aStream.flush();
			aStream.close();
			aStream = null;
			inputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.flush();
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(inputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Compress_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		System.out.print(Integer.toString(inputBytes.length));
		System.out.print(" --> ");
		System.out.println(Integer.toString(outputBytes.length));

		try {
			inputStream = new ByteArrayInputStream(outputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Expand_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		ObjectInputStream objectInputStream = null;
		try {
			objectInputStream = new ObjectInputStream(new ByteArrayInputStream(outputBytes));
			JunOpenGL3dObject newBody = (JunOpenGL3dObject) objectInputStream.readObject();
			newBody.show();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return true;
	}

	/**
	 * Execute all examples.
	 *
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunHuffmanCoderTestExamples();
	}
}
