package jp.co.sra.jun.goodies.huffman;

import java.io.IOException;
import java.io.Writer;

import jp.co.sra.smalltalk.StObject;

/**
 * JunHuffmanNode class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/10/17 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun707 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunHuffmanNode.java,v 9.7 2010/03/09 09:17:10 nisinaka Exp $
 */
public class JunHuffmanNode extends StObject {
	protected int count;
	protected int child0;
	protected int child1;

	/**
	 * Create a new instance of <code>JunHuffmanNode</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunHuffmanNode() {
		super();
		this.initialize();
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		count = 0;
		child0 = Integer.MIN_VALUE;
		child1 = Integer.MIN_VALUE;
	}

	/**
	 * Answer the receiver's child0.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int child0() {
		return child0;
	}

	/**
	 * Set the receiver's child0.
	 * 
	 * @param nodeIndex int
	 * @category accessing
	 */
	public void child0_(int nodeIndex) {
		child0 = nodeIndex;
	}

	/**
	 * Answer the receiver's child1.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int child1() {
		return child1;
	}

	/**
	 * Set the receiver's child1.
	 * 
	 * @param nodeIndex int
	 * @category accessing
	 */
	public void child1_(int nodeIndex) {
		child1 = nodeIndex;
	}

	/**
	 * Answer the receiver's count.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int count() {
		return count;
	}

	/**
	 * Set the receiver's count.
	 * 
	 * @param count int
	 * @category accessing
	 */
	public void count_(int anInteger) {
		count = anInteger;
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException if failed.
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	@Override
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("(");
		aWriter.write(Integer.toString(this.count()));
		aWriter.write(", ");
		aWriter.write(Integer.toString(this.child0()));
		aWriter.write(", ");
		aWriter.write(Integer.toString(this.child1()));
		aWriter.write(")");
	}
}
