package jp.co.sra.jun.goodies.huffman;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

import jp.co.sra.smalltalk.StValueHolder;

/**
 * JunHuffmanValueHolder class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/10/20 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun707 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunHuffmanValueHolder.java,v 9.8 2010/03/09 09:17:10 nisinaka Exp $
 */
public class JunHuffmanValueHolder<V extends Serializable> extends StValueHolder<V> {
	protected StValueHolder<byte[]> bytesValueHolder;

	/**
	 * Create a new instance of <code>JunHuffmanValueHolder</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunHuffmanValueHolder() {
		super();
	}

	/**
	 * Create a new instance of <code>JunHuffmanValueHolder</code> and initialize it.
	 * 
	 * @param anObject java.lang.Object
	 * @category Instance creation
	 */
	public JunHuffmanValueHolder(V anObject) {
		super(anObject);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StValueModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		bytesValueHolder = new StValueHolder<byte[]>();
	}

	/**
	 * Answer the currently stored value.
	 * 
	 * @return java.lang.Object
	 * @see jp.co.sra.smalltalk.StValueModel#value()
	 * @category accessing
	 */
	@Override
	@SuppressWarnings("unchecked")
	public V value() {
		V aValue = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(bytesValueHolder.value());
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Expand_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		ObjectInputStream aStream = null;
		try {
			aStream = new ObjectInputStream(new ByteArrayInputStream(outputBytes));
			aValue = (V) aStream.readObject();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return aValue;
	}

	/**
	 * Set the currently stored value, without notifying dependents.
	 * 
	 * @param aValue java.lang.Object
	 * @see jp.co.sra.smalltalk.StValueModel#setValue_(java.lang.Object)
	 * @category accessing
	 */
	@Override
	public void setValue_(V aValue) {
		byte[] inputBytes = null;
		ObjectOutputStream aStream = null;
		try {
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			aStream = new ObjectOutputStream(byteArrayStream);
			aStream.writeObject(aValue);
			aStream.flush();
			aStream.close();
			aStream = null;
			inputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (aStream != null) {
					aStream.flush();
					aStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}

		InputStream inputStream = null;
		OutputStream outputStream = null;
		byte[] outputBytes = null;
		try {
			inputStream = new ByteArrayInputStream(inputBytes);
			ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
			outputStream = byteArrayStream;

			JunHuffmanCoder.Compress_into_(inputStream, outputStream);
			outputBytes = byteArrayStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.flush();
					outputStream.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}
		bytesValueHolder.setValue_(outputBytes);
	}
}
