package jp.co.sra.jun.goodies.itemlist;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import jp.co.sra.jun.system.framework.JunAbstractController;

/**
 * JunOrderListController class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/12/09 (by Mitsuhiro Asada)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2008/04/22 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun473 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOrderListController.java,v 9.9 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunOrderListController extends JunAbstractController {

	/**
	 * Answer the model as JunOrderListModel.
	 * 
	 * @return jp.co.sra.jun.goodies.itemlist.JunOrderListModel
	 * @category model accessing
	 */
	public JunOrderListModel getModel() {
		return (JunOrderListModel) this.model();
	}

	/**
	 * Answer the view as a JunOrderListView.
	 * 
	 * @return jp.co.sra.jun.goodies.itemlist.JunOrderListView
	 * @category view accessing
	 */
	public JunOrderListView getView() {
		return (JunOrderListView) this.view();
	}

	/**
	 * Invoked when a mouse button has been pressed on the view.
	 * 
	 * @param event java.awt.event.MouseEvent
	 * @see jp.co.sra.smalltalk.StController#mousePressed(MouseEvent)
	 * @category events
	 */
	public void mousePressed(MouseEvent event) {
		Point aPoint = event.getPoint();
		int anIndex = this.getView().whichOfItemRowsAt_(aPoint);
		if (event.isShiftDown()) {
			this.selectWithShiftDownAt_(anIndex);
		} else {
			if (event.isControlDown()) {
				this.selectWithCtrlDownAt_(anIndex);
			} else {
				this.selectAt_(anIndex);
			}
		}
	}

	/**
	 * Select the item object specified an index position.
	 * 
	 * @param anIndex int
	 * @category selecting
	 */
	protected void selectAt_(int anIndex) {
		if (anIndex > -1) {
			if (this.getModel().isSelected_(anIndex)) {
				this.getModel().selectedIndexes_(new ArrayList<Integer>());
			} else {
				ArrayList<Integer> indexes = new ArrayList<Integer>();
				indexes.add(new Integer(anIndex));
				this.getModel().selectedIndexes_(indexes);
			}
		} else {
			this.getModel().selectedIndexes_(new ArrayList<Integer>());
		}
	}

	/**
	 * Select the item object specified an index position with ctrl down.
	 * 
	 * @param anIndex int
	 * @category selecting
	 */
	protected void selectWithCtrlDownAt_(int anIndex) {
		if (anIndex > -1) {
			ArrayList<Integer> selectedIndexes = new ArrayList<Integer>(this.getModel().selectedIndexes());
			if (this.getModel().isSelected_(anIndex)) {
				selectedIndexes.remove(new Integer(anIndex));
			} else {
				selectedIndexes.add(new Integer(anIndex));
			}
			this.getModel().selectedIndexes_(selectedIndexes);
		}
	}

	/**
	 * Select the item object specified an index position with shift down.
	 * 
	 * @param anIndex int
	 * @category selecting
	 */
	protected void selectWithShiftDownAt_(int anIndex) {
		if (anIndex > -1) {
			ArrayList<Integer> selectedIndexes = new ArrayList<Integer>(this.getModel().selectedIndexes());
			if (selectedIndexes.isEmpty() == false) {
				ArrayList<Integer> newSelectedIndexes = new ArrayList<Integer>();
				int firstIndex = selectedIndexes.get(0);
				if (firstIndex < anIndex) {
					for (int index = firstIndex; index <= anIndex; index++) {
						newSelectedIndexes.add(new Integer(index));
					}
				} else {
					for (int index = firstIndex; index >= anIndex; index--) {
						newSelectedIndexes.add(new Integer(index));
					}
				}
				selectedIndexes = newSelectedIndexes;
			}
			this.getModel().selectedIndexes_(selectedIndexes);
		}
	}

}
