package jp.co.sra.jun.goodies.itemlist;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunOrderListDialog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/12/12 (by Mitsuhiro Asada)
 *  @updated   2008/05/15 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun465 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOrderListDialog.java,v 9.13 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunOrderListDialog extends JunAbstractDialog<JunOrderListModel> {

	/**
	 * Create a new instance of JunOrderListDialog.
	 * 
	 * @category Instance creation
	 */
	public JunOrderListDialog() {
		super();
	}

	/**
	 * Open a JunOrderListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @category Utilities
	 */
	public static Object[] Request_labels_(String messageString, String[] stringCollection) {
		return Request_labels_onAccept_onCancel_(messageString, stringCollection, (StBlockClosure1<Boolean, JunOrderListModel>) null, (StBlockClosure1<Boolean, JunOrderListModel>) null);
	}

	/**
	 * Open a JunOrderListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1&lt;java.lang.Boolean, jp.co.sra.jun.goodies.itemlist.JunOrderListModel&gt;
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1&lt;java.lang.Boolean, jp.co.sra.jun.goodies.itemlist.JunOrderListModel&gt;
	 * @category Utilities
	 */
	public static Object[] Request_labels_onAccept_onCancel_(String messageString, String[] stringCollection, StBlockClosure1<Boolean, JunOrderListModel> acceptBlock, StBlockClosure1<Boolean, JunOrderListModel> cancelBlock) {
		JunOrderListDialog itemListDialog = new JunOrderListDialog();
		itemListDialog.itemListModel_(new JunOrderListModel(stringCollection));

		Dialog dialog = itemListDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			itemListDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			itemListDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		if (itemListDialog.accept().value()) {
			return itemListDialog.itemListModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a JunOrderListDialog showing a message string and let a user to select item value.
	 * 
	 * @return java.lang.Object[]
	 * @param messageString java.lang.String
	 * @param stringCollection java.lang.String[]
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Object[] Request_labels_onAccept_onCancel_(String messageString, String[] stringCollection, StBlockClosure<Boolean, JunOrderListModel, ?, ?> acceptBlock, StBlockClosure<Boolean, JunOrderListModel, ?, ?> cancelBlock) {
		JunOrderListDialog itemListDialog = new JunOrderListDialog();
		itemListDialog.itemListModel_(new JunOrderListModel(stringCollection));

		Dialog dialog = itemListDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			itemListDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			itemListDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		if (itemListDialog.accept().value()) {
			return itemListDialog.itemListModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Get the new order list model.
	 * 
	 * @return jp.co.sra.jun.goodies.itemlist.JunOrderListModel
	 * @category accessing
	 */
	public JunOrderListModel itemListModel() {
		return this.model();
	}

	/**
	 * Set the new order list model.
	 * 
	 * @param anItemListModel jp.co.sra.jun.goodies.itemlist.JunOrderListModel
	 * @category accessing
	 */
	public void itemListModel_(JunOrderListModel anItemListModel) {
		this.model_(anItemListModel);
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category viewing
	 */
	protected String windowTitle() {
		return $String("Order List Dialog");
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOrderListDialogViewAwt(this);
		} else {
			return new JunOrderListDialogViewSwing(this);
		}
	}
}
