package jp.co.sra.jun.goodies.itemlist;

import java.awt.Dimension;

import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.dialog.JunAbstractDialogViewSwing;

/**
 * JunOrderListDialogViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/12/12 (by Mitsuhiro Asada)
 *  @updated   2008/05/15 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun465 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOrderListDialogViewSwing.java,v 9.11 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunOrderListDialogViewSwing extends JunAbstractDialogViewSwing implements JunOrderListDialogView {
	protected StView orderListView;

	/**
	 * Create a new instance of JunOrderListDialogViewSwing and initialize it.
	 *
	 * @param aOrderListDialog jp.co.sra.jun.goodies.itemlist.JunOrderListDialog
	 * @category Instance creation
	 */
	public JunOrderListDialogViewSwing(JunOrderListDialog aOrderListDialog) {
		super(aOrderListDialog);
	}
	
	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		orderListView = null;
	}

	/**
	 * Answer my model as a JunOrderListDialog.
	 *
	 * @return jp.co.sra.jun.goodies.itemlist.JunOrderListDialog
	 * @see jp.co.sra.jun.goodies.itemlist.JunOrderListDialogView#getOrderListDialog()
	 * @category model accessing
	 */
	public JunOrderListDialog getOrderListDialog() {
		return (JunOrderListDialog) this.model();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.orderListView().toComponent(), WindowSpecLayout.Position(0f, 8, 0f, 8, 1f, -8, 1f, -48));
		this.add(this.acceptButton(), WindowSpecLayout.Position(0.3f, -48, 1f, -40, 0.3f, 48, 1f, -8));
		this.add(this.cancelButton(), WindowSpecLayout.Position(0.7f, -48, 1f, -40, 0.7f, 48, 1f, -8));
		this.setPreferredSize(new Dimension(300, 200));

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my order list view.
	 *
	 * @return jp.co.sra.smalltalk.StView
	 * @category interface opening
	 */
	protected StView orderListView() {
		if (orderListView == null) {
			orderListView = JunOrderListViewSwing.OnScrollPaneWithButtons(this.getOrderListDialog().itemListModel());
		}
		return orderListView;
	}
}
