package jp.co.sra.jun.goodies.lisp;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

/**
 * JunLispConverter class
 * 
 *  @author    nisinaka
 *  @created   2009/05/12 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispConverter.java,v 9.6 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunLispConverter {

	public static JunLispConverter Default = new JunLispConverter();

	/**
	 * Register the class with the name.
	 * 
	 * @param name java.lang.String
	 * @param aClass java.lang.Class
	 * @category Registering
	 */
	public static void Register(String name, Class<?> aClass) {
		Default.register(name, aClass);
	}

	/**
	 * Register the class with the simple class name.
	 * 
	 * @param aClass java.lang.Class
	 * @category Registering
	 */
	public static void Register(Class<?> aClass) {
		Default.register(aClass);
	}

	/**
	 * Convert the specified JunLispList to an object of a registered class.
	 * 
	 * @param aList jp.motr.jun.lisp.list.JunLispList
	 * @return java.lang.Object
	 * @category converting
	 */
	public static Object Convert(JunLispList aList) {
		return Default.convert(aList);
	}

	////////////////////////////////////////////////////////////////////////////////

	protected Map<String, Class<?>> classTable;

	/**
	 * Create a new instance of JunLispConverter and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunLispConverter() {
		this.flushClassTable();
	}

	/**
	 * Answer my class table.
	 * 
	 * @return java.util.Map<String, Class<?>>
	 * @category accessing
	 */
	protected Map<String, Class<?>> getClassTable() {
		if (classTable == null) {
			classTable = new HashMap<String, Class<?>>();
		}
		return classTable;
	}

	/**
	 * Register the class with the name.
	 * 
	 * @param name java.lang.String
	 * @param aClass java.lang.Class
	 * @category registering
	 */
	public void register(String name, Class<?> aClass) {
		try {
			// check if the class has a constructor with JunLispList.
			aClass.getConstructor(new Class<?>[] { JunLispList.class });
		} catch (NoSuchMethodException e) {
			throw new RuntimeException("The class does not have a constructor with a JunLispList argument - " + aClass.getName());
		}

		this.getClassTable().put(name, aClass);
	}

	/**
	 * Register the class with the simple class name.
	 * 
	 * @param name java.lang.String
	 * @param aClass java.lang.Class
	 * @category registering
	 */
	public void register(Class<?> aClass) {
		this.register(aClass.getSimpleName(), aClass);
	}

	/**
	 * Convert the specified JunLispList to an object of a registered class.
	 * 
	 * @param aList jp.motr.jun.lisp.list.JunLispList
	 * @return java.lang.Object
	 * @category converting
	 */
	public Object convert(JunLispList aList) {
		String name = ((StSymbol) aList.head()).toString();
		Class<?> aClass = this.getClassTable().get(name);
		if (aClass == null) {
			return null;
		}

		Constructor<?> aConstructor = null;
		try {
			aConstructor = aClass.getConstructor(new Class<?>[] { JunLispList.class });
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			return null;
		}

		Object anObject = null;
		try {
			anObject = aConstructor.newInstance(aList);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return anObject;
	}

	/**
	 * Flush the class table.
	 * 
	 * @category flushing
	 */
	protected void flushClassTable() {
		classTable = null;
	}

}
