package jp.co.sra.jun.goodies.lisp;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.PopupMenu;
import java.awt.TextArea;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.WindowSpecLayout;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;
import jp.co.sra.smalltalk.menu.StPopupMenuViewAwt;

import jp.co.sra.jun.system.framework.JunAbstractViewPanel;

/**
 * JunLispInterpreterViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/05/07 (by nisinaka)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2005/03/03 (by nisinaka)
 *  @updated   2008/04/30 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispInterpreterViewAwt.java,v 9.12 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunLispInterpreterViewAwt extends JunAbstractViewPanel implements JunLispInterpreterView {

	protected TextArea textValueView;
	protected TextArea textCollectorView;

	/**
	 * Create a new instance of JunLispInterpreterViewAwt and initialize it.
	 *
	 * @param aLispInterpreter jp.co.sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreterViewAwt(JunLispInterpreter aLispInterpreter) {
		super(aLispInterpreter);
	}

	/**
	 * Answer my model as JunLispInterpreter.
	 *
	 * @return jp.co.sra.jun.goodies.lisp.JunLispInterpreter
	 * @see jp.co.sra.jun.goodies.lisp.JunLispInterpreterView#getLispInterpreter()
	 * @category model accessing
	 */
	public JunLispInterpreter getLispInterpreter() {
		return (JunLispInterpreter) this.model();
	}

	/**
	 * Answer the selected text to 'do it'.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.jun.goodies.lisp.JunLispInterpreterView#getSelectedText()
	 * @category accessing
	 */
	public String getSelectedText() {
		return this.textValueView().getSelectedText();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.textValueView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0.3f, -1));
		this.add(this.textCollectorView(), WindowSpecLayout.Position(0f, 1, 0.3f, 0, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(350, 400));
	}

	/**
	 * Answer the view of my text collector.
	 *
	 * @return java.awt.TextArea
	 */
	protected TextArea textCollectorView() {
		if (textCollectorView == null) {
			textCollectorView = new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
			textCollectorView.setBackground(Color.lightGray);

			ByteArrayOutputStream aByteArrayOutputStream = new ByteArrayOutputStream() {
				public synchronized void write(byte b[], int off, int len) {
					super.write(b, off, len);
					textCollectorView.append(new String(b, off, len));
				}
			};
			this.getLispInterpreter().textCollector_(new PrintStream(aByteArrayOutputStream));
		}
		return textCollectorView;
	}

	/**
	 * Answer the view of my text value.
	 *
	 * @return java.awt.TextArea
	 */
	protected TextArea textValueView() {
		if (textValueView == null) {
			textValueView = new TextArea(this.getLispInterpreter().textValue().value(), 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);

			StPopupMenu aPopupMenu = new StPopupMenu();
			aPopupMenu.add(new StMenuItem("do it", new MenuPerformer(this.getLispInterpreter(), "doIt", this)));

			final PopupMenu aPopupMneu = new StPopupMenuViewAwt(aPopupMenu).toPopupMenu();
			textValueView.add(aPopupMneu);
			textValueView.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isMetaDown() || e.isPopupTrigger()) {
						aPopupMneu.show(textValueView, e.getX(), e.getY());
					}
				}
			});

			this.getLispInterpreter().textValue().compute_(new StBlockClosure1<String, String>() {
				public String value(String newValue) {
					textValueView.setText(newValue);
					return null;
				}
			});
		}
		return textValueView;
	}

}
