package jp.co.sra.jun.goodies.lisp;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.WindowSpecLayout;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;
import jp.co.sra.smalltalk.menu.StPopupMenuViewSwing;

import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;

/**
 * JunLispInterpreterViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/05/07 (by nisinaka)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2005/03/03 (by nisinaka)
 *  @updated   2008/04/30 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispInterpreterViewSwing.java,v 9.11 2010/03/09 09:17:14 nisinaka Exp $
 */
public class JunLispInterpreterViewSwing extends JunAbstractViewJPanel implements JunLispInterpreterView {

	protected JTextArea textValueView;
	protected JScrollPane textValueViewScrollPane;
	protected JTextArea textCollectorView;
	protected JScrollPane textCollectorViewScrollPane;

	/**
	 * Create a new instance of JunLispInterpreterViewSwing and initialize it.
	 *
	 * @param aLispInterpreter jp.co.sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreterViewSwing(JunLispInterpreter aLispInterpreter) {
		super(aLispInterpreter);
	}

	/**
	 * Answer my model as JunLispInterpreter.
	 *
	 * @return jp.co.sra.jun.goodies.lisp.JunLispInterpreter
	 * @see jp.co.sra.jun.goodies.lisp.JunLispInterpreterView#getLispInterpreter()
	 * @category model accessing
	 */
	public JunLispInterpreter getLispInterpreter() {
		return (JunLispInterpreter) this.model();
	}

	/**
	 * Answer the selected text to 'do it'.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.jun.goodies.lisp.JunLispInterpreterView#getSelectedText()
	 * @category accessing
	 */
	public String getSelectedText() {
		return this.textValueView().getSelectedText();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.textValueViewScrollPane(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0.3f, -1));
		this.add(this.textCollectorViewScrollPane(), WindowSpecLayout.Position(0f, 1, 0.3f, 0, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(350, 400));
	}

	/**
	 * Answer the view of my text collector.
	 *
	 * @return javax.swing.JTextArea
	 */
	protected JTextArea textCollectorView() {
		if (textCollectorView == null) {
			textCollectorView = new JTextArea("", 0, 0);
			textCollectorView.setBackground(Color.lightGray);

			ByteArrayOutputStream aByteArrayOutputStream = new ByteArrayOutputStream() {
				public synchronized void write(byte b[], int off, int len) {
					super.write(b, off, len);
					textCollectorView.append(new String(b, off, len));
				}
			};
			this.getLispInterpreter().textCollector_(new PrintStream(aByteArrayOutputStream));
		}
		return textCollectorView;
	}

	/**
	 * Answer the scroll pane for the text collector view.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane textCollectorViewScrollPane() {
		if (textCollectorViewScrollPane == null) {
			textCollectorViewScrollPane = new JScrollPane(this.textCollectorView());
			textCollectorViewScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			textCollectorViewScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return textCollectorViewScrollPane;
	}

	/**
	 * Answer the view of my text value.
	 *
	 * @return javax.swing.JTextArea
	 */
	protected JTextArea textValueView() {
		if (textValueView == null) {
			textValueView = new JTextArea(this.getLispInterpreter().textValue().value(), 0, 0);

			StPopupMenu aPopupMenu = new StPopupMenu();
			aPopupMenu.add(new StMenuItem("do it", new MenuPerformer(this.getLispInterpreter(), "doIt", this)));

			final JPopupMenu aPopupMneu = new StPopupMenuViewSwing(aPopupMenu).toPopupMenu();
			textValueView.add(aPopupMneu);
			textValueView.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isMetaDown() || e.isPopupTrigger()) {
						aPopupMneu.show(textValueView, e.getX(), e.getY());
					}
				}
			});

			this.getLispInterpreter().textValue().compute_(new StBlockClosure1<String, String>() {
				public String value(String newValue) {
					textValueView.setText(newValue);
					return null;
				}
			});
		}
		return textValueView;
	}

	/**
	 * Answer the scroll pane for the text value view.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane textValueViewScrollPane() {
		if (textValueViewScrollPane == null) {
			textValueViewScrollPane = new JScrollPane(this.textValueView());
			textValueViewScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			textValueViewScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return textValueViewScrollPane;
	}

}
