package jp.co.sra.jun.goodies.meter;

import java.awt.Color;
import java.awt.Rectangle;
import java.util.Collection;
import java.util.Random;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StColorValue;

import jp.co.sra.jun.goodies.gauge.JunLevelGaugeModel;

/**
 * JunStereoLevelMeterTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/04/04 (by m-asada)
 *  @updated   2008/04/22 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun652 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunStereoLevelMeterTestExamples.java,v 9.11 2010/03/09 09:17:15 nisinaka Exp $
 */
public class JunStereoLevelMeterTestExamples extends JunLevelGaugeMeterTestExamples {
	/**
	 * Example: Open a JunStereoLevelMeter.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		final JunStereoLevelMeter aMeter = new JunStereoLevelMeter(10, 0.5, 1, 0);
		aMeter.backgroundColor_(StColorValue.VeryLightGray);
		aMeter.frameColor_(StColorValue.VeryLightGray);
		aMeter.showAverage_(false);
		aMeter.open();
		if (aMeter.isEmpty()) {
			return false;
		}
		ExampleDo_(new StBlockClosure1<Object, Random>() {
			public Object value(final Random aRandom) {
				Collection<Double> valueCollection = aMeter.leftMeter().collect_(new StBlockClosure1<Double, JunLevelGaugeModel>() {
					public Double value(JunLevelGaugeModel gauge) {
						return new Double(aRandom.nextDouble() * aMeter.range() + aMeter.minimum());
					}
				});
				Number[] values = valueCollection.toArray(new Double[valueCollection.size()]);
				aMeter.leftValues_(values);

				for (int i = 0; i < values.length; i++) {
					values[i] = new Double(values[i].doubleValue() - ((aRandom.nextDouble() - 0.5) * (aMeter.range() * 0.1)));
				}
				aMeter.rightValues_(values);
				return null;
			}
		});
		return true;
	}

	/**
	 * Example: Open a JunStereoLevelMeter with horizontal.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		final JunStereoLevelMeter aMeter = new JunStereoLevelMeter(10, 0.5, 1, 0);
		aMeter.verticalGauge_(false);
		aMeter.leftMeter().reverseDirection_(!aMeter.leftMeter().reverseDirection());
		aMeter.backgroundColor_(StColorValue.VeryLightGray);
		aMeter.frameColor_(StColorValue.VeryLightGray);
		aMeter.showAverage_(false);
		aMeter.open();
		if (aMeter.isEmpty()) {
			return false;
		}
		ExampleDo_(new StBlockClosure1<Object, Random>() {
			public Object value(final Random aRandom) {
				Collection<Double> valueCollection = aMeter.leftMeter().collect_(new StBlockClosure1<Double, JunLevelGaugeModel>() {
					public Double value(JunLevelGaugeModel gauge) {
						return new Double(aRandom.nextDouble() * aMeter.range() + aMeter.minimum());
					}
				});
				Number[] values = valueCollection.toArray(new Double[valueCollection.size()]);
				aMeter.leftValues_(values);

				for (int i = 0; i < values.length; i++) {
					values[i] = new Double(values[i].doubleValue() - ((aRandom.nextDouble() - 0.5) * (aMeter.range() * 0.1)));
				}
				aMeter.rightValues_(values);
				return null;
			}
		});
		return true;
	}

	/**
	 * Example: Open a JunStereoLevelMeter with reverse direction.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		final JunStereoLevelMeter aMeter = new JunStereoLevelMeter(10, 0.5, 1, 0);
		aMeter.reverseDirection_(true);
		aMeter.backgroundColor_(StColorValue.VeryLightGray);
		aMeter.frameColor_(StColorValue.VeryLightGray);
		aMeter.showAverage_(false);
		aMeter.open();
		if (aMeter.isEmpty()) {
			return false;
		}
		ExampleDo_(new StBlockClosure1<Object, Random>() {
			public Object value(final Random aRandom) {
				Collection<Double> valueCollection = aMeter.leftMeter().collect_(new StBlockClosure1<Double, JunLevelGaugeModel>() {
					public Double value(JunLevelGaugeModel gauge) {
						return new Double(aRandom.nextDouble() * aMeter.range() + aMeter.minimum());
					}
				});
				Number[] values = valueCollection.toArray(new Double[valueCollection.size()]);
				aMeter.leftValues_(values);

				for (int i = 0; i < values.length; i++) {
					values[i] = new Double(values[i].doubleValue() - ((aRandom.nextDouble() - 0.5) * (aMeter.range() * 0.1)));
				}
				aMeter.rightValues_(values);
				return null;
			}
		});
		return true;
	}

	/**
	 * Example: Open a JunStereoLevelMeter with horizontals.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		final JunStereoLevelMeter aMeter = new JunStereoLevelMeter(10, 0.5, 1, 0);
		aMeter.verticalGauge_(false);
		aMeter.reverseDirection_(true);
		aMeter.leftMeter().reverseDirection_(!aMeter.leftMeter().reverseDirection());
		aMeter.backgroundColor_(StColorValue.VeryLightGray);
		aMeter.frameColor_(StColorValue.VeryLightGray);
		aMeter.showAverage_(false);
		aMeter.open();
		if (aMeter.isEmpty()) {
			return false;
		}
		ExampleDo_(new StBlockClosure1<Object, Random>() {
			public Object value(final Random aRandom) {
				Collection<Double> valueCollection = aMeter.leftMeter().collect_(new StBlockClosure1<Double, JunLevelGaugeModel>() {
					public Double value(JunLevelGaugeModel gauge) {
						return new Double(aRandom.nextDouble() * aMeter.range() + aMeter.minimum());
					}
				});
				Number[] values = valueCollection.toArray(new Double[valueCollection.size()]);
				aMeter.leftValues_(values);

				for (int i = 0; i < values.length; i++) {
					values[i] = new Double(values[i].doubleValue() - ((aRandom.nextDouble() - 0.5) * (aMeter.range() * 0.1)));
				}
				aMeter.rightValues_(values);
				return null;
			}
		});
		return true;
	}

	/**
	 * Example: Open a JunStereoLevelMeter with rainbow color.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example5() {
		final JunStereoLevelMeter aMeter = new JunStereoLevelMeter(50, 0.5, 1, 0);
		aMeter.verticalGauge_(false);
		aMeter.leftMeter().reverseDirection_(!aMeter.leftMeter().reverseDirection());
		aMeter.backgroundColor_(StColorValue.VeryLightGray);
		aMeter.frameColor_(StColorValue.VeryLightGray);
		aMeter.valueColor_(StColorValue.VeryLightGray);
		aMeter.hillColor_(Color.white);
		aMeter.averageColor_(Color.white);
		aMeter.daleColor_(Color.white);
		aMeter.edgeColor_(StColorValue.LightGray);
		aMeter.rainbowColor_(true);
		aMeter.gaugeWidth_(5);
		aMeter.cellSize_(3);
		aMeter.cellSpace_(1);
		aMeter.showHill_(true);
		aMeter.showAverage_(false);
		aMeter.showDale_(false);
		aMeter.showEdge_(false);
		aMeter.openIn_(new Rectangle(200, 200, 400, 400));
		if (aMeter.isEmpty()) {
			return false;
		}
		ExampleDo_(new StBlockClosure1<Object, Random>() {
			public Object value(final Random aRandom) {
				Collection<Double> valueCollection = aMeter.leftMeter().collect_(new StBlockClosure1<Double, JunLevelGaugeModel>() {
					public Double value(JunLevelGaugeModel gauge) {
						return new Double(aRandom.nextDouble() * aMeter.range() + aMeter.minimum());
					}
				});
				Number[] values = valueCollection.toArray(new Double[valueCollection.size()]);
				aMeter.leftValues_(values);

				for (int i = 0; i < values.length; i++) {
					values[i] = new Double(values[i].doubleValue() - ((aRandom.nextDouble() - 0.5) * (aMeter.range() * 0.1)));
				}
				aMeter.rightValues_(values);
				return null;
			}
		});
		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunStereoLevelMeterTestExamples();
	}
}
