package jp.co.sra.jun.goodies.milliseconds;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunMillisecondDialog class
 * 
 *  @author    nisinaka
 *  @created   2003/05/28 (by nisinaka)
 *  @updated   2008/05/23 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun473 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMillisecondDialog.java,v 9.9 2010/03/09 09:17:16 nisinaka Exp $
 */
public class JunMillisecondDialog extends JunAbstractDialog<JunMillisecondModel> {
	/**
	 * Create a new instance of <code>JunMillisecondDialog</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMillisecondDialog() {
		super();
	}

	/**
	 * Open a JunMillisecondDialog and let a user to enter millisecond value.
	 *
	 * @return long
	 * @category Utilities
	 */
	public static long Request() {
		return Request(null);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @category Utilities
	 */
	public static long Request(String messageString) {
		return Request(messageString, 0);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue) {
		return Request(messageString, millisecondValue, null, null);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param validateBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category Utilities
	 * @deprecated since Jun701, use Generics.
	 * @deprecated
	 */
	public static long Request(String messageString, long millisecondValue, StBlockClosure<?, ?, ?, ?> validateBlock) {
		return Request(messageString, millisecondValue, null, null, validateBlock);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param validateBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue, StBlockClosure2<Long, Long, Long> validateBlock) {
		return Request(messageString, millisecondValue, null, null, validateBlock);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue, StBlockClosure<Boolean, JunMillisecondModel, ?, ?> acceptBlock, StBlockClosure<Boolean, JunMillisecondModel, ?, ?> cancelBlock) {
		return Request(messageString, millisecondValue, acceptBlock, cancelBlock, null);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue, StBlockClosure1<Boolean, JunMillisecondModel> acceptBlock, StBlockClosure1<Boolean, JunMillisecondModel> cancelBlock) {
		return Request(messageString, millisecondValue, acceptBlock, cancelBlock, null);
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param validateBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue, final StBlockClosure<Boolean, JunMillisecondModel, ?, ?> acceptBlock, final StBlockClosure<Boolean, JunMillisecondModel, ?, ?> cancelBlock,
			final StBlockClosure<?, ?, ?, ?> validateBlock) {
		JunMillisecondDialog millisecondDialog = new JunMillisecondDialog();
		millisecondDialog.millisecondModel_(new JunMillisecondModel(millisecondValue));
		if (validateBlock != null) {
			millisecondDialog.millisecondModel().validateBlock_(validateBlock);
		}

		Dialog dialog = millisecondDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			millisecondDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			millisecondDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		if (millisecondDialog.accept().value()) {
			return millisecondDialog.millisecondModel().milliseconds();
		} else {
			return -1;
		}
	}

	/**
	 * Open a JunMillisecondDialog showing a message string and let a user to enter millisecond value.
	 *
	 * @return long
	 * @param messageString java.lang.String
	 * @param millisecondValue long
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param validateBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @category Utilities
	 */
	public static long Request(String messageString, long millisecondValue, StBlockClosure1<Boolean, JunMillisecondModel> acceptBlock, StBlockClosure1<Boolean, JunMillisecondModel> cancelBlock, StBlockClosure2<Long, Long, Long> validateBlock) {
		JunMillisecondDialog millisecondDialog = new JunMillisecondDialog();
		millisecondDialog.millisecondModel_(new JunMillisecondModel(millisecondValue));
		if (validateBlock != null) {
			millisecondDialog.millisecondModel().validateBlock_(validateBlock);
		}

		Dialog dialog = millisecondDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			millisecondDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			millisecondDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);

		if (millisecondDialog.accept().value()) {
			return millisecondDialog.millisecondModel().milliseconds();
		} else {
			return -1;
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunMillisecondDialogViewAwt(this);
		} else {
			return new JunMillisecondDialogViewSwing(this);
		}
	}

	/**
	 * Answer my current millisecond model.
	 *
	 * @return jp.co.sra.jun.goodies.milliseconds.JunMillisecondModel
	 * @category accessing
	 */
	public JunMillisecondModel millisecondModel() {
		if (this.model() == null) {
			this.model_(new JunMillisecondModel());
		}
		return this.model();
	}

	/**
	 * Set my new millisecond model.
	 *
	 * @param newMillisecondModel jp.co.sra.jun.goodies.milliseconds.JunMillisecondModel
	 * @category accessing
	 */
	public void millisecondModel_(JunMillisecondModel newMillisecondModel) {
		this.model_(newMillisecondModel);
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Time Dialog", "Milliseconds Dialog");
	}
}
