package jp.co.sra.jun.goodies.misc;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StInterval;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.calendar.JunCalendarDateModel;
import jp.co.sra.jun.goodies.calendar.JunCalendarModel;
import jp.co.sra.jun.goodies.tips.JunURL;
import jp.co.sra.jun.goodies.utilities.JunStringUtility;
import jp.co.sra.jun.opengl.display.JunOpenGL3dModel;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayLight;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.texture.JunOpenGLTexture;
import jp.co.sra.jun.opengl.texture.JunOpenGLTextureTestExamples;

/**
 * JunFeelTheMoon class
 * 
 *  @author    nisinaka
 *  @created   2010/03/09 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun758 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFeelTheMoon.java,v 9.3 2010/03/10 01:35:52 nisinaka Exp $
 */
public class JunFeelTheMoon extends JunOpenGL3dModel {

	protected static StInterval RangeOfAgeOfTheMoon = new StInterval(0, 29.5, 0.1);

	protected double ageOfTheMoon;
	protected JunCalendarDateModel calendarDate;

	/**
	 * Calculate the age of the moon from the specified date.
	 * 
	 * @param aDate java.util.Date
	 * @return double
	 * @category Utilities
	 */
	protected static double AgeOfTheMoonFromDate_(Date aDate) {
		Calendar aCalendar = Calendar.getInstance();
		aCalendar.setTime(aDate);
		return AgeOfTheMoonFromCalendar_(aCalendar);
	}

	/**
	 * Calculate the age of the moon from the specified date.
	 * 
	 * @param aCalendar java.util.Calendar
	 * @return double
	 * @category Utilities
	 */
	protected static double AgeOfTheMoonFromCalendar_(Calendar aCalendar) {
		int[] contants = new int[] { 0, 2, 0, 2, 2, 4, 5, 6, 7, 8, 9, 10 };

		int y = aCalendar.get(Calendar.YEAR);
		int m = aCalendar.get(Calendar.MONTH);
		int d = aCalendar.get(Calendar.DAY_OF_MONTH);
		double a = ((y - 11) % 19 * 11 + contants[m] + d) % 30;
		return a;
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();

		ageOfTheMoon = 0;
		calendarDate = new JunCalendarDateModel();
		calendarDate.validateBlock_(new StBlockClosure2<Calendar, Calendar, Calendar>() {
			public Calendar value(Calendar oldCalendar, Calendar newCalendar) {
				JunFeelTheMoon.this.ageOfTheMoon_(AgeOfTheMoonFromCalendar_(newCalendar));
				return newCalendar;
			}
		});

		this.windowLabel_($String("Feel the Moon"));
		this.backgroundColor_(Color.black);
	}

	/**
	 * Answer my current age of the moon.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double ageOfTheMoon() {
		return ageOfTheMoon;
	}

	/**
	 * Set my new age of the moon.
	 * 
	 * @param value double
	 * @category accessing 
	 */
	public void ageOfTheMoon_(double value) {
		ageOfTheMoon = Math.max(RangeOfAgeOfTheMoon.first(), Math.min(value, RangeOfAgeOfTheMoon.last()));
		JunAngle angle = JunAngle.FromDeg_(ageOfTheMoon / Math.ceil(RangeOfAgeOfTheMoon.last()) * -360);
		Jun3dPoint vector = new Jun3dPoint(0, 0, 1).transform_(Jun3dTransformation.RotateY_(angle));
		for (int i = 0; i < 4; i++) {
			this.displayLights()[i].lightDirection().setValue_(vector);
		}
		this.changed_($("light"));
	}

	/**
	 * Answer my calendar date.
	 * 
	 * @return jp.co.sra.jun.goodies.calendar.JunCalendarDateModel
	 * @category accessing
	 */
	public JunCalendarDateModel calendarDate() {
		return calendarDate;
	}

	/**
	 * Answer the current date.
	 * 
	 * @return java.util.Calendar
	 * @category accessing
	 */
	public Calendar date() {
		return this.calendarDate().date();
	}

	/**
	 * Set the new date.
	 * 
	 * @param aCalendar java.util.Calendar
	 * @category accessing
	 */
	public void date_(Calendar aCalendar) {
		this.calendarDate().date_(aCalendar);
	}

	/**
	 * Set the new date.
	 * 
	 * @param aDate java.util.Date
	 * @category accessing
	 */
	public void date_(Date aDate) {
		Calendar aCalendar = Calendar.getInstance();
		aCalendar.setTime(aDate);
		this.date_(aCalendar);
	}

	/**
	 * Answer the current date string.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String dateString() {
		return JunCalendarModel.StringFromDate_(this.date().getTime());
	}

	/**
	 * Set the new date string.
	 * 
	 * @param dateString java.lang.String
	 * @category accessing
	 */
	public void dateString_(String dateString) {
		this.date_(JunCalendarModel.DateFromString_(dateString));
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#openGL3dObject()
	 * @category accessing
	 */
	@SuppressWarnings("unchecked")
	@Override
	public JunOpenGL3dObject openGL3dObject() {
		if (openGL3dObject == null) {
			Object[] xyPointsAndSphere = JunOpenGL3dObject.XyPointsAndSphere_radius_longitude_latitude_(10, 1, 360, 180);
			Vector<Jun2dPoint> collection = (Vector<Jun2dPoint>) xyPointsAndSphere[0];
			Jun2dPoint[] points = new Jun2dPoint[collection.size()];
			collection.copyInto(points);
			Jun2dPoint[] xyPoints = new Jun2dPoint[points.length];
			for (int i = 0; i < points.length; i++) {
				xyPoints[i] = new Jun2dPoint(points[i].x(), 1.0 - points[i].y());
			}

			JunOpenGL3dCompoundObject theMoon = (JunOpenGL3dCompoundObject) xyPointsAndSphere[1];
			theMoon.paint_(Color.white);
			JunOpenGLTexture aTexture = new JunOpenGLTexture(JunOpenGLTextureTestExamples.ImageMoon3());
			aTexture.coordinates_(xyPoints);
			theMoon.texture_(aTexture);
			theMoon.name_("moon");

			openGL3dObject = theMoon;
		}
		return openGL3dObject;
	}

	/**
	 * Answer the array of all display lights.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLDisplayLight[]
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#displayLights()
	 * @category lighting
	 */
	@Override
	public JunOpenGLDisplayLight[] displayLights() {
		if (displayLights == null) {
			JunOpenGLDisplayLight[] lightCollection = new JunOpenGLDisplayLight[5];
			for (int i = 0; i <= 3; i++) {
				lightCollection[i] = JunOpenGLDisplayLight.ParallelLight_color_direction_(true, Color.white, new Jun3dPoint(0, 0, 1));
			}
			lightCollection[4] = JunOpenGLDisplayLight.AmbientLight_color_(true, Color.darkGray);

			this.displayLights_(lightCollection);
		}
		return displayLights;
	}

	/**
	 * Answer my current default eye point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultEyePoint()
	 * @category defaults
	 */
	@Override
	public Jun3dPoint defaultEyePoint() {
		Jun3dPoint point = (Jun3dPoint) this.defaultProjectionTable().get($("eyePoint"));
		if (point == null) {
			point = new Jun3dPoint(-10, 0, 0);
			this.defaultProjectionTable().put($("eyePoint"), point);
		}
		return point;
	}

	/**
	 * Answer my current default sight point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultSightPoint()
	 * @category defaults
	 */
	@Override
	public Jun3dPoint defaultSightPoint() {
		Jun3dPoint point = (Jun3dPoint) this.defaultProjectionTable().get($("sightPoint"));
		if (point == null) {
			point = new Jun3dPoint(0, 0, 0);
			this.defaultProjectionTable().put($("signtPoint"), point);
		}
		return point;
	}

	/**
	 * Answer my current default up vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultUpVector()
	 * @category defaults
	 */
	@Override
	public Jun3dPoint defaultUpVector() {
		Jun3dPoint point = (Jun3dPoint) this.defaultProjectionTable().get($("upVector"));
		if (point == null) {
			point = new Jun3dPoint(0, 0, 1);
			this.defaultProjectionTable().put($("upVector"), point);
		}
		return point;
	}

	/**
	 * Create a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultView()
	 * @category interface opening
	 */
	@Override
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunFeelTheMoonViewAwt(this);
		} else {
			return new JunFeelTheMoonViewSwing(this);
		}
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param evt java.awt.event.WindowEvent
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#noticeOfWindowClose(java.awt.event.WindowEvent)
	 * @category interface closing
	 */
	@Override
	public void noticeOfWindowClose(WindowEvent evt) {
		super.noticeOfWindowClose(evt);
		this.calendarDate().closeRequest();
	}

	/**
	 * Answer my menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		if (_menuBar == null) {
			_menuBar = new StMenuBar();
			_menuBar.add(this._createFileMenu());
			_menuBar.add(this._createLookMenu());
			_menuBar.add(this._createMiscMenu());
		}
		return _menuBar;
	}

	/**
	 * Create a "File" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createFileMenu() {
		StMenu fileMenu = new StMenu($String("File"), $("fileMenu"));
		fileMenu.add(new StMenuItem($String("New"), new MenuPerformer(this, "newModel")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Save as HTML..."), $("saveAsHtmlMenu"), new MenuPerformer(this, "saveAsHTML")));
		fileMenu.add(new StMenuItem($String("Save as image..."), $("saveAsImageMenu"), new MenuPerformer(this, "saveAsImage")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Quit"), new MenuPerformer(this, "quitDoing")));
		return fileMenu;
	}

	/**
	 * Create a "Look" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createLookMenu() {
		StMenu lookMenu = new StMenu($String("Look"), $("lookMenu"));
		lookMenu.add(new StMenuItem($String("Calendar"), new MenuPerformer(this, "openCalendar")));
		lookMenu.addSeparator();
		lookMenu.add(new StMenuItem($String("Reset"), new MenuPerformer(this, "resetView")));
		return lookMenu;
	}

	/**
	 * Create a "Misc" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createMiscMenu() {
		StMenu miscMenu = new StMenu($String("Misc"), $("miscMenu"));
		miscMenu.add(new StMenuItem($String("Spawn"), new MenuPerformer(this, "spawnMoon")));
		return miscMenu;
	}

	/**
	 * Save as HTML.
	 * 
	 * @category menu messages
	 */
	public void saveAsHTML() {
		Dimension extent = new Dimension(96, 96);

		String[] aCollection = JunStringUtility.Separate_dividers_(this.dateString(), new char[] { '/' });
		File aDirectory = new File(aCollection[0] + aCollection[1]);
		if (aDirectory.exists() == false) {
			aDirectory.mkdirs();
		}

		double aNumber = this.ageOfTheMoon();
		Calendar aDate = this.date();
		try {
			// images
			int daysInMonth = aDate.getMaximum(Calendar.DAY_OF_MONTH);
			for (int day = 1; day <= daysInMonth; day++) {
				String aString = String.format("%02d", day);
				this.dateString_(aCollection[0] + "/" + aCollection[1] + "/" + aString);
				StImage anImage = this.asImageExtent_(extent);
				File aFile = new File(aDirectory, aString + ".jpg");
				this.writeImage_to_(anImage, aFile);
			}

			// index.html
			File aFile = new File(aDirectory, "index.html");
			PrintWriter writer = null;
			try {
				writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(aFile), "UTF-8")));

				writer.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
				writer.println("<html lang=\"ja\">");
				writer.println("<head>");
				writer.println("<link rev=\"MADE\" href=\"http://www.sra.co.jp/public/sra/technical/jun/\">");
				writer.println("<link rel=\"INDEX\" href=\"index.html\">");
				writer.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
				writer.println("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">");
				writer.println("<title>" + aCollection[0] + '/' + aCollection[1] + "</title>");
				writer.println("</head>");
				writer.println("<body>");
				writer.println("<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" summary=\"the age of the moon\">");
				writer.println("  <tbody>");
				writer.println("    <tr>");
				writer.println("      <td align=\"center\" colspan=\"7\"><strong>" + aCollection[0] + "</strong>" + $String("Nen", " ") + "<strong>" + aCollection[1] + "</strong>" + $String("Gatsu", "") + "</td>");
				writer.println("    </tr>");
				writer.println("    <tr>");
				for (String each : this.calendarDate().weekdayNames()) {
					writer.println("      <td align=\"center\">" + each + "</td>");
				}
				writer.println("    </tr>");

				int[][] matrix = this.calendarDate().calendarMatrix();
				for (int y = 0; y < matrix.length; y++) {
					writer.println("    <tr>");
					for (int x = 0; x < matrix[y].length; x++) {
						writer.print("      <td align=\"center\">");
						if (matrix[y][x] > 0) {
							writer.print("<strong>" + matrix[y][x] + "</strong>");
							writer.println("<br>");
							writer.print("      ");
							String aString = String.format("%02d", matrix[y][x]);
							writer.print("<img src=\"" + aString + ".jpg\" width=\"" + extent.width + "\" height=\"" + extent.height + "\" alt=\"" + aString + ".jpg\">");
						}
						writer.print("</td>");
						writer.println();
					}
					writer.println("    </tr>");
				}
				writer.println("    </tr>");
				writer.println("  </tbody>");
				writer.println("</table>");
				writer.println("</body>");

			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if (writer != null) {
					writer.close();
				}
			}

			JunURL.Browse_(aFile.toURI().toURL());

		} catch (MalformedURLException e) {
			e.printStackTrace();
		} finally {
			this.date_(aDate);
			this.ageOfTheMoon_(aNumber);
		}
	}

	/**
	 * Save as an image.
	 * 
	 * @category menu messages
	 */
	@Override
	public void saveAsImage() {
		StImage anImage = this.asImage();
		if (anImage == null) {
			return;
		}

		File aFile = this.requestNewImageFilename();
		if (aFile == null) {
			return;
		}

		this.writeImage_to_(anImage, aFile);

		try {
			JunURL.Browse_(aFile.toURI().toURL());
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Open a calendar dialog.
	 * 
	 * @category menu messages
	 */
	public void openCalendar() {
		JunCalendarDateModel calendarDate = this.calendarDate();
		calendarDate.date_(this.date());

		if (calendarDate.builder().window() != null) {
			calendarDate._windowExpandAndRaise();
		} else {
			Window window = this.getWindow();
			if (window == null) {
				calendarDate.open();
			} else {
				StRectangle box = new StRectangle(window.getBounds());
				StRectangle area = new StRectangle(0, 0, box.width(), box.height());
				area = area.align_with_(area.topLeft(), new Point(box.right() + 8, box.top()));
				calendarDate.openAt_(area.origin());
			}
		}
	}

	/**
	 * Reset the view.
	 * 
	 * @category menu messages
	 */
	@Override
	public void resetView() {
		super.resetView();
		this.date_(Calendar.getInstance());
	}

	/**
	 * Spawn the moon.
	 * 
	 * @category menu messages
	 */
	public void spawnMoon() {
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel(this.openGL3dObject());
		displayModel.defaultProjectionTable_(this.projectionTable());
		displayModel.backgroundColor_(Color.black);

		Window aWindow = this.getWindow();
		if (aWindow == null) {
			displayModel.open();
		} else {
			Rectangle bounds = aWindow.getBounds();
			int x = bounds.x + bounds.width + 8;
			int y = bounds.y;
			displayModel.openAt_(new Point(x, y));
		}
	}
}
