package jp.co.sra.jun.goodies.morph;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.SwingUtilities;

import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;

/**
 * JunMorphApplicationViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/11/11 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun714 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMorphApplicationViewSwing.java,v 9.8 2010/03/09 09:17:16 nisinaka Exp $
 */
public class JunMorphApplicationViewSwing extends JunAbstractViewJPanel implements JunMorphApplicationView {
	protected JunMorphViewSwing firstImageView;
	protected JunMorphViewSwing secondImageView;

	/**
	 * Create a new instance of <code>JunMorphApplicationViewSwing</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMorphApplicationViewSwing() {
		super();
	}

	/**
	 * Create a new instance of <code>JunMorphApplicationViewSwing</code> and initialize it.
	 * 
	 * @param newModel jp.co.sra.jun.goodies.morph.JunMorphModel
	 * @category Instance creation
	 */
	public JunMorphApplicationViewSwing(JunMorphModel newModel) {
		super(newModel);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		firstImageView = null;
		secondImageView = null;
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.firstImageView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 0, 0f, 0, 0.5f, -1, 1f, 0));
		this.add(this.secondImageView()._onBorderedPanel(), WindowSpecLayout.Position(0.5f, 1, 0f, 0, 1f, 0, 1f, 0));
		this.setPreferredSize(new Dimension(350, 250));
	}

	/**
	 * Answer the receiver's model as <code>JunMorphModel</code>.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphModel
	 * @see jp.co.sra.jun.goodies.morph.JunMorphApplicationView#getModel()
	 * @category model accessing
	 */
	public JunMorphModel getModel() {
		return (JunMorphModel) this.model();
	}

	/**
	 * Answer the receiver's first image view.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphViewSwing
	 * @category user interface
	 */
	protected JunMorphViewSwing firstImageView() {
		if (firstImageView == null) {
			firstImageView = new JunMorphViewSwing(this.getModel());
			firstImageView.aspectMessage_($("firstImage"));
		}
		return firstImageView;
	}

	/**
	 * Answer the receiver's second image view.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphViewSwing
	 * @category user interface
	 */
	protected JunMorphViewSwing secondImageView() {
		if (secondImageView == null) {
			secondImageView = new JunMorphViewSwing(this.getModel());
			secondImageView.aspectMessage_($("secondImage"));
		}
		return secondImageView;
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow
	 * @see jp.co.sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard 
	 */
	@Override
	public void _setupKeyboard(final Window aWindow) {
		this.addKeyListener(this.getModel()._keyListener());
		this.firstImageView().addKeyListener(this.getModel()._keyListener());
		this.secondImageView().addKeyListener(this.getModel()._keyListener());

		this.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				Component focusOwner = aWindow.getFocusOwner();
				if (focusOwner != null && focusOwner != JunMorphApplicationViewSwing.this) {
					JunMorphApplicationViewSwing.this.requestFocus();
				}
			}
		});

		aWindow.addWindowListener(new WindowAdapter() {
			public void windowActivated(WindowEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Component focusOwner = aWindow.getFocusOwner();
						if (focusOwner == null || focusOwner != JunMorphApplicationViewSwing.this) {
							JunMorphApplicationViewSwing.this.requestFocus();
						}
					}
				});
			}
		});
	}
}
