package jp.co.sra.jun.goodies.morph;

import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;

import jp.co.sra.smalltalk.StAssociation;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun2dBoundingBox;
import jp.co.sra.jun.geometry.curves.Jun2dLine;
import jp.co.sra.jun.goodies.cursors.JunCursors;
import jp.co.sra.jun.system.framework.JunAbstractController;

/**
 * JunMorphController class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/11/11 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun714 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMorphController.java,v 9.7 2010/03/09 09:17:16 nisinaka Exp $
 */
public class JunMorphController extends JunAbstractController {
	protected transient Point startPoint;
	protected transient Point endPoint;
	protected transient Point previousPoint;
	protected transient StAssociation<StSymbol, Jun2dLine> currentAssociation;
	protected transient int dragType;

	protected static final int DRAG_NONE = 0;
	protected static final int DRAG_ADD_LINE = 1;
	protected static final int DRAG_EDIT_LINE = 2;

	/**
	 * Create a new instance of <code>JunMorphController</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMorphController() {
		super();
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release
	 */
	@Override
	public void initialize() {
		super.initialize();

		startPoint = null;
		endPoint = null;
		previousPoint = null;
		currentAssociation = null;
		dragType = DRAG_NONE;
	}

	/**
	 * Activity for add line.
	 * 
	 * @category control defaults
	 */
	public void addLineActivity(MouseEvent evt) {
		Graphics graphicsContext = this.getView().toComponent().getGraphics();
		Point currentPoint = evt.getPoint();
		if (endPoint.equals(currentPoint) == false) {
			endPoint = currentPoint;
			this.getView().displayOn_(graphicsContext);
			this.getView().displayOn_startPoint_endPoint_((Graphics2D) graphicsContext, startPoint, endPoint);
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
			}
		}
	}

	/**
	 * Activity for edit line.
	 * 
	 * @category control defaults
	 */
	public void editLineActivity(MouseEvent evt) {
		if (currentAssociation != null) {
			Point currentPoint = evt.getPoint();
			StSymbol aSymbol = currentAssociation.key();
			Jun2dLine aLine = currentAssociation.value();
			Graphics graphicsContext = this.getView().toComponent().getGraphics();
			if (previousPoint.equals(currentPoint) == false) {
				Jun2dPoint aPoint = new Jun2dPoint(currentPoint);
				if (aSymbol == $("first")) {
					if (aLine.first().equal_(aLine.last())) {
						aLine.from_to_(aPoint, aPoint);
					} else {
						aLine.from_to_(aPoint, aLine.last());
					}
				}
				if (aSymbol == $("last")) {
					if (aLine.first().equal_(aLine.last())) {
						aLine.from_to_(aPoint, aPoint);
					} else {
						aLine.from_to_(aLine.first(), aPoint);
					}
				}
				if (aSymbol == $("line")) {
					aPoint = new Jun2dPoint(currentPoint.x - previousPoint.x, currentPoint.y - previousPoint.y);
					aLine.from_to_(aLine.first().plus_(aPoint), aLine.last().plus_(aPoint));
				}
				this.getView().displayOn_(graphicsContext);
				previousPoint = currentPoint;
			}
		}
	}

	/**
	 * Activity for the red button pressed.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @category control defaults
	 */
	@Override
	protected void redButtonPressedActivity(MouseEvent evt) {
		Point currentPoint = evt.getPoint();
		StAssociation<StSymbol, Jun2dLine> anAssociation = this.whichLine_(currentPoint);
		if (anAssociation != null) {
			previousPoint = currentPoint;
			currentAssociation = anAssociation;
			dragType = DRAG_EDIT_LINE;
			this.getView().toComponent().setCursor(JunCursors.NormalPlusCursor());
			this.editLineActivity(evt);
		}
	}

	/**
	 * Activity for the red button released.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @category control defaults
	 */
	@Override
	protected void redButtonReleasedActivity(MouseEvent evt) {
		try {
		} finally {
			this.getView().toComponent().setCursor(JunCursors.NormalCursor());
			previousPoint = null;
			currentAssociation = null;
			dragType = DRAG_NONE;
		}
	}

	/**
	 * Activity for the yellow button pressed.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @category control defaults
	 */
	protected void yellowButtonPressedActivity(MouseEvent evt) {
		StAssociation<StSymbol, Jun2dLine> anAssociation = this.whichLine_(evt.getPoint());
		if (anAssociation == null) {
			startPoint = evt.getPoint();
			endPoint = startPoint;
			dragType = DRAG_ADD_LINE;
			this.getView().toComponent().setCursor(JunCursors.NormalPlusCursor());
			this.addLineActivity(evt);
		} else {
			Jun2dLine aLine = anAssociation.value();
			this.getModel().focusLine_(aLine);
			this.view()._showPopupMenu(evt.getX(), evt.getY());
		}
	}

	/**
	 * Activity for the yellow button released.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @category control defaults
	 */
	protected void yellowButtonReleasedActivity(MouseEvent evt) {
		try {
			if (evt.isShiftDown()) {
				this.getView().displayOn_(this.getView().toComponent().getGraphics());
				return;
			}
			if (startPoint == null || endPoint == null) {
				return;
			}
			Jun2dLine aLine = new Jun2dPoint(startPoint).to_(new Jun2dPoint(endPoint));
			this.getModel().getFromLines().add(aLine);
			this.getModel().getToLines().add((Jun2dLine) aLine.copy());
			this.getModel().changed();
		} finally {
			startPoint = null;
			endPoint = null;
			this.getView().toComponent().setCursor(JunCursors.NormalCursor());
			dragType = DRAG_NONE;
		}
	}

	/**
	 * Invoked when the mouse enters the view.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mouseEntered(MouseEvent evt) {
		super.mouseEntered(evt);
		this.getView().toComponent().setCursor(Cursor.getDefaultCursor());
	}

	/**
	 * Invoked when the mouse exits the view.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mouseExited(MouseEvent evt) {
		super.mouseMoved(evt);
		this.getView().toComponent().setCursor(Cursor.getDefaultCursor());
	}

	/**
	 * The mouse button has been pressed.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @see jp.co.sra.smalltalk.StController#mousePressed(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mousePressed(MouseEvent evt) {
		if (SwingUtilities.isRightMouseButton(evt)) {
			this.yellowButtonPressedActivity(evt);
		} else if (SwingUtilities.isLeftMouseButton(evt)) {
			this.redButtonPressedActivity(evt);
		}
	}

	/**
	 * The mouse button has been released.
	 * 
	 * @param evt java.awt.event.MouseEvent
	 * @see jp.co.sra.smalltalk.StController#mouseReleased(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mouseReleased(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			this.yellowButtonReleasedActivity(evt);
		} else if (SwingUtilities.isLeftMouseButton(evt)) {
			this.redButtonReleasedActivity(evt);
		}
	}

	/**
	 * Invoked when a mouse is dragged on the view.
	 *
	 * @param evt java.awt.event.MouseEvent 
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 * @category mouse motion events
	 */
	@Override
	public void mouseDragged(MouseEvent evt) {
		switch (dragType) {
			case DRAG_ADD_LINE:
				this.addLineActivity(evt);
				break;
			case DRAG_EDIT_LINE:
				this.editLineActivity(evt);
				break;
			case DRAG_NONE:
				break;
		}
	}

	/**
	 * Invoked when a mouse is moved on the view.
	 *
	 * @param evt java.awt.event.MouseEvent 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 * @category mouse motion events
	 */
	@Override
	public void mouseMoved(MouseEvent evt) {
		if (this.whichLine_(evt.getPoint()) == null) {
			this.getView().toComponent().setCursor(JunCursors.NormalCursor());
		} else {
			this.getView().toComponent().setCursor(JunCursors.NormalMinusCursor());
		}
		super.mouseMoved(evt);
	}

	/**
	 * Answer the line which the specified cursor point.
	 * 
	 * @param cursorPoint java.awt.Point
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine
	 * @category selecting
	 */
	public StAssociation<StSymbol, Jun2dLine> whichLine_(Point cursorPoint) {
		Jun2dPoint aPoint = new Jun2dPoint(cursorPoint);
		Jun2dLine[] aspectLines = this.getView().aspectLines();
		for (int i = aspectLines.length - 1; i >= 0; i--) {
			Jun2dLine aLine = aspectLines[i];
			Jun2dBoundingBox aBox = aLine.first().extent_(new Jun2dPoint(1.0, 1.0)).expandedBy_(1.0);
			if (aBox.containsPoint_(aPoint)) {
				return new StAssociation<StSymbol, Jun2dLine>($("first"), aLine);
			}
			aBox = aLine.last().extent_(new Jun2dPoint(1.0, 1.0)).expandedBy_(1.0);
			if (aBox.containsPoint_(aPoint)) {
				return new StAssociation<StSymbol, Jun2dLine>($("last"), aLine);
			}
			if (aLine.first().equal_(aLine.last()) == false) {
				if (aPoint.distance_(aLine.lineSegmentNearestPointFromPoint_(aPoint)) <= 1.0d) {
					return new StAssociation<StSymbol, Jun2dLine>($("line"), aLine);
				}
			}
		}
		return null;
	}

	/**
	 * Answer the receiver's model as <code>JunMorphModel</code>.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphModel
	 * @category model accessing
	 */
	public JunMorphModel getModel() {
		return (JunMorphModel) this.model();
	}

	/**
	 * Answer the receiver's view as <code>JunMorphView</code>.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphView
	 * @category view accessing
	 */
	public JunMorphView getView() {
		return (JunMorphView) this.view();
	}
}
