package jp.co.sra.jun.goodies.morph;

import java.awt.Frame;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StAssociation;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StInterval;
import jp.co.sra.smalltalk.StReadStream;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;

import jp.co.sra.jun.geometry.curves.Jun2dLine;
import jp.co.sra.jun.goodies.animation.JunCartoonMovie;
import jp.co.sra.jun.goodies.cursors.JunCursors;
import jp.co.sra.jun.goodies.files.JunFileModel;
import jp.co.sra.jun.goodies.image.framework.JunImageDisplayModel;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.goodies.lisp.JunLispParser;
import jp.co.sra.jun.goodies.utilities.JunImageUtility;
import jp.co.sra.jun.goodies.utilities.JunStringUtility;
import jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog;
import jp.co.sra.jun.system.framework.JunApplicationModel;
import jp.co.sra.jun.system.framework.JunDialog;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunMorphModel class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/11/11 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun714 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMorphModel.java,v 9.10 2010/03/09 09:17:16 nisinaka Exp $
 */
public class JunMorphModel extends JunApplicationModel {
	protected BufferedImage firstImage;
	protected BufferedImage secondImage;
	protected ArrayList<Jun2dLine> fromLines;
	protected ArrayList<Jun2dLine> toLines;
	protected Jun2dLine focusLine;
	protected int howMany;
	protected int howLong;

	protected StMenuBar _menuBar;
	protected StPopupMenu _popupMenu;

	/**
	 * Create a new instance of <code>JunMorphModel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMorphModel() {
		super();
	}

	/**
	 * Create a new instance of <code>JunMorphModel</code> and initialize it.
	 * 
	 * @param firstImage java.awt.image.BufferedImage
	 * @param secondImage java.awt.image.BufferedImage
	 * @category Instance creation
	 */
	public JunMorphModel(BufferedImage firstImage, BufferedImage secondImage) {
		this();
		this.firstImage_(firstImage);
		this.secondImage_(secondImage);
	}

	/**
	 * Create a new instance of <code>JunMorphModel</code> and initialize it.
	 * 
	 * @param firstImage java.awt.image.BufferedImage
	 * @param secondImage java.awt.image.BufferedImage
	 * @param fromLines java.util.Collection
	 * @param toLines java.util.Collection
	 * @category Instance creation
	 */
	public JunMorphModel(BufferedImage firstImage, BufferedImage secondImage, Collection<Jun2dLine> fromLines, Collection<Jun2dLine> toLines) {
		this();
		this.firstImage_(firstImage);
		this.secondImage_(secondImage);
		this.fromLines_(fromLines);
		this.toLines_(toLines);
	}

	/**
	 * Create a new instance of <code>JunMorphModel</code> and initialize it.
	 * 
	 * @param firstImage java.awt.image.BufferedImage
	 * @param secondImage java.awt.image.BufferedImage
	 * @param fromLines jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @param toLines jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category Instance creation
	 */
	public JunMorphModel(BufferedImage firstImage, BufferedImage secondImage, Jun2dLine[] fromLines, Jun2dLine[] toLines) {
		this();
		this.firstImage_(firstImage);
		this.secondImage_(secondImage);
		this.fromLines_(fromLines);
		this.toLines_(toLines);
	}

	/**
	 * Create a new instance of <code>JunMorphModel</code> and initialize it.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunMorphModel(JunLispList aList) {
		this();
		this.fromLispList(aList);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();

		firstImage = null;
		secondImage = null;
		fromLines = null;
		toLines = null;
		focusLine = null;
		howMany = 7;
		howLong = 10;

		_menuBar = null;
		_popupMenu = null;
	}

	/**
	 * Answer the receiver's first image.
	 * 
	 * @return firstImage
	 * @category accessing
	 */
	public BufferedImage firstImage() {
		return firstImage;
	}

	/**
	 * Set the receiver's first image.
	 * 
	 * @param anImage java.awt.image.BufferedImage
	 * @category accessing
	 */
	public void firstImage_(BufferedImage anImage) {
		firstImage = anImage;
	}

	/**
	 * Answer the receiver's second image.
	 * 
	 * @return secondImage
	 * @category accessing
	 */
	public BufferedImage secondImage() {
		return secondImage;
	}

	/**
	 * Set the receiver's second image.
	 * 
	 * @param anImage java.awt.image.BufferedImage
	 * @category accessing
	 */
	public void secondImage_(BufferedImage anImage) {
		secondImage = anImage;
	}

	/**
	 * Answer the receiver's from lines.
	 * 
	 * @return java.util.ArrayList
	 * @category accessing
	 */
	public ArrayList<Jun2dLine> getFromLines() {
		if (fromLines == null) {
			fromLines = new ArrayList<Jun2dLine>();
		}
		return fromLines;
	}

	/**
	 * Answer the receiver's from lines.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category accessing
	 */
	public Jun2dLine[] fromLines() {
		return this.getFromLines().toArray(new Jun2dLine[this.getFromLines().size()]);
	}

	/**
	 * Set the receiver's from lines.
	 * 
	 * @param aCollection java.util.Collection
	 * @category accessing
	 */
	public void fromLines_(Collection<Jun2dLine> aCollection) {
		fromLines = new ArrayList<Jun2dLine>(aCollection);
	}

	/**
	 * Set the receiver's from lines.
	 * 
	 * @param fromLines jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category accessing
	 */
	public void fromLines_(Jun2dLine[] aCollection) {
		Collection<Jun2dLine> lines = Arrays.asList(aCollection);
		this.fromLines_(lines);
	}

	/**
	 * Answer the receiver's to lines.
	 * 
	 * @return java.util.ArrayList
	 * @category accessing
	 */
	public ArrayList<Jun2dLine> getToLines() {
		if (toLines == null) {
			toLines = new ArrayList<Jun2dLine>();
		}
		return toLines;
	}

	/**
	 * Answer the receiver's to lines.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category accessing
	 */
	public Jun2dLine[] toLines() {
		return this.getToLines().toArray(new Jun2dLine[this.getToLines().size()]);
	}

	/**
	 * Set the receiver's to lines.
	 * 
	 * @param aCollection java.util.Collection
	 * @category accessing
	 */
	public void toLines_(Collection<Jun2dLine> aCollection) {
		toLines = new ArrayList<Jun2dLine>(aCollection);
	}

	/**
	 * Set the receiver's to lines.
	 * 
	 * @param aCollection jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category accessing
	 */
	public void toLines_(Jun2dLine[] aCollection) {
		Collection<Jun2dLine> lines = Arrays.asList(aCollection);
		this.toLines_(lines);
	}

	/**
	 * Answer the receiver's how many value.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int howMany() {
		return howMany;
	}

	/**
	 * Set the receiver's how many value.
	 * 
	 * @param anInteger int
	 * @category accessing
	 */
	public void howMany_(int anInteger) {
		howMany = Math.max(3, Math.min(anInteger, this.howLong * 30));
	}

	/**
	 * Answer the receiver's how long value.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int howLong() {
		return howLong;
	}

	/**
	 * Set the receiver's howLong.
	 * 
	 * @param howLong int
	 * @category accessing
	 */
	public void howLong_(int anInteger) {
		howLong = Math.max(1, Math.min(anInteger, 60));
	}

	/**
	 * Answer the index value of the specified line.
	 * 
	 * @param aLine jp.co.sra.jun.geometry.curves.Jun2dLine
	 * @return int
	 * @category accessing
	 */
	public int indexOfLine_(Jun2dLine aLine) {
		int anIndex = this.getFromLines().indexOf(aLine);
		if (anIndex > -1) {
			return anIndex;
		}
		return this.getToLines().indexOf(aLine);
	}

	/**
	 * The receiver is changed.  Broadcast it to my dependents.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#changed_(jp.co.sra.smalltalk.DependentEvent)
	 * @category changing
	 */
	@Override
	public void changed_(DependentEvent evt) {
		super.changed_(evt);
		this.updateMenuIndication();
	}

	/**
	 * Answer the receiver's default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunMorphApplicationViewAwt(this);
		} else {
			return new JunMorphApplicationViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	@Override
	protected String windowTitle() {
		return $String("Morph");
	}

	/**
	 * Answer my KeyListener which handles my keyboard events.
	 *
	 * @return java.awt.event.KeyListener
	 * @category keyboard
	 */
	public KeyListener _keyListener() {
		return new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (_keyboardEvent(evt)) {
					evt.consume();
				}
			}
		};
	}

	/**
	 * Process the key event.
	 *
	 * @param evt java.awt.event.KeyEvent
	 * @return boolean
	 * @category keyboard
	 */
	protected boolean _keyboardEvent(KeyEvent evt) {
		if (evt.isConsumed()) {
			return false;
		}

		int code = evt.getKeyCode();
		switch (code) {
			case KeyEvent.VK_DELETE:
			case KeyEvent.VK_BACK_SPACE:
				this.removeLineByKeyboard();
				return true;
		}
		return false;
	}

	/**
	 * Get the receiver's attributes from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public void fromLispList(JunLispList aList) {
		this.firstImageFromLispList(aList);
		this.secondImageFromLispList(aList);
		this.fromLinesFromLispList(aList);
		this.toLinesFromLispList(aList);
	}

	/**
	 * Convert the receiver's attributes to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	public JunLispCons toLispList() {
		JunLispCons list = super.toLispList();
		list.add_(this.firstImageToLispList());
		list.add_(this.secondImageToLispList());
		list.add_(this.fromLinesToLispList());
		list.add_(this.toLinesToLispList());
		return list;
	}

	/**
	 * Get the receiver's first image from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void firstImageFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("firstImage")));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}
		this.firstImage_(((StImage) list.tail()).image());
	}

	/**
	 * Convert the receiver's first image to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons firstImageToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("firstImage"));
		if (this.firstImage() != null) {
			list.tail_(new StImage(this.firstImage()));
		}
		return list;
	}

	/**
	 * Get the receiver's second image from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void secondImageFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("secondImage")));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}
		this.secondImage_(((StImage) list.tail()).image());
	}

	/**
	 * Convert the receiver's second image to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons secondImageToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("secondImage"));
		if (this.firstImage() != null) {
			list.tail_(new StImage(this.secondImage()));
		}
		return list;
	}

	/**
	 * Get the receiver's from lines from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void fromLinesFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("fromLines")));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}

		Object[] items = ((JunLispList) list.tail()).asArray();
		ArrayList<Jun2dLine> aCollection = new ArrayList<Jun2dLine>(items.length);
		for (int i = 0; i < items.length; i++) {
			aCollection.add((Jun2dLine) items[i]);
		}
		this.fromLines_(aCollection);
	}

	/**
	 * Convert the receiver's from lines to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons fromLinesToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("fromLines"));
		Jun2dLine[] lines = this.fromLines();
		for (int i = 0; i < lines.length; i++) {
			list.add_(lines[i]);
		}
		return list;
	}

	/**
	 * Get the receiver's to lines from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void toLinesFromLispList(JunLispList aList) {
		JunLispList list = (JunLispList) aList.detect_ifNone_(new StBlockClosure1<Boolean, Object>() {
			public Boolean value(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("toLines")));
			}
		}, new StBlockClosure0<Object>());
		if (list == null) {
			return;
		}

		Object[] items = ((JunLispList) list.tail()).asArray();
		ArrayList<Jun2dLine> aCollection = new ArrayList<Jun2dLine>(items.length);
		for (int i = 0; i < items.length; i++) {
			aCollection.add((Jun2dLine) items[i]);
		}
		this.toLines_(aCollection);
	}

	/**
	 * Convert the receiver's to lines to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons toLinesToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("toLines"));
		Jun2dLine[] lines = this.toLines();
		for (int i = 0; i < lines.length; i++) {
			list.add_(lines[i]);
		}
		return list;
	}

	/**
	 * Update the menu indication.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#updateMenuIndication()
	 * @category menu accessing
	 */
	@Override
	public void updateMenuIndication() {
		super.updateMenuIndication();
		this.updateFileMenuIndication();
		this.updateMorphMenuIndication();
	}

	/**
	 * Update the file menu indication.
	 * 
	 * @category menu accessing
	 */
	public void updateFileMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("fileMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem = aMenu.atNameKey_($("saveMorph"));
		if (menuItem != null) {
			if (this.firstImage() != null && this.secondImage() != null) {
				menuItem.beEnabled(true);
			} else {
				menuItem.beEnabled(false);
			}
		}
	}

	/**
	 * Update the morph menu indication.
	 * 
	 * @category menu accessing
	 */
	public void updateMorphMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("morphMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem = aMenu.atNameKey_($("makeMorph"));
		if (menuItem != null) {
			if (this.firstImage() != null && this.secondImage() != null) {
				menuItem.beEnabled(true);
			} else {
				menuItem.beEnabled(false);
			}
		}
	}

	/**
	 * duration morph.
	 * 
	 * @category menu messages
	 */
	public void durationMorph() {
		String aString = JunDialog.Request_($String("Duration") + ":", Integer.toString(this.howLong()));
		if (aString == null || aString.length() == 0) {
			return;
		}
		try {
			this.howLong_(Integer.valueOf(aString));
		} catch (NumberFormatException e) {
		}
	}

	/**
	 * Set the receiver's left image.
	 * 
	 * @category menu messages
	 */
	public void leftImage() {
		BufferedImage anImage = JunImageUtility.FromFile();
		if (anImage != null) {
			this.firstImage_(anImage);
			this.changed();
		}
	}

	/**
	 * Make the receiver's morph.
	 * 
	 * @return jp.co.sra.jun.goodies.animation.JunCartoonMovie
	 * @category menu messages
	 */
	public JunCartoonMovie makeMorph() {
		Window aWindow = this.getWindow();
		/*
		Point aPoint = null;
		if (aWindow == null) {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			aPoint = new Point(screenSize.width / 2, screenSize.height / 2);
		} else {
			Dimension windowSize = aWindow.getSize();
			aPoint = new Point(windowSize.width / 2, windowSize.height / 2);
		}
		*/

		StInterval anInterval = new StInterval(0, 1, 1.0 / (this.howMany() - 1));
		final ArrayList<BufferedImage> aCollection = new ArrayList<BufferedImage>(anInterval.size());
		JunImageDisplayModel imageModel = null;
		if (aWindow == null) {
			imageModel = JunImageDisplayModel.Show_(this.firstImage());
		} else {
			Point topRight = new Point(aWindow.getX() + aWindow.getWidth(), aWindow.getY());
			imageModel = JunImageDisplayModel.Show_at_(this.firstImage(), new Point(topRight.x + 8, topRight.y + 0));
		}

		final JunImageDisplayModel aViewer = imageModel;
		anInterval.do_(new StBlockClosure1<Void, Double>() {
			public Void value(Double n) {
				JunMorphModel self = JunMorphModel.this;
				BufferedImage anImage = JunImageUtility.MorphImage_andImage_fromLines_toLines_weight_and_(self.firstImage(), self.secondImage(), self.fromLines(), self.toLines(), 1 - n, n);
				aCollection.add(anImage);
				aViewer.image_(anImage);
				aViewer.changed_($("redisplay"));
				String aString = JunStringUtility.F_value_("5.2", n.floatValue());
				if (aViewer.getWindow() == null) {
					System.out.println(aString);
				} else {
					((Frame) aViewer.getWindow()).setTitle(aString);
				}
				return null;
			}
		});

		JunCartoonMovie aPlayer = new JunCartoonMovie();
		double aTick = this.howLong() * 1000 / (double) anInterval.size();
		for (BufferedImage anImage : aCollection) {
			aPlayer.addImage_keepTime_(new StImage(anImage), aTick);
		}
		aPlayer.loop();
		if (this.getWindow() == null) {
			aPlayer.openLightWeightWindowSpecFitFlag_markerFlag_(true, false);
		} else {
			Point topRight = new Point(aWindow.getX() + aWindow.getWidth(), aWindow.getY());
			aPlayer.openLightWeightWindowSpecAt_fitFlag_markerFlag_(new Point(topRight.x + 8, topRight.y + 0), true, false);
		}
		aViewer.closeRequest();
		aPlayer.start();
		return aPlayer;
	}

	/**
	 * Open the morph file.
	 * 
	 * @category menu messages
	 */
	public void openMorph() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType($String("morph files"), JunSystem.DefaultMorphExtensionPatterns()) };
		File aFile = JunFileRequesterDialog.RequestFile($String("Select a morph file."), fileTypes, fileTypes[0]);
		if (aFile == null) {
			return;
		}
		this.openMorphFrom_(aFile);
	}

	/**
	 * Open the specified morph file.
	 * 
	 * @param aFile java.io.File
	 * @category menu messages
	 */
	public void openMorphFrom_(File aFile) {
		JunMorphModel aModel;
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(aFile), System.getProperty("file.encoding")));
			StringWriter sw = new StringWriter();
			String lineSeparator = (String) System.getProperties().get("line.separator");

			String line;
			while ((line = reader.readLine()) != null) {
				sw.write(line);
				sw.write(lineSeparator);
			}
			reader.close();
			reader = null;

			JunLispCons aList = (JunLispCons) JunLispParser.Parse_(new StReadStream(sw.toString()));
			aModel = new JunMorphModel(aList);
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			return;
		} finally {
			if (reader != null) {
				try {
					reader.close();
					reader = null;
				} catch (IOException e) {
					System.out.println(e.getMessage());
					e.printStackTrace();
				}
			}
		}

		firstImage = aModel.firstImage();
		secondImage = aModel.secondImage();
		fromLines = aModel.getFromLines();
		toLines = aModel.getToLines();
		this.changed();
	}

	/**
	 * Quit this window.
	 * 
	 * @category menu messages
	 */
	public void quitDoing() {
		this.closeRequest();
	}

	/**
	 * Remove the receiver's line.
	 * 
	 * @category menu messages
	 */
	public void removeLine() {
		Jun2dLine aLine = this.focusLine();
		if (aLine != null) {
			int anIndex = this.indexOfLine_(aLine);
			if (anIndex < 0) {
				return;
			}
			this.getFromLines().remove(aLine);
			this.getToLines().remove(aLine);
			this.changed();
		}
	}

	/**
	 * Set the receiver's right image.
	 * 
	 * @category menu messages
	 */
	public void rightImage() {
		BufferedImage anImage = JunImageUtility.FromFile();
		if (anImage != null) {
			this.secondImage_(anImage);
			this.changed();
		}
	}

	/**
	 * Save the receiver.
	 * 
	 * @category menu messages
	 */
	public void saveMorph() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType($String("morph files"), JunSystem.DefaultMorphExtensionPatterns()) };
		File aFile = JunFileRequesterDialog.RequestNewFile($String("Input a morph file."), new File(this.defaultBaseName() + "." + JunSystem.DefaultMorphExtension()), fileTypes, fileTypes[0]);
		if (aFile == null) {
			return;
		}
		this.saveMorphTo_(aFile);
	}

	/**
	 * Save the receiver to the specified file.
	 * 
	 * @param aFile java.io.File
	 * @category menu messages
	 */
	public void saveMorphTo_(File aFile) {
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		BufferedWriter writer = null;
		try {
			cursor._show();
			writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(aFile), System.getProperty("file.encoding")));
			this.toLispList().saveOn_(writer);
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.flush();
					writer.close();
					writer = null;
				} catch (IOException e) {
					System.out.println(e.getMessage());
					e.printStackTrace();
				}
			}
			cursor._restore();
		}
	}

	/**
	 * Tween morph.
	 * 
	 * @category menu messages
	 */
	public void tweenMorph() {
		String aString = JunDialog.Request_($String("Tween") + ":", Integer.toString(this.howMany()));
		if (aString == null || aString.length() == 0) {
			return;
		}
		try {
			this.howMany_(Integer.valueOf(aString));
		} catch (NumberFormatException e) {
		}
	}

	/**
	 * Answer the receiver's menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.smalltalk.StApplicationModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		if (_menuBar == null) {
			_menuBar = new StMenuBar();

			_menuBar.add(this._createFileMenu());
			_menuBar.add(this._createMorphMenu());
		}
		return _menuBar;
	}

	/**
	 * Answer my popup menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StPopupMenu
	 * @see jp.co.sra.smalltalk.StApplicationModel#_popupMenu()
	 * @category resources
	 */
	public StPopupMenu _popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = new StPopupMenu();
			_popupMenu.add(new StMenuItem($String("Remove"), $("removeLine"), new MenuPerformer(this, "removeLine")));
		}
		return _popupMenu;
	}

	/**
	 * Create a "File" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createFileMenu() {
		StMenu fileMenu = new StMenu($String("File"), $("fileMenu"));
		fileMenu.add(new StMenuItem($String("New"), $("newModel"), new MenuPerformer(this, "newModel"), KeyEvent.VK_N, KeyEvent.CTRL_MASK));
		fileMenu.add(new StMenuItem($String("Open") + "...", $("openMorph"), new MenuPerformer(this, "openMorph"), KeyEvent.VK_O, KeyEvent.CTRL_MASK));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Left image..."), $("leftImage"), new MenuPerformer(this, "leftImage")));
		fileMenu.add(new StMenuItem($String("Right image...") + "...", $("rightImage"), new MenuPerformer(this, "rightImage")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Save") + "...", $("saveMorph"), new MenuPerformer(this, "saveMorph"), KeyEvent.VK_S, KeyEvent.ALT_MASK));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Quit"), $("quitDoing"), new MenuPerformer(this, "quitDoing")));
		return fileMenu;
	}

	/**
	 * Create a "Morph" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createMorphMenu() {
		StMenu morphMenu = new StMenu($String("Morph"), $("morphMenu"));
		morphMenu.add(new StMenuItem($String("Tween..."), $("tweenMorph"), new MenuPerformer(this, "tweenMorph")));
		morphMenu.add(new StMenuItem($String("Duration..."), $("durationMorph"), new MenuPerformer(this, "durationMorph")));
		morphMenu.addSeparator();
		morphMenu.add(new StMenuItem($String("Make"), $("makeMorph"), new MenuPerformer(this, "makeMorph")));
		return morphMenu;

	}

	/**
	 * Answer the receiver's focus line.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine
	 * @category private
	 */
	protected Jun2dLine focusLine() {
		return focusLine;
	}

	/**
	 * Set the receiver's focus line.
	 * 
	 * @param aLine jp.co.sra.jun.geometry.curves.Jun2dLine
	 * @category private
	 */
	protected void focusLine_(Jun2dLine aLine) {
		focusLine = aLine;
	}

	/**
	 * Remove line by keyboard.
	 * 
	 * @category private
	 */
	protected void removeLineByKeyboard() {
		StView[] views = this.getViews();
		for (int i = 0; i < views.length; i++) {
			JunMorphController aController = (JunMorphController) views[i].controller();
			StAssociation<StSymbol, Jun2dLine> anAssociation = aController.whichLine_(aController.cursorPoint());
			if (anAssociation != null) {
				Jun2dLine aLine = anAssociation.value();
				try {
					this.focusLine_(aLine);
					this.removeLine();
				} finally {
					this.focusLine_(null);
				}
			}
		}
	}
}
