package jp.co.sra.jun.goodies.morph;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;

import jp.co.sra.jun.geometry.curves.Jun2dLine;
import jp.co.sra.jun.system.framework.JunAbstractView;

/**
 * JunMorphView interface
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/11/11 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun714 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMorphView.java,v 9.7 2010/03/09 09:17:17 nisinaka Exp $
 */
public interface JunMorphView extends JunAbstractView {
	/**
	 * Answer the receiver's model as <code>JunMorphModel</code>.
	 * 
	 * @return jp.co.sra.jun.goodies.morph.JunMorphModel
	 * @category model accessing
	 */
	public JunMorphModel getModel();

	/**
	 * Answer the receiver's aspect lines.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category accessing
	 */
	public Jun2dLine[] aspectLines();

	/**
	 * Display the receiver on the graphics.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @see jp.co.sra.smalltalk.StDisplayable#displayOn_(java.awt.Graphics)
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphics);

	/**
	 * Display the start point and end point on the graphics.
	 * 
	 * @param aGraphics java.awt.Graphics2D
	 * @param startPoint java.awt.Point
	 * @param endPoint java.awt.Point
	 * @category displaying
	 */
	public void displayOn_startPoint_endPoint_(Graphics2D aGraphics, Point startPoint, Point endPoint);
}
