package jp.co.sra.jun.goodies.nib;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunNibChoiceDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/23 (By Nobuto Matsubara)
 *  @updated   2008/05/16 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun492 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceDialog.java,v 9.10 2010/03/09 09:17:19 nisinaka Exp $
 */
public class JunNibChoiceDialog extends JunAbstractDialog<JunNibChoice> {
	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request() {
		return Request($String("Nib Choice Dialog"));
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString) {
		return Request(messageString, new JunNibChoice());
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param nibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoice nibChoiceModel) {
		return Request(messageString, nibChoiceModel, null, null);
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param nibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object[]
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoice nibChoiceModel, StBlockClosure<Boolean, JunNibChoice, ?, ?> acceptBlock, StBlockClosure<Boolean, JunNibChoice, ?, ?> cancelBlock) {
		JunNibChoiceDialog nibChoiceDialog = new JunNibChoiceDialog();
		if (nibChoiceModel != null) {
			nibChoiceDialog.nibChoiceModel_(nibChoiceModel);
		}
		Dialog dialog = nibChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			nibChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			nibChoiceDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (nibChoiceDialog.accept()._booleanValue()) {
			return nibChoiceDialog.nibChoiceModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param nibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoice nibChoiceModel, StBlockClosure1<Boolean, JunNibChoice> acceptBlock, StBlockClosure1<Boolean, JunNibChoice> cancelBlock) {
		JunNibChoiceDialog nibChoiceDialog = new JunNibChoiceDialog();
		if (nibChoiceModel != null) {
			nibChoiceDialog.nibChoiceModel_(nibChoiceModel);
		}
		Dialog dialog = nibChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			nibChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			nibChoiceDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (nibChoiceDialog.accept().value()) {
			return nibChoiceDialog.nibChoiceModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param valueArray Object[]
	 * @return Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, Object[] valueArray) {
		return Request(messageString, new JunNibChoice(valueArray), null, null);
	}

	/**
	 * Answer the current nib choice model.
	 * 
	 * @return jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @category accessing
	 */
	public JunNibChoice nibChoiceModel() {
		if (this.model() == null) {
			this.model_(new JunNibChoice());
		}
		return this.model();
	}

	/**
	 * Set the new nib choice model.
	 * 
	 * @param aModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @category accessing
	 */
	public void nibChoiceModel_(JunNibChoice aModel) {
		this.model_(aModel);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StSimpleDialog#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunNibChoiceDialogViewAwt(this);
		} else {
			return new JunNibChoiceDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StSimpleDialog#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Nib Choice Dialog");
	}
}
