package jp.co.sra.jun.goodies.nib;

import java.awt.Dimension;
import java.awt.Window;

import jp.co.sra.smalltalk.StViewPanel;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.dialog.JunAbstractDialogViewAwt;

/**
 * JunNibChoiceDialogViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/23 (By Nobuto Matsubara)
 *  @updated   2008/05/16 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun492 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceDialogViewAwt.java,v 9.10 2010/03/09 09:17:19 nisinaka Exp $
 */
public class JunNibChoiceDialogViewAwt extends JunAbstractDialogViewAwt implements JunNibChoiceDialogView {
	protected StViewPanel nibChoiceView;

	/**
	 * Create a new instance of JunNibChoiceDialogViewAwt and initialize it with the JunNibChoiceDialog.
	 * 
	 * @param newNibChoiceDialog jp.co.sra.jun.goodies.colors.JunNibChoiceDialog
	 * @category Instance creation
	 */
	public JunNibChoiceDialogViewAwt(JunNibChoiceDialog newNibChoiceDialog) {
		super(newNibChoiceDialog);
	}

	/**
	 * Initialize.
	 * 
	 * @see jp.co.sra.jun.system.dialog.JunAbstractDialogViewAwt#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		nibChoiceView = null;
	}

	/**
	 * Answer my model as a JunNibChoiceDialog.
	 * 
	 * @return jp.co.sra.jun.goodies.colors.JunNibChoiceDialog
	 * @see jp.co.sra.jun.goodies.nib.JunNibChoiceDialogView#getNibChoiceDialog()
	 * @category model accessing
	 */
	public JunNibChoiceDialog getNibChoiceDialog() {
		return (JunNibChoiceDialog) this.model();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewPanel#buildComponent()
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.nibChoiceView(), WindowSpecLayout.Position(0f, 0, 0f, 0, 1f, 0, 1f, -50));
		this.add(this.acceptButton(), WindowSpecLayout.Position(0.25f, -44, 1f, -40, 0.25f, 44, 1f, -8));
		this.add(this.cancelButton(), WindowSpecLayout.Position(0.75f, -44, 1f, -40, 0.75f, 44, 1f, -8));
		this.setPreferredSize(new Dimension(238, 100));

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my nib choice view.
	 *
	 * @return jp.co.sra.smalltalk.StViewPanel
	 * @category interface opening
	 */
	protected StViewPanel nibChoiceView() {
		if (nibChoiceView == null) {
			nibChoiceView = (StViewPanel) JunNibChoiceViewAwt.WithSlider(this.getNibChoiceDialog().nibChoiceModel());
		}
		return nibChoiceView;
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see jp.co.sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard 
	 */
	public void _setupKeyboard(Window aWindow) {
		this.nibChoiceView()._setupKeyboard(aWindow);
	}
}
