package jp.co.sra.jun.goodies.nib;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

/**
 * JunNibChoiceWithColorDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/26 (By Nobuto Matsubara)
 *  @updated   2008/05/16 (By m-asada)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceWithColorDialog.java,v 9.10 2010/03/09 09:17:19 nisinaka Exp $
 */
public class JunNibChoiceWithColorDialog extends JunNibChoiceDialog {
	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request() {
		return Request($String("Nib Choice Dialog"));
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object[]
	 * @return Object
	 * @category Utilities
	 */
	public static Object[] Request(String messageString) {
		return Request(messageString, new JunNibChoiceWithColor());
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param aNibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoiceWithColor
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoiceWithColor aNibChoiceModel) {
		return Request(messageString, aNibChoiceModel, null, null);
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param aNibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object[]
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoiceWithColor aNibChoiceModel, StBlockClosure<Boolean, JunNibChoice, ?, ?> acceptBlock, StBlockClosure<Boolean, JunNibChoice, ?, ?> cancelBlock) {
		JunNibChoiceWithColorDialog nibChoiceDialog = new JunNibChoiceWithColorDialog();
		if (aNibChoiceModel != null) {
			nibChoiceDialog.nibChoiceWithColorModel_(aNibChoiceModel);
		}
		Dialog dialog = nibChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			nibChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			nibChoiceDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (nibChoiceDialog.accept()._booleanValue()) {
			return nibChoiceDialog.nibChoiceWithColorModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param aNibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, JunNibChoiceWithColor aNibChoiceModel, StBlockClosure1<Boolean, JunNibChoice> acceptBlock, StBlockClosure1<Boolean, JunNibChoice> cancelBlock) {
		JunNibChoiceWithColorDialog nibChoiceDialog = new JunNibChoiceWithColorDialog();
		if (aNibChoiceModel != null) {
			nibChoiceDialog.nibChoiceWithColorModel_(aNibChoiceModel);
		}
		Dialog dialog = nibChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			nibChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			nibChoiceDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (nibChoiceDialog.accept().value()) {
			return nibChoiceDialog.nibChoiceWithColorModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param valueArray java.lang.Object[]
	 * @return java.lang.Object[]
	 * @category Utilities
	 */
	public static Object[] Request(String messageString, Object[] valueArray) {
		return Request(messageString, new JunNibChoiceWithColor(valueArray), null, null);
	}

	/**
	 * Answer my current nib choice model.
	 * 
	 * @return jp.co.sra.jun.goodies.nib.JunNibChoice
	 * @see jp.co.sra.jun.goodies.nib.JunNibChoiceDialog#nibChoiceModel()
	 * @category accessing
	 */
	public JunNibChoice nibChoiceModel() {
		if (this.model() == null) {
			this.model_(new JunNibChoiceWithColor());
		}
		return this.model();
	}

	/**
	 * Answer the current nib choice with color model.
	 * 
	 * @return jp.co.sra.jun.goodies.nib.JunNibChoiceWithColor
	 * @category accessing
	 */
	public JunNibChoiceWithColor nibChoiceWithColorModel() {
		return (JunNibChoiceWithColor) this.nibChoiceModel();
	}

	/**
	 * Set new nib choice with color model.
	 * 
	 * @param aNibChoiceModel jp.co.sra.jun.goodies.nib.JunNibChoiceWithColor
	 * @category accessing
	 */
	public void nibChoiceWithColorModel_(JunNibChoiceWithColor aNibChoiceModel) {
		super.nibChoiceModel_(aNibChoiceModel);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StSimpleDialog#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunNibChoiceWithColorDialogViewAwt(this);
		} else {
			return new JunNibChoiceWithColorDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Nib Choice Dialog");
	}
}
