package jp.co.sra.jun.goodies.nib;

import java.awt.Color;
import java.io.StringWriter;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunNibChoiceWithColorDialogTestExamples class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/26 (By Nobuto Matsubara)
 *  @updated   2008/05/16 (By m-asada)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceWithColorDialogTestExamples.java,v 9.10 2010/03/09 09:17:19 nisinaka Exp $
 */
public class JunNibChoiceWithColorDialogTestExamples extends JunTestExamples {
	/**
	 * Example1: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		Object answer = JunNibChoiceWithColorDialog.Request();
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Example2: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		Object answer = JunNibChoiceWithColorDialog.Request($String("Select nib."));
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Example3: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		double[] nibCollection = new double[] { 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25 };
		Object[] valueArray = new Object[] { nibCollection, new Double(13), Color.blue, $("circle") };
		Object answer = JunNibChoiceWithColorDialog.Request($String("Select nib."), valueArray);
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Example4: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		double[] nibCollection = new double[] { 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25 };
		JunNibChoiceWithColor nibChoice = new JunNibChoiceWithColor();
		nibChoice.nibCollection_(nibCollection);
		nibChoice.nibWidth_(13);
		nibChoice.nibColor_(Color.red);
		nibChoice.nibShape_($("circle"));
		Object answer = JunNibChoiceWithColorDialog.Request($String("Select nib."), nibChoice);
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Example5: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example5() {
		double[] nibCollection = new double[] { 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25 };
		JunNibChoiceWithColor nibChoice = new JunNibChoiceWithColor();
		nibChoice.nibCollection_(nibCollection);
		nibChoice.nibWidth_(13);
		nibChoice.nibColor_(Color.red);
		nibChoice.nibShape_($("circle"));
		StBlockClosure acceptBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				JunNibChoiceWithColor model = (JunNibChoiceWithColor) anObject;
				System.out.println($String("Accept button was pressed."));
				PrintAnswer(model.value());
				return Boolean.TRUE;
			}
		};
		StBlockClosure cancelBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				JunNibChoiceWithColor model = (JunNibChoiceWithColor) anObject;
				System.out.println($String("Cancel button was pressed."));
				PrintAnswer(model.value());
				return Boolean.TRUE;
			}
		};
		Object answer = JunNibChoiceWithColorDialog.Request($String("Select nib."), nibChoice, acceptBlock, cancelBlock);
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Example5: open nib choice dialog
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example5_g() {
		double[] nibCollection = new double[] { 1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25 };
		JunNibChoiceWithColor nibChoice = new JunNibChoiceWithColor();
		nibChoice.nibCollection_(nibCollection);
		nibChoice.nibWidth_(13);
		nibChoice.nibColor_(Color.red);
		nibChoice.nibShape_($("circle"));
		StBlockClosure1<Boolean, JunNibChoice> acceptBlock = new StBlockClosure1<Boolean, JunNibChoice>() {
			public Boolean value(JunNibChoice model) {
				System.out.println($String("Accept button was pressed."));
				PrintAnswer(model.value());
				return Boolean.TRUE;
			}
		};
		StBlockClosure1<Boolean, JunNibChoice> cancelBlock = new StBlockClosure1<Boolean, JunNibChoice>() {
			public Boolean value(JunNibChoice model) {
				System.out.println($String("Cancel button was pressed."));
				PrintAnswer(model.value());
				return Boolean.TRUE;
			}
		};
		Object answer = JunNibChoiceWithColorDialog.Request($String("Select nib."), nibChoice, acceptBlock, cancelBlock);
		if (answer == null) {
			return true;
		}
		PrintAnswer((Object[]) answer);
		return true;
	}

	/**
	 * Print the answer to System.out. 
	 * 
	 * @param anArray java.lang.Object[]
	 * @category Utilities
	 */
	private static void PrintAnswer(Object[] anArray) {
		double[] nibCollection = (double[]) anArray[0];
		Double nibWidth = (Double) anArray[1];
		Color nibColor = (Color) anArray[2];
		StSymbol nibShape = (StSymbol) anArray[3];
		StringWriter writer = new StringWriter();
		for (int i = 0; i < nibCollection.length; i++) {
			if (i == 0) {
				writer.write("(");
			} else {
				writer.write(" ");
			}
			writer.write(String.valueOf(nibCollection[i]));
		}
		writer.write(") ");
		writer.write(nibWidth.toString());
		writer.write(" ");
		writer.write("(r=");
		writer.write(String.valueOf(nibColor.getRed()));
		writer.write(",g=");
		writer.write(String.valueOf(nibColor.getGreen()));
		writer.write(",b=");
		writer.write(String.valueOf(nibColor.getBlue()));
		writer.write(",a=");
		writer.write(String.valueOf(nibColor.getAlpha()));
		writer.write(") ");
		writer.write(nibShape.toString());
		System.out.println(writer.toString());
	}

	/**
	 * Execute all examples.
	 *
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunNibChoiceWithColorDialogTestExamples();
	}
}
