package jp.co.sra.jun.goodies.pen;

import java.util.List;
import java.util.Vector;

/**
 * JunPenDirection class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/01/05 (by Hirotsugu Kondo)
 *  @updated   2008/05/20 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPenDirection.java,v 9.9 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPenDirection extends JunPenGeometry {
	private JunPenAngle longitude;
	private JunPenAngle latitude;

	/**
	 * Create a new instance of <code>JunPenDirection</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunPenDirection() {
		super();
		this.longitude = JunPenAngle.Zero();
		this.latitude = JunPenAngle.Zero();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anObject DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPenDirection Coerce_(double[] anObject) {
		int length = anObject.length;

		if (length < 1) {
			return null;
		}

		if (length < 2) {
			return Longitude_(anObject[0]);
		}

		return Longitude_latitude_(anObject[0], anObject[length - 1]);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aValue DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPenDirection Coerce_(double aValue) {
		return Longitude_(aValue);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anObject JunPenDirection or Vector of Double or Double
	 * 
	 * @return DOCUMENT ME!
	 */
	@SuppressWarnings("unchecked")
	public static JunPenDirection Coerce_(Object anObject) {
		if (anObject.getClass() == JunPenDirection.class) {
			return (JunPenDirection) anObject;
		}

		if (anObject.getClass() == Vector.class) {
			List<Double> aList = (List<Double>) anObject;
			return Longitude_latitude_(aList.get(0), aList.get(aList.size() - 1));
		}

		return Longitude_(((Double) anObject).doubleValue());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param longitudeDegrees DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPenDirection Longitude_(double longitudeDegrees) {
		return Longitude_latitude_(longitudeDegrees, 0.0d);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param longitudeDegrees DOCUMENT ME!
	 * @param latitudeDegrees DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPenDirection Longitude_latitude_(double longitudeDegrees, double latitudeDegrees) {
		JunPenDirection direction = new JunPenDirection();
		direction.setLongitude_(longitudeDegrees);
		direction.setLatitude_(latitudeDegrees);

		return direction;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static final JunPenDirection Zero() {
		return new JunPenDirection();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunPenAngle latitude() {
		return this.latitude;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunPenAngle longitude() {
		return this.longitude;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param latitudeDegrees DOCUMENT ME!
	 */
	private void setLatitude_(double latitudeDegrees) {
		this.latitude = JunPenAngle.Coerce_(new Double(latitudeDegrees));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param longitudeDegrees DOCUMENT ME!
	 */
	private void setLongitude_(double longitudeDegrees) {
		this.longitude = JunPenAngle.Coerce_(new Double(longitudeDegrees));
	}
}
