package jp.co.sra.jun.goodies.plotter;

import java.awt.Color;
import java.awt.Frame;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.goodies.pen.JunPen;
import jp.co.sra.jun.goodies.pen.JunPenDirection;
import jp.co.sra.jun.goodies.pen.JunPenLocation;
import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunPlotter class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/30 (by Nobuto Matsubara)
 *  @updated   2005/03/03 (by nisinaka)
 *  @updated   2006/03/16 (by m-asada)
 *  @updated   2008/05/20 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun629 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPlotter.java,v 9.11 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPlotter extends JunApplicationModel {
	protected List<List<Object>> drawings;
	protected StPopupMenu _popupMenu;

	/**
	 * Create a new instance of <code>JunPlotter</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunPlotter() {
		super();
	}

	/**
	 * Drawing a block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category accessing
	 */
	public void drawingsDo_(StBlockClosure<?, List<Object>, ?, ?> aBlock) {
		for(List<Object> array : drawings) {
			aBlock.value_(array);
		}
	}

	/**
	 * Drawing a block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category accessing
	 */
	public void drawingsDo_(StBlockClosure1<Void, List<Object>> aBlock) {
		for(List<Object> array : drawings) {
			aBlock.value(array);
		}
	}

	/**
	 * Answer methods of category programs.
	 * 
	 * @return java.lang.String[]
	 * @category accessing
	 */
	public String[] programSelectors() {
		String[] selectors = {
				"curveOfSinAndCos1",
				"curveOfSinAndCos2",
				"penExample",
				"sample1_triangles",
				"sample2_star",
				"sample3_dragon",
				"sample4_mandala",
				"sample5_spirals",
				"sample6_spirals",
				"sample7_polylines",
				"sample8_fill",
				"sample9_spirals",
				"sampleA_combination" };
		return selectors;
	}

	/**
	 * Answer a plotter controller object.
	 * 
	 * @return JunPlotterController
	 * @category controller accessing
	 */
	public JunPlotterController plotterController() {
		return (JunPlotterController) this.getController();
	}

	/**
	 * Flush menus.
	 * 
	 * @category flushing
	 */
	public void flushMenus() {
		_popupMenu = null;
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		drawings = new ArrayList<List<Object>>();
		_popupMenu = null;
	}

	/**
	 * Answer my default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunPlotterViewAwt(this);
		} else {
			return new JunPlotterViewSwing(this);
		}
	}

	/**
	 * Answer the window title
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Pen Plotter");
	}

	/**
	 * Answer my popup menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StPopupMenu
	 * @see jp.co.sra.smalltalk.StApplicationModel#_popupMenu()
	 * @category resources
	 */
	public StPopupMenu _popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = new StPopupMenu();
			// _popupMenu.add(new StMenuItem($String("Browse Programs", "Browse"), new MenuPerformer(this, "browsePrograms")));
			_popupMenu.add(new StMenuItem($String("Clear"), new MenuPerformer(this, "clearPaper")));
			_popupMenu.add(this._createExecuteMenu());
			_popupMenu.addSeparator();
			_popupMenu.add(new StMenuItem($String("Zoom in"), new MenuPerformer(this, "zoomIn")));
			_popupMenu.add(new StMenuItem($String("Zoom out"), new MenuPerformer(this, "zoomOut")));
			_popupMenu.add(new StMenuItem($String("Reset"), new MenuPerformer(this, "resetView")));
		}
		return _popupMenu;
	}

	/**
	 * Create an "Execute" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	public StMenu _createExecuteMenu() {
		String[] selectors = this.programSelectors();
		if (selectors == null) {
			return null;
		}

		StMenu executeMenu = new StMenu($String("Execute") + "...");
		for (int i = 0; i < selectors.length; ++i) {
			executeMenu.add(new StMenuItem(selectors[i], new MenuPerformer(this, selectors[i])));
		}
		return executeMenu;
	}

	/**
	 * Browse programs.
	 * 
	 * @category menu messages
	 */
	public void browsePrograms() {
		Method[] methods = this.getClass().getDeclaredMethods();
		ArrayList<String> programs = new ArrayList<String>();
		for (int i = 0; i < methods.length; ++i) {
			for (int j = 0; j < this.programSelectors().length; ++j) {
				if (methods[i].getName().equals(this.programSelectors()[j])) {
					programs.add(methods[i].getName());
				}
			}
		}
		java.awt.List aList = new java.awt.List(programs.size(), true);
		int width = 0, height = 0;
		// StComposedText composedText = null;
		for (int i = 0; i < programs.size(); ++i) {
			aList.add(programs.get(i));
			/*
			composedText = new StComposedText(programs.get(i));
			if (width < composedText.width()) {
				width = composedText.width();
			}
			height += composedText.height();
			*/
		}
		Frame listWindow = new Frame();
		listWindow.add(aList);
		listWindow.setSize(width, height);
		listWindow.setVisible(true);
	}

	/**
	 * Clear drawing view.
	 * 
	 * @category menu messages
	 */
	public void clearPaper() {
		drawings = new ArrayList<List<Object>>();
		this.changed_($("clear"));
	}

	/**
	 * Reset view.
	 * 
	 * @category menu messages
	 */
	public void resetView() {
		this.plotterController().resetView();
	}

	/**
	 * Zoom in.
	 * 
	 * @category menu messages
	 */
	public void zoomIn() {
		this.plotterController().zoomIn();
	}

	/**
	 * Zoom out.
	 * 
	 * @category menu messages
	 */
	public void zoomOut() {
		this.plotterController().zoomOut();
	}

	/**
	 * Program to draw a curve of sin and cos.
	 * 
	 * @category programs
	 */
	public void curveOfSinAndCos1() {
		this.clearPaper();

		final JunPen pen1 = this.newPen(Color.red);
		final JunPen pen2 = this.newPen(Color.blue);
		final Jun2dPoint scale = new Jun2dPoint(30, 30);

		final StBlockClosure2<Void, JunPen, StSymbol> curve = new StBlockClosure2<Void, JunPen, StSymbol>() {
			public Void value(final JunPen pen, final StSymbol function) {
				pen.up();
				for (int i = -300; i <= 300; i += 6) {
					final double n = i / 100.0;
					JunPlotter.this.do_forMilliseconds_(new StBlockClosure0<Void>() {
						public Void value() {
							double x = n * Math.PI;
							double y = x;
							if (function == $("sin")) {
								y = Math.sin(y);
							} else if (function == $("cos")) {
								y = Math.cos(y);
							}

							Jun2dPoint p = new Jun2dPoint(x, y);
							p = p.scaledBy_(scale);
							pen.goto_(p);

							if (pen.isUp()) {
								pen.down();
							}
							return null;
						}
					}, 100);
				}

				return null;
			}
		};

		Thread aThread1 = new Thread() {
			public void run() {
				curve.value(pen1, $("sin"));
			}
		};
		aThread1.setPriority(Thread.NORM_PRIORITY - 1);
		aThread1.start();

		Thread aThread2 = new Thread() {
			public void run() {
				curve.value(pen2, $("cos"));
			}
		};
		aThread2.setPriority(Thread.NORM_PRIORITY - 1);
		aThread2.start();
	}

	/**
	 * Program to draw a curve of sin and cos with axes.
	 * 
	 * @category programs
	 */
	public void curveOfSinAndCos2() {
		this.curveOfSinAndCos1();

		final JunPen pen0 = this.newPen(Color.gray);
		final Jun2dPoint scale = new Jun2dPoint(30, 30);

		Thread aThread1 = new Thread() {
			public void run() {
				pen0.location_(new Jun2dPoint(-3 * Math.PI, 0).scaledBy_(scale));
				pen0.goto_(new Jun2dPoint(3 * Math.PI, 0).scaledBy_(scale));
			}
		};
		aThread1.setPriority(Thread.NORM_PRIORITY - 1);
		aThread1.start();

		Thread aThread2 = new Thread() {
			public void run() {
				pen0.location_(new Jun2dPoint(0, -1).scaledBy_(scale));
				pen0.goto_(new Jun2dPoint(0, 1).scaledBy_(scale));
			}
		};
		aThread2.setPriority(Thread.NORM_PRIORITY - 1);
		aThread2.start();
	}

	/**
	 * Program of a pen example.
	 * 
	 * @category programs
	 */
	public void penExample() {
		this.clearPaper();

		final JunPen aPen = this.newPen();
		aPen.nib_(4);
		aPen.color_(Color.gray);

		for (int i = 1; i <= 50; i++) {
			final Integer ii = new Integer(i);
			this.do_forMilliseconds_(new StBlockClosure0<Void>() {
				public Void value() {
					aPen.go_(ii.intValue() * 4);
					aPen.turn_(89);
					return null;
				}
			}, 100);
		}
	}

	/**
	 * Program of triangles.
	 * 
	 * @category programs
	 */
	public void sample1_triangles() {
		this.clearPaper();

		final JunPen aPen = this.newPen();
		aPen.color_(Color.red);
		int increment = 45;

		for (int i = 0; i < 360; i += increment) {
			for (int j = 0; j < 3; j++) {
				this.do_forMilliseconds_(new StBlockClosure0<Void>() {
					public Void value() {
						aPen.go_(100);
						aPen.turn_(120);
						return null;
					}
				}, 100);
			}
			aPen.turn_(increment);
		}
	}

	/**
	 * Program of a star.
	 * 
	 * @category programs
	 */
	public void sample2_star() {
		this.clearPaper();

		JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.color_(Color.red);
		aPen.spiral_angle_tick_(200, 144, 10);
	}

	/**
	 * Program of a dragon.
	 * 
	 * @category programs
	 */
	public void sample3_dragon() {
		this.clearPaper();

		JunPen aPen = this.newPen();
		aPen.color_(Color.red);
		aPen.dragon_distance_tick_(9, 5, 3);
	}

	/**
	 * Program of a mandala.
	 * 
	 * @category programs
	 */
	public void sample4_mandala() {
		this.clearPaper();

		JunPen aPen = this.newPen();
		aPen.color_(Color.red);
		aPen.mandala_diameter_tick_(24, 250, 25);
	}

	/**
	 * Program of a mandala.
	 * 
	 * @category programs
	 */
	public void sample5_spirals() {
		this.clearPaper();

		JunPen[] pen = new JunPen[5];
		pen[0] = this.newPen(Color.red);
		pen[1] = this.newPen(Color.red);
		pen[2] = this.newPen(Color.red);
		pen[3] = this.newPen(Color.red);
		pen[4] = this.newPen(Color.red);

		pen[0].location_(new Jun2dPoint(-75, -75));
		pen[1].location_(new Jun2dPoint(75, -75));
		pen[2].location_(new Jun2dPoint(75, 75));
		pen[3].location_(new Jun2dPoint(-75, 75));
		pen[4].location_(new Jun2dPoint(0, 0));

		for (int i = 0; i < pen.length; i++) {
			pen[i].spiral_angle_tick_(100, 88, 10);
		}
	}

	/**
	 * Program of spirals.
	 * 
	 * @category programs
	 */
	public void sample6_spirals() {
		this.clearPaper();

		JunPen[] pen = new JunPen[5];
		pen[0] = this.newPen(Color.red);
		pen[1] = this.newPen(Color.red);
		pen[2] = this.newPen(Color.red);
		pen[3] = this.newPen(Color.red);
		pen[4] = this.newPen(Color.red);

		pen[0].location_(new Jun2dPoint(-75, -75));
		pen[1].location_(new Jun2dPoint(75, -75));
		pen[2].location_(new Jun2dPoint(75, 75));
		pen[3].location_(new Jun2dPoint(-75, 75));
		pen[4].location_(new Jun2dPoint(0, 0));

		for (int i = 0; i < pen.length; i++) {
			final JunPen aPen = pen[i];
			Thread aThread = new Thread() {
				public void run() {
					aPen.spiral_angle_tick_(100, 88, 10);
				}
			};
			aThread.setPriority(Thread.NORM_PRIORITY - 1);
			aThread.start();
		}
	}

	/**
	 * Program of polylines.
	 * 
	 * @category programs
	 */
	public void sample7_polylines() {
		this.clearPaper();

		JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.location_(JunPenLocation.FromArray_(new double[] { -50, -50, -50 }));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; i++) {
			aPen.go_(100);
			aPen.turn_(90);
		}
		aPen.location_(JunPenLocation.FromArray_(new double[] { 50, -50, -50 }));
		aPen.direction_(JunPenDirection.Longitude_(90));
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; i++) {
			aPen.go_(100);
			aPen.tilt_(90);
		}
		aPen.location_(JunPenLocation.FromArray_(new double[] { -50, 50, -50 }));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; i++) {
			aPen.go_(100);
			aPen.tilt_(90);
		}
	}

	/**
	 * Program of fill.
	 * 
	 * @category programs
	 */
	public void sample8_fill() {
		this.clearPaper();

		final JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.location_(JunPenLocation.FromArray_(new double[] { -50, -50, -50 }));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.red);
		aPen.fill_(new StBlockClosure0<Void>() {
			public Void value() {
				for (int i = 0; i < 4; i++) {
					aPen.go_(100);
					aPen.turn_(90);
				}
				return null;
			}
		});
		aPen.location_(JunPenLocation.FromArray_(new double[] { 50, -50, -50 }));
		aPen.direction_(JunPenDirection.Longitude_(90));
		aPen.color_(Color.blue);
		aPen.fill_(new StBlockClosure0<Void>() {
			public Void value() {
				for (int i = 0; i < 4; i++) {
					aPen.go_(100);
					aPen.tilt_(90);
				}
				return null;
			}
		});
		aPen.location_(JunPenLocation.FromArray_(new double[] { -50, 50, -50 }));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.green);
		aPen.fill_(new StBlockClosure0<Void>() {
			public Void value() {
				for (int i = 0; i < 4; i++) {
					aPen.go_(100);
					aPen.tilt_(90);
				}
				return null;
			}
		});
	}

	/**
	 * Program of spirals
	 * 
	 * @category programs
	 */
	public void sample9_spirals() {
		this.clearPaper();

		final JunPen aPen = this.newPen();
		aPen.nib_(1);
		aPen.color_(Color.blue);

		StBlockClosure1<Void, Double> aBlock = new StBlockClosure1<Void, Double>() {
			public Void value(final Double sign) {
				aPen.home();
				for (int i = 1; i <= 50; i++) {
					final Integer n = new Integer(i);
					JunPlotter.this.do_forMilliseconds_(new StBlockClosure0<Void>() {
						public Void value() {
							JunPenLocation from = aPen.location();
							aPen.up();
							aPen.go_(n.doubleValue() * 4 * sign);
							aPen.goto_(aPen.location().plus_(JunPenLocation.X_y_z_(0, 2 * sign, 0)));
							JunPenLocation to = aPen.location();
							aPen.location_(from);
							aPen.down();
							aPen.goto_(to);
							aPen.tilt_(89);
							return null;
						}
					}, 100);

				}
				return null;
			}
		};

		aBlock.value(1d);
		aBlock.value(-1d);
	}

	/**
	 * Program of combination
	 * 
	 * @category programs
	 */
	public void sampleA_combination() {
		Thread aThread1 = new Thread() {
			public void run() {
				sample4_mandala();
			}
		};
		aThread1.setPriority(Thread.NORM_PRIORITY - 1);
		aThread1.start();

		Thread aThread2 = new Thread() {
			public void run() {
				sample9_spirals();
			}
		};
		aThread2.setPriority(Thread.NORM_PRIORITY - 1);
		aThread2.start();
	}

	/**
	 * Create a new JunPen.
	 * 
	 * @return jp.co.sra.jun.goodies.pen.JunPen
	 * @category menu messages
	 */
	protected JunPen newPen() {
		JunPen aPen = new JunPen();
		aPen.compute_(new StBlockClosure1<Void, List<Object>>() {
			public Void value(List<Object> array) {
				drawings.add(array);
				JunPlotter.this.changed_with_($("draw"), array);
				return null;
			}
		});
		return aPen;
	}

	/**
	 * Create a new pen with the specified color.
	 * 
	 * @param aColor java.awt.Color
	 * @return jp.co.sra.jun.goodies.pen.JunPen
	 * @category private
	 */
	protected JunPen newPen(Color aColor) {
		JunPen aPen = this.newPen();
		aPen.color_(aColor);
		return aPen;
	}

}
