package jp.co.sra.jun.goodies.print;

import java.awt.Point;
import java.awt.event.MouseEvent;

import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.framework.JunAbstractController;

/**
 * JunPrintPreviewController class
 * 
 *  @author    nisinaka
 *  @created   2007/07/27 (by nisinaka)
 *  @updated   2008/08/14 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrintPreviewController.java,v 9.8 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPrintPreviewController extends JunAbstractController {

	protected MouseEvent mousePressedEvent;

	/**
	 * Remove references to objects that may refer to the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StController#release()
	 * @category initialize-release
	 */
	public void release() {
		super.release();
		this.getPrintPreviewCanvas().removeMouseListener(this);
		this.getPrintPreviewCanvas().removeMouseMotionListener(this);
	}

	/**
	 * Setup the listeners.
	 * 
	 * @param aView jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StController#buildListener(jp.co.sra.smalltalk.StView)
	 * @category initialize-release
	 */
	protected void buildListener(StView aView) {
		this.getPrintPreviewCanvas().addMouseListener(this);
		this.getPrintPreviewCanvas().addMouseMotionListener(this);
	}

	/**
	 * Answer my view as PrintPreviewView.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewView
	 * @category view accessing
	 */
	public JunPrintPreviewView getPrintPreviewView() {
		return (JunPrintPreviewView) this.view();
	}

	/**
	 * Answer the print preview canvas of my view.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewCanvas
	 * @category view accessing
	 */
	protected JunPrintPreviewCanvas getPrintPreviewCanvas() {
		return this.getPrintPreviewView().getCanvas();
	}

	/**
	 * Answer my model as PrintPreview.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreview
	 * @category model accessing
	 */
	protected JunPrintPreview getPrintPreview() {
		return this.getPrintPreviewView().getPrintPreview();
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.smalltalk.StController#mousePressed(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 * @category mouse events 
	 */
	public void mousePressed(MouseEvent e) {
		mousePressedEvent = e;
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.smalltalk.StController#mouseReleased(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 * @category mouse events 
	 */
	public void mouseReleased(MouseEvent e) {
		if (this.getPrintPreviewCanvas().getSelectionBounds() != null) {
			// Finish the bounds selection by mouse dragging. 
			this.getPrintPreviewCanvas().setSelectionBounds(null);
		} else {
			int pageIndex = this.getPrintPreviewCanvas().getPageIndexAt(e.getPoint());
			if (pageIndex < 0) {
				this.getPrintPreview().clearPageSelections();
			} else if (e.isControlDown()) {
				this.getPrintPreview().togglePageSelection(pageIndex);
			} else {
				this.getPrintPreview().clearPageSelections();
				this.getPrintPreview().setPageSelection(pageIndex, true);
			}
		}
	}

	/**
	 * Invoked when a mouse is dragged on the view.
	 *
	 * @param e java.awt.event.MouseEvent 
	 * @see jp.co.sra.smalltalk.StController#mouseDragged(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 * @category mouse motion events
	 */
	public void mouseDragged(MouseEvent e) {
		if (e.isControlDown()) {
			return;
		}

		Point pressedPoint = mousePressedEvent.getPoint();
		Point draggedPoint = e.getPoint();
		StRectangle selectionBounds = StRectangle.Vertex_vertex_(pressedPoint, draggedPoint);
		this.getPrintPreviewCanvas().setSelectionBounds(selectionBounds.toRectangle());
	}

	/**
	 * Invoked when a mouse is moved on the view.
	 *
	 * @param e java.awt.event.MouseEvent 
	 * @see jp.co.sra.smalltalk.StController#mouseMoved(java.awt.event.MouseEvent)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 * @category mouse motion events
	 */
	public void mouseMoved(MouseEvent e) {
		//System.out.println(e);
	}

}
