package jp.co.sra.jun.goodies.print;

import java.awt.Dialog;
import java.awt.Frame;

import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunPrintPreviewDialog class
 * 
 *  @author    nisinaka
 *  @created   2007/07/26 (by nisinaka)
 *  @updated   2008/08/14 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrintPreviewDialog.java,v 9.7 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPrintPreviewDialog extends JunDialog {

	protected JunPrintPreview printPreview;

	/**
	 * Open a dialog for previewing the printable object.
	 * 
	 * @param aPrintableObject jp.co.sra.jun.goodies.print.JunPrintableObject
	 * @return boolean
	 * @category Requesting
	 */
	public static boolean Request(JunPrintableObject aPrintableObject) {
		return Request(aPrintableObject, null);
	}

	/**
	 * Open a dialog for previewing the printable object.
	 * 
	 * @param aPrintableObject jp.co.sra.jun.goodies.print.JunPrintableObject
	 * @param owner java.awt.Frame
	 * @return boolean
	 * @category Requesting
	 */
	public static boolean Request(JunPrintableObject aPrintableObject, Frame owner) {
		JunPrintPreviewDialog printPreviewDialog = new JunPrintPreviewDialog(aPrintableObject);
		Dialog aDialog = printPreviewDialog.allButOpenDialog();
		_ShowAtCenterPoint(aDialog);
		Boolean result = printPreviewDialog.accept().value();
		return result.booleanValue();
	}

	/**
	 * Create a new instance of JunPrintPreviewDialog and initialize it.
	 *
	 * @param aPrintableObject jp.co.sra.jun.goodies.print.JunPrintableObject
	 * @category Instance creation
	 */
	protected JunPrintPreviewDialog(JunPrintableObject aPrintableObject) {
		printPreview = new JunPrintPreview(aPrintableObject);
	}

	/**
	 * Answer my current print preview.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreview
	 * @category accessing
	 */
	public JunPrintPreview getPrintPreview() {
		return printPreview;
	}

	/**
	 * Answer my default view.
	 * 
	 * @see jp.co.sra.smalltalk.StSimpleDialog#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		return new JunPrintPreviewDialogViewSwing(this);
	}

	/**
	 * Answer my window title.
	 * 
	 * @see jp.co.sra.smalltalk.StSimpleDialog#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return this.getPrintPreview().windowTitle();
	}

}
