package jp.co.sra.jun.goodies.print;

import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;

import jp.co.sra.smalltalk.StSimpleDialogViewSwing;
import jp.co.sra.smalltalk.WindowSpecLayout;

/**
 * JunPrintPreviewDialogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2007/07/26 (by nisinaka)
 *  @updated   2008/08/14 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrintPreviewDialogViewSwing.java,v 9.8 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPrintPreviewDialogViewSwing extends StSimpleDialogViewSwing implements JunPrintPreviewDialogView {

	protected JunPrintPreviewViewSwing printPreviewView;

	/**
	 * Create a new instance of JunPrintPreviewDialogView and initialize it.
	 *
	 * @param aPrintPreviewDialog jp.co.sra.jun.goodies.print.JunPrintPreviewDialog
	 * @category Instance creation
	 */
	public JunPrintPreviewDialogViewSwing(JunPrintPreviewDialog aPrintPreviewDialog) {
		super(aPrintPreviewDialog);
	}

	/**
	 * Build this component.
	 * 
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.printPreviewView(), WindowSpecLayout.Position(0f, 0, 0f, 0, 1f, 0, 1f, 0));
		this.setPreferredSize(new Dimension(600, 400));

		defaultButton = this.printPreviewView().closeButton();
	}

	/**
	 * Answer my model as JunPrintPreviewDialog.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewDialog
	 * @see jp.co.sra.jun.goodies.print.JunPrintPreviewDialogView#getPrintPreviewDialog()
	 * @category model accessing
	 */
	public JunPrintPreviewDialog getPrintPreviewDialog() {
		return (JunPrintPreviewDialog) this.model();
	}

	/**
	 * Answer the print preview view.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewView
	 * @category user interface
	 */
	protected JunPrintPreviewViewSwing printPreviewView() {
		if (printPreviewView == null) {
			printPreviewView = new JunPrintPreviewViewSwing(this.getPrintPreviewDialog().getPrintPreview()) {
				protected JButton closeButton() {
					if (closeButton == null) {
						closeButton = new JButton(new AbstractAction($String("Close")) {
							public void actionPerformed(ActionEvent e) {
								getPrintPreviewDialog().cancel().value_(true);
							}
						});
					}
					return closeButton;
				}
			};
		}
		return printPreviewView;
	}

}
