package jp.co.sra.jun.goodies.print;

import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.Size2DSyntax;
import javax.print.attribute.standard.MediaPrintableArea;
import javax.print.attribute.standard.MediaSize;
import javax.print.attribute.standard.MediaSizeName;

import jp.co.sra.smalltalk.StImage;

import jp.co.sra.jun.opengl.texture.JunOpenGLTextureTestExamples;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunPrintPreviewTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2007/07/18 (by nisinaka)
 *  @updated   2008/08/14 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrintPreviewTestExamples.java,v 9.7 2010/03/09 09:17:20 nisinaka Exp $
 */
public class JunPrintPreviewTestExamples extends JunTestExamples {

	/**
	 * Example0:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example() {
		final StImage image = new StImage(JunOpenGLTextureTestExamples.ImageSmalltalkBalloon());
		JunPrintableObject printableObject = new JunPrintableObject() {
			protected double getWidth() {
				return image.width();
			}

			protected double getHeight() {
				return image.height();
			}

			protected void print(Graphics aGraphics) {
				image.displayOn_(aGraphics);
			}
		};
		printableObject.setScale(10);

		JunPrintPreview printPreview = new JunPrintPreview(printableObject);
		printPreview.open();

		return true;
	}

	/**
	 * Test_PageSetup1:
	 * 
	 * @return boolean
	 * @category Testing
	 */
	public static boolean Test_PageSetup1() {
		//SystemSupport.PageSetup(null);
		PrinterJob printerJob = PrinterJob.getPrinterJob();
		PageFormat pageFormat = printerJob.pageDialog(new HashPrintRequestAttributeSet());
		if (pageFormat == null) {
			return false;
		}

		JunPrintableObject._Print(pageFormat.getPaper());

		return true;
	}

	/**
	 * Test_PageSetup2:
	 * 
	 * @return boolean
	 * @category Testing
	 */
	public static boolean Test_PageSetup2() {
		PrinterJob printerJob = PrinterJob.getPrinterJob();
		PageFormat pageFormat = printerJob.defaultPage();
		JunPrintableObject._Print(pageFormat.getPaper());
		pageFormat = printerJob.validatePage(pageFormat);
		JunPrintableObject._Print(pageFormat.getPaper());
		pageFormat = printerJob.pageDialog(pageFormat);
		JunPrintableObject._Print(pageFormat.getPaper());
		pageFormat = printerJob.validatePage(pageFormat);
		JunPrintableObject._Print(pageFormat.getPaper());

		return true;
	}

	/**
	 * Test_PrintDialog2:
	 * 
	 * @return boolean
	 * @category Testing
	 */
	public static boolean Test_PrintDialog2() {
		PrinterJob printerJob = PrinterJob.getPrinterJob();
		if (printerJob.printDialog()) {
			System.out.println(printerJob.getPrintService());
		}

		return true;
	}

	/**
	 * Test_Print1:
	 * 
	 * @return boolean
	 * @category Testing
	 */
	public static boolean Test_Print1() {
		PrinterJob printerJob = PrinterJob.getPrinterJob();
		if (printerJob.printDialog() == false) {
			return false;
		}

		PageFormat pageFormat = printerJob.defaultPage();

		Paper paper = pageFormat.getPaper();
		paper.setImageableArea(0, 0, paper.getWidth(), paper.getHeight());
		pageFormat.setPaper(paper);

		pageFormat = printerJob.pageDialog(pageFormat);

		PageFormat validatedPageFormat = printerJob.validatePage(pageFormat);
		printerJob.pageDialog(validatedPageFormat);

		printerJob.setPrintable(new Printable() {
			public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
				if (pageIndex > 0) {
					return Printable.NO_SUCH_PAGE;
				}

				int x = (int) pageFormat.getImageableX();
				int y = (int) pageFormat.getImageableY();
				int width = (int) pageFormat.getImageableWidth();
				int height = (int) pageFormat.getImageableHeight();
				graphics.drawRect(x, y, width, height);
				graphics.drawLine(x, y, x + width, y + height);
				graphics.drawLine(x + width, y, x, y + height);

				return Printable.PAGE_EXISTS;
			}
		}, validatedPageFormat);

		try {
			printerJob.print();
		} catch (PrinterException e) {
			e.printStackTrace();
		}

		return true;
	}

	public static boolean Test_MediaSizeName() {
		MediaSizeName name = MediaSize.findMedia(210.0f, 297.0f, Size2DSyntax.MM);
		System.out.println(name);

		PrinterJob printerJob = PrinterJob.getPrinterJob();
		PageFormat pageFormat = printerJob.defaultPage();
		JunPrintableObject._Print(pageFormat.getPaper());

		float w = (float) JunPrintableObject.POINTtoINCH(pageFormat.getWidth());
		float h = (float) JunPrintableObject.POINTtoINCH(pageFormat.getHeight());
		name = MediaSize.findMedia(w, h, Size2DSyntax.INCH);
		System.out.println(name);

		MediaSize size = MediaSize.getMediaSizeForName(name);
		System.out.println(size);

		PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
		aset.add(name);
		MediaPrintableArea[] printableArea = (MediaPrintableArea[]) printerJob.getPrintService().getSupportedAttributeValues(MediaPrintableArea.class, null, aset);
		System.out.println(printableArea[0]);

		return true;
	}

	/**
	 * Test_PageID:
	 * 
	 * @return boolean
	 * @category Testing
	 */
	public static boolean Test_PageID() {
		for (int i = 0; i < 10; i++) {
			System.out.println(JunPrintableObject.PageID(i, i));
		}

		System.out.println(JunPrintableObject.PageID(25, 25));
		System.out.println(JunPrintableObject.PageID(26, 26));
		System.out.println(JunPrintableObject.PageID(27, 27));
		System.out.println(JunPrintableObject.PageID(26 * 26, 26 * 26));
		System.out.println(JunPrintableObject.PageID(26 * 27 - 1, 26 * 27 - 1));
		System.out.println(JunPrintableObject.PageID(26 * 27, 26 * 27));
		System.out.println(JunPrintableObject.PageID(26 * 27 + 1, 26 * 27 + 1));

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunPrintPreviewTestExamples();
	}

}
