package jp.co.sra.jun.goodies.print;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PrinterException;
import java.util.Arrays;
import java.util.Comparator;

import javax.print.PrintService;
import javax.swing.AbstractAction;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StController;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;

/**
 * JunPrintPreviewViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2007/07/18 (by nisinaka)
 *  @updated   2008/08/14 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrintPreviewViewSwing.java,v 9.10 2010/03/09 09:17:21 nisinaka Exp $
 */
public class JunPrintPreviewViewSwing extends JunAbstractViewJPanel implements JunPrintPreviewView {

	protected JunPrintPreviewCanvas canvas;
	protected JComboBox printersComboBox;
	protected JButton pageSettingsButton;
	protected JSpinner scaleSpinner;
	protected JComboBox fitToComboBox;
	protected JCheckBox printPageIdCheckBox;
	protected JCheckBox printCornerMarksCheckBox;
	protected JButton printButton;
	protected JButton closeButton;

	private boolean isFitting = false;

	/**
	 * Create a new instance of JunPrintPreviewView and initialize it.
	 *
	 * @param aPrintPreview jp.co.sra.jun.goodies.print.JunPrintPreview
	 * @category Instance creation
	 */
	public JunPrintPreviewViewSwing(JunPrintPreview aPrintPreview) {
		super(aPrintPreview);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.canvas(), WindowSpecLayout.Position(0f, 10, 0f, 10, 1f, -200, 1f, -10));

		// Printers
		int top = 20;
		int bottom = top + 30;
		int right = -10;
		int left = -185;
		this.add(this.printersComboBox(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		// Page Settings
		top = bottom + 10;
		bottom = top + 30;
		right = -10;
		left = right - 130;
		this.add(this.pageSettingsButton(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		// Scale
		JLabel scaleLabel = new JLabel($String("Scale") + " :", JLabel.RIGHT);
		top = bottom + 30;
		bottom = top + 25;
		right = -130;
		left = right - scaleLabel.getPreferredSize().width;
		this.add(scaleLabel, WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		left = right + 5;
		right = -60;
		this.add(this.scaleSpinner(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		JLabel unitLabel = new JLabel("%", JLabel.LEFT);
		left = right + 5;
		right = left + unitLabel.getPreferredSize().width;
		this.add(unitLabel, WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		// Fit to
		top = bottom + 10;
		bottom = top + 25;
		left = -160;
		right = -90;
		String fitToPostString = $String("FitTo_POST", "");
		if (fitToPostString.length() == 0) {
			String fitToPreString = $String("FitTo_PRE", "Fit to") + " :";
			JLabel fitToLabel = new JLabel(fitToPreString, JLabel.RIGHT);
			right = -130;
			left = right - fitToLabel.getPreferredSize().width;
			this.add(fitToLabel, WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));
			left = right + 5;
			right = -30;
		}
		this.add(this.fitToComboBox(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));
		if (fitToPostString.length() > 0) {
			JLabel fitToLabel = new JLabel(fitToPostString, JLabel.RIGHT);
			left = right + 5;
			right = left + fitToLabel.getPreferredSize().width;
			this.add(fitToLabel, WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));
		}

		// Print Page ID
		top = bottom + 30;
		bottom = top + 20;
		left = -170;
		right = left + this.printPageIdCheckBox().getPreferredSize().width;
		this.add(this.printPageIdCheckBox(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		// Print Corner Marks
		top = bottom + 5;
		bottom = top + 20;
		right = left + this.printCornerMarksCheckBox().getPreferredSize().width;
		this.add(this.printCornerMarksCheckBox(), WindowSpecLayout.Position(1f, left, 0f, top, 1f, right, 0f, bottom));

		this.add(this.printButton(), WindowSpecLayout.Position(1f, -180, 1f, -40, 1f, -100, 1f, -10));
		this.add(this.closeButton(), WindowSpecLayout.Position(1f, -90, 1f, -40, 1f, -10, 1f, -10));

		this.setPreferredSize(new Dimension(800, 600));
	}

	/**
	 * Answer my model as JunPrintPreview.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreview
	 * @see jp.co.sra.jun.goodies.print.JunPrintPreviewView#getPrintPreview()
	 * @category model accessing
	 */
	public JunPrintPreview getPrintPreview() {
		return (JunPrintPreview) this.model();
	}

	/**
	 * Answer the printable object.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintableObject
	 * @category model accessing
	 */
	public JunPrintableObject getPrintableObject() {
		return this.getPrintPreview().getPrintableObject();
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return jp.co.sra.smalltalk.StController
	 * @see jp.co.sra.smalltalk.StViewJPanel#defaultController()
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunPrintPreviewController();
	}

	/**
	 * Answer my print preview canvas.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewCanvas
	 * @see jp.co.sra.jun.goodies.print.JunPrintPreviewView#getCanvas()
	 * @category accessing
	 */
	public JunPrintPreviewCanvas getCanvas() {
		return this.canvas();
	}

	/**
	 * Answer the print preview canvas.
	 * 
	 * @return jp.co.sra.jun.goodies.print.JunPrintPreviewCanvas
	 * @category user interface
	 */
	protected JunPrintPreviewCanvas canvas() {
		if (canvas == null) {
			canvas = new JunPrintPreviewCanvas(this.getPrintPreview());
		}
		return canvas;
	}

	/**
	 * Answer the printers combobox.
	 * 
	 * @return javax.swing.JComboBox
	 * @category user interface
	 */
	protected JComboBox printersComboBox() {
		if (printersComboBox == null) {
			PrintService[] printServices = null;
			PrintService printService = null;
			try {
				printServices = this.getPrintPreview().getPrinterJob().lookupPrintServices();
				printService = this.getPrintPreview().getPrintService();
			} catch (PrinterException e) {
				String message = e.getMessage();
				JOptionPane.showMessageDialog(this, message);
				return null;
			}

			Arrays.sort(printServices, new Comparator<PrintService>() {
				public int compare(PrintService ps1, PrintService ps2) {
					return ps1.getName().compareTo(ps2.getName());
				}
			});

			printersComboBox = new JComboBox(printServices);
			printersComboBox.setRenderer(new DefaultListCellRenderer() {
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
					if (value instanceof PrintService) {
						value = ((PrintService) value).getName();
					}
					return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				}
			});
			printersComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					PrintService printService = (PrintService) printersComboBox.getSelectedItem();
					try {
						getPrintPreview().setPrintService(printService);
					} catch (PrinterException e1) {
						String message = e1.getMessage();
						JOptionPane.showMessageDialog(JunPrintPreviewViewSwing.this, message);
					}
				}
			});
			printersComboBox.setSelectedItem(printService);
		}
		return printersComboBox;
	}

	/**
	 * Answer the page settings button.
	 * 
	 * @return javax.swing.JButton
	 * @category user interface
	 */
	protected JButton pageSettingsButton() {
		if (pageSettingsButton == null) {
			pageSettingsButton = new JButton(new AbstractAction($String("Page Settings")) {
				public void actionPerformed(ActionEvent e) {
					try {
						getPrintPreview().openPageDialog();
					} catch (PrinterException e1) {
						String message = e1.getMessage();
						JOptionPane.showMessageDialog(JunPrintPreviewViewSwing.this, message);
					}
				}
			});
		}
		return pageSettingsButton;
	}

	/**
	 * Answer the scale spinner.
	 * 
	 * @return javax.swing.JSpinner
	 * @category user interface
	 */
	protected JSpinner scaleSpinner() {
		if (scaleSpinner == null) {
			scaleSpinner = new JSpinner(new SpinnerNumberModel((int) (this.getPrintPreview().getScale() * 100), 10, 9999, 1));
			scaleSpinner.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					double scale = ((Number) scaleSpinner.getValue()).doubleValue() / 100;
					getPrintPreview().setScale(scale);
				}
			});
		}
		return scaleSpinner;
	}

	/**
	 * Answer the fit-to combobox.
	 * 
	 * @return javax.swing.JComboBox
	 * @category user interface
	 */
	protected JComboBox fitToComboBox() {
		if (fitToComboBox == null) {
			String[] options = new String[8];
			options[0] = "";
			options[1] = $String("one page");
			options[2] = $String("one row");
			options[3] = $String("one column");
			options[4] = $String("two columns");
			options[5] = $String("three columns");
			options[6] = $String("four columns");
			options[7] = $String("five columns");

			fitToComboBox = new JComboBox(options);
			fitToComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					isFitting = true;
					try {
						int index = fitToComboBox.getSelectedIndex();
						switch (index) {
							case 0:
								break;
							case 1:
								getPrintPreview().fitToOnePage();
								break;
							case 2:
								getPrintPreview().fitToOneRow();
								break;
							default:
								getPrintPreview().fitToColumns(index - 2);
								break;
						}
					} finally {
						isFitting = false;
					}
				}
			});
		}
		return fitToComboBox;
	}

	/**
	 * Answer the print page ID checkbox.
	 * 
	 * @return javax.swing.JCheckBox
	 * @category user interface
	 */
	protected JCheckBox printPageIdCheckBox() {
		if (printPageIdCheckBox == null) {
			printPageIdCheckBox = new JCheckBox(new AbstractAction($String("Print Page ID")) {
				public void actionPerformed(ActionEvent e) {
					getPrintPreview().doPrintPageID(printPageIdCheckBox.isSelected());
				}
			});

			this.updateForPrintPageID();
		}
		return printPageIdCheckBox;
	}

	/**
	 * Answer the print corner mark checkbox.
	 * 
	 * @return javax.swing.JCheckBox
	 * @category user interface
	 */
	protected JCheckBox printCornerMarksCheckBox() {
		if (printCornerMarksCheckBox == null) {
			printCornerMarksCheckBox = new JCheckBox(new AbstractAction($String("Print Corner Marks")) {
				public void actionPerformed(ActionEvent e) {
					getPrintPreview().doPrintCornerMarks(printCornerMarksCheckBox.isSelected());
				}
			});

			this.updateForPrintCornerMarks();
		}
		return printCornerMarksCheckBox;
	}

	/**
	 * Answer the print button.
	 * 
	 * @return javax.swing.JButton
	 * @category user interface
	 */
	protected JButton printButton() {
		if (printButton == null) {
			printButton = new JButton(new AbstractAction($String("Print")) {
				public void actionPerformed(ActionEvent e) {
					try {
						getPrintPreview().doPrint();
					} catch (PrinterException e1) {
						String message = e1.getMessage();
						JOptionPane.showMessageDialog(JunPrintPreviewViewSwing.this, message);
					}
				}
			});
		}
		return printButton;
	}

	/**
	 * Answer the close button.
	 * 
	 * @return javax.swing.JButton
	 * @category user interface
	 */
	protected JButton closeButton() {
		if (closeButton == null) {
			closeButton = new JButton(new AbstractAction($String("Close")) {
				public void actionPerformed(ActionEvent e) {
					getPrintPreview().closeRequest();
				}
			});
		}
		return closeButton;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent e) {
		StSymbol aspect = e.getAspect();
		if (aspect == $("scale")) {
			this.updateForScale();

			if (isFitting == false) {
				this.fitToComboBox().setSelectedIndex(0);
			}
		} else if (aspect == $("printPageID")) {
			this.updateForPrintPageID();
		} else if (aspect == $("pageFormat")) {
			this.fitToComboBox().setSelectedIndex(0);
		}
	}

	/**
	 * Update for the scale change.
	 * 
	 * @category updating
	 */
	protected void updateForScale() {
		int newScale = (int) Math.round(this.getPrintPreview().getScale() * 100);
		int oldScale = ((Number) scaleSpinner.getValue()).intValue();
		if (newScale == oldScale) {
			return;
		}

		this.scaleSpinner().setValue(new Integer(newScale));
	}

	/**
	 * Update for the print status of the page ID.
	 * 
	 * @category updating
	 */
	protected void updateForPrintPageID() {
		boolean doPrintPageID = this.getPrintPreview().doPrintPageID();
		if (this.printPageIdCheckBox().isSelected() == doPrintPageID) {
			return;
		}

		this.printPageIdCheckBox().setSelected(doPrintPageID);
	}

	/**
	 * Update for the print status of the corner marks.
	 * 
	 * @category updating
	 */
	protected void updateForPrintCornerMarks() {
		boolean doPrintCornerMark = this.getPrintPreview().doPrintCornerMarks();
		if (this.printCornerMarksCheckBox().isSelected() == doPrintCornerMark) {
			return;
		}

		this.printCornerMarksCheckBox().setSelected(doPrintCornerMark);
	}

}
