package jp.co.sra.jun.goodies.progress;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunProgressTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1998/11/18 (by nisinaka)
 *  @updated   2008/05/13 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun715 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunProgressTestExamples.java,v 9.10 2010/03/09 09:17:21 nisinaka Exp $
 */
public class JunProgressTestExamples extends JunTestExamples {

	/**
	 * Example: 
	 * 
	 * @category Examples
	 * @return boolean
	 */
	public static boolean Example() {
		JunProgress aProgress = new JunProgress();
		aProgress.message_("abc");
		aProgress.openWithStopButton_(true);

		return true;
	}

	/**
	 * Example1: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		final int size = 100;
		JunProgress aProgress = new JunProgress();
		aProgress.do_(new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				for (int count = 1; count <= size; count++) {
					float value = (float) count / size;
					progress.message_(String.valueOf(value));
					progress.value_(value);
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				return null;
			}
		});

		return true;
	}

	/**
	 * Example2:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		final int size = 100;
		JunProgress aProgress = new JunProgress();
		aProgress.doWithStopButton_(new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				for (int count = 1; count <= size; count++) {
					float value = (float) count / size;
					progress.message_(String.valueOf(value));
					progress.value_(value);
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				return null;
			}
		});

		return true;
	}

	/**
	 * Example3:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		final int size = 100;
		JunProgress aProgress = new JunProgress();
		StBlockClosure1<Void, JunProgress> aBlock = new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				for (int count = 1; count <= size; count++) {
					float value = (float) count / size;
					if (progress.hasStopButton()) {
						value = 1 - value;
					}
					progress.message_(String.valueOf(value));
					progress.value_(value);
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				return null;
			}
		};

		aProgress.do_(aBlock);
		aProgress.doWithStopButton_(aBlock);

		return true;
	}

	/**
	 * Example4:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		final int size = 100;
		final JunProgress aProgress = new JunProgress();
		aProgress.compute_(new StBlockClosure1<Void, Float>() {
			public Void value(Float value) {
				aProgress.message_(String.format("%4.2f", value));
				return null;
			}
		});
		aProgress.do_(new StBlockClosure0<Void>() {
			public Void value() {
				for (int count = 1; count <= size; count++) {
					float value = (float) count / size;
					aProgress.value_(value);
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				return null;
			}
		});

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunProgressTestExamples();
	}
}
