package jp.co.sra.jun.goodies.prolog;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.PopupMenu;
import java.awt.TextArea;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;
import jp.co.sra.smalltalk.menu.StPopupMenuViewAwt;

import jp.co.sra.jun.system.framework.JunAbstractViewPanel;

/**
 * JunPrologInterpreterViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/05/06 (by nisinaka)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2005/03/03 (by nisinaka)
 *  @updated   2008/05/23 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun490 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrologInterpreterViewAwt.java,v 9.10 2010/03/09 09:17:22 nisinaka Exp $
 */
public class JunPrologInterpreterViewAwt extends JunAbstractViewPanel implements JunPrologInterpreterView {

	protected TextArea textValueView;
	protected TextArea textCollectorView;

	/**
	 * Create a new instance of <code>JunPrologInterpreterViewAwt</code> and initialize it.
	 * 
	 * @param aPrologInterpreter jp.co.sra.jun.goodies.prolog.JunPrologInterpreter
	 * @category Instance creation
	 */
	public JunPrologInterpreterViewAwt(JunPrologInterpreter aPrologInterpreter) {
		super(aPrologInterpreter);
	}

	/**
	 * Answer my model as JunPrologInterpreter.
	 *
	 * @return jp.co.sra.jun.goodies.prolog.JunPrologInterpreter
	 * @see jp.co.sra.jun.goodies.prolog.JunPrologInterpreterView#getPrologInterpreter()
	 * @category model accessing
	 */
	public JunPrologInterpreter getPrologInterpreter() {
		return (JunPrologInterpreter) this.model();
	}

	/**
	 * Answer the selected text to 'do it'.
	 *
	 * @return java.lang.String
	 * @see jp.co.sra.jun.goodies.prolog.JunPrologInterpreterView#getSelectedText()
	 * @category accessing
	 */
	public String getSelectedText() {
		return this.textValueView().getSelectedText();
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 * @category bounds accessing
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		this.textValueView().setBounds(1, 1, width - 2, (int) (height * 0.3) - 2);
		this.textCollectorView().setBounds(1, (int) (height * 0.3), width - 2, (int) (height * 0.7) - 1);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(null);
		this.add(this.textValueView());
		this.add(this.textCollectorView());
		this.setPreferredSize(new Dimension(350, 400));
	}

	/**
	 * Answer the view of my text collector.
	 *
	 * @return java.awt.TextArea
	 * @category interface opening
	 */
	protected TextArea textCollectorView() {
		if (textCollectorView == null) {
			textCollectorView = new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
			textCollectorView.setBackground(Color.lightGray);

			ByteArrayOutputStream aByteArrayOutputStream = new ByteArrayOutputStream() {
				public synchronized void write(byte b[], int off, int len) {
					super.write(b, off, len);
					textCollectorView.append(new String(b, off, len));
				}
			};
			this.getPrologInterpreter().textCollector_(new PrintStream(aByteArrayOutputStream));
		}
		return textCollectorView;
	}

	/**
	 * Answer the view of my text value.
	 *
	 * @return java.awt.TextArea
	 * @category interface opening
	 */
	protected TextArea textValueView() {
		if (textValueView == null) {
			textValueView = new TextArea(this.getPrologInterpreter().textValue().value(), 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);

			StPopupMenu aPopupMenu = new StPopupMenu();
			aPopupMenu.add(new StMenuItem("do it", new MenuPerformer(this.getPrologInterpreter(), "doIt", this)));

			final PopupMenu aPopupMneu = new StPopupMenuViewAwt(aPopupMenu).toPopupMenu();
			textValueView.add(aPopupMneu);
			textValueView.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isMetaDown() || e.isPopupTrigger()) {
						aPopupMneu.show(textValueView, e.getX(), e.getY());
					}
				}
			});

			this.getPrologInterpreter().textValue().compute_(new StBlockClosure1<String, String>() {
				public String value(String newValue) {
					textValueView.setText(newValue);
					return null;
				}
			});
		}
		return textValueView;
	}
}
