package jp.co.sra.jun.goodies.prolog;

import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StReadStream;

/**
 * JunPrologVariable class
 * 
 *  @author    kondo
 *  @created   1999/09/09 (by kondo)
 *  @updated   2003/04/30 (by nisinaka)
 *  @updated   2008/04/21 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun301 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrologVariable.java,v 9.9 2010/03/09 09:17:22 nisinaka Exp $
 */
public class JunPrologVariable extends JunPrologString {
	protected static int JunPrologVariableCounter;
	protected static Map<String, JunPrologVariable> JunPrologVariableTable;

	static {
		Initialize();
	}

	/**
	 * Create a new instance of <code>JunPrologVariable</code> and initialize it.
	 * 
	 * @param aString java.lang.String
	 * @category Instance creation
	 */
	private JunPrologVariable(String aString) {
		super(aString);
	}

	/**
	 * Increment the number of variables.
	 */
	public static void CountUp() {
		JunPrologVariableCounter++;
	}

	/**
	 * Install the string as a JunPrologVariable.
	 *
	 * @return jp.co.sra.jun.goodies.prolog.JunPrologVariable
	 * @param aString java.lang.String
	 */
	public static JunPrologVariable Install_(String aString) {
		JunPrologVariable variable = JunPrologVariableTable.get(aString);
		if (variable == null) {
			variable = new JunPrologVariable(aString);
			JunPrologVariableTable.put(aString, variable);
		}
		return variable;
	}

	/**
	 * Read a JunPrologSymbol from the stream.
	 *
	 * @return jp.co.sra.jun.goodies.prolog.JunPrologEntity
	 * @param aStream jp.co.sra.smalltalk.StReadStream
	 */
	public static JunPrologEntity ReadFrom_(StReadStream aStream) {
		boolean[] map = JunPrologCharMap.SymbolMap();
		StringBuffer buffer = new StringBuffer(20);
		char ch = aStream.peek();
		if (ch == '~') {
			aStream.next();
			CountUp();
			return Install_("~" + JunPrologVariableCounter);
		}
		while (ch != 0 && map[ch]) {
			buffer.append(aStream.next());
			ch = aStream.peek();
		}
		return Install_(buffer.toString());
	}

	/**
	 * Set the number of variables to zero.
	 */
	public static void SetZero() {
		JunPrologVariableCounter = 0;
	}

	/**
	 * Initialize the class variables.
	 * 
	 * @category Class initialization
	 */
	protected static void Initialize() {
		SetZero();
		JunPrologVariableTable = new Hashtable<String, JunPrologVariable>();
	}

	/**
	 * Answer true if the receiver is a kind of JunPrologVariable, otherwise false.
	 *
	 * @return boolean
	 * @category testing
	 */
	public boolean isJunPrologVariable() {
		return true;
	}

	/**
	 * Print my string representation on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException if failed.
	 * @category printing
	 */
	public void printJunPrologOn_(Writer aWriter) throws IOException {
		if (this.asString().charAt(0) == '~') {
			aWriter.write('~');
		} else {
			aWriter.write(this.asString());
		}
	}
}
