package jp.co.sra.jun.goodies.sound;

import java.util.Hashtable;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunSoundMeterModel class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/08/29 (by Hoshi Takanori)
 *  @updated   2008/04/18 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun651 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSoundMeterModel.java,v 9.10 2010/03/09 09:17:23 nisinaka Exp $
 */
public class JunSoundMeterModel extends JunApplicationModel {

	protected StValueHolder<Object[]> soundValueHolder;
	protected Hashtable<Double, double[]> soundLevelData;

	/**
	 * Answer the sound level data.
	 *
	 * @return java.util.Hashtable&lt;java.lang.Double, double[]&gt;
	 * @category accessing
	 */
	public Hashtable<Double, double[]> soundLevelData() {
		if (soundLevelData == null) {
			soundLevelData = new Hashtable<Double, double[]>();
		}
		return soundLevelData;
	}

	/**
	 * Answer the sound level data at key.
	 *
	 * @param key java.lang.Object
	 * @return double[]
	 * @category accessing
	 */
	public double[] soundLevelDataAt_(Object key) {
		if (this.soundLevelData().containsKey(key)) {
			Object value = this.soundLevelData().get(key);
			if (value instanceof double[] && ((double[]) value).length >= 2) {
				return (double[]) value;
			}
		}
		return null;
	}

	/**
	 * Answer the sound value holder.
	 *
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category accessing
	 */
	public StValueHolder<Object[]> soundValueHolder() {
		return soundValueHolder;
	}

	/**
	 * Set the sound value holder to valueHolder.
	 *
	 * @param valueHolder jp.co.sra.smalltalk.StValueHolder
	 * @category accessing
	 */
	public void soundValueHolder_(StValueHolder<Object[]> valueHolder) {
		if (soundValueHolder != null) {
			soundValueHolder.removeDependent_(this);
		}
		soundValueHolder = valueHolder;
		if (soundValueHolder != null) {
			soundValueHolder.addDependent_(this);
		}
	}

	/**
	 * Add an association.
	 *
	 * @param objects java.lang.Object[]
	 * @category adding
	 */
	public void add_(Object[] objects) {
		Double key = (Double) objects[0];
		double[] value = (double[]) objects[1];
		this.soundLevelData().put(key, value);
		this.changed_with_($("wave"), key);
		this.changed_with_($("level"), value);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunSoundMeterViewAwt(this);
		} else {
			return new JunSoundMeterViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	public String windowTitle() {
		return $String("Sound Meter");
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		StSymbol aSymbol = evt.getAspect();
		if (aSymbol == $("value")) {
			if (soundValueHolder != null) {
				this.add_(soundValueHolder.value());
			}
		} else {
			super.update_(evt);
		}
	}
}
