package jp.co.sra.jun.goodies.tables;

import java.io.File;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeSet;

import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StBlockClosure3;

import jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunAdjacencyMatrixTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2006/04/06 (by nisinaka)
 *  @updated   2008/05/21 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun594 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAdjacencyMatrixTestExamples.java,v 9.12 2010/03/09 09:17:24 nisinaka Exp $
 */
public class JunAdjacencyMatrixTestExamples extends JunTestExamples {

	/**
	 * Example01: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example01() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix.beDirectedGraph();
		System.out.println(adjacencyMatrix.printString());

		return true;
	}

	/**
	 * Example02: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example02() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix.beUndirectedGraph();
		System.out.println(adjacencyMatrix.printString());

		return true;
	}

	/**
	 * Example03: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example03() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix = new JunAdjacencyMatrix<String, Integer>(adjacencyMatrix.toLispList());
		System.out.println(adjacencyMatrix.printString());

		return true;
	}

	/**
	 * Example10: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example10() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix.nodesDepthFirstDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(node);
				System.out.println();
				return null;
			}
		});
		System.out.println();
		adjacencyMatrix.arcsDepthFirstDo_(new StBlockClosure3<Void, JunAdjacencyMatrix<String, Integer>.Arc, Integer, Integer>() {
			public Void value(JunAdjacencyMatrix<String, Integer>.Arc arc, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(arc.first);
				System.out.print("==");
				System.out.print(arc.attribute);
				System.out.print("=>");
				System.out.print(arc.last);
				System.out.println();
				return null;
			}
		});

		return true;
	}

	/**
	 * Example11: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example11() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleCycle();
		adjacencyMatrix.nodesDepthFirstDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(node);
				System.out.println();
				return null;
			}
		});
		System.out.println();
		adjacencyMatrix.arcsDepthFirstDo_(new StBlockClosure3<Void, JunAdjacencyMatrix<String, Integer>.Arc, Integer, Integer>() {
			public Void value(JunAdjacencyMatrix<String, Integer>.Arc arc, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(arc.first);
				System.out.print("==");
				System.out.print(arc.attribute);
				System.out.print("=>");
				System.out.print(arc.last);
				System.out.println();
				return null;
			}
		});

		return true;
	}

	/**
	 * Example12: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example12() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix.nodesBreadthFirstDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(node);
				System.out.println();
				return null;
			}
		});
		System.out.println();
		adjacencyMatrix.arcsBreadthFirstDo_(new StBlockClosure3<Void, JunAdjacencyMatrix<String, Integer>.Arc, Integer, Integer>() {
			public Void value(JunAdjacencyMatrix<String, Integer>.Arc arc, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence);
				System.out.print(": ");
				System.out.print(arc.first);
				System.out.print("==");
				System.out.print(arc.attribute);
				System.out.print("=>");
				System.out.print(arc.last);
				System.out.println();
				return null;
			}
		});

		return true;
	}

	/**
	 * Example13: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example13() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		adjacencyMatrix.nodesTopologicalSortDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print('\t');
				}
				System.out.print(sequence.toString());
				System.out.print(": ");
				System.out.print(node);
				System.out.println();
				return null;
			}
		});
		System.out.println();
		Object[] nodes = adjacencyMatrix.nodesTopologicalSort();
		for (int i = 0; i < nodes.length; i++) {
			System.out.println(nodes[i].toString());
		}

		return true;
	}

	/**
	 * Example14: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example14() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleSimple();
		Map<String, Map<String, Integer>> shortestPaths = adjacencyMatrix.shortestPaths();

		TreeSet<Map.Entry<String, Map<String, Integer>>> aTreeSet = new TreeSet<Map.Entry<String, Map<String, Integer>>>(new Comparator<Map.Entry<String, Map<String, Integer>>>() {
			public int compare(Map.Entry<String, Map<String, Integer>> e1, Map.Entry<String, Map<String, Integer>> e2) {
				return e1.getKey().compareTo(e2.getKey());
			}
		});
		aTreeSet.addAll(shortestPaths.entrySet());
		Iterator<Map.Entry<String, Map<String, Integer>>> iterator = aTreeSet.iterator();
		while (iterator.hasNext()) {
			Map.Entry<String, Map<String, Integer>> entry = iterator.next();
			String startNode = entry.getKey();
			Map<String, Integer> shortestPath = entry.getValue();

			TreeSet<Map.Entry<String, Integer>> aTreeSet2 = new TreeSet<Map.Entry<String, Integer>>(new Comparator<Map.Entry<String, Integer>>() {
				public int compare(Map.Entry<String, Integer> e1, Map.Entry<String, Integer> e2) {
					return e1.getKey().compareTo(e2.getKey());
				}
			});
			aTreeSet2.addAll(shortestPath.entrySet());
			Iterator<Map.Entry<String, Integer>> iterator2 = aTreeSet2.iterator();
			while (iterator2.hasNext()) {
				Map.Entry<String, Integer> entry2 = iterator2.next();
				String node = entry2.getKey();
				Integer distance = entry2.getValue();

				System.out.print(startNode);
				System.out.print(" -- ");
				System.out.print(distance);
				System.out.print(" -> ");
				System.out.print(node);
				System.out.println();
			}
		}

		return true;
	}

	/**
	 * Example15: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example15() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = ExampleCycle();
		Map<String, Map<String, Integer>> shortestPaths = adjacencyMatrix.shortestPaths();

		TreeSet<Map.Entry<String, Map<String, Integer>>> aTreeSet = new TreeSet<Map.Entry<String, Map<String, Integer>>>(new Comparator<Map.Entry<String, Map<String, Integer>>>() {
			public int compare(Map.Entry<String, Map<String, Integer>> e1, Map.Entry<String, Map<String, Integer>> e2) {
				return e1.getKey().compareTo(e2.getKey());
			}
		});
		aTreeSet.addAll(shortestPaths.entrySet());
		Iterator<Map.Entry<String, Map<String, Integer>>> iterator = aTreeSet.iterator();
		while (iterator.hasNext()) {
			Map.Entry<String, Map<String, Integer>> entry = iterator.next();
			String startNode = entry.getKey();
			Map<String, Integer> shortestPath = entry.getValue();

			TreeSet<Map.Entry<String, Integer>> aTreeSet2 = new TreeSet<Map.Entry<String, Integer>>(new Comparator<Map.Entry<String, Integer>>() {
				public int compare(Map.Entry<String, Integer> e1, Map.Entry<String, Integer> e2) {
					return e1.getKey().compareTo(e2.getKey());
				}
			});
			aTreeSet2.addAll(shortestPath.entrySet());
			Iterator<Map.Entry<String, Integer>> iterator2 = aTreeSet2.iterator();
			while (iterator2.hasNext()) {
				Map.Entry<String, Integer> entry2 = iterator2.next();
				String node = entry2.getKey();
				Integer distance = entry2.getValue();

				System.out.print(startNode);
				System.out.print(" -- ");
				System.out.print(distance);
				System.out.print(" -> ");
				System.out.print(node);
				System.out.println();
			}
		}

		return true;
	}

	/**
	 * Example16: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example16() {
		JunAdjacencyMatrix<String, Object> aMatrix = new JunAdjacencyMatrix<String, Object>();
		aMatrix.connect_with_("a", "b");
		aMatrix.connect_with_("a", "c");
		aMatrix.connect_with_("b", "c");
		JunAdjacencyMatrix<String, Object> anotherMatrix = new JunAdjacencyMatrix<String, Object>();
		anotherMatrix.connect_with_("f", "d");
		anotherMatrix.connect_with_("e", "d");
		anotherMatrix.connect_with_("e", "f");
		boolean isIsomorphic = aMatrix.isIsomorphicTo_(anotherMatrix);
		System.out.println(isIsomorphic);

		return isIsomorphic;
	}

	/**
	 * Example17: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example17() {
		JunAdjacencyMatrix<String, Integer> aMatrix = ExampleSimple();
		JunAdjacencyMatrix<String, Integer> anotherMatrix = ExampleCycle();
		boolean isIsomorphic = aMatrix.isIsomorphicTo_(anotherMatrix);
		System.out.println(isIsomorphic);

		return !isIsomorphic;
	}

	/**
	 * Example20: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example20() {
		JunAdjacencyMatrix<String, Class<?>> adjacencyMatrix = ExampleTree();
		adjacencyMatrix.nodesDepthFirstDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print("|-- ");
				}
				System.out.println(node);
				return null;
			}
		});

		return true;
	}

	/**
	 * Example21: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example21() {
		JunAdjacencyMatrix<String, Object> adjacencyMatrix = ExampleForest();
		adjacencyMatrix.nodesDepthFirstDo_(new StBlockClosure3<Void, String, Integer, Integer>() {
			public Void value(String node, Integer indent, Integer sequence) {
				for (int i = 0; i < indent; i++) {
					System.out.print("|-- ");
				}
				System.out.println(node);
				return null;
			}
		});

		return true;
	}

	/**
	 * Example22: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example22() {
		JunAdjacencyMatrix<File, Object> adjacencyMatrix = ExampleDirectory_withFiles_(3, true);
		if (adjacencyMatrix == null) {
			return false;
		}

		adjacencyMatrix.visitDepthFirst_nodeDo_arcDo_(null, new StBlockClosure3<Void, File, Integer, Integer>() {
			public Void value(File aFile, Integer indent, Integer sequence) {
				int level = ((Number) indent).intValue();
				if (level > 0) {
					for (int i = 1; i < level; i++) {
						System.out.print("|  ");
					}
					if (aFile.isDirectory()) {
						System.out.print("+- ");
					} else {
						System.out.print("|- ");
					}
				}
				System.out.println(aFile.getName());
				return null;
			}
		}, null);

		return true;
	}

	/**
	 * Create a new simple adjacency matrix.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAdjacencyMatrix
	 * @category Examples
	 */
	protected static JunAdjacencyMatrix<String, Integer> ExampleSimple() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = new JunAdjacencyMatrix<String, Integer>();
		adjacencyMatrix.connect_with_("abc", "def");
		adjacencyMatrix.connect_with_cost_("abc", "ghi", 3);
		adjacencyMatrix.connect_with_("abc", "jkl");
		adjacencyMatrix.connect_with_("def", "mno");
		adjacencyMatrix.connect_with_("ghi", "pqr");
		adjacencyMatrix.connect_with_("ghi", "stu");
		adjacencyMatrix.connect_with_("jkl", "vwx");
		adjacencyMatrix.connect_with_cost_("jkl", "yz0", 2);
		adjacencyMatrix.connect_with_cost_("stu", "123", 4);
		adjacencyMatrix.connect_with_("stu", "456");
		adjacencyMatrix.add_("789");
		return adjacencyMatrix;
	}

	/**
	 * Create a new cycle adjacency matrix.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAdjacencyMatrix
	 * @category Examples
	 */
	protected static JunAdjacencyMatrix<String, Integer> ExampleCycle() {
		JunAdjacencyMatrix<String, Integer> adjacencyMatrix = new JunAdjacencyMatrix<String, Integer>();
		adjacencyMatrix.connect_with_("abc", "def");
		adjacencyMatrix.connect_with_cost_("abc", "ghi", 3);
		adjacencyMatrix.connect_with_("abc", "jkl");
		adjacencyMatrix.connect_with_("def", "mno");
		adjacencyMatrix.connect_with_("ghi", "pqr");
		adjacencyMatrix.connect_with_("ghi", "stu");
		adjacencyMatrix.connect_with_("jkl", "vwx");
		adjacencyMatrix.connect_with_cost_("jkl", "yz0", 2);
		adjacencyMatrix.connect_with_cost_("stu", "123", 4);
		adjacencyMatrix.connect_with_("stu", "456");
		adjacencyMatrix.connect_with_("789", "abc");
		adjacencyMatrix.connect_with_("123", "789");

		return adjacencyMatrix;
	}

	/**
	 * Create a new tree adjacency matrix.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAdjacencyMatrix
	 * @category Examples
	 */
	protected static JunAdjacencyMatrix<String, Class<?>> ExampleTree() {
		StBlockClosure2<Void, JunAdjacencyMatrix<String, Class<?>>, Class<?>> aBlock = new StBlockClosure2<Void, JunAdjacencyMatrix<String, Class<?>>, Class<?>>() {
			public Void value(JunAdjacencyMatrix<String, Class<?>> aMatrix, Class<?> aClass) {
				Class<?> superClass = aClass.getSuperclass();
				if (superClass != null) {
					aMatrix.connect_with_(superClass.getName(), aClass.getName());
					this.value(aMatrix, superClass);
				}
				return null;
			}
		};

		JunAdjacencyMatrix<String, Class<?>> adjacencyMatrix = new JunAdjacencyMatrix<String, Class<?>>();
		aBlock.value(adjacencyMatrix, java.util.ArrayList.class);
		aBlock.value(adjacencyMatrix, java.util.HashMap.class);
		aBlock.value(adjacencyMatrix, java.util.HashSet.class);
		aBlock.value(adjacencyMatrix, java.util.Hashtable.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedHashSet.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedHashMap.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedList.class);
		aBlock.value(adjacencyMatrix, java.util.Stack.class);
		aBlock.value(adjacencyMatrix, java.util.TreeMap.class);
		aBlock.value(adjacencyMatrix, java.util.TreeSet.class);
		aBlock.value(adjacencyMatrix, java.util.Vector.class);

		return adjacencyMatrix;
	}

	/**
	 * Create a new forest adjacency matrix.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAdjacencyMatrix
	 * @category Examples
	 */
	protected static JunAdjacencyMatrix<String, Object> ExampleForest() {
		StBlockClosure2<Void, JunAdjacencyMatrix<String, Object>, Class<?>> aBlock = new StBlockClosure2<Void, JunAdjacencyMatrix<String, Object>, Class<?>>() {
			public Void value(JunAdjacencyMatrix<String, Object> aMatrix, Class<?> aClass) {
				Class<?> superClass = aClass.getSuperclass();
				if (superClass != java.lang.Object.class) {
					aMatrix.connect_with_(superClass.getName(), aClass.getName());
					this.value(aMatrix, superClass);
				}
				return null;
			}
		};

		JunAdjacencyMatrix<String, Object> adjacencyMatrix = new JunAdjacencyMatrix<String, Object>();
		aBlock.value(adjacencyMatrix, java.util.ArrayList.class);
		aBlock.value(adjacencyMatrix, java.util.HashMap.class);
		aBlock.value(adjacencyMatrix, java.util.HashSet.class);
		aBlock.value(adjacencyMatrix, java.util.Hashtable.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedHashSet.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedHashMap.class);
		aBlock.value(adjacencyMatrix, java.util.LinkedList.class);
		aBlock.value(adjacencyMatrix, java.util.Stack.class);
		aBlock.value(adjacencyMatrix, java.util.TreeMap.class);
		aBlock.value(adjacencyMatrix, java.util.TreeSet.class);
		aBlock.value(adjacencyMatrix, java.util.Vector.class);

		return adjacencyMatrix;
	}

	/**
	 * Create a new directory adjacency matrix.
	 * 
	 * @param diveLevel int
	 * @param withFiles boolean
	 * @return jp.co.sra.jun.goodies.tables.JunAdjacencyMatrix
	 * @category Examples
	 */
	protected static JunAdjacencyMatrix<File, Object> ExampleDirectory_withFiles_(final int diveLevel, final boolean withFiles) {
		File theDirectory = JunFileRequesterDialog.RequestDirectory($String("Select a directory."));
		if (theDirectory == null) {
			return null;
		}
		if (theDirectory.isDirectory() == false) {
			return null;
		}

		StBlockClosure3<Void, JunAdjacencyMatrix<File, Object>, File, Integer> aBlock = new StBlockClosure3<Void, JunAdjacencyMatrix<File, Object>, File, Integer>() {
			public Void value(JunAdjacencyMatrix<File, Object> aMatrix, File aDirectory, Integer level) {
				if (level < diveLevel) {
					File[] files = aDirectory.listFiles();
					for (int i = 0; i < files.length; i++) {
						if (files[i].isDirectory()) {
							aMatrix.connect_with_(aDirectory, files[i]);
							this.value(aMatrix, files[i], new Integer(level + 1));
						} else if (withFiles) {
							aMatrix.connect_with_(aDirectory, files[i]);
						}
					}
				}

				return null;
			}
		};

		JunAdjacencyMatrix<File, Object> adjacencyMatrix = new JunAdjacencyMatrix<File, Object>();
		aBlock.value(adjacencyMatrix, theDirectory, new Integer(1));

		return adjacencyMatrix;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunAdjacencyMatrixTestExamples();
	}

}
