package jp.co.sra.jun.goodies.tables;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunAdjacencyTable class
 * 
 *  @author    nisinaka
 *  @created   2006/04/06 (by nisinaka)
 *  @updated   2008/04/18 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun592 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAdjacencyTable.java,v 9.10 2010/03/09 09:17:24 nisinaka Exp $
 */
public class JunAdjacencyTable<K, V> extends JunAttributeTable<K, JunReferenceTable<K, JunAttributeTable<StSymbol, V>>> {

	/**
	 * Create a new instance of JunAdjacencyTable and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunAdjacencyTable() {
		super();
	}

	/**
	 * Create a new instance of JunAdjacencyTable and initialize it.
	 *
	 * @param size int
	 * @category Instance creation
	 */
	public JunAdjacencyTable(int size) {
		super(size);
	}

	/**
	 * Create a new instance of JunAdjacencyTable and initialize it.
	 *
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunAdjacencyTable(JunLispList aList) {
		super(aList);
	}

	/**
	 * Answer my bindings as adjacencies.
	 * 
	 * @return java.util.ArrayList
	 * @category accessing
	 */
	public ArrayList<Map.Entry<K, JunReferenceTable<K, JunAttributeTable<StSymbol, V>>>> adjacencies() {
		return this.bindings();
	}

	/**
	 * Answer the attributes at the first object with the last object. 
	 * 
	 * @param first java.lang.Object
	 * @param last java.lang.Object
	 * @return jp.co.sra.goodies.tables.JunAttributeTable
	 * @category accessing
	 */
	public JunAttributeTable<StSymbol, V> at_with_(K first, K last) {
		JunAttributeTable<K, JunAttributeTable<StSymbol, V>> table = this.at_(first);
		return (table == null) ? null : table.at_(last);
	}

	/**
	 * Put the attributes at the first object with the last object.
	 * 
	 * @param first java.lang.Object
	 * @param last java.lang.Object
	 * @param attributes jp.co.sra.goodies.tables.JunAttributeTable
	 * @category accessing
	 */
	public void at_with_put_(K first, K last, JunAttributeTable<StSymbol, V> attributes) {
		JunReferenceTable<K, JunAttributeTable<StSymbol, V>> table = this.at_(first);
		if (table != null) {
			if (attributes == null) {
				table.removeKey_(last);
			} else {
				table.at_put_(last, attributes);
			}
		}
	}

	/**
	 * Add the node to the receiver. 
	 * 
	 * @param node java.lang.Object
	 * @return jp.co.sra.jun.goodies.tables.JunReferenceTable
	 * @category adding
	 */
	public JunReferenceTable<K, JunAttributeTable<StSymbol, V>> add_(K node) {
		JunReferenceTable<K, JunAttributeTable<StSymbol, V>> table = this.at_(node);
		if (table != null) {
			return null;
		} else {
			table = new JunReferenceTable<K, JunAttributeTable<StSymbol, V>>();
			this.at_put_(node, table);
			return table;
		}
	}

	/**
	 * Remove the node from the receiver.
	 * 
	 * @param node java.lang.Object
	 * @return java.lang.Object
	 * @category removing
	 */
	public JunReferenceTable<K, JunAttributeTable<StSymbol, V>> remove_(K node) {
		Iterator<Map.Entry<K, JunReferenceTable<K, JunAttributeTable<StSymbol, V>>>> iterator = this.entries().iterator();
		while (iterator.hasNext()) {
			Map.Entry<K, JunReferenceTable<K, JunAttributeTable<StSymbol, V>>> entry = iterator.next();
			JunReferenceTable<K, JunAttributeTable<StSymbol, V>> table = entry.getValue();
			table.removeKey_(node);
		}
		return this.removeKey_(node);
	}

	/**
	 * Connect the first object with the last object. 
	 * 
	 * @param first java.lang.Object
	 * @param last java.lang.Object
	 * @param isUndirected boolean
	 * @return java.lang.Boolean
	 * @category connecting
	 */
	public Boolean connect_with_undirected_(K first, K last, boolean isUndirected) {
		Boolean triplet = null;

		this.add_(first);
		this.add_(last);
		if ((first == null) ? (last == null) : first.equals(last)) {
			triplet = null;
		} else {
			triplet = Boolean.FALSE;
			if (this.at_with_(first, last) == null) {
				this.at_with_put_(first, last, new JunAttributeTable<StSymbol, V>());
				triplet = Boolean.TRUE;
			}
			if (isUndirected) {
				if (this.at_with_(last, first) == null) {
					this.at_with_put_(last, first, new JunAttributeTable<StSymbol, V>());
					triplet = Boolean.TRUE;
				}
			}
		}

		return triplet;
	}

	/**
	 * Disconnect the first object with the last object. 
	 * 
	 * @param first java.lang.Object
	 * @param last java.lang.Object
	 * @param isUndirected boolean
	 * @return java.lang.Boolean
	 * @category connecting
	 */
	public Boolean disconnect_with_undirected_(K first, K last, boolean isUndirected) {
		Boolean triplet = null;

		if ((first == null) ? (last == null) : first.equals(last)) {
			triplet = null;
		} else {
			triplet = Boolean.FALSE;
			if (this.at_with_(first, last) != null) {
				this.at_with_put_(first, last, null);
				triplet = Boolean.TRUE;
			}
			if (isUndirected) {
				if (this.at_with_(last, first) != null) {
					this.at_with_put_(last, first, null);
					triplet = Boolean.TRUE;
				}
			}
		}

		return triplet;
	}

}
