package jp.co.sra.jun.goodies.texteditor;

import java.awt.Dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.dialog.JunAbstractDialog;

/**
 * JunTextEditorDialog class
 * 
 *  @author    m-asada
 *  @created   2005/05/11 (by Mitsuhiro Asada)
 *  @updated   2008/05/15 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTextEditorDialog.java,v 9.11 2010/03/09 09:17:25 nisinaka Exp $
 */
public class JunTextEditorDialog extends JunAbstractDialog<JunTextEditor> {

	/**
	 * Open JunGaugeDialog to request a value.
	 * 
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static String Request() {
		return Request($String("Input a new <1p>.", null, $String("Text")));
	}

	/**
	 * Open a JunGaugeDialog to request a value.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static String Request(String messageString) {
		return Request(messageString, new JunTextEditor());
	}

	/**
	 * Open a JunGaugeDialog to request a value with the specified gauge model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param aTextEditor jp.co.sra.jun.goodies.texteditor.JunTextEditor
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static String Request(String messageString, JunTextEditor aTextEditor) {
		return Request(messageString, aTextEditor, null, null);
	}

	/**
	 * Open a JunTextEditorDialog to request a value with the specified text properties model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param aTextEditor jp.co.sra.jun.goodies.texteditor.JunTextEditor
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @deprecated since Jun701, use Generics.
	 * @category Utilities
	 */
	public static String Request(String messageString, JunTextEditor aTextEditor, StBlockClosure<Boolean, JunTextEditor, ?, ?> acceptBlock, StBlockClosure<Boolean, JunTextEditor, ?, ?> cancelBlock) {
		JunTextEditorDialog textPropertiesDialog = new JunTextEditorDialog();
		if (aTextEditor != null) {
			textPropertiesDialog.textPropertiesModel_(aTextEditor);
		}
		Dialog aDialog = textPropertiesDialog.allButOpenDialog();
		if (messageString != null) {
			aDialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			textPropertiesDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			textPropertiesDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(aDialog);

		return (textPropertiesDialog.accept().value()) ? textPropertiesDialog.textPropertiesModel().text() : null;
	}

	/**
	 * Open a JunTextEditorDialog to request a value with the specified text properties model.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param aTextEditor jp.co.sra.jun.goodies.texteditor.JunTextEditor
	 * @param acceptBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param cancelBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static String Request(String messageString, JunTextEditor aTextEditor, StBlockClosure1<Boolean, JunTextEditor> acceptBlock, StBlockClosure1<Boolean, JunTextEditor> cancelBlock) {
		JunTextEditorDialog textPropertiesDialog = new JunTextEditorDialog();
		if (aTextEditor != null) {
			textPropertiesDialog.textPropertiesModel_(aTextEditor);
		}
		Dialog aDialog = textPropertiesDialog.allButOpenDialog();
		if (messageString != null) {
			aDialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			textPropertiesDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			textPropertiesDialog.cancelBlock_(cancelBlock);
		}

		_ShowAtMousePoint(aDialog);

		return (textPropertiesDialog.accept().value()) ? textPropertiesDialog.textPropertiesModel().text() : null;
	}

	/**
	 * Open a JunTextEditorDialog to request a value with the specified initial value.
	 * The message string will be shown as the window title.
	 * 
	 * @param messageString java.lang.String
	 * @param value java.lang.String
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static String Request(String messageString, String value) {
		return Request(messageString, new JunTextEditor(value));
	}

	/**
	 * Answer my current text properties model.
	 * 
	 * @return jp.co.sra.jun.goodies.texteditor.JunTextEditor
	 * @category accessing
	 */
	public JunTextEditor textPropertiesModel() {
		if (this.model() == null) {
			this.model_(new JunTextEditor());
		}
		return this.model();
	}

	/**
	 * Set my new text properties model.
	 * 
	 * @param newTextEditor jp.co.sra.jun.goodies.texteditor.JunTextEditor
	 * @category accessing
	 */
	public void textPropertiesModel_(JunTextEditor newTextEditor) {
		this.model_(newTextEditor);
	}

	/**
	 * Answer my default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category view accessing
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunTextEditorDialogViewAwt(this);
		} else {
			return new JunTextEditorDialogViewSwing(this);
		}
	}
}
