package jp.co.sra.jun.goodies.track;

import java.io.IOException;
import java.io.Writer;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockValue;
import jp.co.sra.smalltalk.StBlockValued;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunSimpleSliderModel class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/08/27 (by Hoshi Takanori)
 *  @updated   2004/09/22 (by nisinaka)
 *  @updated   2008/05/11 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun651 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSimpleSliderModel.java,v 9.10 2010/03/09 09:17:26 nisinaka Exp $
 */
public class JunSimpleSliderModel extends JunApplicationModel implements StBlockValued<Double> {

	protected StValueHolder<Double> valueHolder;

	/**
	 * Create a new instance of <code>JunSimpleSliderModel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunSimpleSliderModel() {
		super();
	}

	/**
	 * Answer a BlockValue.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @deprecated since Jun701, use Generics.
	 * @category constructing
	 */
	public StBlockValue<Object> compute_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.compute_(new StBlockClosure1<Object, Double>() {
			public Object value(Double value) {
				return aBlock.value_(value);
			}
		});
	}

	/**
	 * Answer a BlockValue.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @category constructing
	 */
	public <R> StBlockValue<R> compute_(StBlockClosure1<R, Double> aBlock) {
		return new StBlockValue<R>(aBlock, this);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunSimpleSliderViewAwt(this);
		} else {
			return new JunSimpleSliderViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Simple Slider");
	}

	/**
	 * Answer my value as double.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double doubleValue() {
		return this.value();
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" (");
		aWriter.write(String.valueOf(this.value()));
		aWriter.write(')');
	}

	/**
	 * Get the value of valueHolder.
	 * 
	 * @return java.lang.Double
	 * @see jp.co.sra.smalltalk.StValued#value()
	 * @category accessing
	 */
	public Double value() {
		return valueHolder.value();
	}

	/**
	 * Set the value of valueHolder.
	 * 
	 * @param normalizedNumber double
	 * @category accessing
	 */
	public void value_(double normalizedNumber) {
		valueHolder.value_(Math.max(0, Math.min(normalizedNumber, 1)));
		this.changed_($("value"));
	}

	/**
	 * Get the valueHolder with double value.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder&lt;java.lang.Double&gt;
	 * @category accessing
	 */
	public StValueHolder<Double> valueHolder() {
		if (valueHolder == null) {
			valueHolder = new StValueHolder<Double>(0.0d);
		}
		return valueHolder;
	}

	/**
	 * Initialize the JunSimpleSliderModel.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		valueHolder = null;
		this.valueHolder();
	}

}
