package jp.co.sra.jun.goodies.utilities;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import javax.imageio.ImageIO;

import jp.co.sra.smalltalk.StImage;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.curves.Jun2dLine;
import jp.co.sra.jun.geometry.transformations.Jun2dTransformation;
import jp.co.sra.jun.goodies.animation.JunCartoonMovie;
import jp.co.sra.jun.goodies.image.framework.JunImageDisplayModel;
import jp.co.sra.jun.goodies.morph.JunMorphModel;
import jp.co.sra.jun.goodies.tips.JunURL;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunImageUtilityTestExamples class
 * 
 *  @author    m-asada
 *  @created   2006/04/18 (by m-asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun632 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageUtilityTestExamples.java,v 9.11 2010/03/09 09:17:27 nisinaka Exp $
 */
public class JunImageUtilityTestExamples extends JunTestExamples {
	/**
	 * Example: blend two colors.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_BlendColor() {
		Color blendColor = JunImageUtility.Blend_and_weight_and_(Color.white, Color.black, 0.25, 0.75);
		System.out.println(blendColor);

		return true;
	}

	/**
	 * Example: blend two images.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_BlendImage() {
		BufferedImage anImage = JunImageUtility.BlendImage_andImage_weight_and_(JunImageUtility.FromDisplay_(new Rectangle(2, 0, 128, 128)), JunImageUtility.FromDisplay_(new Rectangle(2, 128, 128, 128)), 0.5, 0.5);
		JunImageUtility.Show_(anImage);

		return true;
	}

	/**
	 * Example: read the image from file.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_FromFile() {
		BufferedImage anImage = JunImageUtility.FromFile();
		JunImageUtility.Show_(anImage);
		return true;
	}

	/**
	 * Example: morph image.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_Morph() {
		BufferedImage firstImage = MorphSampleFirstImage();
		BufferedImage secondImage = MorphSampleSecondImage();
		Jun2dLine[] fromLines = MorphSampleFirstLines();
		Jun2dLine[] toLines = MorphSampleSecondLines();
		JunMorphModel morphModel = new JunMorphModel(firstImage, secondImage, fromLines, toLines);
		morphModel.makeMorph();

		return true;
	}

	/**
	 * Answer the morph sample first image.
	 * 
	 * @return java.awt.image.BufferedImage
	 * @category Examples
	 */
	public static BufferedImage MorphSampleFirstImage() {
		Image anImage = null;
		try {
			File aFile = new File("2.jpg");
			String aString = "http://www.sra.co.jp/people/h-asaoka/picture_movie/images/" + aFile.toString();
			JunURL aURL = JunURL.Named_(aString);
			anImage = ImageIO.read(new URL(aURL.toString()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}

		BufferedImage firstImage = new BufferedImage(anImage.getWidth(null), anImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		Graphics aGraphics = null;
		try {
			aGraphics = firstImage.getGraphics();
			aGraphics.drawImage(anImage, 0, 0, null);
		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
			}
		}
		return firstImage;
	}

	/**
	 * Answer the morph sample first lines.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category Examples
	 */
	public static Jun2dLine[] MorphSampleFirstLines() {
		ArrayList<Jun2dLine> aCollection = new ArrayList<Jun2dLine>();
		/*
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 0), new Jun2dPoint(150, 0)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(150, 0), new Jun2dPoint(150, 184)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 0), new Jun2dPoint(0, 184)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 184), new Jun2dPoint(150, 184)));
		*/
		aCollection.add(new Jun2dLine(new Jun2dPoint(67, 67), new Jun2dPoint(67, 67)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(98, 69), new Jun2dPoint(98, 69)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(84, 69), new Jun2dPoint(85, 91)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(43, 69), new Jun2dPoint(77, 118)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(81, 8), new Jun2dPoint(43, 69)));
		return aCollection.toArray(new Jun2dLine[aCollection.size()]);
	}

	/**
	 * Answer the morph sample second image.
	 * 
	 * @return java.awt.image.BufferedImage
	 * @category Examples
	 */
	public static BufferedImage MorphSampleSecondImage() {
		Image anImage = null;
		try {
			File aFile = new File("3.jpg");
			String aString = "http://www.sra.co.jp/people/h-asaoka/picture_movie/images/" + aFile.toString();
			JunURL aURL = JunURL.Named_(aString);
			anImage = ImageIO.read(new URL(aURL.toString()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}

		BufferedImage firstImage = new BufferedImage(anImage.getWidth(null), anImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		Graphics aGraphics = null;
		try {
			aGraphics = firstImage.getGraphics();
			aGraphics.drawImage(anImage, 0, 0, null);
		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
			}
		}
		return firstImage;
	}

	/**
	 * Answer the morph sample second lines.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.Jun2dLine[]
	 * @category Examples
	 */
	public static Jun2dLine[] MorphSampleSecondLines() {
		ArrayList<Jun2dLine> aCollection = new ArrayList<Jun2dLine>();
		/*
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 0), new Jun2dPoint(150, 0)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(150, 0), new Jun2dPoint(150, 184)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 0), new Jun2dPoint(0, 184)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(0, 184), new Jun2dPoint(150, 184)));
		*/
		aCollection.add(new Jun2dLine(new Jun2dPoint(74, 67), new Jun2dPoint(74, 67)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(101, 68), new Jun2dPoint(101, 68)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(92, 69), new Jun2dPoint(94, 89)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(47, 73), new Jun2dPoint(88, 116)));
		aCollection.add(new Jun2dLine(new Jun2dPoint(83, 12), new Jun2dPoint(47, 73)));
		return aCollection.toArray(new Jun2dLine[aCollection.size()]);
	}

	/**
	 * Example: open new window with the selected image file at the file dialog.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_RequestImage() {
		StImage anImage = JunImageUtility.RequestImage();
		if (anImage != null) {
			JunImageUtility.Show_(anImage);
		}
		return true;
	}

	/**
	 * Example: save to an image from display area. 
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_RequestNewImage() {
		try {
			JunImageUtility.RequestNewImageFileThenWriteImage_(StImage._OfArea(new Rectangle(0, 0, 300, 300)));
		} catch (IOException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * Example: open new window with the image from display area.
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_Show() {
		JunImageUtility.Show_at_(StImage._OfArea(new Rectangle(0, 0, 300, 300)), new Point(0, 0));
		return true;
	}

	/**
	 * Example: warp image. 
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_WarpImage1() {
		BufferedImage anImage = JunImageUtility.FromDisplay_(new Rectangle(2, 0, 100, 100));
		ArrayList<JunImageDisplayModel> aCollection = new ArrayList<JunImageDisplayModel>();
		aCollection.add(JunImageUtility.Show_at_(anImage, new Point(200, 300)));
		anImage = JunImageUtility.WarpImage_toImage_fromLines_toLines_(anImage, new BufferedImage(anImage.getWidth() * 2, anImage.getHeight() * 2, BufferedImage.TYPE_INT_ARGB), new Jun2dLine[] { new Jun2dPoint(50, 50).to_(new Jun2dPoint(100, 0)) },
				new Jun2dLine[] { new Jun2dPoint(100, 100).to_(new Jun2dPoint(200, 100)) });
		aCollection.add(JunImageUtility.Show_at_(anImage, new Point(200 + 112, 300)));
		for (JunImageDisplayModel each : aCollection) {
			each.closeTogether_(aCollection);
		}

		return true;
	}

	/**
	 * Example: warp image. 
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_WarpImage2() {
		BufferedImage sourceImage = JunImageUtility.FromDisplay_(new Rectangle(2, 0, 128, 128));
		BufferedImage destinationImage = new BufferedImage(sourceImage.getWidth(), sourceImage.getHeight(), BufferedImage.TYPE_INT_ARGB);
		JunImageDisplayModel displayModel = JunImageUtility.Show_(sourceImage);
		Jun2dLine aLine = new Jun2dPoint(0, 0).to_(new Jun2dPoint(sourceImage.getWidth(), sourceImage.getHeight()));
		for (int degrees = 0; degrees <= 360; degrees = degrees + 5) {
			BufferedImage anImage = JunImageUtility.WarpImage_toImage_fromLines_toLines_(sourceImage, destinationImage, new Jun2dLine[] { aLine }, new Jun2dLine[] { aLine.transform_(Jun2dTransformation.Rotate_around_(JunAngle.FromDeg_(degrees), aLine
					.atT_(0.5))) });
			displayModel.image_(anImage);
			displayModel.changed_($("redisplay"));
		}

		return true;
	}

	/**
	 * Example: warp point. 
	 *  
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_WarpPoint() {
		Jun2dPoint aPoint = JunImageUtility.WarpPoint_fromLines_toLines_powerValue_(new Jun2dPoint(30, 20), new Jun2dLine[] { new Jun2dPoint(50, 0).to_(new Jun2dPoint(50, 100)) }, new Jun2dLine[] { new Jun2dPoint(0, 50).to_(new Jun2dPoint(100, 50)) }, 0);
		System.out.println(aPoint);

		return true;
	}

	/**
	 * Example_on_web_Warping: an example for warping on the web page.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_on_web_Warping() {
		BufferedImage anImage = JunImageUtility.FromDisplay_(new Rectangle(2, 0, 150, 184));
		Jun2dLine aLine = Jun2dPoint.Zero().to_(new Jun2dPoint(anImage.getWidth(), anImage.getHeight()));
		Jun2dPoint aCenter = aLine.atT_(0.5);
		int step = 10;
		int size = 360 / step + 1;
		int tick = 15000 / size;
		JunCartoonMovie aViewer = new JunCartoonMovie();
		for (int degrees = 0; degrees <= 360; degrees += step) {
			JunAngle anAngle = JunAngle.FromDeg_(degrees);
			Jun2dTransformation aTransformation = Jun2dTransformation.Rotate_around_(anAngle, aCenter);
			BufferedImage warpedImage = JunImageUtility.WarpImage_fromLines_toLines_(anImage, new Jun2dLine[] { aLine }, new Jun2dLine[] { aLine.transform_(aTransformation) });
			aViewer.addImage_keepTime_(new StImage(warpedImage), tick);
			System.out.println(degrees);
		}

		aViewer.openLightWeightWindowSpecFitFlag_markerFlag_(true, false);
		aViewer.loop();
		aViewer.start();

		return true;
	}

	/**
	 * Example_on_web_Blending: an example for blending on the web page.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_on_web_Blending() {
		BufferedImage anImage = JunImageUtility.FromDisplay_(new Rectangle(2, 0, 150, 184));
		Jun2dLine aLine = Jun2dPoint.Zero().to_(new Jun2dPoint(anImage.getWidth(), anImage.getHeight()));
		Jun2dPoint aCenter = aLine.atT_(0.5);
		int step = 10;
		int size = 360 / step + 1;
		int tick = 15000 / size;
		JunCartoonMovie aViewer = new JunCartoonMovie();
		for (int degrees = 0; degrees <= 360; degrees += step) {
			JunAngle anAngle = JunAngle.FromDeg_(degrees);
			Jun2dTransformation aTransformation = Jun2dTransformation.Rotate_around_(anAngle, aCenter);
			BufferedImage warpedImage = JunImageUtility.WarpImage_fromLines_toLines_(anImage, new Jun2dLine[] { aLine }, new Jun2dLine[] { aLine.transform_(aTransformation) });
			BufferedImage blendedImage = JunImageUtility.BlendImage_andImage_weight_(anImage, warpedImage, 0.5);
			aViewer.addImage_keepTime_(new StImage(blendedImage), tick);
			System.out.println(degrees);
		}

		aViewer.openLightWeightWindowSpecFitFlag_markerFlag_(true, false);
		aViewer.loop();
		aViewer.start();

		return true;
	}

	/**
	 * Example_on_web_Morphing: an example for morphing on the web page.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example_on_web_Morphing() {
		BufferedImage firstImage = MorphSampleFirstImage();
		BufferedImage secondImage = MorphSampleSecondImage();
		Jun2dLine[] firstLines = MorphSampleFirstLines();
		Jun2dLine[] secondLines = MorphSampleSecondLines();

		int step = 5;
		int size = 100 / step + 1;
		int tick = 15000 / size;
		JunCartoonMovie aViewer = new JunCartoonMovie();
		for (int n = 0; n <= 100; n += step) {
			double w = n / 100.0;
			BufferedImage morphedImage = JunImageUtility.MorphImage_andImage_fromLines_toLines_weight_and_(firstImage, secondImage, firstLines, secondLines, 1 - w, w);
			aViewer.addImage_keepTime_(new StImage(morphedImage), tick);
			System.out.println(w);
		}

		aViewer.openLightWeightWindowSpecFitFlag_markerFlag_(true, false);
		aViewer.loop();
		aViewer.start();

		return true;
	}

	/**
	 * Execute all test examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunImageUtilityTestExamples();
	}
}
