package jp.co.sra.jun.goodies.wheels;

import java.awt.Image;
import java.io.IOException;
import java.io.Writer;

import javax.imageio.ImageIO;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockValue;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunThumbWheel class
 * 
 *  @author    He Weijie
 *  @created   1998/08/13 (by He Weijie)
 *  @updated   1999/12/17 (by MATSUDA Ryouichi)
 *  @updated   2004/09/22 (by nisinaka)
 *  @updated   2007/03/13 (by Mitsuhiro Asada)
 *  @updated   2008/05/15 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun651 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunThumbWheel.java,v 9.13 2010/03/09 09:17:28 nisinaka Exp $
 */
public class JunThumbWheel extends JunApplicationModel {

	/** A boolean value as a button model. */
	protected StValueHolder<Double> angularValue;
	protected boolean verticalFlag;

	/**
	 * Create a new instance of <code>JunThumbWheel</code> with horizontal.
	 * 
	 * @return jp.co.sra.jun.goodies.wheels.JunThumbWheel
	 * @category Instance creation
	 */
	public static JunThumbWheel Horizontal() {
		JunThumbWheel aModel = new JunThumbWheel();
		aModel.verticalFlag_(false);
		return aModel;
	}

	/**
	 * Create a new instance of <code>JunThumbWheel</code> with vertical.
	 * 
	 * @return jp.co.sra.jun.goodies.wheels.JunThumbWheel
	 * @category Instance creation
	 */
	public static JunThumbWheel Vertical() {
		JunThumbWheel aModel = new JunThumbWheel();
		aModel.verticalFlag_(true);
		return aModel;
	}

	/**
	 * Create a new instance of <code>JunThumbWheel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunThumbWheel() {
		super();
	}

	/**
	 * Create a new instance of <code>JunThumbWheel</code> and initialize it.
	 * 
	 * @param isVertical boolean
	 * @category Instance creation
	 */
	public JunThumbWheel(boolean isVertical) {
		super();
		this.verticalFlag_(isVertical);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		angularValue = new StValueHolder<Double>(0.0);
		verticalFlag = false;
		this.compute_(new StBlockClosure0<Double>() {
			public Double value() {
				JunThumbWheel self = JunThumbWheel.this;
				self.changed_with_($("value"), new Double(self.angularValue()));
				return null;
			}
		});
	}

	/**
	 * Just return the current angularValue.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double angularValue() {
		return angularValue.value();
	}

	/**
	 * Set the new angularValue.
	 * 
	 * @param aValue double
	 * @category accessing
	 */
	public void angularValue_(double aValue) {
		angularValue.value_(aValue);
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @category accessing
	 */
	public <R> StBlockValue<R> compute_(StBlockClosure0<R> aBlock) {
		return new StBlockValue<R>(aBlock, angularValue);
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @category accessing
	 */
	public <R> StBlockValue<R> compute_(StBlockClosure1<R, Double> aBlock) {
		return new StBlockValue<R>(aBlock, angularValue);
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public StBlockValue<Object> compute_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.compute_(new StBlockClosure1<Object, Double>() {
			public Object value(Double value) {
				switch (aBlock.numArgs()) {
					case 0:
						return aBlock.value();
					case 1:
						return aBlock.value_(value);
					default:
						return null;
				}

			}
		});
	}

	/**
	 * Answer the receiver's vertical flag.
	 * 
	 * @return boolean
	 * @category accessing
	 */
	public boolean verticalFlag() {
		return verticalFlag;
	}

	/**
	 * Set the receiver's vertical flag.
	 * 
	 * @param aBoolean boolean
	 * @category accessing
	 */
	public void verticalFlag_(boolean aBoolean) {
		verticalFlag = aBoolean;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunThumbWheelViewAwt(this);
		} else {
			return new JunThumbWheelViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Thumb Wheel");
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		aWriter.write(String.valueOf(this.angularValue()));
	}

	/**
	 * Read an image from a resource specified with the path string.
	 * 
	 * @param pathString java.lang.String
	 * @return java.awt.Image
	 * @category Private
	 */
	protected static Image _GetImageFromResource(String pathString) {
		Image anImage = null;
		try {
			anImage = ImageIO.read(JunThumbWheel.class.getResource(pathString));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return anImage;
	}

}
