package jp.co.sra.jun.graphics.list;

import java.io.File;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunFileListTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2004/01/13 (by nisinaka)
 *  @updated   2006/11/22 (by m-asada)
 *  @updated   2008/05/01 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun690 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileListTestExamples.java,v 9.11 2010/03/09 09:17:29 nisinaka Exp $
 */
public class JunFileListTestExamples extends JunTestExamples {
	/**
	 * Example1: Show the root file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example1() {
		JunFileList fileList = JunFileList.Root();
		fileList.directoryItemBlock_(new StBlockClosure() {
			public Object value_(Object filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example1: Show the root file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1_g() {
		JunFileList fileList = JunFileList.Root();
		fileList.directoryItemBlock_(new StBlockClosure1<JunFileList, File>() {
			public JunFileList value(File filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example2: Show the current file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example2() {
		JunFileList fileList = JunFileList.Current();
		fileList.directoryItemBlock_(new StBlockClosure() {
			public Object value_(Object filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.sortKey_($("extensionUp"));
		fileList.distinctFolder_(false);
		fileList.open();
		return true;
	}

	/**
	 * Example2: Show the current file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2_g() {
		JunFileList fileList = JunFileList.Current();
		fileList.directoryItemBlock_(new StBlockClosure1<JunFileList, File>() {
			public JunFileList value(File filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.sortKey_($("extensionUp"));
		fileList.distinctFolder_(false);
		fileList.open();
		return true;
	}

	/**
	 * Example3: Show the current file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example3() {
		File aFile = (new File(".")).listFiles()[0];
		JunFileList fileList = new JunFileList(aFile);
		fileList.fileItemBlock_(new StBlockClosure() {
			public Object value_(Object filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example3: Show the current file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3_g() {
		File aFile = (new File(".")).listFiles()[0];
		JunFileList fileList = new JunFileList(aFile);
		fileList.fileItemBlock_(new StBlockClosure1<JunFileList, File>() {
			public JunFileList value(File filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example4: Show the null file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example4() {
		JunFileList fileList = JunFileList.Null();
		fileList.nullItemBlock_(new StBlockClosure() {
			public Object value_(Object filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example4: Show the null file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4_g() {
		JunFileList fileList = JunFileList.Null();
		fileList.nullItemBlock_(new StBlockClosure1<JunFileList, File>() {
			public JunFileList value(File filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example5: Show the root file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example5() {
		JunFileList fileList = JunFileList.Root();
		fileList.showContents_(false);
		fileList.directoryItemBlock_(new StBlockClosure() {
			public Object value_(Object filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Example5: Show the root file list.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example5_g() {
		JunFileList fileList = JunFileList.Root();
		fileList.showContents_(false);
		fileList.directoryItemBlock_(new StBlockClosure1<JunFileList, File>() {
			public JunFileList value(File filename) {
				System.out.println(filename);
				return null;
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Test2: Select block.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Test_selectBlock() {
		JunFileList fileList = JunFileList.Current();
		fileList.selectBlock_(new StBlockClosure1<Boolean, File>() {
			public Boolean value(File aFile) {
				return Boolean.valueOf(aFile.isFile());
			}
		});
		fileList.open();
		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunFileListTestExamples();
	}
}
