package jp.co.sra.jun.graphics.navigator;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import javax.swing.KeyStroke;

import jp.co.sra.smalltalk.StBlockClosure0;

import jp.co.sra.jun.graphics.abstracts.JunAbstractVisual;
import jp.co.sra.jun.graphics.abstracts.JunAbstractItem.LookPreferences;

/**
 * JunEmbeddedCenterFileList class
 * 
 *  @author    nisinaka
 *  @created   2004/01/22 (by nisinaka)
 *  @updated   2006/11/29 (by m-asada)
 *  @updated   2008/04/17 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun638 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunEmbeddedCenterFileList.java,v 9.10 2010/03/09 09:17:30 nisinaka Exp $
 */
public class JunEmbeddedCenterFileList extends JunEmbeddedFileList {
	/**
	 * Answer my current look preferences.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.abstracts.JunAbstractItem.LookPreferences
	 * @see jp.co.sra.jun.graphics.list.JunFileList#lookPreferences()
	 * @category visual properties
	 */
	public LookPreferences lookPreferences() {
		if (lookPreferences == null) {
			lookPreferences = new LookPreferences();
			lookPreferences.setForegroundColor(JunAbstractVisual.ForegroundColor());
			lookPreferences.setBackgroundColor(JunAbstractVisual.BackgroundColor());
			lookPreferences.setSelectionForegroundColor(Color.white);
			lookPreferences.setSelectionBackgroundColor(Color.getHSBColor(0.62f, 0.91f, 0.42f));
		}
		return lookPreferences;
	}

	/**
	 * Create the key stroke dispatch table.
	 * 
	 * @return java.util.Map&lt;javax.swing.KeyStroke, jp.co.sra.smalltalk.StBlockClosure&gt;
	 * @see jp.co.sra.jun.graphics.navigator.JunEmbeddedFileList#createKeyStrokeDispatchTable()
	 * @category keyboard
	 */
	protected Map<KeyStroke, StBlockClosure0<Void>> createKeyStrokeDispatchTable() {
		HashMap<KeyStroke, StBlockClosure0<Void>> aMap = new HashMap<KeyStroke, StBlockClosure0<Void>>();
		aMap.put(KeyStroke.getKeyStroke("ENTER"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressENTER();
				return null;
			}
		});
		aMap.put(KeyStroke.getKeyStroke("SPACE"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressSPACE();
				return null;
			}
		});
		aMap.put(KeyStroke.getKeyStroke("LEFT"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressLEFT();
				return null;
			}
		});
		aMap.put(KeyStroke.getKeyStroke("RIGHT"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressRIGHT();
				return null;
			}
		});
		aMap.put(KeyStroke.getKeyStroke("UP"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressUP();
				return null;
			}
		});
		aMap.put(KeyStroke.getKeyStroke("DOWN"), new StBlockClosure0<Void>() {
			public Void value() {
				keyStrokePressDOWN();
				return null;
			}
		});
		return aMap;
	}

	/**
	 * Handle the key stroke ENTER.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressENTER()
	 * @category keyboard
	 */
	protected boolean keyStrokePressENTER() {
		super.keyStrokePressENTER();

		JunFileNavigator navigator = this.fileNavigator();
		if (navigator != null) {
			JunEmbeddedRightFileList rightList = navigator.rightList();
			if (rightList.isFile()) {
				navigator.rightFileItemPressed_(rightList.currentFile());
			}
		}

		return true;
	}

	/**
	 * Handle the key stroke SPACE.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressSPACE()
	 * @category keyboard
	 */
	protected boolean keyStrokePressSPACE() {
		// Do nothing
		return true;
	}

	/**
	 * Handle the key stroke LEFT.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressLEFT()
	 * @category keyboard
	 */
	protected boolean keyStrokePressLEFT() {
		if (super.keyStrokePressLEFT()) {
			JunFileNavigator navigator = this.fileNavigator();
			if (navigator != null && navigator.centerList().isNull() == false) {
				if (navigator.leftList().isNull() && navigator.centerList().isRoot()) {
					navigator.leftNullItemPressed_(null);
					navigator.fixScrollingOffset();
				} else {
					navigator.leftDirectoryItemPressed_(navigator.leftList().selectedFile());
					navigator.fixScrollingOffset();
				}
			}
		}

		return true;
	}

	/**
	 * Handle the key stroke RIGHT.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressRIGHT()
	 * @category keyboard
	 */
	protected boolean keyStrokePressRIGHT() {
		if (super.keyStrokePressRIGHT()) {
			JunFileNavigator navigator = this.fileNavigator();
			if (navigator != null) {
				JunEmbeddedRightFileList rightList = navigator.rightList();
				if (rightList.selectedFile() == null) {
					if (rightList.isEmpty() == false) {
						if (rightList.isFile()) {
							navigator.rightFileItemPressed_(rightList.currentFile());
						} else {
							navigator.rightDirectoryItemPressed_(rightList.fileAt_(0));
							navigator.fixScrollingOffset();
						}
					}
				} else {
					navigator.rightDirectoryItemPressed_(rightList.selectedFile());
					navigator.fixScrollingOffset();
				}
			}
		}

		return true;
	}

	/**
	 * Handle the key stroke UP.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressUP()
	 * @category keyboard
	 */
	protected boolean keyStrokePressUP() {
		if (super.keyStrokePressUP()) {
			JunFileNavigator navigator = this.fileNavigator();
			if (navigator != null && navigator.centerList().isNull() == false) {
				JunEmbeddedCenterFileList centerList = navigator.centerList();
				navigator.centerDirectoryItemPressed_(centerList.selectedFile());
			}
		}

		return true;
	}

	/**
	 * Handle the key stroke DOWN.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.graphics.list.JunFileList#keyStrokePressDOWN()
	 * @category keyboard
	 */
	protected boolean keyStrokePressDOWN() {
		if (super.keyStrokePressDOWN()) {
			JunFileNavigator navigator = this.fileNavigator();
			if (navigator != null && navigator.centerList().isNull() == false) {
				JunEmbeddedCenterFileList centerList = navigator.centerList();
				navigator.centerDirectoryItemPressed_(centerList.selectedFile());
			}
		}

		return true;
	}
}
