package jp.co.sra.jun.graphics.navigator;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.goodies.button.JunButtonViewSwing;
import jp.co.sra.jun.graphics.framework.JunGraphicViewSwing;
import jp.co.sra.jun.graphics.list.JunSequenceListViewSwing;

/**
 * JunFileNavigatorViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2004/01/23 (by nisinaka)
 *  @updated   2006/11/29 (by m-asada)
 *  @updated   2008/05/07 (by m-asada)
 *  @updated   2008/10/16 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun706 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileNavigatorViewSwing.java,v 9.13 2010/03/09 09:17:30 nisinaka Exp $
 */
public class JunFileNavigatorViewSwing extends JunGraphicViewSwing implements JunFileNavigatorView {
	protected JunSequenceListViewSwing centerView;
	protected JunSequenceListViewSwing leftView;
	protected JunSequenceListViewSwing rightView;
	protected JunButtonViewSwing homeButtonView;
	protected JLabel currentFileName;
	protected JLabel statusLabel;
	protected ParentSplitPane parentSplitView;
	protected ChildSplitPane childSplitView;
	protected transient double parentSplitPosition;
	protected transient double childSplitPosition;

	/**
	 * Create a new instance of <code>JunFileNavigatorViewSwing</code> and initialize it.
	 * 
	 * @param aModel jp.co.sra.jun.graphics.navigator.JunFileNavigator
	 * @category Instance creation
	 */
	public JunFileNavigatorViewSwing(JunFileNavigator aModel) {
		super(aModel);
	}

	/**
	 * Answer the receiver's model as <code>JunFileNavigator</code>.
	 * 
	 * @return jp.co.sra.jun.graphics.navigator.JunFileNavigator
	 * @see jp.co.sra.jun.graphics.navigator.JunFileNavigatorView#getFileNavigator()
	 * @category accessing 
	 */
	public JunFileNavigator getFileNavigator() {
		return (JunFileNavigator) this.model();
	}

	/**
	 * Initialize the receiver. 
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release 
	 */
	@Override
	protected void initialize() {
		super.initialize();

		parentSplitView = null;
		childSplitView = null;
		parentSplitPosition = 0.333333333;
		childSplitPosition = 0.5;
	}

	/**
	 * Build the component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.homeButtonView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 25, 0f, 25));
		this.add(this.currentFileName(), WindowSpecLayout.Position(0f, 26, 0f, 1, 1f, -1, 0f, 25));
		this.add(this.parentSplitView(), WindowSpecLayout.Position(0f, 1, 0f, 26, 1f, -1, 1f, -21));
		this.add(this.statusLabel(), WindowSpecLayout.Position(0f, 1, 1f, -20, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(500, 300));
	}

	/**
	 * Answer the center view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing centerView() {
		if (centerView == null) {
			centerView = new JunEmbeddedFileListViewSwing(this.getFileNavigator().centerList());
		}
		return centerView;
	}

	/**
	 * Answer the left view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing leftView() {
		if (leftView == null) {
			leftView = new JunEmbeddedFileListViewSwing(this.getFileNavigator().leftList());
			this.setListViewVisible(leftView, !this.getFileNavigator().isLeftEnd());
		}
		return leftView;
	}

	/**
	 * Answer the right view.
	 * 
	 * @return jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @category user interface
	 */
	protected JunSequenceListViewSwing rightView() {
		if (rightView == null) {
			rightView = new JunSequenceListViewSwing(this.getFileNavigator().rightList());
		}
		return rightView;
	}

	/**
	 * Answer the home button view.
	 * 
	 * @return jp.co.sra.jun.goodies.button.JunButtonViewSwing
	 * @category user interface
	 */
	protected JunButtonViewSwing homeButtonView() {
		if (homeButtonView == null) {
			homeButtonView = new JunButtonViewSwing(this.getFileNavigator().homeButton());
		}
		return homeButtonView;
	}

	/**
	 * Answer the label for the current file name.
	 * 
	 * @return javax.swing.JLabel
	 * @category user interface
	 */
	protected JLabel currentFileName() {
		if (currentFileName == null) {
			currentFileName = new JLabel(this.getFileNavigator().currentFileAsString());
			currentFileName.setFocusable(false);
			currentFileName.setBorder(new EmptyBorder(0, 10, 0, 0));
			if (this.getFileNavigator().currentFileHolder().value() != null) {
				currentFileName.setText(this.getFileNavigator().currentFileAsString());
			}

			this.getFileNavigator().currentFileHolder().compute_(new StBlockClosure1<File, File>() {
				public File value(File anObject) {
					currentFileName.setText(JunFileNavigatorViewSwing.this.getFileNavigator().currentFileAsString());
					return null;
				}
			});
		}
		return currentFileName;
	}

	/**
	 * Answer the label for the status.
	 * 
	 * @return javax.swing.JLabel
	 * @category user interface
	 */
	protected JLabel statusLabel() {
		if (statusLabel == null) {
			statusLabel = new JLabel(this.getFileNavigator().currentStatus());
			statusLabel.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return statusLabel;
	}

	/**
	 * Answer the receiver's parent split pane view.
	 *
	 * @return jp.co.sra.jun.graphics.navigator.JunFileRequesterViewSwing.ParentSplitPane
	 * @category user interface
	 */
	protected ParentSplitPane parentSplitView() {
		if (parentSplitView == null) {
			parentSplitView = new ParentSplitPane(JSplitPane.HORIZONTAL_SPLIT, true);
			parentSplitView.setOneTouchExpandable(false);
			parentSplitView.setResizeWeight(0.0d);
			parentSplitView.setTopComponent(this.leftView());
			parentSplitView.setBottomComponent(this.childSplitView());
			parentSplitView.setSuperDividerLocation(161);
			parentSplitView.setBorder(null);
			parentSplitPosition = 0.33333333333;

			if (this.leftView().isVisible() == false) {
				parentSplitView.setTopComponent(new JPanel());
			}
		}
		return parentSplitView;
	}

	/**
	 * Answer the receiver's parent split pane view.
	 *
	 * @return jp.co.sra.jun.graphics.navigator.JunFileRequesterViewSwing.ChildSplitPane
	 * @category user interface
	 */
	protected ChildSplitPane childSplitView() {
		if (childSplitView == null) {
			childSplitView = new ChildSplitPane(JSplitPane.HORIZONTAL_SPLIT, true);
			childSplitView.setOneTouchExpandable(false);
			childSplitView.setResizeWeight(1.0d);
			childSplitView.setTopComponent(this.centerView());
			childSplitView.setBottomComponent(this.rightView());
			childSplitView.setSuperDividerLocation(166);
			childSplitView.setBorder(null);
			childSplitPosition = 0.5;

			if (this.rightView().isVisible() == false) {
				childSplitView.setBottomComponent(new JPanel());
			}
		}
		return childSplitView;
	}

	/**
	 * To fix the scrolling offset when opened.
	 * 
	 * @see java.awt.Component#addNotify()
	 * @category actions
	 */
	@Override
	public void addNotify() {
		super.addNotify();

		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				JunFileNavigatorViewSwing.this.getFileNavigator().fixScrollingOffset();
			}
		});
	}

	/**
	 * Moves and resizes this component.
	 * 
	 * @param x the new <i>x</i>-coordinate of this component
	 * @param y the new <i>y</i>-coordinate of this component
	 * @param width the new <code>width</code> of this component
	 * @param height the new <code>height</code> of this component
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 * @category bounds accessing
	 */
	@Override
	public void setBounds(int x, int y, int width, int height) {
		if (this.getWidth() == width && this.getHeight() == height) {
			return;
		}

		ParentSplitPane parentView = this.parentSplitView();
		ChildSplitPane childView = this.childSplitView();
		if (width == 0 && height == 0) {
			super.setBounds(x, y, width, height);
			return;
		}

		double oldParentSplitPosition = this.getParentSplitPosition();
		double oldChildSplitPosition = this.getChildSplitPosition();
		super.setBounds(x, y, width, height);
		this.validate();
		parentView.setSuperDividerLocation((int) Math.round(oldParentSplitPosition * (parentView.getMaximumDividerLocation() - parentView.getMinimumDividerLocation()) + parentView.getMinimumDividerLocation()));
		this.validate();
		childView.setSuperDividerLocation((int) Math.round(oldChildSplitPosition * (childView.getMaximumDividerLocation() - childView.getMinimumDividerLocation()) + childView.getMinimumDividerLocation()));
		this.setParentSplitPosition(oldParentSplitPosition);
		this.setChildSplitPosition(oldChildSplitPosition);
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see jp.co.sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	@Override
	public void _setupKeyboard(final Window aWindow) {
		KeyListener keyListener = this.centerView().getSequenceList()._keyListener();
		if (keyListener == null) {
			return;
		}

		this.centerView().addKeyListener(keyListener);

		MouseListener mouseListener = new MouseAdapter() {
			public void mouseEntered(MouseEvent e) {
				if (aWindow.isActive() && aWindow.getFocusOwner() != JunFileNavigatorViewSwing.this.centerView()) {
					JunFileNavigatorViewSwing.this.centerView().requestFocus();
				}
			}
		};
		this.centerView().canvas().addMouseListener(mouseListener);
		this.leftView().canvas().addMouseListener(mouseListener);
		this.rightView().canvas().addMouseListener(mouseListener);

		aWindow.addWindowListener(new WindowAdapter() {
			public void windowActivated(WindowEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						Component focusOwner = aWindow.getFocusOwner();
						if (focusOwner == null || focusOwner != JunFileNavigatorViewSwing.this.centerView()) {
							JunFileNavigatorViewSwing.this.centerView().requestFocus();
						}
					}
				});
			}
		});
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt jp.co.sra.smalltalk.DevendentEvent
	 * @see jp.co.sra.smalltalk.DependentListener#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	@Override
	public void update_(DependentEvent evt) {
		StSymbol aspect = evt.getAspect();
		if (aspect == $("current file")) {
			this.statusLabel().setText(this.getFileNavigator().currentStatus());
			this.setListViewVisible(this.leftView(), !this.getFileNavigator().isLeftEnd());
			this.setListViewVisible(this.rightView(), !this.getFileNavigator().isCurrentFileInaccessibleDirectory());
		} else {
			super.update_(evt);
		}
	}

	/**
	 * Set the list view visibility.
	 * 
	 * @param listView jp.co.sra.jun.graphics.graphics.list.JunSequenceListViewSwing
	 * @param aFlag boolean
	 * @category private
	 */
	protected void setListViewVisible(JunSequenceListViewSwing listView, boolean aFlag) {
		if (listView.isVisible() != aFlag) {
			listView.setVisible(aFlag);

			if (listView.getSequenceList() instanceof JunEmbeddedLeftFileList) {
				int location = this.parentSplitView().getDividerLocation();
				if (aFlag) {
					this.parentSplitView().setTopComponent(listView);
					this.getFileNavigator().leftList().addDependent_(listView);
				} else {
					this.parentSplitView().setTopComponent(new JPanel());
				}
				this.parentSplitView().validate();
				this.parentSplitView().setDividerLocation(location);
			} else if (listView.getSequenceList() instanceof JunEmbeddedRightFileList) {
				int location = this.childSplitView().getDividerLocation();
				if (aFlag) {
					this.childSplitView().setBottomComponent(listView);
					this.getFileNavigator().rightList().addDependent_(listView);
				} else {
					this.childSplitView().setBottomComponent(new JPanel());
				}
				this.parentSplitView().validate();
				this.childSplitView().setDividerLocation(location);
			}
		}
	}

	/**
	 * Answer the receiver's parent split position.
	 * 
	 * @return double
	 * @category private
	 */
	protected double getParentSplitPosition() {
		return parentSplitPosition;
	}

	/**
	 * Set the receiver's parent split position.
	 * 
	 * @param aNumber double
	 * @category private
	 */
	protected void setParentSplitPosition(double aNumber) {
		parentSplitPosition = Math.min(Math.max(0.0, aNumber), 1.0);
	}

	/**
	 * Answer the receiver's child split position.
	 * 
	 * @return double
	 * @category private
	 */
	protected double getChildSplitPosition() {
		return childSplitPosition;
	}

	/**
	 * Set the receiver's child split position.
	 * 
	 * @param aNumber childSplitPosition
	 * @category private
	 */
	protected void setChildSplitPosition(double aNumber) {
		childSplitPosition = Math.min(Math.max(0.0, aNumber), 1.0);
	}

	/**
	 * ParentSplitPane class
	 * 
	 *  @author    Mitsuhiro Asada
	 *  @created   2008/10/16 (by m-asada)
	 *  @updated   N/A
	 */
	protected class ParentSplitPane extends JSplitPane {
		/**
		 * Create a new instance of <code>ParentSplitPane</code> and initialize it.
		 * 
		 * @param newOrientation  <code>JSplitPane.HORIZONTAL_SPLIT</code> or <code>JSplitPane.VERTICAL_SPLIT</code>
		 * @param newContinuousLayout  a boolean, <code>true</code> for the components to redraw continuously as the divider changes position, <code>false</code> to wait until the divider position stops changing to redraw
		 * @exception IllegalArgumentException if <code>orientation</code> is not one of HORIZONTAL_SPLIT or VERTICAL_SPLIT
		 * @category Instance creation
		 */
		public ParentSplitPane(int newOrientation, boolean newContinuousLayout) {
			super(newOrientation, newContinuousLayout);
		}

		/**
		 * Sets the location of the divider.
		 *
		 * @param location an int specifying a UI-specific value (typically a pixel count)
		 * @see javax.swing.JSplitPane#setDividerLocation(int)
		 * @category accessing
		 */
		public void setDividerLocation(int location) {
			if (this.getDividerLocation() == location) {
				return;
			}
			JunFileNavigatorViewSwing.this.setParentSplitPosition(Math.min(Math.max(0.0, (location - this.getMinimumDividerLocation()) / (double) (this.getMaximumDividerLocation() - this.getMinimumDividerLocation())), 1.0));
			this.setSuperDividerLocation(location);
		}

		/**
		 * Sets the location of the divider.
		 *
		 * @param location an int specifying a UI-specific value (typically a pixel count)
		 * @see javax.swing.JSplitPane#setDividerLocation(int)
		 * @category accessing
		 */
		public void setSuperDividerLocation(int location) {
			super.setDividerLocation(location);
		}
	}

	/**
	 * ChildSplitPane class
	 * 
	 *  @author    Mitsuhiro Asada
	 *  @created   2008/10/16 (by m-asada)
	 *  @updated   N/A
	 */
	protected class ChildSplitPane extends JSplitPane {
		/**
		 * Create a new instance of <code>ChildSplitPane</code> and initialize it.
		 * 
		 * @param newOrientation  <code>JSplitPane.HORIZONTAL_SPLIT</code> or <code>JSplitPane.VERTICAL_SPLIT</code>
		 * @param newContinuousLayout  a boolean, <code>true</code> for the components to redraw continuously as the divider changes position, <code>false</code> to wait until the divider position stops changing to redraw
		 * @exception IllegalArgumentException if <code>orientation</code> is not one of HORIZONTAL_SPLIT or VERTICAL_SPLIT
		 * @category Instance creation
		 */
		public ChildSplitPane(int newOrientation, boolean newContinuousLayout) {
			super(newOrientation, newContinuousLayout);
		}

		/**
		 * Sets the location of the divider.
		 *
		 * @param location an int specifying a UI-specific value (typically a pixel count)
		 * @see javax.swing.JSplitPane#setDividerLocation(int)
		 * @category accessing
		 */
		public void setDividerLocation(int location) {
			if (this.getDividerLocation() == location) {
				return;
			}
			JunFileNavigatorViewSwing.this.setChildSplitPosition(Math.min(Math.max(0.0, (location - this.getMinimumDividerLocation()) / (double) (this.getMaximumDividerLocation() - this.getMinimumDividerLocation())), 1.0));
			this.setSuperDividerLocation(location);
		}

		/**
		 * Sets the location of the divider.
		 *
		 * @param location an int specifying a UI-specific value (typically a pixel count)
		 * @see javax.swing.JSplitPane#setDividerLocation(int)
		 * @category accessing
		 */
		public void setSuperDividerLocation(int location) {
			super.setDividerLocation(location);
		}
	}
}
