package jp.co.sra.jun.graphics.navigator;

import java.awt.Dimension;
import java.awt.Window;

import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.dialog.JunAbstractDialogViewAwt;

/**
 * JunFileRequesterDialogViewAwt class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2006/11/30 (by m-asada)
 *  @updated   2008/05/14 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun638 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileRequesterDialogViewAwt.java,v 9.10 2010/03/09 09:17:30 nisinaka Exp $
 */
public class JunFileRequesterDialogViewAwt extends JunAbstractDialogViewAwt implements JunFileRequesterDialogView {

	protected JunFileRequesterViewAwt fileRequesterView;

	/**
	 * Create a new instance of <code>JunFileRequesterDialogViewAwt</code> and initialize it.
	 * 
	 * @param aFileRequesterDialog jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog
	 * @category Instance creation
	 */
	public JunFileRequesterDialogViewAwt(JunFileRequesterDialog aSimpleDialog) {
		super(aSimpleDialog);
	}

	/**
	 * Answer the receiver's model as <code>JunFileRequesterDialog</code>.
	 * 
	 * @return jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog
	 * @see jp.co.sra.jun.graphics.navigator.JunFileRequesterDialogView#getFileRequesterDialog()
	 * @category model accessing
	 */
	public JunFileRequesterDialog getFileRequesterDialog() {
		return (JunFileRequesterDialog) this.model();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewPanel#buildComponent()
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fileRequesterView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -40));
		this.add(this.acceptButton(), WindowSpecLayout.Position(0.33f, -45, 1f, -35, 0.33f, 45, 1f, -5));
		this.add(this.cancelButton(), WindowSpecLayout.Position(0.67f, -45, 1f, -35, 0.67f, 45, 1f, -5));
		this.setPreferredSize(new Dimension(540, 300));

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my file requester view.
	 * 
	 * @return jp.co.sra.jun.graphics.navigator.JunFileRequesterViewAwt
	 * @category user interface
	 */
	protected JunFileRequesterViewAwt fileRequesterView() {
		if (fileRequesterView == null) {
			fileRequesterView = new JunFileRequesterViewAwt(this.getFileRequesterDialog().fileModel());
		}
		return fileRequesterView;
	}

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @see jp.co.sra.smalltalk.StView#_setupKeyboard(java.awt.Window)
	 * @category keyboard
	 */
	public void _setupKeyboard(Window aWindow) {
		this.fileRequesterView()._setupKeyboard(aWindow);
	}

}
