package jp.co.sra.jun.graphics.navigator;

import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.goodies.files.JunFileModel.FileType;

/**
 * JunFileRequesterViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2004/04/09 (by nisinaka)
 *  @updated   2006/11/29 (by m-asada)
 *  @updated   2008/05/07 (by m-asada)
 *  @updated   2008/10/15 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun706 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileRequesterViewSwing.java,v 9.12 2010/03/09 09:17:30 nisinaka Exp $
 */
public class JunFileRequesterViewSwing extends JunFileNavigatorViewSwing implements JunFileRequesterView {
	protected JComboBox fileType;
	protected JTextField tailFileName;

	/**
	 * Create a new instance of <code>JunFileRequesterViewSwing</code> and initialize it.
	 * 
	 * @param aFileRequester jp.co.sra.jun.graphics.navigator.JunFileRequester
	 * @category Instance creation
	 */
	public JunFileRequesterViewSwing(JunFileRequester aFileRequester) {
		super(aFileRequester);
	}

	/**
	 * Answer the receiver's model as <code>JunFileRequester</code>.
	 * 
	 * @return jp.co.sra.jun.graphics.navigator.JunFileRequester
	 * @see jp.co.sra.jun.graphics.navigator.JunFileRequesterView#getFileRequester()
	 * @category model accessing
	 */
	public JunFileRequester getFileRequester() {
		return (JunFileRequester) this.model();
	}

	/**
	 * Initialize the receiver. 
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release 
	 */
	@Override
	protected void initialize() {
		super.initialize();

		fileType = null;
		tailFileName = null;
	}

	/**
	 * Build the component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.homeButtonView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 25, 0f, 25));
		this.add(this.currentFileName(), WindowSpecLayout.Position(0f, 26, 0f, 1, 1f, -1, 0f, 25));
		this.add(this.parentSplitView(), WindowSpecLayout.Position(0f, 1, 0f, 26, 1f, -1, 1f, -26));
		this.add(this.fileType(), WindowSpecLayout.Position(0f, 1, 1f, -25, 0.5f, -1, 1f, -1));
		this.add(this.tailFileName(), WindowSpecLayout.Position(0.5f, 0, 1f, -25, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(500, 308));
	}

	/**
	 * Answer the file type combo box.
	 *
	 * @return javax.swing.JComboBox
	 * @category user interface
	 */
	protected JComboBox fileType() {
		if (fileType == null) {
			fileType = new JComboBox();
			fileType.setLightWeightPopupEnabled(false);
			FileType[] fileTypes = this.getFileRequester().fileTypes();
			for (int i = 0; i < fileTypes.length; i++) {
				fileType.addItem(fileTypes[i].name());
			}
			fileType.setSelectedItem(this.getFileRequester().fileType().value().name());

			fileType.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						JunFileRequesterViewSwing.this.getFileRequester().fileTypeChanged(fileType.getSelectedIndex());
					}
				}
			});
		}
		return fileType;
	}

	/**
	 * Answer the text field for the current file name.
	 * 
	 * @return javax.swing.JTextField
	 * @category user interface
	 */
	protected JTextField tailFileName() {
		if (tailFileName == null) {
			tailFileName = new JTextField();

			StSymbol requestType = this.getFileRequester().requestType();
			if (requestType == $("oldFile") || requestType == $("oldDirectory")) {
				tailFileName.setEnabled(false);
			} else {
				tailFileName.setText(this.getFileRequester().tailFileName());
				tailFileName.getDocument().addDocumentListener(new DocumentListener() {
					public void changedUpdate(DocumentEvent e) {
					}

					public void insertUpdate(DocumentEvent e) {
						JunFileRequesterViewSwing.this.getFileRequester().tailFileName_(tailFileName.getText());
					}

					public void removeUpdate(DocumentEvent e) {
						JunFileRequesterViewSwing.this.getFileRequester().tailFileName_(tailFileName.getText());
					}
				});
			}

			this.getFileRequester().tailFileNameHolder().compute_(new StBlockClosure1<String, String>() {
				public String value(String anObject) {
					if (tailFileName.getText().equals(anObject) == false) {
						tailFileName.setText(anObject);
					}
					return null;
				}
			});
		}
		return tailFileName;
	}
}
