package jp.co.sra.jun.metaball.atoms;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.metaball.abstracts.JunMetaball;

/**
 * JunTransformedMetaball class
 * 
 *  @author    nishihara
 *  @created   1999/11/05 (by nishihara)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTransformedMetaball.java,v 9.9 2010/03/09 09:17:31 nisinaka Exp $
 */
public class JunTransformedMetaball extends JunMetaball {

	/** metaball. */
	private JunMetaball metaball;

	/** Inverted transformation. */
	private Jun3dTransformation transformationInv;

	/**
	 * Create a new instance of JunTransformedMetaball and set it with solid,
	 * insideValue and outsideValue.
	 * 
	 * @param aJunMetaball
	 * @param aJun3dTransformation
	 * 
	 * @return jp.co.sra.jun.metaball.atoms.JunSolidMetaball
	 */
	public static JunTransformedMetaball On_transform_(JunMetaball aJunMetaball, Jun3dTransformation aJun3dTransformation) {
		JunTransformedMetaball aJunTransformedMetaball = new JunTransformedMetaball();
		aJunTransformedMetaball.setMetaball_transformation_(aJunMetaball, aJun3dTransformation);

		return aJunTransformedMetaball;
	}

	/**
	 * Answer the JunTransformedMetaball which is applied the specified
	 * transformation.
	 * 
	 * @param aJun3dTransformation
	 * 
	 * @return jp.co.sra.jun.metaball.atoms.JunTransformedMetaball
	 */
	public JunTransformedMetaball transform_(Jun3dTransformation aJun3dTransformation) {
		return JunTransformedMetaball.On_transform_(metaball, transformationInv.inverse().transform_(aJun3dTransformation));
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint
	 * 
	 * @return double
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		return metaball.valueAt_(aJun3dPoint.transform_(transformationInv));
	}

	/**
	 * set the JunTransformedMetaball which is applied the specified metaball
	 * and transformation.
	 * 
	 * @param aJunMetaball
	 * @param aJun3dTransformation
	 * 
	 * @return jp.co.sra.jun.metaball.solid.JunTransformedMetaball
	 */
	protected JunTransformedMetaball setMetaball_transformation_(JunMetaball aJunMetaball, Jun3dTransformation aJun3dTransformation) {
		this.metaball = aJunMetaball;
		this.transformationInv = aJun3dTransformation.inverse();
		return this;
	}

}
