package jp.co.sra.jun.metaball.solid;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StInterval;

import jp.co.sra.jun.csg.atoms.JunSolidCSG;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.boundaries.JunBorderGenerator;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.metaball.abstracts.JunMetaball;
import jp.co.sra.jun.octree.basic.JunOctree;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.solid.abstracts.JunSolidModel;
import jp.co.sra.jun.topology.elements.JunBody;

/**
 * JunMetaballSolid class
 * 
 *  @author    nishihara
 *  @created   1999/11/05 (by nishihara)
 *  @updated   2008/05/27 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMetaballSolid.java,v 9.11 2010/03/09 09:17:31 nisinaka Exp $
 */
public class JunMetaballSolid extends JunSolidModel {

	public static final double defaultThreshold = 1.0d;

	/** metaballs. */
	protected ArrayList<JunMetaball> metaballs;

	/** threshold. */
	protected double threshold;

	/**
	 * Create a new instance of <code>JunMetaballSolid</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunMetaballSolid() {
		super();
	}

	/**
	 * Create a new instance of JunMetaballSolid and initialize it with the
	 * threshold (double).
	 * 
	 * @param aNumber double
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid Threshold_(double aNumber) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.threshold_(aNumber);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and initialize it with the
	 * threshold (int).
	 * 
	 * @param aNumber int
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid Threshold_(int aNumber) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.threshold_(aNumber);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 1 metaball.
	 * 
	 * @param aJunMetaball jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_(JunMetaball aJunMetaball) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.add_(aJunMetaball);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 1 metaball and
	 * threshold.
	 * 
	 * @param aJunMetaball jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aNumber double
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_threshold_(JunMetaball aJunMetaball, double aNumber) {
		JunMetaballSolid solid = JunMetaballSolid.Threshold_(aNumber);
		solid.add_(aJunMetaball);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 2 metaballs.
	 * 
	 * @param aJunMetaball1 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball2 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_with_(JunMetaball aJunMetaball1, JunMetaball aJunMetaball2) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.add_(aJunMetaball1);
		solid.add_(aJunMetaball2);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 2 metaballs
	 * and threshold.
	 * 
	 * @param aJunMetaball1 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball2 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aNumber double
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_with_threshold_(JunMetaball aJunMetaball1, JunMetaball aJunMetaball2, double aNumber) {
		JunMetaballSolid solid = JunMetaballSolid.Threshold_(aNumber);
		solid.add_(aJunMetaball1);
		solid.add_(aJunMetaball2);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 3 metaballs.
	 * 
	 * @param aJunMetaball1 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball2 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball3 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_with_with_(JunMetaball aJunMetaball1, JunMetaball aJunMetaball2, JunMetaball aJunMetaball3) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.add_(aJunMetaball1);
		solid.add_(aJunMetaball2);
		solid.add_(aJunMetaball3);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 3 metaballs
	 * and threshold.
	 * 
	 * @param aJunMetaball1 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball2 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball3 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aNumber double
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_with_with_threshold_(JunMetaball aJunMetaball1, JunMetaball aJunMetaball2, JunMetaball aJunMetaball3, double aNumber) {
		JunMetaballSolid solid = JunMetaballSolid.Threshold_(aNumber);
		solid.add_(aJunMetaball1);
		solid.add_(aJunMetaball2);
		solid.add_(aJunMetaball3);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with 4 metaballs.
	 * 
	 * @param aJunMetaball1 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball2 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball3 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param aJunMetaball4 jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid With_with_with_with_(JunMetaball aJunMetaball1, JunMetaball aJunMetaball2, JunMetaball aJunMetaball3, JunMetaball aJunMetaball4) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.add_(aJunMetaball1);
		solid.add_(aJunMetaball2);
		solid.add_(aJunMetaball3);
		solid.add_(aJunMetaball4);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with vectors of
	 * metaballs.
	 * 
	 * @param aCollectionOfJunMetaball java.util.Collection
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid WithAll_(Collection<JunMetaball> aCollectionOfJunMetaball) {
		JunMetaballSolid solid = new JunMetaballSolid();
		solid.addAll_(aCollectionOfJunMetaball);
		return solid;
	}

	/**
	 * Create a new instance of JunMetaballSolid and set it with vectors of
	 * metaballs and threshold.
	 * 
	 * @param aCollectionOfJunMetaball java.util.Collection
	 * @param aNumber double
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 */
	public static final JunMetaballSolid WithAll_threshold_(Collection<JunMetaball> aCollectionOfJunMetaball, double aNumber) {
		JunMetaballSolid solid = JunMetaballSolid.Threshold_(aNumber);
		solid.addAll_(aCollectionOfJunMetaball);
		return solid;
	}

	/**
	 * add the specified element of JunMetaball.
	 * 
	 * @param aJunMetaball jp.co.sra.jun.metaball.abstracts.JunMetaball
	 */
	public void add_(JunMetaball aJunMetaball) {
		metaballs.add(aJunMetaball);
	}

	/**
	 * add all the specified elements of JunMetaball.
	 * 
	 * @param aCollectionOfJunMetaball java.util.Collection
	 */
	public void addAll_(Collection<JunMetaball> aCollectionOfJunMetaball) {
		Iterator<JunMetaball> i = aCollectionOfJunMetaball.iterator();
		while (i.hasNext()) {
			this.add_(i.next());
		}
	}

	/**
	 * return the JunSolidCSG created on this.
	 * 
	 * @return jp.co.sra.jun.csg.atoms.JunSolidCSG
	 */
	public JunSolidCSG asCSGSolid() {
		return JunSolidCSG.On_(this);
	}

	/**
	 * return the JunBody created on this with x intervals, y intervals and z
	 * intervals.
	 * 
	 * @param xInterval jp.co.sra.smalltalk.StInterval
	 * @param yInterval jp.co.sra.smalltalk.StInterval
	 * @param zInterval jp.co.sra.smalltalk.StInterval
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 */
	public JunBody asJunBodyXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		return JunBorderGenerator.ContourBodyFrom_at_sign_xInterval_yInterval_zInterval_interim_(new StBlockClosure3<Double, Double, Double, Double>() {
			public Double value(Double x, Double y, Double z) {
				return JunMetaballSolid.this.valueAt_(new Jun3dPoint(x, y, z));
			}
		}, threshold, 1, xInterval, yInterval, zInterval, null);
	}

	/**
	 * return the JunOctree created on this.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @return jp.co.sra.jun.octree.basic.JunOctree;
	 */
	public JunOctree asJunBoxelOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		return JunOctree.FromBlock_depth_bounds_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint point) {
				return JunMetaballSolid.this.valueAt_(point);
			}
		}, anInteger, aJun3dBoundingBox);
	}

	/**
	 * return the JunOctree created on this.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @return jp.co.sra.jun.octree.basic.JunOctree;
	 */
	public JunOctree asJunBrepsOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		octree.brepsBlock_depth_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint point) {
				return JunMetaballSolid.this.valueAt_(point);
			}
		}, anInteger);
		return octree;
	}

	/**
	 * return the JunOpenGL3dObject created on this with x intervals, y
	 * intervals and z intervals.
	 * 
	 * @param xInterval jp.co.sra.smalltalk.StInterval
	 * @param yInterval jp.co.sra.smalltalk.StInterval
	 * @param zInterval jp.co.sra.smalltalk.StInterval
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		JunOpenGL3dObject aJunOpenGL3dObject;
		aJunOpenGL3dObject = this.asJunBodyXInterval_yInterval_zInterval_(xInterval, yInterval, zInterval).asJunOpenGL3dObject();
		aJunOpenGL3dObject.paint_(null);
		return aJunOpenGL3dObject;
	}

	/**
	 * Answer true if the receiver contains the specified point, otherwise false.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return boolean
	 * @see jp.co.sra.jun.solid.abstracts.JunSolidModel#containsPoint_(jp.co.sra.jun.geometry.basic.Jun3dPoint)
	 * @category testing
	 */
	public boolean containsPoint_(Jun3dPoint aJun3dPoint) {
		return this.valueAt_(aJun3dPoint) >= 0;
	}

	/**
	 * remove the specified element of JunMetaball.
	 * 
	 * @param aJunMetaball jp.co.sra.jun.metaball.abstracts.JunMetaball
	 */
	public void remove_(JunMetaball aJunMetaball) {
		metaballs.remove(aJunMetaball);
	}

	/**
	 * remove the specified element of JunMetaball. if none evaluate errorBlock.
	 * 
	 * @param aJunMetaball jp.co.sra.jun.metaball.abstracts.JunMetaball
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.metaball.abstracts.JunMetaball
	 */
	public JunMetaball remove_ifAbsent_(JunMetaball aJunMetaball, StBlockClosure0<JunMetaball> errorBlock) {
		if (metaballs.contains(aJunMetaball)) {
			metaballs.remove(aJunMetaball);
			return aJunMetaball;
		} else {
			return errorBlock.value();
		}
	}

	/**
	 * answer the threshold.
	 * 
	 * @return double
	 */
	public double threshold() {
		return this.threshold;
	}

	/**
	 * set the threshold.
	 * 
	 * @param aNumber double
	 */
	public void threshold_(double aNumber) {
		this.threshold = aNumber;
	}

	/**
	 * Answer the JunTransformedMetaballSolid which is applied the specified
	 * transformation.
	 * 
	 * @param aJun3dTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @return jp.co.sra.jun.metaball.solid.JunTransformedMetaballSolid
	 */
	public JunTransformedMetaballSolid transform_(Jun3dTransformation aJun3dTransformation) {
		return JunTransformedMetaballSolid.WithAll_threshold_transform_(metaballs, this.threshold, aJun3dTransformation);
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return double
	 * @see jp.co.sra.jun.solid.abstracts.JunSolidModel#valueAt_(jp.co.sra.jun.geometry.basic.Jun3dPoint)
	 * @category accessing
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		double value = 0;

		Iterator<JunMetaball> i = metaballs.iterator();
		while (i.hasNext()) {
			value += i.next().valueAt_(aJun3dPoint);
		}

		return value - threshold;
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		this.threshold = defaultThreshold;
		this.metaballs = new ArrayList<JunMetaball>();
	}

}
