package jp.co.sra.jun.metaball.solid;

import java.util.Collection;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.metaball.abstracts.JunMetaball;

/**
 * JunTransformedMetaballSolid class
 * 
 *  @author    nishihara
 *  @created   1999/11/05 (by nishihara)
 *  @updated   2008/04/17 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTransformedMetaballSolid.java,v 9.10 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunTransformedMetaballSolid extends JunMetaballSolid {

	/** Inverted transformation. */
	protected Jun3dTransformation transformationInv;

	/**
	 * Create a new instance of JunTransformedMetaballSolid and set it with
	 * Transformation, threshold
	 * 
	 * @param aNumber double
	 * @param aJun3dTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @return jp.co.sra.jun.metaball.solid.JunTransformedMetaballSolid
	 */
	public static JunTransformedMetaballSolid Threshold_transform_(double aNumber, Jun3dTransformation aJun3dTransformation) {
		JunTransformedMetaballSolid solid;
		solid = (JunTransformedMetaballSolid) JunMetaballSolid.Threshold_(aNumber);
		solid.setTransformation_(aJun3dTransformation);
		return solid;
	}

	/**
	 * Create a new instance of JunTransformedMetaballSolid and set it with
	 * collection of JunMetaball, transformation, threshold
	 * 
	 * @param aCollectionOfJunMetaball java.util.Collection
	 * @param aNumber double
	 * @param aJun3dTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @return jp.co.sra.jun.metaball.solid.JunTransformedMetaballSolid
	 */
	public static JunTransformedMetaballSolid WithAll_threshold_transform_(Collection<JunMetaball> aCollectionOfJunMetaball, double aNumber, Jun3dTransformation aJun3dTransformation) {
		JunTransformedMetaballSolid solid;
		solid = JunTransformedMetaballSolid.Threshold_transform_(aNumber, aJun3dTransformation);
		solid.addAll_(aCollectionOfJunMetaball);
		return solid;
	}

	/**
	 * Answer the JunTransformedMetaballSolid which is applied the specified
	 * transformation.
	 * 
	 * @param aJun3dTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @return jp.co.sra.jun.metaball.solid.JunTransformedMetaballSolid
	 */
	public JunTransformedMetaballSolid transform_(Jun3dTransformation aJun3dTransformation) {
		Jun3dTransformation aT = transformationInv.inverse().transform_(aJun3dTransformation);
		return JunTransformedMetaballSolid.WithAll_threshold_transform_(metaballs, threshold, aT);
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return double
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		return super.valueAt_(aJun3dPoint.transform_(transformationInv));
	}

	/**
	 * set the specified transformation.
	 * 
	 * @param aJun3dTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 */
	protected void setTransformation_(Jun3dTransformation aJun3dTransformation) {
		transformationInv = aJun3dTransformation.inverse();
	}

}
