package jp.co.sra.jun.obj;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StReadStream;

/**
 * JunAbstractOBJParser class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/25 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAbstractOBJParser.java,v 9.7 2010/03/09 09:17:32 nisinaka Exp $
 */
public abstract class JunAbstractOBJParser extends JunOBJScanner {

	protected Map<String, String> keyTable;

	/**
	 * Create a read stream from the object.
	 * 
	 * @return jp.co.sra.smalltalk.StReadStream
	 * @param anObject java.lang.Object
	 * @category private
	 */
	protected static StReadStream MakeStream_(Object anObject) {
		if (anObject instanceof StReadStream) {
			return (StReadStream) anObject;
		} else if (anObject instanceof String) {
			return new StReadStream((String) anObject);
		} else if (anObject instanceof File) {
			try {
				return new StReadStream((File) anObject);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return new StReadStream(anObject.toString());
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.obj.JunOBJScanner#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		this.initKeyTable();
	}

	/**
	 * Initialize the keyword table.
	 * 
	 * @category initialize-release
	 */
	protected void initKeyTable() {
		keyTable = new HashMap<String, String>();
	}

	/**
	 * Parse the stream and generate an object.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @return java.lang.Object
	 * @category parsing
	 */
	protected Object parse_(StReadStream sourceStream) {
		this.init_(sourceStream);
		this.parseStream();
		return null;
	}

	/**
	 * Parse the stream.
	 * 
	 * @category parsing
	 */
	protected void parseStream() {
		while (source.atEnd() == false) {
			this.parseLine();
		}
	}

	/**
	 * Parse the line.
	 * 
	 * @category parsing
	 */
	protected void parseLine() {
		this.nextToken();

		String action = this.action();
		if (action != null) {
			try {
				this.perform_(action);
			} catch (Exception e) {
				throw new SmalltalkException(e);
			}
		}
		this.skipLine();
	}

	/**
	 * Answer the action string for the current token.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	protected String action() {
		return keyTable.get(token);
	}

	/**
	 * Initialize with the source stream.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @category private
	 */
	protected void init_(StReadStream sourceStream) {
		this.on_(sourceStream);
	}

}
