package jp.co.sra.jun.obj;

import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StReadStream;

/**
 * JunMTLParser class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMTLParser.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunMTLParser extends JunAbstractOBJParser {

	protected List<JunOBJMaterial> materials;
	protected JunOBJMaterial currentMaterial;

	/**
	 * Parse the object and generate.
	 * 
	 * @param anObject java.lang.Object
	 * @return java.util.List<JunOBJMaterial>
	 * @category Utilities
	 */
	public static List<JunOBJMaterial> Parse_(Object anObject) {
		JunMTLParser parser = new JunMTLParser();
		return parser.parse_(MakeStream_(anObject));
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.obj.JunAbstractOBJParser#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		materials = new ArrayList<JunOBJMaterial>();
		currentMaterial = null;
	}

	/**
	 * Initialize the keyword table.
	 * 
	 * @see jp.co.sra.jun.obj.JunAbstractOBJParser#initKeyTable()
	 * @category initialize-release
	 */
	@Override
	protected void initKeyTable() {
		super.initKeyTable();

		//keyTable.put("Ka", "parseAmbientColor");
		//keyTable.put("Ks", "parseSpecularColor");
		//keyTable.put("Ns", "parseShininess");
		keyTable.put("Kd", "parseDiffuseColor");
		keyTable.put("Tr", "parseTransparency");
		keyTable.put("newmtl", "parseNewMaterial");
		keyTable.put("map_Kd", "parseTexture");
	}

	/**
	 * Add the specified material to my list.
	 * 
	 * @param aJunObjMaterial jp.co.sra.jun.obj.JunOBJMaterial
	 * @category adding
	 */
	protected void addMaterial_(JunOBJMaterial aJunObjMaterial) {
		materials.add(aJunObjMaterial);
	}

	/**
	 * Parse the stream and generate an object.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @return java.lang.Object
	 * @see jp.co.sra.jun.obj.JunAbstractOBJParser#parse_(jp.co.sra.smalltalk.StReadStream)
	 * @category parsing
	 */
	@Override
	protected List<JunOBJMaterial> parse_(StReadStream sourceStream) {
		super.parse_(sourceStream);
		if (currentMaterial != null) {
			this.addMaterial_(currentMaterial);
		}
		return materials;
	}

	/**
	 * Parse the diffuse color.
	 * 
	 * 		Kd float float float
	 * 
	 * Diffuse color r g b
	 * 
	 * @category parsing
	 */
	public void parseDiffuseColor() {
		if (currentMaterial == null) {
			this.failed_("No current material available to assign a diffuse color");
			return;
		}

		float[] array = new float[3];
		for (int i = 0; i < array.length; i++) {
			this.nextToken();
			if (tokenType != $("number")) {
				this.failed_("Diffuse color not complete");
				return;
			}
			array[i] = ((Number) token).floatValue();
		}
		currentMaterial.diffuseColor_(array);
	}

	/**
	 * Parse the new material.
	 * 
	 * 		newmtl string
	 * 
	 * Material name
	 * 
	 * @category parsing
	 */
	public void parseNewMaterial() {
		if (currentMaterial != null) {
			this.addMaterial_(currentMaterial);
		}
		currentMaterial = new JunOBJMaterial(this.nextToken().toString());
	}

	/**
	 * Parse the texture.
	 * 
	 * 		map_Kd string
	 * 
	 * Texture filename
	 * 
	 * @category parsing
	 */
	public void parseTexture() {
		if (currentMaterial == null) {
			this.failed_("No current material available to assign a texture (file)name");
			return;
		}

		currentMaterial.textureImageName_(this.nextToken().toString());
	}

	/**
	 * Parse the transparency.
	 * 
	 * 		Tr float
	 * 
	 * Transparency Tr (alpha)
	 * 
	 * @category parsing
	 */
	public void parseTransparency() {
		if (currentMaterial == null) {
			this.failed_("No current material available to assign a transparency value");
			return;
		}

		currentMaterial.transparency_(((Number) this.nextToken()).floatValue());
	}

}
