package jp.co.sra.jun.obj;

import java.awt.Color;

import jp.co.sra.smalltalk.StObject;

/**
 * JunMaterialKey class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/08/18 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMaterialKey.java,v 9.3 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunMaterialKey extends StObject {

	protected Color colorValue;
	protected float alpha = 1.0f;
	protected String textureImageName;

	/**
	 * Create a new instance of JunMaterialKey and initialize it.
	 *
	 * @param colorValue
	 * @param transparency
	 * @param textureImageName
	 * @category Instance creation
	 */
	public JunMaterialKey(Color aColorValue, float alpha, String textureImageName) {
		this.colorValue_(aColorValue);
		this.alpha_(alpha);
		this.textureImageName_(textureImageName);
	}

	/**
	 * Create a new instance of JunMaterialKey and initialize it.
	 *
	 * @param aColorValue java.awt.Color
	 * @category Instance creation
	 */
	public JunMaterialKey(Color aColorValue) {
		this(aColorValue, 1.0f, "");
	}

	/**
	 * Answer my current color value.
	 * 
	 * @return java.awt.Color
	 * @category accessing
	 */
	public Color colorValue() {
		return colorValue;
	}

	/**
	 * Set my new color value.
	 * 
	 * @param aColor java.awt.Color
	 * @category accessing
	 */
	public void colorValue_(Color aColor) {
		if (aColor != null) {
			float[] rgba = aColor.getRGBComponents(null);
			aColor = new Color(rgba[0], rgba[1], rgba[2], alpha);
		}
		colorValue = aColor;
	}

	/**
	 * Answer my current alpha.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float alpha() {
		return alpha;
	}

	/**
	 * Set my new alpha.
	 * 
	 * @param newAlpha float
	 * @category accessing
	 */
	public void alpha_(float newAlpha) {
		alpha = newAlpha;
		this.colorValue_(colorValue);
	}

	/**
	 * Answer my current texture image name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String textureImageName() {
		return textureImageName;
	}

	/**
	 * Set my new texture image name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public void textureImageName_(String aString) {
		textureImageName = aString;
	}

	/**
	 * Returns a hash code value for the object.
	 * 
	 * @return int
	 * @see java.lang.Object#hashCode()
	 * @category comparing
	 */
	@Override
	public int hashCode() {
		int hash = 1;
		hash = hash * 31 + colorValue.hashCode();
		hash = hash * 31 + textureImageName.hashCode();
		return hash;
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * 
	 * @param anObject java.lang.Object
	 * @return boolean
	 * @see java.lang.Object#equals(java.lang.Object)
	 * @category comparing
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject == null) {
			return false;
		}
		if (anObject.getClass() != this.getClass()) {
			return false;
		}

		JunMaterialKey anotherKey = (JunMaterialKey) anObject;
		if (colorValue == null) {
			if (anotherKey.colorValue != null) {
				return false;
			}
		} else {
			if (colorValue.equals(anotherKey.colorValue) == false) {
				return false;
			}
		}

		if (textureImageName == null) {
			if (anotherKey.textureImageName != null) {
				return false;
			}
		} else {
			if (textureImageName.equals(anotherKey.textureImageName) == false) {
				return false;
			}
		}

		return alpha == anotherKey.alpha;
	}

}
