package jp.co.sra.jun.obj;

import java.util.ArrayList;
import java.util.List;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOBJGroup class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJGroup.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJGroup extends JunOBJAbstractGroup {

	protected List<JunOBJMesh> meshes;

	/**
	 * Create a new instance of JunOBJGroup and initialize it.
	 *
	 * @param aString java.lang.String
	 * @category Instance creation
	 */
	public JunOBJGroup(String aString) {
		super(aString);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.obj.JunOBJAbstractGroup#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		meshes = new ArrayList<JunOBJMesh>();
	}

	/**
	 * Answer my current meshes.
	 * 
	 * @return java.util.List<JunOBJMesh>
	 * @category accessing
	 */
	public List<JunOBJMesh> meshes() {
		return meshes;
	}

	/**
	 * Set my new meshes.
	 * 
	 * @param aList java.util.List<JunOBJMesh>
	 * @category accessing
	 */
	public void meshes_(List<JunOBJMesh> aList) {
		meshes = aList;
	}

	/**
	 * Answer the number of the meshes.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return meshes.size();
	}

	/**
	 * Add the specified mesh to my list.
	 * 
	 * @param aJunOBJMesh jp.co.sra.jun.obj.JunOBJMesh
	 * @category adding
	 */
	public void addMesh(JunOBJMesh aJunOBJMesh) {
		meshes.add(aJunOBJMesh);
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @param aJunOBJParser jp.co.sra.jun.obj.JunOBJParser
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectUsing_(JunOBJParser aJunOBJParser) {
		return aJunOBJParser.junOpenGL3dObjectFromGroup_(this);
	}

}
