package jp.co.sra.jun.obj;

import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOBJIndexedFace class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJIndexedFace.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJIndexedFace extends StObject {

	protected List<JunOBJIndexedVertex> vertices;

	/**
	 * Create a new instance of JunOBJIndexedFace and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOBJIndexedFace() {
		this.initialize();
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		vertices = new ArrayList<JunOBJIndexedVertex>();
	}

	/**
	 * Answer my current vertices.
	 * 
	 * @return java.util.List<JunOBJIndexedVertex>
	 * @category accessing
	 */
	public List<JunOBJIndexedVertex> vertices() {
		return vertices;
	}

	/**
	 * Answer the array of the position indices.
	 * 
	 * @return java.lang.Integer[]
	 * @category accessing
	 */
	public Integer[] positionIndices() {
		Integer[] indices = new Integer[vertices.size()];
		for (int i = 0; i < indices.length; i++) {
			indices[i] = vertices.get(i).positionIndex();
		}
		return indices;
	}

	/**
	 * Answer the array of the normal indices.
	 * 
	 * @return java.lang.Integer[]
	 * @category accessing
	 */
	public Integer[] normalIndices() {
		Integer[] indices = new Integer[vertices.size()];
		for (int i = 0; i < indices.length; i++) {
			indices[i] = vertices.get(i).normalIndex();
		}
		return indices;
	}

	/**
	 * Answer the number of the vertices.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return vertices.size();
	}

	/**
	 * Add the specified vertex to my list.
	 * 
	 * @param aJunOBJIndexedVertex jp.co.sra.jun.obj.JunOBJIndexedVertex
	 * @category adding
	 */
	public void addIndexedVertex_(JunOBJIndexedVertex aJunOBJIndexedVertex) {
		vertices.add(aJunOBJIndexedVertex);
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @param aJunOBJParser jp.co.sra.jun.obj.JunOBJParser
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectUsing_(JunOBJParser aJunOBJParser) {
		return aJunOBJParser.junOpenGL3dObjectFromFace_(this);
	}

	/**
	 * Answer true if the receiver has normals, otherwise false.
	 * 
	 * @return boolean
	 * @category testing 
	 */
	public boolean hasNormals() {
		assert vertices.isEmpty() == false;
		return vertices.get(0).size() >= 3;
	}

	/**
	 * Answer true if the receiver has equal sized vertices, otherwise false.
	 * 
	 * @return boolean
	 * @category testing 
	 */
	public boolean hasEqualSizedVertices() {
		assert vertices.isEmpty() == false;
		int firstSize = vertices.get(0).size();
		for (int i = 1; i < this.size(); i++) {
			if (vertices.get(i).size() != firstSize) {
				return false;
			}
		}
		return true;
	}

}
