package jp.co.sra.jun.obj;

import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StObject;

/**
 * JunOBJIndexedVertex class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJIndexedVertex.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJIndexedVertex extends StObject {

	protected List<Integer> indices;

	/**
	 * Create a new instance of JunOBJIndexedVertex and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOBJIndexedVertex() {
		this.initialize();
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		indices = new ArrayList<Integer>(3);
	}

	/**
	 * Answer the number of the indices.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return indices.size();
	}

	/**
	 * Answer the position index.
	 * 
	 * @return java.lang.Integer
	 * @category accessing
	 */
	public Integer positionIndex() {
		return indices.get(0);
	}

	/**
	 * Answer the texture index.
	 * 
	 * @return java.lang.Integer
	 * @category accessing
	 */
	public Integer textureIndex() {
		return indices.size() >= 2 ? indices.get(1) : null;
	}

	/**
	 * Answer the normal index.
	 * 
	 * @return java.lang.Integer
	 * @category accessing
	 */
	public Integer normalIndex() {
		return indices.size() >= 3 ? indices.get(2) : null;
	}

	/**
	 * Add an integer as an index.
	 * 
	 * @param anInteger java.lang.Integer
	 * @category accessing
	 */
	public void addIndex_(Integer anInteger) {
		indices.add(anInteger);
	}

}
