package jp.co.sra.jun.obj;

import java.awt.Color;
import java.io.BufferedWriter;
import java.io.IOException;

import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.opengl.texture.JunOpenGLTexture;

/**
 * JunOBJMaterial class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJMaterial.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJMaterial extends StObject {

	protected static float[] DefaultDiffuseColor = new float[] { 0.8f, 0.8f, 0.8f };
	protected static float DefaultTransparency = 1.0f;

	protected String name;
	protected float[] diffuseColor;
	protected float transparency;
	protected JunOpenGLTexture texture;
	protected String textureImageName;

	/**
	 * Create a new instance of JunOBJMaterial and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOBJMaterial() {
		this.initialize();
	}

	/**
	 * Create a new instance of JunOBJMaterial and initialize it.
	 *
	 * @param aString java.lang.String
	 * @category Instance creation
	 */
	public JunOBJMaterial(String aString) {
		this();
		this.name_(aString);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		name = null;
		diffuseColor = DefaultDiffuseColor;
		transparency = -1;
		texture = null;
		textureImageName = null;
	}

	/**
	 * Answer my current name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String name() {
		return name;
	}

	/**
	 * Set my new name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public void name_(String aString) {
		name = aString;
	}

	/**
	 * Answer my current diffuse color.
	 * 
	 * @return float[]
	 * @category accessing
	 */
	public float[] diffuseColor() {
		return diffuseColor;
	}

	/**
	 * Set my new diffuse color.
	 * 
	 * @param anArray float[]
	 * @category accessing
	 */
	public void diffuseColor_(float[] anArray) {
		diffuseColor = anArray;
	}

	/**
	 * Answer my current transparency.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float transparency() {
		return transparency;
	}

	/**
	 * Set my new transparency.
	 * 
	 * @param floatValue float
	 * @category accessing
	 */
	public void transparency_(float floatValue) {
		transparency = floatValue;
	}

	/**
	 * Answer my current texture.
	 * 
	 * @return jp.co.sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public JunOpenGLTexture texture() {
		return texture;
	}

	/**
	 * Set my new texture.
	 * 
	 * @param aTexture jp.co.sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public void texture_(JunOpenGLTexture aTexture) {
		texture = aTexture;
	}

	/**
	 * Answer my current texture image name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String textureImageName() {
		return textureImageName;
	}

	/**
	 * Set my new texture image name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public void textureImageName_(String aString) {
		textureImageName = aString;
	}

	/**
	 * Answer my current color value.
	 * 
	 * @return java.lang.Color
	 * @category accessing
	 */
	public Color colorValue() {
		if (diffuseColor == null) {
			return null;
		}

		return new Color(diffuseColor[0], diffuseColor[1], diffuseColor[2]);
	}

	/**
	 * Set my new color value.
	 * 
	 * @param aColor java.lang.Color
	 * @category accessing
	 */
	public void colorValue_(Color aColor) {
		this.diffuseColor_(aColor.getRGBColorComponents(null));
	}

	/**
	 * Answer my current material key.
	 * 
	 * @return jp.co.sra.jun.obj.JunMaterialKey
	 * @category accessing
	 */
	public JunMaterialKey materialKey() {
		return new JunMaterialKey(this.colorValue(), transparency, textureImageName);
	}

	/**
	 * Write myself on the specified writer.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @throws java.io.IOException
	 * @category writing
	 */
	public void writeOn_(BufferedWriter aWriter) throws IOException {
		aWriter.write("newmtl ");
		aWriter.write(name);
		aWriter.newLine();

		if (diffuseColor != null) {
			aWriter.write("Kd");
			for (float f : diffuseColor) {
				aWriter.write(' ');
				aWriter.write(String.valueOf(f));
			}
			aWriter.newLine();
		}

		if (0 <= transparency && transparency <= 1) {
			aWriter.write("Tr ");
			aWriter.write(String.valueOf(transparency));
			aWriter.newLine();
		}

		if (textureImageName != null && textureImageName.length() > 0) {
			aWriter.write("map_Kd ");
			aWriter.write(textureImageName);
			aWriter.newLine();
		}

		aWriter.newLine();
	}

}
