package jp.co.sra.jun.obj;

import java.util.ArrayList;
import java.util.List;

import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOBJMesh class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/26 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJMesh.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJMesh extends StObject {

	protected JunOBJMaterial material;
	protected List<JunOBJIndexedFace> faces;

	/**
	 * Create a new instance of JunOBJMesh and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOBJMesh() {
		this.initialize();
	}

	/**
	 * Create a new instance of JunOBJMesh and initialize it.
	 *
	 * @param aJunOBJMaterial jp.co.sra.jun.obj.JunOBJMaterial
	 * @category Instance creation
	 */
	public JunOBJMesh(JunOBJMaterial aJunOBJMaterial) {
		this();
		this.material_(aJunOBJMaterial);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		material = null;
		faces = new ArrayList<JunOBJIndexedFace>();
	}

	/**
	 * Answer my current material.
	 * 
	 * @return jp.co.sra.jun.obj.JunOBJMaterial
	 * @category accessing
	 */
	public JunOBJMaterial material() {
		return material;
	}

	/**
	 * Set my new material.
	 * 
	 * @param aJunOBJMaterial jp.co.sra.jun.obj.JunOBJMaterial
	 * @category accessing
	 */
	public void material_(JunOBJMaterial aJunOBJMaterial) {
		material = aJunOBJMaterial;
	}

	/**
	 * Answer my current faces.
	 * 
	 * @return java.util.List<JunOBJIndexedFace>
	 * @category accessing
	 */
	public List<JunOBJIndexedFace> faces() {
		return faces;
	}

	/**
	 * Set my new faces.
	 * 
	 * @param aList java.util.List<JunOBJIndexedFace>
	 * @category accessing
	 */
	public void faces_(List<JunOBJIndexedFace> aList) {
		faces = aList;
	}

	/**
	 * Answer the number of the faces.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return faces.size();
	}

	/**
	 * Answer the array of the texture indices.
	 * 
	 * @return java.lang.Integer[]
	 * @category accessing
	 */
	public Integer[] textureIndices() {
		ArrayList<Integer> aList = new ArrayList<Integer>();
		for (JunOBJIndexedFace face : faces) {
			for (JunOBJIndexedVertex vertex : face.vertices()) {
				aList.add(vertex.textureIndex());
			}
		}

		return aList.toArray(new Integer[aList.size()]);
	}

	/**
	 * Add the specified face to my list.
	 * 
	 * @param aJunOBJIndexedFace jp.co.sra.jun.obj.JunOBJIndexedFace
	 * @category adding
	 */
	public void addIndexedFace_(JunOBJIndexedFace aJunOBJIndexedFace) {
		faces.add(aJunOBJIndexedFace);
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @param aJunOBJParser jp.co.sra.jun.obj.JunOBJParser
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectUsing_(JunOBJParser aJunOBJParser) {
		return aJunOBJParser.junOpenGL3dObjectFromMesh_(this);
	}

}
