package jp.co.sra.jun.obj;

import java.io.PrintWriter;
import java.io.StringWriter;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.support.JunTestExamples;

/**
 * JunOBJParserTestExamples class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/25 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun732 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJParserTestExamples.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJParserTestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunOpenGL3dObject anObject = JunOBJParser.Parse_(ColoredBoxModel());
		anObject.show();

		return true;
	}

	/**
	 * Example2:
	 * 
	 * Normally an obj file will be parsed that automatically parses an MTL file.
	 * In this case we parse a stream; so we have to parse the material stream explicitly.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		JunOBJParser objParser = new JunOBJParser();
		objParser.materials_(JunMTLParser.Parse_(JunMTLParserTestExamples.ColoredBoxMaterial()));
		JunOpenGL3dObject anObject = objParser.parse_(JunOBJParser.MakeStream_(ColoredBoxModel()));
		anObject.show();

		return true;
	}

	/**
	 * Create a sample string of OBJ format.
	 * 
	 * @return java.lang.String
	 * @category Examples
	 */
	protected static String ColoredBoxModel() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("# Wavefront OBJ file");
		pw.println("# coloredBox.obj");
		pw.println("#");
		pw.println();
		pw.println("# mtllib coloredBox.mtl");
		pw.println();
		pw.println("g Model");
		pw.println();
		pw.println("usemtl Red");
		pw.println("v -15.7428 -5.68434e-014 8.04128");
		pw.println("vt -8.04128 -5.68434e-014");
		pw.println("vn 1 0 0");
		pw.println("v -15.7428 -148.25 8.04128");
		pw.println("vt -8.04128 -148.25");
		pw.println("v -15.7428 -148.25 -12.2087");
		pw.println("vt 12.2087 -148.25");
		pw.println("v -15.7428 -5.68434e-014 -12.2087");
		pw.println("vt 12.2087 -5.68434e-014");
		pw.println("v -15.7428 7.5 -12.2087");
		pw.println("vt 12.2087 7.5");
		pw.println("v -15.7428 7.5 8.04128");
		pw.println("vt -8.04128 7.5");
		pw.println("f 1/1/1 2/2/1 3/3/1 4/4/1 5/5/1 6/6/1 ");
		pw.println();
		pw.println("usemtl Red");
		pw.println("v 40.7572 -7.4607e-014 -12.2087");
		pw.println("vt -12.2087 -7.4607e-014");
		pw.println("vn -1 3.08149e-031 0");
		pw.println("v 40.7572 -148.25 -12.2087");
		pw.println("vt -12.2087 -148.25");
		pw.println("v 40.7572 -148.25 8.04128");
		pw.println("vt 8.04128 -148.25");
		pw.println("v 40.7572 -7.4607e-014 8.04128");
		pw.println("vt 8.04128 -7.4607e-014");
		pw.println("v 40.7572 7.5 8.04128");
		pw.println("vt 8.04128 7.5");
		pw.println("v 40.7572 7.5 -12.2087");
		pw.println("vt -12.2087 7.5");
		pw.println("f 7/7/2 8/8/2 9/9/2 10/10/2 11/11/2 12/12/2 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("v -18.3131 -5.68434e-014 11.221");
		pw.println("vt -18.3131 -5.68434e-014");
		pw.println("vn 0 0 1");
		pw.println("v 42.9369 -5.68434e-014 11.221");
		pw.println("vt 42.9369 -5.68434e-014");
		pw.println("v 42.9369 7.5 11.221");
		pw.println("vt 42.9369 7.5");
		pw.println("v -18.3131 7.5 11.221");
		pw.println("vt -18.3131 7.5");
		pw.println("f 13/13/3 14/14/3 15/15/3 16/16/3 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("v -18.3131 -5.68434e-014 -14.779");
		pw.println("vt -14.779 -5.68434e-014");
		pw.println("vn -1 0 0");
		pw.println("vt 11.221 -5.68434e-014");
		pw.println("vt 11.221 7.5");
		pw.println("v -18.3131 7.5 -14.779");
		pw.println("vt -14.779 7.5");
		pw.println("f 17/17/4 13/18/4 16/19/4 18/20/4 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("v 42.9369 -5.68434e-014 -14.779");
		pw.println("vt -42.9369 -5.68434e-014");
		pw.println("vn 0 0 -1");
		pw.println("vt 18.3131 -5.68434e-014");
		pw.println("vt 18.3131 7.5");
		pw.println("v 42.9369 7.5 -14.779");
		pw.println("vt -42.9369 7.5");
		pw.println("f 19/21/5 17/22/5 18/23/5 20/24/5 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("vt -11.221 -5.68434e-014");
		pw.println("vt 14.779 -5.68434e-014");
		pw.println("vt 14.779 7.5");
		pw.println("vt -11.221 7.5");
		pw.println("f 14/25/1 19/26/1 20/27/1 15/28/1 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("vt 18.3131 14.779");
		pw.println("vn -1.46027e-016 -1 6.92729e-018");
		pw.println("vt 15.7428 12.2087");
		pw.println("vt 18.3131 -11.221");
		pw.println("f 17/29/6 4/30/6 13/31/6");
		pw.println("vt -42.9369 14.779");
		pw.println("f 4/30/6 17/29/6 19/32/6");
		pw.println("vt -40.7572 12.2087");
		pw.println("f 4/30/6 19/32/6 7/33/6");
		pw.println("vt -40.7572 -8.04128");
		pw.println("f 7/33/6 19/32/6 10/34/6");
		pw.println("vt 15.7428 -8.04128");
		pw.println("vt -42.9369 -11.221");
		pw.println("f 13/31/6 1/35/6 14/36/6");
		pw.println("f 1/35/6 13/31/6 4/30/6");
		pw.println("f 14/36/6 1/35/6 10/34/6");
		pw.println("f 14/36/6 10/34/6 19/32/6");
		pw.println();
		pw.println("usemtl Red");
		pw.println("vt 40.7572 -148.25");
		pw.println("vn -1.2576e-016 -3.06965e-033 1");
		pw.println("vt 40.7572 -7.4607e-014");
		pw.println("vt -15.7428 -5.68434e-014");
		pw.println("vt -15.7428 -148.25");
		pw.println("f 8/37/7 7/38/7 4/39/7 3/40/7 ");
		pw.println();
		pw.println("usemtl DarkTurquoise");
		pw.println("vt -18.3131 -11.221");
		pw.println("vn 1.7435e-016 1 1.78253e-016");
		pw.println("vt -15.7428 -8.04128");
		pw.println("vt -18.3131 14.779");
		pw.println("f 16/41/8 6/42/8 18/43/8");
		pw.println("vt 42.9369 -11.221");
		pw.println("f 6/42/8 16/41/8 15/44/8");
		pw.println("vt 40.7572 -8.04128");
		pw.println("f 6/42/8 15/44/8 11/45/8");
		pw.println("vt 40.7572 12.2087");
		pw.println("f 11/45/8 15/44/8 12/46/8");
		pw.println("vt -15.7428 12.2087");
		pw.println("vt 42.9369 14.779");
		pw.println("f 18/43/8 5/47/8 20/48/8");
		pw.println("f 5/47/8 18/43/8 6/42/8");
		pw.println("f 20/48/8 5/47/8 12/46/8");
		pw.println("f 20/48/8 12/46/8 15/44/8");
		pw.println();
		pw.println("usemtl Red");
		pw.println("vt 15.7428 -5.68434e-014");
		pw.println("vt 15.7428 7.5");
		pw.println("vt -40.7572 7.5");
		pw.println("vt -40.7572 -7.4607e-014");
		pw.println("f 1/49/5 6/50/5 11/51/5 10/52/5 ");
		pw.println();
		pw.println("usemtl Red");
		pw.println("vn -1.2576e-016 2.1843e-032 1");
		pw.println("vt 40.7572 7.5");
		pw.println("vt -15.7428 7.5");
		pw.println("f 7/38/9 12/53/9 5/54/9 4/39/9 ");
		pw.println();
		pw.println("usemtl DarkSlateGray");
		pw.println("vt 15.7428 12.2087");
		pw.println("vn 2.01216e-015 -1 -1.30747e-031");
		pw.println("vt -40.7572 12.2087");
		pw.println("vt -40.7572 -8.04128");
		pw.println("vt 15.7428 -8.04128");
		pw.println("f 3/55/10 8/56/10 9/57/10 2/58/10 ");
		pw.println();
		pw.println("usemtl Red");
		pw.println("vt 15.7428 -148.25");
		pw.println("vt -40.7572 -148.25");
		pw.println("f 2/59/5 1/49/5 10/52/5 9/60/5");

		return sw.toString();
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunOBJParserTestExamples();
	}

}
