package jp.co.sra.jun.obj;

import jp.co.sra.smalltalk.StObject;
import jp.co.sra.smalltalk.StSymbol;

/**
 * JunOBJScannerTable class
 * 
 *  @author    Albert Kerssies
 *  @created   2009/03/25 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJScannerTable.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJScannerTable extends StObject {

	protected static JunOBJScannerTable ScanningTable = null;

	/**
	 * Answer the scanner table for JunOBJScanner.
	 * 
	 * @return jp.co.sra.jun.obj.JunOBJScannerTable
	 * @category Table access
	 */
	public static JunOBJScannerTable ScannerTable() {
		if (ScanningTable == null) {
			JunOBJScannerTable newTable = new JunOBJScannerTable(256);

			for (int i = 1; i <= 127; i++) {
				newTable.at_put_(i, $("xBinary"));
			}
			newTable.at_put_(10, $("eolOrf"));
			newTable.at_put_(13, $("eolOrf"));
			newTable.at_put_(9, $("xSpace"));
			newTable.at_put_(32, $("xSpace"));
			for (int i = '0'; i <= '9'; i++) {
				newTable.at_put_(i, $("xDigit"));
			}
			for (int i = 'A'; i <= 'Z'; i++) {
				newTable.at_put_(i, $("xString"));
			}
			for (int i = 'a'; i <= 'z'; i++) {
				newTable.at_put_(i, $("xString"));
			}
			for (int i = 128; i <= 256; i++) {
				newTable.at_put_(i, $("xString"));
			}
			newTable.at_put_('+', $("xSign"));
			newTable.at_put_('-', $("xSign"));
			newTable.at_put_('#', $("xComment"));
			newTable.at_put_('/', $("slash"));
			newTable.at_put_('!', $("eolOrf"));

			ScanningTable = newTable;
		}
		return ScanningTable;
	}

	////////////////////////////////////////////////////////////////////////////////

	protected StSymbol[] _table;

	/**
	 * Create a new instance of JunOBJScannerTable and initialize it.
	 *
	 * @param size int
	 * @category Instance creation
	 */
	protected JunOBJScannerTable(int size) {
		_table = new StSymbol[size];
	}

	/**
	 * Answer the symbol at the specified index.
	 * 
	 * @param index int
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category accessing
	 */
	public StSymbol at_(int index) {
		if (257 <= index && index <= 65536) {
			if (Character.isLetter(index - 1)) {
				return $("xSymbol");
			} else {
				return $("xBinary");
			}
		} else {
			return _table[index - 1];
		}
	}

	/**
	 * Set the specified symbol at the specified index.
	 * 
	 * @param index int
	 * @param aSymbol jp.co.sra.smalltalk.StSymbol
	 * @category accessing
	 */
	public void at_put_(int index, StSymbol aSymbol) {
		_table[index - 1] = aSymbol;
	}

}
