package jp.co.sra.jun.obj.support;

import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StReadStream;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StCheckBoxMenuItem;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuItem;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.cursors.JunCursors;
import jp.co.sra.jun.goodies.files.JunFileModel;
import jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog;
import jp.co.sra.jun.obj.JunOBJParser;
import jp.co.sra.jun.obj.JunOBJWriter;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunOBJModel class
 * 
 *  @author    nisinaka
 *  @created   2009/03/27 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun734 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOBJModel.java,v 9.6 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOBJModel extends JunOpenGLDisplayModel {

	protected JunOpenGLTypicalObjects typicalObjects;

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		typicalObjects = null;
		this.windowLabel_($String("OBJ Viewer"));
	}

	/**
	 * Answer my current typical objects.
	 * 
	 * @return jp.co.sra.jun.obj.support.JunOBJModel.typicalObjects()
	 * @category accessing
	 */
	public JunOpenGLTypicalObjects typicalObjects() {
		if (typicalObjects == null) {
			typicalObjects = new JunOpenGLTypicalObjects(TypicalObjectMethods());
			typicalObjects.compute_(new StBlockClosure1<Void, JunOpenGL3dObject>() {
				public Void value(JunOpenGL3dObject anOpenGL3dObject) {
					JunOBJModel.this.displayObject_(anOpenGL3dObject);
					JunOBJModel.this.resetView();
					return null;
				}
			});
			typicalObjects.classToSpawn_(this.getClass());
		}
		return typicalObjects;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#noticeOfWindowClose(java.awt.event.WindowEvent)
	 * @category interface closing
	 */
	@Override
	public void noticeOfWindowClose(WindowEvent e) {
		super.noticeOfWindowClose(e);
		this.typicalObjects().closeRequest();
	}

	/**
	 * 
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Object clone() throws CloneNotSupportedException {
		// TODO Auto-generated method stub
		return super.clone();
	}

	/**
	 * Create a "File" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_createFileMenu()
	 * @category resources
	 */
	@Override
	protected StMenu _createFileMenu() {
		StMenu fileMenu = new StMenu($String("File"), $("fileMenu"));

		// New
		fileMenu.add(new StMenuItem($String("New"), new MenuPerformer(this, "newModel")));

		// Open...
		fileMenu.add(new StMenuItem($String("Open") + "...", new MenuPerformer(this, "openOBJ")));

		// Open as...
		StMenu openAsMenu = new StMenu($String("Open as..."));
		openAsMenu.add(new StMenuItem("LST...", new MenuPerformer(this, "openLST")));
		openAsMenu.add(new StMenuItem("VRML1.0...", new MenuPerformer(this, "openWRL10")));
		openAsMenu.add(new StMenuItem("VRML97...", new MenuPerformer(this, "openWRL97")));
		fileMenu.add(openAsMenu);

		//
		fileMenu.addSeparator();

		// Save...
		fileMenu.add(new StMenuItem($String("Save") + "...", $("saveMenu"), new MenuPerformer(this, "saveOBJ")));

		// Save as...
		StMenu saveAsMenu = new StMenu($String("Save as..."), $("saveAsMenu"));
		saveAsMenu.add(new StMenuItem("LST...", new MenuPerformer(this, "saveLST")));
		saveAsMenu.add(new StMenuItem("VRML1.0...", new MenuPerformer(this, "saveWRL10")));
		saveAsMenu.add(new StMenuItem("VRML97...", new MenuPerformer(this, "saveWRL97")));
		fileMenu.add(saveAsMenu);

		// Save as image...
		fileMenu.add(new StMenuItem($String("Save as image..."), $("saveAsImageMenu"), new MenuPerformer(this, "saveAsImage")));

		//
		fileMenu.addSeparator();

		// Quit
		fileMenu.add(new StMenuItem($String("Quit"), new MenuPerformer(this, "quitDoing")));

		return fileMenu;
	}

	/**
	 * Create a "Misc" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_createMiscMenu()
	 * @category resources
	 */
	@Override
	protected StMenu _createMiscMenu() {
		StMenu miscMenu = new StMenu($String("Misc"), $("miscMenu"));

		// Typical Objects
		miscMenu.add(new StMenuItem($String("Typical Objects"), new MenuPerformer(this, "openTypicalObjects")));
		miscMenu.addSeparator();

		// Texture
		StMenu textureMenu = new StMenu($String("Texture"), $("textureMenu"));
		textureMenu.add(new StMenuItem($String("From user..."), $("textureFromUserMenu"), new MenuPerformer(this, "textureFromUser")));
		textureMenu.add(new StMenuItem($String("From file..."), $("textureFromFileMenu"), new MenuPerformer(this, "textureFromFile")));
		textureMenu.add(new StMenuItem($String("Take away"), new MenuPerformer(this, "textureTakeAway")));
		textureMenu.addSeparator();
		textureMenu.add(new StCheckBoxMenuItem($String("Linear approx."), $("textureLinearMenu"), new MenuPerformer(this, "textureLinear")));
		textureMenu.add(new StCheckBoxMenuItem($String("Clamp"), $("textureClampMenu"), new MenuPerformer(this, "textureClamp")));
		textureMenu.add(new StCheckBoxMenuItem($String("Modulate"), $("textureModulateMenu"), new MenuPerformer(this, "textureModulate")));
		textureMenu.add(new StCheckBoxMenuItem($String("Mipmap"), $("textureMipmapMenu"), new MenuPerformer(this, "textureMipmap")));
		miscMenu.add(textureMenu);

		// Spawn
		miscMenu.add(new StMenuItem($String("Spawn"), $("spawnMenu"), new MenuPerformer(this, "spawnObject")));

		// Viewport
		miscMenu.add(new StMenuItem($String("Viewport"), $("viewportMenu"), new MenuPerformer(this, "spawnViewport")));

		// Bounds
		miscMenu.add(new StMenuItem($String("Bounds"), $("boundsMenu"), new MenuPerformer(this, "showBounds")));

		return miscMenu;
	}

	/**
	 * Menu message: Open an OBJ file.
	 * 
	 * @category menu messages
	 */
	public void openOBJ() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType($String("<1p> files", null, "OBJ"), new String[] { "*.obj", "*.OBJ" }) };
		File file = JunFileRequesterDialog.Request($String("Select an <1p> file.", null, "OBJ"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		JunOpenGL3dObject object = null;
		try {
			object = this.readFromOBJ_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
		if (object == null) {
			return;
		}

		this.displayObject_(object);
		this.resetView();

		if (this.showModel() != null) {
			this.showModel().resetView();
		}
	}

	/**
	 * Menu message: Save the display object to an OBJ file.
	 * 
	 * @category menu messages
	 */
	public void saveOBJ() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType($String("<1p> files", null, "OBJ"), new String[] { "*.obj", "*.OBJ" }) };
		File file = JunFileRequesterDialog.RequestNewFile($String("Input an <1p> file.", null, "OBJ"), new File(this.defaultBaseName() + ".obj"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		JunOpenGL3dObject object = this.displayObject();
		Jun3dBoundingBox box = object.boundingBox();
		if (box.center().isZero() == false) {
			Jun3dTransformation transformation = Jun3dTransformation.Translate_(Jun3dPoint.Zero().minus_(box.center()));
			object = object.transform_(transformation);
		}

		try {
			this.writeToOBJ_object_(file, object);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Menu message: Open the typical objects.
	 * 
	 * @category menu messages
	 */
	public void openTypicalObjects() {
		JunOpenGLTypicalObjects typicalObjects = this.typicalObjects();
		if (typicalObjects.getWindow() != null) {
			typicalObjects._windowExpandAndRaise();
		} else {
			Window aWindow = this.getWindow();
			if (aWindow == null) {
				typicalObjects.open();
			} else {
				StRectangle box = new StRectangle(aWindow.getBounds());
				typicalObjects.openIn_(new Rectangle(box.right(), box.top(), 200, box.height()));
			}
		}
	}

	/**
	 * Read the OBJ file and create a JunOpenGL3dObject.
	 * 
	 * @param aFile java.io.File
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @exception java.io.IOException
	 * @category reading
	 */
	public JunOpenGL3dObject readFromOBJ_(File aFile) throws IOException {
		if (aFile == null) {
			return null;
		}
		if (aFile.exists() == false) {
			return null;
		}

		StReadStream stream = new StReadStream(aFile);
		return this.loadFromOBJ_(stream);
	}

	/**
	 * Load an OBJ file and create a JunOpenGL3dObject.
	 * 
	 * @param aStream jp.co.sra.smalltalk.StReadStream
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category reading
	 */
	public JunOpenGL3dObject loadFromOBJ_(StReadStream aStream) {
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			return JunOBJParser.Parse_(aStream);

		} catch (SmalltalkException e) {
			JunDialog.Warn_("Failed loading.\n" + e.getMessage());
			return null;
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the OBJ file.
	 * JunOBJWriter may need a file for writing a material file, so pass the file to JunOBJWriter.
	 * 
	 * @param aFile java.io.File
	 * @param anObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @exception java.io.IOException
	 * @category writing
	 */
	public void writeToOBJ_object_(File aFile, JunOpenGL3dObject anObject) throws IOException {
		if (aFile == null) {
			return;
		}

		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			JunOBJWriter objWriter = new JunOBJWriter(aFile, anObject);
			objWriter.save();

		} finally {
			cursor._restore();
		}
	}

	/**
	 * Answer the methods for creating typical objects.
	 * 
	 * @return java.lang.reflect.Method[]
	 * @category Examples
	 */
	public static Method[] TypicalObjectMethods() {
		return JunOpenGLTypicalObjects.TypicalObjectMethods();
	}

}
