package jp.co.sra.jun.octree.basic;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.octree.editor.JunOctreeEditModel;
import jp.co.sra.jun.octree.nodes.JunOctreeNode;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOctree class
 * 
 *  @author    nisinaka
 *  @created   1999/12/21 (by nisinaka)
 *  @updated   2008/04/17 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOctree.java,v 9.11 2010/03/09 09:17:32 nisinaka Exp $
 */
public class JunOctree extends StObject {

	/** The node of the octree. */
	protected JunOctreeNode node = null;

	/** The bounding box of the octree. */
	protected Jun3dBoundingBox bounds = null;

	/**
	 * Create an instance of JunOctree and initialize it.
	 * 
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 * @category Instance creation
	 */
	public static JunOctree Bounds_(Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = new JunOctree();
		octree.setBounds_(aJun3dBoundingBox);
		return octree;
	}

	/**
	 * Create an instance of JunOctree and initialize it.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 * @category Instance creation
	 */
	public static JunOctree FromBlock_depth_bounds_(StBlockClosure1<Double, Jun3dPoint> aBlock, int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		octree.block_depth_(aBlock, anInteger);
		return octree;
	}

	/**
	 * Create an instance of JunOctree and initialize it.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param anInteger int
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 * @category Instance creation
	 * @deprecated since Jun701, use Generics.
	 */
	public static JunOctree FromBlock_depth_bounds_(StBlockClosure<?, ?, ?, ?> aBlock, int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		octree.block_depth_(aBlock, anInteger);
		return octree;
	}

	/**
	 * Create a unit JunOctree.
	 * 
	 * @return jp.co.sra.jun.octree.basic.JunOctree
	 * @category Constants
	 */
	public static JunOctree Unity() {
		return JunOctree.Bounds_(Jun3dBoundingBox.Origin_corner_(Jun3dPoint.Zero(), Jun3dPoint.Unity()));
	}

	/**
	 * Convert to JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		final ArrayList<JunOpenGL3dObject> components = new ArrayList<JunOpenGL3dObject>();
		this.originExtentTipsDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
			public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode tip) {
				JunOpenGL3dObject component = tip.asJunOpenGL3dObject();
				if (component != null) {
					components.add(component.scaledBy_(extent).translatedBy_(origin));
				}
				return null;
			}
		});

		return new JunOpenGL3dCompoundObject(components);
	}

	/**
	 * Make the octree to be compact.
	 * 
	 * @category functions
	 */
	public void beCompact() {
		node.beCompact();
	}

	/**
	 * Set the block and the accuracy.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aNumber double
	 * @category functions
	 */
	public void block_accuracy_(StBlockClosure1<Double, Jun3dPoint> aBlock, double aNumber) {
		node.origin_extent_accuracy_block_(this.origin(), this.extent(), aNumber, aBlock);
	}

	/**
	 * Set the block and the accuracy.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aNumber double
	 * @category functions
	 * @deprecated since Jun701, use Generics.
	 */
	public void block_accuracy_(final StBlockClosure<?, ?, ?, ?> aBlock, double aNumber) {
		this.block_accuracy_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint aPoint) {
				return (Double) aBlock.value_(aPoint);
			}
		}, aNumber);
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param anInteger int
	 * @category functions
	 */
	public void block_depth_(StBlockClosure1<Double, Jun3dPoint> aBlock, int anInteger) {
		node.origin_extent_depth_block_(this.origin(), this.extent(), anInteger, aBlock);
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param anInteger int
	 * @category functions
	 * @deprecated since Jun701, use Generics.
	 */
	public void block_depth_(final StBlockClosure<?, ?, ?, ?> aBlock, int anInteger) {
		this.block_depth_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint aPoint) {
				return (Double) aBlock.value_(aPoint);
			}
		}, anInteger);
	}

	/**
	 * Answer the bounds.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @category bounds access
	 */
	public Jun3dBoundingBox bounds() {
		return bounds;
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aNumber double
	 * @category functions
	 */
	public void brepsBlock_accuracy_(StBlockClosure1<Double, Jun3dPoint> aBlock, double aNumber) {
		node.brepsOrigin_extent_accuracy_block_(this.origin(), this.extent(), aNumber, aBlock);
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aNumber double
	 * @category functions
	 * @deprecated since Jun701, use Generics.
	 */
	public void brepsBlock_accuracy_(final StBlockClosure<?, ?, ?, ?> aBlock, double aNumber) {
		this.brepsBlock_accuracy_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint aPoint) {
				return (Double) aBlock.value_(aPoint);
			}
		}, aNumber);
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param anInteger int
	 * @category functions
	 */
	public void brepsBlock_depth_(StBlockClosure1<Double, Jun3dPoint> aBlock, int anInteger) {
		node.brepsOrigin_extent_depth_block_(this.origin(), this.extent(), anInteger, aBlock);
	}

	/**
	 * Set the block and the depth.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param anInteger int
	 * @category functions
	 * @deprecated since Jun701, use Generics.
	 */
	public void brepsBlock_depth_(final StBlockClosure<?, ?, ?, ?> aBlock, int anInteger) {
		this.brepsBlock_depth_(new StBlockClosure1<Double, Jun3dPoint>() {
			public Double value(Jun3dPoint aPoint) {
				return (Double) aBlock.value_(aPoint);
			}
		}, anInteger);
	}

	/**
	 * Answer the center point of the octree.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category bounds access
	 */
	public Jun3dPoint center() {
		return this.bounds().center();
	}

	/**
	 * Answer true if the octree contains the Jun3dPoint.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return boolean
	 * @category testing
	 */
	public boolean containsPoint_(Jun3dPoint aJun3dPoint) {
		return this.bounds().containsPoint_(aJun3dPoint) && this.tipForX_y_z_(aJun3dPoint.x(), aJun3dPoint.y(), aJun3dPoint.z()).containsPoint_(aJun3dPoint);
	}

	/**
	 * Answer the corner point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category bounds access
	 */
	public Jun3dPoint corner() {
		return this.bounds().corner();
	}

	/**
	 * Answer the depth of the octree.
	 * 
	 * @return double
	 * @category bounds access
	 */
	public double depth() {
		return this.bounds().depth();
	}

	/**
	 * Divide the node at the specified point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category functions
	 */
	public void divideTipForPoint_(Jun3dPoint aJun3dPoint) {
		this.tipForX_y_z_(aJun3dPoint.x(), aJun3dPoint.y(), aJun3dPoint.z()).beDivided();
	}

	/**
	 * Edit the JunOctree.
	 * 
	 * @category utilities
	 */
	public void edit() {
		JunOctreeEditModel.Edit_(this);
	}

	/**
	 * Empty the node at the specified point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category functions
	 */
	public void emptyTipForPoint_(Jun3dPoint aJun3dPoint) {
		this.tipForX_y_z_(aJun3dPoint.x(), aJun3dPoint.y(), aJun3dPoint.z()).beEmpty();
	}

	/**
	 * Answer the extent.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category bounds access
	 */
	public Jun3dPoint extent() {
		return this.bounds().extent();
	}

	/**
	 * Fill the node at the specified point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category functions
	 */
	public void fillTipForPoint_(Jun3dPoint aJun3dPoint) {
		this.tipForX_y_z_(aJun3dPoint.x(), aJun3dPoint.y(), aJun3dPoint.z()).beFilled();
	}

	/**
	 * Answer the height of the octree.
	 * 
	 * @return double
	 * @category bounds access
	 */
	public double height() {
		return this.bounds().height();
	}

	/**
	 * Answer the origin point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category bounds access
	 */
	public Jun3dPoint origin() {
		return this.bounds().origin();
	}

	/**
	 * Enumerate the nodes and evaluate the block closure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure3
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R originExtentNodesDo_(StBlockClosure3<R, Jun3dPoint, Jun3dPoint, JunOctreeNode> aBlock) {
		return node.origin_extent_nodesDo_(this.origin(), this.extent(), aBlock);
	}

	/**
	 * Enumerate the nodes and evaluate the block closure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object originExtentNodesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.originExtentNodesDo_(new StBlockClosure3<Object, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
			public Object value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
				return aBlock.value_value_value_(origin, extent, node);
			}
		});
	}

	/**
	 * Enumerate all nodes except aJunOctreeNode and evaluate the block.
	 * 
	 * @param aJunOctreeNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category private
	 */
	public Object originExtentNodesWithout_do_(JunOctreeNode aJunOctreeNode, StBlockClosure3<Object, Jun3dPoint, Jun3dPoint, JunOctreeNode> aBlock) {
		return node.origin_extent_nodesWithout_do_(this.origin(), this.extent(), aJunOctreeNode, aBlock);
	}

	/**
	 * Enumerate all nodes except aJunOctreeNode and evaluate the block.
	 * 
	 * @param aJunOctreeNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category private
	 * @deprecated since Jun701, use Generics.
	 */
	public Object originExtentNodesWithout_do_(JunOctreeNode aJunOctreeNode, final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.originExtentNodesWithout_do_(aJunOctreeNode, new StBlockClosure3<Object, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
			public Object value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
				return aBlock.value_value_value_(origin, extent, node);
			}
		});
	}

	/**
	 * Enumerate all nodes except aCollectionOfJunOctreeNode and evaluate the block.
	 * 
	 * @param aCollectionOfJunOctreeNode java.util.Collection
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category private
	 */
	public <R> R originExtentNodesWithoutAll_do_(Collection<JunOctreeNode> aCollectionOfJunOctreeNode, StBlockClosure3<R, Jun3dPoint, Jun3dPoint, JunOctreeNode> aBlock) {
		return node.origin_extent_nodesWithoutAll_do_(this.origin(), this.extent(), aCollectionOfJunOctreeNode, aBlock);
	}

	/**
	 * Enumerate all nodes except aCollectionOfJunOctreeNode and evaluate the block.
	 * 
	 * @param aCollectionOfJunOctreeNode java.util.Collection
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category private
	 * @deprecated since Jun701, use Generics.
	 */
	public Object originExtentNodesWithoutAll_do_(Collection<JunOctreeNode> aCollectionOfJunOctreeNode, final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.originExtentNodesWithoutAll_do_(aCollectionOfJunOctreeNode, new StBlockClosure3<Object, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
			public Object value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
				return aBlock.value_value_value_(origin, extent, node);
			}
		});
	}

	/**
	 * Enumerate the tips and evaluate the block closure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R originExtentTipsDo_(StBlockClosure3<R, Jun3dPoint, Jun3dPoint, JunOctreeNode> aBlock) {
		return node.origin_extent_tipsDo_(this.origin(), this.extent(), aBlock);
	}

	/**
	 * Enumerate the tips and evaluate the block closure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object originExtentTipsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.originExtentTipsDo_(new StBlockClosure3<Object, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
			public Object value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
				return aBlock.value_value_value_(origin, extent, node);
			}
		});
	}

	/**
	 * Set the node.
	 * 
	 * @param aJunOctreeNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category private
	 */
	public void setNode_(JunOctreeNode aJunOctreeNode) {
		node = aJunOctreeNode;
	}

	/**
	 * Show the JunOctree as a JunOpenGL3dObject.
	 * 
	 * @category utilities
	 */
	public void show() {
		JunOpenGL3dObject anObject = this.asJunOpenGL3dObject();
		anObject.paint_(Color.white);
		anObject.show();
	}

	/**
	 * Answer the width of the octree.
	 * 
	 * @return double
	 * @category bounds access
	 */
	public double width() {
		return this.bounds().width();
	}

	/**
	 * Create a new node.
	 * 
	 * @return jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category private
	 */
	protected JunOctreeNode _newNode() {
		return JunOctreeNode.Filled();
	}

	/**
	 * Set the bounds.
	 * 
	 * @param aJun3dBoundingBox jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @category private
	 */
	protected void setBounds_(Jun3dBoundingBox aJun3dBoundingBox) {
		node = this._newNode();
		bounds = aJun3dBoundingBox;
	}

	/**
	 * Answer the node at the specified point.
	 * 
	 * @param x double
	 * @param y double
	 * @param z double
	 * @return jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category private
	 */
	protected JunOctreeNode tipForX_y_z_(double x, double y, double z) {
		JunOctreeNode n = node;

		while (n.isTip() == false) {
			int childIndex = 0;

			if (x >= 0.5d) {
				childIndex++;
				x -= 0.5d;
			}

			x *= 2.0d;

			if (y >= 0.5d) {
				childIndex += 2;
				y -= 0.5d;
			}

			y *= 2.0d;

			if (z >= 0.5d) {
				childIndex += 4;
				z -= 0.5d;
			}

			z *= 2.0d;
			n = n.childAt_(childIndex);
		}

		return n;
	}
}
