package jp.co.sra.jun.octree.editor;

import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.surfaces.JunPlane;
import jp.co.sra.jun.geometry.surfaces.JunSphereSurface;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.metaball.abstracts.JunMetaball;
import jp.co.sra.jun.metaball.atoms.JunMetaSphere;
import jp.co.sra.jun.metaball.solid.JunMetaballSolid;
import jp.co.sra.jun.octree.basic.JunOctree;
import jp.co.sra.jun.octree.nodes.JunOctreeNode;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.opengl.lights.JunOpenGLLight;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.picking.JunOpenGLObjectPicker;
import jp.co.sra.jun.opengl.picking.JunOpenGLPickingRenderingContext;
import jp.co.sra.jun.opengl.projection.JunOpenGLProjection;
import jp.co.sra.jun.opengl.support.JunOpenGLDrawable;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderer;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext;
import jp.co.sra.jun.system.framework.JunApplicationModel;
import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunOctreeEditModel class
 * 
 *  @author    nisinaka
 *  @created   1999/12/28 (by nisinaka)
 *  @updated   2005/03/02 (by nisinaka)
 *  @updated   2007/08/24 (by nisinaka)
 *  @updated   2008/05/27 (by nisinaka)
 *  @updated   2008/11/26 (by m-asada)
 *  @version   758 (with StPL9.7) based on Jun710 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOctreeEditModel.java,v 9.13 2010/03/09 09:17:33 nisinaka Exp $
 */
public class JunOctreeEditModel extends JunOpenGLDisplayModel {

	public static final double ThetaAccuracy = 0.001;

	/** The octrees to be edited. */
	protected JunOctree[] octrees;

	/** Selected octree nodes. */
	protected ArrayList<JunOctreeNode> selection;

	/**
	 * Create a new instance of <code>JunOctreeEditModel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOctreeEditModel() {
		super();
	}

	/**
	 * Create a new instance of <code>JunOctreeEditModel</code> and initialize it.
	 * 
	 * @param aJunOctree jp.co.sra.jun.octree.basic.JunOctree
	 * @category Instance creation
	 */
	public JunOctreeEditModel(JunOctree aJunOctree) {
		this();
		ArrayList<JunOctree> octrees = new ArrayList<JunOctree>(1);
		octrees.add(aJunOctree);
		this.setOctrees_(octrees);
	}

	/**
	 * Create a new instance of JunOctreeEditModel with the specified JunOctree and open it.
	 * 
	 * @param aJunOctree jp.co.sra.jun.octree.basic.JunOctree
	 * @return jp.co.sra.jun.octree.editor.JunOctreeEditModel
	 * @category Instance creation
	 */
	public static JunOctreeEditModel Edit_(JunOctree aJunOctree) {
		JunOctreeEditModel editModel = new JunOctreeEditModel(aJunOctree);
		editModel.open();

		return editModel;
	}

	/**
	 * Create a new instance of JunOctreeEditModel with the specified JunOctree.
	 * 
	 * @param aJunOctree jp.co.sra.jun.octree.basic.JunOctree
	 * @return jp.co.sra.jun.octree.editor.JunOctreeEditModel
	 * @category Instance creation
	 * @deprecated since Jun713
	 */
	public static JunOctreeEditModel On_(JunOctree aJunOctree) {
		return new JunOctreeEditModel(aJunOctree);
	}

	/**
	 * Add a selected node.
	 * 
	 * @param aJunOctreeNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category selection
	 */
	public void addSelection_(JunOctreeNode aJunOctreeNode) {
		if (selection.contains(aJunOctreeNode) == false) {
			selection.add(aJunOctreeNode);
		}
	}

	/**
	 * Called when a node is picked with the alt key down.
	 * 
	 * @param pickedNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category manipulating
	 */
	public void altClick_(JunOctreeNode pickedNode) {
		// do nothing.
	}

	/**
	 * Answer the bounding box of the octrees.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#boundingBox()
	 * @category bounds accessing
	 */
	@Override
	public Jun3dBoundingBox boundingBox() {
		if (this.hasOctree() == false) {
			return Jun3dBoundingBox.Origin_corner_(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 0));
		} else {
			Jun3dBoundingBox bounds = octrees[0].bounds();
			for (int i = 1; i < octrees.length; i++) {
				bounds = bounds.merge_(octrees[i].bounds());
			}
			return bounds;
		}
	}

	/**
	 * Clear the selected nodes.
	 * 
	 * @category selection
	 */
	public void clearSelection() {
		selection = new ArrayList<JunOctreeNode>();
	}

	/**
	 * Called when a node is picked with the control key down.
	 * 
	 * @param pickedNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category manipulating
	 */
	public void ctrlClick_(JunOctreeNode pickedNode) {
		// do nothing.
	}

	/**
	 * Answer the default eye point.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.Jun3dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultEyePoint()
	 * @category defaults
	 */
	@Override
	public Jun3dPoint defaultEyePoint() {
		if (this.hasOctree() == false) {
			return new Jun3dPoint(10000, 10000, 10000);
		}

		Jun3dPoint defaultEyePoint = (Jun3dPoint) this.defaultProjectionTable().get($("eyePoint"));
		if (defaultEyePoint != null) {
			return defaultEyePoint;
		} else {
			Jun3dBoundingBox box = this.boundingBox();
			double distance = box.origin().distance_(box.corner());
			distance = distance * 2;
			return new Jun3dPoint(distance, distance, distance);
		}
	}

	/**
	 * Answer the default sight point.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.Jun3dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#defaultSightPoint()
	 * @category defaults
	 */
	@Override
	public Jun3dPoint defaultSightPoint() {
		if (this.hasOctree() == false) {
			return new Jun3dPoint(0, 0, 0);
		}

		Jun3dPoint defaultSightPoint = (Jun3dPoint) this.defaultProjectionTable().get($("sightPoint"));
		if (defaultSightPoint != null) {
			return defaultSightPoint;
		} else {
			return this.boundingBox().center();
		}
	}

	/**
	 * Divide the selected nodes.
	 * 
	 * @category menu messages
	 */
	public void divideSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = selection.toArray(new JunOctreeNode[selection.size()]);

			ArrayList<JunOctreeNode> newSelection = new ArrayList<JunOctreeNode>(nodes.length * 8);

			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beDivided();

				JunOctreeNode[] moreNodes = (JunOctreeNode[]) nodes[i].contents();

				for (int j = 0; j < moreNodes.length; j++) {
					if (newSelection.contains(moreNodes[j]) == false) {
						newSelection.add(moreNodes[j]);
					}
				}
			}

			selection = newSelection;
			this.changed_($("object"));
		}
	}

	/**
	 * Erase the selected nodes.
	 * 
	 * @category menu messages
	 */
	public void eraseSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = selection.toArray(new JunOctreeNode[selection.size()]);
			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beEmpty();
			}
			this.changed_($("object"));
		}
	}

	/**
	 * Fill the selected nodes.
	 * 
	 * @category menu messages
	 */
	public void fillSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = selection.toArray(new JunOctreeNode[selection.size()]);
			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beFilled();
			}
			this.changed_($("object"));
		}
	}

	/**
	 * Fit the projector to display the entire objects.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#fit()
	 * @category projection
	 */
	@Override
	public void fit() {
		if (this.hasOctree()) {
			this.fitSilently();
			this.changed_($("projection"));
		}
	}

	/**
	 * Fit the sight point to the current view.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#fitSight()
	 * @category projection
	 */
	@Override
	public void fitSight() {
		if (this.hasOctree()) {
			Jun3dPoint sightPoint = this.computeSightPoint();
			this.sightPoint_(sightPoint);
		}
	}

	/**
	 * Fit the 3d object to the current view.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#fitSilently()
	 * @category projection
	 */
	@Override
	public void fitSilently() {
		if (this.hasOctree()) {
			Jun3dPoint sightPoint = this.computeSightPoint();
			this.displayProjector().sightPoint_(sightPoint);

			double zoomHeight = this.computeZoomHeight();
			this.displayProjector().zoomHeight_(zoomHeight);
		}
	}

	/**
	 * Fit the zoom height.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#fitZoom()
	 * @category projection
	 */
	@Override
	public void fitZoom() {
		if (this.hasOctree()) {
			double zoomHeight = this.computeZoomHeight();
			this.zoomHeight_(zoomHeight);
		}
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param deltaPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param shiftDown boolean
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#grab_(jp.co.sra.jun.geometry.basic.Jun2dPoint, boolean)
	 * @category manipulating
	 */
	@Override
	public void grab_(Jun2dPoint deltaPoint, boolean shiftDown) {
		if (this.hasOctree()) {
			Jun2dPoint fromPoint = new Jun2dPoint(0, 0);
			Jun2dPoint toPoint;

			if (shiftDown) {
				toPoint = new Jun2dPoint(deltaPoint.x() / 10, deltaPoint.y() / 10);
			} else {
				toPoint = new Jun2dPoint(deltaPoint.x(), deltaPoint.y());
			}

			this.grab_xy_(fromPoint, toPoint);
		}
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param from2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param to2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#grab_xy_(jp.co.sra.jun.geometry.basic.Jun2dPoint, jp.co.sra.jun.geometry.basic.Jun2dPoint)
	 * @category manipulating
	 */
	@Override
	public void grab_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			JunSphereSurface grabSphere = this.grabSphere();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			JunAngle rotationAngle = (new Jun3dLine(grabSphere.center(), toPoint)).angleWithLine_(new Jun3dLine(grabSphere.center(), fromPoint));

			if (Math.abs(rotationAngle.rad()) > ThetaAccuracy) {
				Jun3dLine rotationAxis = new Jun3dLine(grabSphere.center(), grabSphere.center().minus_(fromPoint.minus_(grabSphere.center()).product_(toPoint.minus_(grabSphere.center()))));
				Jun3dTransformation transformation = Jun3dTransformation.Rotate_around_(rotationAngle, rotationAxis);
				Jun3dTransformation transformationInv = Jun3dTransformation.Rotate_around_(rotationAngle.mul_(-1), rotationAxis);
				Jun3dPoint upPoint = aProjection.sightPoint().plus_(aProjection.unitUpVector());
				aProjection.eyePoint_(transformationInv.applyTo_(aProjection.eyePoint()));
				aProjection.upVector_(transformation.applyTo_(upPoint).minus_(aProjection.sightPoint()));
				this.displayProjection_(aProjection);
				this.changed_($("projection"));
			}
		}
	}

	/**
	 * Answer <code>true</code> if the receiver has octrees, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasOctree() {
		return (octrees != null && octrees.length > 0);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();

		octrees = new JunOctree[0];
		selection = new ArrayList<JunOctreeNode>();
	}

	/**
	 * Focus on the specified area.
	 * 
	 * @param from2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param to2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#look_xy_(jp.co.sra.jun.geometry.basic.Jun2dPoint, jp.co.sra.jun.geometry.basic.Jun2dPoint)
	 * @category manipulating
	 */
	@Override
	public void look_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			aProjection.eyePoint_(aProjection.eyePoint().minus_(toPoint.minus_(fromPoint)));
			this.displayProjection_(aProjection);
			this.changed_($("projection"));
		}
	}

	/**
	 * Specify an accuracy and compute a melted octree.
	 * 
	 * @category menu messages
	 */
	public void meltWithAccuracy() {
		if (this.hasOctree()) {
			String accuracyString = JunDialog.Request_($String("Accuracy?"), "1.0d");
			if (accuracyString == null || accuracyString.length() == 0) {
				return;
			}

			double accuracy = Double.valueOf(accuracyString).doubleValue();
			if (accuracy < (1.0d - 8)) {
				return;
			}

			this.meltWithAccuracy_(accuracy);
			this.changed_($("object"));
		}
	}

	/**
	 * Specify a depth and compute a melted octree.
	 * 
	 * @category menu messages
	 */
	public void meltWithDepth() {
		if (this.hasOctree()) {
			String depthString = JunDialog.Request_($String("Depth?"), "1");
			if (depthString == null || depthString.length() == 0) {
				return;
			}

			int depth = Math.round(Float.valueOf(depthString).floatValue());
			if (depth < 1) {
				return;
			}

			this.meltWithDepth_(depth);
			this.changed_($("object"));
		}
	}

	/**
	 * Create new model and open it.
	 * 
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#newModel()
	 * @category menu messages
	 */
	@Override
	public JunApplicationModel newModel() {
		try {
			JunOctree newOctree = (JunOctree) _PerformWith(octrees[0].getClass(), "Bounds_", Jun3dBoundingBox.Origin_extent_(Jun3dPoint.Zero(), Jun3dPoint.Unity()));
			return JunOctreeEditModel.Edit_(newOctree);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Create an application window of the default view and open it.
	 * 
	 * @return java.awt.Frame
	 * @see jp.co.sra.smalltalk.StApplicationModel#open()
	 * @category interface opening
	 */
	@Override
	public Frame open() {
		this.flushProjector();
		this.fitSilently();
		this.updateMenuIndication();

		return super.open();
	}

	/**
	 * Pick a JunOctreeNode at the specified point on the view.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint,MouseEvent
	 * @param event DOCUMENT ME!
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#pick_with_(jp.co.sra.jun.geometry.basic.Jun2dPoint, java.awt.event.MouseEvent)
	 * @category manipulating
	 */
	@Override
	public JunOpenGL3dObject pick_with_(Jun2dPoint aPoint, MouseEvent event) {
		if (this.hasOctree()) {
			JunOctreeNode pickedTip = this.pickTipAt_(aPoint);

			if (pickedTip == null) {
				if (selection.isEmpty() == false) {
					this.clearSelection();
					this.changed_($("selection"));
				}
			} else {
				if (event.isShiftDown()) {
					if (selection.contains(pickedTip)) {
						this.removeSelection_(pickedTip);
						this.changed_($("selection"));
					} else {
						this.addSelection_(pickedTip);
						this.changed_($("selection"));
					}
				} else {
					if (event.isControlDown()) {
						this.ctrlClick_(pickedTip);
					} else if (event.isAltDown()) {
						this.altClick_(pickedTip);
					} else {
						this.clearSelection();
						this.addSelection_(pickedTip);
						this.changed_($("selection"));
					}
				}
			}
		}

		return null;
	}

	/**
	 * Remove the node from the currently selected nodes.
	 * 
	 * @param aJunOctreeNode jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category selection
	 */
	public void removeSelection_(JunOctreeNode aJunOctreeNode) {
		selection.remove(aJunOctreeNode);
	}

	/**
	 * Render the octrees on the rendering context.
	 * 
	 * @param renderingContext jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#renderOn_(jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext)
	 * @category displaying
	 */
	@Override
	public void renderOn_(final JunOpenGLRenderingContext renderingContext) {
		renderingContext.clear();
		renderingContext.cullFaceBack();

		JunOpenGLLight[] lights = this.displayLightCollection();

		if (this.hasOctree() == false) {
			this.displayProjector().projectOn_(renderingContext);
		} else {
			this.displayProjector().project_withLights_on_(null, lights, renderingContext);
			renderingContext.paint_(Color.black);
			renderingContext.lineWidth_(4);

			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentNodesDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
					public Void value(Jun3dPoint origin, Jun3dPoint extent, final JunOctreeNode node) {
						if (selection.contains(node)) {
							renderingContext.productTransformation_while_(Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure0<Void>() {
								public Void value() {
									node.renderWireframeOn_(renderingContext);
									return null;
								}
							});
						}

						return null;
					}
				});
			}

			renderingContext.pointSize_(5);

			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentTipsDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
					public Void value(Jun3dPoint origin, Jun3dPoint extent, final JunOctreeNode tip) {
						renderingContext.productTransformation_while_(Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure0<Void>() {
							public Void value() {
								if (tip.isEmpty() == false) {
									renderingContext.paint_(Color.white);
									tip.renderPolygonOn_(renderingContext);
								} else {
									renderingContext.paint_(Color.black);
									tip.renderVertexOn_(renderingContext);
								}
								return null;
							}
						});

						return null;
					}
				});
			}

			renderingContext.flush();
		}
	}

	/**
	 * Select all nodes.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#selectAll()
	 * @category menu messages
	 */
	@Override
	public void selectAll() {
		this.clearSelection();

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentTipsDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
				public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode tip) {
					JunOctreeEditModel.this.addSelection_(tip);
					return null;
				}
			});
		}

		this.changed_($("selection"));
		this.updateMenuIndication();
	}

	/**
	 * Simplify the entire octree.
	 * 
	 * @category menu messages
	 */
	public void simplify() {
		if (this.hasOctree()) {
			for (int i = 0; i < octrees.length; i++) {
				octrees[i].beCompact();
			}
		}

		this.clearSelection();
		this.changed_($("object"));
	}

	/**
	 * Slide the projection.
	 * 
	 * @param from2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param to2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#slide_xy_(jp.co.sra.jun.geometry.basic.Jun2dPoint, jp.co.sra.jun.geometry.basic.Jun2dPoint)
	 * @category manipulating
	 */
	@Override
	public void slide_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			aProjection.sightPoint_(aProjection.sightPoint().minus_(toPoint.minus_(fromPoint)));
			aProjection.eyePoint_(aProjection.eyePoint().minus_(toPoint.minus_(fromPoint)));
			this.displayProjection_(aProjection);
			this.changed_($("projection"));
		}
	}

	/**
	 * Spawning the JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#spawningObject()
	 * @category menu messages
	 */
	@Override
	public JunOpenGL3dObject spawningObject() {
		if (this.hasOctree()) {
			ArrayList<JunOpenGL3dObject> objects = new ArrayList<JunOpenGL3dObject>();
			for (int i = 0; i < octrees.length; i++) {
				objects.add(octrees[i].asJunOpenGL3dObject());
			}

			return new JunOpenGL3dCompoundObject(objects);
		} else {
			return null;
		}
	}

	/**
	 * Spawn as an OpenGL object.
	 * 
	 * @category menu messages
	 */
	public void spawnOpenGLObject() {
		JunOpenGLDisplayModel displayModel = this.spawningObject().show();
		displayModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
		displayModel.changed_($("object"));
	}

	/**
	 * Spawn the selected nodes.
	 * 
	 * @category menu messages
	 */
	public void spawnSelection() {
		if (selection.isEmpty() == false) {
			final ArrayList<JunOctree> spawningOctrees = new ArrayList<JunOctree>(selection.size());
			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentNodesDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
					public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
						if (selection.contains(node)) {
							Jun3dBoundingBox bbox = Jun3dBoundingBox.Origin_extent_(origin, extent);
							JunOctree newOctree = JunOctree.Bounds_(bbox);
							spawningOctrees.add(newOctree);
						}

						return null;
					}
				});
			}

			JunOctreeEditModel editModel = new JunOctreeEditModel();
			editModel.open();
			editModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
			editModel.setOctrees_(spawningOctrees);
			editModel.changed_($("object"));
		}
	}

	/**
	 * Unselect all nodes.
	 * 
	 * @category menu messages
	 */
	public void unselectAll() {
		this.clearSelection();
		this.changed_($("selection"));
	}

	/**
	 * Compute the current zoom height.
	 * 
	 * @return double
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#computeZoomHeight()
	 * @category private
	 */
	@Override
	protected double computeZoomHeight() {
		double depth = this.displayProjection().distance();
		Jun3dPoint up = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, -1), depth);
		Jun3dPoint down = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, 1), depth);
		Jun3dPoint right = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(1, 0), depth);
		Jun3dPoint left = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(-1, 0), depth);
		final JunPlane horizontal = new JunPlane(this.displayProjection().eyePoint(), left, right);
		final JunPlane vertical = new JunPlane(this.displayProjection().eyePoint(), up, down);

		double maxOffsetX = 0;
		double maxOffsetY = 0;
		double maxHeight = 0;
		if (this.hasOctree()) {
			for (int i = 0; i < octrees.length; i++) {
				JunOctree octree = octrees[i];
				Jun3dPoint origin = octree.origin();
				Jun3dPoint corner = octree.corner();
				Jun3dPoint[] points = new Jun3dPoint[8];
				points[0] = new Jun3dPoint(origin.x(), origin.y(), origin.z());
				points[1] = new Jun3dPoint(corner.x(), origin.y(), origin.z());
				points[2] = new Jun3dPoint(origin.x(), corner.y(), origin.z());
				points[3] = new Jun3dPoint(corner.x(), corner.y(), origin.z());
				points[4] = new Jun3dPoint(origin.x(), origin.y(), corner.z());
				points[5] = new Jun3dPoint(corner.x(), origin.y(), corner.z());
				points[6] = new Jun3dPoint(origin.x(), corner.y(), corner.z());
				points[7] = new Jun3dPoint(corner.x(), corner.y(), corner.z());

				for (int j = 0; j < points.length; j++) {
					double offsetX = vertical.distanceFromPoint_(points[j]);
					double offsetY = horizontal.distanceFromPoint_(points[j]);
					if (offsetX > maxOffsetX) {
						maxOffsetX = offsetX;
					}
					if (offsetY > maxOffsetY) {
						maxOffsetY = offsetY;
					}
				}
			}
		}

		double factor = 2.5;
		if (maxOffsetX > maxOffsetY) {
			maxHeight = maxOffsetX * factor;
		} else {
			maxHeight = maxOffsetY * factor;
		}

		return maxHeight;
	}

	/**
	 * Specify an accuracy and compute a melted octree.
	 * 
	 * @param accuracy double
	 * @category private-functions
	 */
	protected void meltWithAccuracy_(final double accuracy) {
		final ArrayList<JunOctreeNode> meltNodes = new ArrayList<JunOctreeNode>(selection);
		final JunMetaballSolid metaballSolid = this.metaballSolidOrder_withoutAll_(4, meltNodes);

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
				public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
					if (meltNodes.contains(node)) {
						node.origin_extent_accuracy_block_(origin, extent, accuracy, new StBlockClosure1<Double, Jun3dPoint>() {
							public Double value(Jun3dPoint aPoint) {
								return new Double(metaballSolid.valueAt_(aPoint));
							}
						});
					}

					return null;
				}
			});
		}
	}

	/**
	 * Specify a depth and compute a melted octree.
	 * 
	 * @param depth double
	 * @category private-functions
	 */
	protected void meltWithDepth_(final int depth) {
		final ArrayList<JunOctreeNode> meltNodes = new ArrayList<JunOctreeNode>(selection);
		final JunMetaballSolid metaballSolid = this.metaballSolidOrder_withoutAll_(3, meltNodes);

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
				public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
					if (meltNodes.contains(node)) {
						node.origin_extent_depth_block_(origin, extent, depth + 1, new StBlockClosure1<Double, Jun3dPoint>() {
							public Double value(Jun3dPoint aPoint) {
								return new Double(metaballSolid.valueAt_(aPoint));
							}
						});
					}

					return null;
				}
			});
		}
	}

	/**
	 * Create a new JunMetaballSolid.
	 * 
	 * @param order int
	 * @param aCollectionOfJunOctreeNode java.util.Collection
	 * @return jp.co.sra.jun.metaball.solid.JunMetaballSolid
	 * @category private-functions
	 */
	protected JunMetaballSolid metaballSolidOrder_withoutAll_(final int order, Collection<JunOctreeNode> aCollectionOfJunOctreeNode) {
		final JunMetaballSolid metaballSolid = new JunMetaballSolid();

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesWithoutAll_do_(aCollectionOfJunOctreeNode, new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
				public Void value(Jun3dPoint origin, Jun3dPoint extent, JunOctreeNode node) {
					if (node.isTip()) {
						double r = (extent.x() * extent.y() * extent.z() * 3.0d) / 4.0d / Math.PI;
						r = Math.pow(r, 1.0d / 3.0d);

						double sign = node.isFilled() ? 1.0d : -1.0d;
						double weight = r * sign;
						JunMetaball metaball = JunMetaSphere.Center_order_weight_(extent, order, weight);
						metaballSolid.add_(metaball);
					}

					return null;
				}
			});
		}

		return metaballSolid;
	}

	/**
	 * Pick a JunOctreeNode at the specified point.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category private
	 */
	protected JunOctreeNode pickTipAt_(Jun2dPoint aPoint) {
		Jun3dPoint sightPoint = this.displayProjection().translateTo3dPointFromPoint_(aPoint);
		double zoomHeight = sightPoint.minus_(this.displayProjection().translateTo3dPointFromPoint_(aPoint.plus_(new Jun2dPoint(0, -0.01d)))).length();
		JunOpenGLProjection projection = (JunOpenGLProjection) this.displayProjection().copy();
		projection.sightPoint_(sightPoint);

		if (this.displayProjection().isParallel()) {
			projection.eyePoint_(projection.eyePoint().plus_(sightPoint).minus_(this.displayProjection().sightPoint()));
		}

		projection.zoomHeight_(zoomHeight * 5);

		return this.pickTipProjection_(projection);
	}

	/**
	 * Pick a tip with the projection.
	 * 
	 * @param projection jp.co.sra.jun.opengl.projection.JunOpenGLProjection
	 * @return jp.co.sra.jun.octree.nodes.JunOctreeNode
	 * @category private
	 */
	protected JunOctreeNode pickTipProjection_(final JunOpenGLProjection projection) {
		final int[] pickedIndex = new int[2];
		final ArrayList<JunOctreeNode> tips = new ArrayList<JunOctreeNode>();

		JunOpenGLObjectPicker objectPicker = new JunOpenGLObjectPicker();
		objectPicker.setRenderer(new JunOpenGLRenderer() {
			public void renderOn_(JunOpenGLDrawable aDrawable) {
				final JunOpenGLPickingRenderingContext pickingRenderingContext = new JunOpenGLPickingRenderingContext((JunOpenGLObjectPicker) aDrawable);
				pickingRenderingContext.projection_(projection);

				pickedIndex[1] = -1;
				pickedIndex[0] = pickingRenderingContext.selectNearestObjectWhile_(new StBlockClosure0<Void>() {
					public Void value() {
						for (int i = 0; i < octrees.length; i++) {
							octrees[i].originExtentTipsDo_(new StBlockClosure3<Void, Jun3dPoint, Jun3dPoint, JunOctreeNode>() {
								public Void value(Jun3dPoint origin, Jun3dPoint extent, final JunOctreeNode tip) {
									pickedIndex[1]++;
									tips.add(tip);
									pickingRenderingContext.setId_(pickedIndex[1]);
									pickingRenderingContext.productTransformation_while_(Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure0<Void>() {
										public Void value() {
											if (tip.isFilled()) {
												tip.renderPolygonOn_(pickingRenderingContext);
											} else {
												tip.renderVertexOn_(pickingRenderingContext);
											}
											return null;
										}
									});
									return null;
								}
							});
						}
						return null;
					}
				});
			}

			public void superimposeOn_(Graphics aGraphics, JunOpenGLDrawable aDrawable) {
			}
		});

		return (pickedIndex[0] >= 0) ? (JunOctreeNode) tips.get(pickedIndex[0]) : null;
	}

	/**
	 * Set the octrees.
	 * 
	 * @param aCollectionOfJunOctree java.util.Collection
	 * @category private
	 */
	protected void setOctrees_(Collection<JunOctree> aCollectionOfJunOctree) {
		if (aCollectionOfJunOctree == null || aCollectionOfJunOctree.isEmpty()) {
			octrees = new JunOctree[0];
		} else {
			octrees = aCollectionOfJunOctree.toArray(new JunOctree[aCollectionOfJunOctree.size()]);
		}

		this.clearSelection();
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	@Override
	protected String windowTitle() {
		return $String("Octree Editor");
	}

	/**
	 * Answer my menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.smalltalk.StApplicationModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		if (_menuBar == null) {
			_menuBar = new StMenuBar();
			_menuBar.add(this._createFileMenu());
			_menuBar.add(this._createEditMenu());
			_menuBar.add(this._createManipulateMenu());
			_menuBar.add(this._createLightMenu());
			_menuBar.add(this._createMiscMenu());
		}
		return _menuBar;
	}

	/**
	 * Create a "File" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_createFileMenu()
	 * @category resources
	 */
	@Override
	protected StMenu _createFileMenu() {
		StMenu fileMenu = new StMenu($String("File"), $("fileMenu"));
		fileMenu.add(new StMenuItem($String("New"), new MenuPerformer(this, "newModel")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Quit"), new MenuPerformer(this, "quitDoing")));
		return fileMenu;
	}

	/**
	 * Create an "Edit" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_createEditMenu()
	 * @category resources
	 */
	@Override
	protected StMenu _createEditMenu() {
		StMenu editMenu = new StMenu($String("Edit"), $("editMenu"));
		editMenu.add(new StMenuItem($String("Select all"), new MenuPerformer(this, "selectAll")));
		editMenu.add(new StMenuItem($String("Unselect all"), new MenuPerformer(this, "unselectAll")));
		return editMenu;
	}

	/**
	 * Create a "Manipulate" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createManipulateMenu() {
		StMenu manipulateMenu = new StMenu($String("Manipulate"), $("manipulateMenu"));

		manipulateMenu.add(new StMenuItem($String("Divide"), new MenuPerformer(this, "divideSelection")));
		manipulateMenu.add(new StMenuItem($String("Erase"), new MenuPerformer(this, "eraseSelection")));
		manipulateMenu.add(new StMenuItem($String("Fill"), new MenuPerformer(this, "fillSelection")));
		StMenu meltMenu = new StMenu($String("Melt"));
		meltMenu.add(new StMenuItem($String("Accuracy") + "...", new MenuPerformer(this, "meltWithAccuracy")));
		meltMenu.add(new StMenuItem($String("Depth") + "...", new MenuPerformer(this, "meltWithDepth")));
		manipulateMenu.add(meltMenu);
		manipulateMenu.add(new StMenuItem($String("Simplify"), new MenuPerformer(this, "simplify")));
		manipulateMenu.addSeparator();
		manipulateMenu.add(new StMenuItem($String("Spawn"), new MenuPerformer(this, "spawnSelection")));
		return manipulateMenu;
	}

	/**
	 * Create a "Misc" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_createMiscMenu()
	 * @category resources
	 */
	@Override
	protected StMenu _createMiscMenu() {
		StMenu miscMenu = new StMenu($String("Misc"), $("miscMenu"));
		miscMenu.add(new StMenuItem($String("Viewfinder"), new MenuPerformer(this, "spawnOpenGLObject")));
		return miscMenu;
	}

}
