package jp.co.sra.jun.opengl.catalog;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StColorValue;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StReadStream;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StCheckBoxMenuItem;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;

import jp.co.sra.jun.goodies.calendar.JunCalendarModel;
import jp.co.sra.jun.goodies.files.JunFileModel;
import jp.co.sra.jun.goodies.image.support.JunImageProcessor;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.goodies.lisp.JunLispParser;
import jp.co.sra.jun.goodies.progress.JunProgress;
import jp.co.sra.jun.goodies.tables.JunAttributeTable;
import jp.co.sra.jun.goodies.tips.JunURL;
import jp.co.sra.jun.goodies.tools.JunDummyWriter;
import jp.co.sra.jun.goodies.utilities.JunFileUtility;
import jp.co.sra.jun.goodies.utilities.JunSensorUtility;
import jp.co.sra.jun.goodies.utilities.JunStringUtility;
import jp.co.sra.jun.graphics.navigator.JunFileRequesterDialog;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.system.framework.JunApplicationModel;
import jp.co.sra.jun.system.framework.JunDialog;
import jp.co.sra.jun.system.generics.JunTrioObject;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunOpenGLCatalogModel class
 * 
 *  @author    nisinaka
 *  @created   2009/06/15 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun771 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLCatalogModel.java,v 9.4 2010/03/09 09:17:34 nisinaka Exp $
 */
public class JunOpenGLCatalogModel extends JunApplicationModel {

	public static Color DefaultBorderNormalColor = StColorValue.VeryLightGray;
	public static Color DefaultBorderSelectedColor = StColorValue.Red;
	public static int DefaultColumnSize = 3;
	public static int DefaultRowSize = 3;
	protected static DecimalFormat EightDigitsDecimalFormat = new DecimalFormat("00000000");

	protected int columnSize;
	protected int rowSize;
	protected ArrayList<JunOpenGLCatalogElement> catalogElements;
	protected ArrayList<JunOpenGLCatalogViewer> catalogViewers;
	protected File catalogFile;
	protected ArrayList<JunOpenGLCatalogElement> selectedElements;
	protected JunAttributeTable<StSymbol, Object> preferenceTable;
	protected int startIndex;
	protected StMenuBar _menuBar;

	/**
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOpenGLCatalogModel() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @param aList java.util.List<JunOpenGLCatalogElement>
	 * @category Instance creation
	 */
	public JunOpenGLCatalogModel(List<JunOpenGLCatalogElement> aList) {
		this.elements_(aList);
	}

	/**
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunOpenGLCatalogModel(JunLispList aList) {
		super(aList);
	}

	/**
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @param catalogFile java.io.File
	 * @category Instance creation
	 */
	public static JunOpenGLCatalogModel FromFile(final File catalogFile) {
		if (catalogFile == null || catalogFile.exists() == false) {
			return new JunOpenGLCatalogModel();
		}

		final StValueHolder<JunLispList> valueHolder = new StValueHolder<JunLispList>();

		final JunProgress progress = new JunProgress();
		progress.message_($String("Please wait to estimate."));
		progress.do_(new StBlockClosure0<Void>() {
			public Void value() {
				progress.value_(0);

				final long fileSize = catalogFile.length();

				try {
					final StReadStream stream = new StReadStream(catalogFile);

					Thread thread = new Thread() {
						public void run() {
							progress.message_($String("reading <1p>...", null, catalogFile.getName()));
							while (progress.value() < 1) {
								progress.value_((float) stream.position() / fileSize);

								try {
									Thread.sleep(100);
								} catch (InterruptedException e) {
									e.printStackTrace();
								}
							}

						}
					};
					thread.setPriority(Thread.NORM_PRIORITY + 1);
					thread.start();

					valueHolder.value_((JunLispList) JunLispParser.Parse_(stream));

				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					progress.value_(1);
				}

				return null;
			}
		});

		JunLispList aList = valueHolder.value();
		if (valueHolder.value() == null) {
			return null;
		}

		JunOpenGLCatalogModel aCatalogModel = new JunOpenGLCatalogModel(aList);
		aCatalogModel.file_(catalogFile);
		return aCatalogModel;
	}

	/**
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @param directory java.io.File
	 * @category Instance creation
	 */
	public static JunOpenGLCatalogModel FromDirectory(File directory) {
		return FromDirectories(new File[] { directory });
	}

	/**
	 * 
	 * Create a new instance of JunOpenGLCatalogModel and initialize it.
	 *
	 * @param directories java.io.File[]
	 * @category Instance creation
	 */
	public static JunOpenGLCatalogModel FromDirectories(File[] directories) {
		final ArrayList<File> fileList = new ArrayList<File>();
		for (File directory : directories) {
			if (directory.exists() && directory.isDirectory()) {
				File[] files = directory.listFiles(new FileFilter() {
					public boolean accept(File pathname) {
						return "lst".equals(JunFileUtility.ExtensionStringOf_(pathname));
					}
				});

				fileList.addAll(Arrays.asList(files));
			}
		}
		if (fileList.isEmpty()) {
			return new JunOpenGLCatalogModel();
		}

		final ArrayList<JunOpenGLCatalogElement> elementList = new ArrayList<JunOpenGLCatalogElement>();
		new JunProgress().do_(new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				int size = fileList.size();
				for (int n = 0; n < size; n++) {
					File file = fileList.get(n);
					if (file.exists()) {
						progress.message_($String("reading <1p>...", null, file.getName()));

						try {
							elementList.add(JunOpenGLCatalogElement.FromFile(file));
						} catch (IOException e) {
							e.printStackTrace();
						}
					}

					progress.value_((float) (n + 1) / size);
				}
				return null;
			}
		});

		return new JunOpenGLCatalogModel(elementList);
	}

	/**
	 * Request a catalog file and create a new instance of JunOpenGLCatalogModel.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogModel
	 * @category Utilities
	 */
	public static JunOpenGLCatalogModel Request() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType(JunSystem.$String("Catalog of 3D Objects"), new String[] { "*.c3o", "*.C3O" }), JunFileModel.FileType.All($String("All files")) };
		File file = JunFileRequesterDialog.RequestFile(JunSystem.$String("Select a catalog file."), fileTypes, fileTypes[0]);
		if (file == null) {
			return null;
		}

		return JunOpenGLCatalogModel.FromFile(file);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		columnSize = 0;
		rowSize = 0;
		catalogElements = null;
		catalogViewers = null;
		catalogFile = null;
		selectedElements = null;
		preferenceTable = null;
		startIndex = -1;
		_menuBar = null;
	}

	/**
	 * Answer my current column size.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int columnSize() {
		if (columnSize <= 0) {
			columnSize = DefaultColumnSize;
		}
		return columnSize;
	}

	/**
	 * Set my new column size.
	 * 
	 * @param numberOfColumns int
	 * @category accessing
	 */
	public void columnSize_(int numberOfColumns) {
		columnSize = numberOfColumns;
		this.elements_(this.elements());
	}

	/**
	 * Answer my current row size.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int rowSize() {
		if (rowSize <= 0) {
			rowSize = DefaultRowSize;
		}
		return rowSize;
	}

	/**
	 * Set my new row size.
	 * 
	 * @param numberOfRows int
	 * @category accessing
	 */
	public void rowSize_(int numberOfRows) {
		rowSize = numberOfRows;
		this.elements_(this.elements());
	}

	/**
	 * Answer my current elements.
	 * 
	 * @return java.util.List<JunOpenGLCatalogElement>
	 * @category accessing
	 */
	public List<JunOpenGLCatalogElement> elements() {
		if (catalogElements == null) {
			catalogElements = new ArrayList<JunOpenGLCatalogElement>();
		}
		return catalogElements;
	}

	/**
	 * Set my new elements.
	 * 
	 * @param aList java.util.List<JunOpenGLCatalogElement>
	 * @category accessing
	 */
	public void elements_(List<JunOpenGLCatalogElement> aList) {
		catalogElements = new ArrayList<JunOpenGLCatalogElement>(aList);
	}

	/**
	 * Answer an element of the specified viewer.
	 * 
	 * @param catalogViewer jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewer
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category accessing
	 */
	public JunOpenGLCatalogElement elementOfViewer_(JunOpenGLCatalogViewer catalogViewer) {
		int index = this.viewers().indexOf(catalogViewer);
		if (index < 0) {
			return null;
		}

		index += this.startIndex();
		if (index >= this.numberOfElements()) {
			return null;
		}

		return this.elements().get(index);
	}

	/**
	 * Answer the number of my current elements.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfElements() {
		return this.elements().size();
	}

	/**
	 * Answer my current viewers.
	 * 
	 * @return java.util.List<JunOpenGLCatalogViewer>
	 * @category accessing
	 */
	public List<JunOpenGLCatalogViewer> viewers() {
		if (catalogViewers == null) {
			catalogViewers = new ArrayList<JunOpenGLCatalogViewer>();
			this.addCatalogViewers();
		}
		return catalogViewers;
	}

	/**
	 * Answer an element of the specified viewer.
	 * 
	 * @param catalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category accessing
	 */
	public JunOpenGLCatalogViewer viewerOfElement_(JunOpenGLCatalogElement catalogElement) {
		int index = this.elements().indexOf(catalogElement);
		if (index < 0) {
			return null;
		}
		if (index < this.startIndex() || this.lastIndex() < index) {
			return null;
		}

		return this.viewers().get(index - this.startIndex());
	}

	/**
	 * Answer the number of my current viewers.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfViewers() {
		return this.viewers().size();
	}

	/**
	 * Answer my current file.
	 * 
	 * @return java.io.File
	 * @category accessing
	 */
	public File file() {
		return catalogFile;
	}

	/**
	 * Set my new file.
	 * 
	 * @param aFile java.io.File
	 * @category accessing
	 */
	public void file_(File aFile) {
		catalogFile = aFile;
		this.updateWindowTitle();
	}

	/**
	 * Move the element.
	 * 
	 * @param fromElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @param toElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category actions
	 */
	public void moveElement_(JunOpenGLCatalogElement fromElement, JunOpenGLCatalogElement toElement) {
		int toIndex = this.elements().indexOf(toElement);
		this.elements().remove(fromElement);
		this.elements().add(toIndex, fromElement);
		this.updateViewers();
	}

	/**
	 * Select the specified catalog viewer.
	 * 
	 * @param aCatalogViewer jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewer
	 * @param shiftDown boolean
	 * @category actions
	 */
	public void selectViewer_(JunOpenGLCatalogViewer aCatalogViewer, boolean shiftDown) {
		JunOpenGLCatalogElement catalogElement = this.elementOfViewer_(aCatalogViewer);
		if (catalogElement == null) {
			return;
		}

		boolean isSelected = this.selections().contains(catalogElement);
		if (shiftDown) {
			if (isSelected) {
				this.removeSelection_(catalogElement);
			} else {
				this.addSelection_(catalogElement);
			}
		} else {
			if (isSelected && this.selections().size() == 1) {
				this.clearSelections();
			} else {
				this.clearSelections();
				this.addSelection_(catalogElement);
			}
		}

		this.updateEditMenuIndication();
		this.changed_($("selections"));
	}

	/**
	 * Make a snapshot in the catalog viewfinder to the catalog viewer.
	 * 
	 * @param catalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @param displayViewer jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewfinder
	 * @param catalogViewer jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewer
	 * @category actions
	 */
	public void snapshot_in_to_(JunOpenGLCatalogElement catalogElement, JunOpenGLCatalogViewfinder displayViewer, JunOpenGLCatalogViewer catalogViewer) {
		if (displayViewer == null) {
			return;
		}

		catalogElement.applyPreferecesFrom_(displayViewer);
		catalogElement.applyPreferencesTo_(catalogViewer);
		catalogViewer.changed_($("object"));
	}

	/**
	 * Convert the receiver as an image.
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#asImage()
	 * @category converting
	 */
	@Override
	public StImage asImage() {
		JunOpenGLCatalogView aView = (JunOpenGLCatalogView) this.getView();
		if (aView == null || aView.toComponent().isVisible() == false) {
			return this.asImageOfElements_(this.elements());
		}

		Component catalogComposite = aView.getCatalogCompositeComponent();
		Point marginPoint = new Point(1, 1);
		int width = catalogComposite.getWidth() + marginPoint.x * 2;
		int height = catalogComposite.getHeight() + marginPoint.y * 2;
		StImage anImage = new StImage(width, height);

		Graphics aGraphics = null;
		try {
			aGraphics = anImage.image().getGraphics();
			aGraphics.translate(marginPoint.x, marginPoint.y);

			catalogComposite.paint(aGraphics);

			JunOpenGLCatalogViewerView[] views = aView.getCatalogViewerViews();
			for (int i = 0; i < views.length; i++) {
				views[i].getOpenGLCatalogViewer().asImage().displayOn_at_(aGraphics, views[i].toComponent().getLocation());
			}

		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
				aGraphics = null;
			}
		}

		return anImage;
	}

	/**
	 * Convert the list of elements as an image.
	 * 
	 * @param aList java.util.List<JunOpenGLCatalogElement>
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 */
	public StImage asImageOfElements_(List<JunOpenGLCatalogElement> aList) {
		Dimension eachImageExtent = this.defaultEachImageExtent();
		int howManyRows = (aList.size() + this.columnSize() - 1) / this.columnSize();

		Point marginPoint = new Point(1, 1);
		int width = (eachImageExtent.width + marginPoint.x) * this.columnSize() + marginPoint.x;
		int height = (eachImageExtent.height + marginPoint.y) * howManyRows + marginPoint.y;
		StImage anImage = new StImage(width, height);

		Graphics aGraphics = null;
		try {
			aGraphics = anImage.image().getGraphics();

			for (int i = 0; i < aList.size(); i++) {
				JunOpenGLCatalogElement element = aList.get(i);
				int x = (i % this.columnSize()) * (eachImageExtent.width + marginPoint.x) + marginPoint.x;
				int y = (i / this.columnSize()) * (eachImageExtent.height + marginPoint.y) + marginPoint.y;
				Point p = new Point(x, y);
				StRectangle aBox = new StRectangle(p, eachImageExtent).expandedBy_(marginPoint);

				aGraphics.setColor(StColorValue.VeryLightGray);
				aGraphics.fillRect(aBox.x(), aBox.y(), aBox.width(), aBox.height());

				StImage eachImage = this.asImageOfElement_extent_(element, eachImageExtent);
				eachImage.displayOn_at_(aGraphics, p);
			}

		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
				aGraphics = null;
			}
		}

		return anImage;
	}

	/**
	 * Convert the specified catalog element as an image with the specified extent.
	 * 
	 * @param aCatalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @param aDimension java.awt.Dimension
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 */
	protected StImage asImageOfElement_extent_(JunOpenGLCatalogElement aCatalogElement, Dimension aDimension) {
		StImage anImage = new StImage(aDimension);

		Graphics aGraphics = null;
		try {
			aGraphics = anImage.image().getGraphics();

			JunOpenGLCatalogViewer catalogViewer = new JunOpenGLCatalogViewer(aCatalogElement.displayObject());
			catalogViewer.catalogModel_(this);
			aCatalogElement.applyPreferencesTo_(catalogViewer);
			JunOpenGLCatalogViewer viewer = this.viewerOfElement_(aCatalogElement);
			if (viewer != null) {
				catalogViewer.selectedObjects_(viewer.selectedObjects());
			}
			catalogViewer.asImageExtent_(aDimension).displayOn_(aGraphics);

		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
				aGraphics = null;
			}
		}

		return anImage;
	}

	/**
	 * Answer my current selected elements.
	 * 
	 * @return java.util.List<JunOpenGLCatalogElement>
	 * @category selecting
	 */
	public List<JunOpenGLCatalogElement> selections() {
		if (selectedElements == null) {
			selectedElements = new ArrayList<JunOpenGLCatalogElement>();
		}
		return selectedElements;
	}

	/**
	 * Set my new selected elements.
	 * 
	 * @param aList java.util.List<JunOpenGLCatalogElement>
	 * @category selecting
	 */
	public void selections_(List<JunOpenGLCatalogElement> aList) {
		this.clearSelections();

		if (aList != null) {
			for (JunOpenGLCatalogElement catalogElement : aList) {
				if (catalogElement != null) {
					this.selections().add(catalogElement);
				}
			}
		}

		if (this.selections().isEmpty()) {
			this.updateEditMenuIndication();
		} else {
			// this.redisplay();
			this.updateMenuIndication();
			this.changed_($("selections"));
		}
	}

	/**
	 * Answer the last selected element.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category selecting
	 */
	public JunOpenGLCatalogElement selection() {
		if (this.selections().isEmpty()) {
			return null;
		}
		return this.selections().get(this.selections().size() - 1);
	}

	/**
	 * Set the single selected element.
	 * 
	 * @param aCatalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category selecting
	 */
	public void selection_(JunOpenGLCatalogElement aCatalogElement) {
		ArrayList<JunOpenGLCatalogElement> aList = new ArrayList<JunOpenGLCatalogElement>();
		if (aCatalogElement != null) {
			aList.add(aCatalogElement);
		}
		this.selections_(aList);
	}

	/**
	 * Clear the current selections.
	 * 
	 * @category selecting 
	 */
	public void clearSelections() {
		selectedElements = null;
	}

	/**
	 * Add the specified catalog element to the list of selections.
	 * 
	 * @param aCatalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category selecting
	 */
	public void addSelection_(JunOpenGLCatalogElement aCatalogElement) {
		this.selections().add(aCatalogElement);
	}

	/**
	 * Remove the specified catalog element to the list of selections.
	 * 
	 * @param aCatalogElement jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category selecting
	 */
	public void removeSelection_(JunOpenGLCatalogElement aCatalogElement) {
		this.selections().remove(aCatalogElement);
	}

	/**
	 * Scroll to the specified start index.
	 * 
	 * @param newStartIndex int
	 * @category scrolling
	 */
	public void scroll_(int newStartIndex) {
		if (this.isEmpty()) {
			return;
		}
		if (newStartIndex == this.startIndex()) {
			return;
		}

		for (int index = this.startIndex(); index <= this.lastIndex(); index++) {
			JunOpenGLCatalogViewer catalogViewer = this.viewers().get(index - this.startIndex());
			JunOpenGLCatalogElement catalogElement = this.elements().get(index);
			catalogElement.applyPreferecesFrom_(catalogViewer);
		}

		startIndex = newStartIndex;
		this.updateViewers();
	}

	/**
	 * Answer my current preference table.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAttributeTable<StSymbol, Object>
	 * @category preferences
	 */
	public JunAttributeTable<StSymbol, Object> preferenceTable() {
		if (preferenceTable == null) {
			preferenceTable = new JunAttributeTable<StSymbol, Object>();
		}
		return preferenceTable;
	}

	/**
	 * Set my new preference table.
	 * 
	 * @param anAttributeTable jp.co.sra.jun.goodies.tables.JunAttributeTable<StSymbol, Object>
	 * @category preferences
	 */
	public void preferenceTable_(JunAttributeTable<StSymbol, Object> anAttributeTable) {
		preferenceTable = anAttributeTable;
	}

	/**
	 * Answer my current preference of the object name visibility.
	 * 
	 * @return boolean
	 * @category preferences
	 */
	public boolean visibleObjectName() {
		return (Boolean) this.preferenceTable().at_ifAbsentPut_($("visibleObjectName"), new StBlockClosure0<Object>() {
			public Object value() {
				return Boolean.TRUE;
			}
		});
	}

	/**
	 * Set my new preference of the catalog index visibility.
	 * 
	 * @param aBoolean boolean
	 * @category preferences
	 */
	public void visibleObjectName_(boolean aBoolean) {
		this.preferenceTable().at_put_($("visibleObjectName"), aBoolean);
	}

	/**
	 * Answer my current preference of the catalog index visibility.
	 * 
	 * @return boolean
	 * @category preferences
	 */
	public boolean visibleCatalogIndex() {
		return (Boolean) this.preferenceTable().at_ifAbsentPut_($("visibleCatalogIndex"), new StBlockClosure0<Object>() {
			public Object value() {
				return Boolean.FALSE;
			}
		});
	}

	/**
	 * Set my new preference of the catalog index visibility.
	 * 
	 * @param aBoolean boolean
	 * @category preferences
	 */
	public void visibleCatalogIndex_(boolean aBoolean) {
		this.preferenceTable().at_put_($("visibleCatalogIndex"), aBoolean);
	}

	/**
	 * Answer true if the receiver has no elements, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isEmpty() {
		return this.elements().isEmpty();
	}

	/**
	 * Answer true if the all elements are selected, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isAllSelections() {
		if (this.selections().isEmpty()) {
			return false;
		}

		for (JunOpenGLCatalogElement catalogElement : this.elements()) {
			if (this.selections().contains(catalogElement) == false) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Repaint the receiver's view.
	 * 
	 * @category displaying
	 */
	public void redisplay() {
		this.updateMenuIndication();
		this.changed_($("redisplay"));
	}

	/**
	 * Answer the default each image extent.
	 * 
	 * @return java.awt.Dimension
	 * @category defaults
	 */
	protected Dimension defaultEachImageExtent() {
		Dimension extent = new Dimension(256, 256);

		if (this.viewers().isEmpty() == false) {
			StView aView = this.viewers().get(0).getView();
			if (aView != null && aView.toComponent().isVisible()) {
				extent = aView.toComponent().getSize();
			}
		}

		return extent;
	}

	/**
	 * Answer the default window label.
	 * 
	 * @return java.lang.String
	 * @category defaults
	 */
	protected String defaultWindowLabel() {
		return $String("Catalog of 3D Objects");
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	@Override
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			// TODO
			return new JunOpenGLCatalogViewSwing(this);
		} else {
			return new JunOpenGLCatalogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#windowTitle()
	 * @category interface opening
	 */
	@Override
	protected String windowTitle() {
		String aString = this.defaultWindowLabel();
		File aFile = this.file();
		if (aFile != null) {
			aString = aString + '[' + aFile.getName() + ']';
		}
		return aString;
	}

	/**
	 * Answer my menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.smalltalk.StApplicationModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		if (_menuBar == null) {
			_menuBar = new StMenuBar();
			_menuBar.add(this._createFileMenu());
			_menuBar.add(this._createEditMenu());
			_menuBar.add(this._createLookMenu());
		}
		return _menuBar;
	}

	/**
	 * Create a "File" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createFileMenu() {
		StMenu fileMenu = new StMenu($String("File"), $("fileMenu"));

		fileMenu.add(new StMenuItem($String("New"), new MenuPerformer(this, "newModel")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Select directory..."), new MenuPerformer(this, "selectDirectory")));
		fileMenu.add(new StMenuItem($String("Open catalog..."), new MenuPerformer(this, "openCatalog")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Save catalog..."), $("saveCatalog"), new MenuPerformer(this, "saveCatalog")));
		fileMenu.add(new StMenuItem($String("Save as catalog..."), $("saveAsCatalog"), new MenuPerformer(this, "saveAsCatalog")));
		fileMenu.add(new StMenuItem($String("Save to directory..."), $("saveToDirectory"), new MenuPerformer(this, "saveToDirectory")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Save as image..."), $("saveAsImage"), new MenuPerformer(this, "saveAsImage")));
		fileMenu.add(new StMenuItem($String("Save as HTML..."), $("saveAsHTML"), new MenuPerformer(this, "saveAsHTML")));
		fileMenu.addSeparator();
		fileMenu.add(new StMenuItem($String("Quit"), new MenuPerformer(this, "quitDoing")));

		return fileMenu;
	}

	/**
	 * Create an "Edit" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createEditMenu() {
		StMenu editMenu = new StMenu($String("Edit"), $("editMenu"));

		editMenu.add(new StMenuItem($String("Open Viewfinder"), $("openDisplayViewer"), new MenuPerformer(this, "openDisplayViewer")));
		editMenu.add(new StMenuItem($String("Remove"), $("removeElements"), new MenuPerformer(this, "removeElements")));
		editMenu.addSeparator();
		editMenu.add(new StMenuItem($String("Select All"), $("selectAll"), new MenuPerformer(this, "selectAll")));

		return editMenu;
	}

	/**
	 * Create a "Look" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createLookMenu() {
		StMenu lookMenu = new StMenu($String("Look"), $("lookMenu"));

		lookMenu.add(new StCheckBoxMenuItem($String("Show Indexes"), $("toggleCatalogIndex"), new MenuPerformer(this, "toggleCatalogIndex")));
		lookMenu.add(new StCheckBoxMenuItem($String("Show Names"), $("toggleObjectName"), new MenuPerformer(this, "toggleObjectName")));
		lookMenu.addSeparator();
		lookMenu.add(new StMenuItem($String("Column Size * Row Size"), $("arrangeColumnSizeAndRowSize"), new MenuPerformer(this, "arrangeColumnSizeAndRowSize")));

		return lookMenu;
	}

	/**
	 * Update the menu indication.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#updateMenuIndication()
	 * @category menu accessing
	 */
	@Override
	public void updateMenuIndication() {
		this.updateFileMenuIndication();
		this.updateEditMenuIndication();
		this.updateLookMenuIndication();
	}

	/**
	 * Update the file menu indication.
	 * 
	 * @category menu accessing
	 */
	protected void updateFileMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("fileMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem;
		menuItem = aMenu.atNameKey_($("saveCatalog"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
		menuItem = aMenu.atNameKey_($("saveAsCatalog"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
		menuItem = aMenu.atNameKey_($("saveToDirectory"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
		menuItem = aMenu.atNameKey_($("saveAsImage"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
		menuItem = aMenu.atNameKey_($("saveAsHTML"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
	}

	/**
	 * Update the edit menu indication.
	 * 
	 * @category menu accessing
	 */
	protected void updateEditMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("editMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem;
		menuItem = aMenu.atNameKey_($("openDisplayViewer"));
		if (menuItem != null) {
			if (this.isEmpty() || this.selections().isEmpty()) {
				menuItem.disable();
			} else {
				menuItem.enable();
			}
		}
		menuItem = aMenu.atNameKey_($("removeElements"));
		if (menuItem != null) {
			if (this.isEmpty() || this.selections().isEmpty()) {
				menuItem.disable();
			} else {
				menuItem.enable();
			}
		}
		menuItem = aMenu.atNameKey_($("selectAll"));
		if (menuItem != null) {
			if (this.isEmpty() || this.isAllSelections()) {
				menuItem.disable();
			} else {
				menuItem.enable();
			}
		}
	}

	/**
	 * Update the file menu indication.
	 * 
	 * @category menu accessing
	 */
	protected void updateLookMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("lookMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem;
		menuItem = aMenu.atNameKey_($("toggleCatalogIndex"));
		if (menuItem != null) {
			menuItem.enable();
			((StCheckBoxMenuItem) menuItem).beSelected(this.visibleCatalogIndex());
		}
		menuItem = aMenu.atNameKey_($("toggleObjectName"));
		if (menuItem != null) {
			menuItem.enable();
			((StCheckBoxMenuItem) menuItem).beSelected(this.visibleObjectName());
		}
		menuItem = aMenu.atNameKey_($("arrangeColumnSizeAndRowSize"));
		if (menuItem != null) {
			menuItem.beEnabled(!this.isEmpty());
		}
	}

	/**
	 * Select a directory and load all LST files as elements.
	 * 
	 * @category menu messages
	 */
	public void selectDirectory() {
		File directory = JunFileRequesterDialog.RequestDirectory();
		if (directory == null) {
			return;
		}

		JunOpenGLCatalogModel catalog = JunOpenGLCatalogModel.FromDirectory(directory);
		ArrayList<JunOpenGLCatalogElement> newElements = new ArrayList<JunOpenGLCatalogElement>(this.elements());
		newElements.addAll(catalog.elements());
		this.elements_(newElements);
		this.updateViewers();
		this.changed_($("catalogComposite"));
		this.updateMenuIndication();
	}

	/** 
	 * Open a catalog file.
	 * 
	 * @category menu messages
	 */
	public void openCatalog() {
		JunOpenGLCatalogModel catalogModel = JunOpenGLCatalogModel.Request();
		if (catalogModel == null) {
			return;
		}

		this.file_(catalogModel.file());
		this.columnSize_(catalogModel.columnSize());
		this.rowSize_(catalogModel.rowSize());
		this.elements_(catalogModel.elements());
		this.selections_(catalogModel.selections());
		this.preferenceTable_(catalogModel.preferenceTable());

		this.removeCatalogViewers();
		this.addCatalogViewers();
		this.updateViewers();
		this.changed_($("catalogComposite"));
		this.updateMenuIndication();
	}

	/** 
	 * Save to the catalog file.
	 * 
	 * @category menu messages
	 */
	public void saveCatalog() {
		if (this.isEmpty()) {
			return;
		}

		if (this.file() == null) {
			this.saveAsCatalog();
		} else {
			this.saveCatalogTo_(this.file());
		}
	}

	/**
	 * Save the catalog to the specified file.
	 * 
	 * @param aFile java.io.File
	 * @category menu messages
	 */
	protected void saveCatalogTo_(final File aFile) {
		final JunProgress progress = new JunProgress();
		progress.message_($String("Please wait to estimate."));
		progress.do_(new StBlockClosure0<Void>() {
			public Void value() {
				progress.value_(0);

				Writer writer = null;
				try {
					FileOutputStream fileOutputStream = new FileOutputStream(aFile);
					final DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
					writer = new OutputStreamWriter(dataOutputStream);

					final int estimatedSize = JunOpenGLCatalogModel.this.estimatedFileSizeToSave();

					Thread thread = new Thread() {
						public void run() {
							progress.message_($String("writing <1p>...", null, aFile.getName()));
							while (progress.value() < 1) {
								progress.value_((float) dataOutputStream.size() / estimatedSize);

								try {
									Thread.sleep(100);
								} catch (InterruptedException e) {
									e.printStackTrace();
								}
							}
						}
					};
					thread.setPriority(Thread.NORM_PRIORITY + 1);
					thread.start();

					JunOpenGLCatalogModel.this.saveCatalogOn_(writer);

				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					progress.value_(1);

					if (writer != null) {
						try {
							writer.flush();
							writer.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				}

				return null;
			}
		});
	}

	/**
	 * Write the catalog on the specified writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException 
	 * @category menu messages
	 */
	protected void saveCatalogOn_(Writer aWriter) throws IOException {
		BufferedWriter bufferedWriter = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
		bufferedWriter.write('%');
		bufferedWriter.newLine();
		bufferedWriter.write("% Catalog of 3D objects (created by " + JunSystem._System + JunSystem._Version + " on " + JunCalendarModel.StringFromDateAndTime() + ')');
		bufferedWriter.newLine();
		bufferedWriter.write('%');
		bufferedWriter.newLine();

		this.toLispList().saveOn_(bufferedWriter);
	}

	/** 
	 * Save as a catalog file.
	 * 
	 * @category menu messages
	 */
	public void saveAsCatalog() {
		if (this.isEmpty()) {
			return;
		}

		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType(JunSystem.$String("Catalog of 3D Objects"), new String[] { "*.c3o", "*.C3O" }), JunFileModel.FileType.All($String("All files")) };
		File file = JunFileRequesterDialog.RequestNewFile(JunSystem.$String("Input a catalog file."), new File(this.defaultBaseName() + ".c3o"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		this.saveCatalogTo_(file);
		this.file_(file);
	}

	/** 
	 * Save the elements to a directory.
	 * 
	 * @category menu messages
	 */
	public void saveToDirectory() {
		if (this.isEmpty()) {
			return;
		}

		final File baseDirectory = JunFileRequesterDialog.RequestNewDirectory(JunSystem.$String("Input a directory."), new File(this.defaultBaseName()));
		if (baseDirectory == null) {
			return;
		}
		if (baseDirectory.exists() == false) {
			if (JunDialog.Confirm_($String("The directory does not exist.") + '\n' + $String("Create a new one?"))) {
				baseDirectory.mkdirs();
			} else {
				return;
			}
		}

		new JunProgress().do_(new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				List<JunOpenGLCatalogElement> elements = JunOpenGLCatalogModel.this.elements();
				int size = elements.size();
				for (int i = 0; i < size; i++) {
					JunOpenGLCatalogElement element = elements.get(i);
					String name = element.displayObjectName();
					name = name.replaceAll("[^A-Za-z0-9]", "_");
					if (name.length() == 0) {
						name = JunOpenGLCatalogModel.this.defaultBaseName();
					}
					name += ".lst";

					progress.message_($String("writing <1p>...", null, name));

					File file = new File(baseDirectory, name);
					try {
						new JunOpenGLDisplayModel().writeToLST10_object_(file, element.displayObject());
					} catch (IOException e) {
						e.printStackTrace();
					}

					progress.value_((float) i / size);
				}

				return null;
			}
		});
	}

	/** 
	 * Save the receiver's image.
	 * 
	 * @category menu messages
	 */
	public void saveAsImage() {
		boolean shiftDown = JunSensorUtility.ShiftDown();
		boolean altDown = JunSensorUtility.AltDown();

		StImage anImage = null;
		if (altDown) {
			anImage = this.asImageOfElements_(this.elements());
		} else {
			anImage = this.asImage();
		}

		if (anImage == null) {
			return;
		}

		if (shiftDown) {
			anImage = anImage.convertToPalette_(JunImageProcessor.GrayPalette256());
		}

		File aFile = this.requestNewImageFilename();
		if (aFile == null) {
			return;
		}

		this.writeImage_to_(anImage, aFile);
	}

	/** 
	 * Save the receiver as HTML.
	 * 
	 * @category menu messages
	 */
	public void saveAsHTML() {
		if (this.isEmpty()) {
			return;
		}

		File baseDirectory = JunFileRequesterDialog.RequestNewDirectory($String("Input a directory."), new File(this.defaultBaseName()));
		if (baseDirectory == null) {
			return;
		}

		if (baseDirectory.exists() == false) {
			if (JunDialog.Confirm_($String("The directory does not exist.") + '\n' + $String("Create a new one?"))) {
				baseDirectory.mkdirs();
			} else {
				return;
			}
		}

		File imageDirectory = new File(baseDirectory, "images");
		if (imageDirectory.exists() == false) {
			imageDirectory.mkdir();
		}

		File thumbnailDirectory = new File(baseDirectory, "thumbnails");
		if (thumbnailDirectory.exists() == false) {
			thumbnailDirectory.mkdir();
		}

		List<JunTrioObject<JunOpenGLCatalogElement, File, File>> aList = new ArrayList<JunTrioObject<JunOpenGLCatalogElement, File, File>>(this.elements().size());
		for (int index = 0; index < this.elements().size(); index++) {
			JunOpenGLCatalogElement element = this.elements().get(index);
			String filename = EightDigitsDecimalFormat.format(index) + ".jpg";
			File imageFile = new File(imageDirectory, filename);
			File thumbnailFile = new File(thumbnailDirectory, filename);

			aList.add(new JunTrioObject<JunOpenGLCatalogElement, File, File>(element, imageFile, thumbnailFile));
		}

		File htmlFile = new File(baseDirectory, "index.html");
		this.htmlTo_with_(htmlFile, aList);

		try {
			JunURL.Browse_(htmlFile.toURI().toURL());
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	/** 
	 * Open a display viewer for the selected elements.
	 * 
	 * @category menu messages
	 */
	public void openDisplayViewer() {
		for (JunOpenGLCatalogElement element : this.selections()) {
			JunOpenGLCatalogViewer viewer = this.viewerOfElement_(element);
			if (viewer != null) {
				viewer.openDisplayViewer();
			}
		}
	}

	/** 
	 * Remove the selected elements.
	 * 
	 * @category menu messages
	 */
	public void removeElements() {
		List<JunOpenGLCatalogElement> selections = this.selections();
		if (selections.isEmpty()) {
			return;
		}

		if (JunDialog.Confirm_($String("Really remove?")) == false) {
			return;
		}

		ArrayList<JunOpenGLCatalogElement> newElements = new ArrayList<JunOpenGLCatalogElement>();
		for (JunOpenGLCatalogElement element : this.elements()) {
			if (selections.contains(element) == false) {
				newElements.add(element);
			}
		}

		this.elements_(newElements);
		this.clearSelections();
		this.updateViewers();
		this.changed_($("catalogComposite"));
		this.updateMenuIndication();
	}

	/** 
	 * Select all elements.
	 * 
	 * @category menu messages
	 */
	public void selectAll() {
		this.clearSelections();

		for (JunOpenGLCatalogElement element : this.elements()) {
			this.addSelection_(element);
		}

		this.changed_($("selections"));
		this.updateEditMenuIndication();
	}

	/** 
	 * Toggle the visibility of the catalog index.
	 * 
	 * @category menu messages
	 */
	public void toggleCatalogIndex() {
		this.visibleCatalogIndex_(!this.visibleCatalogIndex());
		this.redisplay();
	}

	/** 
	 * Toggle the visibility of the object name.
	 * 
	 * @category menu messages
	 */
	public void toggleObjectName() {
		this.visibleObjectName_(!this.visibleObjectName());
		this.redisplay();
	}

	/** 
	 * Change column size and row size.
	 * 
	 * @category menu messages
	 */
	public void arrangeColumnSizeAndRowSize() {
		int numberOfColumns = this.columnSize();
		int numberOfRows = this.rowSize();
		String aString = JunDialog.Request_($String("How many column size and row size?"), String.valueOf(numberOfColumns) + " * " + String.valueOf(numberOfRows));
		if (aString == null || aString.length() == 0) {
			return;
		}

		String dividers = new String(JunStringUtility.Separators()) + "*x,";
		String[] aCollection = JunStringUtility.Separate_dividers_(aString, dividers.toCharArray());
		if (aCollection.length < 2) {
			return;
		}

		try {
			numberOfColumns = Integer.parseInt(aCollection[0]);
			numberOfRows = Integer.parseInt(aCollection[1]);
		} catch (NumberFormatException e) {
			return;
		}

		if (numberOfColumns < 1) {
			return;
		}
		if (numberOfRows < 1) {
			return;
		}

		this.columnSize_(numberOfColumns);
		this.rowSize_(numberOfRows);

		this.removeCatalogViewers();
		this.addCatalogViewers();
		this.updateViewers();
	}

	/**
	 * Get the receiver's attributes from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#fromLispList(jp.co.sra.jun.goodies.lisp.JunLispList)
	 * @category lisp support
	 */
	@Override
	public void fromLispList(JunLispList aList) {
		super.fromLispList(aList);

		this.columnSizeFromLispList(aList);
		this.rowSizeFromLispList(aList);
		this.catalogElementsFromLispList(aList);
		//this.selectedElementsFromLispList(aList);
		//this.preferenceTableFromLispList(aList);
	}

	/**
	 * Convert the receiver's attributes to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#toLispList()
	 * @category lisp support
	 */
	@Override
	public JunLispCons toLispList() {
		JunLispCons aList = super.toLispList();

		aList.add_(this.columnSizeToLispList());
		aList.add_(this.rowSizeToLispList());
		aList.add_(this.catalogElementsToLispList());
		aList.add_(this.selectedElementsToLispList());
		if (this.preferenceTable().isEmpty() == false) {
			aList.add_(this.preferenceTableToLispList());
		}

		return aList;
	}

	/**
	 * Get my column size from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void columnSizeFromLispList(JunLispList aList) {
		JunLispList columnSizeList = aList._findSublistWithHead($("columnSize"));
		if (columnSizeList == null) {
			return;
		}

		this.columnSize_((Integer) columnSizeList.tail());
	}

	/**
	 * Convert the receiver's column size to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons columnSizeToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("columnSize"));
		aList.tail_(this.columnSize());
		return aList;
	}

	/**
	 * Get my row size from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void rowSizeFromLispList(JunLispList aList) {
		JunLispList rowSizeList = aList._findSublistWithHead($("rowSize"));
		if (rowSizeList == null) {
			return;
		}

		this.rowSize_((Integer) rowSizeList.tail());
	}

	/**
	 * Convert the receiver's row size to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons rowSizeToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("rowSize"));
		aList.tail_(this.rowSize());
		return aList;
	}

	/**
	 * Get my catalog elements from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void catalogElementsFromLispList(JunLispList aList) {
		JunLispList catalogElementsList = aList._findSublistWithHead($("catalogElements"));
		if (catalogElementsList == null) {
			return;
		}

		final ArrayList<JunOpenGLCatalogElement> catalogElements = new ArrayList<JunOpenGLCatalogElement>();
		((JunLispList) catalogElementsList.tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object anObject) {
				catalogElements.add(new JunOpenGLCatalogElement((JunLispList) anObject));
				return null;
			}
		});

		this.elements_(catalogElements);
	}

	/**
	 * Convert the receiver's catalog elements to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons catalogElementsToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("catalogElements"));
		for (JunOpenGLCatalogElement element : this.elements()) {
			aList.add_(element.toLispList());
		}
		return aList;
	}

	/**
	 * Get my selected elements from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void selectedElementsFromLispList(JunLispList aList) {
		JunLispList selectedElementsList = aList._findSublistWithHead($("selectedElements"));
		if (selectedElementsList == null) {
			return;
		}

		final ArrayList<JunOpenGLCatalogElement> selectedElements = new ArrayList<JunOpenGLCatalogElement>();

		((JunLispList) selectedElementsList.tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object anObject) {
				selectedElements.add(JunOpenGLCatalogModel.this.elements().get(((Integer) anObject) - 1));
				return null;
			}
		});

		this.selections_(selectedElements);
	}

	/**
	 * Convert the receiver's selected elements to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons selectedElementsToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("selectedElements"));
		for (JunOpenGLCatalogElement element : this.selections()) {
			aList.add_(this.elements().indexOf(element) + 1);
		}
		return aList;
	}

	/**
	 * Get my preference table from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void preferenceTableFromLispList(JunLispList aList) {
		JunLispList preferenceTableList = aList._findSublistWithHead($("preferenceTable"));
		if (preferenceTableList == null) {
			return;
		}

		this.preferenceTable_(new JunAttributeTable<StSymbol, Object>((JunLispList) preferenceTableList.tail()));
	}

	/**
	 * Convert the receiver's preference table to a lisp list.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons preferenceTableToLispList() {
		JunLispCons aList = this.lispCons();
		aList.head_($("preferenceTable"));
		aList.tail_(this.preferenceTable().toLispList());
		return aList;
	}

	/**
	 * Answer the current start index.
	 * 
	 * @return int
	 * @category private
	 */
	protected int startIndex() {
		if (startIndex < 0) {
			startIndex = 0;
		}
		return startIndex;
	}

	/**
	 * Answer the current last index.
	 * 
	 * @return int
	 * @category private
	 */
	protected int lastIndex() {
		return Math.min(this.startIndex() + this.numberOfViewers(), this.numberOfElements()) - 1;
	}

	/**
	 * Add the catalog viewers.
	 * 
	 * @category private
	 */
	protected synchronized void addCatalogViewers() {
		int numberOfViewers = this.columnSize() * this.rowSize();
		if (numberOfViewers <= 0) {
			return;
		}

		for (int i = 0; i < numberOfViewers; i++) {
			JunOpenGLCatalogViewer aViewer = new JunOpenGLCatalogViewer();
			aViewer.catalogModel_(this);
			this.viewers().add(aViewer);
		}

		this.changed_($("catalogComposite"));
	}

	/**
	 * Remove the catalog viewers.
	 * 
	 * @category private
	 */
	protected synchronized void removeCatalogViewers() {
		catalogViewers = null;
	}

	/**
	 * Update the viewers.
	 * 
	 * @category private
	 */
	protected void updateViewers() {
		for (int index = this.startIndex(); index <= this.lastIndex(); index++) {
			JunOpenGLCatalogViewer catalogViewer = this.viewers().get(index - this.startIndex());
			JunOpenGLCatalogElement catalogElement = this.elements().get(index);
			catalogViewer.displayObject_(null);
			catalogViewer.flushObject();
			catalogViewer.flushMovementVector();
			catalogElement.applyPreferencesTo_(catalogViewer);
			catalogViewer.changed_($("object"));
		}
		for (int index = this.lastIndex() - this.startIndex() + 1; index < this.numberOfViewers(); index++) {
			JunOpenGLCatalogViewer catalogViewer = this.viewers().get(index);
			catalogViewer.displayObject_(null);
			catalogViewer.flushObject();
			catalogViewer.flushMovementVector();
			catalogViewer.changed_($("object"));
		}

		// this.changed_($("viewers"));
	}

	/**
	 * Estimate the file size for saving.
	 * 
	 * @return int
	 * @category private
	 */
	protected int estimatedFileSizeToSave() {
		int estimatedSize = 0;

		JunDummyWriter dummyWriter = null;
		try {
			dummyWriter = new JunDummyWriter();

			this.toLispList().saveOn_(dummyWriter);
			estimatedSize = dummyWriter.position();

		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				dummyWriter.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return estimatedSize;
	}

	/**
	 * Write the HTML to the file.
	 * 
	 * @param aFile java.io.File
	 * @param aList java.util.List<JunTrioObject<JunOpenGLCatalogElement, File, File>>
	 * @category private 
	 */
	protected void htmlTo_with_(final File aFile, final List<JunTrioObject<JunOpenGLCatalogElement, File, File>> aList) {
		new JunProgress().do_(new StBlockClosure1<Void, JunProgress>() {
			public Void value(JunProgress progress) {
				JunOpenGLCatalogModel catalogModel = JunOpenGLCatalogModel.this;
				Dimension eachImageExtent = catalogModel.defaultEachImageExtent();
				for (int index = 0; index < aList.size(); index++) {
					JunTrioObject<JunOpenGLCatalogElement, File, File> trio = aList.get(index);
					JunOpenGLCatalogElement element = trio.first();
					File imageFile = trio.second();
					File thumbnailFile = trio.third();

					progress.message_(element.displayObjectName());

					catalogModel.writeImage_to_(catalogModel.asImageOfElement_extent_(element, eachImageExtent), thumbnailFile);
					catalogModel.writeImage_to_(catalogModel.asImageOfElement_extent_(element, new Dimension(eachImageExtent.width * 2, eachImageExtent.height * 2)), imageFile);

					progress.value_((float) (index + 1) / aList.size());
				}

				PrintWriter writer = null;
				try {
					writer = new PrintWriter(aFile, "UTF-8");

					writer.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
					writer.println("<html lang=\"ja\">");
					writer.println("<head>");
					writer.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
					writer.println("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">");
					writer.println("<link rel=\"index\" href=\"index.html\">");
					writer.println("<title>" + catalogModel.windowTitle() + "</title>");
					writer.println("<style type=\"text/css\">");
					writer.println("<!--");
					writer.println("body {");
					writer.println("  background-color : #ffffff;");
					writer.println("  margin : 20px;");
					writer.println("  padding : 10px;");
					writer.println("  font-family : serif;");
					writer.println("  font-size : 10pt;");
					writer.println("}");
					writer.println("img.borderless {");
					writer.println("  border-width : 0px;");
					writer.println("  vertical-align : middle;");
					writer.println("}");
					writer.println("table.belt {");
					writer.println("  border-style : solid;");
					writer.println("  border-width : 0px;");
					writer.println("  border-color : #000000;");
					writer.println("  background-color : #eeeeee;");
					writer.println("  padding : 0px 0px;");
					writer.println("}");
					writer.println("table.content {");
					writer.println("  border-style : solid;");
					writer.println("  border-width : 0px;");
					writer.println("  border-color : #000000;");
					writer.println("  padding : 0px 0px;");
					writer.println("}");
					writer.println("table.element {");
					writer.println("  border-style : solid;");
					writer.println("  border-width : 0px;");
					writer.println("  border-color : #000000;");
					writer.println("  padding : 0px 0px;");
					writer.println("}");
					writer.println("td.element {");
					writer.println("  text-align : right;");
					writer.println("  font-size : 8pt;");
					writer.println("  padding : 0px 0px;");
					writer.println("  background-color : #ffffff;");
					writer.println("}");
					writer.println("-->");
					writer.println("</style>");
					writer.println("</head>");
					writer.println("<body>");
					writer.println("<table class=\"belt\" summary=\"belt\">");
					writer.println("  <tbody>");
					writer.println("    <tr>");
					writer.println("      <td>");
					writer.println("      <table class=\"content\" summary=\"content\">");
					writer.println("        <tbody>");
					writer.println("          <tr>");

					for (int index = 0; index < aList.size(); index++) {
						JunTrioObject<JunOpenGLCatalogElement, File, File> trio = aList.get(index);
						File imageFile = trio.second();
						File thumbnailFile = trio.third();

						writer.print("<td class=\"element\">");
						writer.print("<a href=\"images/");
						writer.print(imageFile.getName());
						writer.print("\">");
						writer.print("<img class=\"borderless\" src=\"thumbnails/");
						writer.print(thumbnailFile.getName());
						writer.print("\" width=\"");
						writer.print(eachImageExtent.width);
						writer.print("\" height=\"");
						writer.print(eachImageExtent.height);
						writer.print("\" alt=\"");
						writer.print(imageFile.getName());
						writer.print("\">");
						writer.print("</a>");
						writer.print("</td>");
						writer.println();

						if ((index + 1) % catalogModel.columnSize() == 0 && index < aList.size()) {
							writer.println("          </tr>");
							writer.println("          <tr>");
						}
					}

					writer.println("          </tr>");
					writer.println("        </tbody>");
					writer.println("      </table>");
					writer.println("      </td>");
					writer.println("    </tr>");
					writer.println("  </tbody>");
					writer.println("</table>");
					writer.println("</body>");
					writer.println("</html>");
					writer.println("");

				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					if (writer != null) {
						writer.flush();
						writer.close();
					}
				}

				return null;
			}
		});
	}
}
