package jp.co.sra.jun.opengl.catalog;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollBar;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;

/**
 * JunOpenGLCatalogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2009/06/18 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun771 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLCatalogViewSwing.java,v 9.3 2010/03/09 09:17:34 nisinaka Exp $
 */
public class JunOpenGLCatalogViewSwing extends JunAbstractViewJPanel implements JunOpenGLCatalogView {

	protected JPanel catalogComposite;
	protected JScrollBar catalogScroller;
	protected DataFlavor myDataFlavor;

	/**
	 * Create a new instance of JunOpenGLCatalogViewSwing and initialize it.
	 *
	 * @param anOpenGLCatalogModel jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogModel
	 * @category Instance creation
	 */
	public JunOpenGLCatalogViewSwing(JunOpenGLCatalogModel anOpenGLCatalogModel) {
		super(anOpenGLCatalogModel);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.catalogComposite(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -18, 1f, -1));
		this.add(this.catalogScroller(), WindowSpecLayout.Position(1f, -17, 0f, 1, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(600, 600));
		this.setLightWeightPopupEnabled(false);

	}

	/**
	 * Answer my model as JunOpenGLCatalogModel.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogModel
	 * @see jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogView#getOpenGLCatalogModel()
	 * @category model accessing
	 */
	public JunOpenGLCatalogModel getOpenGLCatalogModel() {
		return (JunOpenGLCatalogModel) this.model();
	}

	/**
	 * Answer my catalog composite as a component.
	 * 
	 * @return java.awt.Component
	 * @see jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogView#getCatalogCompositeComponent()
	 * @category accessing
	 */
	public Component getCatalogCompositeComponent() {
		return this.catalogComposite();
	}

	/**
	 * Answer my catalog viewer views.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewerView[]
	 * @see jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogView#getCatalogViewerViews()
	 * @category accessing
	 */
	public JunOpenGLCatalogViewerView[] getCatalogViewerViews() {
		ArrayList<JunOpenGLCatalogViewerView> aList = new ArrayList<JunOpenGLCatalogViewerView>();
		for (Component aComponent : this.catalogComposite().getComponents()) {
			if (aComponent instanceof JunOpenGLCatalogViewerView) {
				aList.add((JunOpenGLCatalogViewerView) aComponent);
			}
		}
		return aList.toArray(new JunOpenGLCatalogViewerView[aList.size()]);
	}

	/**
	 * Answer the catalog composite.
	 * 
	 * @return javax.swing.JPanel
	 * @category user interface
	 */
	protected JPanel catalogComposite() {
		if (catalogComposite == null) {
			catalogComposite = new JPanel();
			catalogComposite.setLayout(new WindowSpecLayout());

			this.updateCatalogComposite();
			this.getOpenGLCatalogModel().updateViewers();
		}
		return catalogComposite;
	}

	/**
	 * Answer the catalog scrollbar.
	 * 
	 * @return javax.swing.JScrollBar
	 * @category user interface
	 */
	protected JScrollBar catalogScroller() {
		if (catalogScroller == null) {
			catalogScroller = new JScrollBar(JScrollBar.VERTICAL);
			catalogScroller.addAdjustmentListener(new AdjustmentListener() {
				public void adjustmentValueChanged(AdjustmentEvent e) {
					int value = e.getValue();
					int startIndex = value * JunOpenGLCatalogViewSwing.this.getOpenGLCatalogModel().columnSize();
					JunOpenGLCatalogViewSwing.this.getOpenGLCatalogModel().scroll_(startIndex);
				}
			});

			this.updateCatalogScroller();
		}
		return catalogScroller;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	@Override
	public void update_(DependentEvent e) {
		StSymbol aspect = e.getAspect();
		if (aspect == $("catalogComposite")) {
			this.updateCatalogComposite();
		} else if (aspect == $("redisplay")) {
			this.updateForRedisplay();
		} else if (aspect == $("selections")) {
			this.updateForSelections();
		} else if (aspect == $("viewers")) {
			System.out.println(aspect);
		} else {
			super.update_(e);
		}
	}

	/**
	 * Update the catalog composite.
	 * 
	 * @category updating
	 */
	protected void updateCatalogComposite() {
		this.catalogComposite().removeAll();

		int rowSize = this.getOpenGLCatalogModel().rowSize();
		int columnSize = this.getOpenGLCatalogModel().columnSize();
		for (int y = 0; y < rowSize; y++) {
			for (int x = 0; x < columnSize; x++) {
				int index = x + y * columnSize;
				JunOpenGLCatalogViewer catalogViewer = this.getOpenGLCatalogModel().viewers().get(index);
				JunOpenGLCatalogViewerViewSwing catalogViewerView = new JunOpenGLCatalogViewerViewSwing(catalogViewer);
				catalogViewerView.setBorder(BorderFactory.createLineBorder(JunOpenGLCatalogModel.DefaultBorderNormalColor));
				float left = ((float) x) / columnSize;
				float right = ((float) x + 1) / columnSize;
				float top = ((float) y) / rowSize;
				float bottom = ((float) y + 1) / rowSize;
				this.catalogComposite().add(catalogViewerView, WindowSpecLayout.Position(left, 1, top, 1, right, -1, bottom, -1));
			}
		}

		this.catalogComposite().validate();
		this.updateCatalogScroller();
	}

	/**
	 * Update the catalog scroller.
	 * 
	 * @category updating
	 */
	protected void updateCatalogScroller() {
		int numberOfElements = this.getOpenGLCatalogModel().numberOfElements();
		int rowSize = this.getOpenGLCatalogModel().rowSize();
		int columnSize = this.getOpenGLCatalogModel().columnSize();

		this.catalogScroller().setValues(0, rowSize, 0, (numberOfElements + columnSize - 1) / columnSize);
	}

	/**
	 * Update for the redisplay.
	 * 
	 * @category updating
	 */
	protected void updateForRedisplay() {
		this.redisplayAllViewers();
	}

	/**
	 * Update for the change notice of the selections.
	 * 
	 * @category updating
	 */
	protected void updateForSelections() {
		this.redisplayAllViewers();
	}

	/**
	 * Redisplay all viewers.
	 * 
	 * @category private
	 */
	protected void redisplayAllViewers() {
		for (Component aComponent : this.catalogComposite().getComponents()) {
			if (aComponent instanceof JunOpenGLCatalogViewerViewSwing) {
				((JunOpenGLCatalogViewerViewSwing) aComponent).getOpenGLDrawable().redisplay();
			}
		}
	}

}
