package jp.co.sra.jun.opengl.catalog;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.SwingUtilities;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StColorValue;
import jp.co.sra.smalltalk.StComposedText;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.goodies.utilities.JunSensorUtility;
import jp.co.sra.jun.opengl.display.JunOpenGL3dView;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOpenGLCatalogViewer class
 * 
 *  @author    nisinaka
 *  @created   2009/06/15 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun734 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLCatalogViewer.java,v 9.3 2010/03/09 09:17:34 nisinaka Exp $
 */
public class JunOpenGLCatalogViewer extends JunOpenGLDisplayModel {

	protected JunOpenGLCatalogModel catalogModel;
	protected StPopupMenu _popupMenu;

	/**
	 * Create a new instance of JunOpenGLCatalogViewer and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOpenGLCatalogViewer() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLCatalogViewer and initialize it.
	 *
	 * @param anOpenGL3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	public JunOpenGLCatalogViewer(JunOpenGL3dObject anOpenGL3dObject) {
		super(anOpenGL3dObject);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		catalogModel = null;
		_popupMenu = null;

		this.superimposeBlock_(new StBlockClosure2<Void, Graphics, JunOpenGL3dView>() {
			public Void value(Graphics aGraphics, JunOpenGL3dView aView) {
				JunOpenGLCatalogViewer.this.displaySuperimposeInView_(aGraphics, aView);
				return null;
			}
		});
	}

	/**
	 * Answer my current catalog model.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogModel
	 * @category accessing
	 */
	public JunOpenGLCatalogModel catalogModel() {
		return catalogModel;
	}

	/**
	 * Set my new catalog model.
	 * 
	 * @param aCatalogModel jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogModel
	 * @category accessing
	 */
	public void catalogModel_(JunOpenGLCatalogModel aCatalogModel) {
		catalogModel = aCatalogModel;
	}

	/**
	 * Answer my current catalog element.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogElement
	 * @category accessing
	 */
	public JunOpenGLCatalogElement catalogElement() {
		if (this.hasCatalogModel() == false) {
			return null;
		}

		return this.catalogModel().elementOfViewer_(this);
	}

	/**
	 * Apply the preferences to the element.
	 * 
	 * @category applying
	 */
	public void applyPreferencesToElement() {
		JunOpenGLCatalogElement catalogElement = this.catalogElement();
		if (catalogElement == null) {
			return;
		}

		catalogElement.applyPreferecesFrom_(this);
	}

	/**
	 * Action for a click on the viewer.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param shiftDown boolean
	 * @category actions
	 */
	protected void clickAt_(Jun2dPoint aPoint, boolean shiftDown) {
		if (this.hasCatalogModel()) {
			this.catalogModel().selectViewer_(this, shiftDown);
		}
	}

	/**
	 * Action for a double click on the viewer.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category actions
	 */
	protected void doubleClickAt_(Jun2dPoint aPoint) {
		this.openDisplayViewer();
	}

	/**
	 * Action for a move on the viewer.
	 * 
	 * @param aCatalogViewer jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewer
	 * @category actions
	 */
	protected void movedFrom_(JunOpenGLCatalogViewer aCatalogViewer) {
		this.catalogModel().moveElement_(aCatalogViewer.catalogElement(), this.catalogElement());
	}

	/**
	 * Display the superimpose in the specified view.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aView jp.co.sra.jun.opengl.display.JunOpenGL3dView
	 * @category displaying
	 */
	protected void displaySuperimposeInView_(Graphics aGraphics, JunOpenGL3dView aView) {
		Dimension size = aView.getOpenGLDrawable().toComponent().getSize();
		this.displaySuperimposeInBounds_(aGraphics, new Rectangle(size));
	}

	/**
	 * Display the superimpose in the specified bounds.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aRectangle java.awt.Rectangle
	 * @category displaying
	 */
	protected void displaySuperimposeInBounds_(Graphics aGraphics, Rectangle aRectangle) {
		this.displaySuperimposeObjectNameInBounds_(aGraphics, aRectangle);
		this.displaySuperimposeCatalogIndexInBounds_(aGraphics, aRectangle);
		this.displaySelectionInBounds_(aGraphics, aRectangle);
	}

	/**
	 * Display the object name as a superimpose in the specified bounds.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aRectangle java.awt.Rectangle
	 * @category displaying
	 */
	protected void displaySuperimposeObjectNameInBounds_(Graphics aGraphics, Rectangle aRectangle) {
		if (this.hasCatalogModel() == false) {
			return;
		}
		if (this.catalogModel().visibleObjectName() == false) {
			return;
		}
		if (this.displayObject() == null) {
			return;
		}
		if (this.displayObject().name() == null) {
			return;
		}

		StRectangle bounds = new StRectangle(aRectangle.getSize()).insetBy_(1);
		String aString = this.displayObject().name();
		StComposedText composedText = new StComposedText(aString);
		if (composedText.width() > aRectangle.width) {
			composedText = new StComposedText(aString, aRectangle.width);
		}

		this.displayComposedText_on_at_textColor_vergeColor_alignmentSymbol_(composedText, aGraphics, bounds.bottomRight(), StColorValue.Black, StColorValue.White, $("bottomRight"));
	}

	/**
	 * Display the catalog index as a superimpose in the specified bounds.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aRectangle java.awt.Rectangle
	 * @category displaying
	 */
	protected void displaySuperimposeCatalogIndexInBounds_(Graphics aGraphics, Rectangle aRectangle) {
		if (this.hasCatalogModel() == false) {
			return;
		}
		if (this.catalogModel().visibleCatalogIndex() == false) {
			return;
		}
		if (this.displayObject() == null) {
			return;
		}

		int catalogIndex = -1;
		for (int i = 0; i < this.catalogModel().numberOfElements(); i++) {
			if (this.catalogModel().elements().get(i).displayObject() == this.displayObject()) {
				catalogIndex = i;
				break;
			}
		}
		if (catalogIndex < 0) {
			return;
		}

		StRectangle bounds = new StRectangle(aRectangle.getSize()).insetBy_(1);
		StComposedText composedText = new StComposedText(String.valueOf(catalogIndex + 1));
		this.displayComposedText_on_at_textColor_vergeColor_alignmentSymbol_(composedText, aGraphics, bounds.topLeft(), StColorValue.Blue, StColorValue.White, $("topLeft"));
	}

	/**
	 * Display the selection in the specified bounds.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aRectangle java.awt.Rectangle
	 * @category displaying
	 */
	protected void displaySelectionInBounds_(Graphics aGraphics, Rectangle aRectangle) {
		if (this.hasCatalogModel() == false) {
			return;
		}

		JunOpenGLCatalogElement catalogElement = this.catalogModel().elementOfViewer_(this);
		if (this.catalogModel().selections().contains(catalogElement)) {
			aGraphics.setColor(JunOpenGLCatalogModel.DefaultBorderSelectedColor);
		} else {
			aGraphics.setColor(JunOpenGLCatalogModel.DefaultBorderNormalColor);
		}

		aGraphics.drawRect(0, 0, aRectangle.width - 1, aRectangle.height - 1);
	}

	/**
	 * Convert the receiver as an image.
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#asImage()
	 * @category converting
	 */
	@Override
	public StImage asImage() {
		StView aView = this.getView();

		Dimension imageExtent;
		if (aView == null || aView.toComponent().isVisible() == false) {
			imageExtent = this.defaultImageExtent();
		} else {
			imageExtent = aView.toComponent().getSize();
		}

		return this.asImageExtent_(imageExtent);
	}

	/**
	 * Convert the receiver as an image with the specified extent.
	 * 
	 * @param aDimension java.awt.Dimension
	 * @return jp.co.sra.smalltalk.StImage
	 * @category converting
	 */
	public StImage asImageExtent_(Dimension aDimension) {
		StImage anImage = super.asImageExtent_(aDimension);

		Graphics aGraphics = null;
		try {
			aGraphics = anImage.image().getGraphics();

			Rectangle aRectangle = new Rectangle(aDimension);
			this.displaySuperimposeObjectNameInBounds_(aGraphics, aRectangle);
			this.displaySuperimposeCatalogIndexInBounds_(aGraphics, aRectangle);

		} finally {
			if (aGraphics != null) {
				aGraphics.dispose();
				aGraphics = null;
			}
		}

		return anImage;
	}

	/**
	 * Answer true if the receiver has a catalog model, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	protected boolean hasCatalogModel() {
		return catalogModel != null;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#defaultView()
	 * @category interface opening
	 */
	@Override
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			// TODO
			return new JunOpenGLCatalogViewerViewSwing(this);
		} else {
			return new JunOpenGLCatalogViewerViewSwing(this);
		}
	}

	/**
	 * Answer my menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		return null;
	}

	/**
	 * Answer my popup menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StPopupMenu
	 * @see jp.co.sra.smalltalk.StApplicationModel#_popupMenu()
	 * @category resources
	 */
	@Override
	public StPopupMenu _popupMenu() {
		if (this.displayObject() == null) {
			return null;
		}

		if (_popupMenu == null) {
			_popupMenu = new StPopupMenu();
			_popupMenu.add(new StMenuItem($String("Open Viewfinder"), new MenuPerformer(this, "openDisplayViewer")));
			_popupMenu.addSeparator();
			_popupMenu.add(new StMenuItem($String("Reset"), new MenuPerformer(this, "resetView")));
		}
		return _popupMenu;
	}

	/**
	 * Open a display viewer.
	 * 
	 * @category menu messages
	 */
	public void openDisplayViewer() {
		if (this.displayObject() == null) {
			return;
		}
		if (this.catalogModel() == null) {
			return;
		}

		final JunOpenGLCatalogViewfinder aViewer = new JunOpenGLCatalogViewfinder();
		StView aView = this.getView();
		if (aView == null) {
			aViewer.open();
		} else {
			Dimension size = aView.toComponent().getSize();
			Point p = new Point(size.width / 2, size.height / 2);
			SwingUtilities.convertPointToScreen(p, aView.toComponent());

			StRectangle box = new StRectangle(0, 0, 308, 346);
			box = box.align_with_(box.center(), p);
			aViewer.openIn_(_MakeRectangleVisible(box.toRectangle()));
		}

		this.catalogElement().applyPreferencesTo_(aViewer);
		aViewer.defaultProjectionTable_(this.defaultProjectionTable());
		aViewer.projectionTable_(this.projectionTable());
		aViewer.closeBlock_(new StBlockClosure0<Void>() {
			public Void value() {
				if (JunSensorUtility.ShiftDown() == false) {
					JunOpenGLCatalogViewer self = JunOpenGLCatalogViewer.this;
					JunOpenGLCatalogViewer.this.catalogModel().snapshot_in_to_(self.catalogElement(), aViewer, self);
				}
				return null;
			}
		});
	}

	/**
	 * Reset the view.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGL3dModel#resetView()
	 * @category menu messages
	 */
	@Override
	public void resetView() {
		super.resetView();
		this.applyPreferencesToElement();
	}

}
