package jp.co.sra.jun.opengl.catalog;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.Window;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceAdapter;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.MouseEvent;
import java.io.IOException;

import javax.swing.SwingUtilities;

import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.goodies.cursors.JunCursors;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayController;

/**
 * JunOpenGLCatalogViewerController class
 * 
 *  @author    nisinaka
 *  @created   2009/06/15 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun754 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLCatalogViewerController.java,v 9.3 2010/03/09 09:17:34 nisinaka Exp $
 */
public class JunOpenGLCatalogViewerController extends JunOpenGLDisplayController {

	protected static DataFlavor MyDataFlavor;
	static {
		try {
			MyDataFlavor = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType + ";class=" + JunOpenGLCatalogViewerController.class.getName());
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	protected StSymbol state;
	protected Thread _waitThread;

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#initialize()
	 * @category initialize-release
	 */
	@Override
	public void initialize() {
		super.initialize();
		state = null;
		_waitThread = null;
	}

	/**
	 * Add myself as a listener of the view.
	 * 
	 * @param aView jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#buildListener(jp.co.sra.smalltalk.StView)
	 * @category initialize-release
	 */
	@Override
	protected void buildListener(StView newView) {
		super.buildListener(newView);

		DragSource dragSource = new DragSource();
		dragSource.createDefaultDragGestureRecognizer(this.getOpenGLDrawable().toComponent(), DnDConstants.ACTION_COPY_OR_MOVE, new DragGestureListener() {
			public void dragGestureRecognized(DragGestureEvent dge) {
				if (state == $("move")) {
					JunOpenGLCatalogViewerController.this._redButtonReleasedActivity();

					dge.startDrag(DragSource.DefaultMoveDrop, new Transferable() {
						public DataFlavor[] getTransferDataFlavors() {
							return new DataFlavor[] { MyDataFlavor };
						}

						public boolean isDataFlavorSupported(DataFlavor flavor) {
							return MyDataFlavor.equals(flavor);
						}

						public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
							return JunOpenGLCatalogViewerController.this;
						}
					}, new DragSourceAdapter() {
					});
				}
			}
		});

		new DropTarget(this.getOpenGLDrawable().toComponent(), new DropTargetAdapter() {
			public void drop(DropTargetDropEvent dtde) {
				Transferable transferable = dtde.getTransferable();
				if (transferable.isDataFlavorSupported(MyDataFlavor) == false) {
					dtde.rejectDrop();
					return;
				}

				Object transferData = null;
				dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				try {
					transferData = transferable.getTransferData(MyDataFlavor);
					if (transferData instanceof JunOpenGLCatalogViewerController) {
						JunOpenGLCatalogViewerController dragSourceController = (JunOpenGLCatalogViewerController) transferData;
						JunOpenGLCatalogViewerController.this.moveActivity(dragSourceController);
					}
					dtde.dropComplete(true);
				} catch (UnsupportedFlavorException e1) {
					dtde.dropComplete(false);
				} catch (IOException e1) {
					dtde.dropComplete(false);
				}
			}
		});
	}

	/**
	 * Answer my model as JunOpenGLCatalogViewer.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewer
	 * @category model accessing
	 */
	public JunOpenGLCatalogViewer getOpenGLCatalogViewer() {
		return (JunOpenGLCatalogViewer) this.model();
	}

	/**
	 * Answer my view as JunOpenGLCatalogViewerView.
	 * 
	 * @return jp.co.sra.jun.opengl.catalog.JunOpenGLCatalogViewerView
	 * @category view accessing
	 */
	public JunOpenGLCatalogViewerView getOpenGLCatalogViewerView() {
		return (JunOpenGLCatalogViewerView) this.view();
	}

	/**
	 * Invoked when a mouse is entered on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#mouseEntered(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mouseEntered(MouseEvent e) {
		// do not set a cursor here.
	}

	/**
	 * Invoked when a mouse is exited on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#mouseExited(java.awt.event.MouseEvent)
	 * @category mouse events
	 */
	@Override
	public void mouseExited(MouseEvent e) {
		// do not set a cursor here.
	}

	/**
	 * Activity for the red button pressed.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#redButtonPressedActivity(java.awt.event.MouseEvent)
	 * @category control defaults
	 */
	@Override
	protected void redButtonPressedActivity(MouseEvent e) {
		_waitThread = new Thread() {
			public void run() {
				synchronized (JunOpenGLCatalogViewerController.this) {
					try {
						JunOpenGLCatalogViewerController.this.wait(250);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}

					if (_waitThread == null) {
						// The single or double click activity has been done.
					} else if (state == null) {
						state = $("move");
						JunOpenGLCatalogViewerController.this.setMouseCursor(DragSource.DefaultMoveDrop);
					}

					_waitThread = null;
				}
			}
		};
		_waitThread.start();

		if (e.isAltDown()) {
			state = $("grab");
			this.setMouseCursor(JunCursors.HandCursor());
		}

		// This may flush the movement vector, though.
		super.redButtonPressedActivity(e);

		// So set the new movement vector to the catalog element.
		this.getOpenGLCatalogViewer().applyPreferencesToElement();
	}

	/**
	 * Activity for the red button released.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#redButtonReleasedActivity(java.awt.event.MouseEvent)
	 * @category control defaults
	 */
	@Override
	protected void redButtonReleasedActivity(MouseEvent e) {
		if (_startPoint != null) {
			synchronized (this) {
				if (state == null) {
					if (_waitThread != null && _waitThread.isAlive()) {
						_waitThread = null;
						this.notify();

						switch (e.getClickCount()) {
							case 1:
								state = $("singleClick");
								break;
							case 2:
								state = $("doubleClick");
								break;
						}
					}
				}
			}

			if (state == $("singleClick")) {
				this.clickActivity(e);

			} else if (state == $("doubleClick")) {
				this.doubleClickActivity(e);

			} else if (state == $("grab")) {
				if (System.currentTimeMillis() - _pressTime <= _InertiaTime) {
					if (this.movementVector() != null) {
						this.createMovementProcess();
					}
				}
			}

			_startPoint = null;
		}

		this._redButtonReleasedActivity();
	}

	/**
	 * The post redButtonReleasedActivity activity.
	 * 
	 * @category private
	 */
	private void _redButtonReleasedActivity() {
		this.restoreMouseCursor();
		state = null;
	}

	/**
	 * Activity for the yellow button.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#yellowButtonActivity(java.awt.event.MouseEvent)
	 * @category control defaults
	 */
	@Override
	protected void yellowButtonActivity(MouseEvent e) {
		if (e.isAltDown()) {
			// TODO:
			// this.dragActivityFrom_to_with_();
		} else {
			this.flushMovementVector();
			this.view()._showPopupMenu(e.getX(), e.getY());
		}
	}

	/**
	 * Activity for the single click.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @category control activities
	 */
	protected void clickActivity(MouseEvent e) {
		this.getOpenGLCatalogViewer().clickAt_(this.mouse2dPoint_(e.getPoint()), e.isShiftDown());
	}

	/**
	 * Activity for the double click.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @category control activities
	 */
	protected void doubleClickActivity(MouseEvent e) {
		this.getOpenGLCatalogViewer().doubleClickAt_(this.mouse2dPoint_(e.getPoint()));
	}

	/**
	 * Activity for the move.
	 * 
	 * @param dragSourceController
	 * @category control activities
	 */
	protected void moveActivity(JunOpenGLCatalogViewerController dragSourceController) {
		this.getOpenGLCatalogViewer().movedFrom_(dragSourceController.getOpenGLCatalogViewer());
	}

	/**
	 * Action for the mouse drag.
	 * 
	 * @param fromPoint java.awt.Point
	 * @param toPoint java.awt.Point
	 * @param event java.awt.event.MouseEvent
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayController#mouseDraggedFrom_to_with_(java.awt.Point, java.awt.Point, java.awt.event.MouseEvent)
	 * @category control defaults
	 */
	@Override
	protected void mouseDraggedFrom_to_with_(Point fromPoint, Point toPoint, MouseEvent event) {
		if (state == $("grab")) {
			this.grabActivityFrom_to_(fromPoint, toPoint);
		} else if (state == $("drag")) {
			this.dragActivityFrom_to_with_(fromPoint, toPoint, event);
		}
	}

	/**
	 * Set the mouse cursor.
	 * 
	 * @param aCursor
	 * @category private
	 */
	private void setMouseCursor(Cursor aCursor) {
		Window aWindow = SwingUtilities.getWindowAncestor(this.getOpenGLCatalogViewerView().toComponent());

		_savedCursor = aWindow.getCursor();
		aWindow.setCursor(aCursor);

	}

	/**
	 * Restore the mouse cursor.
	 * 
	 * @category private
	 */
	private void restoreMouseCursor() {
		if (_savedCursor != null) {
			Window aWindow = SwingUtilities.getWindowAncestor(this.getOpenGLCatalogViewerView().toComponent());
			aWindow.setCursor(_savedCursor);
			_savedCursor = null;
		}
	}

}
