package jp.co.sra.jun.opengl.display;

import java.awt.Dimension;
import java.awt.Graphics;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.opengl.support.JunOpenGLDrawable;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderer;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLAngledDisplayViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2006/10/16 (by m-asada)
 *  @updated   2007/08/27 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun621 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLAngledDisplayViewSwing.java,v 9.9 2010/03/09 09:17:36 nisinaka Exp $
 */
public class JunOpenGLAngledDisplayViewSwing extends JunOpenGLDisplayViewSwing implements JunOpenGLAngledDisplayView {

	protected JunAngle angle;

	/**
	 * Create a new instance of <code>JunOpenGLAngledDisplayViewSwing</code> and initialize it.
	 * 
	 * @param anOpenGLStereoDisplayModel jp.co.sra.jun.opengl.display.JunOpenGLStereoDisplayModel
	 * @param anAngle jp.co.sra.jun.geometry.basic.JunAngle
	 * @category Instance creation
	 */
	public JunOpenGLAngledDisplayViewSwing(JunOpenGLStereoDisplayModel anOpenGLStereoDisplayModel, JunAngle anAngle) {
		super(anOpenGLStereoDisplayModel);
		this.angle_(anAngle);
		this.getOpenGLDrawable().setRenderer(new JunOpenGLRenderer() {
			public void renderOn_(JunOpenGLDrawable aDrawable) {
				JunOpenGLRenderingContext aRenderingContext = new JunOpenGLRenderingContext(aDrawable);
				if (angle() == null) {
					getOpenGLStereoDisplayModel().renderOn_(aRenderingContext);
				} else {
					getOpenGLStereoDisplayModel().renderOn_angle_(aRenderingContext, angle());
				}
			}

			public void superimposeOn_(Graphics aGraphics, JunOpenGLDrawable aDrawable) {
			}
		});
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayViewSwing#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		angle = null;
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.getOpenGLDrawable().toComponent(), WindowSpecLayout.Position(0f, 0, 0f, 0, 1f, 0, 1f, 0));
		this.setPreferredSize(new Dimension(300, 300));
	}

	/**
	 * Answer the receiver's model as JunOpenGLStereoDisplayModel.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLStereoDisplayModel
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLAngledDisplayView#getOpenGLStereoDisplayModel()
	 * @category model accessing
	 */
	public JunOpenGLStereoDisplayModel getOpenGLStereoDisplayModel() {
		return (JunOpenGLStereoDisplayModel) this.model();
	}

	/**
	 * Answer the receiver's angle.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.JunAngle
	 * @category accessing
	 */
	public JunAngle angle() {
		return angle;
	}

	/**
	 * Set the receiver's angle.
	 * 
	 * @param anAngle jp.co.sra.jun.geometry.basic.JunAngle
	 * @category accessing
	 */
	public void angle_(JunAngle anAngle) {
		angle = anAngle;
	}

	/**
	 * Receive a change notice from an object of whom the receiver is a
	 * dependent.  The argument anAspectSymbol is typically a Symbol that
	 * indicates what change has occurred.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.DependentListener#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayViewSwing#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent e) {
		if (this.isShowing() == false) {
			return;
		}

		if (e.getAspect() == $("crossing")) {
			this.getOpenGLDrawable().redisplay();
		} else {
			super.update_(e);
		}
	}

}
