package jp.co.sra.jun.opengl.display;

import java.lang.reflect.Method;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOpenGLDisplayObjects class
 * 
 *  @author    nisinaka
 *  @created   2006/07/25 (by nisinaka)
 *  @updated   2008/05/22 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun682 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayObjects.java,v 9.11 2010/03/09 09:17:36 nisinaka Exp $
 */
public class JunOpenGLDisplayObjects extends JunOpenGLDisplayModel {

	protected JunOpenGLTypicalObjects typicalObjects;

	/**
	 * Create a new instance of JunOpenGLDisplayObjects and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOpenGLDisplayObjects(Method[] methods) {
		this.methods_(methods);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		typicalObjects = null;
	}

	/**
	 * Answer my typical objects.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects
	 * @category accessing
	 */
	public JunOpenGLTypicalObjects typicalObjects() {
		if (typicalObjects == null) {
			typicalObjects = new JunOpenGLTypicalObjects();
			typicalObjects.compute_(new StBlockClosure1<Void, JunOpenGL3dObject>() {
				public Void value(JunOpenGL3dObject anOpenGL3dObject) {
					JunOpenGLDisplayObjects.this.displayObject_(anOpenGL3dObject);
					JunOpenGLDisplayObjects.this.resetView();
					return null;
				}
			});
		}
		return typicalObjects;
	}

	/**
	 * Answer my current methods.
	 * 
	 * @return java.lang.reflect.Method[]
	 * @category accessing
	 */
	public Method[] methods() {
		return this.typicalObjects().methods();
	}

	/**
	 * Set my new methods.
	 * 
	 * @param newMethods java.lang.reflect.Method[]
	 * @category accessing
	 */
	public void methods_(Method[] newMethods) {
		this.typicalObjects().methods_(newMethods);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLDisplayObjectsViewAwt(this);
		} else {
			return new JunOpenGLDisplayObjectsViewSwing(this);
		}
	}

	/**
	 * Answer this window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Display Objects");
	}

	/**
	 * Answer the class to spawn.
	 * 
	 * @return java.lang.Class
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#classToSpawn()
	 * @category private
	 */
	protected Class<? extends JunOpenGLDisplayModel> classToSpawn() {
		return JunOpenGLDisplayModel.class;
	}

	/**
	 * Answer the methods for creating typical objects.
	 * 
	 * @return java.lang.reflect.Method[]
	 * @category Examples
	 */
	public static Method[] TypicalObjectMethods() {
		return JunOpenGLTypicalObjects.TypicalObjectMethods();
	}

}
