package jp.co.sra.jun.opengl.display;

import java.awt.Dimension;

import jp.co.sra.smalltalk.StController;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.goodies.button.JunButtonViewSwing;
import jp.co.sra.jun.goodies.wheels.JunThumbWheelViewSwing;

/**
 * JunOpenGLDisplayViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2003/05/15 (by nisinaka)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2004/12/02 (by Mitsuhiro Asada)
 *  @updated   2007/08/23 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun475 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayViewSwing.java,v 9.8 2010/03/09 09:17:36 nisinaka Exp $
 */
public class JunOpenGLDisplayViewSwing extends JunOpenGL3dViewSwing implements JunOpenGLDisplayView {

	/**
	 * Create a new instance of JunOpenGLDisplayViewSwing and initialize it.
	 * 
	 * @param anOpenGLDisplayModel jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @category Instance creation
	 */
	public JunOpenGLDisplayViewSwing(JunOpenGLDisplayModel anOpenGLDisplayModel) {
		super(anOpenGLDisplayModel);
	}

	/**
	 * Create a new view for the JunOpenGLDisplayModel which does not contain any widgets.
	 * 
	 * @param anOpenGLDisplayModel jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @return jp.co.sra.smalltalk.StView
	 * @category Utilities
	 */
	public static StView WithoutWidgets(JunOpenGLDisplayModel anOpenGLDisplayModel) {
		return new JunOpenGLDisplayViewSwing(anOpenGLDisplayModel) {
			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.createOpenGLDrawableOnBorderedPanel(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -1));
				this.setPreferredSize(new Dimension(300, 300));
			}
		};
	}

	/**
	 * Release the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#release()
	 * @category initialize-release
	 */
	public void release() {
		if (controller instanceof JunOpenGLDisplayController) {
			this.getOpenGLDisplayController().flushMovementVector();
		}

		super.release();
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.createOpenGLDrawableOnBorderedPanel(), WindowSpecLayout.Position(0f, 20, 0f, 1, 1f, -20, 1f, -20));
		this.add(new JunButtonViewSwing(this.getOpenGLDisplayModel().pickButton()), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 19, 0f, 19));
		this.add(new JunButtonViewSwing(this.getOpenGLDisplayModel().grabButton()), WindowSpecLayout.Position(0f, 1, 0f, 20, 0f, 19, 0f, 38));
		this.add(new JunButtonViewSwing(this.getOpenGLDisplayModel().dragButton()), WindowSpecLayout.Position(0f, 1, 0f, 39, 0f, 19, 0f, 57));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLDisplayModel().xThumbWheel()), WindowSpecLayout.Position(0f, 1, 0.5f, -61, 0f, 19, 0.5f, 61));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLDisplayModel().yThumbWheel()), WindowSpecLayout.Position(0.5f, -61, 1f, -19, 0.5f, 61, 1f, -1));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLDisplayModel().zThumbWheel()), WindowSpecLayout.Position(1f, -19, 0.5f, -61, 1f, -1, 0.5f, 61));
		this.add(new JunButtonViewSwing(this.getOpenGLDisplayModel().focusButton()), WindowSpecLayout.Position(1f, -19, 0.5f, -80, 1f, -1, 0.5f, -62));
		this.add(new JunButtonViewSwing(this.getOpenGLDisplayModel().dollyButton()), WindowSpecLayout.Position(1f, -19, 0.5f, 62, 1f, -1, 0.5f, 80));
		this.setPreferredSize(new Dimension(300, 300));

		this.getOpenGLDisplayModel().builder().componentAt_put_($("openGLView"), this.getOpenGLView().toComponent());
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayView#getOpenGLDisplayModel()
	 * @category model accessing
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my controller as JunOpenGLDisplayController.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLDisplayController
	 * @category controller accessing
	 */
	public JunOpenGLDisplayController getOpenGLDisplayController() {
		return (JunOpenGLDisplayController) this.controller();
	}

	/**
	 * Answer my default controller.
	 * 
	 * @return jp.co.sra.smalltalk.StController
	 * @see jp.co.sra.smalltalk.StViewJPanel#defaultController()
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunOpenGLDisplayController();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGL3dViewWrapper
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayView#getOpenGLView()
	 * @deprecated since Jun683, the former JunOpenGL3dView is replaced with JunOpenGLDrawable.
	 * @category accessing
	 */
	public JunOpenGL3dViewWrapper getOpenGLView() {
		return new JunOpenGL3dViewWrapper(this);
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGL3dView
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayViewSwing#openGLView()
	 * @deprecated since Jun683, the former JunOpenGL3dView is replaced with JunOpenGLDrawable.
	 * @category user interface
	 */
	protected JunOpenGL3dView openGLView() {
		if (openGLView == null) {
			openGLView = new JunOpenGL3dViewWrapper(this);
		}
		return openGLView;
	}

	protected JunOpenGL3dView openGLView;

}
