package jp.co.sra.jun.opengl.display;

import java.awt.Dimension;

import jp.co.sra.smalltalk.StController;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.goodies.button.JunButtonViewSwing;
import jp.co.sra.jun.goodies.wheels.JunThumbWheelViewSwing;

/**
 * JunOpenGLShowViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2000/01/09 (by MATSUDA Ryouichi)
 *  @updated   2003/05/15 (by nisinaka)
 *  @updated   2004/09/22 (by nisinaka)
 *  @updated   2007/08/24 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLShowViewSwing.java,v 9.8 2010/03/09 09:17:37 nisinaka Exp $
 */
public class JunOpenGLShowViewSwing extends JunOpenGLDisplayViewSwing implements JunOpenGLShowView {

	/**
	 * Create a new instance of <code>JunOpenGLShowViewSwing</code> and initialize it.
	 * 
	 * @param anOpenGLShowModel jp.co.sra.jun.opengl.display.JunOpenGLShowModel
	 * @category Instance creation
	 */
	public JunOpenGLShowViewSwing(JunOpenGLShowModel anOpenGLShowModel) {
		super(anOpenGLShowModel);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.createOpenGLDrawableOnBorderedPanel(), WindowSpecLayout.Position(0f, 20, 0f, 1, 1f, -20, 1f, -20));
		this.add(new JunButtonViewSwing(this.getOpenGLShowModel().bullButton()), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 19, 0f, 19));
		this.add(new JunButtonViewSwing(this.getOpenGLShowModel().grabButton()), WindowSpecLayout.Position(0f, 1, 0f, 20, 0f, 19, 0f, 38));
		this.add(new JunButtonViewSwing(this.getOpenGLShowModel().dragButton()), WindowSpecLayout.Position(0f, 1, 0f, 39, 0f, 19, 0f, 57));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLShowModel().xThumbWheel()), WindowSpecLayout.Position(0f, 1, 0.5f, -61, 0f, 19, 0.5f, 61));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLShowModel().yThumbWheel()), WindowSpecLayout.Position(0.5f, -61, 1f, -19, 0.5f, 61, 1f, -1));
		this.add(new JunThumbWheelViewSwing(this.getOpenGLShowModel().zThumbWheel()), WindowSpecLayout.Position(1f, -19, 0.5f, -61, 1f, -1, 0.5f, 61));
		this.setPreferredSize(new Dimension(300, 300));
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayView#getOpenGLDisplayModel()
	 * @category model accessing
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return this.getOpenGLShowModel().displayModel();
	}

	/**
	 * Answer my model as JunOpenGLShowModel.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLShowModel
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLShowView#getOpenGLShowModel()
	 * @category model accessing
	 */
	public JunOpenGLShowModel getOpenGLShowModel() {
		return (JunOpenGLShowModel) this.model();
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return jp.co.sra.smalltalk.StController
	 * @see jp.co.sra.smalltalk.StViewJPanel#defaultController()
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunOpenGLShowController();
	}

	/**
	 * Answer my show view.
	 *
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLShowViewSwing#showView()
	 * @return jp.co.sra.jun.opengl.display.JunOpenGL3dViewSwing
	 * @deprecated since Jun683, the former JunOpenGL3dView is replaced with JunOpenGLDrawable.
	 * @category user interface
	 */
	public JunOpenGL3dView showView() {
		if (showView == null) {
			showView = new JunOpenGL3dViewWrapper(this);
		}
		return showView;
	}

	protected JunOpenGL3dView showView;

}
