package jp.co.sra.jun.opengl.display;

import java.awt.Point;
import java.awt.Window;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.Comparator;
import java.util.TreeSet;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockValue;
import jp.co.sra.smalltalk.StObject;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;
import jp.co.sra.smalltalk.menu.MenuPerformer;
import jp.co.sra.smalltalk.menu.StMenu;
import jp.co.sra.smalltalk.menu.StMenuBar;
import jp.co.sra.smalltalk.menu.StMenuItem;
import jp.co.sra.smalltalk.menu.StPopupMenu;
import jp.co.sra.smalltalk.menu.StRadioButtonGroup;
import jp.co.sra.smalltalk.menu.StRadioButtonMenuItem;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.framework.JunAbstractModel;
import jp.co.sra.jun.system.support.Category;

/**
 * JunOpenGLTypicalObjects class
 * 
 *  @author    nisinaka
 *  @created   2009/08/19 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun738 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLTypicalObjects.java,v 9.3 2010/03/09 09:17:37 nisinaka Exp $
 */
public class JunOpenGLTypicalObjects extends JunAbstractModel {

	protected static Class<? extends JunOpenGLDisplayModel> DefalutClassToSpawn = JunOpenGLDisplayModel.class;

	protected Method[] methods;
	protected int selectionIndex;
	protected StValueHolder<JunOpenGL3dObject> typicalObjectHolder;
	protected StMenuBar _menuBar;
	protected StPopupMenu _popupMenu;

	/**
	 * Create a new instance of JunOpenGLTypicalObjects and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunOpenGLTypicalObjects() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLTypicalObjects and initialize it.
	 *
	 * @param methods java.lang.reflect.Method[]
	 * @category Instance creation
	 */
	public JunOpenGLTypicalObjects(Method[] methods) {
		this.methods_(methods);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractModel#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		methods = null;
		selectionIndex = -1;
		typicalObjectHolder = new StValueHolder<JunOpenGL3dObject>(null);
	}

	/**
	 * Answer the method at the specified index.
	 * 
	 * @param index int
	 * @return java.lang.reflect.Method
	 * @category accessing
	 */
	public Method methodAt(int index) {
		if (this.methods() == null) {
			return null;
		}

		Method[] methods = this.methods();
		if (index < 0 || methods.length <= index) {
			return null;
		}

		return methods[index];
	}

	/**
	 * Answer my current methods.
	 * 
	 * @return java.lang.reflect.Method[]
	 * @category accessing
	 */
	public Method[] methods() {
		return methods;
	}

	/**
	 * Set my new methods.
	 * 
	 * @param newMethods java.lang.reflect.Method[]
	 * @category accessing
	 */
	public void methods_(Method[] newMethods) {
		methods = newMethods;
		this.remakeList();
	}

	/**
	 * Answer my current selection index.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int selectionIndex() {
		return selectionIndex;
	}

	/**
	 * Set my new selection index.
	 * 
	 * @param index int
	 * @category accessing
	 */
	public void selectionIndex_(int index) {
		if (index == selectionIndex) {
			return;
		}

		JunOpenGL3dObject typicalObject = null;
		Method aMethod = this.methodAt(index);
		if (aMethod != null) {
			try {
				Object anObject = aMethod.invoke(null, (Object[]) null);
				if (anObject instanceof JunOpenGL3dObject) {
					typicalObject = (JunOpenGL3dObject) anObject;
				} else if (anObject instanceof StObject) {
					StObject anStObject = (StObject) anObject;
					if (anStObject.respondsTo_("asJunOpenGL3dObject")) {
						typicalObject = (JunOpenGL3dObject) anStObject.perform_("asJunOpenGL3dObject");
					}
				}
			} catch (Exception e) {
				throw new Error(e.getMessage());
			}
		}

		typicalObjectHolder.value_(typicalObject);
		this.updateMenuIndication();

		selectionIndex = index;
		this.changed_($("selection"));
	}

	/**
	 * Answer my currently selected typical object.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category accessing
	 */
	public JunOpenGL3dObject object() {
		return typicalObjectHolder.value();
	}

	/**
	 * Answer the class to spawn.
	 * 
	 * @return java.lang.Class
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#classToSpawn()
	 * @category attributes
	 */
	@SuppressWarnings("unchecked")
	public Class<? extends JunOpenGLDisplayModel> classToSpawn() {
		if (this.attributeTable().includesKey_($("classToSpawn")) == false) {
			this.attributeTable().at_put_($("classToSpawn"), DefalutClassToSpawn);
		}
		return (Class<? extends JunOpenGLDisplayModel>) this.attributeTable().at_($("classToSpawn"));
	}

	/**
	 * Set the class to spawn.
	 * 
	 * @param aClass java.lang.Class<? extends JunOpenGLDisplayModel>
	 * @category attributes
	 */
	public void classToSpawn_(Class<? extends JunOpenGLDisplayModel> aClass) {
		this.attributeTable().at_put_($("classToSpawn"), aClass);
	}

	/**
	 * Answer the symbol to sort.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category attributes
	 */
	protected StSymbol symbolToSort() {
		if (this.attributeTable().includesKey_($("symbolToSort")) == false) {
			this.attributeTable().at_put_($("symbolToSort"), $("category"));
		}
		return (StSymbol) this.attributeTable().at_($("symbolToSort"));
	}

	/**
	 * Set the symbol to sort.
	 * 
	 * @param aSymbol jp.co.sra.smalltalk.StSymbol
	 * @category attributes
	 */
	protected void symbolToSort_(StSymbol aSymbol) {
		if (aSymbol != $("selector") && aSymbol != $("category") && aSymbol != $("receiver")) {
			return;
		}

		this.attributeTable().at_put_($("symbolToSort"), aSymbol);
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1<R, JunOpenGL3dObject>
	 * @return jp.co.sra.smalltalk.StBlockValue<R>
	 * @category constructing
	 */
	public <R> StBlockValue<R> compute_(final StBlockClosure1<R, JunOpenGL3dObject> aBlock) {
		return typicalObjectHolder.compute_(new StBlockClosure1<R, JunOpenGL3dObject>() {
			public R value(JunOpenGL3dObject anOpenGL3dObject) {
				return aBlock.value(anOpenGL3dObject);
			}
		});
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category interface opening
	 */
	@Override
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLTypicalObjectsViewAwt(this);
		} else {
			return new JunOpenGLTypicalObjectsViewSwing(this);
		}
	}

	/**
	 * Answer this window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Typical Objects");
	}

	/**
	 * Answer my menu bar.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenuBar
	 * @see jp.co.sra.smalltalk.StApplicationModel#_menuBar()
	 * @category resources
	 */
	@Override
	public StMenuBar _menuBar() {
		if (_menuBar == null) {
			StMenuBar aMenuBar = new StMenuBar();
			aMenuBar.add(this._createViewMenu());
			_menuBar = aMenuBar;
		}
		return _menuBar;
	}

	/**
	 * Create a "View" menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StMenu
	 * @category resources
	 */
	protected StMenu _createViewMenu() {
		StMenu viewMenu = new StMenu($String("View"), $("viewMenu"));
		viewMenu.add(new StMenuItem($String("Spawn"), $("spawnObject"), new MenuPerformer(this, "spawnObject")));
		viewMenu.addSeparator();
		StRadioButtonGroup sortGroup = new StRadioButtonGroup();
		viewMenu.add(new StRadioButtonMenuItem($String("Sort by selector"), $("sortBySelector"), sortGroup, new MenuPerformer(this, "sortBySelector")));
		viewMenu.add(new StRadioButtonMenuItem($String("Sort by category"), $("sortByCategory"), sortGroup, new MenuPerformer(this, "sortByCategory")));
		viewMenu.add(new StRadioButtonMenuItem($String("Sort by receiver"), $("sortByRecevier"), sortGroup, new MenuPerformer(this, "sortByReceiver")));
		return viewMenu;
	}

	/**
	 * Answer my popup menu.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StPopupMenu
	 * @see jp.co.sra.smalltalk.StApplicationModel#_popupMenu()
	 * @category resources
	 */
	@Override
	public StPopupMenu _popupMenu() {
		if (_popupMenu == null) {
			StMenu viewMenu = (StMenu) this._menuBar().atNameKey_($("viewMenu"));
			if (viewMenu != null) {
				StPopupMenu aPopupMenu = new StPopupMenu();
				aPopupMenu.addAll(viewMenu.menuItems());
				_popupMenu = aPopupMenu;
			}
		}
		return _popupMenu;
	}

	/**
	 * Update the menu indication.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunApplicationModel#updateMenuIndication()
	 * @category menu accessing
	 */
	@Override
	public void updateMenuIndication() {
		this.updateViewMenuIndication();
	}

	/**
	 * Update the view menu indication.
	 * 
	 * @category menu accessing
	 */
	protected void updateViewMenuIndication() {
		StMenu aMenu = (StMenu) this._menuBar().atNameKey_($("viewMenu"));
		if (aMenu == null) {
			return;
		}

		StMenuItem menuItem;
		menuItem = aMenu.atNameKey_($("spawnObject"));
		if (menuItem != null) {
			menuItem.beEnabled(this.object() != null);
		}

		StRadioButtonMenuItem radioButtonMenuItem;
		radioButtonMenuItem = (StRadioButtonMenuItem) aMenu.atNameKey_($("sortBySelector"));
		if (radioButtonMenuItem != null) {
			radioButtonMenuItem.beSelected(this.symbolToSort() == $("selector"));
		}
		radioButtonMenuItem = (StRadioButtonMenuItem) aMenu.atNameKey_($("sortByCategory"));
		if (radioButtonMenuItem != null) {
			radioButtonMenuItem.beSelected(this.symbolToSort() == $("category"));
		}
		radioButtonMenuItem = (StRadioButtonMenuItem) aMenu.atNameKey_($("sortByReceiver"));
		if (radioButtonMenuItem != null) {
			radioButtonMenuItem.beSelected(this.symbolToSort() == $("receiver"));
		}
	}

	/**
	 * Spawn the selected object with a viewfinder.
	 * 
	 * @category menu messages
	 */
	public void spawnObject() {
		if (this.object() == null) {
			return;
		}

		JunOpenGLDisplayModel displayModel = null;
		try {
			displayModel = this.classToSpawn().newInstance();
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}
		displayModel.displayObject_(this.object());

		Window aWindow = this.getWindow();
		if (aWindow == null) {
			displayModel.open();
		} else {
			StRectangle box = new StRectangle(aWindow.getBounds());
			displayModel.openAt_(new Point(box.right() + 8, box.top() + 0));
		}

		this.closeTogether().add(displayModel);
	}

	/**
	 * Sort the method list by category.
	 * 
	 * @category menu messages
	 */
	public void sortByCategory() {
		this.symbolToSort_($("category"));
		this.remakeList();
	}

	/**
	 * Sort the method list by receiver.
	 * 
	 * @category menu messages
	 */
	public void sortByReceiver() {
		this.symbolToSort_($("receiver"));
		this.remakeList();
	}

	/**
	 * Sort the method list by selector.
	 * 
	 * @category menu messages
	 */
	public void sortBySelector() {
		this.symbolToSort_($("selector"));
		this.remakeList();
	}

	/**
	 * Answer the category name of the specified method.
	 * 
	 * @param aMethod java.lang.reflect.Method
	 * @return java.lang.String
	 * @category private
	 */
	protected String categoryNameOfMethod_(Method aMethod) {
		Category category = aMethod.getAnnotation(Category.class);
		String categoryname = (category != null) ? category.value() : "***";
		if (categoryname.startsWith("typical objects ")) {
			categoryname = categoryname.substring(16);
		}
		return categoryname;
	}

	/**
	 * Remake the method list.
	 * 
	 * @category private
	 */
	protected synchronized void remakeList() {
		if (methods == null) {
			return;
		}

		Method selectedMethod = this.methodAt(this.selectionIndex());

		if (this.symbolToSort() == $("selector")) {
			Arrays.sort(methods, new Comparator<Method>() {
				public int compare(Method m1, Method m2) {
					int result = m1.getName().compareTo(m2.getName());
					if (result == 0) {
						result = categoryNameOfMethod_(m1).compareTo(categoryNameOfMethod_(m2));
						if (result == 0) {
							result = m1.getDeclaringClass().getName().compareTo(m2.getDeclaringClass().getName());
						}
					}
					return result;
				}
			});
		} else if (this.symbolToSort() == $("receiver")) {
			Arrays.sort(methods, new Comparator<Method>() {
				public int compare(Method m1, Method m2) {
					int result = m1.getDeclaringClass().getName().compareTo(m2.getDeclaringClass().getName());
					if (result == 0) {
						result = m1.getName().compareTo(m2.getName());
						if (result == 0) {
							result = categoryNameOfMethod_(m1).compareTo(categoryNameOfMethod_(m2));
						}
					}
					return result;
				}
			});
		} else if (this.symbolToSort() == $("category")) {
			Arrays.sort(methods, new Comparator<Method>() {
				public int compare(Method m1, Method m2) {
					int result = categoryNameOfMethod_(m1).compareTo(categoryNameOfMethod_(m2));
					if (result == 0) {
						result = m1.getName().compareTo(m2.getName());
						if (result == 0) {
							result = m1.getDeclaringClass().getName().compareTo(m2.getDeclaringClass().getName());
						}
					}
					return result;
				}
			});
		}

		if (selectedMethod != null) {
			int index = Arrays.asList(this.methods()).indexOf(selectedMethod);
			if (index >= 0) {
				this.selectionIndex_(index);
			}
		}

		this.changed_($("methods"));
	}

	/**
	 * Answer the methods for creating typical objects.
	 * 
	 * @return java.lang.reflect.Method[]
	 * @category Examples
	 */
	public static Method[] TypicalObjectMethods() {
		TreeSet<Method> aTreeSet = new TreeSet<Method>(new Comparator<Method>() {
			public int compare(Method m1, Method m2) {
				return m1.getName().compareTo(m2.getName());
			}
		});

		Class<?> aClass = JunOpenGL3dObject.class;
		Method[] methods = aClass.getDeclaredMethods();
		for (int i = 0; i < methods.length; i++) {
			Method aMethod = methods[i];
			int modifiers = aMethod.getModifiers();
			if (Modifier.isPublic(modifiers) == false) {
				continue;
			}
			if (Modifier.isStatic(modifiers) == false) {
				continue;
			}
			if (aMethod.getParameterTypes().length > 0) {
				continue;
			}
			if (JunOpenGL3dObject.class.isAssignableFrom(aMethod.getReturnType()) == false) {
				continue;
			}
			if (aMethod.getName().startsWith("Image")) {
				continue;
			}

			aTreeSet.add(aMethod);
		}

		return aTreeSet.toArray(new Method[aTreeSet.size()]);
	}

}
