package jp.co.sra.jun.opengl.display;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.List;
import java.awt.PopupMenu;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.lang.reflect.Method;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunAbstractViewPanel;

/**
 * JunOpenGLTypicalObjectsViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2009/08/19 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun738 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLTypicalObjectsViewAwt.java,v 9.3 2010/03/09 09:17:37 nisinaka Exp $
 */
public class JunOpenGLTypicalObjectsViewAwt extends JunAbstractViewPanel implements JunOpenGLTypicalObjectsView {

	protected List methodList;

	/**
	 * Create a new instance of JunOpenGLTypicalObjectsViewAwt and initialize it.
	 *
	 * @param anOpenGLTypicalObjects jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects
	 * @category Instance creation
	 */
	public JunOpenGLTypicalObjectsViewAwt(JunOpenGLTypicalObjects anOpenGLTypicalObjects) {
		super(anOpenGLTypicalObjects);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.methodList(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(250, 300));
	}

	/**
	 * Answer my model as JunOpenGLTypicalObjects.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjectsView#getOpenGLTypicalObjects()
	 * @category model accessing
	 */
	public JunOpenGLTypicalObjects getOpenGLTypicalObjects() {
		return (JunOpenGLTypicalObjects) this.model();
	}

	/**
	 * Answer the component of my method list.
	 * 
	 * @return java.awt.Component
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjectsView#methodListComponent()
	 * @category accessing
	 */
	public Component methodListComponent() {
		return this.methodList();
	}

	/**
	 * Answer the method list.
	 * 
	 * @return java.awt.List
	 * @category user interface
	 */
	protected List methodList() {
		if (methodList == null) {
			methodList = new List();
			methodList.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						getOpenGLTypicalObjects().selectionIndex_(((Number) e.getItem()).intValue());
					}
				}
			});
			methodList.addMouseListener(this.controller());

			this.updateMethodList();
		}
		return methodList;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent e) {
		if (e.getAspect() == $("methods")) {
			this.updateMethodList();
		} else if (e.getAspect() == $("selection")) {
			this.updateSelection();
		} else {
			super.update_(e);
		}
	}

	/**
	 * Update the method list.
	 * 
	 * @category updating
	 */
	protected void updateMethodList() {
		this.methodList().removeAll();

		Method[] methods = this.getOpenGLTypicalObjects().methods();
		for (int i = 0; i < methods.length; i++) {
			Method aMethod = methods[i];
			String selectorname = aMethod.getName();
			String categoryname = "[" + getOpenGLTypicalObjects().categoryNameOfMethod_(aMethod) + "]";
			String receivername = aMethod.getDeclaringClass().getName();
			receivername = receivername.substring(receivername.lastIndexOf('.') + 1);
			receivername = "{" + receivername + "}";

			String label = "";
			if (getOpenGLTypicalObjects().symbolToSort() == $("selector")) {
				label += selectorname + " " + categoryname + " " + receivername;
			} else if (getOpenGLTypicalObjects().symbolToSort() == $("category")) {
				label += categoryname + " " + selectorname + " " + receivername;
			} else if (getOpenGLTypicalObjects().symbolToSort() == $("receiver")) {
				label += receivername + " " + selectorname + " " + categoryname;
			}

			this.methodList().add(label);
		}

		this.updateSelection();
	}

	/**
	 * Update the selection.
	 * 
	 * @category updating
	 */
	protected void updateSelection() {
		int selectionIndex = this.getOpenGLTypicalObjects().selectionIndex();
		if (selectionIndex == this.methodList().getSelectedIndex()) {
			return;
		}

		this.methodList().select(selectionIndex);
		this.methodList().makeVisible(selectionIndex);
	}

	/**
	 * Show the popup menu at the specified point on the view.
	 *
	 * @param x int
	 * @param y int
	 * @see jp.co.sra.smalltalk.StViewPanel#_showPopupMenu(int, int)
	 * @category popup menu
	 */
	@Override
	public void _showPopupMenu(int x, int y) {
		PopupMenu popupMenu = this.popupMenuView();
		if (popupMenu != null) {
			popupMenu.show(this.methodList(), x, y);
		}
	}

}
