package jp.co.sra.jun.opengl.display;

import java.awt.Component;
import java.awt.Dimension;
import java.lang.reflect.Method;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;

/**
 * JunOpenGLTypicalObjectsViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2009/08/19 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun738 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLTypicalObjectsViewSwing.java,v 9.3 2010/03/09 09:17:37 nisinaka Exp $
 */
public class JunOpenGLTypicalObjectsViewSwing extends JunAbstractViewJPanel implements JunOpenGLTypicalObjectsView {

	protected JList methodList;

	/**
	 * Create a new instance of JunOpenGLTypicalObjectsViewSwing and initialize it.
	 *
	 * @param anOpenGLTypicalObjects jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects
	 * @category Instance creation
	 */
	public JunOpenGLTypicalObjectsViewSwing(JunOpenGLTypicalObjects anOpenGLTypicalObjects) {
		super(anOpenGLTypicalObjects);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(new JScrollPane(this.methodList()), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(250, 300));
	}

	/**
	 * Answer my model as JunOpenGLTypicalObjects.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjects
	 * @see jp.co.sra.jun.opengl.display.JunOpenGLTypicalObjectsView#getOpenGLTypicalObjects()
	 * @category model accessing
	 */
	public JunOpenGLTypicalObjects getOpenGLTypicalObjects() {
		return (JunOpenGLTypicalObjects) this.model();
	}

	/**
	 * Answer the method list.
	 * 
	 * @return javax.swing.JList
	 * @category user interface
	 */
	protected JList methodList() {
		if (methodList == null) {
			methodList = new JList(new AbstractListModel() {
				public Object getElementAt(int index) {
					return getOpenGLTypicalObjects().methodAt(index);
				}

				public int getSize() {
					return getOpenGLTypicalObjects().methods().length;
				}
			});
			methodList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			methodList.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if (e.getValueIsAdjusting()) {
						return;
					}

					getOpenGLTypicalObjects().selectionIndex_(methodList.getSelectedIndex());
				}
			});
			methodList.setCellRenderer(new DefaultListCellRenderer() {
				public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
					Method aMethod = (Method) value;
					String selectorname = "<B>" + aMethod.getName() + "</B>";
					String categoryname = "[" + getOpenGLTypicalObjects().categoryNameOfMethod_(aMethod) + "]";
					String receivername = aMethod.getDeclaringClass().getName();
					receivername = receivername.substring(receivername.lastIndexOf('.') + 1);
					receivername = "{" + receivername + "}";

					String label = "<HTML>";
					if (getOpenGLTypicalObjects().symbolToSort() == $("selector")) {
						label += selectorname + " " + categoryname + " " + receivername;
					} else if (getOpenGLTypicalObjects().symbolToSort() == $("category")) {
						label += categoryname + " " + selectorname + " " + receivername;
					} else if (getOpenGLTypicalObjects().symbolToSort() == $("receiver")) {
						label += receivername + " " + selectorname + " " + categoryname;
					}
					label += "</HTML>";

					return super.getListCellRendererComponent(list, label, index, isSelected, cellHasFocus);
				}
			});
			methodList.addMouseListener(this.controller());

			this.updateMethodList();
		}
		return methodList;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent e) {
		if (e.getAspect() == $("methods")) {
			this.updateMethodList();
		} else if (e.getAspect() == $("selection")) {
			this.updateSelection();
		} else {
			super.update_(e);
		}
	}

	/**
	 * Update the method list.
	 * 
	 * @category updating
	 */
	protected void updateMethodList() {
		this.repaint();
		this.updateSelection();
	}

	/**
	 * Update the selection.
	 * 
	 * @category updating
	 */
	protected void updateSelection() {
		int selectionIndex = this.getOpenGLTypicalObjects().selectionIndex();
		if (selectionIndex == this.methodList().getSelectedIndex()) {
			return;
		}

		this.methodList().setSelectedIndex(selectionIndex);
		this.methodList().ensureIndexIsVisible(selectionIndex);
	}

	/**
	 * Show the popup menu at the specified point on the view.
	 *
	 * @param x int
	 * @param y int
	 * @see jp.co.sra.smalltalk.StViewJPanel#_showPopupMenu(int, int)
	 * @category popup menu
	 */
	@Override
	public void _showPopupMenu(int x, int y) {
		JPopupMenu popupMenu = this.popupMenuView();
		if (popupMenu != null) {
			popupMenu.show(this.methodList(), x, y);
		}
	}

}
