package jp.co.sra.jun.opengl.flux;

import java.io.IOException;
import java.io.Serializable;
import java.io.Writer;
import java.util.ArrayList;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.framework.JunAbstractObject;

/**
 * JunOpenGLFluxAbstract class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/25 (by MATSUDA Ryouichi)
 *  @updated   2004/11/12 (by Mitsuhiro Asada)
 *  @updated   2006/10/12 (by nisinaka)
 *  @updated   2008/06/10 (by nisinaka)
 *  @updated   2008/11/04 (by Mitsuhiro Asada)
 *  @updated   2009/03/28 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun721 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxAbstract.java,v 9.11 2010/03/09 09:17:37 nisinaka Exp $
 */
public abstract class JunOpenGLFluxAbstract extends JunAbstractObject implements Serializable {

	protected JunOpenGL3dObject originalObject;

	/**
	 * Create a new instance of <code>JunOpenGLFluxAbstract</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGLFluxAbstract() {
		super();
	}

	/**
	 * Create a new instance of <code>JunOpenGLFluxAbstract</code> and initialize it.
	 *
	 * @param a3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	public JunOpenGLFluxAbstract(JunOpenGL3dObject a3dObject) {
		this();
		this.originalObject_(a3dObject);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	@Override
	protected void initialize() {
		super.initialize();
		originalObject = null;
	}

	/**
	 * Answer my current original object.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category accessing
	 */
	public JunOpenGL3dObject originalObject() {
		return originalObject;
	}

	/**
	 * Set my new original object.
	 * 
	 * @param a3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category accessing
	 */
	public void originalObject_(JunOpenGL3dObject a3dObject) {
		originalObject = a3dObject;
		this.flushBounds();
	}

	/**
	 * Answer my current bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category bounds accessing
	 */
	public Jun3dBoundingBox boundingBox() {
		return this.preferredBoundingBox();
	}

	/**
	 * Answer my preferred bounding box.
	 * 
	 * @return jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox
	 * @category bounds accessing
	 */
	public abstract Jun3dBoundingBox preferredBoundingBox();

	/**
	 * Flush my bounds.
	 * 
	 * @category bounds accessing
	 */
	public void flushBounds() {
	}

	/**
	 * Answer the receiver as an array of points.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @category converting
	 */
	public Jun3dPoint[] asPointArray() {
		final ArrayList<Jun3dPoint> aList = new ArrayList<Jun3dPoint>();
		this.pointsDo_(new StBlockClosure1<Void, Jun3dPoint>() {
			public Void value(Jun3dPoint point) {
				aList.add(point);
				return null;
			}
		});
		return aList.toArray(new Jun3dPoint[aList.size()]);
	}

	/**
	 * Enumerate every points and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category enumerating
	 */
	public abstract <R> R pointsDo_(StBlockClosure1<R, Jun3dPoint> aBlock);

	/**
	 * Enumerate every points and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public void pointsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.pointsDo_(new StBlockClosure1<Void, Jun3dPoint>() {
			public Void value(Jun3dPoint point) {
				aBlock.value_(point);
				return null;
			}
		});
	}

	/**
	 * Convert the transformation as a lisp list.
	 * 
	 * @param aTransformation jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected JunLispList transformationToLispList_(Jun3dTransformation aTransformation) {
		return JunOpenGL3dObject.TransformationToLispList_(aTransformation);
	}

	/**
	 * Write the receiver's list representation on the Writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @category lisp support
	 */
	public void saveOn_(Writer aWriter) throws IOException {
		JunLispList aList = this.toLispList();
		aList.saveOn_(aWriter);
	}

}
