package jp.co.sra.jun.opengl.objects.typical;

import java.awt.Color;
import java.awt.Dimension;

import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StImage;
import jp.co.sra.smalltalk.StInterval;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.goodies.image.support.JunImageAdjuster;
import jp.co.sra.jun.opengl.texture.JunOpenGLTextureTestExamples;

/**
 * JunOpenGL3dTypicalObjectsImage class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2007/08/24 (by m-asada)
 *  @updated   2008/05/08 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun757 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dTypicalObjectsImage.java,v 9.10 2010/03/09 09:17:40 nisinaka Exp $
 */
public class JunOpenGL3dTypicalObjectsImage extends JunOpenGL3dTypicalObjects {

	/**
	 * Typical objects - image earth
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageEarth() {
		return new StImage(JunOpenGLTextureTestExamples.ImageEarth2());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageBMWC() {
		return ImageBMWC_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageBMWC_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(y, x, 1.0f);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageGCWY() {
		return ImageGCWY_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageGCWY_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(x, 1.0f, y);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageKBCG() {
		return ImageKBCG_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageKBCG_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(0.0f, x, y);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageKGYR() {
		return ImageKGYR_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageKGYR_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(x, y, 0.0f);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageKRMB() {
		return ImageKRMB_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageKRMB_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(y, 0.0f, x);
			}
		});
	}

	/**
	 * Typical objects - image moon
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageMoon() {
		return new StImage(JunOpenGLTextureTestExamples.ImageMoon2());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageRBBR() {
		return ImageRBBR_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageRBBR_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return Color.getHSBColor(x.floatValue(), 1.0f, 1.0f - y.floatValue());
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageRGBR() {
		return ImageRGBR_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageRGBR_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				Jun2dPoint aPoint = new Jun2dPoint(x.doubleValue() - 0.5, y.doubleValue() - 0.5);
				double aHue = Math.toDegrees(aPoint.theta()) / 360.0d;
				double aSaturation = aPoint.r() / 0.5;
				return Color.getHSBColor((float) aHue, Math.min((float) aSaturation, 1.0f), 1.0f);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @category Typical objects image
	 */
	public static StImage ImageRYWM() {
		return ImageRYWM_(DefaultImageSize());
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize int
	 * @category Typical objects image
	 */
	public static StImage ImageRYWM_(int imageSize) {
		return ImageSize_colorBlock_(new Dimension(imageSize, imageSize), new StBlockClosure2<Color, Float, Float>() {
			public Color value(Float x, Float y) {
				return new Color(1, y, x);
			}
		});
	}

	/**
	 * Typical objects - objects image
	 * 
	 * @return jp.co.sra.smalltalk.StImage
	 * @param imageSize java.awt.Dimension
	 * @param colorBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @category Typical objects image
	 */
	protected static StImage ImageSize_colorBlock_(Dimension imageSize, StBlockClosure2<Color, Float, Float> colorBlock) {
		final StImage anImage = new StImage(imageSize.width, imageSize.height);
		int divisionSize = Math.min(anImage.width(), anImage.height());
		final StInterval[] xIntervals = (new JunImageAdjuster()).computeIntervals_divisionSize_(anImage.width(), divisionSize);
		final StInterval[] yIntervals = (new JunImageAdjuster()).computeIntervals_divisionSize_(anImage.height(), divisionSize);
		int xHowMany = xIntervals.length - 1;
		int yHowMany = yIntervals.length - 1;
		for (int y = 0; y <= yHowMany; y++) {
			int j = y;
			for (int x = 0; x <= xHowMany; x++) {
				final int i = x;
				Color colorValue = colorBlock.value((float) x / xHowMany, (float) y / yHowMany);
				final int colorIndex = colorValue.getRGB();
				yIntervals[j].do_(new StBlockClosure1<Void, Double>() {
					public Void value(final Double n) {
						xIntervals[i].do_(new StBlockClosure1<Void, Double>() {
							public Void value(Double m) {
								anImage.setPixel(m.intValue(), n.intValue(), colorIndex);
								return null;
							}
						});
						return null;
					}
				});
			}
		}
		return anImage;
	}

}
