package jp.co.sra.jun.opengl.parameterizedbody;

import java.awt.Color;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StModel;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.topology.elements.JunBody;

/**
 * JunPaintedBody class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/08 (by MATSUDA Ryouichi)
 *  @updated   2004/10/21 (by Mitsuhiro Asada)
 *  @updated   2008/06/10 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun582 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPaintedBody.java,v 9.10 2010/03/09 09:17:41 nisinaka Exp $
 */
public class JunPaintedBody extends StModel implements JunParameterizedOrPaintedBody {
	/** The name of the object. */
	protected String name;

	/** The body of the object. */
	protected JunBody body;

	/** The paint of the object. */
	protected Color paint;

	/** The cached3dObject of the object. */
	protected JunOpenGL3dObject cached3dObject;

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunPaintedBody(JunBody aBody) {
		super();
		this.body_(aBody);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aColorValue java.awt.Color
	 * @category Instance creation
	 */
	public JunPaintedBody(JunBody aBody, Color aColorValue) {
		super();
		this.body_(aBody);
		this.paint_(aColorValue);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunPaintedBody(String aString, JunBody aBody) {
		super();
		this.body_(aBody);
		this.name_(aString);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aColorValue java.awt.Color
	 * @category Instance creation
	 */
	public JunPaintedBody(String aString, JunBody aBody, Color aColorValue) {
		this(aBody, aColorValue);
		this.name_(aString);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunPaintedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunPaintedBody Body_(JunBody aBody) {
		return new JunPaintedBody(aBody);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aColorValue java.awt.Color
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunPaintedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunPaintedBody Body_paint_(JunBody aBody, Color aColorValue) {
		return new JunPaintedBody(aBody, aColorValue);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunPaintedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunPaintedBody Name_body_(String aString, JunBody aBody) {
		return new JunPaintedBody(aString, aBody);
	}

	/**
	 * Create a new instance of <code>JunPaintedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aColorValue java.awt.Color
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunPaintedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunPaintedBody Name_body_paint_(String aString, JunBody aBody, Color aColorValue) {
		return new JunPaintedBody(aString, aBody, aColorValue);
	}

	/**
	 * Convert receiver to JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody#asJunOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		if ((cached3dObject == null) && (this.body() != null)) {
			cached3dObject = this.body().asJunOpenGL3dObject();
			cached3dObject.objectsDo_(new StBlockClosure1<Void, JunOpenGL3dObject>() {
				public Void value(JunOpenGL3dObject each) {
					each.paint_(JunPaintedBody.this.paint());
					return null;
				}
			});
		}
		return cached3dObject;
	}

	/**
	 * Answer the body.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @see jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody#body()
	 * @category accessing
	 */
	public final JunBody body() {
		return body;
	}

	/**
	 * Set the body.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public void body_(JunBody aBody) {
		body = aBody;
		this.flushCache();
	}

	/**
	 * Flush the cached3dObject.
	 * 
	 * @category updating
	 */
	public void flushCache() {
		cached3dObject = null;
		this.changed();
	}

	/**
	 * Answer the name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public final String name() {
		return name;
	}

	/**
	 * Set the name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public final void name_(String aString) {
		name = aString;
	}

	/**
	 * Answer the paint.
	 * 
	 * @return java.awt.Color
	 * @category accessing
	 */
	public final Color paint() {
		return paint;
	}

	/**
	 * Set the paint.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category accessing
	 */
	public void paint_(Color aColorValue) {
		paint = aColorValue;
		this.flushCache();
	}

	/**
	 * Display receiver.
	 * 
	 * @category utilities
	 */
	public void show() {
		this.asJunOpenGL3dObject().show();
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		this.flushCache();
	}
}
