package jp.co.sra.jun.opengl.parameterizedbody;

import java.util.ArrayList;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StModel;
import jp.co.sra.smalltalk.StObject;

import jp.co.sra.jun.goodies.parameter.JunParameter;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.setoperators.JunUNION;

/**
 * JunParameterizedBody class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/08 (by MATSUDA Ryouichi)
 *  @updated   2008/06/10 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedBody.java,v 9.10 2010/03/09 09:17:42 nisinaka Exp $
 */
public class JunParameterizedBody extends StModel implements JunParameterizedOrPaintedBody {
	/** The name of the object. */
	protected String name;

	/** The components of the objects. */
	protected JunParameterizedOrPaintedBody[] components;

	/** The parameterizedOperator of the object. */
	protected JunParameterizedOperator parameterizedOperator;

	/** The cachedBody of the object. */
	protected JunBody cachedBody;

	/** The cached3dObject of the object. */
	protected JunOpenGL3dObject cached3dObject;

	/** The _allParameters of the objects. */
	protected JunParameter[] _allParameters;

	/**
	 * Answer a class of default ApplicationModel.
	 * 
	 * @return java.lang.Class
	 * @category Defaults
	 */
	public static Class<? extends JunParameterizedBodyModel> DefaultApplicationModelClass() {
		return JunParameterizedBodyModel.class;
	}

	/**
	 * Answer a class of default ParameterizedOperator.
	 * 
	 * @return java.lang.Class
	 * @category Defaults
	 */
	public static Class<? extends JunParameterizedOperator> DefaultParameterizedOperatorClass() {
		return JunParameterizedOperator.class;
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and initialize it.
	 * 
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		super();
		this.components_(anArrayOfComponents);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		this(anArrayOfComponents);
		this.name_(aString);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		this(aParameterizedOperator, anArrayOfComponents);
		this.name_(aString);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody aComponent) {
		this(aString, aParameterizedOperator, new JunParameterizedOrPaintedBody[] { aComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		this(aString, aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent,
			JunParameterizedOrPaintedBody fourthComponent) {
		this(aString, aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent, fourthComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOrPaintedBody aComponent) {
		this(aString, new JunParameterizedOrPaintedBody[] { aComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		this(aString, aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		this(aString, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		this(aString, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent, JunParameterizedOrPaintedBody fourthComponent) {
		this(aString, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent, fourthComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		this(anArrayOfComponents);
		this.parameterizedOperator_(aParameterizedOperator);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody aComponent) {
		this(aParameterizedOperator, new JunParameterizedOrPaintedBody[] { aComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		this(aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		this(aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent,
			JunParameterizedOrPaintedBody fourthComponent) {
		this(aParameterizedOperator, new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent, fourthComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOrPaintedBody aComponent) {
		this(new JunParameterizedOrPaintedBody[] { aComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		this(new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		this(new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @category Instance creation
	 */
	public JunParameterizedBody(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent, JunParameterizedOrPaintedBody fourthComponent) {
		this(new JunParameterizedOrPaintedBody[] { firstComponent, secondComponent, thirdComponent, fourthComponent });
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and initialize it.
	 * 
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Components_(JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		return new JunParameterizedBody(anArrayOfComponents);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_components_(String aString, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		return new JunParameterizedBody(aString, anArrayOfComponents);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_parameterizedOperator_components_(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		return new JunParameterizedBody(aString, aParameterizedOperator, anArrayOfComponents);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_parameterizedOperator_with_(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody aComponent) {
		return new JunParameterizedBody(aString, aParameterizedOperator, aComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_parameterizedOperator_with_with_(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		return new JunParameterizedBody(aString, aParameterizedOperator, firstComponent, secondComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_parameterizedOperator_with_with_with_(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent,
			JunParameterizedOrPaintedBody thirdComponent) {
		return new JunParameterizedBody(aString, aParameterizedOperator, firstComponent, secondComponent, thirdComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_parameterizedOperator_with_with_with_with_(String aString, JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent,
			JunParameterizedOrPaintedBody thirdComponent, JunParameterizedOrPaintedBody fourthComponent) {
		return new JunParameterizedBody(aString, aParameterizedOperator, firstComponent, secondComponent, thirdComponent, fourthComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_with_(String aString, JunParameterizedOrPaintedBody aComponent) {
		return new JunParameterizedBody(aString, aComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_with_with_(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		return new JunParameterizedBody(aString, firstComponent, secondComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_with_with_with_(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		return new JunParameterizedBody(aString, firstComponent, secondComponent, thirdComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aString java.lang.String
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody Name_with_with_with_with_(String aString, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent,
			JunParameterizedOrPaintedBody fourthComponent) {
		return new JunParameterizedBody(aString, firstComponent, secondComponent, thirdComponent, fourthComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody ParameterizedOperator_components_(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		return new JunParameterizedBody(aParameterizedOperator, anArrayOfComponents);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody ParameterizedOperator_with_(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody aComponent) {
		return new JunParameterizedBody(aParameterizedOperator, aComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody ParameterizedOperator_with_with_(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		return new JunParameterizedBody(aParameterizedOperator, firstComponent, secondComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody ParameterizedOperator_with_with_with_(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent,
			JunParameterizedOrPaintedBody thirdComponent) {
		return new JunParameterizedBody(aParameterizedOperator, firstComponent, secondComponent, thirdComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody ParameterizedOperator_with_with_with_with_(JunParameterizedOperator aParameterizedOperator, JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent,
			JunParameterizedOrPaintedBody thirdComponent, JunParameterizedOrPaintedBody fourthComponent) {
		return new JunParameterizedBody(aParameterizedOperator, firstComponent, secondComponent, thirdComponent, fourthComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param aComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody With_(JunParameterizedOrPaintedBody aComponent) {
		return new JunParameterizedBody(aComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody With_with_(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent) {
		return new JunParameterizedBody(firstComponent, secondComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody With_with_with_(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent) {
		return new JunParameterizedBody(firstComponent, secondComponent, thirdComponent);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBody</code> and set parameters.
	 * 
	 * @param firstComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param secondComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param thirdComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @param fourthComponent jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedBody With_with_with_with_(JunParameterizedOrPaintedBody firstComponent, JunParameterizedOrPaintedBody secondComponent, JunParameterizedOrPaintedBody thirdComponent, JunParameterizedOrPaintedBody fourthComponent) {
		return new JunParameterizedBody(firstComponent, secondComponent, thirdComponent, fourthComponent);
	}

	/**
	 * Answer all parameters.
	 * 
	 * @return jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category accessing
	 */
	public JunParameter[] allParameters() {
		if (_allParameters == null) {
			ArrayList<JunParameter> parameters = new ArrayList<JunParameter>();
			for (int i = 0; i < this.parameterizedOperator().parameters().length; i++) {
				parameters.add(this.parameterizedOperator().parameters()[i]);
			}

			for (int i = 0; i < this.components().length; i++) {
				if (this.components()[i] instanceof JunParameterizedBody) {
					JunParameterizedBody each = (JunParameterizedBody) this.components()[i];
					for (int j = 0; j < each.allParameters().length; j++) {
						JunParameter parameter = each.allParameters()[j];
						if (parameters.contains(parameter) == false) {
							parameters.add(parameter);
						}
					}
				}
			}

			_allParameters = new JunParameter[parameters.size()];
			parameters.toArray(_allParameters);
		}

		return _allParameters;
	}

	/**
	 * Convert receiver to JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody#asJunOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		if (cached3dObject == null) {
			cached3dObject = new JunOpenGL3dCompoundObject();
			for (int i = 0; i < this.components().length; i++) {
				JunParameterizedOrPaintedBody each = this.components()[i];
				((JunOpenGL3dCompoundObject) cached3dObject).add_(each.asJunOpenGL3dObject());
			}
			cached3dObject = this.parameterizedOperator().applyTransformationsTo_(cached3dObject);
		}
		return cached3dObject;
	}

	/**
	 * Answer a body.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @see jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody#body()
	 * @category converting
	 */
	public JunBody body() {
		if (cachedBody == null) {
			cachedBody = (JunBody) this.bodyAt_(0).copy();
			for (int each = 1; each < this.components().length; each++) {
				new JunUNION(cachedBody, (JunBody) this.bodyAt_(each).copy()).doOperation();
			}
			this.parameterizedOperator().applyOperatorsTo_(cachedBody);
		}
		return cachedBody;
	}

	/**
	 * Answer a body at index.
	 * 
	 * @param anIndex int
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @category converting
	 */
	public JunBody bodyAt_(int anIndex) {
		return this.components()[anIndex].body();
	}

	/**
	 * Answer the components.
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category accessing
	 */
	public JunParameterizedOrPaintedBody[] components() {
		if (components == null) {
			components = new JunParameterizedOrPaintedBody[] {};
		}

		return components;
	}

	/**
	 * Set the components.
	 * 
	 * @param anArrayOfComponents jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOrPaintedBody[]
	 * @category accessing
	 */
	public void components_(JunParameterizedOrPaintedBody[] anArrayOfComponents) {
		if (components != null) {
			for (int i = 0; i < components.length; i++) {
				StObject each = (StObject) components[i];
				each.removeDependent_(this);
			}
		}

		components = anArrayOfComponents;

		if (components != null) {
			for (int i = 0; i < components.length; i++) {
				StObject each = (StObject) components[i];
				each.addDependent_(this);
			}
		}

		this.flushCache();
	}

	/**
	 * Flush the cachedBody, cached3dObject.
	 * 
	 * @category updating
	 */
	public void flushCache() {
		cachedBody = null;
		cached3dObject = null;
		this.changed();
	}

	/**
	 * Answer the name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public final String name() {
		return name;
	}

	/**
	 * Set the name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public final void name_(String aString) {
		name = aString;
	}

	/**
	 * Answer a named parameter.
	 * 
	 * @param aString java.lang.String
	 * @return jp.co.sra.jun.goodies.JunParameter
	 * @category accessing
	 */
	public JunParameter namedParameter_(String aString) {
		for (int i = 0; i < this.allParameters().length; i++) {
			JunParameter each = this.allParameters()[i];
			if (each.name().equals(aString)) {
				return each;
			}
		}

		return null;
	}

	/**
	 * Set value to named parameter.
	 * 
	 * @param aString java.lang.String
	 * @param aValue double
	 * @category accessing
	 */
	public void namedParameter_value_(String aString, double aValue) {
		JunParameter aParameter = this.namedParameter_(aString);
		if (aParameter != null) {
			aParameter.value_(aValue);
		}
	}

	/**
	 * Answer the parameterizedOperator.
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @category accessing
	 */
	public JunParameterizedOperator parameterizedOperator() {
		if (parameterizedOperator == null) {
			parameterizedOperator = new JunParameterizedOperator();
		}

		return parameterizedOperator;
	}

	/**
	 * Set the parameterizedOperator.
	 * 
	 * @param aParameterizedOperator jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @category accessing
	 */
	public void parameterizedOperator_(JunParameterizedOperator aParameterizedOperator) {
		if (parameterizedOperator != null) {
			parameterizedOperator.removeDependent_(this);
		}
		parameterizedOperator = aParameterizedOperator;
		if (parameterizedOperator != null) {
			parameterizedOperator.addDependent_(this);
		}

		this.flushCache();
	}

	/**
	 * Display receiver.
	 * 
	 * @category utilities
	 */
	public void show() {
		JunParameterizedBodyModel.OpenOnParameterizedBody_(this);
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		this.flushCache();
	}
}
