package jp.co.sra.jun.opengl.parameterizedbody;

import java.awt.event.ActionEvent;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.goodies.parameter.JunParameter;
import jp.co.sra.jun.goodies.parameter.JunParametersModel;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayModel;
import jp.co.sra.jun.opengl.display.JunOpenGLEmbeddedDisplayModel;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunParameterizedBodyModel class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/08 (by MATSUDA Ryouichi)
 *  @updated   2000/01/19 (by MATSUDA Ryouichi)
 *  @updated   2004/09/21 (by nisinaka)
 *  @updated   2008/06/10 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on Jun471 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedBodyModel.java,v 9.10 2010/03/09 09:17:42 nisinaka Exp $
 */
public class JunParameterizedBodyModel extends JunApplicationModel {

	/** The parameterizedBody of the object. */
	protected JunParameterizedBody parameterizedBody;

	/** The displayModel of the object. */
	protected JunOpenGLEmbeddedDisplayModel displayModel;

	/** The parametersModel of the object. */
	protected JunParametersModel parametersModel;

	/** The interlockHolder of the object. */
	protected StValueHolder<Boolean> interlockHolder;

	/**
	 * Answer a class of default OpenGLDisplayModel.
	 * 
	 * @return java.lang.Class
	 * @category Defaults
	 */
	public static Class<? extends JunOpenGLDisplayModel> DefaultOpenGLDisplayModelClass() {
		return JunOpenGLEmbeddedDisplayModel.class;
	}

	/**
	 * Answer a class of default ParametersModel.
	 * 
	 * @return java.lang.Class
	 * @category Defaults
	 */
	public static Class<? extends JunParametersModel> DefaultParametersModelClass() {
		return JunParametersModel.class;
	}

	/**
	 * Create a new instance of <code>JunParameterizedBodyModel</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunParameterizedBodyModel() {
		super();
	}

	/**
	 * Create a new instance of <code>JunParameterizedBodyModel</code> and initialize it.
	 * 
	 * @param aParameterizedBody jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @category Instance creation
	 */
	public JunParameterizedBodyModel(JunParameterizedBody aParameterizedBody) {
		super();
		this.parameterizedBody_(aParameterizedBody);
	}

	/**
	 * Create a new instance of <code>JunParameterizedBodyModel</code> with aParameterizedBody.
	 * 
	 * @param aParameterizedBody jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 * @category Interface opening
	 */
	public static JunParameterizedBodyModel OpenOnParameterizedBody_(JunParameterizedBody aParameterizedBody) {
		JunParameterizedBodyModel anApplicationModel = new JunParameterizedBodyModel(aParameterizedBody);
		anApplicationModel.open();
		return anApplicationModel;
	}

	/**
	 * Create a new instance of <code>JunParameterizedBodyModel</code> and set parameters.
	 * 
	 * @param aParameterizedBody jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 * @deprecated since Jun701, using constructor.
	 * @category Interface opening
	 */
	public static JunParameterizedBodyModel ParameterizedBody_(JunParameterizedBody aParameterizedBody) {
		return new JunParameterizedBodyModel(aParameterizedBody);
	}

	/**
	 * Action performed event.
	 * 
	 * @param e java.awt.event.ActionEvent
	 * @category actions
	 */
	public void actionPerformed(ActionEvent e) {
		return;
	}

	/**
	 * Answer all parameters.
	 * 
	 * @return jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category accessing
	 */
	public JunParameter[] allParameters() {
		if (this.parameterizedBody() == null) {
			return new JunParameter[0];
		}

		return this.parameterizedBody().allParameters();
	}

	/**
	 * Create a Body and show it.
	 * 
	 * @category actions
	 */
	public void createBody() {
		if (this.parameterizedBody() != null) {
			this.parameterizedBody().asJunOpenGL3dObject().show();
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunParameterizedBodyViewAwt(this);
		} else {
			return new JunParameterizedBodyViewSwing(this);
		}
	}

	/**
	 * Answer the displayModel.
	 * 
	 * @return jp.co.sra.jun.opengl.display.JunOpenGLEmbeddedDisplayModel
	 * @category accessing
	 */
	public JunOpenGLEmbeddedDisplayModel displayModel() {
		if (displayModel == null) {
			displayModel = new JunOpenGLEmbeddedDisplayModel();
		}

		return displayModel;
	}

	/**
	 * Answer a now interlock.
	 * 
	 * @return boolean
	 * @category accessing
	 */
	public boolean interlock() {
		return this.interlockHolder().value();
	}

	/**
	 * Answer the interlockHolder.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category accessing
	 */
	public StValueHolder<Boolean> interlockHolder() {
		if (interlockHolder == null) {
			interlockHolder = new StValueHolder<Boolean>(Boolean.TRUE);
			interlockHolder.addDependent_(this);
		}

		return interlockHolder;
	}

	/**
	 * Answer the parameterizedBody.
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @category accessing
	 */
	public JunParameterizedBody parameterizedBody() {
		return parameterizedBody;
	}

	/**
	 * Set the parameterizedBody.
	 * 
	 * @param aParameterizedBody jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @category accessing
	 */
	public void parameterizedBody_(JunParameterizedBody aParameterizedBody) {
		if (parameterizedBody != null) {
			parameterizedBody.removeDependent_(this);
		}
		parameterizedBody = aParameterizedBody;
		if (parameterizedBody != null) {
			parameterizedBody.addDependent_(this);
		}
		this.updateDisplayObject();
	}

	/**
	 * Answer the parametersModel.
	 * 
	 * @return jp.co.sra.jun.goodies.parameter.JunParametersModel
	 * @category accessing
	 */
	public JunParametersModel parametersModel() {
		if (parametersModel == null) {
			parametersModel = new JunParametersModel(this.allParameters());
		}
		return parametersModel;
	}

	/**
	 * Set parameters.
	 * 
	 * @category actions
	 */
	public void setParameters() {
		this.updateDisplayObject();
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		if (this.interlock()) {
			this.updateDisplayObject();
		}
	}

	/**
	 * Update the displayObject.
	 * 
	 * @category updating
	 */
	public void updateDisplayObject() {
		JunOpenGL3dObject displayObject;

		if (this.parameterizedBody() != null) {
			displayObject = this.parameterizedBody().asJunOpenGL3dObject();
			this.displayModel().setDisplayObject_(displayObject);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return $String("Parameterized Body");
	}

}
