package jp.co.sra.jun.opengl.parameterizedbody;

import java.awt.Color;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.goodies.parameter.JunParameter;
import jp.co.sra.jun.opengl.flux.JunOpenGLFluxMutableObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.globaloperators.JunLineSCALE;
import jp.co.sra.jun.topology.globaloperators.JunPlaneSCALE;
import jp.co.sra.jun.topology.globaloperators.JunPointSCALE;
import jp.co.sra.jun.topology.globaloperators.JunROTATE;

/**
 * JunParameterizedBodyTestExamples class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   2008/06/10 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedBodyTestExamples.java,v 9.10 2010/03/09 09:17:42 nisinaka Exp $
 */
public class JunParameterizedBodyTestExamples extends JunTestExamples {
	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("chair", new JunPaintedBody("foot", JunBody.CubeOrigin_corner_(new Jun3dPoint(-3.5, -3.5, -4), new Jun3dPoint(3.5, 3.5, -3)), Color.blue), new JunPaintedBody("leg", JunBody
				.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -3), new Jun3dPoint(0.5, 0.5, 3)), Color.green), new JunPaintedBody("seat", JunBody.CubeOrigin_corner_(new Jun3dPoint(-4, -4, 3), new Jun3dPoint(4, 4, 4)), Color.yellow), new JunPaintedBody(
				"back", JunBody.CubeOrigin_corner_(new Jun3dPoint(-5, -4, 6), new Jun3dPoint(-4, 4, 12)), Color.red));
		aParameterizedBody.show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		JunParameter height = new JunParameter("height", new double[] { 6, 3, 12 }, $String("Height"));
		JunParameterizedOperator scaleOperator = new JunParameterizedOperator(height, "(PlaneSCALE 0 0 -3 0 0 1 (/ height 6))");
		JunParameterizedOperator moveOperator = new JunParameterizedOperator(height, "(MOVE 0 0 (- height 6))");
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("chair", new JunPaintedBody("foot", JunBody.CubeOrigin_corner_(new Jun3dPoint(-3.5, -3.5, -4), new Jun3dPoint(3.5, 3.5, -3)), Color.blue), new JunParameterizedBody("scaled leg",
				scaleOperator, new JunPaintedBody("leg", JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -3), new Jun3dPoint(0.5, 0.5, 3)), Color.green)), new JunParameterizedBody(moveOperator, new JunPaintedBody("seat", JunBody.CubeOrigin_corner_(
				new Jun3dPoint(-4, -4, 3), new Jun3dPoint(4, 4, 4)), Color.yellow), new JunPaintedBody("back", JunBody.CubeOrigin_corner_(new Jun3dPoint(-5, -4, 6), new Jun3dPoint(-4, 4, 12)), Color.red)));
		aParameterizedBody.show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		JunParameter height = new JunParameter("height", new double[] { 6, 3, 12 }, $String("Height"));
		JunParameter angle = new JunParameter("angle", new double[] { 15, 0, 60 }, $String("Angle"));
		JunParameterizedOperator scaleOperator = new JunParameterizedOperator(height, "(PlaneSCALE 0 0 -3 0 0 1 (/ height 6))");
		JunParameterizedOperator moveOperator = new JunParameterizedOperator(height, "(MOVE 0 0 (- height 6))");
		JunParameterizedOperator rotateOperator = new JunParameterizedOperator(angle, "(ROTATEDeg -4 0 4 0 1 0 (- 0 angle))");
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("chair", new JunPaintedBody("foot", JunBody.CubeOrigin_corner_(new Jun3dPoint(-3.5, -3.5, -4), new Jun3dPoint(3.5, 3.5, -3)), Color.blue), new JunParameterizedBody("scaled leg",
				scaleOperator, new JunPaintedBody("leg", JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -3), new Jun3dPoint(0.5, 0.5, 3)), Color.green)), new JunParameterizedBody(moveOperator, new JunPaintedBody("seat", JunBody.CubeOrigin_corner_(
				new Jun3dPoint(-4, -4, 3), new Jun3dPoint(4, 4, 4)), Color.yellow), new JunParameterizedBody("leaned back", rotateOperator, new JunPaintedBody("back", JunBody.CubeOrigin_corner_(new Jun3dPoint(-5, -4, 6), new Jun3dPoint(-4, 4, 12)),
				Color.red))));
		aParameterizedBody.show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		JunParameter footsize = new JunParameter("footsize", new double[] { 7, 2, 12, 1 }, $String("Foot Size"));
		JunParameter height = new JunParameter("height", new double[] { 6, 3, 12, 1 }, $String("Height"));
		JunParameter width = new JunParameter("width", new double[] { 8, 2, 12, 1 }, $String("Width"));
		JunParameter depth = new JunParameter("depth", new double[] { 8, 2, 12, 1 }, $String("Depth"));
		JunParameter angle = new JunParameter("angle", new double[] { 15, 0, 60, 5 }, $String("Angle"));
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("chair", new JunParameterizedBody("scaled foot", new JunParameterizedOperator(new JunParameter[] { footsize }, "(LineSCALE 0 0 -3.5 0 0 1 (/ footsize 7))"), new JunPaintedBody(
				"foot", JunBody.CubeOrigin_corner_(new Jun3dPoint(-3.5, -3.5, -4), new Jun3dPoint(3.5, 3.5, -3)), Color.blue)), new JunParameterizedBody("scaled leg", new JunParameterizedOperator(new JunParameter[] { height },
				"(PlaneSCALE 0 0 -3 0 0 1 (/ height 6))"), new JunPaintedBody("leg", JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -3), new Jun3dPoint(0.5, 0.5, 3)), Color.green)), new JunParameterizedBody(new JunParameterizedOperator(
				new JunParameter[] { height }, "(MOVE 0 0 (- height 6))"), new JunParameterizedBody("scaled seat", new JunParameterizedOperator(new JunParameter[] { width, depth },
				"(COMPOUND (PlaneSCALE 0 0 3.5 0 1 0 (/ width 8)) (PlaneSCALE 0 0 3.5 1 0 0 (/ depth 8)))"), new JunPaintedBody("seat", JunBody.CubeOrigin_corner_(new Jun3dPoint(-4, -4, 3), new Jun3dPoint(4, 4, 4)), Color.yellow)),
				new JunParameterizedBody("leaned back", new JunParameterizedOperator(new JunParameter[] { width, angle, depth }, "(COMPOUND (PlaneSCALE -4 0 4 0 1 0 (/ width 8)) (ROTATEDeg -4 0 4 0 1 0 (- 0 angle)) (MOVE (/ (- 8 depth) 2) 0 0))"),
						new JunPaintedBody("back", JunBody.CubeOrigin_corner_(new Jun3dPoint(-5, -4, 6), new Jun3dPoint(-4, 4, 12)), Color.red))));
		aParameterizedBody.show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example5() {
		_CreateDataForExample5().show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example6() {
		_CreateDataForExample6().show();

		return true;
	}

	/**
	 * Example:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example6Flux() {
		JunParameterizedBody aParameterizedBody;
		JunOpenGL3dCompoundObject objects;
		JunOpenGLFluxMutableObject flux;

		aParameterizedBody = _CreateDataForExample6();
		objects = new JunOpenGL3dCompoundObject();

		for (int each = 0; each <= 90; each += 5) {
			aParameterizedBody.namedParameter_value_("spin1", each);
			aParameterizedBody.namedParameter_value_("fold1", each);
			aParameterizedBody.namedParameter_value_("spin2", each);
			aParameterizedBody.namedParameter_value_("fold2", each);
			aParameterizedBody.namedParameter_value_("spin3", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		for (int each = 90; 0 <= each; each -= 5) {
			aParameterizedBody.namedParameter_value_("spin1", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		for (int each = 90; 0 <= each; each -= 5) {
			aParameterizedBody.namedParameter_value_("fold1", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		for (int each = 90; 0 <= each; each -= 5) {
			aParameterizedBody.namedParameter_value_("spin2", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		for (int each = 90; 0 <= each; each -= 5) {
			aParameterizedBody.namedParameter_value_("fold2", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		for (int each = 90; 0 <= each; each -= 5) {
			aParameterizedBody.namedParameter_value_("spin3", each);
			objects.add_(aParameterizedBody.asJunOpenGL3dObject());
		}

		flux = new JunOpenGLFluxMutableObject(objects);
		flux.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunParameterizedBodyTestExamples();
	}

	/**
	 * Create data for Example5.
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @category Private
	 */
	protected static JunParameterizedBody _CreateDataForExample5() {
		JunParameter footsize = new JunParameter("footsize", new double[] { 7, 2, 12, 1 }, $String("Foot Size"));
		JunParameter height = new JunParameter("height", new double[] { 6, 3, 12, 1 }, $String("Height"));
		JunParameter width = new JunParameter("width", new double[] { 8, 2, 12, 1 }, $String("Width"));
		JunParameter depth = new JunParameter("depth", new double[] { 8, 2, 12, 1 }, $String("Depth"));
		JunParameter angle = new JunParameter("angle", new double[] { 15, 0, 60, 5 }, $String("Angle"));
		JunBody unitCube = JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -0.5), new Jun3dPoint(0.5, 0.5, 0.5));
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("chair", new JunParameterizedBody("foot", new JunParameterizedOperator(new JunParameter[] { footsize }, "(COMPOUND (LineSCALE 0 0 0 0 0 1 footsize) (MOVE 0 0 -3.5))"),
				new JunPaintedBody(unitCube, Color.blue)), new JunParameterizedBody("leg", new JunParameterizedOperator(new JunParameter[] { height }, "(COMPOUND (PlaneSCALE 0 0 -0.5 0 0 1 height) (MOVE 0 0 -2.5))"), new JunPaintedBody(unitCube,
				Color.green)), new JunParameterizedBody(new JunParameterizedOperator(new JunParameter[] { height }, "(MOVE 0 0 (- height 2.5))"), new JunParameterizedBody("seat", new JunParameterizedOperator(new JunParameter[] { width, depth },
				"(COMPOUND (PlaneSCALE 0 0 0 0 1 0 width) (PlaneSCALE 0 0 0 1 0 0 depth))"), new JunPaintedBody(unitCube, Color.yellow)), new JunParameterizedBody("back", new JunParameterizedOperator(new JunParameter[] { width, angle, depth },
				"(COMPOUND (MOVE -0.5 0 2.5) (PlaneSCALE 0 0 0 0 1 0 width) (PlaneSCALE 0 0 2 0 0 1 6) (ROTATEDeg 0 0 0 0 1 0 (- 0 angle)) (MOVE (- 0 (/ depth 2)) 0 0.5))"), new JunPaintedBody(unitCube, Color.red))));
		return aParameterizedBody;
	}

	/**
	 * Create data for Example6.
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBody
	 * @category Private
	 */
	protected static JunParameterizedBody _CreateDataForExample6() {
		double segmentLength = 2;
		int cylinderDiviisions = 8;
		JunParameter spin1 = new JunParameter("spin1", new double[] { 0, -90, 90, 10 }, $String("Spin") + "1");
		JunParameter fold1 = new JunParameter("fold1", new double[] { 0, 0, 90, 5 }, $String("Fold") + "1");
		JunParameter spin2 = new JunParameter("spin2", new double[] { 0, -90, 90, 10 }, $String("Spin") + "2");
		JunParameter fold2 = new JunParameter("fold2", new double[] { 0, 0, 90, 5 }, $String("Fold") + "2");
		JunParameter spin3 = new JunParameter("spin3", new double[] { 0, -90, 90, 10 }, $String("Spin") + "3");
		JunBody unitCube = JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -0.5), new Jun3dPoint(0.5, 0.5, 0.5));

		//segment = (JunBody) unitCube.copy();
		JunBody segment = JunBody.CubeOrigin_corner_(new Jun3dPoint(-0.5, -0.5, -0.5), new Jun3dPoint(0.5, 0.5, 0.5));
		new JunLineSCALE(segment, new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 1), 0.8).doOperation();
		new JunPlaneSCALE(segment, new Jun3dPoint(0, 0, -0.5), new Jun3dPoint(0, 0, 1), segmentLength).doOperation();
		JunBody cylinder = (JunBody) JunBody.UnitCylinderArrayDivisions_(cylinderDiviisions)[0];
		new JunPointSCALE(cylinder, 0.4).doOperation();
		new JunPlaneSCALE(cylinder, new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 1), 0.9).doOperation();
		new JunROTATE(cylinder, new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0), JunAngle.FromDeg_(90)).doOperation();
		JunParameterizedBody carpal = new JunParameterizedBody("carpal", new JunParameterizedOperator(new JunParameter[] { spin3 }, "(COMPOUND (ROTATEDeg 0 0 0 0 0 1 spin3) (MOVE 0 0 " + segmentLength + "))"), new JunPaintedBody(cylinder, new Color(159,
				31, 223)));
		// Color purple
		JunParameterizedBody antebrachii = new JunParameterizedBody("antebrachii", new JunParameterizedOperator(new JunParameter[] { spin2 }, "(COMPOUND (ROTATEDeg 0 0 0 0 0 1 spin2) (MOVE 0 0 " + segmentLength + "))"), new JunPaintedBody(cylinder,
				new Color(255, 127, 0)), new JunParameterizedBody(new JunParameterizedOperator(new JunParameter[] { fold2 }, "(COMPOUND (MOVE 0 0 1) (ROTATEDeg 0 0 0 0 1 0 fold2))"), new JunPaintedBody(segment, Color.red), carpal));
		JunParameterizedBody humerus = new JunParameterizedBody("humerus", new JunParameterizedOperator(new JunParameter[] { spin1 }, "(ROTATEDeg 0 0 0 0 0 1 spin1)"), new JunPaintedBody(cylinder, Color.green), new JunParameterizedBody(
				new JunParameterizedOperator(new JunParameter[] { fold1 }, "(COMPOUND (MOVE 0 0 1) (ROTATEDeg 0 0 0 0 1 0 fold1))"), new JunPaintedBody(segment, Color.yellow), antebrachii));
		JunParameterizedBody aParameterizedBody = new JunParameterizedBody("robot arm", new JunParameterizedBody("basement", new JunParameterizedOperator("(COMPOUND (LineSCALE 0 0 0 0 0 1 4.8) (MOVE 0 0 -1))"), new JunPaintedBody(unitCube, Color.blue)),
				humerus);

		return aParameterizedBody;
	}
}
