package jp.co.sra.jun.opengl.parameterizedbody;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import jp.co.sra.smalltalk.StModel;
import jp.co.sra.jun.goodies.parameter.JunParametersViewSwing;
import jp.co.sra.jun.opengl.display.JunOpenGLDisplayViewSwing;
import jp.co.sra.jun.system.framework.JunAbstractViewJPanel;
import jp.co.sra.jun.system.support.JunSystem;

/**
 * JunParameterizedBodyViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   2000/01/19 (by MATSUDA Ryouichi)
 *  @updated   2004/09/21 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun471 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedBodyViewSwing.java,v 9.9 2010/03/09 09:17:42 nisinaka Exp $
 */
public class JunParameterizedBodyViewSwing extends JunAbstractViewJPanel implements JunParameterizedBodyView {

	protected JunOpenGLDisplayViewSwing displayView;
	protected JunParametersViewSwing parametersView;
	protected JCheckBox interlockCheckbox;
	protected JButton createBodyButton;

	/**
	 * Create a new instance of <code>JunParameterizedBodyViewSwing</code> and initialize it.
	 * 
	 * @param aModel jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 * @category Instance creation
	 */
	public JunParameterizedBodyViewSwing(JunParameterizedBodyModel aModel) {
		super(aModel);
	}

	/**
	 * Answer insets.
	 * 
	 * @return java.awt.Insets
	 * @see javax.swing.JComponent#getInsets()
	 * @category bounds accessing
	 */
	public Insets getInsets() {
		return new Insets(4, 4, 4, 4);
	}

	/**
	 * model property accessing
	 * 
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 * @category model accessing
	 */
	public JunParameterizedBodyModel getModel() {
		return (JunParameterizedBodyModel) this.model();
	}

	/**
	 * model property accessing
	 * 
	 * @param aModel jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 * @category model accessing
	 */
	public void setModel(JunParameterizedBodyModel aModel) {
		this.model_(aModel);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		JunParameterizedBodyModel theModel = this.getModel();

		JLabel label = new JLabel(JunSystem.$String("Parameter", "Parameters") + " :");
		label.setFont(new Font("dialog", Font.BOLD, 12));

		JCheckBox interlockCheckbox = new JCheckBox(JunSystem.$String("Interlock"), null, true);
		interlockCheckbox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getModel().interlockHolder().value_(Boolean.TRUE);
				}
				if (e.getStateChange() == ItemEvent.DESELECTED) {
					getModel().interlockHolder().value_(Boolean.FALSE);
				}
			}
		});

		JPanel northPanel = new JPanel();
		northPanel.setLayout(new BorderLayout());
		northPanel.add(label, BorderLayout.WEST);
		northPanel.add(interlockCheckbox, BorderLayout.EAST);

		parametersView = new JunParametersViewSwing(theModel.parametersModel());

		JButton createBodyButton = new JButton(JunSystem.$String("Create Body"));
		createBodyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getModel().createBody();
			}
		});

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(northPanel, BorderLayout.NORTH);
		centerPanel.add(parametersView, BorderLayout.CENTER);
		centerPanel.add(createBodyButton, BorderLayout.SOUTH);

		displayView = new JunOpenGLDisplayViewSwing(theModel.displayModel());

		// Layout components.
		this.setLayout(new GridLayout(1, 2));
		((GridLayout) this.getLayout()).setHgap(8);
		this.add(displayView);
		this.add(centerPanel);
	}

	/**
	 * Answer a default model.
	 * 
	 * @return jp.co.sra.smalltalk.StModel
	 * @see jp.co.sra.smalltalk.StViewJPanel#defaultModel()
	 * @category model accessing
	 */
	protected StModel defaultModel() {
		return new JunParameterizedBodyModel();
	}
}
