package jp.co.sra.jun.opengl.parameterizedbody;

import java.util.ArrayList;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StModel;
import jp.co.sra.smalltalk.StValueHolder;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispInterpreter;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.goodies.lisp.JunLispNil;
import jp.co.sra.jun.goodies.lisp.JunLispParser;
import jp.co.sra.jun.goodies.parameter.JunParameter;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.globaloperators.JunLineSCALE;
import jp.co.sra.jun.topology.globaloperators.JunMOVE;
import jp.co.sra.jun.topology.globaloperators.JunPlaneSCALE;
import jp.co.sra.jun.topology.globaloperators.JunPointSCALE;
import jp.co.sra.jun.topology.globaloperators.JunROTATE;
import jp.co.sra.jun.topology.support.JunGeometryTransformer;

/**
 * JunParameterizedOperator class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/08 (by MATSUDA Ryouichi)
 *  @updated   2008/06/10 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedOperator.java,v 9.11 2010/03/09 09:17:42 nisinaka Exp $
 */
public class JunParameterizedOperator extends StModel {
	/** The parameters of the objects. */
	protected JunParameter[] parameters;

	/** The sExpression of the object. */
	protected JunLispList sExpression;

	/** The lispInterpreter of the object. */
	protected JunLispInterpreter lispInterpreter;

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @category Instance creation
	 */
	public JunParameterizedOperator() {
		super();
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter aParameter) {
		this(new JunParameter[] { aParameter });
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @param anSExpression java.lang.String
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter aParameter, String anSExpression) {
		this(new JunParameter[] { aParameter }, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter aParameter, JunLispList anSExpression) {
		this(new JunParameter[] { aParameter }, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameter_(JunParameter aParameter) {
		return Parameters_(new JunParameter[] { aParameter });
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter[] anArrayOfParameters) {
		super();
		this.parameters_(anArrayOfParameters);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression java.lang.String
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter[] anArrayOfParameters, String anSExpression) {
		super();
		this.parameters_sExpression_(anArrayOfParameters, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunParameter[] anArrayOfParameters, JunLispList anSExpression) {
		super();
		this.parameters_sExpression_(anArrayOfParameters, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anSExpression java.lang.String
	 * @category Instance creation
	 */
	public JunParameterizedOperator(String anSExpression) {
		super();
		this.sExpression_(anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunParameterizedOperator(JunLispList anSExpression) {
		super();
		this.sExpression_(anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @param anSExpression java.lang.String
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameter_sExpression_(JunParameter aParameter, String anSExpression) {
		return new JunParameterizedOperator(aParameter, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param aParameter jp.co.sra.jun.goodies.parameter.JunParameter
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameter_sExpression_(JunParameter aParameter, JunLispList anSExpression) {
		return new JunParameterizedOperator(aParameter, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameters_(JunParameter[] anArrayOfParameters) {
		return new JunParameterizedOperator(anArrayOfParameters);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression java.lang.String
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameters_sExpression_(JunParameter[] anArrayOfParameters, String anSExpression) {
		return new JunParameterizedOperator(anArrayOfParameters, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator Parameters_sExpression_(JunParameter[] anArrayOfParameters, JunLispList anSExpression) {
		return new JunParameterizedOperator(anArrayOfParameters, anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anSExpression java.lang.String
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator SExpression_(String anSExpression) {
		return new JunParameterizedOperator(anSExpression);
	}

	/**
	 * Create a new instance of <code>JunParameterizedOperator</code> and set parameters.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.opengl.parameterizedbody.JunParameterizedOperator
	 * @deprecated since Jun701, using constructor.
	 * @category Instance creation
	 */
	public static JunParameterizedOperator SExpression_(JunLispList anSExpression) {
		return new JunParameterizedOperator(anSExpression);
	}

	/**
	 * Smalltalk like copyFrom:to: method.
	 * 
	 * @param arguments double[]
	 * @param from int
	 * @param to int
	 * @return double[]
	 * @category Private
	 */
	private static double[] _Copy_from_to_(double[] arguments, int from, int to) {
		double[] array = new double[to - from + 1];
		for (int i = 0; i < (to - from + 1); i++) {
			array[i] = arguments[from + i];
		}
		return array;
	}

	/**
	 * Apply operators to aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category applying
	 */
	public void applyOperatorsTo_(JunBody aBody) {
		if (this.sExpression() == JunLispNil.NullList()) {
			return;
		}

		this.beginInterpretation();
		try {
			this.applyOperators_to_(this.sExpression(), aBody);
		} finally {
			this.endInterpretation();
		}
	}

	/**
	 * Apply transformations to aJunOpenGL3dObject.
	 * 
	 * @param aJunOpenGL3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @category applying
	 */
	public JunOpenGL3dObject applyTransformationsTo_(JunOpenGL3dObject aJunOpenGL3dObject) {
		if (this.sExpression() == JunLispNil.NullList()) {
			return aJunOpenGL3dObject;
		}

		this.beginInterpretation();
		try {
			return this.applyTransformations_to_(this.sExpression(), aJunOpenGL3dObject);
		} finally {
			this.endInterpretation();
		}
	}

	/**
	 * Answer the parameters.
	 * 
	 * @return jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category accessing
	 */
	public JunParameter[] parameters() {
		if (parameters == null) {
			this.setParameters_(new JunParameter[] {});
		}
		return parameters;
	}

	/**
	 * Set the parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category accessing
	 */
	public void parameters_(JunParameter[] anArrayOfParameters) {
		this.setParameters_(anArrayOfParameters);
		this.changed_($("parameters"));
	}

	/**
	 * Set the parameters, sExpression.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression java.lang.String
	 * @category accessing
	 */
	public void parameters_sExpression_(JunParameter[] anArrayOfParameters, String anSExpression) {
		this.setParameters_(anArrayOfParameters);
		this.setSExpression_(anSExpression);
		this.changed_($("parameters"));
	}

	/**
	 * Set the parameters, sExpression.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category accessing
	 */
	public void parameters_sExpression_(JunParameter[] anArrayOfParameters, JunLispList anSExpression) {
		this.setParameters_(anArrayOfParameters);
		this.setSExpression_(anSExpression);
		this.changed_($("parameters"));
	}

	/**
	 * Answer the sExpression.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category accessing
	 */
	public JunLispList sExpression() {
		if (sExpression == null) {
			sExpression = JunLispNil.NullList();
		}
		return sExpression;
	}

	/**
	 * Set the sExpression.
	 * 
	 * @param anSExpression java.lang.String
	 * @category accessing
	 */
	public void sExpression_(String anSExpression) {
		this.setSExpression_(anSExpression);
		this.changed_($("sExpression"));
	}

	/**
	 * Set the sExpression.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category accessing
	 */
	public void sExpression_(JunLispList anSExpression) {
		this.setSExpression_(anSExpression);
		this.changed_($("sExpression"));
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StObject#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("value")) {
			this.changed_($("value"));
		}
	}

	/**
	 * Apply operators anSExpression to aBody.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category private
	 */
	protected void applyOperators_to_(JunLispList anSExpression, final JunBody aBody) {
		final JunParameterizedOperator this_ = this;
		if (((JunLispCons) anSExpression).head() == $("COMPOUND")) {
			((JunLispList) ((JunLispCons) anSExpression).tail()).do_(new StBlockClosure1<Void, Object>() {
				public Void value(Object o) {
					JunLispList each = (JunLispList) o;
					this_.applyOperators_to_(each, aBody);

					return null;
				}
			});
		} else {
			JunAbstractOperator operator = this.instanciateOperator_(anSExpression);
			operator.body_(aBody);
			operator.doOperation();
		}
	}

	/**
	 * Apply transformations anSExpression to aJunOpenGL3dObject.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @param aJunOpenGL3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category private
	 */
	protected JunOpenGL3dObject applyTransformations_to_(JunLispList anSExpression, JunOpenGL3dObject aJunOpenGL3dObject) {
		final JunParameterizedOperator this_ = this;
		final StValueHolder<JunOpenGL3dObject> transformedObject_ = new StValueHolder<JunOpenGL3dObject>(); // JunOpenGL3dObject
		Jun3dTransformation transformation;

		if (((JunLispCons) anSExpression).head() == $("COMPOUND")) {
			transformedObject_.value_(aJunOpenGL3dObject);
			((JunLispList) ((JunLispCons) anSExpression).tail()).do_(new StBlockClosure1<Void, Object>() {
				public Void value(Object o) {
					JunLispList each = (JunLispList) o;
					transformedObject_.value_(this_.applyTransformations_to_(each, transformedObject_.value()));
					return null;
				}
			});
		} else {
			transformation = this.instanciateTransformation_(anSExpression);
			transformedObject_.value_(aJunOpenGL3dObject.transform_(transformation));
		}

		return transformedObject_.value();
	}

	/**
	 * Begin interpretation.
	 * 
	 * @category private
	 */
	protected void beginInterpretation() {
		if (lispInterpreter == null) {
			lispInterpreter = new JunLispInterpreter();
			lispInterpreter.evaluateTopLevel_ifFail_(JunLispNil.NullList(), new StBlockClosure1<Object, String>() {
				public Object value(String errorMessage) {
					JunParameterizedOperator.this.error_(errorMessage);
					return null;
				}
			});
		}

		lispInterpreter.bindMark();
		for (int i = 0; i < this.parameters().length; i++) {
			JunParameter each = this.parameters()[i];
			lispInterpreter.bind_value_($(each.name()), new Double(each.value()));
		}
	}

	/**
	 * End interpretation.
	 * 
	 * @category private
	 */
	protected void endInterpretation() {
		lispInterpreter.unbind();
	}

	/**
	 * Evaluate arguments anSExpression count aNumber.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @param aNumber int
	 * @return double[]
	 * @category private
	 */
	protected double[] evaluateArguments_count_(JunLispList anSExpression, int aNumber) {
		final ArrayList<Number> arguments = new ArrayList<Number>();
		if (((JunLispCons) ((JunLispCons) anSExpression).tail()).length() != aNumber) {
			//return lispInterpreter.fatal_("too few or many arguments for " + ((JunLispCons) anSExpression).head());
			return null;
		}

		((JunLispList) ((JunLispCons) anSExpression).tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object each) {
				arguments.add((Number) lispInterpreter.evaluate_(each));
				return null;
			}
		});

		double[] _arguments = new double[arguments.size()];
		for (int i = 0; i < arguments.size(); i++) {
			_arguments[i] = arguments.get(i).doubleValue();
		}
		return _arguments;
	}

	/**
	 * Instanciate operator anSExpression.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.topology.abstracts.JunGlobalOperator
	 * @category private
	 */
	protected JunGlobalOperator instanciateOperator_(JunLispList anSExpression) {
		double[] arguments;
		Jun3dPoint point;
		Jun3dPoint vector;
		JunAngle angle;
		double factor;

		if (anSExpression.head() == $("MOVE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 3);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));

			return new JunMOVE(null, point);
		}

		if (anSExpression.head() == $("ROTATE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			angle = JunAngle.FromRad_(_Copy_from_to_(arguments, 6, 6)[0]);

			return new JunROTATE(null, point, vector, angle);
		}

		if (anSExpression.head() == $("ROTATEDeg")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			angle = JunAngle.FromDeg_(_Copy_from_to_(arguments, 6, 6)[0]);

			return new JunROTATE(null, point, vector, angle);
		}

		if (anSExpression.head() == $("PointSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 4);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			factor = _Copy_from_to_(arguments, 3, 3)[0];

			return new JunPointSCALE(null, point, factor);
		}

		if (anSExpression.head() == $("LineSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			factor = _Copy_from_to_(arguments, 6, 6)[0];

			return new JunLineSCALE(null, point, vector, factor);
		}

		if (anSExpression.head() == $("PlaneSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			factor = _Copy_from_to_(arguments, 6, 6)[0];

			return new JunPlaneSCALE(null, point, vector, factor);
		}

		//return this.invalidOperatorError_(anSExpression);
		return null;
	}

	/**
	 * Instanciate transformation anSExpression.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 * @category private
	 */
	protected Jun3dTransformation instanciateTransformation_(JunLispList anSExpression) {
		double[] arguments;
		Jun3dPoint point;
		Jun3dPoint vector;
		// Jun3dLine scaleAxis;
		JunAngle angle;
		// double factor;

		if (anSExpression.head() == $("MOVE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 3);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			return Jun3dTransformation.Translate_(point);
		}

		if (anSExpression.head() == $("ROTATE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			angle = JunAngle.FromRad_(_Copy_from_to_(arguments, 6, 6)[0]);
			return Jun3dTransformation.Rotate_around_(angle, new Jun3dLine(point, point.plus_(vector)));
		}

		if (anSExpression.head() == $("ROTATEDeg")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			angle = JunAngle.FromDeg_(_Copy_from_to_(arguments, 6, 6)[0]);
			return Jun3dTransformation.Rotate_around_(angle, new Jun3dLine(point, point.plus_(vector)));
		}

		if (anSExpression.head() == $("PointSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 4);
			final Jun3dPoint aPoint = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			final double factor = _Copy_from_to_(arguments, 3, 3)[0];
			return JunGeometryTransformer.Block_(new StBlockClosure1<Jun3dPoint, Jun3dPoint>() {
				public Jun3dPoint value(Jun3dPoint value) {
					Jun3dPoint relativePosition = value.minus_(aPoint);
					return relativePosition.multipliedBy_(factor).plus_(aPoint);
				}
			});
		}

		if (anSExpression.head() == $("LineSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			point = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			vector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			final Jun3dLine scaleAxis = new Jun3dLine(point, point.plus_(vector.unitVector()));
			final double factor = _Copy_from_to_(arguments, 6, 6)[0];
			return JunGeometryTransformer.Block_(new StBlockClosure1<Jun3dPoint, Jun3dPoint>() {
				public Jun3dPoint value(Jun3dPoint value) {
					Jun3dPoint referencePoint = scaleAxis.nearestPointFromPoint_(value);
					Jun3dPoint relativePosition = value.minus_(referencePoint);
					return relativePosition.multipliedBy_(factor).plus_(referencePoint);
				}
			});
		}

		if (anSExpression.head() == $("PlaneSCALE")) {
			arguments = this.evaluateArguments_count_(anSExpression, 7);
			final Jun3dPoint aPoint = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 0, 2));
			final Jun3dPoint aVector = Jun3dPoint.FromArray_(_Copy_from_to_(arguments, 3, 5));
			final Jun3dLine scaleAxis = new Jun3dLine(aPoint, aPoint.plus_(aVector.unitVector()));
			final double factor = _Copy_from_to_(arguments, 6, 6)[0];
			return JunGeometryTransformer.Block_(new StBlockClosure1<Jun3dPoint, Jun3dPoint>() {
				public Jun3dPoint value(Jun3dPoint value) {
					Jun3dPoint pVector = value.minus_(aPoint);
					double pDistance = pVector.length();
					if (pDistance < pVector.Accuracy()) {
						return value;
					} else {
						JunAngle anAngle = scaleAxis.angleWithLine_(new Jun3dLine(aPoint, value));
						double qLength = anAngle.cos() * pDistance;
						Jun3dPoint qVector = aVector.unitVector().multipliedBy_(qLength);
						if (qVector.innerProduct_(pVector) < 0) {
							qVector = qVector.negated();
						}
						Jun3dPoint rVector = pVector.minus_(qVector);
						return qVector.multipliedBy_(factor).plus_(rVector).plus_(aPoint);
					}
				}
			});
		}

		//return this.invalidOperatorError_(anSExpression);
		return null;
	}

	/**
	 * Invalid operator error anSExpression.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return java.lang.String
	 * @category private
	 */
	protected String invalidOperatorError_(JunLispList anSExpression) {
		return lispInterpreter.fatal_("invalid operator: ") + anSExpression.head().toString();
	}

	/**
	 * Set the parameters.
	 * 
	 * @param anArrayOfParameters jp.co.sra.jun.goodies.parameterJunParameter[]
	 * @category private
	 */
	protected void setParameters_(JunParameter[] anArrayOfParameters) {
		if (parameters != null) {
			for (int i = 0; i < parameters.length; i++) {
				JunParameter each = parameters[i];
				each.removeDependent_(this);
			}
		}

		parameters = anArrayOfParameters;

		if (parameters != null) {
			for (int i = 0; i < parameters.length; i++) {
				JunParameter each = parameters[i];
				each.addDependent_(this);
			}
		}
	}

	/**
	 * Set the sExpression.
	 * 
	 * @param anSExpression java.lang.String
	 * @category private
	 */
	protected void setSExpression_(String anSExpression) {
		sExpression = (JunLispList) JunLispParser.Parse_(anSExpression);
	}

	/**
	 * Set the sExpression.
	 * 
	 * @param anSExpression jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category private
	 */
	protected final void setSExpression_(JunLispList anSExpression) {
		sExpression = anSExpression;
	}
}
