package jp.co.sra.jun.opengl.rotation;

import java.util.List;
import java.util.Vector;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StModel;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.curves.Jun2dLine;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;

/**
 * JunOpenGLRotationPolyline class
 * 
 *  @author    nisinaka
 *  @created   1998/12/09 (by nisinaka)
 *  @updated   2008/06/03 (by Mitsuhiro Asada)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLRotationPolyline.java,v 9.10 2010/03/09 09:17:44 nisinaka Exp $
 */
public class JunOpenGLRotationPolyline extends StModel {
	/** The collection of vertices. */
	protected Vector<Jun2dPoint> vertices = new Vector<Jun2dPoint>();

	/** The flag is true if the polyline make a loop. */
	protected boolean loop = false;

	/**
	 * Create a new instance of <code>JunOpenGLRotationPolyline</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGLRotationPolyline() {
		super();
	}

	/**
	 * Create a new instance of <code>JunOpenGLRotationPolyline</code> and initialize it.
	 * 
	 * @param arrayOfJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @category Instance creation
	 */
	public JunOpenGLRotationPolyline(Jun2dPoint[] arrayOfJun2dPoint) {
		this();
		this.addAll_(arrayOfJun2dPoint);
	}

	/**
	 * Create a new instance of <code>JunOpenGLRotationPolyline</code> and initialize it.
	 * 
	 * @param arrayOfJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @param aBoolean boolean
	 * @category Instance creation
	 */
	public JunOpenGLRotationPolyline(Jun2dPoint[] arrayOfJun2dPoint, boolean aBoolean) {
		this();
		this.addAll_(arrayOfJun2dPoint);
		this.loop_(aBoolean);
	}

	/**
	 * Create a new instance of <code>JunOpenGLRotationPolyline</code> and initialize it with the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @return jp.co.sra.jun.opengl.rotation.JunOpenGLRotationPolyline
	 * @category Lisp support
	 */
	public static JunOpenGLRotationPolyline FromLispList_(JunLispCons aList) {
		if (aList.head() != KindName()) {
			throw SmalltalkException.Error("unexpected error");
		}

		JunOpenGLRotationPolyline aPolyline = new JunOpenGLRotationPolyline();
		aPolyline.fromLispList_(aList);

		return aPolyline;
	}

	/**
	 * Answer the string name which represents the kind of this class.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category Lisp support
	 */
	public static StSymbol KindName() {
		return $("JunOpenGLRotationPolyline");
	}

	/**
	 * Add the vertex to the receiver.
	 * 
	 * @param aJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category adding
	 */
	public Jun2dPoint add_(Jun2dPoint aJun2dPoint) {
		this.vertices().add(aJun2dPoint);
		this.changed_($("vertices"));

		return aJun2dPoint;
	}

	/**
	 * Add the vertex right before the specified index.
	 * 
	 * @param aJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @param index int
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category adding
	 */
	public Jun2dPoint add_beforeIndex_(Jun2dPoint aJun2dPoint, int index) {
		this.vertices().insertElementAt(aJun2dPoint, index);
		this.changed_($("vertices"));
		return aJun2dPoint;
	}

	/**
	 * Add all points of arrayOfJun2dPoint.
	 * 
	 * @param arrayOfJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @category adding
	 */
	public Jun2dPoint[] addAll_(Jun2dPoint[] arrayOfJun2dPoint) {
		for (int i = 0; i < arrayOfJun2dPoint.length; i++) {
			this.vertices().add(arrayOfJun2dPoint[i]);
		}
		this.changed_($("vertices"));
		return arrayOfJun2dPoint;
	}

	/**
	 * Answer the Jun2dPoint with the specified index.
	 * 
	 * @param index int
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint at_(int index) {
		return this.vertices().get(index);
	}

	/**
	 * Put the Jun2dPoint at the index .
	 * 
	 * @param anIndex int
	 * @param aJun2dPoint jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public void at_put_(int anIndex, Jun2dPoint aJun2dPoint) {
		this.vertices().set(anIndex, aJun2dPoint);
		this.changed_($("vertices"));
	}

	/**
	 * Enumerate all edges and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @deprecated since Jun701, use Generics.
	 * @category enumerating
	 */
	public Object edgesDo_(StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.edgesDo_flag_(aBlock, 0);
	}

	/**
	 * Enumerate all edges and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aFlag int
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R edgesDo_(StBlockClosure1<R, Jun2dLine> aBlock) {
		return this.edgesDo_flag_(aBlock, 0);
	}

	/**
	 * Answer the first vertex of the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint first() {
		return this.at_(0);
	}

	/**
	 * Initialize the attributes of the receiver with the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	public void fromLispList_(JunLispCons aList) {
		if (aList.head() != this.kindName()) {
			throw SmalltalkException.Error("unexpected error");
		}

		final JunOpenGLRotationPolyline self = this;
		((JunLispList) aList.tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object anObject) {
				JunLispCons each = (JunLispCons) anObject;
				if (each.head() == $("loop")) {
					self.loop_(((Boolean) each.tail()).booleanValue());
				}
				if (each.head() == $("points")) {
					((JunLispList) each.tail()).do_(new StBlockClosure1<Void, Object>() {
						public Void value(Object point) {
							self.add_((Jun2dPoint) point);
							return null;
						}
					});
				}
				return null;
			}
		});
	}

	/**
	 * Answer the string name which represents the kind of the object.
	 * 
	 * @return jp.co.sra.smalltalk.StSymbol
	 * @category lisp support
	 */
	public StSymbol kindName() {
		return KindName();
	}

	/**
	 * Answer the last vertex of the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint last() {
		return this.at_(this.size() - 1);
	}

	/**
	 * Answer true if the receiver makes a loop, otherwise false.
	 * 
	 * @return boolean
	 * @category accessing
	 */
	public boolean loop() {
		return loop;
	}

	/**
	 * Set the flag whether the receiver makes a loop or not.
	 * 
	 * @param aBoolean boolean
	 * @category accessing
	 */
	public void loop_(boolean aBoolean) {
		loop = aBoolean;
		this.changed_($("loop"));
	}

	/**
	 * Convert the flag loop to a JunLispList.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList loopToLispList() {
		return new JunLispCons($("loop"), new Boolean(this.loop()));
	}

	/**
	 * Remove the vertex at the specified index.
	 * 
	 * @param anIndex int
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint
	 * @category removing
	 */
	public Jun2dPoint removeAtIndex_(int anIndex) {
		Jun2dPoint aVertex = this.vertices().get(anIndex);
		this.vertices().remove(anIndex);
		this.changed_($("vertices"));
		return aVertex;
	}

	/**
	 * Reverse the order of vertices.
	 * 
	 * @return java.util.List
	 * @category enumerating
	 */
	public List<Jun2dPoint> reverseVertices() {
		List<Jun2dPoint> oldVertices = this.vertices();
		int size = oldVertices.size();
		vertices = new Vector<Jun2dPoint>(size);
		for (int i = 0; i < size; i++) {
			vertices.add(oldVertices.get(size - i - 1));
		}
		this.changed_($("vertices"));
		return oldVertices;
	}

	/**
	 * Answer the number of vertices.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int size() {
		return this.vertices().size();
	}

	/**
	 * Answer the list representation of the JunOpenGLRotationPolyline.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList toLispList() {
		JunLispCons list = JunLispCons.Cell();
		list.head_(this.kindName());
		list.add_(this.loopToLispList());
		list.add_(this.verticesToLispList());
		return list;
	}

	/**
	 * Answer the Vector of vertices.
	 * 
	 * @return java.util.Vector
	 * @category accessing
	 */
	public Vector<Jun2dPoint> vertices() {
		return vertices;
	}

	/**
	 * Enumerate all vertices and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @deprecated since Jun701, use Generics.
	 * @category enumerating
	 */
	public Object verticesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.verticesDo_(new StBlockClosure1<Object, Jun2dPoint>() {
			public Object value(Jun2dPoint aPoint) {
				return aBlock.value_(aPoint);
			}
		});
	}

	/**
	 * Enumerate all vertices and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R verticesDo_(StBlockClosure1<R, Jun2dPoint> aBlock) {
		Jun2dPoint[] anArray = this._verticesAsArray();
		for (int i = 0; i < anArray.length; i++) {
			R result = aBlock.value(anArray[i]);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/**
	 * Convert the vertices to a JunLispList.
	 * 
	 * @return jp.co.sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList verticesToLispList() {
		return new JunLispCons($("points"), JunLispCons.List_(this.vertices()));
	}

	/**
	 * Convert the vertices as an array of Jun2dPoint.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun2dPoint[]
	 * @category accessing
	 */
	protected Jun2dPoint[] _verticesAsArray() {
		return vertices.toArray(new Jun2dPoint[this.size()]);
	}

	/**
	 * Enumerate all edges and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aFlag int
	 * @return java.lang.Object
	 * @deprecated since Jun701, use Generics.
	 * @category enumerating
	 */
	protected Object edgesDo_flag_(final StBlockClosure<?, ?, ?, ?> aBlock, int aFlag) {
		return this.edgesDo_flag_(new StBlockClosure1<Object, Jun2dLine>() {
			public Object value(Jun2dLine aLine) {
				return aBlock.value_(aLine);
			}
		}, aFlag);
	}

	/**
	 * Enumerate all edges and evaluate the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aFlag int
	 * @return java.lang.Object
	 * @category enumerating
	 */
	protected <R> R edgesDo_flag_(StBlockClosure1<R, Jun2dLine> aBlock, int aFlag) {
		int size = this.size();

		if (size > 0) {
			Jun2dPoint firstPoint = this.first();
			Jun2dPoint lastPoint = this.last();
			R result = null;

			if (this.loop()) {
				if (aFlag == 1) {
					result = aBlock.value(new Jun2dLine(firstPoint.plus_(lastPoint).dividedBy_(2), firstPoint));
				} else if (aFlag == 2) {
					result = aBlock.value(new Jun2dLine(lastPoint, firstPoint));
				}

				if (result != null) {
					return result;
				}
			} else {
				if (aFlag == 2) {
					result = aBlock.value(new Jun2dLine(new Jun2dPoint(lastPoint.x(), 0), new Jun2dPoint(firstPoint.x(), 0)));
					if (result != null) {
						return result;
					}
				}

				result = aBlock.value(new Jun2dLine(new Jun2dPoint(firstPoint.x(), 0), firstPoint));
				if (result != null) {
					return result;
				}
			}

			for (int i = 0; i < (size - 1); i++) {
				result = aBlock.value(new Jun2dLine(this.at_(i), this.at_(i + 1)));
				if (result != null) {
					return result;
				}
			}

			if (this.loop()) {
				if (aFlag == 1) {
					result = aBlock.value(new Jun2dLine(lastPoint, firstPoint.plus_(lastPoint).dividedBy_(2)));
				} else {
					result = aBlock.value(new Jun2dLine(lastPoint, firstPoint));
				}
				if (result != null) {
					return result;
				}
			} else {
				result = aBlock.value(new Jun2dLine(lastPoint, new Jun2dPoint(lastPoint.x(), 0)));
				if (result != null) {
					return result;
				}
				if (aFlag == 2) {
					result = aBlock.value(new Jun2dLine(new Jun2dPoint(lastPoint.x(), 0), new Jun2dPoint(firstPoint.x(), 0)));
					if (result != null) {
						return result;
					}
				}
			}
		}

		return null;
	}
}
