package jp.co.sra.jun.system.dialog;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StSimpleDialog;

import jp.co.sra.jun.system.framework.JunApplicationModel;

/**
 * JunAbstractDialog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/05/14 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun465 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAbstractDialog.java,v 9.10 2010/03/09 09:17:47 nisinaka Exp $
 */
public abstract class JunAbstractDialog<T extends JunApplicationModel> extends StSimpleDialog {
	protected T applicationModel;
	protected StBlockClosure1<Boolean, T> acceptBlock;
	protected StBlockClosure1<Boolean, T> cancelBlock;

	/**
	 * Initialize this object.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		applicationModel = null;
		acceptBlock = null;
		cancelBlock = null;
	}

	/**
	 * Set the new accept block.
	 *
	 * @param newCancelBlock jp.co.sra.smalltalk.StBlockClosure&lt;java.lang.Boolean, jp.co.sra.jun.system.framework.JunApplicationModel, ?, ?&gt;
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	protected void acceptBlock_(final StBlockClosure<Boolean, T, ?, ?> newAcceptBlock) {
		acceptBlock = new StBlockClosure1<Boolean, T>() {
			public Boolean value(T value1) {
				return newAcceptBlock.value_(value1);
			}
		};
	}

	/**
	 * Set the new accept block.
	 *
	 * @param newAcceptBlock jp.co.sra.smalltalk.StBlockClosure1&lt;java.lang.Boolean, jp.co.sra.jun.system.framework.JunApplicationModel&gt;
	 * @category accessing
	 */
	protected void acceptBlock_(StBlockClosure1<Boolean, T> newAcceptBlock) {
		acceptBlock = newAcceptBlock;
	}

	/**
	 * Set the new cancel block.
	 *
	 * @param newCancelBlock jp.co.sra.smalltalk.StBlockClosure&lt;java.lang.Boolean, jp.co.sra.jun.system.framework.JunApplicationModel, ?, ?&gt;
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	protected void cancelBlock_(final StBlockClosure<Boolean, T, ?, ?> newCancelBlock) {
		cancelBlock = new StBlockClosure1<Boolean, T>() {
			public Boolean value(T value1) {
				return newCancelBlock.value_(value1);
			}
		};
	}

	/**
	 * Set the new cancel block.
	 *
	 * @param newCancelBlock jp.co.sra.smalltalk.StBlockClosure1&lt;java.lang.Boolean, jp.co.sra.jun.system.framework.JunApplicationModel&gt;
	 * @category accessing
	 */
	protected void cancelBlock_(StBlockClosure1<Boolean, T> newCancelBlock) {
		cancelBlock = newCancelBlock;
	}

	/**
	 * Answer the receiver's model as <code>JunApplicationModel</code>
	 * 
	 * @return jp.co.sra.jun.system.framework.JunApplicationModel
	 * @category accessing
	 */
	public T model() {
		return applicationModel;
	}

	/**
	 * Answer the receiver's model as <code>JunApplicationModel</code>
	 * 
	 * @return jp.co.sra.jun.system.framework.JunApplicationModel
	 * @category accessing
	 */
	public void model_(T aModel) {
		applicationModel = aModel;
	}

	/**
	 * Called when an accept button is pressed.
	 *
	 * @return boolean
	 * @category actions
	 */
	public boolean acceptButtonPressed() {
		if (acceptBlock != null) {
			return acceptBlock.value(this.model());
		}
		return true;
	}

	/**
	 * Called when a cancel button is pressed.
	 *
	 * @return boolean
	 * @category actions
	 */
	public boolean cancelButtonPressed() {
		if (cancelBlock != null) {
			return cancelBlock.value(this.model());
		}
		return true;
	}
}
