package jp.co.sra.jun.system.dialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import jp.co.sra.smalltalk.StSimpleDialogViewSwing;

/**
 * JunAbstractDialogViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2008/05/14 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun465 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAbstractDialogViewSwing.java,v 9.9 2010/03/09 09:17:47 nisinaka Exp $
 */
public abstract class JunAbstractDialogViewSwing extends StSimpleDialogViewSwing implements JunAbstractDialogView {
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of <code>JunAbstractDialogViewSwing</code> and initialize it.
	 * 
	 * @param aDialog jp.co.sra.jun.system.framework.JunAbstractDialog
	 * @category Instance creation
	 */
	public JunAbstractDialogViewSwing(JunAbstractDialog<?> aDialog) {
		super(aDialog);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		acceptButton = null;
		cancelButton = null;
	}

	/**
	 * Answer the receiver's model as <code>JunAbstractDialog</code>.
	 *
	 * @return jp.co.sra.jun.system.dialog.JunAbstractDialog
	 * @see jp.co.sra.jun.system.dialog.JunAbstractDialogView#getDialog()
	 * @category model accessing
	 */
	public JunAbstractDialog<?> getDialog() {
		return (JunAbstractDialog<?>) this.model();
	}

	/**
	 * Answer the receiver's accept button.
	 *
	 * @return javax.swing.JButton
	 * @category interface opening
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton($String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (JunAbstractDialogViewSwing.this.getDialog().acceptButtonPressed()) {
						JunAbstractDialogViewSwing.this.getDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Answer the receiver's cancel button.
	 *
	 * @return javax.swing.JButton
	 * @category interface opening
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton($String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (JunAbstractDialogViewSwing.this.getDialog().cancelButtonPressed()) {
						JunAbstractDialogViewSwing.this.getDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}
}
