package jp.co.sra.jun.system.dialog;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Dimension;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JLabel;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StSimpleDialogViewAwt;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunInternetConnectionDialogViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2008/08/27 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun704 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunInternetConnectionDialogViewAwt.java,v 9.7 2010/03/09 09:17:47 nisinaka Exp $
 */
public class JunInternetConnectionDialogViewAwt extends StSimpleDialogViewAwt implements JunInternetConnectionDialogView {

	protected Button acceptButton;
	protected Button cancelButton;
	protected Checkbox directConnectionCheckbox;
	protected Checkbox useHttpProxyCheckbox;
	protected TextField proxyHostField;
	protected TextField portNumberField;

	private CheckboxGroup _checkboxGroup;

	/**
	 * Create a new instance of JunInternetConnectionDialogViewAwt and initialize it.
	 *
	 * @param anInternectConnectionDialog jp.co.sra.jun.system.dialog.JunInternetConnectionDialog
	 * @category Instance creation
	 */
	public JunInternetConnectionDialogViewAwt(JunInternetConnectionDialog anInternectConnectionDialog) {
		super(anInternectConnectionDialog);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -296, 0f, 152, 1f, -208, 0f, 184));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -152, 0f, 152, 1f, -64, 0f, 184));
		this.add(this.directConnectionCheckbox(), WindowSpecLayout.Position(0f, 16, 0f, 16));
		this.add(this.useHttpProxyCheckbox(), WindowSpecLayout.Position(0f, 16, 0f, 48));
		this.add(new JLabel($String("Proxy Host")), WindowSpecLayout.Position(0f, 48, 0f, 80));
		this.add(new JLabel($String("Port Number")), WindowSpecLayout.Position(0f, 48, 0f, 112));
		this.add(this.proxyHostField(), WindowSpecLayout.Position(0f, 144, 0f, 80, 0f, 344, 0f, 104));
		this.add(this.portNumberField(), WindowSpecLayout.Position(0f, 144, 0f, 112, 0f, 200, 0f, 136));
		this.setPreferredSize(new Dimension(360, 200));
	}

	/**
	 * Answer my model as JunInternetConnectionDialog.
	 * 
	 * @return jp.co.sra.jun.system.dialog.JunInternetConnectionDialog
	 * @see jp.co.sra.jun.system.dialog.JunInternetConnectionDialogView#getInternetConnectionDialog()
	 * @category model accessing
	 */
	public JunInternetConnectionDialog getInternetConnectionDialog() {
		return (JunInternetConnectionDialog) this.model();
	}

	/**
	 * Answer the accept button.
	 * 
	 * @return java.awt.Button
	 * @category user interface
	 */
	protected Button acceptButton() {
		if (acceptButton == null) {
			acceptButton = new Button($String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						int portNumber = Integer.parseInt(portNumberField().getText());
						getInternetConnectionDialog().portNumber_(portNumber);
					} catch (NumberFormatException ex) {
						JunDialog.Warn_(getInternetConnectionDialog().errorMessage());
						return;
					}

					getInternetConnectionDialog().useHttpProxy_(useHttpProxyCheckbox().getState());
					getInternetConnectionDialog().proxyHost_(proxyHostField().getText());

					if (getInternetConnectionDialog().acceptButtonPressed()) {
						getInternetConnectionDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Answer the cancel button.
	 * 
	 * @return java.awt.Button
	 * @category user interface
	 */
	protected Button cancelButton() {
		if (cancelButton == null) {
			cancelButton = new Button($String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (getInternetConnectionDialog().cancelButtonPressed()) {
						getInternetConnectionDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer the direct connection checkbox.
	 * 
	 * @return java.awt.Checkbox
	 * @category user interface
	 */
	protected Checkbox directConnectionCheckbox() {
		if (directConnectionCheckbox == null) {
			boolean useHttpProxy = this.getInternetConnectionDialog().useHttpProxy();
			directConnectionCheckbox = new Checkbox($String("Direct Connection"), useHttpProxy == false, this.checkboxGroup());
			directConnectionCheckbox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					getInternetConnectionDialog().directConnectionButtonPressed();
				}
			});
			this.updateForUseHttpProxy();
		}
		return directConnectionCheckbox;
	}

	/**
	 * Answer the HTTP proxy using checkbox.
	 * 
	 * @return java.awt.Checkbox
	 * @category user interface
	 */
	protected Checkbox useHttpProxyCheckbox() {
		if (useHttpProxyCheckbox == null) {
			boolean useHttpProxy = this.getInternetConnectionDialog().useHttpProxy();
			useHttpProxyCheckbox = new Checkbox($String("Use HTTP Proxy"), useHttpProxy == true, this.checkboxGroup());
			useHttpProxyCheckbox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					getInternetConnectionDialog().useHttpProxyButtonPressed();
				}
			});
			this.updateForUseHttpProxy();
		}
		return useHttpProxyCheckbox;
	}

	/**
	 * Answer the text field for "Proxy Host".
	 * 
	 * @return java.awt.TextField
	 * @category user interface
	 */
	protected TextField proxyHostField() {
		if (proxyHostField == null) {
			proxyHostField = new TextField();
			this.updateForProxyHost();
		}
		return proxyHostField;
	}

	/**
	 * Answer the text field for "Port Number".
	 * 
	 * @return javax.swing.JTextField
	 * @category user interface
	 */
	protected TextField portNumberField() {
		if (portNumberField == null) {
			portNumberField = new TextField();
			this.updateForPortNumber();
		}
		return portNumberField;
	}

	/**
	 * Answer the checkbox group for the radio buttons.
	 * 
	 * @return java.awt.CheckboxGroup
	 * @category
	 */
	private CheckboxGroup checkboxGroup() {
		if (_checkboxGroup == null) {
			_checkboxGroup = new CheckboxGroup();
		}
		return _checkboxGroup;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	@Override
	public void update_(DependentEvent e) {
		StSymbol aspect = e.getAspect();

		if (aspect == $("proxyHost")) {
			this.updateForProxyHost();
		} else if (aspect == $("portNumber")) {
			this.updateForPortNumber();
		} else if (aspect == $("useHttpProxy")) {
			this.updateForUseHttpProxy();
		}
	}

	/**
	 * Update for the change notice of the proxy host.
	 * 
	 * @category updating
	 */
	protected void updateForProxyHost() {
		String proxyHost = this.getInternetConnectionDialog().proxyHost();
		if (proxyHost.equals(this.proxyHostField().getText())) {
			return;
		}

		this.proxyHostField().setText(proxyHost);
	}

	/**
	 * Update for the change notice of the port number.
	 * 
	 * @category updating
	 */
	protected void updateForPortNumber() {
		int portNumber = this.getInternetConnectionDialog().portNumber();
		this.portNumberField().setText(String.valueOf(portNumber));
	}

	/**
	 * Update for the change notice of the proxy type.
	 * 
	 * @category updating
	 */
	protected void updateForUseHttpProxy() {
		boolean useHttpProxy = this.getInternetConnectionDialog().useHttpProxy();
		if (useHttpProxy) {
			this.useHttpProxyCheckbox().setState(true);
		} else {
			this.directConnectionCheckbox().setState(true);
		}
		this.enableProxyFields(useHttpProxy);
	}

	/**
	 * Enables/Disables the proxy fields.
	 * 
	 * @param enabled boolean
	 * @category private
	 */
	protected void enableProxyFields(boolean enabled) {
		this.proxyHostField().setEnabled(enabled);
		this.portNumberField().setEnabled(enabled);
	}

}
