package jp.co.sra.jun.system.dialog;

import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.StSimpleDialogViewSwing;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.WindowSpecLayout;

import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunInternetConnectionDialogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2008/08/15 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun704 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunInternetConnectionDialogViewSwing.java,v 9.7 2010/03/09 09:17:47 nisinaka Exp $
 */
public class JunInternetConnectionDialogViewSwing extends StSimpleDialogViewSwing implements JunInternetConnectionDialogView {

	protected JButton acceptButton;
	protected JButton cancelButton;
	protected JRadioButton directConnectionButton;
	protected JRadioButton useHttpProxyButton;
	protected JTextField proxyHostField;
	protected JTextField portNumberField;

	private ButtonGroup _buttonGroup;

	/**
	 * Create a new instance of JunInternetConnectionDialogViewSwing and initialize it.
	 *
	 * @param anInternectConnectionDialog jp.co.sra.jun.system.dialog.JunInternetConnectionDialog
	 * @category Instance creation
	 */
	public JunInternetConnectionDialogViewSwing(JunInternetConnectionDialog anInternectConnectionDialog) {
		super(anInternectConnectionDialog);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	@Override
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -296, 0f, 152, 1f, -208, 0f, 184));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -152, 0f, 152, 1f, -64, 0f, 184));
		this.add(this.directConnectionButton(), WindowSpecLayout.Position(0f, 16, 0f, 16));
		this.add(this.useHttpProxyButton(), WindowSpecLayout.Position(0f, 16, 0f, 48));
		this.add(new JLabel($String("Proxy Host")), WindowSpecLayout.Position(0f, 48, 0f, 80));
		this.add(new JLabel($String("Port Number")), WindowSpecLayout.Position(0f, 48, 0f, 112));
		this.add(this.proxyHostField(), WindowSpecLayout.Position(0f, 144, 0f, 80, 0f, 344, 0f, 104));
		this.add(this.portNumberField(), WindowSpecLayout.Position(0f, 144, 0f, 112, 0f, 200, 0f, 136));
		this.setPreferredSize(new Dimension(360, 200));
	}

	/**
	 * Answer my model as JunInternetConnectionDialog.
	 * 
	 * @return jp.co.sra.jun.system.dialog.JunInternetConnectionDialog
	 * @see jp.co.sra.jun.system.dialog.JunInternetConnectionDialogView#getInternetConnectionDialog()
	 * @category model accessing
	 */
	public JunInternetConnectionDialog getInternetConnectionDialog() {
		return (JunInternetConnectionDialog) this.model();
	}

	/**
	 * Answer the accept button.
	 * 
	 * @return jp.co.sra.jun.system.dialog.JunInternetConnectionDialogViewSwing.acceptButton()
	 * @category user interface
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(new AbstractAction($String("Accept")) {
				public void actionPerformed(ActionEvent e) {
					try {
						int portNumber = Integer.parseInt(portNumberField().getText());
						getInternetConnectionDialog().portNumber_(portNumber);
					} catch (NumberFormatException ex) {
						JunDialog.Warn_(getInternetConnectionDialog().errorMessage());
						return;
					}

					getInternetConnectionDialog().useHttpProxy_(useHttpProxyButton().isSelected());
					getInternetConnectionDialog().proxyHost_(proxyHostField().getText());

					if (getInternetConnectionDialog().acceptButtonPressed()) {
						getInternetConnectionDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Answer the cancel button.
	 * 
	 * @return jp.co.sra.jun.system.dialog.JunInternetConnectionDialogViewSwing.cancelButton()
	 * @category user interface
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(new AbstractAction($String("Cancel")) {
				public void actionPerformed(ActionEvent e) {
					if (getInternetConnectionDialog().cancelButtonPressed()) {
						getInternetConnectionDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer the direct connection radio button.
	 * 
	 * @return javax.swing.JRadioButton
	 * @category user interface
	 */
	protected JRadioButton directConnectionButton() {
		if (directConnectionButton == null) {
			directConnectionButton = new JRadioButton(new AbstractAction($String("Direct Connection")) {
				public void actionPerformed(ActionEvent e) {
					getInternetConnectionDialog().directConnectionButtonPressed();
				}
			});
			this.buttonGroup().add(directConnectionButton);
			this.updateForUseHttpProxy();
		}
		return directConnectionButton;
	}

	/**
	 * Answer the HTTP proxy using radio button.
	 * 
	 * @return javax.swing.JRadioButton
	 * @category user interface
	 */
	protected JRadioButton useHttpProxyButton() {
		if (useHttpProxyButton == null) {
			useHttpProxyButton = new JRadioButton(new AbstractAction($String("Use HTTP Proxy")) {
				public void actionPerformed(ActionEvent e) {
					getInternetConnectionDialog().useHttpProxyButtonPressed();
				}
			});
			this.buttonGroup().add(useHttpProxyButton);
			this.updateForUseHttpProxy();
		}
		return useHttpProxyButton;
	}

	/**
	 * Answer the text field for "Proxy Host".
	 * 
	 * @return javax.swing.JTextField
	 * @category user interface
	 */
	protected JTextField proxyHostField() {
		if (proxyHostField == null) {
			proxyHostField = new JTextField();
			this.updateForProxyHost();
		}
		return proxyHostField;
	}

	/**
	 * Answer the text field for "Port Number".
	 * 
	 * @return javax.swing.JTextField
	 * @category user interface
	 */
	protected JTextField portNumberField() {
		if (portNumberField == null) {
			portNumberField = new JTextField();
			this.updateForPortNumber();
		}
		return portNumberField;
	}

	/**
	 * Answer the button group for the radio buttons.
	 * 
	 * @return javax.swing.ButtonGroup
	 * @category user interface
	 */
	private ButtonGroup buttonGroup() {
		if (_buttonGroup == null) {
			_buttonGroup = new ButtonGroup();
		}
		return _buttonGroup;
	}

	/**
	 * Update for the change notice.
	 * 
	 * @param e jp.co.sra.smalltalk.DependentEvent
	 * @see jp.co.sra.smalltalk.StViewJPanel#update_(jp.co.sra.smalltalk.DependentEvent)
	 * @category updating
	 */
	@Override
	public void update_(DependentEvent e) {
		StSymbol aspect = e.getAspect();

		if (aspect == $("proxyHost")) {
			this.updateForProxyHost();
		} else if (aspect == $("portNumber")) {
			this.updateForPortNumber();
		} else if (aspect == $("useHttpProxy")) {
			this.updateForUseHttpProxy();
		}
	}

	/**
	 * Update for the change notice of the proxy host.
	 * 
	 * @category updating
	 */
	protected void updateForProxyHost() {
		String proxyHost = this.getInternetConnectionDialog().proxyHost();
		if (proxyHost.equals(this.proxyHostField().getText())) {
			return;
		}

		this.proxyHostField().setText(proxyHost);
	}

	/**
	 * Update for the change notice of the port number.
	 * 
	 * @category updating
	 */
	protected void updateForPortNumber() {
		int portNumber = this.getInternetConnectionDialog().portNumber();
		this.portNumberField().setText(String.valueOf(portNumber));
	}

	/**
	 * Update for the change notice of the proxy type.
	 * 
	 * @category updating
	 */
	protected void updateForUseHttpProxy() {
		boolean useHttpProxy = this.getInternetConnectionDialog().useHttpProxy();
		if (useHttpProxy) {
			this.useHttpProxyButton().setSelected(true);
		} else {
			this.directConnectionButton().setSelected(true);
		}
		this.enableProxyFields(useHttpProxy);
	}

	/**
	 * Enables/Disables the proxy fields.
	 * 
	 * @param enabled boolean
	 * @category private
	 */
	protected void enableProxyFields(boolean enabled) {
		this.proxyHostField().setEnabled(enabled);
		this.portNumberField().setEnabled(enabled);
	}

}
