package jp.co.sra.jun.system.dialog;

import java.awt.Component;
import java.awt.Rectangle;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StBlockValue;
import jp.co.sra.smalltalk.StRectangle;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

/**
 * JunTextInputField class
 * 
 *  @author    nisinaka
 *  @created   2008/05/15 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun706 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTextInputField.java,v 9.13 2010/03/09 09:17:47 nisinaka Exp $
 */
public class JunTextInputField extends JunPopUpDialog {

	protected StValueHolder<String> textHolder;
	protected String initialAnswer;
	protected StBlockClosure2<Void, Character, String> checkBlock;

	/**
	 * Popup the dialog and return the value.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static String PopUpIn_(Rectangle aRectangle) {
		JunTextInputField aDialog = new JunTextInputField();
		aDialog.popUpIn_(new StRectangle(aRectangle));
		return (String) aDialog.answer();
	}

	/**
	 * Popup the dialog and return the value.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static String PopUpIn_initialAnswer_(Rectangle aRectangle, String aString) {
		JunTextInputField aDialog = new JunTextInputField();
		aDialog.initialAnswer_(aString);
		aDialog.popUpIn_(new StRectangle(aRectangle));
		return (String) aDialog.answer();
	}

	/**
	 * Popup the dialog and return the value.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * @param aComponent java.awt.Component
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static String PopUpIn_forView_(Rectangle aRectangle, Component aComponent) {
		JunTextInputField aDialog = new JunTextInputField();
		aDialog.popUpIn_forView_(aRectangle, aComponent);
		return (String) aDialog.answer();
	}

	/**
	 * Popup the dialog and return the value.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * @param aComponent java.awt.Component
	 * @param aString java.lang.String
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static String PopUpIn_forView_initialAnswer_(Rectangle aRectangle, Component aComponent, String aString) {
		JunTextInputField aDialog = new JunTextInputField();
		aDialog.initialAnswer_(aString);
		aDialog.popUpIn_forView_(aRectangle, aComponent);
		return (String) aDialog.answer();
	}

	/**
	 * Popup the dialog and return the value.
	 * 
	 * @param aComponent java.awt.Component
	 * @param aString java.lang.String
	 * @return java.lang.String
	 * @category Utilities
	 */
	public static String PopUpForView_initialAnswer_(Component aComponent, String aString) {
		JunTextInputField aDialog = new JunTextInputField();
		aDialog.initialAnswer_(aString);
		aDialog.popUpForView_(aComponent);
		return (String) aDialog.answer();
	}

	/**
	 * Answer the size of the line grid.
	 * 
	 * @return int
	 * @category Utilities
	 */
	public static int LineGrid() {
		return 14;
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StSimpleDialog#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		textHolder = null;
		initialAnswer = "";
		checkBlock = null;
	}

	/**
	 * Answer my value.
	 * 
	 * @return java.lang.Object
	 * @see jp.co.sra.jun.system.dialog.JunPopUpDialog#answer()
	 * @category accessing
	 */
	public Object answer() {
		return this.string();
	}

	/**
	 * Answer my current string value.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String string() {
		return this.asString();
	}

	/**
	 * Set my new string value.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public void string_(String aString) {
		this.textHolder().value_(aString);
	}

	/**
	 * Answer my text holder.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<String> textHolder() {
		if (textHolder == null) {
			textHolder = new StValueHolder<String>("");
		}
		return textHolder;
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return jp.co.sra.smalltalk.StBlockValue1
	 * @category constructing
	 */
	public <R> StBlockValue<R> compute_(StBlockClosure1<R, String> aBlock) {
		return this.textHolder().compute_(aBlock);
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as the argument.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.smalltalk.StBlockValue
	 * @deprecated since Jun701, use Generics.
	 * @category constructing
	 */
	public <R> StBlockValue<R> compute_(final StBlockClosure<R, ?, ?, ?> aBlock) {
		return this.compute_(new StBlockClosure1<R, String>() {
			public R value(String aString) {
				switch (aBlock.numArgs()) {
					case 0:
						return aBlock.value();
					case 1:
						return aBlock.value_(aString);
					default:
						return null;
				}
			}
		});
	}

	/**
	 * Set a block closure for checking.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @category constructing
	 */
	public void check_(final StBlockClosure0<Void> aBlock) {
		this.check_(new StBlockClosure2<Void, Character, String>() {
			public Void value(Character aCharacter, String aString) {
				if (aBlock != null) {
					aBlock.value();
				}
				return null;
			}
		});
	}

	/**
	 * Set a block closure for checking.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category constructing
	 */
	public void check_(final StBlockClosure1<Void, Character> aBlock) {
		this.check_(new StBlockClosure2<Void, Character, String>() {
			public Void value(Character aCharacter, String aString) {
				if (aBlock != null) {
					aBlock.value(aCharacter);
				}
				return null;
			}
		});
	}

	/**
	 * Set a block closure for checking.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @category constructing
	 */
	public void check_(StBlockClosure2<Void, Character, String> aBlock) {
		checkBlock = aBlock;
	}

	/**
	 * Set a block closure for checking.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @deprecated since Jun701, use Generics.
	 * @category constructing
	 */
	public void check_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.check_(new StBlockClosure2<Void, Character, String>() {
			public Void value(Character aCharacter, String aString) {
				if (aBlock != null) {
					switch (aBlock.numArgs()) {
						case 0:
							aBlock.value();
							break;
						case 1:
							aBlock.value_(aCharacter);
							break;
						case 2:
							aBlock.value_value_(aCharacter, aString);
							break;
					}
				}
				return null;
			}
		});
	}

	/**
	 * Convert the receiver to a string.
	 * 
	 * @return java.lang.String
	 * @category converting
	 */
	public String asString() {
		return this.textHolder().value();
	}

	/**
	 * Answer a default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.jun.system.dialog.JunPopUpDialog#defaultView()
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunTextInputFieldViewAwt(this);
		} else {
			return new JunTextInputFieldViewSwing(this);
		}
	}

	/**
	 * Popup the receiver in the specified size and place.
	 * 
	 * @param aRectangle jp.co.sra.smalltalk.StRectangle
	 * @param aString java.lang.String
	 * @category interface opening
	 */
	public void popUpIn_initialAnswer_(StRectangle aRectangle, String aString) {
		this.initialAnswer_(aString);
		this.popUpIn_(aRectangle);
	}

	/**
	 * Set my initial answer.
	 * 
	 * @param aString java.lang.String
	 * @category private
	 */
	protected void initialAnswer_(String aString) {
		initialAnswer = aString;
		this.string_(initialAnswer);
	}

	/**
	 * Action when closing by the ESC key.
	 * 
	 * @see jp.co.sra.jun.system.dialog.JunPopUpDialog#actionByESC()
	 * @category private
	 */
	protected void actionByESC() {
		this.string_(initialAnswer);
	}

	/**
	 * Check for the character.
	 * 
	 * @param keyChar char
	 * @return boolean
	 * @see jp.co.sra.jun.system.dialog.JunPopUpDialog#checkCharacter_(char)
	 * @category private 
	 */
	protected boolean checkCharacter_(final char keyChar) {
		Thread aThread = new Thread() {
			public void run() {
				Thread.yield();

				if (checkBlock != null) {
					checkBlock.value(keyChar, string());
				}
			}
		};
		aThread.start();

		return super.checkCharacter_(keyChar);
	}

}
