package jp.co.sra.jun.system.dialog;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.TextField;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;

import jp.co.sra.smalltalk.DependentEvent;
import jp.co.sra.smalltalk.DependentListener;
import jp.co.sra.smalltalk.WindowSpecLayout;

/**
 * JunTextInputFieldViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2006/10/16 (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on Jun651 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTextInputFieldViewAwt.java,v 9.9 2010/03/09 09:17:47 nisinaka Exp $
 */
public class JunTextInputFieldViewAwt extends JunPopUpDialogViewAwt implements JunTextInputFieldView {

	protected TextField textField;

	/**
	 * Create a new instance of JunTextInputFieldViewAwt and initialize it.
	 *
	 * @param aTextInputField jp.co.sra.jun.system.dialog.JunTextInputField
	 * @category Instance creation
	 */
	public JunTextInputFieldViewAwt(JunTextInputField aTextInputField) {
		super(aTextInputField);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.smalltalk.StViewJPanel#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.textField(), WindowSpecLayout.Position(0f, 0, 0f, 0, 1f, 0, 1f, 0));
		this.setPreferredSize(new Dimension(200, 24));
	}

	/**
	 * Answer my model as JunTextInputField.
	 * 
	 * @return jp.co.sra.jun.system.dialog.JunTextInputField
	 * @see jp.co.sra.jun.system.dialog.JunTextInputFieldView#getTextInputField()
	 * @category model accessing
	 */
	public JunTextInputField getTextInputField() {
		return (JunTextInputField) this.model();
	}

	/**
	 * Answer my focused component.
	 * 
	 * @return java.awt.Component
	 * @see jp.co.sra.jun.system.dialog.JunPopUpDialogViewAwt#_focusedComponent()
	 * @category user interface
	 */
	public Component _focusedComponent() {
		return this.textField();
	}

	/**
	 * Answer the text field.
	 * 
	 * @return java.awt.TextField
	 * @category user interface
	 */
	protected TextField textField() {
		if (textField == null) {
			textField = new TextField();
			textField.setText(this.getTextInputField().string());
			textField.selectAll();
			textField.addTextListener(new TextListener() {
				public void textValueChanged(TextEvent e) {
					getTextInputField().string_(textField.getText());
				}
			});

			this.getTextInputField().textHolder().addDependentListener(new DependentListener() {
				public void update_(DependentEvent evt) {
					String aString = getTextInputField().string();
					if (textField.getText().equals(aString) == false) {
						textField.setText(aString);
					}
				}
			});
		}
		return textField;
	}

}
