package jp.co.sra.jun.system.support;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import jp.co.sra.smalltalk.StApplicationModel;

import jp.co.sra.jun.goodies.display.JunDisplayModel;
import jp.co.sra.jun.goodies.display.JunDisplayViewSwing;
import jp.co.sra.jun.goodies.utilities.JunControlUtility;

/**
 * JunLauncherViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/11/11 (by MATSUDA Ryouichi)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/19 (by nisinaka)
 *  @updated   2003/03/20 (by Mitsuhiro Asada)
 *  @updated   2007/03/12 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLauncherViewSwing.java,v 9.8 2010/03/09 09:17:48 nisinaka Exp $
 */
public class JunLauncherViewSwing extends JunDisplayViewSwing implements JunLauncherView {

	/**
	 * Create a new instance of JunLauncherViewSwing and initialize it.
	 * 
	 * @param aLauncher jp.co.sra.jun.system.support.JunLauncher
	 * @category Instance creation
	 */
	public JunLauncherViewSwing(JunLauncher aLauncher) {
		super(aLauncher);
	}

	/**
	 * Build this component.
	 * 
	 * @see jp.co.sra.jun.goodies.display.JunDisplayViewSwing#buildComponent()
	 * @category initialize-release
	 */
	protected void buildComponent() {
		super.buildComponent();

		this.canvas().setBackground(Color.white);
	}

	/**
	 * Open a banner.
	 *
	 * @param visual java.lang.Object
	 * @see jp.kidresearch.art.system.support.ArtLauncherView#_openBanner(java.lang.Object)
	 * @category utilities
	 */
	public void _openBanner(Object visual) {
		JunDisplayModel displayModel = new JunDisplayModel(visual);
		displayModel._isScrollable(false);
		JunDisplayViewSwing displayView = new JunDisplayViewSwing(displayModel);

		final JDialog aDialog = new JDialog((Frame) null, true);
		aDialog.setUndecorated(true);
		aDialog.setResizable(false);
		aDialog.getContentPane().add(displayView);
		aDialog.pack();

		displayView.canvasComponent().addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				synchronized (JunLauncherViewSwing.this) {
					if (aDialog.isShowing()) {
						aDialog.dispose();
					}
				}
			}
		});

		Thread aThread = new Thread() {
			public void run() {
				JunControlUtility.WaitForMilliseconds_(5000);
				synchronized (JunLauncherViewSwing.this) {
					if (aDialog.isShowing()) {
						aDialog.dispose();
					}
				}
			}
		};
		aThread.start();

		StApplicationModel._ShowAtCenterPoint(aDialog);
	}

	/**
	 * Open a dialog.
	 *
	 * @param title java.lang.String
	 * @param visual java.lang.Object
	 * @param size java.awt.Dimension
	 * @see jp.co.sra.jun.system.support.JunLauncherView#_openDialog_(java.lang.String, java.lang.Object, java.awt.Dimension)
	 * @category utilities
	 */
	public void _openDialog_(final String title, final Object visual, final Dimension size) {
		JunDisplayModel aModel = new JunDisplayModel(visual);
		aModel._isScrollable(false);
		JunDisplayViewSwing aComponent = new JunDisplayViewSwing(aModel);

		JFrame aFrame = new JFrame(title);
		aFrame.getContentPane().add(aComponent);
		aFrame.pack();
		aFrame.setResizable(false);
		if (size != null) {
			aFrame.setSize(size);
		}
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
		aFrame.pack();

		StApplicationModel._ShowAtCenterPoint(aFrame);
	}

	/**
	 * Open a text viewer.
	 *
	 * @param title java.lang.String
	 * @param message java.lang.String 
	 * @param size java.awt.Dimension
	 * @category utilities
	 * @see jp.co.sra.jun.system.support.JunLauncherView#_openTextViewer_(java.lang.String, java.lang.String, java.awt.Dimension)
	 * @category utilities
	 */
	public void _openTextViewer_(final String title, final String message, final Dimension size) {
		JTextArea aTextArea = new JTextArea(message, 0, 0);
		aTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		aTextArea.setBackground(Color.white);
		aTextArea.setEditable(false);
		aTextArea.setLineWrap(true);
		aTextArea.setWrapStyleWord(true);
		aTextArea.setCaretPosition(0);

		JScrollPane aScrollPane = new JScrollPane(aTextArea);
		aScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		aScrollPane.setPreferredSize(size);

		JFrame aFrame = new JFrame(title);
		aFrame.getContentPane().add(aScrollPane);
		aFrame.pack();
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		StApplicationModel._ShowAtCenterPoint(aFrame);
	}

}
