package jp.co.sra.jun.system.support;

import java.awt.Color;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.goodies.display.JunDisplayModel;

/**
 * JunSmallCompilerTestExamples class
 * 
 *  @author    ASTI Beijing
 *  @created   1998/12/09 (by ASTI Beijing)
 *  @updated   2003/05/07 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSmallCompilerTestExamples.java,v 9.9 2010/03/09 09:17:48 nisinaka Exp $
 */
public class JunSmallCompilerTestExamples extends JunTestExamples {

	/**
	 * Example_LongNumber: evaluate a long number.
	 * 
	 * @return boolean
	 */
	public static boolean Example_LongNumber() {
		Object longNumber = JunSmallCompiler.Evaluate_("2002112612345600000");
		System.out.println(longNumber);
		return (longNumber instanceof Long && ((Long) longNumber).longValue() == 2002112612345600000L);
	}

	/**
	 * Example for some constant values.
	 * 
	 * @return boolean
	 */
	public static boolean Example0() {
		Object nilObject = JunSmallCompiler.Evaluate_("(nil)");
		System.out.println(nilObject);
		Object trueObject = JunSmallCompiler.Evaluate_("(true)");
		System.out.println(trueObject);
		Object falseObject = JunSmallCompiler.Evaluate_("(false)");
		System.out.println(falseObject);

		return nilObject == null && trueObject instanceof Boolean && ((Boolean) trueObject).booleanValue() == true && falseObject instanceof Boolean && ((Boolean) falseObject).booleanValue() == false;
	}

	/**
	 * Example for some numbers.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		Object value1 = JunSmallCompiler.Evaluate_("(123)");
		System.out.println(value1);
		Object value2 = JunSmallCompiler.Evaluate_("(123.456)");
		System.out.println(value2);
		Object value3 = JunSmallCompiler.Evaluate_("(123.456d7)");
		System.out.println(value3);
		Object value4 = JunSmallCompiler.Evaluate_("(-123)");
		System.out.println(value4);
		Object value5 = JunSmallCompiler.Evaluate_("(-123.456)");
		System.out.println(value5);
		Object value6 = JunSmallCompiler.Evaluate_("(-123.456d7)");
		System.out.println(value6);
		Object value7 = JunSmallCompiler.Evaluate_("(-123.456d-7)");
		System.out.println(value7);

		return value1 instanceof Integer && ((Integer) value1).intValue() == 123 && value2 instanceof Float && ((Float) value2).floatValue() == 123.456f && value3 instanceof Double && ((Double) value3).doubleValue() == 1.23456E9
				&& value4 instanceof Integer && ((Integer) value4).intValue() == -123 && value5 instanceof Float && ((Float) value5).floatValue() == -123.456 && value6 instanceof Double && ((Double) value6).doubleValue() == -1.23456E9
				&& value7 instanceof Double && ((Double) value7).doubleValue() == -1.23456E5;
	}

	/**
	 * Example for some literal values.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		Object aCharacter = JunSmallCompiler.Evaluate_("($c)");
		System.out.println(aCharacter);
		Object aString = JunSmallCompiler.Evaluate_("('string')");
		System.out.println(aString);
		Object aSymbol = JunSmallCompiler.Evaluate_("(#symbol)");
		System.out.println(aSymbol);

		return aCharacter instanceof Character && ((Character) aCharacter).charValue() == 'c' && aString instanceof String && "string".equals(aString) && aSymbol instanceof StSymbol && aSymbol == StSymbol.Intern_("symbol");
	}

	/**
	 * Example for some points.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		Object point1 = JunSmallCompiler.Evaluate_("(123 @ 456)");
		System.out.println(point1);
		Object point2 = JunSmallCompiler.Evaluate_("(123 , 456)");
		System.out.println(point2);
		Object point3 = JunSmallCompiler.Evaluate_("(123 , 456 , 789)");
		System.out.println(point3);
		Object point4 = JunSmallCompiler.Evaluate_("(Point x: 123 y: 456)");
		System.out.println(point4);
		Object point5 = JunSmallCompiler.Evaluate_("(Jun2dPoint x: 123 y: 456)");
		System.out.println(point5);
		Object point6 = JunSmallCompiler.Evaluate_("(Jun3dPoint x: 123 y: 456 z: 789)");
		System.out.println(point6);

		return point1 instanceof Jun2dPoint && ((Jun2dPoint) point1).x() == 123 && ((Jun2dPoint) point1).y() == 456 && point2 instanceof Jun2dPoint && ((Jun2dPoint) point2).x() == 123 && ((Jun2dPoint) point2).y() == 456 && point3 instanceof Jun3dPoint
				&& ((Jun3dPoint) point3).x() == 123 && ((Jun3dPoint) point3).y() == 456 && ((Jun3dPoint) point3).z() == 789 && point4 instanceof Jun2dPoint && ((Jun2dPoint) point4).x() == 123 && ((Jun2dPoint) point4).y() == 456
				&& point5 instanceof Jun2dPoint && ((Jun2dPoint) point5).x() == 123 && ((Jun2dPoint) point5).y() == 456 && point6 instanceof Jun3dPoint && ((Jun3dPoint) point6).x() == 123 && ((Jun3dPoint) point6).y() == 456
				&& ((Jun3dPoint) point6).z() == 789;
	}

	/**
	 * Example for some colors.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		Object color1 = JunSmallCompiler.Evaluate_("(ColorValue hue: 0.57 saturation: 0.46 brightness: 1.0)");
		System.out.println(color1);
		Object color2 = JunSmallCompiler.Evaluate_("(ColorValue red: 0.57 green: 0.46 blue: 1.0)");
		System.out.println(color2);
		/*
		Object color3 = JunSmallCompiler.Evaluate_("(ColorValue cyan: 0.57 magenta: 0.46 yellow: 1.0)");
		System.out.println(color3);
		Object color4 = JunSmallCompiler.Evaluate_("(ColorValue brightness: 0.7)");
		System.out.println(color4);
		*/
		Object color5 = JunSmallCompiler.Evaluate_("(ColorValue scaledRed: 546 scaledGreen: 546 scaledBlue: 546)");
		System.out.println(color5);

		return color1 instanceof Color && ((Color) color1).getRed() == 138 && ((Color) color1).getGreen() == 206 && ((Color) color1).getBlue() == 255 && color2 instanceof Color && ((Color) color2).getRed() == 145 && ((Color) color2).getGreen() == 117
				&& ((Color) color2).getBlue() == 255 && color5 instanceof Color && ((Color) color5).getRed() == 17 && ((Color) color5).getGreen() == 17 && ((Color) color5).getBlue() == 17;
	}

	/**
	 * Example for an image.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		Object anImage = JunSmallCompiler
				.Evaluate_("(Image extent: 6@7 depth: 24 bitsPerPixel: 24 palette: (FixedPalette redShift: 3 redMask: 31 greenShift: 11 greenMask: 31 blueShift: 19 blueMask: 31) usingBits: #[127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 255 255 255 0 0 255 255 255 255 255 255 127 0 0 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 127 0 0 0 0 127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 0 0])");
		JunDisplayModel.Show_(anImage);

		return true;
	}

	/**
	 * Example for an image.
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		Object anImage = JunSmallCompiler
				.Evaluate_("(Image extent: 14@15 depth: 24 bitsPerPixel: 24 palette: (FixedPalette redShift: 3 redMask: 31 greenShift: 11 greenMask: 31 blueShift: 19 blueMask: 31) usingBits: (ByteArray fromPackedString: \'????????????????????????????@@@@????????????@@@@????????@@@@@@C???????????<@@@C???<@@@@@@@@@@@C???<@@@@@@@@@@@C???<@@O???0@@@O???????0@@@O???????0@@@O???????????0@@@O???????0@@????@@@@????????@@@@????????@@@@????????????@@@@????????@@C???<@@@C???????<@@@C???????<@@@C???????????<@@@C???????<@@O???0@@@O???????0@@@O???????0@@@O???????????????0@@@O???0@@????????????????????????????????????????????????????????@@C???????????????????????????????????????????????????????<@@O???????????????????????????????????????????????????????0@@????????????????????????????????????????????????????????@@@@@@@@@@@@@@C???????<@@@@@@@@@@@@@@@@@@@C???<@@@@@@@@@@@@@@O???????????0@@@O???0@@@O???????????????????0@@@O???????0@@????????????@@@@????@@@@@@@@@@@@@@@@????????@@@@@@@@@@@@@@C???????????<@@@C???<@@@C???????????<@@@C???<@@@C???????<@@O???0@@@@@@@O???????????????????????0@@@O???????????????0@@\'))");
		JunDisplayModel.Show_(anImage);

		return true;
	}

	/**
	 * Example for an image.
	 * 
	 * @return boolean
	 */
	public static boolean Example9() {
		Object anImage = JunSmallCompiler.Evaluate_("(Image extent: 16@16 depth: 1 bitsPerPixel: 1 palette: MonoMappedPalette whiteBlack " + " usingBits: #[ " + "   0 255   0 255   0 255   0 255   0 255   0 255   0 255   0 255 "
				+ " 255   0 255   0 255   0 255   0 255   0 255   0 255   0 255   0 " + "   0 255   0 255   0 255   0 255   0 255   0 255   0 255   0 255 " + " 255   0 255   0 255   0 255   0 255   0 255   0 255   0 255   0 " + " ]) ");
		JunDisplayModel.Show_(anImage);

		return true;
	}

	/**
	 * Example for an image.
	 * 
	 * @return boolean
	 */
	public static boolean ExampleX() {
		Object anImage = JunSmallCompiler
				.Evaluate_("(Image extent: 32@16 depth: 24 bitsPerPixel: 24 palette: (FixedPalette redShift: 16 redMask: 255 greenShift: 8 greenMask: 255 blueShift: 0 blueMask: 255) usingBits: (ByteArray fromPackedString: '@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD?0@@?0@@?0@@?0@@?0@@?0@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@@@BD@@BD@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@?0@@??<@??<@??<@??<@??<@??<@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@@HRD@HRD@@@@?0@@?0@@?0@@?0@@?0@@?0@@??<@??<@??<@??<@??<@??<@??<@??<@??<@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@HRD@HRD@HRD@@@@@@@@@@@@@@@@??<@??<@??<@??<@??<@??<@??<@??<@??<@??<@??<@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@@O??@O??@HRD@HRD@HRD@HRD@HRD@HRD@HRD??<@??<@??<@??<@@@@@@@@@??<@??<@@@@@@@@@??<@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@O??@O??@@@@@@@@@HRD@HRD@HRD@HRD@HRD??<@??<@@@@@@@@@??<@??<@??<@??<@??<@??<@??<@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@@O??@O??@O??@O??@O??@O??@HRD@HRD@HRD@HRD@HRD@@@@@@@@??<@??<@??<@??<@??<@??<@??<@??<@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@O??@O??@O??@O??@O??@O??@O??@HRD@HRD@HRD@HRD@HRD@HRD@@@@@@@@??<@??<@??<@??<@@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@@@@@@@@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@O??@O??@O??@O??@O??@O??@O??@O??@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@O??@O??@O??@O??@O??@O??@@@@@@@@@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD@@BD'))");
		JunDisplayModel.Show_(anImage);

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunSmallCompilerTestExamples();
	}

}
