package jp.co.sra.jun.topology.abstracts;

import java.util.ArrayList;

import jp.co.sra.jun.topology.compositeoperators.JunCompoundOperator;

/**
 * JunSetOperator class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/22 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSetOperator.java,v 9.9 2010/03/09 09:17:52 nisinaka Exp $
 */
public abstract class JunSetOperator extends JunAbstractOperator {

	protected ArrayList<JunAbstractOperator> suboperators;
	protected JunAbstractOperator inverse;

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		suboperators = null;
		inverse = null;
	}

	/**
	 * Answer the receiver's suboperators as an array of JunAbstractOperator.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator[]
	 * @category accessing
	 */
	public JunAbstractOperator[] _suboperatorsAsArray() {
		if (suboperators == null) {
			return null;
		}
		return suboperators.toArray(new JunAbstractOperator[suboperators.size()]);
	}

	/**
	 * Answer the inverse operator of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		if (inverse == null) {
			inverse = JunCompoundOperator.Body_suboperators_(this.body(), this._suboperatorsAsArray()).inverse();
		}
		return inverse;
	}

	/**
	 * Answer <code>true</code> if the receiver can be inverted, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#isInversable()
	 * @category testing
	 */
	public boolean isInversable() {
		return true;
	}

	/**
	 * Redo the operator.
	 * 
	 * @category executing
	 */
	public void redo() {
		this.inverse().inverse().doOperation();
	}

}
