package jp.co.sra.jun.topology.compositeoperators;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.elements.JunBody;

/**
 * JunCompoundOperator class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCompoundOperator.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunCompoundOperator extends JunAbstractOperator {
	/** An array of the sub operators. */
	protected JunAbstractOperator[] suboperators;

	/** A flag of the inversability. */
	protected boolean inversable;

	/**
	 * Create a new JunCompoundOperator with a JunBody and an array of <code>JunAbstractOperators</code>.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anArrayOfJunAbstractOperator jp.co.sra.jun.topology.abstracts.JunAbstractOperator[]
	 * @return jp.co.sra.jun.topology.compositeoperators.JunCompoundOperator
	 * @category Instance creation
	 */
	public static JunCompoundOperator Body_suboperators_(JunBody aBody, JunAbstractOperator[] anArrayOfJunAbstractOperator) {
		JunCompoundOperator anOperator;
		anOperator = new JunCompoundOperator();
		anOperator.body_(aBody);
		anOperator.setSuboperators_(anArrayOfJunAbstractOperator);

		return anOperator;
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		suboperators = null;
		inversable = false;
	}

	/**
	 * Execute all of the operaters.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		for (int i = 0; i < suboperators.length; i++) {
			suboperators[i].doOperation();
		}
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		int size = suboperators.length;
		JunAbstractOperator[] operators = new JunAbstractOperator[size];

		for (int i = 0; i < size; i++) {
			operators[i] = suboperators[size - 1 - i].inverse();
		}

		return Body_suboperators_(this.body(), operators);
	}

	/**
	 * Answer <code>true</code> if the receiver can be inverted.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#isInversable()
	 * @category testing
	 */
	public boolean isInversable() {
		return inversable;
	}

	/**
	 * Set the receivers sub-operators and initialize the inversable flag.
	 * 
	 * @param anArrayOfJunAbstractOperators jp.co.sra.jun.topology.abstracts.JunAbstractOperator[]
	 * @category private
	 */
	protected void setSuboperators_(JunAbstractOperator[] anArrayOfJunAbstractOperators) {
		suboperators = anArrayOfJunAbstractOperators;
		inversable = true;

		for (int i = 0; i < suboperators.length; i++) {
			inversable = inversable && suboperators[i].isInversable();

			if (!inversable) {
				break;
			}
		}
	}
}
