package jp.co.sra.jun.topology.elements;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure3;
import jp.co.sra.smalltalk.StValueHolder;

import jp.co.sra.jun.geometry.abstracts.JunCurve;
import jp.co.sra.jun.geometry.abstracts.JunSurface;
import jp.co.sra.jun.geometry.basic.Jun2dPoint;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.boundaries.Jun2dPolygonalBoundary;
import jp.co.sra.jun.geometry.boundaries.Jun3dBoundingBox;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.curves.JunNurbsCurve;
import jp.co.sra.jun.geometry.surfaces.Jun2dPolygon;
import jp.co.sra.jun.geometry.surfaces.Jun3dPolygon;
import jp.co.sra.jun.geometry.surfaces.JunNurbsSurface;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElement;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy;

/**
 * JunLoop class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2008/05/22 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLoop.java,v 9.9 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunLoop extends JunTopologicalElement {

	/** The edge contained in the loop. */
	protected JunEdge edge;

	/** The parent loop. */
	protected JunLoop parentLoop;

	/** The child loop. */
	protected JunLoop childLoop;

	/** An surface which corresponds to the loop. */
	protected JunSurface surface;

	/**
	 * Create a new instance of <code>JunLoop</code> and initialize it with the JunEdge.
	 * 
	 * @param anEdge jp.co.sra.jun.topology.elements.JunEdge
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category Instance creation
	 */
	public static final JunLoop Edge_(JunEdge anEdge) {
		JunLoop aLoop = new JunLoop();
		aLoop.edge_(anEdge);
		return aLoop;
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		edge = null;
		parentLoop = null;
		childLoop = null;
		surface = null;
	}

	/**
	 * Add a loop as a child loop of the receiver.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category adding
	 */
	public JunLoop addChildLoop_(JunLoop aLoop) {
		if (this.isChildLoop() == true) {
			// return this.parentLoop().addChildLoop_(aLoop);
			throw SmalltalkException.Error("should not be a child loop.");
		}

		aLoop.parentLoop_(this);
		aLoop.childLoop_(this.childLoop());
		childLoop = aLoop;
		return aLoop;
	}

	/**
	 * Convert the surface of the receiver as a JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#asJunOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		JunSurface surface = this.surface();

		if (surface == null) {
			return null;
		}
		if (surface instanceof Jun3dPolygon == false) {
			// So far, it's only support JunPolygon as a surface.
			return null;
		}

		final Jun3dPolygon polygon = (Jun3dPolygon) surface;
		if (this.hasChildLoop() == false) {
			return polygon.asJunOpenGL3dObject();
		}

		Jun2dPolygon parameterPolygon = polygon.parameterPolygon();
		final Jun2dPolygonalBoundary boundary = new Jun2dPolygonalBoundary();
		boundary.addPolygon_(parameterPolygon);
		this.childLoopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop childPolygon) {
				ArrayList<Jun2dPoint> parameterPoints = new ArrayList<Jun2dPoint>();
				Jun3dPoint[] points = childPolygon.points();
				for (int i = 0; i < points.length; i++) {
					parameterPoints.add(polygon.parameterPointAtPoint_(points[i]));
				}
				boundary.addHole_(new Jun2dPolygon(parameterPoints));
				return null;
			}
		});

		Jun2dPolygon[] newParameterPolygons = boundary.asArrayOfPolygons();
		if (newParameterPolygons.length == 1) {
			Jun3dPolygon p = Jun3dPolygon.Origin_uVector_vVector_parameterPolygon_(polygon.origin(), polygon.uVector(), polygon.vVector(), newParameterPolygons[0]);
			return p.asJunOpenGL3dObject();
		} else {
			JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
			for (int i = 0; i < newParameterPolygons.length; i++) {
				Jun3dPolygon p = Jun3dPolygon.Origin_uVector_vVector_parameterPolygon_(polygon.origin(), polygon.uVector(), polygon.vVector(), newParameterPolygons[i]);
				compoundObject.add_(p.asJunOpenGL3dObject());
			}
			return compoundObject;
		}
	}

	/**
	 * Convert the receiver as a proxy in aBody.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @category converting
	 */
	public JunLoopProxy asProxyIn_(JunBody aJunBody) {
		return (JunLoopProxy) this.asProxyIn_advise_(aJunBody, null);
	}

	/**
	 * Convert the receiver as a proxy in aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aTopologicalElementProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @return jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy#asProxyIn_advise_(jp.co.sra.jun.topology.elements.JunBody, jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy)
	 * @category converting
	 */
	public JunTopologicalElementProxy asProxyIn_advise_(JunBody aBody, JunTopologicalElementProxy aTopologicalElementProxy) {
		JunLoopProxy proxy = (JunLoopProxy) aBody.proxyForLoop_(this);
		if (proxy != null) {
			return proxy;
		} else {
			return aBody.addLoop_as_(this, (JunLoopProxy) aTopologicalElementProxy);
		}
	}

	/**
	 * Convert the corresponding geometry as a wireframed JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#asWireframedOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asWireframedOpenGL3dObject() {
		final JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		this.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge e) {
				JunCurve _curve = e.curve();
				if (_curve instanceof Jun3dLine) {
					compoundObject.add_(((Jun3dLine) _curve).asJunOpenGL3dObject());
				} else if (_curve instanceof JunNurbsCurve) {
					compoundObject.add_(((JunNurbsCurve) _curve).asJunOpenGL3dObject());
				}
				return null;
			}
		});

		return compoundObject;
	}

	/**
	 * Answer the surface of the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public final JunSurface basicSurface() {
		return surface;
	}

	/**
	 * Answer the bounding box of the surface of the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dBoundingBox
	 * @category bounds accessing
	 */
	public Jun3dBoundingBox boundingBox() {
		return ((Jun3dPolygon) this.surface()).boundingBox();
	}

	/**
	 * Answer the child loop of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public final JunLoop childLoop() {
		return childLoop;
	}

	/**
	 * Set a child loop of the receiver.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public final void childLoop_(JunLoop aLoop) {
		childLoop = aLoop;
	}

	/**
	 * Answer the array of my child loops.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop[]
	 * @category accessing
	 */
	public JunLoop[] childLoops() {
		final ArrayList<JunLoop> loops = new ArrayList<JunLoop>();
		this.childLoopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				loops.add(loop);
				return null;
			}
		});

		return loops.toArray(new JunLoop[loops.size()]);
	}

	/**
	 * Enumerate the child loops and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R childLoopsDo_(StBlockClosure1<R, JunLoop> aBlock) {
		if (this.isChildLoop() == true) {
			// this.parentLoop().childLoopsDo_(aBlock);
			throw SmalltalkException.Error("should not be a child loop.");
		} else {
			JunLoop currentLoop = this.childLoop();
			while (currentLoop != null) {
				JunLoop nextLoop = currentLoop.childLoop();
				R result = aBlock.value(currentLoop);
				if (result != null) {
					return result;
				}
				currentLoop = nextLoop;
			}
		}

		return null;
	}

	/**
	 * Enumerate the child loops and evaluate the block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object childLoopsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.childLoopsDo_(new StBlockClosure1<Object, JunLoop>() {
			public Object value(JunLoop loop) {
				return aBlock.value_(loop);
			}
		});
	}

	/**
	 * Detect an edge which matches the condition aBlock. If none found,
	 * evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category enumerating
	 */
	public JunEdge detectEdge_ifNone_(final StBlockClosure1<Boolean, JunEdge> aBlock, StBlockClosure0<JunEdge> errorBlock) {
		JunEdge result = this.edgesDo_(new StBlockClosure1<JunEdge, JunEdge>() {
			public JunEdge value(JunEdge e) {
				if (aBlock.value(e)) {
					return e;
				}
				return null;
			}
		});

		if (result == null) {
			if (errorBlock != null) {
				result = errorBlock.value();
			}
		}
		return result;
	}

	/**
	 * Detect an edge which matches the condition aBlock. If none found,
	 * evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunEdge detectEdge_ifNone_(final StBlockClosure<?, ?, ?, ?> aBlock, final StBlockClosure<?, ?, ?, ?> errorBlock) {
		StBlockClosure0<JunEdge> myErrorBlock = null;
		if (errorBlock != null) {
			myErrorBlock = new StBlockClosure0<JunEdge>() {
				public JunEdge value() {
					return (JunEdge) errorBlock.value();
				}
			};
		}

		return this.detectEdge_ifNone_(new StBlockClosure1<Boolean, JunEdge>() {
			public Boolean value(JunEdge edge) {
				return (Boolean) aBlock.value_(edge);
			}
		}, myErrorBlock);
	}

	/**
	 * Detect a vertex which matches the condition aBlock. If none found,
	 * evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category enumerating
	 */
	public JunVertex detectVertex_ifNone_(final StBlockClosure1<Boolean, JunVertex> aBlock, StBlockClosure0<JunVertex> errorBlock) {
		JunVertex result = this.vertexEdgeVertexDo_(new StBlockClosure3<JunVertex, JunVertex, JunEdge, JunVertex>() {
			public JunVertex value(JunVertex v1, JunEdge e, JunVertex v2) {
				if (aBlock.value(v1)) {
					return v1;
				}
				return null;
			}
		});

		if (result == null) {
			if (errorBlock != null) {
				result = errorBlock.value();
			}
		}
		return result;
	}

	/**
	 * Detect a vertex which matches the condition aBlock. If none found,
	 * evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunVertex detectVertex_ifNone_(final StBlockClosure<?, ?, ?, ?> aBlock, final StBlockClosure<?, ?, ?, ?> errorBlock) {
		StBlockClosure0<JunVertex> myErrorBlock = null;
		if (errorBlock != null) {
			myErrorBlock = new StBlockClosure0<JunVertex>() {
				public JunVertex value() {
					return (JunVertex) errorBlock.value();
				}
			};
		}

		return this.detectVertex_ifNone_(new StBlockClosure1<Boolean, JunVertex>() {
			public Boolean value(JunVertex vertex) {
				return (Boolean) aBlock.value_(vertex);
			}
		}, myErrorBlock);
	}

	/**
	 * Answer the edge of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return edge;
	}

	/**
	 * Set an edge of the receiver.
	 * 
	 * @param anEdge jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public void edge_(JunEdge anEdge) {
		edge = anEdge;
	}

	/**
	 * Answer the edge specified with two vertexes.
	 * 
	 * @param aVertex1 jp.co.sra.jun.topology.elements.JunVertex
	 * @param aVertex2 jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edgeForVertex_and_(final JunVertex aVertex1, final JunVertex aVertex2) {
		return this.vertexEdgeVertexDo_(new StBlockClosure3<JunEdge, JunVertex, JunEdge, JunVertex>() {
			public JunEdge value(JunVertex v1, JunEdge e, JunVertex v2) {
				if ((v1 == aVertex1 && v2 == aVertex2) || (v1 == aVertex2 && v2 == aVertex1)) {
					return e;
				}
				return null;
			}
		});
	}

	/**
	 * Answer the edge which starts from the vertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edgeFromVertex_(final JunVertex aVertex) {
		return this.vertexEdgeVertexDo_(new StBlockClosure3<JunEdge, JunVertex, JunEdge, JunVertex>() {
			public JunEdge value(JunVertex v1, JunEdge e, JunVertex v2) {
				if (v1 == aVertex) {
					return e;
				}
				return null;
			}
		});
	}

	/**
	 * Answer the edges contained in the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge[]
	 * @category accessing
	 */
	public JunEdge[] edges() {
		final ArrayList<JunEdge> edges = new ArrayList<JunEdge>(this.numberOfEdges());
		this.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				edges.add(edge);
				return null;
			}
		});

		return edges.toArray(new JunEdge[edges.size()]);
	}

	/**
	 * Enumerate the edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R edgesDo_(final StBlockClosure1<R, JunEdge> aBlock) {
		return this.vertexEdgeVertexDo_(new StBlockClosure3<R, JunVertex, JunEdge, JunVertex>() {
			public R value(JunVertex v1, JunEdge e, JunVertex v2) {
				return aBlock.value(e);
			}
		});
	}

	/**
	 * Enumerate the edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object edgesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.edgesDo_(new StBlockClosure1<Object, JunEdge>() {
			public Object value(JunEdge edge) {
				return aBlock.value_(edge);
			}
		});
	}

	/**
	 * Enumerate the edges and evaluate aBlock while aConditionBlock returns true.
	 * 
	 * @param aConditionBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R edgesWhileTrue_do_(final StBlockClosure1<Boolean, JunEdge> aConditionBlock, final StBlockClosure1<R, JunEdge> aBlock) {
		return this.edgesDo_(new StBlockClosure1<R, JunEdge>() {
			public R value(JunEdge e) {
				if (aConditionBlock.value(e) == false) {
					return null;
				}
				return aBlock.value(e);
			}
		});
	}

	/**
	 * Enumerate the edges and evaluate aBlock while aConditionBlock returns true.
	 * 
	 * @param aConditionBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object edgesWhileTrue_do_(final StBlockClosure<?, ?, ?, ?> aConditionBlock, final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.edgesWhileTrue_do_(new StBlockClosure1<Boolean, JunEdge>() {
			public Boolean value(JunEdge edge) {
				return (Boolean) aConditionBlock.value_(edge);
			}
		}, new StBlockClosure1<Object, JunEdge>() {
			public Object value(JunEdge edge) {
				return aBlock.value_(edge);
			}
		});
	}

	/**
	 * Answer the edge which ends with the vertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunEdge edgeToVertex_(final JunVertex aVertex) {
		StBlockClosure3<JunEdge, JunVertex, JunEdge, JunVertex> aBlock = new StBlockClosure3<JunEdge, JunVertex, JunEdge, JunVertex>() {
			public JunEdge value(JunVertex v1, JunEdge e, JunVertex v2) {
				if (aVertex == v2) {
					return e;
				}
				return null;
			}
		};
		return this.edge().onLoop_vertexEdgeVertexDo_(this, aBlock);
	}

	/**
	 * Answer the first edge of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category private
	 */
	public JunEdge firstEdge() {
		return this.edge();
	}

	/**
	 * Answer the first vertex of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category private
	 */
	public JunVertex firstVertex() {
		JunEdge firstEdge = this.firstEdge();
		return firstEdge.oppositeVertexOfVertex_(firstEdge.directionVertexOnLoop_(this));
	}

	/**
	 * Forget every instance variables.
	 * 
	 * @category private
	 */
	public void forget() {
		if (this.isChildLoop() == true) {
			throw SmalltalkException.Error("should not be a child loop.");
		}

		this.detouchFromParentChildReference();
		edge = null;
		parentLoop = null;
		childLoop = null;
		surface = null;
	}

	/**
	 * Initialize the receiver with the lisp list.
	 * 
	 * @param aLispList jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @param loopArray jp.co.sra.jun.topology.elements.JunLoop[]
	 * @param edgeArray jp.co.sra.jun.topology.elements.JunEdge[]
	 * @param vertexArray jp.co.sra.jun.topology.elements.JunVertex[]
	 * @category lisp support
	 */
	public void fromLispList_forLoops_edges_vertexes_(JunLispCons aLispList, final JunLoop[] loopArray, final JunEdge[] edgeArray, JunVertex[] vertexArray) {
		((JunLispCons) aLispList.tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object anObject) {
				JunLispCons tuple = (JunLispCons) anObject;
				if (tuple.head() == $("edge")) {
					edge = edgeArray[((Integer) tuple.tail()).intValue() - 1];
				}
				if (tuple.head() == $("parentLoop")) {
					if (tuple.tail() == JunLispList.NullList()) {
						parentLoop = null;
					} else {
						parentLoop = loopArray[((Integer) tuple.tail()).intValue() - 1];
					}
				}
				if (tuple.head() == $("childLoop")) {
					if (tuple.tail() == JunLispList.NullList()) {
						childLoop = null;
					} else {
						childLoop = loopArray[((Integer) tuple.tail()).intValue() - 1];
					}
				}
				if (tuple.head() == $("surface")) {
					if (tuple.tail() == JunLispList.NullList()) {
						surface = null;
					} else {
						surface = JunLoop.this.surfaceFromList_((JunLispList) tuple.tail());
					}
				}
				return null;
			}
		});
	}

	/**
	 * Answer true if I have a child loop.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasChildLoop() {
		return this.isParentLoop() && (childLoop != null);
	}

	/**
	 * Answer true if the receiver contains the edge, otherwise false.
	 * 
	 * @param anEdge jp.co.sra.jun.topology.elements.JunEdge
	 * @return boolean
	 * @category testing
	 */
	public boolean includesEdge_(final JunEdge anEdge) {
		Boolean result = this.edgesDo_(new StBlockClosure1<Boolean, JunEdge>() {
			public Boolean value(JunEdge e) {
				if (e == anEdge) {
					return Boolean.TRUE;
				}
				return null;
			}
		});

		return (result != null);
	}

	/**
	 * Answer true if the receiver contains a path from aVertex1 to aVertex2, otherwise false.
	 * 
	 * @param aVertex1 jp.co.sra.jun.topology.elements.JunVertex
	 * @param aVertex2 jp.co.sra.jun.topology.elements.JunVertex
	 * @return boolean
	 * @category testing
	 */
	public boolean includesPathFrom_to_(final JunVertex aVertex1, final JunVertex aVertex2) {
		Object result = this.vertexEdgeVertexDo_(new StBlockClosure3<Boolean, JunVertex, JunEdge, JunVertex>() {
			public Boolean value(JunVertex v1, JunEdge e, JunVertex v2) {
				if (v1 == aVertex1 && v2 == aVertex2) {
					return Boolean.TRUE;
				}
				return null;
			}
		});

		return (result != null);
	}

	/**
	 * Answer true if the receiver contains aVertex, otherwise false.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return boolean
	 * @category testing
	 */
	public boolean includesVertex_(final JunVertex aVertex) {
		Object result = this.vertexesDo_(new StBlockClosure1<Boolean, JunVertex>() {
			public Boolean value(JunVertex v) {
				if (v == aVertex) {
					return Boolean.TRUE;
				}
				return null;
			}
		});

		return (result != null);
	}

	/**
	 * Answer true if the receiver is a child loop, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isChildLoop() {
		if (this.isParentLoop() == true) {
			return false;
		}

		return true;
	}

	/**
	 * Answer true if the receiver is killed, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isKilled() {
		return (edge == null);
	}

	/**
	 * Answer true if the receiver is a parent loop, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isParentLoop() {
		if (this.parentLoop() == null) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Enumerate the neighbor loops and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public void neighborLoopsDo_(final StBlockClosure1<Void, JunLoop> aBlock) {
		this.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge myEdge) {
				JunLoop loop = myEdge.oppositeLoopOfLoop_(JunLoop.this);
				if (JunLoop.this != loop) {
					aBlock.value(loop);
				}
				return null;
			}
		});
	}

	/**
	 * Enumerate the neighbor loops and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public void neighborLoopsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.neighborLoopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				aBlock.value_(loop);
				return null;
			}
		});
	}

	/**
	 * Answer the neighbor loop specified with the vertex.
	 * 
	 * @param aJunVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunLoop neighborLoopWithVertex_(JunVertex aJunVertex) {
		return aJunVertex.loopsDo_(new StBlockClosure1<JunLoop, JunLoop>() {
			public JunLoop value(JunLoop hisLoop) {
				if (hisLoop != JunLoop.this) {
					return hisLoop;
				}
				return null;
			}
		});
	}

	/**
	 * Answer the vertex which is next to aVertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex nextVertexOfVertex_(final JunVertex aVertex) {
		return this.edge().onLoop_vertexEdgeVertexDo_(this, new StBlockClosure3<JunVertex, JunVertex, JunEdge, JunVertex>() {
			public JunVertex value(JunVertex v1, JunEdge e, JunVertex v2) {
				if (aVertex == v1) {
					return v2;
				}
				return null;
			}
		});
	}

	/**
	 * Answer the number of my child loops.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfChildLoops() {
		final StValueHolder<Integer> num = new StValueHolder<Integer>(0);
		if (this.isChildLoop() == false) {
			this.childLoopsDo_(new StBlockClosure1<Void, JunLoop>() {
				public Void value(JunLoop loop) {
					num.value_(num.value() + 1);
					return null;
				}
			});
		}
		return num.value();
	}

	/**
	 * Answer the number of edges contained in the receiver.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfEdges() {
		final StValueHolder<Integer> numberOfEdges = new StValueHolder<Integer>(0);
		this.vertexEdgeVertexDo_(new StBlockClosure3<Void, JunVertex, JunEdge, JunVertex>() {
			public Void value(JunVertex v1, JunEdge e, JunVertex v2) {
				numberOfEdges.value_(numberOfEdges.value() + 1);
				return null;
			}
		});
		return numberOfEdges.value();
	}

	/**
	 * Answer the number of vertexes contained in the receiver.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfVertexes() {
		final StValueHolder<Integer> numberOfVertexes = new StValueHolder<Integer>(0);
		this.vertexEdgeVertexDo_(new StBlockClosure3<Void, JunVertex, JunEdge, JunVertex>() {
			public Void value(JunVertex v1, JunEdge e, JunVertex v2) {
				numberOfVertexes.value_(numberOfVertexes.value() + 1);
				return null;
			}
		});
		return numberOfVertexes.value();
	}

	/**
	 * Answer the parent loop of the receiver
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public final JunLoop parentLoop() {
		return parentLoop;
	}

	/**
	 * Set a loop as the receiver's parent loop.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @category private
	 */
	public final void parentLoop_(JunLoop aLoop) {
		parentLoop = aLoop;
	}

	/**
	 * Answer the array of Jun3dPoint which corresponds to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @category accessing
	 */
	public Jun3dPoint[] points() {
		final ArrayList<Jun3dPoint> points = new ArrayList<Jun3dPoint>();
		this.pointsDo_(new StBlockClosure1<Void, Jun3dPoint>() {
			public Void value(Jun3dPoint p) {
				points.add(p);
				return null;
			}
		});

		return points.toArray(new Jun3dPoint[points.size()]);
	}

	/**
	 * Enumerate the points and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public <R> R pointsDo_(final StBlockClosure1<R, Jun3dPoint> aBlock) {
		StBlockClosure3<R, JunVertex, JunEdge, JunVertex> b = new StBlockClosure3<R, JunVertex, JunEdge, JunVertex>() {
			public R value(JunVertex v1, JunEdge e, JunVertex v2) {
				return aBlock.value(v1.point());
			}
		};
		return this.vertexEdgeVertexDo_(b);
	}

	/**
	 * Enumerate the points and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public void pointsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.pointsDo_(new StBlockClosure1<Void, Jun3dPoint>() {
			public Void value(Jun3dPoint p) {
				aBlock.value_(p);
				return null;
			}
		});
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		final BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
		try {
			bw.write("Loop (");
			if (this.edge() != null) {
				this.vertexEdgeVertexDo_(new StBlockClosure3<Void, JunVertex, JunEdge, JunVertex>() {
					public Void value(JunVertex v1, JunEdge e, JunVertex v2) {
						try {
							bw.newLine();
							v1.printOn_(bw);
							bw.write("->");
							v2.printOn_(bw);
						} catch (IOException ex) {
						}
						return null;
					}
				});
			} else {
				bw.write("removed");
			}
			bw.write(')');
		} finally {
			bw.flush();
		}
	}

	/**
	 * Remove the loop from the child loops of the receiver.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @category removing
	 */
	public void removeChildLoop_(final JunLoop aLoop) {
		if (this.isChildLoop() == true) {
			throw SmalltalkException.Error("should not be a child loop.");
		}
		aLoop.parentLoop_(null);
		if (this.childLoop() == aLoop) {
			this.childLoop_(aLoop.childLoop());
			aLoop.childLoop_(null);
			return;
		}
		JunLoop result = this.childLoopsDo_(new StBlockClosure1<JunLoop, JunLoop>() {
			public JunLoop value(JunLoop loop) {
				if (loop.childLoop() == aLoop) {
					loop.childLoop_(aLoop.childLoop());
					aLoop.childLoop_(null);
					return aLoop;
				}
				return null;
			}
		});
		if (result == null) {
			throw SmalltalkException.Error("internal error");
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aRenderingContext DOCUMENT ME!
	 * @category displaying
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		((JunNurbsSurface) this.surface()).renderOn_(aRenderingContext);
	}

	/**
	 * Repatch the receiver.
	 * 
	 * @category accessing
	 */
	public void repatch() {
		if (this.basicSurface() == null) {
			return;
		}
		this.surface_(null);
	}

	/**
	 * Set a surface of the receiver.
	 * 
	 * @param aSurface jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public final void setSurface_(JunSurface aSurface) {
		surface = aSurface;
	}

	/**
	 * Answer the surface of the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public JunSurface surface() {
		JunSurface s = this.basicSurface();
		if (s != null) {
			return s;
		}
		return this.defaultSurface();
	}

	/**
	 * Set a surface of the receiver.
	 * 
	 * @param aSurface jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public final void surface_(JunSurface aSurface) {
		surface = aSurface;
	}

	/**
	 * Answer the surface created from the lisp list.
	 * 
	 * @param aList jp.co.sra.jun.goodies.lisp.JunLispList
	 * @return jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category lisp support
	 */
	public JunSurface surfaceFromList_(JunLispList aList) {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Enumerate the surrounding edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category enumerating
	 */
	public void surroundingEdgesDo_(final StBlockClosure1<Void, JunEdge> aBlock) {
		this.edgesDo_(aBlock);
		this.childLoopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop child) {
				child.edgesDo_(aBlock);
				return null;
			}
		});
	}

	/**
	 * Enumerate the surrounding edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public void surroundingEdgesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.surroundingEdgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				aBlock.value_(edge);
				return null;
			}
		});
	}

	/**
	 * Answer the lisp list which represents the receiver.
	 * 
	 * @param loopArray jp.co.sra.jun.topology.elements.JunLoop[]
	 * @param edgeArray jp.co.sra.jun.topology.elements.JunEdge[]
	 * @param vertexArray jp.co.sra.jun.topology.elements.JunVertex[]
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#toLispListForLoops_edges_vertexes_(jp.co.sra.jun.topology.elements.JunLoop[], jp.co.sra.jun.topology.elements.JunEdge[], jp.co.sra.jun.topology.elements.JunVertex[])
	 * @category lisp support
	 */
	public JunLispList toLispListForLoops_edges_vertexes_(final JunLoop[] loopArray, final JunEdge[] edgeArray, final JunVertex[] vertexArray) {
		int indexOfEdge;
		int indexOfParentLoop = -1;
		int indexOfChildLoop = -1;
		for (indexOfEdge = 0; indexOfEdge < edgeArray.length; indexOfEdge++) {
			if (edgeArray[indexOfEdge] == edge) {
				break;
			}
		}
		for (int i = 0; i < loopArray.length; i++) {
			if (loopArray[i] == parentLoop) {
				indexOfParentLoop = i;
			}
			if (loopArray[i] == childLoop) {
				indexOfChildLoop = i;
			}
		}
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());
		list.add_(JunLispCons.Head_tail_($("edge"), new Integer(indexOfEdge + 1)));
		list.add_(JunLispCons.Head_tail_($("parentLoop"), (parentLoop == null) ? (Object) this.lispNil() : (Object) new Integer(indexOfParentLoop + 1)));
		list.add_(JunLispCons.Head_tail_($("childLoop"), (childLoop == null) ? (Object) this.lispNil() : (Object) new Integer(indexOfChildLoop + 1)));
		list.add_(JunLispCons.Head_tail_($("surface"), this.lispNil()));
		return list;
	}

	/**
	 * Answer the representative vertex of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex() {
		return this.edge().startVertex();
	}

	/**
	 * Enumerate the Vertex-Edge-Vertex trios and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R vertexEdgeVertexDo_(StBlockClosure3<R, JunVertex, JunEdge, JunVertex> aBlock) {
		JunEdge firstEdge = this.firstEdge();
		JunEdge focusedEdge = firstEdge;
		JunVertex firstVertex = this.firstVertex();
		JunVertex focusedVertex = firstVertex;
		while (focusedEdge != null) {
			JunVertex oppositeVertex = focusedEdge.oppositeVertexOfVertex_(focusedVertex);
			R result = aBlock.value(focusedVertex, focusedEdge, oppositeVertex);
			if (result != null) {
				return result;
			}
			focusedVertex = oppositeVertex;
			focusedEdge = focusedEdge.nextEdgeWithVertex_(focusedVertex);
			if (firstEdge == focusedEdge && focusedVertex == firstVertex) {
				return null;
			}
		}

		return null;
	}

	/**
	 * Enumerate the Vertex-Edge-Vertex trios and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object vertexEdgeVertexDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		StBlockClosure3<Object, JunVertex, JunEdge, JunVertex> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure3<Object, JunVertex, JunEdge, JunVertex>() {
				public Object value(JunVertex vertex1, JunEdge edge, JunVertex vertex2) {
					return aBlock.value_value_value_(vertex1, edge, vertex2);
				}
			};
		}

		return this.vertexEdgeVertexDo_(myBlock);
	}

	/**
	 * Answer the vertexes contained in the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex[]
	 * @category accessing
	 */
	public JunVertex[] vertexes() {
		final ArrayList<JunVertex> vertexes = new ArrayList<JunVertex>(this.numberOfVertexes());
		this.vertexesDo_(new StBlockClosure1<Void, JunVertex>() {
			public Void value(JunVertex v) {
				vertexes.add(v);
				return null;
			}
		});

		return vertexes.toArray(new JunVertex[vertexes.size()]);
	}

	/**
	 * Enumerate the vertexes and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R vertexesDo_(final StBlockClosure1<R, JunVertex> aBlock) {
		return this.vertexEdgeVertexDo_(new StBlockClosure3<R, JunVertex, JunEdge, JunVertex>() {
			public R value(JunVertex v1, JunEdge e, JunVertex v2) {
				return aBlock.value(v1);
			}
		});
	}

	/**
	 * Enumerate the vertexes and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object vertexesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return this.vertexesDo_(new StBlockClosure1<Object, JunVertex>() {
			public Object value(JunVertex vertex) {
				return aBlock.value_(vertex);
			}
		});
	}

	/**
	 * Answer the default surface of JunLoop.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category private
	 */
	protected JunSurface defaultSurface() {
		JunVertex[] vertexes = this.vertexes();
		Jun3dPoint[] points = new Jun3dPoint[vertexes.length];
		for (int index = 0; index < vertexes.length; index++) {
			points[index] = vertexes[index].point();
		}
		return Jun3dPolygon.Vertexes_(points);
	}

	/**
	 * Detach the receiver from the parent loop.
	 * 
	 * @category private
	 */
	private void detouchFromParentChildReference() {
		if (this.isChildLoop()) {
			this.parentLoop().removeChildLoop_(this);
		}
	}

	/**
	 * Answer my volume.
	 * 
	 * @return double
	 * @category private
	 */
	protected double volume() {
		JunVertex[] vertexes = this.vertexes();
		Jun3dPoint[] points = new Jun3dPoint[vertexes.length];
		for (int i = 0; i < vertexes.length; i++) {
			points[i] = vertexes[i].point();
		}
		double area = Jun2dPolygon.SignedAreaForPoints_(points);
		double height = 0;
		for (int i = 0; i < points.length; i++) {
			height += points[i].z();
		}
		return (area * height) / points.length;
	}

}
