package jp.co.sra.jun.topology.elements;

import java.awt.Color;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StValueHolder;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.goodies.lisp.JunLispCons;
import jp.co.sra.jun.goodies.lisp.JunLispList;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dVertex;
import jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElement;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy;

/**
 * JunVertex class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/22 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVertex.java,v 9.9 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunVertex extends JunTopologicalElement {

	/** The edge in which the JunVertex is contained. */
	protected JunEdge edge;

	/** The Jun3dPoint which corresponds to the JunVertex. */
	protected Jun3dPoint point;

	/**
	 * Create a new instance of <code>JunVertex</code> and initialize it with the specified point.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category Instance creation
	 */
	public static final JunVertex Point_(Jun3dPoint aPoint) {
		JunVertex aVertex = new JunVertex();
		aVertex.point_(aPoint);
		return aVertex;
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		edge = null;
		point = null;
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#asJunOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		return JunOpenGL3dVertex.Point_(this.point());
	}

	/**
	 * Convert the receiver as a proxy in aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @category converting
	 */
	public JunTopologicalElementProxy asProxyIn_(JunBody aBody) {
		return this.asProxyIn_advise_(aBody, null);
	}

	/**
	 * Convert the receiver as a proxy in aBody.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aTopologicalElementProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @return jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy#asProxyIn_advise_(jp.co.sra.jun.topology.elements.JunBody, jp.co.sra.jun.topology.abstracts.JunTopologicalElementProxy)
	 * @category converting
	 */
	public JunTopologicalElementProxy asProxyIn_advise_(JunBody aBody, JunTopologicalElementProxy aTopologicalElementProxy) {
		JunVertexProxy proxy;
		proxy = (JunVertexProxy) aBody.proxyForVertex_(this);

		if (proxy != null) {
			return proxy;
		} else {
			return aBody.addVertex_as_(this, (JunVertexProxy) aTopologicalElementProxy);
		}
	}

	/**
	 * Convert the corresponding geometry as a wireframed JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#asWireframedOpenGL3dObject()
	 * @category converting
	 */
	public JunOpenGL3dObject asWireframedOpenGL3dObject() {
		return JunOpenGL3dVertex.Point_(this.point());
	}

	/**
	 * Answer the common loop of the receiver specified with the vertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop commonLoopWithVertex_(final JunVertex aVertex) {
		return this.loopsDo_(new StBlockClosure1<JunLoop, JunLoop>() {
			public JunLoop value(final JunLoop myLoop) {
				return aVertex.loopsDo_(new StBlockClosure1<JunLoop, JunLoop>() {
					public JunLoop value(JunLoop hisLoop) {
						if (myLoop == hisLoop) {
							return myLoop;
						}
						return null;
					}
				});
			}
		});
	}

	/**
	 * Detect an edge which meets the condition specified with aBlock. If not
	 * found, evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category enumerating
	 */
	public JunEdge detectEdge_ifNone_(final StBlockClosure1<Boolean, JunEdge> aBlock, StBlockClosure0<JunEdge> errorBlock) {
		JunEdge result = this.edgesDo_(new StBlockClosure1<JunEdge, JunEdge>() {
			public JunEdge value(JunEdge myEdge) {
				if (aBlock.value(myEdge)) {
					return myEdge;
				}
				return null;
			}
		});

		if (result == null) {
			if (errorBlock != null) {
				result = errorBlock.value();
			}
		}

		return result;
	}

	/**
	 * Detect an edge which meets the condition specified with aBlock. If not
	 * found, evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunEdge detectEdge_ifNone_(final StBlockClosure<?, ?, ?, ?> aBlock, final StBlockClosure<?, ?, ?, ?> errorBlock) {
		StBlockClosure1<Boolean, JunLoop> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure1<Boolean, JunLoop>() {
				public Boolean value(JunLoop aLoop) {
					return (Boolean) aBlock.value_(aLoop);
				}
			};
		}

		StBlockClosure0<JunLoop> myErrorBlock = null;
		if (errorBlock != null) {
			myErrorBlock = new StBlockClosure0<JunLoop>() {
				public JunLoop value() {
					return (JunLoop) errorBlock.value();
				}
			};
		}

		return this.detectEdge_ifNone_(myBlock, myErrorBlock);
	}

	/**
	 * Detect a loop which meets the condition specified with aBlock. If not
	 * found, evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category enumerating
	 */
	public JunLoop detectLoop_ifNone_(final StBlockClosure1<Boolean, JunLoop> aBlock, StBlockClosure0<JunLoop> errorBlock) {
		JunLoop result = this.loopsDo_(new StBlockClosure1<JunLoop, JunLoop>() {
			public JunLoop value(JunLoop myLoop) {
				if (aBlock.value(myLoop)) {
					return myLoop;
				}
				return null;
			}
		});

		if (result == null) {
			if (errorBlock != null) {
				result = errorBlock.value();
			}
		}
		return result;
	}

	/**
	 * Detect a loop which meets the condition specified with aBlock. If not
	 * found, evaluate errorBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public JunLoop detectLoop_ifNone_(final StBlockClosure<?, ?, ?, ?> aBlock, final StBlockClosure<?, ?, ?, ?> errorBlock) {
		StBlockClosure1<Boolean, JunLoop> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure1<Boolean, JunLoop>() {
				public Boolean value(JunLoop aLoop) {
					return (Boolean) aBlock.value_(aLoop);
				}
			};
		}

		StBlockClosure0<JunLoop> myErrorBlock = null;
		if (errorBlock != null) {
			myErrorBlock = new StBlockClosure0<JunLoop>() {
				public JunLoop value() {
					return (JunLoop) errorBlock.value();
				}
			};
		}

		return this.detectLoop_ifNone_(myBlock, myErrorBlock);
	}

	/**
	 * Answer the edge of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return edge;
	}

	/**
	 * Set an edge of the receiver.
	 * 
	 * @param anEdge jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public void edge_(JunEdge anEdge) {
		edge = anEdge;
	}

	/**
	 * Enumerate the edge and loop pairs and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R edgeAndLoopsDo_(StBlockClosure2<R, JunEdge, JunLoop> aBlock) {
		JunEdge currentEdge = this.edge();
		JunLoop currentLoop = currentEdge.loopToVertex_(this);
		R result = aBlock.value(currentEdge, currentLoop);
		if (result != null) {
			return result;
		}

		JunEdge nextEdge = currentEdge.nextEdgeWithVertex_(this);
		while (nextEdge != this.edge()) {
			currentEdge = nextEdge;
			currentLoop = currentEdge.loopToVertex_(this);

			result = aBlock.value(currentEdge, currentLoop);
			if (result != null) {
				return result;
			}

			nextEdge = currentEdge.nextEdgeWithVertex_(this);
		}

		return null;
	}

	/**
	 * Enumerate the edge and loop pairs and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object edgeAndLoopsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		StBlockClosure2<Object, JunEdge, JunLoop> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure2<Object, JunEdge, JunLoop>() {
				public Object value(JunEdge edge, JunLoop loop) {
					return aBlock.value_value_(edge, loop);
				}
			};
		}

		return this.edgeAndLoopsDo_(myBlock);
	}

	/**
	 * Answer the edge on the loop.  If none found, evaluate the error block.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edgeOnLoop_ifAbsent_(final JunLoop aLoop, StBlockClosure0<JunEdge> errorBlock) {
		StBlockClosure1<Boolean, JunEdge> b = new StBlockClosure1<Boolean, JunEdge>() {
			public Boolean value(JunEdge myEdge) {
				return aLoop.includesEdge_(myEdge);
			}
		};

		return this.detectEdge_ifNone_(b, errorBlock);
	}

	/**
	 * Answer the edge on the loop.  If none found, evaluate the error block.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @param errorBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public JunEdge edgeOnLoop_ifAbsent_(JunLoop aLoop, final StBlockClosure<?, ?, ?, ?> errorBlock) {
		StBlockClosure0<JunEdge> myErrorBlock = null;
		if (errorBlock != null) {
			myErrorBlock = new StBlockClosure0<JunEdge>() {
				public JunEdge value() {
					return (JunEdge) errorBlock.value();
				}
			};
		}

		return this.edgeOnLoop_ifAbsent_(aLoop, myErrorBlock);
	}

	/**
	 * Answer the edges.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge[]
	 * @category accessing
	 */
	public JunEdge[] edges() {
		final ArrayList<JunEdge> edges = new ArrayList<JunEdge>(this.numberOfEdges());
		this.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge anEdge) {
				edges.add(anEdge);
				return null;
			}
		});

		return edges.toArray(new JunEdge[edges.size()]);
	}

	/**
	 * Enumerate the edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object1
	 * @category enumerating
	 */
	public <R> R edgesDo_(final StBlockClosure1<R, JunEdge> aBlock) {
		StBlockClosure2<R, JunEdge, JunLoop> b = new StBlockClosure2<R, JunEdge, JunLoop>() {
			public R value(JunEdge e, JunLoop l) {
				return aBlock.value(e);
			}
		};

		return this.edgeAndLoopsDo_(b);
	}

	/**
	 * Enumerate the edges and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object edgesDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		StBlockClosure1<Object, JunEdge> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure1<Object, JunEdge>() {
				public Object value(JunEdge edge) {
					return aBlock.value_(edge);
				}
			};
		}

		return this.edgesDo_(myBlock);
	}

	/**
	 * Answer the edge from the receiver to aVertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edgeToVertex_(final JunVertex aVertex) {
		if (this.isKilled()) {
			return null;
		}

		StBlockClosure1<JunEdge, JunEdge> aBlock = new StBlockClosure1<JunEdge, JunEdge>() {
			public JunEdge value(JunEdge myEdge) {
				if (myEdge.oppositeVertexOfVertex_(JunVertex.this) == aVertex) {
					return myEdge;
				}
				return null;
			}
		};

		return this.edgesDo_(aBlock);
	}

	/**
	 * Forget every instance variables.
	 * 
	 * @category private
	 */
	public void forget() {
		edge = null;
		point = null;
	}

	/**
	 * Initialize the receiver with the lisp list.
	 * 
	 * @param aLispList jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @param loopArray jp.co.sra.jun.topology.elements.JunLoop[]
	 * @param edgeArray jp.co.sra.jun.topology.elements.JunEdge[]
	 * @param vertexArray jp.co.sra.jun.topology.elements.JunVertex[]
	 * @category lisp support
	 */
	public void fromLispList_forLoops_edges_vertexes_(JunLispCons aLispList, JunLoop[] loopArray, final JunEdge[] edgeArray, JunVertex[] vertexArray) {
		((JunLispCons) aLispList.tail()).do_(new StBlockClosure1<Void, Object>() {
			public Void value(Object anObject) {
				JunLispCons tuple = (JunLispCons) anObject;
				if (tuple.head() == $("edge")) {
					edge = edgeArray[((Integer) tuple.tail()).intValue() - 1];
				}
				if (tuple.head() == $("point")) {
					point = (Jun3dPoint) tuple.tail();
				}
				return null;
			}
		});
	}

	/**
	 * Answer true if there exists an edge from the receiver to aVertex,
	 * otherwise false.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return boolean
	 * @category testing
	 */
	public boolean hasEdgeToVertex_(JunVertex aVertex) {
		return (this.edgeToVertex_(aVertex) != null);
	}

	/**
	 * Answer true if there exists a loop which contains the receiver,
	 * otherwise false.
	 * 
	 * @param aLoop jp.co.sra.jun.topology.elements.JunLoop
	 * @return boolean
	 * @category testing
	 */
	public boolean hasLoop_(final JunLoop aLoop) {
		if (this.isKilled()) {
			return false;
		}

		Boolean result = this.loopsDo_(new StBlockClosure1<Boolean, JunLoop>() {
			public Boolean value(JunLoop myLoop) {
				if (myLoop == aLoop) {
					return Boolean.TRUE;
				}
				return null;
			}
		});

		return (result != null);
	}

	/**
	 * Answer true if there exists the edge which contains the receiver.
	 * 
	 * @param anEdge jp.co.sra.jun.topology.elements.JunEdge
	 * @return boolean
	 * @category testing
	 */
	public boolean includesEdge_(final JunEdge anEdge) {
		Boolean result = this.edgesDo_(new StBlockClosure1<Boolean, JunEdge>() {
			public Boolean value(JunEdge myEdge) {
				if (myEdge == anEdge) {
					return Boolean.TRUE;
				}
				return null;
			}
		});

		return (result != null);
	}

	/**
	 * Answer true if the receiver is killed, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isKilled() {
		return (edge == null);
	}

	/**
	 * Answer the loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return this.edge().loop();
	}

	/**
	 * Answer the loops related to the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop[]
	 * @category accessing
	 */
	public JunLoop[] loops() {
		final ArrayList<JunLoop> loops = new ArrayList<JunLoop>(this.numberOfLoops());
		this.loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop aLoop) {
				loops.add(aLoop);
				return null;
			}
		});

		return loops.toArray(new JunLoop[loops.size()]);
	}

	/**
	 * Enumerate the loops and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public <R> R loopsDo_(final StBlockClosure1<R, JunLoop> aBlock) {
		return this.edgeAndLoopsDo_(new StBlockClosure2<R, JunEdge, JunLoop>() {
			public R value(JunEdge e, JunLoop l) {
				return aBlock.value(l);
			}
		});
	}

	/**
	 * Enumerate the loops and evaluate aBlock.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 * @deprecated since Jun701, use Generics.
	 */
	public Object loopsDo_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		StBlockClosure1<Object, JunLoop> myBlock = null;
		if (aBlock != null) {
			myBlock = new StBlockClosure1<Object, JunLoop>() {
				public Object value(JunLoop aLoop) {
					return aBlock.value_(aLoop);
				}
			};
		}

		return this.loopsDo_(myBlock);
	}

	/**
	 * Answer the loop from me to the vertex.
	 * 
	 * @param aVertex jp.co.sra.jun.topology.elements.JunVertex
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loopToVertex_(JunVertex aVertex) {
		JunEdge theEdge = this.edgeToVertex_(aVertex);

		if (theEdge == null) {
			return null;
		}

		return theEdge.loopToVertex_(aVertex);
	}

	/**
	 * Answer the number of the edges related to the receiver.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfEdges() {
		final StValueHolder<Integer> numberOfEdges = new StValueHolder<Integer>(0);
		this.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge anEdge) {
				numberOfEdges.value_(numberOfEdges.value() + 1);

				return null;
			}
		});

		return numberOfEdges.value();
	}

	/**
	 * Answer the number of the loops related to the receiver.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numberOfLoops() {
		final StValueHolder<Integer> numberOfLoops = new StValueHolder<Integer>(0);
		this.loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop aLoop) {
				numberOfLoops.value_(numberOfLoops.value() + 1);
				return null;
			}
		});

		return numberOfLoops.value();
	}

	/**
	 * Answer the Jun3dPoint which corresponds to the receiver.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point() {
		return point;
	}

	/**
	 * Set the Jun3dPoint as a corresponding geometry.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void point_(Jun3dPoint aPoint) {
		point = aPoint;
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException DOCUMENT ME!
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("Vertex (");
		this.point.printOn_(aWriter);
		aWriter.write(")");
	}

	/**
	 * Render the point of the receiver on the rendering context.
	 * 
	 * @param aRenderingContext jp.co.sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @category displaying
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		JunOpenGL3dVertex vertex;
		vertex = JunOpenGL3dVertex.Point_paint_(this.point(), Color.black);
		vertex.size_(5);
		vertex.renderOn_(aRenderingContext);
	}

	/**
	 * Answer the lisp list which represents the receiver.
	 * 
	 * @param loopArray jp.co.sra.jun.topology.elements.JunLoop[]
	 * @param edgeArray jp.co.sra.jun.topology.elements.JunEdge[]
	 * @param vertexArray jp.co.sra.jun.topology.elements.JunVertex[]
	 * @return jp.co.sra.jun.goodies.lisp.JunLispCons
	 * @see jp.co.sra.jun.topology.abstracts.JunTopologicalElement#toLispListForLoops_edges_vertexes_(jp.co.sra.jun.topology.elements.JunLoop[], jp.co.sra.jun.topology.elements.JunEdge[], jp.co.sra.jun.topology.elements.JunVertex[])
	 * @category lisp support
	 */
	public JunLispList toLispListForLoops_edges_vertexes_(JunLoop[] loopArray, JunEdge[] edgeArray, JunVertex[] vertexArray) {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());
		int index = -1;
		for (int i = 0; i < edgeArray.length; i++) {
			if (edgeArray[i] == edge) {
				index = i;
				break;
			}
		}

		list.add_(JunLispCons.Head_tail_($("edge"), new Integer(index + 1)));
		list.add_(JunLispCons.Head_tail_($("point"), point));
		return list;
	}

}
