package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;

/**
 * JunKCLMPL class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKCLMPL.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunKCLMPL extends JunEulerOperator {
	protected JunLoopProxy loopProxy1;
	protected JunLoopProxy loopProxy2;

	/**
	 * Create a new instance of <code>JunKCLMPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKCLMPL
	 * @deprecated since Jun700, using constructor
	 * @category Instance creation
	 */
	public static final JunKCLMPL Body_loop_(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy) {
		return new JunKCLMPL(aJunBody, aJunLoopOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKCLMPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param adviseLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKCLMPL
	 * @deprecated since Jun700, using constructor
	 * @category Instance creation
	 */
	public static final JunKCLMPL Body_loop_adviseLoop_(JunBody aJunBody, JunTopologicalElementOrProxy aLoopOrProxy, JunTopologicalElementOrProxy adviseLoopOrProxy) {
		return new JunKCLMPL(aJunBody, aLoopOrProxy, adviseLoopOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKCLMPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKCLMPL(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy) {
		super();
		this.body_(aJunBody);
		this.loop2_(aJunLoopOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKCLMPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param adviseLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKCLMPL(JunBody aJunBody, JunTopologicalElementOrProxy aLoopOrProxy, JunTopologicalElementOrProxy adviseLoopOrProxy) {
		super();
		this.body_(aJunBody);
		this.loop1_(adviseLoopOrProxy);
		this.loop2_(aLoopOrProxy);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		loopProxy1 = null;
		loopProxy2 = null;
	}

	/**
	 * Answer the receiver's changed loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop changedLoop() {
		return this.loop2();
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.loop1_(this.loop2().parentLoop());
		this.loop1().removeChildLoop_(this.loop2());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMCLKPL(this.body(), loopProxy1, loopProxy2);
	}

	/**
	 * Answer the receiver's loop1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop1() {
		return (JunLoop) this.getTopologicalElement_(this.loopProxy1);
	}

	/**
	 * Set the receiver's loop1.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop1_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy1 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy1);
	}

	/**
	 * Answer the receiver's loop2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop2() {
		return (JunLoop) this.getTopologicalElement_(this.loopProxy2);
	}

	/**
	 * Set the receiver's loop1.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop2_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy2 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy2);
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunLoop loop2 = this.loop2();
		if (loop2 == null) {
			return false;
		}
		return super.precondition() && loop2.isChildLoop();
	}
}
