package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.geometricoperators.JunTV;

/**
 * JunKDEVTestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKDEVTestExamples.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunKDEVTestExamples extends JunTestExamples {
	/**
	 * Example.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunVertex v000 = null;
		JunVertex v100 = null;
		JunVertex v010 = null;
		Jun3dPoint[] a3dPoint = new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0), new Jun3dPoint(0, 0, 1) };
		JunBody body = JunBody.Tetrahedron_(a3dPoint);

		JunVertex[] temp = body.vertexes();
		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(0, 0, 0))) {
				v000 = temp[i];
				break;
			}
		}

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(1, 0, 0))) {
				v100 = temp[i];
				break;
			}
		}

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(0, 1, 0))) {
				v010 = temp[i];
				break;
			}
		}

		JunEdge edge1 = body.edgeForVertex_and_(v000, v100);
		JunEdge edge2 = body.edgeForVertex_and_(v000, v010);
		JunMDEV mdev = JunMDEV.Body_vertex_edge_edge_(body, v000, edge1, edge2);
		mdev.doOperation();
		JunKDEV kdev = new JunKDEV(body, mdev.newEdge());
		kdev.doOperation();
		body.show();

		return true;
	}

	/**
	 * The example on JunKDEV.htm
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean ManualExample1() {
		JunBody body = new JunBody();

		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();

		JunLoop loop = mevvl.newLoop();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, loop, new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunVertex vertex000 = mev.newVertex();
		JunEdge edge1 = mev.newEdge();

		JunMEL mel = new JunMEL(body, loop, vertex010, vertex000);
		mel.doOperation();

		JunEdge edge2 = mel.newEdge();

		JunMDEV mdev = JunMDEV.Body_vertex_edge_edge_(body, vertex000, edge1, edge2);
		mdev.doOperation();
		vertex000 = mdev.newVertex();

		JunEdge edge = mdev.newEdge();

		JunTV tv = new JunTV(body, vertex000, new Jun3dPoint(1, 1, 0));
		tv.doOperation();

		JunVertex vertex110 = mdev.newVertex();

		body.show().wireframePresentation();

		JunKDEV kdev = new JunKDEV(body, edge, vertex110);
		kdev.doOperation();
		body.show().wireframePresentation();

		System.out.println("-------- #killedEdge");
		System.out.println(kdev.killedEdge());
		System.out.println("-------- #killedVertex");
		System.out.println(kdev.killedVertex());
		System.out.println("-------- #vertex2");
		System.out.println(kdev.vertex2());
		System.out.println("-------- #edge2");
		System.out.println(kdev.edge2());
		System.out.println("-------- #edge3");
		System.out.println(kdev.edge3());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunKDEVTestExamples();
	}
}
