package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.smalltalk.StBlockClosure3;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunKEML class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/23 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKEML.java,v 9.9 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunKEML extends JunEulerOperator {

	protected JunEdgeProxy edgeProxy;
	protected JunLoopProxy loopProxy1;
	protected JunLoopProxy loopProxy2;
	protected JunVertexProxy vertexProxy1;
	protected JunVertexProxy vertexProxy2;
	protected JunEdgeProxy edgeProxy2;
	protected JunEdgeProxy edgeProxy3;

	/**
	 * Create a new instance of <code>JunKEML</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKEML
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKEML Body_edge_(JunBody aBody, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		return new JunKEML(aBody, aJunEdgeOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKEML</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy3 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKEML
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKEML Body_edge_adviseLoop_vertex_vertex_edge_edge_(JunBody aJunBody, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy1,
			JunTopologicalElementOrProxy aJunVertexOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy3) {
		return new JunKEML(aJunBody, aJunEdgeOrProxy, aJunLoopOrProxy1, aJunVertexOrProxy1, aJunVertexOrProxy2, aJunEdgeOrProxy2, aJunEdgeOrProxy3);
	}

	/**
	 * Create a new instance of <code>JunKEML</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKEML
	 * @category Instance creation
	 */
	public JunKEML(JunBody aBody, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		this(aBody, aJunEdgeOrProxy, null, null, null, null, null);
	}

	/**
	 * Create a new instance of <code>JunKEML</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy3 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKEML
	 * @category Instance creation
	 */
	public JunKEML(JunBody aJunBody, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2,
			JunTopologicalElementOrProxy aJunEdgeOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy3) {
		super();
		this.body_(aJunBody);
		this.edge_(aJunEdgeOrProxy);
		this.edge2_(aJunEdgeOrProxy2);
		this.edge3_(aJunEdgeOrProxy3);
		this.loop1_(aJunLoopOrProxy1);
		this.vertex1_(aJunVertexOrProxy1);
		this.vertex2_(aJunVertexOrProxy2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		edgeProxy = null;
		loopProxy1 = null;
		loopProxy2 = null;
		vertexProxy1 = null;
		vertexProxy2 = null;
		edgeProxy2 = null;
		edgeProxy3 = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy);
	}

	/**
	 * Answer the receiver's edge2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge2() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy2);
	}

	/**
	 * Set the receiver's edge2.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge2_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy2 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy2);
	}

	/**
	 * Answer the receiver's edge3.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge3() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy3);
	}

	/**
	 * Set the receiver's edge3.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge3_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy3 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy3);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		if (this.vertex1() == null) {
			this.vertex1_(this.edge().startVertex());
		}

		if (this.vertex2() == null) {
			this.vertex2_(this.edge().endVertex());
		}

		this.loop2_(this.edge().leftLoop());

		JunEdge fromEdge1;
		if (this.edge2() == null) {
			fromEdge1 = this.edge().prevEdgeWithVertex_(this.vertex1());
			this.edge2_(fromEdge1);
		} else {
			fromEdge1 = this.edge2();
		}
		JunEdge toEdge1 = this.edge().nextEdgeWithVertex_(this.vertex1());
		JunEdge fromEdge2;
		if (this.edge3() == null) {
			fromEdge2 = this.edge().prevEdgeWithVertex_(this.vertex2());
			this.edge3_(fromEdge2);
		} else {
			fromEdge2 = this.edge3();
		}
		JunEdge toEdge2 = this.edge().nextEdgeWithVertex_(this.vertex2());
		this.loop1_(JunLoop.Edge_(toEdge1));
		this.loop2().edge_(toEdge2);
		fromEdge1.setNextEdge_withVertex_(toEdge1, this.vertex1());
		fromEdge2.setNextEdge_withVertex_(toEdge2, this.vertex2());

		fromEdge1.directionVertex_vertexEdgeVertexDo_(this.vertex1(), new StBlockClosure3<Void, JunVertex, JunEdge, JunVertex>() {
			public Void value(JunVertex v1, JunEdge e, JunVertex v2) {
				e.setLoop_toVertex_(JunKEML.this.loop1(), v2);
				return null;
			}
		});

		if (this.vertex1().edge().equals(this.edge())) {
			this.vertex1().edge_(fromEdge1);
		}

		if (this.vertex2().edge().equals(this.edge())) {
			this.vertex2().edge_(fromEdge2);
		}

		this.edge().forget();
		this.body().removeEdge_(this.edge());
		this.body().addLoop_(this.loop1());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMEKL(this.body(), loopProxy1, loopProxy2, vertexProxy1, vertexProxy2, edgeProxy, edgeProxy2, edgeProxy3);
	}

	/**
	 * Answer the receiver's killed edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge killedEdge() {
		return this.edge();
	}

	/**
	 * Answer the receiver's loop1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop1() {
		return (JunLoop) this.getTopologicalElement_(loopProxy1);
	}

	/**
	 * Set the receiver's loop1.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop1_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy1 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy1);
	}

	/**
	 * Answer the receiver's loop2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop2() {
		return (JunLoop) this.getTopologicalElement_(loopProxy2);
	}

	/**
	 * Set the receiver's loop2.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop2_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy2 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy2);
	}

	/**
	 * Answer the receiver's new loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop newLoop() {
		return this.loop1();
	}

	/**
	 * Answer the receiver's original loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop originalLoop() {
		return this.loop2();
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (super.precondition() == false) {
			return false;
		}
		JunEdge edge = this.edge();
		if (edge == null) {
			return false;
		}
		if (this.loop1() != null) {
			return false;
		}
		if (edge.leftLoop().equals(edge.rightLoop()) == false) {
			return false;
		}
		return true;
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}

	/**
	 * Answer the receiver's vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex2() {
		return (JunVertex) this.getTopologicalElement_(this.vertexProxy2);
	}

	/**
	 * Set the receiver's vertex2.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex2_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy2 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy2);
	}

}
