package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunKEVVL class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKEVVL.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunKEVVL extends JunEulerOperator {
	protected Jun3dPoint point1;
	protected Jun3dPoint point2;
	protected JunEdgeProxy edgeProxy;
	protected JunVertexProxy vertexProxy1;
	protected JunVertexProxy vertexProxy2;
	protected JunLoopProxy loopProxy;

	/**
	 * Create a new instance of <code>JunKEVVL</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKEVVL
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKEVVL Body_edge_(JunBody aBody, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		return new JunKEVVL(aBody, aJunEdgeOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKEVVL</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKEVVL(JunBody aBody, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		super(aBody);
		this.edge_(aJunEdgeOrProxy);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		point1 = null;
		point2 = null;
		edgeProxy = null;
		vertexProxy1 = null;
		vertexProxy2 = null;
		loopProxy = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.vertex1_(this.edge().startVertex());
		this.vertex2_(this.edge().endVertex());
		this.loop_(this.edge().leftLoop());
		point1 = this.vertex1().point();
		point2 = this.vertex2().point();
		this.edge().forget();
		this.vertex1().forget();
		this.vertex2().forget();
		this.loop().forget();
		this.body().removeLoop_(this.loop());
		this.body().removeEdge_(this.edge());
		this.body().removeVertex_(this.vertex1());
		this.body().removeVertex_(this.vertex2());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMEVVL(this.body(), this.point1(), this.point2(), edgeProxy, vertexProxy1, vertexProxy2, loopProxy);
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return (JunLoop) this.getTopologicalElement_(loopProxy);
	}

	/**
	 * Set the receiver's loop.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy);
	}

	/**
	 * Answer the receiver's point1.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point1() {
		return point1;
	}

	/**
	 * Answer the receiver's point2.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point2() {
		return point2;
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (super.precondition() == false) {
			return false;
		}

		JunEdge edge = this.edge();
		if (edge == null) {
			return false;
		}
		if (edge.equals(edge.startRightEdge()) == false) {
			return false;
		}
		if (edge != edge.endLeftEdge()) {
			return false;
		}
		if (edge.leftLoop().equals(edge.rightLoop()) == false) {
			return false;
		}

		return true;
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}

	/**
	 * Answer the receiver's vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex2() {
		return (JunVertex) this.getTopologicalElement_(this.vertexProxy2);
	}

	/**
	 * Set the receiver's vertex2.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex2_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		this.vertexProxy2 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, this.vertexProxy2);
	}
}
