package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunKVE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKVE.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunKVE extends JunEulerOperator {
	protected JunVertexProxy vertexProxy;
	protected JunVertexProxy vertexProxy1;
	protected JunEdgeProxy edgeProxy1;
	protected JunEdgeProxy edgeProxy2;
	protected Jun3dPoint point;

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKVE
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKVE Body_vertex_(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy) {
		return new JunKVE(aBody, aJunVertexOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKVE
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKVE Body_vertex_adviseEdge_edge_(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunEdgeOrProxy1, JunTopologicalElementOrProxy aJunEdgeOrProxy2) {
		return new JunKVE(aBody, aJunVertexOrProxy, aJunEdgeOrProxy1, aJunEdgeOrProxy2);
	}

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunKVE
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunKVE Body_vertex_killEdge_(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		return new JunKVE(aBody, aJunVertexOrProxy, aJunEdgeOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKVE(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy) {
		super();
		this.body_(aBody);
		this.vertex_(aJunVertexOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKVE(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		super();
		this.body_(aBody);
		this.vertex_(aJunVertexOrProxy);
		this.edge1_(aJunEdgeOrProxy);
	}

	/**
	 * Create a new instance of <code>JunKVE</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKVE(JunBody aBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunEdgeOrProxy1, JunTopologicalElementOrProxy aJunEdgeOrProxy2) {
		super();
		this.body_(aBody);
		this.vertex_(aJunVertexOrProxy);
		this.edge1_(aJunEdgeOrProxy1);
		this.edge2_(aJunEdgeOrProxy2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		vertexProxy = null;
		vertexProxy1 = null;
		edgeProxy1 = null;
		edgeProxy2 = null;
		point = null;
	}

	/**
	 * Answer the receiver's alive edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge aliveEdge() {
		return this.edge2();
	}

	/**
	 * Answer the receiver's edge1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge1() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy1);
	}

	/**
	 * Set the receiver's edge1.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge1_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy1 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy1);
	}

	/**
	 * Answer the receiver's edge2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge2() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy2);
	}

	/**
	 * Set the receiver's edge2.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge2_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy2 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy2);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		if (this.edge1() == null) {
			this.edge1_(this.vertex().edge());
		}

		if (this.edge2() == null) {
			this.edge2_(this.edge1().nextEdgeWithVertex_(this.vertex()));
		}

		point = this.vertex().point();
		JunVertex oppositeVertex = this.edge1().oppositeVertexOfVertex_(this.vertex());
		this.vertex1_(oppositeVertex);
		JunEdge toEdge = this.edge1().nextEdgeWithVertex_(oppositeVertex);
		JunEdge fromEdge = this.edge1().prevEdgeWithVertex_(oppositeVertex);

		if (fromEdge.equals(this.edge1())) {
			fromEdge = this.edge2();
		}

		this.edge2().setNextEdge_withVertex_(toEdge, this.vertex());
		this.edge2().setVertex_insteadOf_(oppositeVertex, this.vertex());
		fromEdge.setNextEdge_withVertex_(this.edge2(), oppositeVertex);

		if (oppositeVertex.edge().equals(this.edge1())) {
			oppositeVertex.edge_(this.edge2());
		}

		if (this.edge1().leftLoop().edge().equals(this.edge1())) {
			this.edge1().leftLoop().edge_(this.edge2());
		}

		if (this.edge1().rightLoop().edge().equals(this.edge1())) {
			this.edge1().rightLoop().edge_(this.edge2());
		}

		this.vertex().forget();
		this.edge1().forget();
		this.body().removeVertex_(this.vertex());
		this.body().removeEdge_(this.edge1());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMVE(this.body(), edgeProxy2, this.point(), vertexProxy, edgeProxy1, vertexProxy1);
	}

	/**
	 * Answer the receiver's killed edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge killedEdge() {
		return this.edge1();
	}

	/**
	 * Answer the receiver's point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point() {
		return point;
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (super.precondition() == false) {
			return false;
		}

		JunVertex vertex = this.vertex();
		if (vertex == null) {
			return false;
		}

		JunEdge[] edges = vertex.edges();
		if (edges.length != 2) {
			return false;
		}
		for (int i = 0; i < edges.length; i++) {
			JunEdge edge = edges[i];
			if (edge.oppositeVertexOfVertex_(vertex).equals(vertex)) {
				return false;
			}
		}

		if ((this.edgeProxy1 != null && ((this.edge1() == null) || (this.edge1().isKilled()))) == true) {
			return false;
		}
		if ((this.edgeProxy2 != null && ((this.edge2() == null) || (this.edge2().isKilled()))) == true) {
			return false;
		}

		return true;
	}

	/**
	 * Answer the receiver's vertex.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy);
	}

	/**
	 * Set the receiver's vertex.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy);
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}
}
