package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;

/**
 * JunMCLKPL class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMCLKPL.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunMCLKPL extends JunEulerOperator {
	protected JunLoopProxy loopProxy1;
	protected JunLoopProxy loopProxy2;

	/**
	 * Create a new instance of <code>JunMCLKPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aLoopOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMCLKPL
	 * @deprecated since Jun700, using constructor
	 * @category Instance creation
	 */
	public static final JunMCLKPL Body_loop_loop_(JunBody aJunBody, JunTopologicalElementOrProxy aLoopOrProxy1, JunTopologicalElementOrProxy aLoopOrProxy2) {
		return new JunMCLKPL(aJunBody, aLoopOrProxy1, aLoopOrProxy2);
	}

	/**
	 * Create a new instance of <code>JunMCLKPL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aLoopOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMCLKPL(JunBody aJunBody, JunTopologicalElementOrProxy aLoopOrProxy1, JunTopologicalElementOrProxy aLoopOrProxy2) {
		super();
		this.body_(aJunBody);
		this.loop1_(aLoopOrProxy1);
		this.loop2_(aLoopOrProxy2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		loopProxy1 = null;
		loopProxy2 = null;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.loop1().addChildLoop_(this.loop2());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKCLMPL(this.body(), loopProxy2, loopProxy1);
	}

	/**
	 * Answer the receiver's loop1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop1() {
		return (JunLoop) this.getTopologicalElement_(loopProxy1);
	}

	/**
	 * Set the receiver's loop1.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop1_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy1 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy1);
	}

	/**
	 * Answer the receiver's loop2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop2() {
		return (JunLoop) this.getTopologicalElement_(loopProxy2);
	}

	/**
	 * Set the receiver's loop2.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop2_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy2 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy2);
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunLoop loop1 = this.loop1();
		if (loop1 == null) {
			return false;
		}
		JunLoop loop2 = this.loop2();
		if (loop2 == null) {
			return false;
		}

		return (super.precondition() && loop1.isParentLoop() && loop2.isParentLoop());
	}
}
