package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;

/**
 * JunMDEVTestExamples class
 * 
 *  @author    BNR, ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMDEVTestExamples.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunMDEVTestExamples extends JunTestExamples {
	/**
	 * Example.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();

		//JunLoop loop1 = mevvl.newLoop();
		JunLoop loop1 = mevvl.loop();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();
		JunMEV mev = new JunMEV(body, vertex100, loop1, new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunEdge edge1 = mev.newEdge();
		JunVertex vertex000 = mev.newVertex();
		JunMEL mel = new JunMEL(body, loop1, vertex010, vertex000);
		mel.doOperation();
		body.show();

		JunEdge edge2 = mel.newEdge();
		JunMDEV mdev = JunMDEV.Body_vertex_edge_edge_(body, vertex000, edge1, edge2);
		mdev.doOperation();
		body.show();
		System.out.println("-------- #newEdge");
		System.out.println(mdev.newEdge());
		System.out.println("-------- #newVertex");
		System.out.println(mdev.newVertex());

		return true;
	}

	/**
	 * Example.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		JunVertex v000 = null;
		JunVertex v100 = null;
		JunVertex v010 = null;
		JunVertex v001 = null;
		Jun3dPoint[] a3dPoint = new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0), new Jun3dPoint(0, 0, 1) };
		JunBody body = JunBody.Tetrahedron_(a3dPoint);
		JunVertex[] temp = body.vertexes();

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(0, 0, 0))) {
				v000 = temp[i];
				break;
			}
		}

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(1, 0, 0))) {
				v100 = temp[i];
				break;
			}
		}

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(0, 1, 0))) {
				v010 = temp[i];
				break;
			}
		}

		for (int i = 0; i < temp.length; i++) {
			if ((temp[i].point()).equals(new Jun3dPoint(0, 0, 1))) {
				v001 = temp[i];
				break;
			}
		}

		JunEdge edge1 = body.edgeForVertex_and_(v000, v100);
		JunEdge edge2 = body.edgeForVertex_and_(v000, v010);
		JunMDEV mdev = JunMDEV.Body_vertex_edge_edge_(body, v000, edge1, edge2);
		mdev.doOperation();
		mdev.vertex1().point_((new Jun3dPoint(0.5, -0.5, -0.5)));
		new JunMEL(body, body.loopForVertex_and_(v001, v000), v001, v000).doOperation();
		new JunMEL(body, body.loopForVertex_and_(v000, v100), v000, v100).doOperation();

		// return body;
		body.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunMDEVTestExamples();
	}
}
