package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunMEL class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/08/23 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMEL.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunMEL extends JunEulerOperator {
	protected JunEdgeProxy edgeProxy;
	protected JunLoopProxy loopProxy1;
	protected JunLoopProxy loopProxy2;
	protected JunVertexProxy vertexProxy1;
	protected JunVertexProxy vertexProxy2;
	protected JunEdgeProxy edgeProxy2;
	protected JunEdgeProxy edgeProxy3;

	/**
	 * Create a new instance of <code>JunMEL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEL
	 * @deprecated since Jun700, using constructor
	 * @category Instance creation
	 */
	public static final JunMEL Body_loop_vertex_vertex_(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2) {
		return new JunMEL(aJunBody, aJunLoopOrProxy, aJunVertexOrProxy1, aJunVertexOrProxy2);
	}

	/**
	 * Create a new instance of <code>JunMEL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy3 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEL
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEL Body_loop_vertex_vertex_adviseEdge_loop_edge_edge_(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2,
			JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy3) {
		return new JunMEL(aJunBody, aJunLoopOrProxy, aJunVertexOrProxy1, aJunVertexOrProxy2, aJunEdgeOrProxy, aJunLoopOrProxy2, aJunEdgeOrProxy2, aJunEdgeOrProxy3);
	}

	/**
	 * Create a new instance of <code>JunMEL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMEL(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2) {
		this(aJunBody, aJunLoopOrProxy, aJunVertexOrProxy1, aJunVertexOrProxy2, null, null, null, null);
	}

	/**
	 * Create a new instance of <code>JunMEL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy3 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMEL(JunBody aJunBody, JunTopologicalElementOrProxy aJunLoopOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy,
			JunTopologicalElementOrProxy aJunLoopOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy3) {
		super();
		this.body_(aJunBody);
		this.loop1_(aJunLoopOrProxy2);
		this.loop2_(aJunLoopOrProxy);
		this.vertex1_(aJunVertexOrProxy1);
		this.vertex2_(aJunVertexOrProxy2);
		this.edge_(aJunEdgeOrProxy);
		this.edge2_(aJunEdgeOrProxy2);
		this.edge3_(aJunEdgeOrProxy3);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		edgeProxy = null;
		loopProxy1 = null;
		loopProxy2 = null;
		vertexProxy1 = null;
		vertexProxy2 = null;
		edgeProxy2 = null;
		edgeProxy3 = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy);
	}

	/**
	 * Answer the receiver's edge2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge2() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy2);
	}

	/**
	 * Set the receiver's edge2.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge2_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy2 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy2);
	}

	/**
	 * Answer the receiver's edge3.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge3() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy3);
	}

	/**
	 * Answer the receiver's edge3.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public void edge3_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy3 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy3);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.edge_(JunEdge.StartVertex_endVertex_(this.vertex2(), this.vertex1()));
		this.loop1_(JunLoop.Edge_(this.edge()));

		JunVertex targetVertex = this.vertex1();
		JunVertex directionVertex, prevVertex;
		JunEdge prevEdge, currentEdge, fromEdge1, toEdge1, fromEdge2, toEdge2;

		if (this.edge2() == null) {
			fromEdge2 = this.loop2().edgeToVertex_(this.vertex2());
			directionVertex = this.vertex2();
			prevEdge = fromEdge2;
			currentEdge = prevEdge.nextEdgeWithVertex_(directionVertex);
			prevVertex = directionVertex;
			directionVertex = currentEdge.oppositeVertexOfVertex_(prevVertex);
			toEdge2 = currentEdge;
			if (prevVertex.equals(this.vertex2())) {
				fromEdge2 = prevEdge;
				toEdge2 = currentEdge;
			}
			while (prevVertex.equals(targetVertex) == false) {
				prevEdge = currentEdge;
				currentEdge = currentEdge.nextEdgeWithVertex_(directionVertex);
				prevVertex = directionVertex;
				directionVertex = currentEdge.oppositeVertexOfVertex_(prevVertex);
				if (prevVertex.equals(this.vertex2())) {
					fromEdge2 = prevEdge;
					toEdge2 = currentEdge;
				}
			}
			fromEdge1 = prevEdge;
			toEdge1 = currentEdge;
		} else {
			fromEdge1 = this.edge2();
			toEdge1 = fromEdge1.nextEdgeWithVertex_(this.vertex1());
			fromEdge2 = this.edge3();
			toEdge2 = fromEdge2.nextEdgeWithVertex_(this.vertex2());
		}

		directionVertex = this.vertex2();
		prevEdge = fromEdge2;
		currentEdge = toEdge2;
		prevVertex = directionVertex;
		directionVertex = currentEdge.oppositeVertexOfVertex_(prevVertex);
		while ((prevVertex.equals(targetVertex) && currentEdge.equals(toEdge1)) == false) {
			currentEdge.setLoop_toVertex_(this.newLoop(), directionVertex);
			prevEdge = currentEdge;
			currentEdge = currentEdge.nextEdgeWithVertex_(directionVertex);
			prevVertex = directionVertex;
			directionVertex = currentEdge.oppositeVertexOfVertex_(prevVertex);
		}
		fromEdge2.setNextEdge_withVertex_(this.edge(), this.vertex2());
		this.edge().setNextEdge_withVertex_(toEdge1, this.vertex1());
		this.edge().setNextEdge_withVertex_(toEdge2, this.vertex2());
		fromEdge1.setNextEdge_withVertex_(this.edge(), this.vertex1());
		this.edge().setLoop_toVertex_(this.loop1(), this.vertex2());
		this.edge().setLoop_toVertex_(this.loop2(), this.vertex1());
		this.loop2().edge_(this.edge());
		this.body().addEdge_(this.edge());
		this.body().addLoop_(this.loop1());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKEL(this.body(), edgeProxy, loopProxy1, loopProxy2, vertexProxy1, vertexProxy2);
	}

	/**
	 * Answer the receiver's loop1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop1() {
		return (JunLoop) this.getTopologicalElement_(loopProxy1);
	}

	/**
	 * Set the receiver's loop1.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop1_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy1 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy1);
	}

	/**
	 * Answer the receiver's loop2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop2() {
		return (JunLoop) this.getTopologicalElement_(loopProxy2);
	}

	/**
	 * Set the receiver's loop2.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop2_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy2 = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy2);
	}

	/**
	 * Answer the receiver's new edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.Junedge
	 * @category accessing
	 */
	public JunEdge newEdge() {
		return this.edge();
	}

	/**
	 * Answer the receiver's new loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop newLoop() {
		return this.loop1();
	}

	/**
	 * Answer the receiver's original loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop originalLoop() {
		return this.loop2();
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (super.precondition() == false) {
			return false;
		}
		if (this.edge() != null) {
			return false;
		}
		if (this.loop1() != null) {
			return false;
		}
		JunLoop loop2 = this.loop2();
		if (loop2 == null) {
			return false;
		}
		JunVertex vertex1 = this.vertex1();
		if (vertex1 == null) {
			return false;
		}
		JunVertex vertex2 = this.vertex2();
		if (vertex2 == null) {
			return false;
		}
		if (loop2.includesVertex_(vertex1) == false) {
			return false;
		}
		if (loop2.includesVertex_(vertex2) == false) {
			return false;
		}
		return true;
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) getTopologicalElement_(vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}

	/**
	 * Answer the receiver's vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex2() {
		return (JunVertex) getTopologicalElement_(vertexProxy2);
	}

	/**
	 * Set the receiver's vertex2.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex2_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy2 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy2);
	}
}
