package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunMEV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMEV.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunMEV extends JunEulerOperator {
	protected JunEdgeProxy edgeProxy;
	protected JunVertexProxy vertexProxy1;
	protected JunVertexProxy vertexProxy2;
	protected JunLoopProxy loopProxy;
	protected Jun3dPoint point;
	protected JunEdgeProxy edgeProxy2;

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEV
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEV Body_vertex_loop_point_(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint) {
		return new JunMEV(aJunBody, aJunVertexOrProxy, aJunLoopOrProxy, aJun3dPoint);
	}

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEV
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEV Body_vertex_loop_point_adviseEdge_vertex_(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint, JunTopologicalElementOrProxy aJunEdgeOrProxy,
			JunTopologicalElementOrProxy aJunVertexOrProxy2) {
		return new JunMEV(aJunBody, aJunVertexOrProxy, aJunLoopOrProxy, aJun3dPoint, aJunEdgeOrProxy, aJunVertexOrProxy2);
	}

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEV
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEV Body_vertex_loop_point_adviseEdge_vertex_edge_(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint,
			JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy2, JunTopologicalElementOrProxy aJunEdgeOrProxy2) {
		return new JunMEV(aJunBody, aJunVertexOrProxy, aJunLoopOrProxy, aJun3dPoint, aJunEdgeOrProxy, aJunVertexOrProxy2, aJunEdgeOrProxy2);
	}

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunMEV(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint) {
		this(aJunBody, aJunVertexOrProxy, aJunLoopOrProxy, aJun3dPoint, null, null);
	}

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMEV(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy2) {
		super();
		this.body_(aJunBody);
		this.vertex1_(aJunVertexOrProxy2);
		this.vertext2_(aJunVertexOrProxy);
		this.loop_(aJunLoopOrProxy);
		this.point_(aJun3dPoint);
		this.edge_(aJunEdgeOrProxy);
	}

	/**
	 * Create a new instance of <code>JunMEV</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunEdgeOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMEV(JunBody aJunBody, JunTopologicalElementOrProxy aJunVertexOrProxy, JunTopologicalElementOrProxy aJunLoopOrProxy, Jun3dPoint aJun3dPoint, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy2,
			JunTopologicalElementOrProxy aJunEdgeOrProxy2) {
		super();
		this.body_(aJunBody);
		this.vertex1_(aJunVertexOrProxy2);
		this.vertext2_(aJunVertexOrProxy);
		this.loop_(aJunLoopOrProxy);
		this.point_(aJun3dPoint);
		this.edge_(aJunEdgeOrProxy);
		this.edge2_(aJunEdgeOrProxy2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		edgeProxy = null;
		vertexProxy1 = null;
		vertexProxy2 = null;
		loopProxy = null;
		point = null;
		edgeProxy2 = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy);
	}

	/**
	 * Answer the receiver's edge2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge2() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy2);
	}

	/**
	 * Set the receiver's edge2.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge2_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy2 = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy2);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.vertex1_(JunVertex.Point_(this.point()));
		this.edge_(JunEdge.StartVertex_endVertex_(this.vertex1(), this.vertex2()));
		this.vertex1().edge_(this.edge());
		this.edge().leftLoop_(this.loop());
		this.edge().rightLoop_(this.loop());

		JunEdge fromEdge;
		if (this.edge2() == null) {
			fromEdge = this.loop().edgeToVertex_(this.vertex2());
			this.edge2_(fromEdge);
		} else {
			fromEdge = this.edge2();
		}

		JunEdge toEdge = fromEdge.nextEdgeWithVertex_(this.vertex2());
		fromEdge.setNextEdge_withVertex_(this.edge(), this.vertex2());
		this.edge().setNextEdge_withVertex_(this.edge(), this.vertex1());
		this.edge().setNextEdge_withVertex_(toEdge, this.vertex2());
		this.body().addVertex_(this.vertex1());
		this.body().addEdge_(this.edge());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKEV(this.body(), edgeProxy, vertexProxy1, edgeProxy2);
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return (JunLoop) this.getTopologicalElement_(loopProxy);
	}

	/**
	 * Set the receiver's loop.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy);
	}

	/**
	 * Answer the receiver's new edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge newEdge() {
		return this.edge();
	}

	/**
	 * Answer the receiver's new vertex.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex newVertex() {
		return this.vertex1();
	}

	/**
	 * Answer the receiver's point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point() {
		return point;
	}

	/**
	 * Set the receiver's point.
	 * 
	 * @param a3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void point_(Jun3dPoint a3dPoint) {
		point = a3dPoint;
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (this.edge() != null) {
			return false;
		}
		JunLoop loop = this.loop();
		if (loop == null) {
			return false;
		}
		if (this.vertex1() != null) {
			return false;
		}
		JunVertex vertex2 = this.vertex2();
		if (vertex2 == null) {
			return false;
		}
		return super.precondition() && loop.includesVertex_(vertex2);
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) this.getTopologicalElement_(this.vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}

	/**
	 * Answer the receiver's vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex2() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy2);
	}

	/**
	 * Set the receiver's vertex2.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertext2_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy2 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy2);
	}
}
