package jp.co.sra.jun.topology.euleroperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunEulerOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunMEVVL class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/08/04 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMEVVL.java,v 9.8 2010/03/09 09:17:54 nisinaka Exp $
 */
public class JunMEVVL extends JunEulerOperator {
	protected JunEdgeProxy edgeProxy;
	protected JunVertexProxy vertexProxy1;
	protected JunVertexProxy vertexProxy2;
	protected JunLoopProxy loopProxy;
	protected Jun3dPoint point1;
	protected Jun3dPoint point2;

	/**
	 * Create a new instance of <code>JunMEVVL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEVVL
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEVVL Body_point_point_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2) {
		return new JunMEVVL(aJunBody, aJun3dPoint1, aJun3dPoint2);
	}

	/**
	 * Create a new instance of <code>JunMEVVL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.euleroperators.JunMEVVL
	 * @deprecated since Jun700, using constructor.
	 * @category Instance creation
	 */
	public static final JunMEVVL Body_point_point_adviseEdge_vertex_vertex_loop_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1,
			JunTopologicalElementOrProxy aJunVertexOrProxy2, JunTopologicalElementOrProxy aJunLoopOrProxy) {
		return new JunMEVVL(aJunBody, aJun3dPoint1, aJun3dPoint2, aJunEdgeOrProxy, aJunVertexOrProxy1, aJunVertexOrProxy2, aJunLoopOrProxy);
	}

	/**
	 * Create a new instance of <code>JunMEVVL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunMEVVL(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2) {
		this(aJunBody, aJun3dPoint1, aJun3dPoint2, null, null, null, null);
	}

	/**
	 * Create a new instance of <code>JunMEVVL</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy1 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunVertexOrProxy2 jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunMEVVL(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, JunTopologicalElementOrProxy aJunEdgeOrProxy, JunTopologicalElementOrProxy aJunVertexOrProxy1, JunTopologicalElementOrProxy aJunVertexOrProxy2,
			JunTopologicalElementOrProxy aJunLoopOrProxy) {
		super(aJunBody);
		this.point1_(aJun3dPoint1);
		this.point2_(aJun3dPoint2);
		this.edge_(aJunEdgeOrProxy);
		this.vertex1_(aJunVertexOrProxy1);
		this.vertex2_(aJunVertexOrProxy2);
		this.loop_(aJunLoopOrProxy);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		edgeProxy = null;
		vertexProxy1 = null;
		vertexProxy2 = null;
		loopProxy = null;
		point1 = null;
		point2 = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param aJunEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy aJunEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(aJunEdgeOrProxy, edgeProxy);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.vertex1_(JunVertex.Point_(this.point1()));
		this.vertex2_(JunVertex.Point_(this.point2()));
		this.edge_(JunEdge.StartVertex_endVertex_(this.vertex1(), this.vertex2()));
		this.vertex1().edge_(this.edge());
		this.vertex2().edge_(this.edge());
		this.loop_(JunLoop.Edge_(this.edge()));
		this.vertex1().edge_(this.edge());
		this.vertex2().edge_(this.edge());
		this.edge().startRightEdge_(this.edge());
		this.edge().endLeftEdge_(this.edge());
		this.edge().leftLoop_(this.loop());
		this.edge().rightLoop_(this.loop());
		this.body().addVertex_(this.vertex1());
		this.body().addVertex_(this.vertex2());
		this.body().addEdge_(this.edge());
		this.body().addLoop_(this.loop());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKEVVL(this.body(), edgeProxy);
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return (JunLoop) this.getTopologicalElement_(loopProxy);
	}

	/**
	 * Set the receiver's loop.
	 * 
	 * @param aJunLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop_(JunTopologicalElementOrProxy aJunLoopOrProxy) {
		loopProxy = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aJunLoopOrProxy, loopProxy);
	}

	/**
	 * Answer the receiver's new edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge newEdge() {
		return this.edge();
	}

	/**
	 * Answer the receiver's new loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop newLoop() {
		return this.loop();
	}

	/**
	 * Answer the receiver's new vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex newVertex1() {
		return this.vertex1();
	}

	/**
	 * Answer the receiver's new vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex newVertex2() {
		return this.vertex2();
	}

	/**
	 * Answer the receiver's point1.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point1() {
		return point1;
	}

	/**
	 * Set the receiver's point1.
	 * 
	 * @param a3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void point1_(Jun3dPoint a3dPoint) {
		point1 = a3dPoint;
	}

	/**
	 * Answer the receiver's point2.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point2() {
		return point2;
	}

	/**
	 * Set the receiver's point2.
	 * 
	 * @param a3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void point2_(Jun3dPoint a3dPoint) {
		point2 = a3dPoint;
	}

	/**
	 * Answer <code>true</code> if the pre-condition is valid, otherwise <code>false</code>.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		if (this.edge() != null) {
			return false;
		} else if (this.vertex1() != null) {
			return false;
		} else if (this.vertex2() != null) {
			return false;
		} else if (this.loop() != null) {
			return false;
		} else {
			return (super.precondition());
		}
	}

	/**
	 * Answer the receiver's vertex1.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex1() {
		return (JunVertex) this.getTopologicalElement_(this.vertexProxy1);
	}

	/**
	 * Set the receiver's vertex1.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex1_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy1 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy1);
	}

	/**
	 * Answer the receiver's vertex2.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex2() {
		return (JunVertex) this.getTopologicalElement_(this.vertexProxy2);
	}

	/**
	 * Set the receiver's vertex2.
	 * 
	 * @param aJunVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex2_(JunTopologicalElementOrProxy aJunVertexOrProxy) {
		vertexProxy2 = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aJunVertexOrProxy, vertexProxy2);
	}
}
