package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.abstracts.JunCurve;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;

/**
 * JunKCV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKCV.java,v 9.8 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunKCV extends JunGeometricOperator {
	/** The topology on which the receiver operates. */
	protected JunEdgeProxy edgeProxy;

	/** The geometry which corresponds to the receiver's topology. */
	protected JunCurve curve;

	/**
	 * Create a new instance of <code>JunKCV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category Instance creation
	 */
	public JunKCV(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy) {
		super();
		this.body_(aBody);
		this.edge_(anEdgeOrProxy);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @return jp.co.sra.jun.topology.geometricoperators.JunKCV
	 * @deprecated since Jun700, using JunKCV(JunBody, JunTopologicalElementOrProxy)
	 * @category Instance creation
	 */
	public static final JunKCV Body_edge_(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy) {
		return new JunKCV(aBody, anEdgeOrProxy);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		edgeProxy = null;
		curve = null;
	}

	/**
	 * Answer the receiver's curve.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunCurve
	 * @category accessing
	 */
	public JunCurve curve() {
		return curve;
	}

	/**
	 * Set the receiver's curve.
	 * 
	 * @param aCurve jp.co.sra.jun.geometry.abstracts.JunCurve
	 * @category accessing
	 */
	public void curve_(JunCurve aCurve) {
		curve = aCurve;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy anEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(anEdgeOrProxy, edgeProxy);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.curve_(this.edge().basicCurve());
		this.edge().curve_(null);
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMCV(this.body(), edgeProxy, this.curve());
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunEdge edge = this.edge();
		if (edge == null) {
			return false;
		}
		if (edge.basicCurve() == null) {
			return false;
		}
		return super.precondition();
	}
}
