package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.curves.JunNurbsCurve;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;
import jp.co.sra.jun.topology.euleroperators.JunMVE;

/**
 * JunKCVTestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/04/16 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKCVTestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunKCVTestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		mevvl.doOperation();
		JunEdge edge = mevvl.edge();
		JunMEL mel = new JunMEL(body, mevvl.loop(), mevvl.vertex1(), mevvl.vertex2());
		mel.doOperation();
		JunMVE mve = new JunMVE(body, mel.newEdge(), new Jun3dPoint(0, -1, 0));
		mve.doOperation();

		JunNurbsCurve curve = JunNurbsCurve.BezierControlPoints_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(0.5, 0, 0.5), new Jun3dPoint(1, 0, 0) });
		JunMCV mcv = new JunMCV(body, edge, curve);
		mcv.doOperation();
		body.asWireframedOpenGL3dObject().show();
		JunKCV kcv = new JunKCV(body, edge);
		kcv.doOperation();
		body.asWireframedOpenGL3dObject().show();

		return true;
	}

	/**
	 * ManualExample1:
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean ManualExample1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();

		JunLoop loop = mevvl.newLoop();
		JunEdge edge = mevvl.newEdge();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, loop, new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body, loop, vertex010, vertex000);
		mel.doOperation();

		Jun3dPoint[] controlPoints = new Jun3dPoint[] { new Jun3dPoint(1, 0, 0), new Jun3dPoint(0.5, 0.5, -0.5), new Jun3dPoint(0, 1, 0) };
		JunNurbsCurve curve = JunNurbsCurve.BezierControlPoints_(controlPoints);
		JunMCV mcv = new JunMCV(body, edge, curve);
		mcv.doOperation();

		body.show();

		JunKCV kcv = new JunKCV(body, edge);
		kcv.doOperation();
		body.show();

		System.out.println("-------- #curve");
		System.out.println(kcv.curve());
		System.out.println("-------- #edge");
		System.out.println(kcv.edge());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunKCVTestExamples();
	}

}
