package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.abstracts.JunCurve;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;

/**
 * JunMCV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMCV.java,v 9.8 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunMCV extends JunGeometricOperator {
	protected JunEdgeProxy edgeProxy;
	protected JunCurve curve;

	/**
	 * Create a new instance of <code>JunMCV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aCurve jp.co.sra.jun.geometry.abstracts.JunCurve
	 * @category Instance creation
	 */
	public JunMCV(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy, JunCurve aCurve) {
		super();
		this.body_(aBody);
		this.edge_(anEdgeOrProxy);
		this.curve_(aCurve);
	}

	/**
	 * Create a new instance of <code>JunMCV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aCurve jp.co.sra.jun.geometry.abstracts.JunCurve
	 * @return jp.co.sra.jun.topology.geometricoperators.JunMCV
	 * @deprecated since Jun700, using JunMCV(JunBody, JunTopologicalElementOrProxy, JunCurve)
	 * @category Instance creation
	 */
	public static final JunMCV Body_edge_curve_(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy, JunCurve aCurve) {
		return new JunMCV(aBody, anEdgeOrProxy, aCurve);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		edgeProxy = null;
		curve = null;
	}

	/**
	 * Answer the receive's curve.
	 * 
	 * @return jp.co.sra.jun.geometry.curves.JunCurve
	 * @category accessing
	 */
	public JunCurve curve() {
		return curve;
	}

	/**
	 * Set the receive's curve.
	 * 
	 * @param aCurve jp.co.sra.jun.geometry.abstracts.JunCurve
	 * @category accessing
	 */
	public void curve_(JunCurve aCurve) {
		curve = aCurve;
	}

	/**
	 * Answer the receive's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receive's edge.
	 * 
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy anEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(anEdgeOrProxy, edgeProxy);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.edge().curve_(this.curve());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKCV(this.body(), edgeProxy);
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunEdge edge = this.edge();
		if (edge == null) {
			return false;
		}
		if (edge.basicCurve() != null) {
			return false;
		}
		return super.precondition();
	}
}
