package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.abstracts.JunSurface;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;

/**
 * JunMSF class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMSF.java,v 9.8 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunMSF extends JunGeometricOperator {
	/** The topology on which the receiver operates. */
	protected JunLoopProxy loopProxy;

	/** The geometry which corresponds to the receiver's topology. */
	protected JunSurface surface;

	/**
	 * Create a new instance of <code>JunMSF</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aSurface jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category Instance creation
	 */
	public JunMSF(JunBody aBody, JunTopologicalElementOrProxy aLoopOrProxy, JunSurface aSurface) {
		super();
		this.body_(aBody);
		this.loop_(aLoopOrProxy);
		this.surface_(aSurface);
	}

	/**
	 * Create a new instance of <code>JunMSF</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aSurface jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @return jp.co.sra.jun.topology.geometricoperators.JunKSF
	 * @deprecated since Jun700, using JunMSF(JunBody, JunTopologicalElementOrProxy, JunSurface)
	 * @category Instance creation
	 */
	public static final JunMSF Body_loop_surface_(JunBody aBody, JunTopologicalElementOrProxy aLoopOrProxy, JunSurface aSurface) {
		return new JunMSF(aBody, aLoopOrProxy, aSurface);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		loopProxy = null;
		surface = null;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		this.loop().surface_(this.surface());
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunKSF(this.body(), loopProxy);
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return (JunLoop) this.getTopologicalElement_(loopProxy);
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void loop_(JunTopologicalElementOrProxy aLoopOrProxy) {
		loopProxy = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aLoopOrProxy, loopProxy);
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunLoop loop = this.loop();
		if (loop == null) {
			return false;
		}
		if (loop.basicSurface() != null) {
			return false;
		}
		return super.precondition();
	}

	/**
	 * Answer the receiver's surface.
	 * 
	 * @return jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public JunSurface surface() {
		return surface;
	}

	/**
	 * Set the receiver's surface.
	 * 
	 * @param aSurface jp.co.sra.jun.geometry.abstracts.JunSurface
	 * @category accessing
	 */
	public void surface_(JunSurface aSurface) {
		surface = aSurface;
	}
}
