package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.abstracts.JunCurve;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.curves.JunNurbsCurve;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;

/**
 * JunTCV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTCV.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTCV extends JunGeometricOperator {

	protected JunEdgeProxy edgeProxy;
	protected int number;
	protected Jun3dPoint vector;

	/**
	 * Create a new instance of <code>JunTCV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param anInteger int
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunTCV(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy, int anInteger, Jun3dPoint aPoint) {
		super();
		this.body_(aBody);
		this.edge_(anEdgeOrProxy);
		this.number_(anInteger);
		this.vector_(aPoint);
	}

	/**
	 * Create a new instance of <code>JunTCV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param anInteger int
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.geometricoperators.JunTCV
	 * @deprecated since Jun700, using JunTCV(JunBody, JunTopologicalElementOrProxy, int, Jun3dPoint)
	 * @category Instance creation
	 */
	public static final JunTCV Body_edge_number_vector_(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy, int anInteger, Jun3dPoint aPoint) {
		return new JunTCV(aBody, anEdgeOrProxy, anInteger, aPoint);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		edgeProxy = null;
		number = Integer.MIN_VALUE;
		vector = null;
	}

	/**
	 * Answer the receiver's edge.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunEdge
	 * @category accessing
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Set the receiver's edge.
	 * 
	 * @param anEdgeOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void edge_(JunTopologicalElementOrProxy anEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(anEdgeOrProxy, edgeProxy);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		JunNurbsCurve curve = (JunNurbsCurve) ((this.edge()).basicCurve());
		Jun3dPoint newPoint = curve.controlPointAt_(this.number()).plus_(this.vector());
		curve.controlPointAt_put_(this.number(), newPoint);
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunTCV(this.body(), edgeProxy, this.number(), this.vector().negated());
	}

	/**
	 * Answer the receiver's number.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int number() {
		return number;
	}

	/**
	 * Set the receiver's number.
	 * 
	 * @param anInteger int
	 * @category accessing
	 */
	public void number_(int anInteger) {
		number = anInteger;
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunEdge edge = this.edge();

		if (edge == null) {
			return false;
		}

		JunCurve curve = edge.basicCurve();

		if (curve == null) {
			return false;
		}

		if (!(curve instanceof JunNurbsCurve)) {
			return false;
		}

		if (((JunNurbsCurve) curve).controlPointSize() < this.number()) {
			return false;
		}

		return super.precondition();
	}

	/**
	 * Answer the receiver's vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the receiver's vector.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void vector_(Jun3dPoint aPoint) {
		vector = aPoint;
	}

}
