package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.curves.JunNurbsCurve;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;
import jp.co.sra.jun.topology.euleroperators.JunMVE;

/**
 * JunTCVTestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTCVTestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTCVTestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		mevvl.doOperation();
		JunEdge edge = mevvl.edge();
		JunMEL mel = new JunMEL(body, mevvl.loop(), mevvl.vertex1(), mevvl.vertex2());
		mel.doOperation();
		JunMVE mve = new JunMVE(body, mel.newEdge(), new Jun3dPoint(0, -1, 0));
		mve.doOperation();
		JunNurbsCurve curve = JunNurbsCurve.BezierControlPoints_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(0.5, 0, 0.5), new Jun3dPoint(1, 0, 0) });
		JunMCV mcv = new JunMCV(body, edge, curve);
		mcv.doOperation();
		body.asWireframedOpenGL3dObject().show();
		JunTCV tcv = new JunTCV(body, edge, 1, new Jun3dPoint(0, 0, -1));
		tcv.doOperation();
		body.asWireframedOpenGL3dObject().show();

		return true;
	}

	/**
	 * ManualExample1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean ManualExample1() {
		JunBody body = new JunBody();

		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();

		JunEdge edge = mevvl.newEdge();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, mevvl.loop(), new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body, mevvl.loop(), vertex010, vertex000);
		mel.doOperation();

		JunNurbsCurve curve = JunNurbsCurve.BezierControlPoints_(new Jun3dPoint[] { new Jun3dPoint(1, 0, 0), new Jun3dPoint(0.5, 0.5, -0.5), new Jun3dPoint(0, 1, 0) });
		JunMCV mcv = new JunMCV(body, edge, curve);
		mcv.doOperation();

		body.show();

		JunTCV tcv = new JunTCV(body, edge, 2, new Jun3dPoint(0, 0, 1));
		tcv.doOperation();
		body.show();

		System.out.println("-------- edge");
		System.out.println(tcv.edge());
		System.out.println("-------- number");
		System.out.println(tcv.number());
		System.out.println("-------- vector");
		System.out.println(tcv.vector());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunTCVTestExamples();
	}

}
