package jp.co.sra.jun.topology.geometricoperators;

import java.awt.Point;

import jp.co.sra.jun.geometry.abstracts.JunSurface;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.surfaces.JunNurbsSurface;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;

/**
 * JunTSF class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTSF.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTSF extends JunGeometricOperator {

	protected JunLoopProxy loopProxy;
	protected Point number;
	protected Jun3dPoint vector;

	/**
	 * Create a new instance of <code>JunTSF</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aPoint java.awt.Point
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunTSF(JunBody aBody, JunTopologicalElementOrProxy aLoopOrProxy, Point aPoint, Jun3dPoint aJun3dPoint) {
		super();
		this.body_(aBody);
		this.loop_(aLoopOrProxy);
		this.number_(aPoint);
		this.vector_(aJun3dPoint);
	}

	/**
	 * Create a new instance of <code>JunTSF</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aLoopOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aPoint java.awt.Point
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.geometricoperators.JunTSF
	 * @deprecated since Jun700, 
	 * @category Instance creation
	 */
	public static final JunTSF Body_loop_number_vector_(JunBody aBody, JunTopologicalElementOrProxy aLoopOrProxy, Point aPoint, Jun3dPoint aJun3dPoint) {
		return new JunTSF(aBody, aLoopOrProxy, aPoint, aJun3dPoint);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		loopProxy = null;
		number = null;
		vector = null;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		JunNurbsSurface surface = (JunNurbsSurface) ((this.loop()).basicSurface());
		Jun3dPoint newPoint = surface.controlPointAt_(this.number()).plus_(this.vector());
		surface.controlPointAt_put_(this.number(), newPoint);
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunTSF(this.body(), loopProxy, this.number(), this.vector().negated());
	}

	/**
	 * Answer the receiver's loop.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public JunLoop loop() {
		return (JunLoop) this.getTopologicalElement_(loopProxy);
	}

	/**
	 * Set the receiver's loop.
	 * 
	 * @param aLoopOrProxy jp.co.sra.jun.topology.elements.JunLoop
	 * @category accessing
	 */
	public void loop_(JunTopologicalElementOrProxy aLoopOrProxy) {
		loopProxy = (JunLoopProxy) this.getTopologicalElementProxy_advise_(aLoopOrProxy, loopProxy);
	}

	/**
	 * Answer the receiver's number.
	 * 
	 * @return java.awt.Point
	 * @category accessing
	 */
	public Point number() {
		return number;
	}

	/**
	 * Set the receiver's number.
	 * 
	 * @param aPoint java.awt.Point
	 * @category accessing
	 */
	public void number_(Point aPoint) {
		number = aPoint;
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		JunLoop loop = this.loop();
		if (loop == null) {
			return false;
		}

		JunSurface surface = loop.basicSurface();
		if (surface == null) {
			return false;
		}
		if (surface instanceof JunNurbsSurface == false) {
			return false;
		}

		Point size = ((JunNurbsSurface) surface).controlPointSize();
		if (size.x < this.number().x) {
			return false;
		}
		if (size.y < this.number().y) {
			return false;
		}
		return super.precondition();
	}

	/**
	 * Answer the receiver's vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the receiver's vector.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void vector_(Jun3dPoint aPoint) {
		vector = aPoint;
	}

}
