package jp.co.sra.jun.topology.geometricoperators;

import java.awt.Point;

import jp.co.sra.jun.collections.sequences.JunMatrix;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.surfaces.JunNurbsSurface;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;
import jp.co.sra.jun.topology.euleroperators.JunMVE;

/**
 * JunTSFTestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTSFTestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTSFTestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		mevvl.doOperation();
		JunMEL mel = new JunMEL(body, mevvl.loop(), mevvl.vertex1(), mevvl.vertex2());
		mel.doOperation();
		JunLoop loop = mel.newLoop();
		JunMVE mve = new JunMVE(body, mel.newEdge(), new Jun3dPoint(0, -1, 0));
		mve.doOperation();
		JunMatrix<Jun3dPoint> matrix = new JunMatrix<Jun3dPoint>(3);
		matrix._put(0, 0, new Jun3dPoint(0, 0, 0));
		matrix._put(0, 1, new Jun3dPoint(0.5, 0, 0));
		matrix._put(0, 2, new Jun3dPoint(1, 0, 0));
		matrix._put(1, 0, new Jun3dPoint(0.25, -0.25, 0));
		matrix._put(1, 1, new Jun3dPoint(0.375, -0.375, 1));
		matrix._put(1, 2, new Jun3dPoint(0.5, -0.5, 0));
		matrix._put(2, 0, new Jun3dPoint(0, -0.5, 0));
		matrix._put(2, 1, new Jun3dPoint(0, -0.75, 0));
		matrix._put(2, 2, new Jun3dPoint(0, -1, 0));
		JunNurbsSurface surface = JunNurbsSurface.BezierControlPoints_(matrix);
		JunMSF msf = new JunMSF(body, loop, surface);
		msf.doOperation();
		body.show();

		body = new JunBody();
		mevvl = new JunMEVVL(body, new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		mevvl.doOperation();
		mel = new JunMEL(body, mevvl.loop(), mevvl.vertex1(), mevvl.vertex2());
		mel.doOperation();
		loop = mel.newLoop();
		mve = new JunMVE(body, mel.newEdge(), new Jun3dPoint(0, -1, 0));
		mve.doOperation();
		matrix = new JunMatrix<Jun3dPoint>(3);
		matrix._put(0, 0, new Jun3dPoint(0, 0, 0));
		matrix._put(0, 1, new Jun3dPoint(0.5, 0, 0));
		matrix._put(0, 2, new Jun3dPoint(1, 0, 0));
		matrix._put(1, 0, new Jun3dPoint(0.25, -0.25, 0));
		matrix._put(1, 1, new Jun3dPoint(0.375, -0.375, 1));
		matrix._put(1, 2, new Jun3dPoint(0.5, -0.5, 0));
		matrix._put(2, 0, new Jun3dPoint(0, -0.5, 0));
		matrix._put(2, 1, new Jun3dPoint(0, -0.75, 0));
		matrix._put(2, 2, new Jun3dPoint(0, -1, 0));
		surface = JunNurbsSurface.BezierControlPoints_(matrix);
		msf = new JunMSF(body, loop, surface);
		msf.doOperation();

		// body.show();
		JunTSF tsf = new JunTSF(body, loop, new Point(1, 1), new Jun3dPoint(0, 0, -3));
		tsf.doOperation();
		body.show();

		return true;
	}

	/**
	 * ManualExample1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean ManualExample1() {
		JunBody body = new JunBody();

		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();
		JunLoop loop = mevvl.newLoop();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, loop, new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body, loop, vertex010, vertex000);
		mel.doOperation();

		JunMatrix<Jun3dPoint> matrix = new JunMatrix<Jun3dPoint>(3);
		matrix._put(0, 0, new Jun3dPoint(0, 0, 0));
		matrix._put(0, 1, new Jun3dPoint(0.5, 0, 0));
		matrix._put(0, 2, new Jun3dPoint(1, 0, 0));
		matrix._put(1, 0, new Jun3dPoint(0, 0, 0));
		matrix._put(1, 1, new Jun3dPoint(0.3, 0.3, -0.3));
		matrix._put(1, 2, new Jun3dPoint(0.5, 0.5, -0.5));
		matrix._put(2, 0, new Jun3dPoint(0, 0, 0));
		matrix._put(2, 1, new Jun3dPoint(0, 0.5, 0));
		matrix._put(2, 2, new Jun3dPoint(0, 1, 0));
		JunNurbsSurface surface = JunNurbsSurface.BezierControlPoints_(matrix);

		JunMSF msf = new JunMSF(body, loop, surface);
		msf.doOperation();

		body.show();
		JunTSF tsf = new JunTSF(body, loop, new Point(1, 2), new Jun3dPoint(0, 0, 1));
		tsf.doOperation();
		body.show();

		System.out.println("-------- loop");
		System.out.println(tsf.loop());
		System.out.println("-------- number");
		System.out.println(tsf.number());
		System.out.println("-------- vector");
		System.out.println(tsf.vector());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunTSFTestExamples();
	}

}
