package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGeometricOperator;
import jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunTV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTV.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTV extends JunGeometricOperator {

	protected JunVertexProxy vertexProxy;
	protected Jun3dPoint vector;

	/**
	 * Create a new instance of <code>JunTV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aVertex jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunTV(JunBody aBody, JunTopologicalElementOrProxy aVertex, Jun3dPoint aPoint) {
		super();
		this.body_(aBody);
		this.vertex_(aVertex);
		this.vector_(aPoint);
	}

	/**
	 * Create a new instance of <code>JunTV</code> and initialize it.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aVertex jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.geometricoperators.JunTV
	 * @deprecated since Jun700, using JunTV(JunBody, JunTopologicalElementOrProxy, Jun3dPoint)
	 * @category Instance creation
	 */
	public static final JunTV Body_vertex_vector_(JunBody aBody, JunTopologicalElementOrProxy aVertex, Jun3dPoint aPoint) {
		return new JunTV(aBody, aVertex, aPoint);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		vertexProxy = null;
		vector = null;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		JunVertex vertex = this.vertex();
		vertex.point_(vertex.point().plus_(this.vector()));
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunTV(this.body(), vertexProxy, this.vector().negated());
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#precondition()
	 * @category executing
	 */
	public boolean precondition() {
		return (this.vertex() != null && super.precondition());
	}

	/**
	 * Answer the receiver's vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the receiver's vector.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void vector_(Jun3dPoint aPoint) {
		vector = aPoint;
	}

	/**
	 * Answer the receiver's vector.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunVertex
	 * @category accessing
	 */
	public JunVertex vertex() {
		return (JunVertex) this.getTopologicalElement_(vertexProxy);
	}

	/**
	 * Set the receiver's vector.
	 * 
	 * @param aVertexOrProxy jp.co.sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @category accessing
	 */
	public void vertex_(JunTopologicalElementOrProxy aVertexOrProxy) {
		vertexProxy = (JunVertexProxy) this.getTopologicalElementProxy_advise_(aVertexOrProxy, vertexProxy);
	}

}
