package jp.co.sra.jun.topology.geometricoperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;
import jp.co.sra.jun.topology.euleroperators.JunMVE;

/**
 * JunTVTestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTVTestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunTVTestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();
		JunVertex vertex100 = mevvl.vertex1();
		JunVertex vertex010 = mevvl.vertex2();
		JunMEL mel = new JunMEL(body, mevvl.loop(), mevvl.vertex1(), mevvl.vertex2());
		mel.doOperation();
		JunLoop loop = mel.newLoop();
		JunMVE mve = new JunMVE(body, mel.newEdge(), new Jun3dPoint(0, 0, 1));
		mve.doOperation();
		JunVertex vertex001 = mve.newVertex();
		JunMEV mev = new JunMEV(body, vertex100, loop, new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		JunVertex vertex000 = mev.newVertex();
		new JunMEL(body, loop, vertex000, vertex010).doOperation();
		new JunMEL(body, loop, vertex000, vertex001).doOperation();
		body.show();
		JunTV tv = new JunTV(body, vertex100, new Jun3dPoint(2, 0, 0));
		tv.doOperation();
		body.show();

		return true;
	}

	/**
	 * ManualExample1a:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean ManualExample1a() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, mevvl.loop(), new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body, mev.loop(), vertex010, vertex000);
		mel.doOperation();

		body.show();
		JunTV tv = new JunTV(body, vertex000, new Jun3dPoint(1, 1, 0));
		tv.doOperation();
		body.show();

		return true;
	}

	/**
	 * ManualExample1b:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean ManualExample1b() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex010 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body, vertex100, mevvl.loop(), new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body, mev.loop(), vertex010, vertex000);
		mel.doOperation();

		body.show();
		JunTV tv = new JunTV(body, vertex000, new Jun3dPoint(1, 1, 0));
		tv.doOperation();
		body.show();

		System.out.println("-------- #vector");
		System.out.println(tv.vector());
		System.out.println("-------- #vertex");
		System.out.println(tv.vertex());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunTVTestExamples();
	}

}
