package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.smalltalk.StBlockClosure2;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunADD class
 * 
 *  @author    nisinaka
 *  @created   1998/11/06 (by nisinaka)
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunADD.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunADD extends JunGlobalOperator {

	/** Another body to add. */
	protected JunBody body2;

	/**
	 * Create a new instance of <code>JunADD</code> and initialize it with the specified two bodies.
	 * 
	 * @param aBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunADD(JunBody aBody1, JunBody aBody2) {
		super();
		this.body_(aBody1);
		this.body2_(aBody2);
	}

	/**
	 * Create a new instance of <code>JunADD</code> and initialize it with the specified two bodies.
	 * 
	 * @param aBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.globaloperators.JunADD
	 * @deprecated since Jun700, using JunADD(JunBody, JunBody)
	 * @category Instance creation
	 */
	public static final JunADD Body_body_(JunBody aBody1, JunBody aBody2) {
		return new JunADD(aBody1, aBody2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		body2 = null;
	}

	/**
	 * Answer another body.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public final JunBody body2() {
		return body2;
	}

	/**
	 * Set another body.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public final void body2_(JunBody aBody) {
		body2 = aBody;
	}

	/**
	 * Execute the receiver.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		final JunBody body_ = this.body();
		body2.proxiesAndLoopsDo_(new StBlockClosure2<Void, JunLoopProxy, JunLoop>() {
			public Void value(JunLoopProxy proxy, JunLoop loop) {
				body_.addLoop_as_(loop, proxy);
				return null;
			}
		});
		body2.proxiesAndEdgesDo_(new StBlockClosure2<Void, JunEdgeProxy, JunEdge>() {
			public Void value(JunEdgeProxy proxy, JunEdge edge) {
				body_.addEdge_as_(edge, proxy);
				return null;
			}
		});
		body2.proxiesAndVertexesDo_(new StBlockClosure2<Void, JunVertexProxy, JunVertex>() {
			public Void value(JunVertexProxy proxy, JunVertex vertex) {
				body_.addVertex_as_(vertex, proxy);
				return null;
			}
		});
	}

	/**
	 * Answer the inverse operator of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunSEPARATE(this.body(), this.body2());
	}

}
