package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.surfaces.Jun3dPolygon;
import jp.co.sra.jun.geometry.surfaces.JunPlane;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.support.JunGeometryTransformer;

/**
 * JunMIRROR class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMIRROR.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunMIRROR extends JunGlobalOperator {

	protected Jun3dPoint point;
	protected Jun3dPoint vector;

	/**
	 * Create a new instance of <code>JunMIRROR</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunMIRROR(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2) {
		super();
		this.body_(aJunBody);
		this.point_(aJun3dPoint1);
		this.vector_(aJun3dPoint2);
	}

	/**
	 * Create a new instance of <code>JunMIRROR</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.globaloperators.JunMIRROR
	 * @deprecated since Jun700, using JunMIRROR(JunBody, Jun3dPoint, Jun3dPoint)
	 * @category Instance creation
	 */
	public static final JunMIRROR Body_point_vector_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2) {
		return new JunMIRROR(aJunBody, aJun3dPoint1, aJun3dPoint2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		point = null;
		vector = null;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		final JunPlane mirrorPlane = this.mirrorPlane();
		final Jun3dPoint normalVector = this.vector().unitVector();
		final JunGeometryTransformer transformer = new JunGeometryTransformer(new StBlockClosure1<Jun3dPoint, Jun3dPoint>() {
			public Jun3dPoint value(Jun3dPoint p) {
				double distance = mirrorPlane.distanceFromPoint_(p);
				Jun3dPoint mirrorVector = normalVector.multipliedBy_(distance).multipliedBy_(2);
				Jun3dPoint mirrorPoint1 = p.plus_(mirrorVector);
				Jun3dPoint mirrorPoint2 = p.minus_(mirrorVector);

				if (mirrorPlane.distanceFromPoint_(mirrorPoint1) < mirrorPlane.distanceFromPoint_(mirrorPoint2)) {
					return mirrorPoint1;
				} else {
					return mirrorPoint2;
				}
			}
		});

		this.body().vertexesDo_(new StBlockClosure1<Void, JunVertex>() {
			public Void value(JunVertex vertex) {
				if (vertex.point() != null) {
					vertex.point_(vertex.point().transform_(transformer));
				}
				return null;
			}
		});
		this.body().edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();
				if (curve != null) {
					edge.setCurve_(curve.transform_(transformer));
				}
				return null;
			}
		});
		this.body().loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				Jun3dPolygon surface = (Jun3dPolygon) loop.basicSurface();
				if (surface != null) {
					loop.setSurface_(surface.transform_(transformer));
				}
				return null;
			}
		});

		new JunNEGATE(this.body()).doOperation();
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMIRROR(this.body(), this.point(), this.vector());
	}

	/**
	 * Answer the receiver's mirror plane.
	 * 
	 * @return jp.co.sra.jun.geometry.surfaces.JunPlane
	 * @category accessing
	 */
	public JunPlane mirrorPlane() {
		return JunPlane.On_vertical_(this.point(), new Jun3dLine(Jun3dPoint.Zero(), this.vector()));
	}

	/**
	 * Answer the receiver's point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint point() {
		return this.point;
	}

	/**
	 * Set the receiver's point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void point_(Jun3dPoint aJun3dPoint) {
		this.point = aJun3dPoint;
	}

	/**
	 * Answer the receiver's vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public Jun3dPoint vector() {
		return this.vector;
	}

	/**
	 * Set the receiver's vector.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public void vector_(Jun3dPoint aJun3dPoint) {
		vector = aJun3dPoint;
	}

}
