package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.surfaces.Jun3dPolygon;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;

/**
 * JunMOVE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMOVE.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunMOVE extends JunGlobalOperator {

	protected Jun3dPoint vector;

	/**
	 * Create a new instance of <code>JunMOVE</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category Instance creation
	 */
	public JunMOVE(JunBody aJunBody, Jun3dPoint aJun3dPoint) {
		super();
		this.body_(aJunBody);
		this.vector_(aJun3dPoint);
	}

	/**
	 * Create a new instance of JunMOVE and initialize it with a body.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.topology.globaloperators.JunMOVE
	 * @deprecated since Jun700, using JunMOVE(JunBody, Jun3dPoint)
	 * @category Instance creation
	 */
	public static final JunMOVE Body_vector_(JunBody aJunBody, Jun3dPoint aJun3dPoint) {
		return new JunMOVE(aJunBody, aJun3dPoint);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		vector = null;
	}

	/**
	 * Execute the receiver.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		final Jun3dTransformation translation = Jun3dTransformation.Translate_(this.vector());
		this.body().vertexesDo_(new StBlockClosure1<Void, JunVertex>() {
			public Void value(JunVertex vertex) {
				if (vertex.point() != null) {
					vertex.point_(vertex.point().transform_(translation));
				}
				return null;
			}
		});
		this.body().edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();
				if (curve != null) {
					edge.setCurve_(curve.transform_(translation));
				}
				return null;
			}
		});
		this.body().loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				Jun3dPolygon surface = (Jun3dPolygon) loop.basicSurface();
				if (surface != null) {
					loop.setSurface_(surface.transform_(translation));
				}
				return null;
			}
		});
	}

	/**
	 * Answer the inverse operator of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunMOVE(this.body(), this.vector().negated());
	}

	/**
	 * Answer the vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the vector.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final void vector_(Jun3dPoint aJun3dPoint) {
		vector = aJun3dPoint;
	}

}
