package jp.co.sra.jun.topology.globaloperators;

import java.util.ArrayList;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.surfaces.Jun3dPolygon;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;

/**
 * JunNEGATE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNEGATE.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunNEGATE extends JunGlobalOperator {
	/**
	 * Create a new instance of <code>JunNEGATE</code> and initialize it with the specified body.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunNEGATE(JunBody aBody) {
		super(aBody);
	}

	/**
	 * Create a new instance of JunNEGATE and initialize it with a body.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.globaloperators.JunNEGATE
	 * @deprecated since Jun700, using JunNEGATE(JunBody)
	 * @category Instance creation
	 */
	public static final JunNEGATE Body_(JunBody aBody) {
		return new JunNEGATE(aBody);
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		final ArrayList<JunEdge[]> connections = new ArrayList<JunEdge[]>(body.numberOfEdges());
		body.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				JunEdge rightPrev = edge.prevEdgeWithVertex_(edge.startVertex());
				JunEdge leftPrev = edge.prevEdgeWithVertex_(edge.endVertex());
				JunEdge[] anArray = { edge, rightPrev, leftPrev };
				connections.add(anArray);
				return null;
			}
		});

		int size = connections.size();
		for (int i = 0; i < size; i++) {
			JunEdge[] edgeRightLeft = connections.get(i);
			JunEdge edge = edgeRightLeft[0];
			JunEdge startRightEdge = edgeRightLeft[1];
			JunEdge endLeftEdge = edgeRightLeft[2];
			edge.startRightEdge_(startRightEdge);
			edge.endLeftEdge_(endLeftEdge);

			JunLoop leftLoop = edge.leftLoop();
			JunLoop rightLoop = edge.rightLoop();
			edge.leftLoop_(rightLoop);
			edge.rightLoop_(leftLoop);
		}

		body.edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();
				if (curve != null) {
					edge.setCurve_(curve.reversed());
				}
				return null;
			}
		});
		body.loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				Jun3dPolygon surface = (Jun3dPolygon) loop.basicSurface();
				if (surface != null) {
					loop.setSurface_(surface.reversed());
				}
				return null;
			}
		});
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunNEGATE(this.body());
	}

}
