package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;

/**
 * JunNEGATETestExamples class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNEGATETestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunNEGATETestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();
		JunLoop loop = mevvl.newLoop();
		JunVertex v100 = mevvl.newVertex1();
		JunVertex v010 = mevvl.newVertex2();
		JunMEV mev = new JunMEV(body, v100, loop, new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		JunVertex v000 = mev.newVertex();
		JunMEL mel = new JunMEL(body, loop, v010, v000);
		mel.doOperation();
		body.show();
		JunNEGATE negate = new JunNEGATE(body);
		negate.doOperation();
		negate.body().show();

		JunVertex[] vertexes = body.vertexes();
		for (int i = 0; i < vertexes.length; i++) {
			JunVertex vertex = vertexes[i];
			System.out.println("-------- #vertex");
			System.out.println(vertex);
		}

		return true;
	}

	/**
	 * Example2:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		Jun3dPoint[] points = { new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0), new Jun3dPoint(0, 0, 1) };
		JunBody body = JunBody.Tetrahedron_(points);
		JunNEGATE negate = new JunNEGATE(body);
		negate.doOperation();
		body.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new JunNEGATETestExamples();
	}

}
