package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMCLKPL;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;

/**
 * JunPointSCALETestExamples class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPointSCALETestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunPointSCALETestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		JunBody body = new JunBody();
		JunMEVVL mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 1, 0));
		mevvl.doOperation();

		JunLoop loop1 = mevvl.loop();
		JunVertex v100 = mevvl.newVertex1();
		JunVertex v010 = mevvl.newVertex2();
		JunMEV mev = new JunMEV(body, v100, loop1, new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunVertex v000 = mev.newVertex();
		JunMEL mel = new JunMEL(body, loop1, v010, v000);
		mel.doOperation();
		body = new JunBody();
		mevvl = new JunMEVVL(body, new Jun3dPoint(1, 0, 1), new Jun3dPoint(0, 1, 1));
		mevvl.doOperation();

		JunLoop loop2 = mevvl.loop();
		JunVertex v101 = mevvl.newVertex1();
		JunVertex v011 = mevvl.newVertex2();
		mev = new JunMEV(body, v101, loop2, new Jun3dPoint(0, 0, 1));
		mev.doOperation();

		JunVertex v001 = mev.newVertex();
		mel = new JunMEL(body, loop2, v011, v001);
		mel.doOperation();

		JunMCLKPL mcl = new JunMCLKPL(body, loop1, loop2);
		mcl.doOperation();

		JunPointSCALE scale = new JunPointSCALE(body, new Jun3dPoint(0, 0, 0), 0.5);
		scale.doOperation();
		scale.body().show();

		JunVertex[] vertexes = body.vertexes();

		for (int i = 0; i < vertexes.length; i++) {
			JunVertex vertex = vertexes[i];
			System.out.println("-------- #vertex");
			System.out.println(vertex);
		}

		return true;
	}

	/**
	 * Example2:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		JunBody body;
		JunPointSCALE scale;
		Jun3dPoint p1 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint p2 = new Jun3dPoint(1, 0, 0);
		Jun3dPoint p3 = new Jun3dPoint(0, 1, 0);
		Jun3dPoint p4 = new Jun3dPoint(0, 0, 1);
		Jun3dPoint[] a3dPoint = new Jun3dPoint[4];
		a3dPoint[0] = p1;
		a3dPoint[1] = p2;
		a3dPoint[2] = p3;
		a3dPoint[3] = p4;
		body = JunBody.Tetrahedron_(a3dPoint);
		body.show();

		//	body = JunBody.Tetrahedron_((StArray)StArray.With_with_with_with_(StArray.class,p1,p2,p3,p4));
		scale = new JunPointSCALE(body, p1 /*Jun3dPoint.Zero()*/
		, 2.0d);
		scale.doOperation();
		scale.body().show();

		//	return scale;
		return true;
	}

	/**
	 * Example3:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		JunBody body;
		JunPointSCALE scale;
		Jun3dPoint p1 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint p2 = new Jun3dPoint(1, 0, 0);
		Jun3dPoint p3 = new Jun3dPoint(0, 1, 0);
		Jun3dPoint p4 = new Jun3dPoint(0, 0, 1);
		Jun3dPoint[] a3dPoint = new Jun3dPoint[4];
		a3dPoint[0] = p1;
		a3dPoint[1] = p2;
		a3dPoint[2] = p3;
		a3dPoint[3] = p4;
		body = JunBody.Tetrahedron_(a3dPoint);

		//	body = JunBody.Tetrahedron_((StArray)StArray.With_with_with_with_(StArray.class,p1,p2,p3,p4));
		scale = new JunPointSCALE(body, p1 /*Jun3dPoint.Zero()*/
		, 2.0d);
		body.show();
		scale.doOperation();
		body.show();
		scale.inverse().doOperation();
		body.show();

		return true;
	}

	/**
	 * Example4:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example4() {
		JunBody body;
		JunPointSCALE scale;
		Jun3dPoint p1 = new Jun3dPoint(10, 10, 10);
		Jun3dPoint p2 = new Jun3dPoint(11, 10, 10);
		Jun3dPoint p3 = new Jun3dPoint(10, 11, 10);
		Jun3dPoint p4 = new Jun3dPoint(10, 10, 11);
		Jun3dPoint[] a3dPoint = new Jun3dPoint[4];
		a3dPoint[0] = p1;
		a3dPoint[1] = p2;
		a3dPoint[2] = p3;
		a3dPoint[3] = p4;
		body = JunBody.Tetrahedron_(a3dPoint);

		//	body = JunBody.Tetrahedron_((StArray)StArray.With_with_with_with_(StArray.class,p1,p2,p3,p4));
		scale = new JunPointSCALE(body, p2, 2.0d);
		body.show();
		scale.doOperation();
		body.show();
		scale.inverse().doOperation();
		body.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(java.lang.String[] args) {
		new JunPointSCALETestExamples();
	}

}
