package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.surfaces.Jun3dPolygon;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;

/**
 * JunROTATE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunROTATE.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunROTATE extends JunGlobalOperator {

	protected Jun3dPoint point;
	protected Jun3dPoint vector;
	protected JunAngle angle;

	/**
	 * Create a new instance of <code>JunROTATE</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 * @category Instance creation
	 */
	public JunROTATE(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, double aNumber) {
		super();
		this.body_(aJunBody);
		this.point_(aJun3dPoint1);
		this.vector_(aJun3dPoint2);
		this.angle_(aNumber);
	}

	/**
	 * Create a new instance of <code>JunROTATE</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle jp.co.sra.jun.geometry.basic.JunAngle
	 * @category Instance creation
	 */
	public JunROTATE(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, JunAngle aJunAngle) {
		super();
		this.body_(aJunBody);
		this.point_(aJun3dPoint1);
		this.vector_(aJun3dPoint2);
		this.angle_(aJunAngle);
	}

	/**
	 * Create a new instance of <code>JunROTATE</code> and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 * @return jp.co.sra.jun.topology.globaloperators.JunROTATE
	 * @deprecated since Jun700, using JunROTATE(JunBody, Jun3dPoint, Jun3dPoint, double)
	 * @category Instance creation
	 */
	public static final JunROTATE Body_point_vector_angle_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, double aNumber) {
		return new JunROTATE(aJunBody, aJun3dPoint1, aJun3dPoint2, aNumber);
	}

	/**
	 * Create a new instance of JunROTATE and initialize it.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle jp.co.sra.jun.geometry.basic.JunAngle
	 * @return jp.co.sra.jun.topology.globaloperators.JunROTATE
	 * @deprecated since Jun700, using JunROTATE(JunBody, Jun3dPoint, Jun3dPoint, JunAngle)
	 * @category Instance creation
	 */
	public static final JunROTATE Body_point_vector_angle_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, JunAngle aJunAngle) {
		return new JunROTATE(aJunBody, aJun3dPoint1, aJun3dPoint2, aJunAngle);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		point = null;
		vector = null;
		angle = null;
	}

	/**
	 * Answer the angle.
	 * 
	 * @return double
	 * @category accessing
	 */
	public final JunAngle angle() {
		return angle;
	}

	/**
	 * Set the angle.
	 * 
	 * @param aNumber double
	 * @category accessing
	 */
	public void angle_(double aNumber) {
		angle = JunAngle.FromRad_(aNumber);
	}

	/**
	 * Set the angle.
	 * 
	 * @param anAngle jp.co.sra.jun.geometry.basic.JunAngle
	 * @category accessing
	 */
	public final void angle_(JunAngle anAngle) {
		angle = anAngle;
	}

	/**
	 * Execute the receiver.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		final Jun3dTransformation rotation = Jun3dTransformation.Rotate_around_(this.angle(), new Jun3dLine(this.point(), this.point().plus_(this.vector())));
		this.body().vertexesDo_(new StBlockClosure1<Void, JunVertex>() {
			public Void value(JunVertex vertex) {
				if (vertex.point() != null) {
					vertex.point_(vertex.point().transform_(rotation));
				}
				return null;
			}
		});
		this.body().edgesDo_(new StBlockClosure1<Void, JunEdge>() {
			public Void value(JunEdge edge) {
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();
				if (curve != null) {
					edge.setCurve_(curve.transform_(rotation));
				}
				return null;
			}
		});
		this.body().loopsDo_(new StBlockClosure1<Void, JunLoop>() {
			public Void value(JunLoop loop) {
				Jun3dPolygon surface = (Jun3dPolygon) loop.basicSurface();
				if (surface != null) {
					loop.setSurface_(surface.transform_(rotation));
				}
				return null;
			}
		});
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunROTATE(this.body(), this.point(), this.vector(), this.angle().negated());
	}

	/**
	 * Answer the point.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final Jun3dPoint point() {
		return point;
	}

	/**
	 * Set the point.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final void point_(Jun3dPoint aJun3dPoint) {
		point = aJun3dPoint;
	}

	/**
	 * Answer the vector.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the vector.
	 * 
	 * @param aJun3dPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category accessing
	 */
	public final void vector_(Jun3dPoint aJun3dPoint) {
		vector = aJun3dPoint;
	}

}
