package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunGlobalOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunEdge;
import jp.co.sra.jun.topology.elements.JunEdgeProxy;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunLoopProxy;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.elements.JunVertexProxy;

/**
 * JunSEPARATE class
 * 
 *  @author    nisinaka
 *  @created   1998/11/xx (by nisinaka)
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSEPARATE.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunSEPARATE extends JunGlobalOperator {

	/** Another body to separate. */
	protected JunBody body2;

	/**
	 * Create a new instance of <code>JunSEPARATE</code> and initialize it.
	 * 
	 * @param aBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunSEPARATE(JunBody aBody1, JunBody aBody2) {
		super();
		this.body_(aBody1);
		this.body2_(aBody2);
	}

	/**
	 * Create an JunSEPARATE operator with two bodies.
	 * 
	 * @param aBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.globaloperators.JunSEPARATE
	 * @deprecated since Jun700, using JunSEPARATE(JunBody, JunBody)
	 * @category Instance creation
	 */
	public static final JunSEPARATE Body_body_(JunBody aBody1, JunBody aBody2) {
		return new JunSEPARATE(aBody1, aBody2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		body2 = null;
	}

	/**
	 * Answer another body.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public JunBody body2() {
		return body2;
	}

	/**
	 * Set another body.
	 * 
	 * @param aBody jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public void body2_(JunBody aBody) {
		body2 = aBody;
	}

	/**
	 * Execute the receiver.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		JunLoopProxy[] loopProxies = body2.loopProxies();
		for (int i = 0; i < loopProxies.length; i++) {
			JunLoopProxy proxy = loopProxies[i];
			JunLoop loop = this.body().loopAt_(proxy);
			body2.addLoop_as_(loop, proxy);
			body.removeLoop_(loop);
		}

		JunEdgeProxy[] edgeProxies = body2.edgeProxies();
		for (int i = 0; i < edgeProxies.length; i++) {
			JunEdgeProxy proxy = edgeProxies[i];
			JunEdge edge = this.body().edgeAt_(proxy);
			body2.addEdge_as_(edge, proxy);
			body.removeEdge_(edge);
		}

		JunVertexProxy[] vertexProxies = body2.vertexProxies();
		for (int i = 0; i < vertexProxies.length; i++) {
			JunVertexProxy proxy = vertexProxies[i];
			JunVertex vertex = this.body().vertexAt_(proxy);
			body2.addVertex_as_(vertex, proxy);
			body.removeVertex_(vertex);
		}
	}

	/**
	 * Answer the inverse operator of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.abstracts.JunAbstractOperator
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#inverse()
	 * @category inverting
	 */
	public JunAbstractOperator inverse() {
		return new JunADD(this.body(), this.body2());
	}

}
