package jp.co.sra.jun.topology.globaloperators;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.system.support.JunTestExamples;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.elements.JunLoop;
import jp.co.sra.jun.topology.elements.JunVertex;
import jp.co.sra.jun.topology.euleroperators.JunMEL;
import jp.co.sra.jun.topology.euleroperators.JunMEV;
import jp.co.sra.jun.topology.euleroperators.JunMEVVL;

/**
 * JunSEPARATETestExamples class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSEPARATETestExamples.java,v 9.9 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunSEPARATETestExamples extends JunTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean Example1() {
		Jun3dPoint point1 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint point2 = new Jun3dPoint(-1, -1, -1);
		Jun3dPoint point3 = new Jun3dPoint(1, 1, 1);
		JunBody body1 = JunBody.GlobeLatitudes_longitudes_center_radius_(9, 18, point1, 1.5);
		JunBody body2 = JunBody.CubeOrigin_corner_(point2, point3);
		JunADD add = new JunADD(body1, body2);
		add.doOperation();

		JunSEPARATE separate = new JunSEPARATE(body1, body2);
		separate.doOperation();
		separate.body().show();

		return true;
	}

	/**
	 * Example2:
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean Example2() {
		Jun3dPoint point1 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint point2 = new Jun3dPoint(-1, -1, -1);
		Jun3dPoint point3 = new Jun3dPoint(1, 1, 1);
		JunBody body1 = JunBody.GlobeLatitudes_longitudes_center_radius_(9, 18, point1, 1.5);
		JunBody body2 = JunBody.CubeOrigin_corner_(point2, point3);
		JunADD add = new JunADD(body1, body2);
		add.doOperation();

		JunSEPARATE separate = new JunSEPARATE(body1, body2);
		separate.doOperation();
		separate.inverse().doOperation();
		separate.body().show();

		return true;
	}

	/**
	 * ManualExample1:
	 * 
	 * @return boolean.
	 * @category Examples
	 */
	public static boolean ManualExample1() {
		JunBody body1 = new JunBody();

		JunMEVVL mevvl = new JunMEVVL(body1, new Jun3dPoint(1, 0, 0), new Jun3dPoint(0, 0, 1));
		mevvl.doOperation();

		JunLoop loop1 = mevvl.loop();
		JunVertex vertex100 = mevvl.newVertex1();
		JunVertex vertex001 = mevvl.newVertex2();

		JunMEV mev = new JunMEV(body1, vertex100, loop1, new Jun3dPoint(0, 0, 0));
		mev.doOperation();

		JunVertex vertex000 = mev.newVertex();

		JunMEL mel = new JunMEL(body1, loop1, vertex001, vertex000);
		mel.doOperation();

		JunBody body2 = new JunBody();

		mevvl = new JunMEVVL(body2, new Jun3dPoint(0, 1, 0), new Jun3dPoint(0, 0, 1));
		mevvl.doOperation();

		JunLoop loop2 = mevvl.loop();
		JunVertex vertex010 = mevvl.newVertex1();
		vertex001 = mevvl.newVertex2();

		mev = new JunMEV(body2, vertex010, loop2, new Jun3dPoint(0, 0, 0));
		mev.doOperation();
		vertex000 = mev.newVertex();

		mel = new JunMEL(body2, loop2, vertex001, vertex000);
		mel.doOperation();

		JunADD add = new JunADD(body1, body2);
		add.doOperation();

		body1.show();

		JunSEPARATE separate = new JunSEPARATE(body1, body2);
		separate.doOperation();
		body1.show();
		body2.show();

		System.out.println("-------- body2");
		System.out.println(separate.body2());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 * @category Main
	 */
	public static void main(java.lang.String[] args) {
		new JunSEPARATETestExamples();
	}

}
