package jp.co.sra.jun.topology.graph;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;
import jp.co.sra.smalltalk.StView;

import jp.co.sra.jun.goodies.tables.JunAttributeTable;

/**
 * JunElementalGraphSettings class
 * 
 *  @author    nisinaka
 *  @created   2006/04/20 (by nisinaka)
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun593 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunElementalGraphSettings.java,v 9.10 2010/03/09 09:17:55 nisinaka Exp $
 */
public class JunElementalGraphSettings extends JunElementalStuffSettings<JunElementalGraph> {

	protected StValueHolder<String> labelStringHolder;
	protected StValueHolder<Color> selectionBorderColorHolder;
	protected StValueHolder<Integer> selectionBorderWidthHolder;
	protected StValueHolder<StSymbol> arrangeFormatHolder;
	protected StValueHolder<Integer> arrangeForestIntervalHolder;
	protected StValueHolder<Integer> arrangeForestMarginHolder;
	protected StValueHolder<Integer> arrangeConcentricRadiusHolder;
	protected StValueHolder<Float> arrangeConcentricSpiralHolder;
	protected StValueHolder<Boolean> arrangeConcentricGradationHolder;
	protected StValueHolder<StSymbol> arrangeAttributeSymbolHolder;

	/**
	 * Create a new instance of JunElementalGraphSettings and initialize it.
	 *
	 * @category Instance creation
	 */
	public JunElementalGraphSettings() {
		super();
	}

	/**
	 * Create a new instance of JunElementalGraphSettings and initialize it.
	 *
	 * @param anElementalGraph jp.co.sra.jun.topology.graph.JunElementalGraph
	 * @category Instance creation
	 */
	public JunElementalGraphSettings(JunElementalGraph anElementalGraph) {
		this.stuff_(anElementalGraph);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		StBlockClosure0<Void> updateSettingsBlock = new StBlockClosure0<Void>() {
			public Void value() {
				updateSettings();
				return null;
			}
		};

		labelStringHolder = new StValueHolder<String>(JunElementalGraph.DefaultLabelString());
		labelStringHolder.compute_(updateSettingsBlock);

		selectionBorderColorHolder = new StValueHolder<Color>(JunElementalGraph.DefaultSelectionBorderColor());
		selectionBorderColorHolder.compute_(updateSettingsBlock);

		selectionBorderWidthHolder = new StValueHolder<Integer>(JunElementalGraph.DefaultSelectionBorderWidth());
		selectionBorderWidthHolder.compute_(updateSettingsBlock);

		arrangeFormatHolder = new StValueHolder<StSymbol>(JunElementalGraph.DefaultArrangeFormat());
		arrangeFormatHolder.compute_(updateSettingsBlock);

		arrangeForestIntervalHolder = new StValueHolder<Integer>(JunElementalGraph.DefaultArrangeForestInterval());
		arrangeForestIntervalHolder.compute_(updateSettingsBlock);

		arrangeForestMarginHolder = new StValueHolder<Integer>(JunElementalGraph.DefaultArrangeForestMargin());
		arrangeForestMarginHolder.compute_(updateSettingsBlock);

		arrangeConcentricRadiusHolder = new StValueHolder<Integer>(JunElementalGraph.DefaultArrangeConcentricRadius());
		arrangeConcentricRadiusHolder.compute_(updateSettingsBlock);

		arrangeConcentricSpiralHolder = new StValueHolder<Float>(JunElementalGraph.DefaultArrangeConcentricSpiral());
		arrangeConcentricSpiralHolder.compute_(updateSettingsBlock);

		arrangeConcentricGradationHolder = new StValueHolder<Boolean>(JunElementalGraph.DefaultArrangeConcentricGradation());
		arrangeConcentricGradationHolder.compute_(updateSettingsBlock);

		arrangeAttributeSymbolHolder = new StValueHolder<StSymbol>(JunElementalGraph.DefaultArrangeAttributeSymbol());
		arrangeAttributeSymbolHolder.compute_(updateSettingsBlock);
	}

	/**
	 * Answer the current settings.
	 * 
	 * @return java.util.Map
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#currentSettings()
	 * @category accessing
	 */
	public Map<StSymbol, Object> currentSettings() {
		HashMap<StSymbol, Object> aMap = new HashMap<StSymbol, Object>(10);
		aMap.put($("labelString"), this.labelStringHolder().value());
		aMap.put($("selectionBorderColor"), this.selectionBorderColorHolder().value());
		aMap.put($("selectionBorderWidth"), this.selectionBorderWidthHolder().value());
		aMap.put($("arrangeFormat"), this.arrangeFormatHolder().value());
		aMap.put($("arrangeForestInterval"), this.arrangeForestIntervalHolder().value());
		aMap.put($("arrangeForestMargin"), this.arrangeForestMarginHolder().value());
		aMap.put($("arrangeConcentricRadius"), this.arrangeConcentricRadiusHolder().value());
		aMap.put($("arrangeConcentricSpiral"), this.arrangeConcentricSpiralHolder().value());
		aMap.put($("arrangeConcentricGradation"), this.arrangeConcentricGradationHolder().value());
		aMap.put($("arrangeAttributeSymbol"), this.arrangeAttributeSymbolHolder().value());
		return aMap;
	}

	/**
	 * Answer the stuff settings.
	 * 
	 * @return java.util.Map
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#currentSettings()
	 * @category accessing
	 */
	public Map<StSymbol, Object> stuffSettings() {
		HashMap<StSymbol, Object> aMap = new HashMap<StSymbol, Object>(10);
		if (this.stuff() == null) {
			JunAttributeTable<StSymbol, Object> attributes = this.defaultAttributes();
			aMap.put($("labelStringHolder"), attributes.at_($("labelString")));
			aMap.put($("selectionBorderColorHolder"), attributes.at_($("selectionBorderColor")));
			aMap.put($("selectionBorderWidthHolder"), attributes.at_($("selectionBorderWidth")));
			aMap.put($("arrangeFormatHolder"), attributes.at_($("arrangeFormat")));
			aMap.put($("arrangeForestIntervalHolder"), attributes.at_($("arrangeForestInterval")));
			aMap.put($("arrangeForestMarginHolder"), attributes.at_($("arrangeForestMargin")));
			aMap.put($("arrangeConcentricRadiusHolder"), attributes.at_($("arrangeConcentricRadius")));
			aMap.put($("arrangeConcentricSpiralHolder"), attributes.at_($("arrangeConcentricSpiral")));
			aMap.put($("arrangeConcentricGradationHolder"), attributes.at_($("arrangeConcentricGradation")));
			aMap.put($("arrangeAttributeSymbolHolder"), attributes.at_($("arrangeAttributeSymbol")));
		} else {
			JunElementalGraph aGraph = this.stuff();
			aMap.put($("labelStringHolder"), aGraph.labelString());
			aMap.put($("selectionBorderColorHolder"), aGraph.selectionBorderColor());
			aMap.put($("selectionBorderWidthHolder"), new Integer(aGraph.selectionBorderWidth()));
			aMap.put($("arrangeFormatHolder"), aGraph.arrangeFormat());
			aMap.put($("arrangeForestIntervalHolder"), new Integer(aGraph.arrangeForestInterval()));
			aMap.put($("arrangeForestMarginHolder"), new Integer(aGraph.arrangeForestMargin()));
			aMap.put($("arrangeConcentricRadiusHolder"), new Integer(aGraph.arrangeConcentricRadius()));
			aMap.put($("arrangeConcentricSpiralHolder"), new Float(aGraph.arrangeConcentricSpiral()));
			aMap.put($("arrangeConcentricGradationHolder"), aGraph.arrangeConcentricGradation());
			aMap.put($("arrangeAttributeSymbolHolder"), aGraph.arrangeAttributeSymbol());
		}
		return aMap;
	}

	/**
	 * Answer the value holder for the label string.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<String> labelStringHolder() {
		return labelStringHolder;
	}

	/**
	 * Answer the value holder for the selection border color.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Color> selectionBorderColorHolder() {
		return selectionBorderColorHolder;
	}

	/**
	 * Answer the value holder for the selection border width.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Integer> selectionBorderWidthHolder() {
		return selectionBorderWidthHolder;
	}

	/**
	 * Answer the value holder for the arrange format.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<StSymbol> arrangeFormatHolder() {
		return arrangeFormatHolder;
	}

	/**
	 * Answer the value holder for the arrange forest interval.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Integer> arrangeForestIntervalHolder() {
		return arrangeForestIntervalHolder;
	}

	/**
	 * Answer the value holder for the arrange forest margin.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Integer> arrangeForestMarginHolder() {
		return arrangeForestMarginHolder;
	}

	/**
	 * Answer the value holder for the arrange concentric radius.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Integer> arrangeConcentricRadiusHolder() {
		return arrangeConcentricRadiusHolder;
	}

	/**
	 * Answer the value holder for the arrange concentric spiral.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Float> arrangeConcentricSpiralHolder() {
		return arrangeConcentricSpiralHolder;
	}

	/**
	 * Answer the value holder for the arrange concentric gradation.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<Boolean> arrangeConcentricGradationHolder() {
		return arrangeConcentricGradationHolder;
	}

	/**
	 * Answer the value holder for the arrange attribute symbol.
	 * 
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category aspects
	 */
	protected StValueHolder<StSymbol> arrangeAttributeSymbolHolder() {
		return arrangeAttributeSymbolHolder;
	}

	/**
	 * Create my default view.
	 * 
	 * @return jp.co.sra.smalltalk.StView
	 * @see jp.co.sra.smalltalk.StApplicationModel#defaultView()
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunElementalGraphSettingsViewAwt(this);
		} else {
			return new JunElementalGraphSettingsViewSwing(this);
		}
	}

	/**
	 * Answer the default attribute table.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAttributeTable
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#defaultAttributes()
	 * @category defaults
	 */
	protected JunAttributeTable<StSymbol, Object> defaultAttributes() {
		return JunElementalGraph.DefaultAttributes;
	}

	/**
	 * Answer the window title.
	 * 
	 * @return java.lang.String
	 * @see jp.co.sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		String aString = $String("Graph Settings");
		if (this.stuff() == null) {
			aString += " (==" + $String("Global") + "==)";
		} else {
			aString += " " + this.stuff().displayString();
		}
		return aString;
	}

	/**
	 * Action for the selection border color button.
	 * 
	 * @category actions
	 */
	protected void selectionBorderColorButtonPressed() {
		this._chooseColorFor(this.selectionBorderColorHolder());
	}

	/**
	 * Copy settings from current to stuff.
	 * 
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#settingsFromCurrentToStuff()
	 * @category private
	 */
	protected void settingsFromCurrentToStuff() {
		Map<StSymbol, Object> aMap = this.currentSettings();
		if (this.stuff() == null) {
			JunAttributeTable<StSymbol, Object> attributes = this.defaultAttributes();
			attributes.at_put_($("labelString"), aMap.get($("labelString")));
			attributes.at_put_($("selectionBorderColor"), aMap.get($("selectionBorderColor")));
			attributes.at_put_($("selectionBorderWidth"), aMap.get($("selectionBorderWidth")));
			attributes.at_put_($("arrangeFormat"), aMap.get($("arrangeFormat")));
			attributes.at_put_($("arrangeForestInterval"), aMap.get($("arrangeForestInterval")));
			attributes.at_put_($("arrangeForestMargin"), aMap.get($("arrangeForestMargin")));
			attributes.at_put_($("arrangeConcentricRadius"), aMap.get($("arrangeConcentricRadius")));
			attributes.at_put_($("arrangeConcentricSpiral"), aMap.get($("arrangeConcentricSpiral")));
			attributes.at_put_($("arrangeConcentricGradation"), aMap.get($("arrangeConcentricGradation")));
			attributes.at_put_($("arrangeAttributeSymbol"), aMap.get($("arrangeAttributeSymbol")));
		} else {
			JunElementalGraph aGraph = this.stuff();
			aGraph.labelString_((String) aMap.get($("labelString")));
			aGraph.selectionBorderColor_((Color) aMap.get($("selectionBorderColor")));
			aGraph.selectionBorderWidth_((Integer) aMap.get($("selectionBorderWidth")));
			aGraph.arrangeFormat_((StSymbol) aMap.get($("arrangeFormat")));
			aGraph.arrangeForestInterval_((Integer) aMap.get($("arrangeForestInterval")));
			aGraph.arrangeForestMargin_((Integer) aMap.get($("arrangeForestMargin")));
			aGraph.arrangeConcentricRadius_((Integer) aMap.get($("arrangeConcentricRadius")));
			aGraph.arrangeConcentricSpiral_((Float) aMap.get($("arrangeConcentricSpiral")));
			aGraph.arrangeConcentricGradation_((Boolean) aMap.get($("arrangeConcentricGradation")));
			aGraph.arrangeAttributeSymbol_((StSymbol) aMap.get($("arrangeAttributeSymbol")));
		}
	}

	/**
	 * Copy settings from stuff to current.
	 * 
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#settingsFromStuffToCurrent()
	 * @category private
	 */
	protected void settingsFromStuffToCurrent() {
		Map<StSymbol, Object> aMap = this.stuffSettings();
		this.labelStringHolder().setValue_((String) aMap.get($("labelStringHolder")));
		this.selectionBorderColorHolder().setValue_((Color) aMap.get($("selectionBorderColorHolder")));
		this.selectionBorderWidthHolder().setValue_((Integer) aMap.get($("selectionBorderWidthHolder")));
		this.arrangeFormatHolder().setValue_((StSymbol) aMap.get($("arrangeFormatHolder")));
		this.arrangeForestIntervalHolder().setValue_((Integer) aMap.get($("arrangeForestIntervalHolder")));
		this.arrangeForestMarginHolder().setValue_((Integer) aMap.get($("arrangeForestMarginHolder")));
		this.arrangeConcentricRadiusHolder().setValue_((Integer) aMap.get($("arrangeConcentricRadiusHolder")));
		this.arrangeConcentricSpiralHolder().setValue_((Float) aMap.get($("arrangeConcentricSpiralHolder")));
		this.arrangeConcentricGradationHolder().setValue_((Boolean) aMap.get($("arrangeConcentricGradationHolder")));
		this.arrangeAttributeSymbolHolder().setValue_((StSymbol) aMap.get($("arrangeAttributeSymbolHolder")));

		settingsHolder.value_(this.stuffSettings());
		this.updateWidgets();
	}

	/**
	 * Reset the default attributes.
	 * 
	 * @see jp.co.sra.jun.topology.graph.JunElementalStuffSettings#resetDefaultAttributes()
	 * @category private
	 */
	protected void resetDefaultAttributes() {
		JunAttributeTable<StSymbol, Object> attributes = this.defaultAttributes();
		attributes.at_put_($("labelString"), JunElementalGraph.DefaultLabelString());
		attributes.at_put_($("selectionBorderColor"), JunElementalGraph.DefaultSelectionBorderColor());
		attributes.at_put_($("selectionBorderWidth"), new Integer(JunElementalGraph.DefaultSelectionBorderWidth()));
		attributes.at_put_($("arrangeFormat"), JunElementalGraph.DefaultArrangeFormat());
		attributes.at_put_($("arrangeForestInterval"), new Integer(JunElementalGraph.DefaultArrangeForestInterval()));
		attributes.at_put_($("arrangeForestMargin"), new Integer(JunElementalGraph.DefaultArrangeForestMargin()));
		attributes.at_put_($("arrangeConcentricRadius"), new Integer(JunElementalGraph.DefaultArrangeConcentricRadius()));
		attributes.at_put_($("arrangeConcentricSpiral"), new Float(JunElementalGraph.DefaultArrangeConcentricSpiral()));
		attributes.at_put_($("arrangeConcentricGradation"), JunElementalGraph.DefaultArrangeConcentricGradation());
		attributes.at_put_($("arrangeAttributeSymbol"), JunElementalGraph.DefaultArrangeAttributeSymbol());
	}

}
