package jp.co.sra.jun.topology.graph;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StBlockClosure2;
import jp.co.sra.smalltalk.StSymbol;
import jp.co.sra.smalltalk.StValueHolder;

import jp.co.sra.jun.goodies.colors.JunColorChoiceDialog;
import jp.co.sra.jun.goodies.colors.JunColorChoiceHBS;
import jp.co.sra.jun.goodies.colors.JunColorChoiceModel;
import jp.co.sra.jun.goodies.tables.JunAttributeTable;
import jp.co.sra.jun.system.framework.JunApplicationModel;
import jp.co.sra.jun.system.framework.JunDialog;

/**
 * JunElementalStuffSettings class
 * 
 *  @author    nisinaka
 *  @created   2006/04/17 (by nisinaka)
 *  @updated   2008/04/21 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun598 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunElementalStuffSettings.java,v 9.11 2010/03/09 09:17:55 nisinaka Exp $
 */
public abstract class JunElementalStuffSettings<E extends JunElementalStuff> extends JunApplicationModel {

	protected E elementalStuff;
	protected StValueHolder<Map<StSymbol, Object>> settingsHolder;
	protected StBlockClosure2<Void, Map<StSymbol, Object>, E> computeBlock;

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.smalltalk.StApplicationModel#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		elementalStuff = null;
		settingsHolder = new StValueHolder<Map<StSymbol, Object>>(new HashMap<StSymbol, Object>());
		computeBlock = null;
	}

	/**
	 * Answer my current elemental stuff.
	 * 
	 * @return jp.co.sra.jun.topology.graph.JunElementalStuff
	 * @category accessing
	 */
	public E stuff() {
		return elementalStuff;
	}

	/**
	 * Set my new elemental stuff.
	 * 
	 * @param aStuff jp.co.sra.jun.topology.graph.JunElementalStuff
	 * @category accessing
	 */
	public void stuff_(E aStuff) {
		elementalStuff = aStuff;

		this.settingsFromStuffToCurrent();
		this.updateWindowTitle();
	}

	/**
	 * Answer the current settings.
	 * 
	 * @return java.util.Map
	 * @category accessing
	 */
	public Map<StSymbol, Object> currentSettings() {
		return new HashMap<StSymbol, Object>();
	}

	/**
	 * Answer the stuff settings.
	 * 
	 * @return java.util.Map
	 * @category accessing
	 */
	public Map<StSymbol, Object> stuffSettings() {
		return new HashMap<StSymbol, Object>();
	}

	/**
	 * Set the compute block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @category constructing
	 */
	public void compute_(StBlockClosure2<Void, Map<StSymbol, Object>, E> aBlock) {
		computeBlock = aBlock;
	}

	/**
	 * Set the compute block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category constructing
	 */
	public void compute_(final StBlockClosure1<Void, Map<StSymbol, Object>> aBlock) {
		this.compute_(new StBlockClosure2<Void, Map<StSymbol, Object>, E>() {
			public Void value(Map<StSymbol, Object> table, E stuff) {
				aBlock.value(table);
				return null;
			}
		});
	}

	/**
	 * Set the compute block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @category constructing
	 */
	public void compute_(final StBlockClosure0<Void> aBlock) {
		this.compute_(new StBlockClosure2<Void, Map<StSymbol, Object>, E>() {
			public Void value(Map<StSymbol, Object> table, E stuff) {
				aBlock.value();
				return null;
			}
		});
	}

	/**
	 * Set the compute block.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category constructing
	 * @deprecated since Jun701, use Generics.
	 */
	public void compute_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		this.compute_(new StBlockClosure2<Void, Map<StSymbol, Object>, E>() {
			public Void value(Map<StSymbol, Object> table, E stuff) {
				switch (aBlock.numArgs()) {
					case 0:
						aBlock.value();
						break;
					case 1:
						aBlock.value_(table);
						break;
					case 2:
						aBlock.value_value_(table, stuff);
						break;
				}
				return null;
			}
		});
	}

	/**
	 * Answer the default attribute table.
	 * 
	 * @return jp.co.sra.jun.goodies.tables.JunAttributeTable
	 * @category defaults
	 */
	protected abstract JunAttributeTable<StSymbol, Object> defaultAttributes();

	/**
	 * Action for the default button.
	 * 
	 * @category actions
	 */
	protected void defaultButtonPressed() {
		if (JunDialog.Confirm_($String("Really default?"), true) == false) {
			return;
		}

		if (this.stuff() != null) {
			this.stuff().resetAttributesByDefaults();
		} else {
			this.resetDefaultAttributes();
		}
		this.settingsFromStuffToCurrent();
		this.updateSettings();
	}

	/**
	 * Update the settings.
	 * 
	 * @category private
	 */
	protected void updateSettings() {
		Map<StSymbol, Object> newSettings = this.currentSettings();
		Map<StSymbol, Object> oldSettings = settingsHolder.value();
		if (newSettings.equals(oldSettings)) {
			return;
		}

		settingsHolder.value_(newSettings);
		this.updateWidgets();
		this.settingsFromCurrentToStuff();
		if (computeBlock != null) {
			computeBlock.value(settingsHolder.value(), this.stuff());
		}
	}

	/**
	 * Update the widgets.
	 * 
	 * @category private
	 */
	protected void updateWidgets() {
		this.changed_($("widgets"));
	}

	/**
	 * Copy settings from current to stuff.
	 * 
	 * @category private
	 */
	protected abstract void settingsFromCurrentToStuff();

	/**
	 * Copy settings from stuff to current.
	 * 
	 * @category private
	 */
	protected abstract void settingsFromStuffToCurrent();

	/**
	 * Reset the default attributes.
	 * 
	 * @category private
	 */
	protected abstract void resetDefaultAttributes();

	/**
	 * Choose a color.
	 * 
	 * @param colorHolder jp.co.sra.smalltalk.StValueHolder
	 * @category private
	 */
	protected void _chooseColorFor(StValueHolder<Color> colorHolder) {
		Color oldColor = colorHolder.value();

		JunColorChoiceModel colorChoiceModel = new JunColorChoiceHBS();
		colorChoiceModel.color_(oldColor);
		Color newColor = JunColorChoiceDialog.Request_colorChoice_($String("Select a color."), colorChoiceModel);
		if (newColor == null || newColor.equals(oldColor)) {
			return;
		}

		colorHolder.value_(newColor);
	}

}
