package jp.co.sra.jun.topology.setoperators;

import java.util.ArrayList;

import jp.co.sra.jun.topology.abstracts.JunAbstractOperator;
import jp.co.sra.jun.topology.abstracts.JunSetOperator;
import jp.co.sra.jun.topology.elements.JunBody;
import jp.co.sra.jun.topology.globaloperators.JunNEGATE;

/**
 * JunDIFF class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   2008/05/22 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDIFF.java,v 9.9 2010/03/09 09:17:57 nisinaka Exp $
 */
public class JunDIFF extends JunSetOperator {

	/** Another JunBody on which the receiver operate. */
	protected JunBody body2;

	/**
	 * Create a new instance of <code>JunDIFF</code> and initialize it with the specified two bodies.
	 * 
	 * @param aBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @category Instance creation
	 */
	public JunDIFF(JunBody aBody1, JunBody aBody2) {
		super();
		this.body_(aBody1);
		this.body2_(aBody2);
	}

	/**
	 * Create an instance of JunDIFF with two bodies.
	 * 
	 * @param aJunBody1 jp.co.sra.jun.topology.elements.JunBody
	 * @param aJunBody2 jp.co.sra.jun.topology.elements.JunBody
	 * @return jp.co.sra.jun.topology.setoperators.JunDIFF
	 * @deprecated since Jun700, using JunDiff(JunBody, JunBody)
	 * @category Instance creation
	 */
	public static final JunDIFF Body_body_(JunBody aJunBody1, JunBody aJunBody2) {
		return new JunDIFF(aJunBody1, aJunBody2);
	}

	/**
	 * Initialize the receiver when created.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		body2 = null;
	}

	/**
	 * Answer the second body of the receiver.
	 * 
	 * @return jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public JunBody body2() {
		return body2;
	}

	/**
	 * Set the second body of the receiver.
	 * 
	 * @param aJunBody jp.co.sra.jun.topology.elements.JunBody
	 * @category accessing
	 */
	public void body2_(JunBody aJunBody) {
		body2 = aJunBody;
	}

	/**
	 * Execute the operation.
	 * 
	 * @see jp.co.sra.jun.topology.abstracts.JunAbstractOperator#execute()
	 * @category executing
	 */
	public void execute() {
		JunNEGATE negate1 = new JunNEGATE(this.body());
		negate1.doOperation();

		JunUNION union = new JunUNION(this.body(), this.body2());
		union.doOperation();

		JunNEGATE negate2 = new JunNEGATE(this.body());
		negate2.doOperation();

		int size = union.suboperators().size();
		suboperators = new ArrayList<JunAbstractOperator>(size + 2);
		suboperators.add(negate1);
		for (int index = 0; index < size; index++) {
			suboperators.add(union.suboperators().get(index));
		}
		suboperators.add(negate2);
	}

}
