package jp.co.sra.jun.topology.support;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure;
import jp.co.sra.smalltalk.StBlockClosure1;

import jp.co.sra.jun.collections.sequences.JunDoubleMatrix;
import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunPoint;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;

/**
 * JunGeometryTransformer class
 * 
 *  @author    nisinaka
 *  @created   1998/12/02 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @updated   2008/05/26 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Jun697 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunGeometryTransformer.java,v 9.9 2010/03/09 09:17:58 nisinaka Exp $
 */
public class JunGeometryTransformer extends Jun3dTransformation {

	/** The block closure to be used for a transformation. */
	protected StBlockClosure1<Jun3dPoint, Jun3dPoint> block;

	/**
	 * Create a new instance of <code>JunGeometryTransformer</code> and initialize it with the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return jp.co.sra.jun.topology.support.JunGeometryTransformer
	 * @category Instance creation
	 */
	public static JunGeometryTransformer Block_(StBlockClosure1<Jun3dPoint, Jun3dPoint> aBlock) {
		return new JunGeometryTransformer(aBlock);
	}

	/**
	 * Create a new instance of <code>JunGeometryTransformer</code> and initialize it with the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.topology.support.JunGeometryTransformer
	 * @category Instance creation
	 * @deprecated since Jun701, use Generics.
	 */
	public static JunGeometryTransformer Block_(final StBlockClosure<?, ?, ?, ?> aBlock) {
		return new JunGeometryTransformer(new StBlockClosure1<Jun3dPoint, Jun3dPoint>() {
			public Jun3dPoint value(Jun3dPoint aPoint) {
				return (Jun3dPoint) aBlock.value_(aPoint);
			}
		});
	}

	/**
	 * Create a new instance of <code>JunGeometryTransformer</code> and initialize it with the StBlockClosure.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category Instance creation
	 */
	public JunGeometryTransformer(StBlockClosure1<Jun3dPoint, Jun3dPoint> aBlock) {
		this.block_(aBlock);
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @see jp.co.sra.jun.system.framework.JunAbstractObject#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		block = null;
	}

	/**
	 * Apply this transformation to aPoint and return the result.
	 * 
	 * @param aPoint jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 * @category applying transformation
	 */
	public Jun3dPoint applyToPoint_(JunPoint aPoint) {
		return block.value(Jun3dPoint.Coerce_(aPoint));
	}

	/**
	 * Convert this transformation as an array.
	 * 
	 * @return double[]
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @see jp.co.sra.jun.geometry.transformations.Jun3dTransformation#asArray()
	 * @category converting
	 */
	public double[] asArray() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Convert this transformation as a matrix.
	 * 
	 * @return jp.co.sra.jun.collections.sequences.JunDoubleMatrix
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @see jp.co.sra.jun.geometry.transformations.Jun3dTransformation#asMatrix()
	 * @category converting
	 */
	public JunDoubleMatrix asMatrix() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Answer true if aTransformation is equal to this point, otherwise false.
	 * 
	 * @param anObject java.lang.Object
	 * @return boolean
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @see jp.co.sra.jun.geometry.transformations.Jun3dTransformation#equals(java.lang.Object)
	 * @category comparing
	 */
	public boolean equals(Object anObject) {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Set the instance variable "block".
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category private
	 */
	private void block_(StBlockClosure1<Jun3dPoint, Jun3dPoint> aBlock) {
		block = aBlock;
	}

}
