package jp.co.sra.jun.vrml.field.mf;

import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.jun.vrml.field.abstracts.JunVrmlMFField;

/**
 * JunVrmlMFInt32Field class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/27 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlMFInt32Field.java,v 9.9 2010/03/09 09:18:00 nisinaka Exp $
 */
public class JunVrmlMFInt32Field extends JunVrmlMFField {

	protected int[] value;

	/**
	 * Create a new instance of JunVrmlMFInt32Field and initialize it.
	 * 
	 * @param numbers int[]
	 */
	public JunVrmlMFInt32Field(int[] numbers) {
		super();
		this.value_(numbers);
	}

	/**
	 * Create a JunVrmlMFInt32Field and initialize it.
	 * 
	 * @param aCollection java.util.Collection
	 */
	public JunVrmlMFInt32Field(Collection<Number> aCollection) {
		this.value_(_ConvertToIntegerArray(aCollection));
	}

	/**
	 * Create a JunVrmlMFInt32Field and initialize it.
	 * 
	 * @param anObject java.lang.Object
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlMFInt32Field(Object anObject) {
		this((Collection<Number>) anObject);
	}

	/**
	 * Answer my value.
	 * 
	 * @return int[]
	 */
	public int[] value() {
		return value;
	}

	/**
	 * Set my value.
	 * 
	 * @param numbers int[]
	 */
	public void value_(int[] numbers) {
		value = numbers;
	}

	/**
	 * Convert the value to a point array.
	 * 
	 * @return int[][]
	 */
	public int[][] asPointArray() {
		ArrayList<int[]> collection = new ArrayList<int[]>();
		int lastIndex = 0;
		for (int i = 0; i < value.length; i++) {
			if (value[i] < 0) {
				int size = i - lastIndex;
				int[] array = new int[size];
				for (int j = 0; j < size; j++) {
					array[j] = value[lastIndex + j];
				}

				collection.add(array);
				lastIndex = i + 1;
			}
		}

		if (lastIndex < value.length) {
			int size = value.length - lastIndex;
			int[] array = new int[size];
			for (int j = 0; j < size; j++) {
				array[j] = value[lastIndex + j];
			}

			collection.add(array);
		}

		return collection.toArray(new int[collection.size()][]);
	}

}
