package jp.co.sra.jun.vrml.field.mf;

import java.util.ArrayList;
import java.util.Collection;

import jp.co.sra.jun.vrml.field.abstracts.JunVrmlMFField;

/**
 * JunVrmlMFLongField class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/27 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlMFLongField.java,v 9.9 2010/03/09 09:18:00 nisinaka Exp $
 */
public class JunVrmlMFLongField extends JunVrmlMFField {

	protected long[] value;

	/**
	 * Create a new instance of JunVrmlMFLongField and initialize it.
	 * 
	 * @param numbers long[]
	 */
	public JunVrmlMFLongField(long[] numbers) {
		super();
		this.value_(numbers);
	}

	/**
	 * Create a JunVrmlMFLongField and initialize it.
	 * 
	 * @param aCollection java.util.Collection
	 */
	public JunVrmlMFLongField(Collection<Number> aCollection) {
		this.value_(_ConvertToLongArray(aCollection));
	}

	/**
	 * Create a JunVrmlMFLongField and initialize it.
	 * 
	 * @param anObject java.lang.Object
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlMFLongField(Object anObject) {
		this((Collection<Number>) anObject);
	}

	/**
	 * Answer my value.
	 * 
	 * @return long[]
	 */
	public long[] value() {
		return value;
	}

	/**
	 * Set my value.
	 * 
	 * @param numbers long[]
	 */
	public void value_(long[] numbers) {
		value = numbers;
	}

	/**
	 * Convert the value to a point array.
	 * 
	 * @return long[][]
	 */
	public long[][] asPointArray() {
		ArrayList<long[]> collection = new ArrayList<long[]>();
		int lastIndex = 0;
		for (int i = 0; i < value.length; i++) {
			if (value[i] < 0) {
				int size = i - lastIndex;
				long[] array = new long[size];
				for (int j = 0; j < size; j++) {
					array[j] = value[lastIndex + j];
				}

				collection.add(array);
				lastIndex = i + 1;
			}
		}

		if (lastIndex < value.length) {
			int size = value.length - lastIndex;
			long[] array = new long[size];
			for (int j = 0; j < size; j++) {
				array[j] = value[lastIndex + j];
			}

			collection.add(array);
		}

		return collection.toArray(new long[collection.size()][]);
	}
}
