package jp.co.sra.jun.vrml.field.sf;

import java.util.Collection;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.geometry.basic.JunAngle;
import jp.co.sra.jun.geometry.curves.Jun3dLine;
import jp.co.sra.jun.geometry.transformations.Jun3dTransformation;
import jp.co.sra.jun.vrml.field.abstracts.JunVrmlSFField;

/**
 * JunVrmlSFRotationField class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/04/17 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSFRotationField.java,v 9.9 2010/03/09 09:18:01 nisinaka Exp $
 */
public class JunVrmlSFRotationField extends JunVrmlSFField {

	protected float[] value;

	/**
	 * Create a new instance of JunVrmlSFRotationField and initialize it.
	 * 
	 * @param numbers float[]
	 */
	public JunVrmlSFRotationField(float[] numbers) {
		this.value_(numbers);
	}

	/**
	 * Create a JunVrmlSFRotationField and initialize it.
	 * 
	 * @param aCollection java.util.Collection
	 */
	public JunVrmlSFRotationField(Collection<Number> aCollection) {
		this.value_(_ConvertToFloatArray(aCollection));
	}

	/**
	 * Create a JunVrmlSFRotationField and initialize it.
	 * 
	 * @param anObject java.lang.Object
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlSFRotationField(Object anObject) {
		this((Collection<Number>) anObject);
	}

	/**
	 * Answer my value.
	 * 
	 * @return float[]
	 */
	public float[] value() {
		return value;
	}

	/**
	 * Set my value.
	 * 
	 * @param numbers float[]
	 */
	public void value_(float[] numbers) {
		value = numbers;
	}

	/**
	 * Convert the value to a Jun3dTransformation.
	 * 
	 * @return jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation as3dTransformation() {
		Jun3dPoint aPoint = new Jun3dPoint(value[0], value[1], value[2]);
		JunAngle anAngle = JunAngle.FromRad_(value[3]);
		return Jun3dTransformation.Rotate_around_(anAngle, new Jun3dLine(Jun3dPoint.Zero(), aPoint));
	}

	/**
	 * Convert the value to a reversed Jun3dTransformation.
	 * 
	 * @return jp.co.sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation asReverse3dTransformation() {
		Jun3dPoint aPoint = new Jun3dPoint(value[0], value[1], value[2]);
		JunAngle anAngle = JunAngle.FromRad_(value[3] * -1);
		return Jun3dTransformation.Rotate_around_(anAngle, new Jun3dLine(Jun3dPoint.Zero(), aPoint));
	}

}
