package jp.co.sra.jun.vrml.node.geometry;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dSphere;
import jp.co.sra.jun.vrml.field.sf.JunVrmlSFFloatField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode;

/**
 * JunVrmlSphereNode class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/27 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSphereNode.java,v 9.9 2010/03/09 09:18:01 nisinaka Exp $
 */
public class JunVrmlSphereNode extends JunVrmlAbstractGeometryNode {

	protected JunVrmlSFFloatField radiusField;

	/**
	 * Create a new instance of JunVrmlSphereNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlSphereNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("radius"))) {
			radiusField = new JunVrmlSFFloatField(aDictionary.get($("radius")));
		}
	}

	/**
	 * Create a new instance of JunVrmlSphereNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlSphereNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of radius.
	 * 
	 * @return float
	 */
	public static final float DefaultRadius() {
		return 1;
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		return new JunOpenGL3dSphere(this.radius());
	}

	/**
	 * Answer the radius.
	 * 
	 * @return float
	 */
	public float radius() {
		return this.radiusField().value();
	}

	/**
	 * Answer the radius.
	 * 
	 * @return jp.co.sra.jun.vrml.field.sf.JunVrmlSFFloatField
	 */
	protected JunVrmlSFFloatField radiusField() {
		if (radiusField == null) {
			radiusField = new JunVrmlSFFloatField(DefaultRadius());
		}
		return radiusField;
	}

}
