package jp.co.sra.jun.vrml.node.geometry10;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dPolyline;
import jp.co.sra.jun.vrml.field.mf.JunVrmlMFLongField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode;
import jp.co.sra.jun.vrml.node.property10.JunVrmlCoordinate3Node;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlIndexedLineSetNode10 class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   2000/04/14 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlIndexedLineSetNode10.java,v 9.9 2010/03/09 09:18:02 nisinaka Exp $
 */
public class JunVrmlIndexedLineSetNode10 extends JunVrmlAbstractGeometryNode {

	protected JunVrmlMFLongField coordIndexField;
	protected JunVrmlMFLongField materialIndexField;
	protected JunVrmlMFLongField normalIndexField;
	protected JunVrmlMFLongField textureCoordIndexField;

	/**
	 * Create a new instance of JunVrmlIndexedFaceSetNode10 and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlIndexedLineSetNode10(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("coordIndex"))) {
			coordIndexField = new JunVrmlMFLongField(aDictionary.get($("coordIndex")));
		}
		if (aDictionary.containsKey($("materialIndex"))) {
			materialIndexField = new JunVrmlMFLongField(aDictionary.get($("coordIndex")));
		}
		if (aDictionary.containsKey($("normalIndex"))) {
			normalIndexField = new JunVrmlMFLongField(aDictionary.get($("coordIndex")));
		}
		if (aDictionary.containsKey($("textureCoordIndex"))) {
			textureCoordIndexField = new JunVrmlMFLongField(aDictionary.get($("coordIndex")));
		}
	}

	/**
	 * Create a new instance of JunVrmlIndexedLineSetNode10 and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlIndexedLineSetNode10(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of coordIndex.
	 * 
	 * @return long[]
	 */
	public static final long[] DefaultCoordIndex() {
		return new long[] { 0 };
	}

	/**
	 * Answer the default value of materialIndex.
	 * 
	 * @return long[]
	 */
	public static final long[] DefaultMaterialIndex() {
		return new long[] { -1 };
	}

	/**
	 * Answer the default value of normalIndex.
	 * 
	 * @return long[]
	 */
	public static final long[] DefaultNormalIndex() {
		return new long[] { -1 };
	}

	/**
	 * Answer the default value of textureCoordIndex.
	 * 
	 * @return long[]
	 */
	public static final long[] DefaultTextureCoordIndex() {
		return new long[] { -1 };
	}

	/**
	 * Convert the geometry node as a JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		throw new SmalltalkException("Use 'asJunOpenGL3dObjectWithCoordinate3_'");
	}

	/**
	 * Convert the geometry node as a JunOpenGL3dObject.
	 * 
	 * @param aCoordinate3Node
	 *        jp.co.sra.jun.vrml.node.property10.JunVrmlCoordinate3Node
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectWithCoordinate3_(JunVrmlCoordinate3Node aCoordinate3Node) {
		long[][] pointArray = this.coordIndexAsPointArray();
		JunOpenGL3dObject[] objects = new JunOpenGL3dObject[pointArray.length];
		Jun3dPoint[] coordinates = aCoordinate3Node.pointAs3dPoints();
		for (int i = 0; i < pointArray.length; i++) {
			long[] indexes = pointArray[i];
			Jun3dPoint[] vertexes = new Jun3dPoint[indexes.length];
			for (int j = 0; j < indexes.length; j++) {
				vertexes[j] = coordinates[(int) indexes[j]];
			}
			objects[i] = new JunOpenGL3dPolyline(vertexes);
		}

		if (objects.length == 0) {
			return null;
		}
		if (objects.length == 1) {
			return objects[0];
		}
		return new JunOpenGL3dCompoundObject(objects);
	}

	/**
	 * Answer the coordIndex.
	 * 
	 * @return long[]
	 */
	public long[] coordIndex() {
		return this.coordIndexField().value();
	}

	/**
	 * Answer the point array of coordIndex.
	 * 
	 * @return long[][]
	 */
	public long[][] coordIndexAsPointArray() {
		return this.coordIndexField().asPointArray();
	}

	/**
	 * Answer the materialIndex.
	 * 
	 * @return long[]
	 */
	public long[] materialIndex() {
		return this.materialIndexField().value();
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doIndexedLineSet_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Answer the normalIndex.
	 * 
	 * @return long[]
	 */
	public long[] normalIndex() {
		return this.normalIndexField().value();
	}

	/**
	 * Answer the textureCoordIndex.
	 * 
	 * @return long[]
	 */
	public long[] textureCoordIndex() {
		return this.textureCoordIndexField().value();
	}

	/**
	 * Answer the coordIndex field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFLongField
	 */
	protected JunVrmlMFLongField coordIndexField() {
		if (coordIndexField == null) {
			coordIndexField = new JunVrmlMFLongField(DefaultCoordIndex());
		}
		return coordIndexField;
	}

	/**
	 * Answer the materialIndex field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFLongField
	 */
	protected JunVrmlMFLongField materialIndexField() {
		if (materialIndexField == null) {
			materialIndexField = new JunVrmlMFLongField(DefaultCoordIndex());
		}
		return materialIndexField;
	}

	/**
	 * Answer the normalIndex field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFLongField
	 */
	protected JunVrmlMFLongField normalIndexField() {
		if (normalIndexField == null) {
			normalIndexField = new JunVrmlMFLongField(DefaultCoordIndex());
		}
		return normalIndexField;
	}

	/**
	 * Answer the textureCoordIndex field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFLongField
	 */
	protected JunVrmlMFLongField textureCoordIndexField() {
		if (textureCoordIndexField == null) {
			textureCoordIndexField = new JunVrmlMFLongField(DefaultCoordIndex());
		}
		return textureCoordIndexField;
	}

}
