package jp.co.sra.jun.vrml.node.geometry20;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.vrml.field.sf.JunVrmlSFVec3fField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlBoxNode class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/28 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlBoxNode.java,v 9.9 2010/03/09 09:18:02 nisinaka Exp $
 */
public class JunVrmlBoxNode extends JunVrmlAbstractGeometryNode {

	protected JunVrmlSFVec3fField sizeField;

	/**
	 * Create a new instance of JunVrmlBoxNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlBoxNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("size"))) {
			sizeField = new JunVrmlSFVec3fField(aDictionary.get($("size")));
		}
	}

	/**
	 * Create a new instance of JunVrmlBoxNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlBoxNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of size.
	 * 
	 * @return float[]
	 */
	public static final float[] DefaultSize() {
		return new float[] { 2, 2, 2 };
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @return jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		Jun3dPoint aPoint = this.sizeAs3dPoint();
		JunOpenGL3dObject aBox = JunOpenGL3dObject.Box_(aPoint);
		aBox = aBox.translatedBy_(aPoint.dividedBy_(-2.0));
		return aBox;
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doBox_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Answer the size.
	 * 
	 * @return float[]
	 */
	public float[] size() {
		return this.sizeField().value();
	}

	/**
	 * Answer the size as a Jun3dPoint.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint sizeAs3dPoint() {
		return this.sizeField().as3dPoint();
	}

	/**
	 * Answer the size field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.sf.JunVrmlSFVec3fField
	 */
	protected JunVrmlSFVec3fField sizeField() {
		if (sizeField == null) {
			sizeField = new JunVrmlSFVec3fField(DefaultSize());
		}
		return sizeField;
	}

}
