package jp.co.sra.jun.vrml.node.grouping20;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.vrml.field.sf.JunVrmlSFVec3fField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGroupingNode;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlGroupNode class
 * 
 *  @author    nisinaka
 *  @created   2000/04/17 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlGroupNode.java,v 9.9 2010/03/09 09:18:02 nisinaka Exp $
 */
public class JunVrmlGroupNode extends JunVrmlAbstractGroupingNode {

	protected JunVrmlNode[] children;
	protected JunVrmlSFVec3fField bboxCenterField;
	protected JunVrmlSFVec3fField bboxSizeField;

	/**
	 * Create a new instance of JunVrmlGroupNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlGroupNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("children"))) {
			this.children_(aDictionary.get($("children")));
		}
		if (aDictionary.containsKey($("bboxCenter"))) {
			bboxCenterField = new JunVrmlSFVec3fField(aDictionary.get($("bboxCenter")));
		}
		if (aDictionary.containsKey($("bboxSize"))) {
			bboxSizeField = new JunVrmlSFVec3fField(aDictionary.get($("bboxSize")));
		}
	}

	/**
	 * Create a new instance of JunVrmlGroupNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlGroupNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of bboxCenter.
	 * 
	 * @return java.lang.Integer[]
	 */
	public static final float[] DefaultBboxCenter() {
		return new float[] { 0, 0, 0 };
	}

	/**
	 * Answer the default value of bboxSize.
	 * 
	 * @return java.lang.Integer[]
	 */
	public static final float[] DefaultBboxSize() {
		return new float[] { -1, -1, -1 };
	}

	/**
	 * Answer the bboxCenter.
	 * 
	 * @return float[]
	 */
	public float[] bboxCenter() {
		return this.bboxCenterField().value();
	}

	/**
	 * Answer the bboxSize.
	 * 
	 * @return float[]
	 */
	public float[] bboxSize() {
		return this.bboxSizeField().value();
	}

	/**
	 * Answer the children.
	 * 
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode[]
	 */
	public JunVrmlNode[] children() {
		return children;
	}

	/**
	 * Set the children nodes.
	 * 
	 * @param aNode java.lang.Object
	 * 
	 * @throws jp.co.sra.smalltalk.SmalltalkException DOCUMENT ME!
	 */
	@SuppressWarnings("unchecked")
	public void children_(Object aNode) {
		if (aNode instanceof JunVrmlNode) {
			children = new JunVrmlNode[] { (JunVrmlNode) aNode };
		} else if (aNode instanceof Collection) {
			Collection<JunVrmlNode> collection = (Collection<JunVrmlNode>) aNode;
			children = collection.toArray(new JunVrmlNode[collection.size()]);
		} else {
			throw new jp.co.sra.smalltalk.SmalltalkException();
		}
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doGroup_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Answer the bboxCenter field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.sf.JunVrmlSFVec3fField
	 */
	protected JunVrmlSFVec3fField bboxCenterField() {
		if (bboxCenterField == null) {
			bboxCenterField = new JunVrmlSFVec3fField(DefaultBboxCenter());
		}
		return bboxCenterField;
	}

	/**
	 * Answer the bboxSize field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.sf.JunVrmlSFVec3fField
	 */
	protected JunVrmlSFVec3fField bboxSizeField() {
		if (bboxSizeField == null) {
			bboxSizeField = new JunVrmlSFVec3fField(DefaultBboxSize());
		}
		return bboxSizeField;
	}

}
