package jp.co.sra.jun.vrml.node.grouping20;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGroupingNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlAppearanceNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlShapeNode class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/04/17 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlShapeNode.java,v 9.9 2010/03/09 09:18:02 nisinaka Exp $
 */
public class JunVrmlShapeNode extends JunVrmlAbstractGroupingNode {

	protected JunVrmlAppearanceNode appearance;
	protected JunVrmlAbstractGeometryNode geometry;

	/**
	 * Create a new instance of JunVrmlShapeNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlShapeNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("appearance"))) {
			this.appearance_((JunVrmlAppearanceNode) aDictionary.get($("appearance")));
		}
		if (aDictionary.containsKey($("geometry"))) {
			this.geometry_((JunVrmlAbstractGeometryNode) aDictionary.get($("geometry")));
		}
	}

	/**
	 * Create a new instance of JunVrmlShapeNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlShapeNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the appearance node.
	 * 
	 * @return jp.co.sra.jun.vrml.node.property20.JunVrmlAppearanceNode
	 */
	public JunVrmlAppearanceNode appearance() {
		return appearance;
	}

	/**
	 * Set the appearance node.
	 * 
	 * @param aNode jp.co.sra.jun.vrml.node.property20.JunVrmlAppearanceNode
	 */
	public void appearance_(JunVrmlAppearanceNode aNode) {
		appearance = aNode;
	}

	/**
	 * Answer the geometry node.
	 * 
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode
	 */
	public JunVrmlAbstractGeometryNode geometry() {
		return geometry;
	}

	/**
	 * Set the geometry node.
	 * 
	 * @param aNode  jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode
	 */
	public void geometry_(JunVrmlAbstractGeometryNode aNode) {
		geometry = aNode;
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doShape_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

}
