package jp.co.sra.jun.vrml.node.property;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.geometry.basic.Jun3dPoint;
import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.vrml.field.mf.JunVrmlMFVec3fField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlNormalNode class
 * 
 *  @author    nisinaka
 *  @created   2000/03/29 (by nisinaka)
 *  @updated   2003/09/09 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlNormalNode.java,v 9.10 2010/03/09 09:18:02 nisinaka Exp $
 */
public class JunVrmlNormalNode extends JunVrmlAbstractPropertyNode {

	protected JunVrmlMFVec3fField vectorField;

	/**
	 * The default constructor.
	 * 
	 * @category Instance creation
	 */
	public JunVrmlNormalNode() {
		super();
	}

	/**
	 * Create a new instance of JunVrmlNormalNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 * @category Instance creation
	 */
	public JunVrmlNormalNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("vector"))) {
			vectorField = new JunVrmlMFVec3fField(aDictionary.get($("vector")));
		}
	}

	/**
	 * Create a new instance of JunVrmlNormalNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlNormalNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of vector.
	 * 
	 * @return float[]
	 * @category Defaults
	 */
	public static final float[] DefaultVector() {
		return new float[0];
	}

	/**
	 * Initialize the JunVrmlNormalNode.
	 * 
	 * @see jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode#initialize()
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();
		vectorField = null;
	}

	/**
	 * Answer the vector.
	 * 
	 * @return float[]
	 * @category accessing
	 */
	public float[] vector() {
		return this.vectorField().value();
	}

	/**
	 * Answer the vector field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFVec3fField
	 * @category accessing
	 */
	protected JunVrmlMFVec3fField vectorField() {
		if (vectorField == null) {
			vectorField = new JunVrmlMFVec3fField(DefaultVector());
		}
		return vectorField;
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 * @category enumerating
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doNormal_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Answer the vector as an array of Jun3dPoint.
	 * 
	 * @return jp.co.sra.jun.geometry.basic.Jun3dPoint[]
	 * @category code generation
	 */
	public Jun3dPoint[] vectorAs3dPoints() {
		return this.vectorField().as3dPoints();
	}

	/**
	 * Set my properties on the JunOpenGL3dObject.
	 * 
	 * @param anOpenGL3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category code generation
	 */
	public void propertyOn_(JunOpenGL3dObject anOpenGL3dObject) {
	}

}
