package jp.co.sra.jun.vrml.node.property20;

import java.awt.Color;
import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.vrml.field.mf.JunVrmlMFColorField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlBackgroundNode class
 * 
 *  @author    nisinaka
 *  @created   2004/11/02 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on Nsn156 for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlBackgroundNode.java,v 9.9 2010/03/09 09:18:03 nisinaka Exp $
 */
public class JunVrmlBackgroundNode extends JunVrmlAbstractPropertyNode {

	protected JunVrmlMFColorField skyColorField;

	public static float[] DefaultSkyColor = new float[] { 0, 0, 0 };

	/**
	 * Create a new instance of JunVrmlBackgroundNode and initialize it.
	 *
	 * @param aDictionary java.util.Map
	 * @category Instance creation
	 */
	public JunVrmlBackgroundNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("skyColor"))) {
			skyColorField = new JunVrmlMFColorField(aDictionary.get($("skyColor")));
		}
	}

	/**
	 * Create a new instance of JunVrmlBackgroundNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlBackgroundNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer my skyColor field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFColorField
	 * @category field accessing
	 */
	protected JunVrmlMFColorField skyColorField() {
		if (skyColorField == null) {
			skyColorField = new JunVrmlMFColorField(DefaultSkyColor);
		}
		return skyColorField;
	}

	/**
	 * Process myself with the generator. 
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 * @see jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode#nodeDo_(jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor)
	 * @category enumerating
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doBackground_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Set my properties on the JunOpenGL3dObject.
	 * 
	 * @param a3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @see jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode#propertyOn_(jp.co.sra.jun.opengl.objects.JunOpenGL3dObject)
	 * @category code generation
	 */
	public void propertyOn_(JunOpenGL3dObject a3dObject) {
		// The background is a property of JunOpenGL3dModel.
	}

	/**
	 * Answer the sky color as an array of Color.
	 * 
	 * @return java.awt.Color[]
	 * @category code generation
	 */
	public Color[] skyColorAsColors() {
		return this.skyColorField().asColors();
	}

}
