package jp.co.sra.jun.vrml.node.property20;

import java.awt.Color;
import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.vrml.field.mf.JunVrmlMFColorField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlColorNode class
 * 
 *  @author    nisinaka
 *  @created   2000/03/29 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlColorNode.java,v 9.9 2010/03/09 09:18:03 nisinaka Exp $
 */
public class JunVrmlColorNode extends JunVrmlAbstractPropertyNode {

	protected JunVrmlMFColorField colorField;

	/**
	 * The default constructor.
	 * 
	 * @category Instance creation
	 */
	public JunVrmlColorNode() {
	}

	/**
	 * Create a new instance of JunVrmlColorNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 * @category Instance creation
	 */
	public JunVrmlColorNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("color"))) {
			colorField = new JunVrmlMFColorField(aDictionary.get($("color")));
		}
	}

	/**
	 * Create a new instance of JunVrmlColorNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlColorNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of color.
	 * 
	 * @return float[]
	 * @category Defaults
	 */
	public static final float[] DefaultColor() {
		return new float[] {};
	}

	/**
	 * Answer the color value.
	 * 
	 * @return float[]
	 * @category accessing
	 */
	public float[] color() {
		return this.colorField().value();
	}

	/**
	 * Answer the color field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFColorField
	 * @category accessing
	 */
	protected JunVrmlMFColorField colorField() {
		if (colorField == null) {
			colorField = new JunVrmlMFColorField(DefaultColor());
		}
		return colorField;
	}

	/**
	 * Set my properties on the JunOpenGL3dObject.
	 * 
	 * @param anOpenGL3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category code generation
	 */
	public void propertyOn_(JunOpenGL3dObject anOpenGL3dObject) {
		anOpenGL3dObject.paint_(this.colorField().asColors()[0]);
	}

	/**
	 * Answer the color field values as an array of Color.
	 * 
	 * @return java.awt.Color
	 * @category code generation
	 */
	public Color[] colorAsColors() {
		return this.colorField().asColors();
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 * @category enumerating
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doColor_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

}
