package jp.co.sra.jun.vrml.node.property20;

import java.util.Hashtable;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.opengl.objects.JunOpenGL3dObject;
import jp.co.sra.jun.vrml.field.mf.JunVrmlMFStringField;
import jp.co.sra.jun.vrml.field.sf.JunVrmlSFStringField;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode;
import jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlWorldInfoNode class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/04/17 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlWorldInfoNode.java,v 9.9 2010/03/09 09:18:04 nisinaka Exp $
 */
public class JunVrmlWorldInfoNode extends JunVrmlAbstractPropertyNode {

	protected JunVrmlMFStringField infoField;
	protected JunVrmlSFStringField titleField;

	/**
	 * Create a new instance of JunVrmlWorldInfoNode and initialize it.
	 * 
	 * @param aDictionary java.util.Map
	 */
	public JunVrmlWorldInfoNode(Map<StSymbol, Object> aDictionary) {
		if (aDictionary.containsKey($("info"))) {
			infoField = new JunVrmlMFStringField(aDictionary.get($("info")));
		}
		if (aDictionary.containsKey($("title"))) {
			titleField = new JunVrmlSFStringField(aDictionary.get($("title")));
		}
	}

	/**
	 * Create a new instance of JunVrmlWorldInfoNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlWorldInfoNode(Hashtable<?, ?> aDictionary) {
		this((Map<StSymbol, Object>) aDictionary);
	}

	/**
	 * Answer the default value of info.
	 * 
	 * @return java.lang.String[]
	 */
	public static String[] DefaultInfo() {
		return new String[] {};
	}

	/**
	 * Answer the default value of title.
	 * 
	 * @return java.lang.String
	 */
	public static String DefaultTitle() {
		return "";
	}

	/**
	 * Answer the info.
	 * 
	 * @return java.lang.String[]
	 */
	public String[] info() {
		return this.infoField().value();
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor jp.co.sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doWorldInfo_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Set my properties on the JunOpenGL3dObject.
	 * 
	 * @param a3dObject jp.co.sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void propertyOn_(JunOpenGL3dObject a3dObject) {
	}

	/**
	 * Answer the title.
	 * 
	 * @return java.lang.String
	 */
	public String title() {
		return this.titleField().value();
	}

	/**
	 * Answer the info field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.mf.JunVrmlMFStringField
	 */
	protected JunVrmlMFStringField infoField() {
		if (infoField == null) {
			infoField = new JunVrmlMFStringField(DefaultInfo());
		}
		return infoField;
	}

	/**
	 * Answer the title field.
	 * 
	 * @return jp.co.sra.jun.vrml.field.sf.JunVrmlSFStringField
	 */
	protected JunVrmlSFStringField titleField() {
		if (titleField == null) {
			titleField = new JunVrmlSFStringField(DefaultTitle());
		}
		return titleField;
	}

}
