package jp.co.sra.jun.vrml.support;

import java.util.Collection;
import java.util.Stack;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure1;
import jp.co.sra.smalltalk.StReadStream;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode;

/**
 * JunVrmlParser class
 * 
 *  @author    nisinaka
 *  @created   2000/03/17 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2005 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlParser.java,v 9.9 2010/03/09 09:18:05 nisinaka Exp $
 */
public abstract class JunVrmlParser extends JunVrmlScanner {

	protected Object parseNode = null;
	protected JunVrmlSyntaxTreeBuilder builder = null;
	protected Stack<Integer> prevMark = null;
	protected Stack<Object> prevToken = null;
	protected Stack<StSymbol> prevTokenType = null;

	/**
	 * Create a read stream from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return jp.co.sra.smalltalk.StReadStream
	 */
	public static StReadStream MakeStream_(Object anObject) {
		if (anObject instanceof StReadStream) {
			return (StReadStream) anObject;
		}

		if (anObject instanceof String) {
			return new StReadStream((String) anObject);
		}

		return new StReadStream(anObject.toString());
	}

	/**
	 * Initialize the VRML parser.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @param aBuilder jp.co.sra.jun.vrml.support.JunVrmlSyntaxTreeBuilder
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @category initialize-release
	 */
	protected void init_builder_ifFail_(StReadStream sourceStream, JunVrmlSyntaxTreeBuilder aBuilder, StBlockClosure1<Void, String> aBlock) {
		super.on_(sourceStream);
		builder = aBuilder;
		failBlock = aBlock;
		prevMark = new Stack<Integer>();
		prevToken = new Stack<Object>();
		prevTokenType = new Stack<StSymbol>();
	}

	/**
	 * Get a next token.
	 * 
	 * @return java.lang.Object
	 */
	public Object nextToken() {
		if (tokenType != $("xComment")) {
			prevMark.push(new Integer(mark));
			prevToken.push(token);
			prevTokenType.push(tokenType);
		}

		return super.nextToken();
	}

	/**
	 * Parse the input stream and generate a VRML node tree.
	 * 
	 * @param anObject java.lang.Object
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode[]
	 */
	public JunVrmlNode[] parse_(Object anObject) {
		return this.parse_builder_(MakeStream_(anObject), this.defaultBuilder());
	}

	/**
	 * Parse the input stream with the syntax tree builder and generate a VRML node tree.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @param aBuilder jp.co.sra.jun.vrml.support.JunVrmlSyntaxTreeBuilder
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode[]
	 */
	public JunVrmlNode[] parse_builder_(StReadStream sourceStream, JunVrmlSyntaxTreeBuilder aBuilder) {
		return this.parse_builder_ifFail_(sourceStream, aBuilder, new StBlockClosure1<Void, String>() {
			public Void value(String errorMessage) {
				String label = errorMessage + " near " + token.toString();
				String string = source.upToEnd().toString();
				if (string.length() == 0) {
					string = "--> end of file ";
				} else {
					string = "--> " + string;
				}
				throw SmalltalkException.Error(label + " : " + string);
			}
		});
	}

	/**
	 * Parse the input stream with the syntax tree builder and generate a VRML node tree.
	 * 
	 * @param sourceStream jp.co.sra.smalltalk.StReadStream
	 * @param aBuilder jp.co.sra.jun.vrml.support.JunVrmlSyntaxTreeBuilder
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode[]
	 */
	@SuppressWarnings("unchecked")
	public JunVrmlNode[] parse_builder_ifFail_(StReadStream sourceStream, JunVrmlSyntaxTreeBuilder aBuilder, StBlockClosure1<Void, String> aBlock) {
		this.init_builder_ifFail_(sourceStream, aBuilder, aBlock);

		if (this.parseVrml() == false) {
			throw SmalltalkException.Error("Syntax error");
		}

		if (parseNode == null) {
			return null;
		}
		if (parseNode instanceof JunVrmlNode) {
			return new JunVrmlNode[] { (JunVrmlNode) parseNode };
		}
		if (parseNode instanceof Collection) {
			Collection<JunVrmlNode> nodes = (Collection<JunVrmlNode>) parseNode;
			return nodes.toArray(new JunVrmlNode[nodes.size()]);
		}

		throw SmalltalkException.Error("Unknown type of parsed node");
	}

	/**
	 * Answer true if the receiver successfully parse the source stream.
	 * Creation date: (2000/03/22 18:44:39)
	 * 
	 * @return boolean
	 */
	public boolean parseVrml() {
		if (source.atEnd()) {
			parseNode = null;
			return true;
		}
		if (this.parseVrmlScene()) {
			this.nextToken();
			return tokenType == $("eof");
		}

		failBlock.value("Syntex error");
		return false;
	}

	/**
	 * Answer true if the receiver successfully parse the source stream as a VRML scene.
	 * 
	 * @return boolean
	 */
	public abstract boolean parseVrmlScene();

	/**
	 * Unget a token.
	 */
	public void unNextToken() {
		super.unNextToken();
		mark = prevMark.pop();
		token = prevToken.pop();
		tokenType = prevTokenType.pop();
	}

	/**
	 * Answer an instance of the default syntax tree builder.
	 * 
	 * @return jp.co.sra.jun.vrml.support.JunVrmlSyntaxTreeBuilder
	 */
	protected abstract JunVrmlSyntaxTreeBuilder defaultBuilder();

}
