package jp.co.sra.jun.vrml.support;

import java.util.Map;

import jp.co.sra.smalltalk.SmalltalkException;
import jp.co.sra.smalltalk.StBlockClosure0;
import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.system.framework.JunAbstractObject;
import jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode;

/**
 * JunVrmlSyntaxTreeBuilder class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/24 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSyntaxTreeBuilder.java,v 9.9 2010/03/09 09:18:05 nisinaka Exp $
 */
public abstract class JunVrmlSyntaxTreeBuilder extends JunAbstractObject {

	/**
	 * Create a new VRML node.
	 * 
	 * @param aSymbol jp.co.sra.smalltalk.StSymbol
	 * @param anObject java.lang.Object
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode
	 */
	public JunVrmlNode newNode_with_(final StSymbol aSymbol, Object anObject) {
		return this.newNode_with_ifFailed_(aSymbol, anObject, new StBlockClosure0<JunVrmlNode>() {
			public JunVrmlNode value() {
				throw SmalltalkException.Error("Unknown node - " + aSymbol.toString());
			}
		});
	}

	/**
	 * Create a new VRML node.
	 * 
	 * @param aSymbol jp.co.sra.smalltalk.StSymbol
	 * @param anObject java.lang.Object
	 * @param failBlock jp.co.sra.smalltalk.StBlockClosure
	 * @return jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode
	 */
	public JunVrmlNode newNode_with_ifFailed_(StSymbol aSymbol, Object anObject, StBlockClosure0<JunVrmlNode> failBlock) {
		Class<? extends JunVrmlNode> aNodeClass = this.nodeTypeTable().get(aSymbol);
		if (aNodeClass == null) {
			return failBlock.value();
		}

		return _New(aNodeClass, anObject);
	}

	/**
	 * Answer a node type table.
	 * 
	 * @return java.util.Map
	 */
	public abstract Map<StSymbol, Class<? extends JunVrmlNode>> nodeTypeTable();

}
