package jp.co.sra.jun.vrml.support;

import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode;
import jp.co.sra.jun.vrml.node.geometry.JunVrmlSphereNode;
import jp.co.sra.jun.vrml.node.geometry10.JunVrmlCubeNode;
import jp.co.sra.jun.vrml.node.geometry10.JunVrmlIndexedFaceSetNode10;
import jp.co.sra.jun.vrml.node.geometry10.JunVrmlIndexedLineSetNode10;
import jp.co.sra.jun.vrml.node.grouping10.JunVrmlSeparatorNode;
import jp.co.sra.jun.vrml.node.property.JunVrmlNormalNode;
import jp.co.sra.jun.vrml.node.property10.JunVrmlCoordinate3Node;
import jp.co.sra.jun.vrml.node.property10.JunVrmlMaterialNode10;
import jp.co.sra.jun.vrml.node.property10.JunVrmlTexture2Node;
import jp.co.sra.jun.vrml.node.property10.JunVrmlTextureCoordinate2Node;

/**
 * JunVrmlSyntaxTreeBuilder10 class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2000/03/24 (by nisinaka)
 *  @updated   2008/05/28 (by nisinaka)
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSyntaxTreeBuilder10.java,v 9.9 2010/03/09 09:18:05 nisinaka Exp $
 */
public class JunVrmlSyntaxTreeBuilder10 extends JunVrmlSyntaxTreeBuilder {

	/** The node type table of VRML1.0. */
	protected static HashMap<StSymbol, Class<? extends JunVrmlNode>> NodeTypeTable = null;

	static {
		_InitializeNodeTypeTable();
	}

	/**
	 * Initialize the NodeTypeTable of VRML1.0.
	 */
	protected static void _InitializeNodeTypeTable() {
		NodeTypeTable = new HashMap<StSymbol, Class<? extends JunVrmlNode>>();
		NodeTypeTable.put($("Coordinate3"), JunVrmlCoordinate3Node.class);
		NodeTypeTable.put($("Cube"), JunVrmlCubeNode.class);
		NodeTypeTable.put($("IndexedFaceSet"), JunVrmlIndexedFaceSetNode10.class);
		NodeTypeTable.put($("IndexedLineSet"), JunVrmlIndexedLineSetNode10.class);
		NodeTypeTable.put($("Material"), JunVrmlMaterialNode10.class);
		NodeTypeTable.put($("Normal"), JunVrmlNormalNode.class);
		NodeTypeTable.put($("Separator"), JunVrmlSeparatorNode.class);
		NodeTypeTable.put($("Sphere"), JunVrmlSphereNode.class);
		NodeTypeTable.put($("Texture2"), JunVrmlTexture2Node.class);
		NodeTypeTable.put($("TextureCoordinate2"), JunVrmlTextureCoordinate2Node.class);
	}

	/**
	 * Answer a node type table.
	 * 
	 * @return java.util.Map
	 */
	public Map<StSymbol, Class<? extends JunVrmlNode>> nodeTypeTable() {
		return NodeTypeTable;
	}

}
