package jp.co.sra.jun.vrml.support;

import java.util.HashMap;
import java.util.Map;

import jp.co.sra.smalltalk.StSymbol;

import jp.co.sra.jun.vrml.node.abstracts.JunVrmlNode;
import jp.co.sra.jun.vrml.node.geometry.JunVrmlSphereNode;
import jp.co.sra.jun.vrml.node.geometry20.JunVrmlBoxNode;
import jp.co.sra.jun.vrml.node.geometry20.JunVrmlIndexedFaceSetNode20;
import jp.co.sra.jun.vrml.node.geometry20.JunVrmlIndexedLineSetNode20;
import jp.co.sra.jun.vrml.node.grouping20.JunVrmlGroupNode;
import jp.co.sra.jun.vrml.node.grouping20.JunVrmlShapeNode;
import jp.co.sra.jun.vrml.node.grouping20.JunVrmlTransformNode;
import jp.co.sra.jun.vrml.node.property.JunVrmlNormalNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlAppearanceNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlBackgroundNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlColorNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlCoordinateNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlMaterialNode20;
import jp.co.sra.jun.vrml.node.property20.JunVrmlNavigationInfoNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlPixelTextureNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlTextureCoordinateNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlViewpointNode;
import jp.co.sra.jun.vrml.node.property20.JunVrmlWorldInfoNode;

/**
 * JunVrmlSyntaxTreeBuilder20 class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   758 (with StPL9.7) based on JunXXX for Smalltalk
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlSyntaxTreeBuilder20.java,v 9.9 2010/03/09 09:18:05 nisinaka Exp $
 */
public class JunVrmlSyntaxTreeBuilder20 extends JunVrmlSyntaxTreeBuilder {

	/** The node type table of VRML2.0. */
	protected static HashMap<StSymbol, Class<? extends JunVrmlNode>> NodeTypeTable = null;

	static {
		_InitializeNodeTypeTable();
	}

	/**
	 * Initialize the NodeTypeTable of VRML2.0.
	 */
	protected static void _InitializeNodeTypeTable() {
		NodeTypeTable = new HashMap<StSymbol, Class<? extends JunVrmlNode>>();
		NodeTypeTable.put($("Appearance"), JunVrmlAppearanceNode.class);
		NodeTypeTable.put($("Background"), JunVrmlBackgroundNode.class);
		NodeTypeTable.put($("Box"), JunVrmlBoxNode.class);
		NodeTypeTable.put($("Color"), JunVrmlColorNode.class);
		NodeTypeTable.put($("Coordinate"), JunVrmlCoordinateNode.class);
		NodeTypeTable.put($("Group"), JunVrmlGroupNode.class);
		NodeTypeTable.put($("Material"), JunVrmlMaterialNode20.class);
		NodeTypeTable.put($("NavigationInfo"), JunVrmlNavigationInfoNode.class);
		NodeTypeTable.put($("Normal"), JunVrmlNormalNode.class);
		NodeTypeTable.put($("IndexedFaceSet"), JunVrmlIndexedFaceSetNode20.class);
		NodeTypeTable.put($("IndexedLineSet"), JunVrmlIndexedLineSetNode20.class);
		NodeTypeTable.put($("PixelTexture"), JunVrmlPixelTextureNode.class);
		NodeTypeTable.put($("Shape"), JunVrmlShapeNode.class);
		NodeTypeTable.put($("Sphere"), JunVrmlSphereNode.class);
		NodeTypeTable.put($("TextureCoordinate"), JunVrmlTextureCoordinateNode.class);
		NodeTypeTable.put($("Transform"), JunVrmlTransformNode.class);
		NodeTypeTable.put($("Viewpoint"), JunVrmlViewpointNode.class);
		NodeTypeTable.put($("WorldInfo"), JunVrmlWorldInfoNode.class);
	}

	/**
	 * Answer a node type table.
	 * 
	 * @return java.util.Map
	 */
	public Map<StSymbol, Class<? extends JunVrmlNode>> nodeTypeTable() {
		return NodeTypeTable;
	}

}
