package jp.co.sra.smalltalk;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/**
 * StBlockClosure class
 * 
 *  @author    He Weijie
 *  @created   1998/08/06 (by He Weijie)
 *  @updated   2008/05/16 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StBlockClosure.java,v 9.14 2010/03/09 09:18:06 nisinaka Exp $
 */
public class StBlockClosure<R, S, T, U> extends StObject implements StValued<R> {

	/**
	 * Just return the number of arguments.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int numArgs() {
		Class<?> myClass = this.getClass();
		if (myClass == StBlockClosure.class) {
			return 0;
		}

		Method[] myMethods = myClass.getDeclaredMethods();
		for (int i = 0; i < myMethods.length; i++) {
			if (Modifier.isVolatile(myMethods[i].getModifiers())) {
				continue;
			}

			String name = myMethods[i].getName();
			if ("value".equals(name)) {
				Class<?>[] parameterTypes = myMethods[i].getParameterTypes();
				return parameterTypes.length;
			} else if ("value_".equals(name)) {
				return 1;
			} else if ("value_value_".equals(name)) {
				return 2;
			} else if ("value_value_value_".equals(name)) {
				return 3;
			} else if ("valueWithArguments_".equals(name)) {
				return 4;
			}
		}

		throw new SmalltalkException("Unexpected name of the method");
	}

	/**
	 * This is an abstract method for value
	 * 
	 * @return java.lang.Object
	 * @see jp.co.sra.smalltalk.StValued#value()
	 * @category evaluating
	 */
	public R value() {
		return null;
	}

	/**
	 * This is an abstract method for value:
	 * 
	 * @param argument java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public R value_(Object argument) {
		return this.value((S) argument);
	}

	/**
	 * This is an abstract method for value: value:
	 * 
	 * @param argument1 java.lang.Object
	 * @param argument2 java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public R value_value_(Object argument1, Object argument2) {
		return this.value((S) argument1, (T) argument2);
	}

	/**
	 * This is an abstract method for value: value: value:
	 * 
	 * @param argument1 java.lang.Object
	 * @param argument2 java.lang.Object
	 * @param argument3 java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 * @deprecated since Jun701, use Generics.
	 */
	@SuppressWarnings("unchecked")
	public R value_value_value_(Object argument1, Object argument2, Object argument3) {
		return this.value((S) argument1, (T) argument2, (U) argument3);
	}

	/**
	 * This is an abstract method for valueWithArguments:
	 * 
	 * @param args java.lang.Object[]
	 * @return java.lang.Object
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category evaluating
	 */
	@SuppressWarnings("unchecked")
	public R valueWithArguments_(Object[] args) {
		if (args == null) {
			return null;
		}

		switch (args.length) {
			case 0:
				return this.value();
			case 1:
				return this.value((S) args[0]);
			case 2:
				return this.value((S) args[0], (T) args[1]);
			case 3:
				return this.value((S) args[0], (T) args[1], (U) args[2]);
		}

		throw new SmalltalkException("The method 'valueWithArguments_(Object[])' should be overridden.");
	}

	/**
	 * This is an abstract method for value with a generic argument.
	 * 
	 * @param arg1 java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 */
	public R value(S arg1) {
		return null;
	}

	/**
	 * This is an abstract method for value with two generic arguments.
	 * 
	 * @param arg1 java.lang.Object
	 * @param arg2 java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 */
	public R value(S arg1, T arg2) {
		return null;
	}

	/**
	 * This is an abstract method for value with three generic arguments.
	 * 
	 * @param arg1 java.lang.Object
	 * @param arg2 java.lang.Object
	 * @param arg3 java.lang.Object
	 * @return java.lang.Object
	 * @category evaluating
	 */
	public R value(S arg1, T arg2, U arg3) {
		return null;
	}

}
