package jp.co.sra.smalltalk;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * StBlockValue class
 * 
 *  @author    nisinaka
 *  @created   1998/10/15 (by nisinaka)
 *  @updated   1998/11/09 (by nisinaka)
 *  @updated   2003/03/07 (by nisinaka)
 *  @updated   2006/04/18 (by nisinaka)
 *  @updated   2008/04/22 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StBlockValue.java,v 9.10 2010/03/09 09:18:06 nisinaka Exp $
 */
public class StBlockValue<V> extends StComputedValue<V> {

	/** The block closure to evaluate. */
	protected StBlockClosure<V, ?, ?, ?> block = null;

	/** The array of StBlockValued, each of the object has a dependency. */
	protected ArrayList<StBlockValued<?>> arguments = null;

	/** The number of arguments block takes. */
	protected int numArgs;

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure<V, ?, ?, ?> aBlock) {
		this.setBlock_(aBlock);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aValued jp.co.sra.smalltalk.StBlockValued
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure<V, ?, ?, ?> aBlock, StBlockValued<?> aValued) {
		this.setBlock_arguments_(aBlock, new StBlockValued<?>[] { aValued });
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param anArray jp.co.sra.smalltalk.StBlockValued[]
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure<V, ?, ?, ?> aBlock, StBlockValued<?>[] anArray) {
		this.setBlock_arguments_(aBlock, anArray);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aCollection java.util.Collection
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure<V, ?, ?, ?> aBlock, Collection<StBlockValued<?>> aCollection) {
		this.setBlock_arguments_(aBlock, aCollection);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure0
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure0<V> aBlock) {
		this.setBlock_(aBlock);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure1
	 * @param aValued jp.co.sra.smalltalk.StBlockValued
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure1<V, ?> aBlock, StBlockValued<?> aValued) {
		this.setBlock_arguments_(aBlock, new StBlockValued<?>[] { aValued });
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure2
	 * @param aValued1 jp.co.sra.smalltalk.StBlockValued
	 * @param aValued2 jp.co.sra.smalltalk.StBlockValued
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure2<V, ?, ?> aBlock, StBlockValued<?> aValued1, StBlockValued<?> aValued2) {
		this.setBlock_arguments_(aBlock, new StBlockValued<?>[] { aValued1, aValued2 });
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure3
	 * @param aValued1 jp.co.sra.smalltalk.StBlockValued
	 * @param aValued2 jp.co.sra.smalltalk.StBlockValued
	 * @param aValued3 jp.co.sra.smalltalk.StBlockValued
	 * @category Instance creation
	 */
	public StBlockValue(StBlockClosure3<V, ?, ?, ?> aBlock, StBlockValued<?> aValued1, StBlockValued<?> aValued2, StBlockValued<?> aValued3) {
		this.setBlock_arguments_(aBlock, new StBlockValued<?>[] { aValued1, aValued2, aValued3 });
	}

	/**
	 * Answer a collection of objects that have the receiver as a dependent.
	 * 
	 * @return jp.co.sra.smalltalk.StBlockValued[]
	 * @see jp.co.sra.smalltalk.StComputedValue#parts()
	 * @category accessing
	 */
	public StBlockValued<?>[] parts() {
		if (arguments == null) {
			arguments = new ArrayList<StBlockValued<?>>();
		}
		return arguments.toArray(new StBlockValued[arguments.size()]);
	}

	/**
	 * Compute a value for the receiver.
	 * 
	 * @return java.lang.Object
	 * @category private
	 */
	@SuppressWarnings("deprecation")
	public V computeValue() {
		switch (numArgs) {
			case 0:
				return block.value();
			case 1:
				return block.value_(arguments.get(0).value());
			case 2:
				return block.value_value_(arguments.get(0).value(), arguments.get(1).value());
			case 3:
				return block.value_value_value_(arguments.get(0).value(), arguments.get(1).value(), arguments.get(2).value());
		}

		Object[] anArray = new Object[Math.min(numArgs, arguments.size())];
		for (int i = 0; i < anArray.length; i++) {
			anArray[i] = arguments.get(i).value();
		}
		return block.valueWithArguments_(anArray);
	}

	/**
	 * Set the block for the receiver.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @category private
	 */
	protected void setBlock_(StBlockClosure<V, ?, ?, ?> aBlock) {
		block = aBlock;
		numArgs = aBlock.numArgs();
		cachedValue = UnassignedValue;
		arguments = new ArrayList<StBlockValued<?>>();
	}

	/**
	 * Set the receiver's block and the arguments.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param anArray jp.co.sra.smalltalk.StBlockValued[]
	 * @category private
	 */
	protected void setBlock_arguments_(StBlockClosure<V, ?, ?, ?> aBlock, StBlockValued<?>[] anArray) {
		this.setBlock_(aBlock);

		if (anArray != null) {
			for (int i = 0; i < anArray.length; i++) {
				anArray[i].addDependentListener(this);
				arguments.add(anArray[i]);
			}
		}
	}

	/**
	 * Set the receiver's block and the arguments.
	 * 
	 * @param aBlock jp.co.sra.smalltalk.StBlockClosure
	 * @param aCollection java.util.Collection
	 * @category private
	 */
	protected void setBlock_arguments_(StBlockClosure<V, ?, ?, ?> aBlock, Collection<StBlockValued<?>> aCollection) {
		this.setBlock_(aBlock);

		if (aCollection != null) {
			for (Iterator<StBlockValued<?>> i = aCollection.iterator(); i.hasNext();) {
				StBlockValued<?> aValued = i.next();
				aValued.addDependentListener(this);
				arguments.add(aValued);
			}
		}
	}

}
