package jp.co.sra.smalltalk;

import java.util.ArrayList;

/**
 * StObjectTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1998/10/07 (by nisinaka)
 *  @updated   2008/04/14 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StObjectTestExamples.java,v 9.11 2010/03/09 09:18:07 nisinaka Exp $
 */
public class StObjectTestExamples extends SmalltalkTestExamples {

	public static String StObjectTestExamples_variable = "This is a class instance variable";

	/**
	 * The usage of StObject._New().
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1() {
		try {
			String aString = "an instance of String";
			Object[] arguments = { aString };
			String newString = StObject._New(String.class, arguments);

			System.out.println(newString);

			return aString.equals(newString);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * The usage of StObject._ParameterTypesOf().
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2() {
		try {
			Object[] arguments = { "string", new Double(1.0) };
			Class<?>[] parameterTypes = StObject._ParameterTypesOf(arguments);

			return (parameterTypes[0] == String.class) && (parameterTypes[1] == Double.TYPE);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * The usage of StObject._ListOfPossibleParameterTypes().
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3() {
		try {
			Class<?> aClass = String.class;
			Class<?>[] anArray = new Class<?>[] { aClass };
			ArrayList<Class<?>[]> aList = StObject._ListOfPossibleParameterTypes(anArray, aClass);

			if (aList.size() != 4) {
				return false;
			}

			if (aList.get(0).length != 2) {
				return false;
			}

			if (aList.get(0)[0] != String.class) {
				return false;
			}

			if (aList.get(0)[1] != String.class) {
				return false;
			}

			if (aList.get(1).length != 2) {
				return false;
			}

			if (aList.get(1)[0] != String.class) {
				return false;
			}

			if (aList.get(1)[1] != Object.class) {
				return false;
			}

			if (aList.get(2).length != 2) {
				return false;
			}

			if (aList.get(2)[0] != Object.class) {
				return false;
			}

			if (aList.get(2)[1] != String.class) {
				return false;
			}

			if (aList.get(3).length != 2) {
				return false;
			}

			if (aList.get(3)[0] != Object.class) {
				return false;
			}

			if (aList.get(3)[1] != Object.class) {
				return false;
			}

			return true;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * The usage of StObject._ClassInstanceVariable().
	 * 
	 * @return boolean
	 * @throws jp.co.sra.smalltalk.SmalltalkException
	 * @category Examples
	 */
	public static boolean Example4() {
		try {
			Object anObject = StObject._ClassInstanceVariable(StObjectTestExamples.class, "variable");

			return StObjectTestExamples_variable.equals(anObject);
		} catch (Exception e) {
			throw new SmalltalkException(e);
			// return false;
		}
	}

	/**
	 * The usage of StObject._ClassInstanceVariable().
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example5() {
		try {
			StObject._ClassInstanceVariable(StObjectTestExamples.class, "variable", new String("new value"));

			return StObjectTestExamples_variable.equals("new value");
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * The usage of StObject._PerformWithArguments().
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example6() {
		try {
			Object[] arguments = { new Character('A') };
			Character c = (Character) StObject._PerformWithArguments(Character.class, "toLowerCase", arguments);

			return c.charValue() == 'a';
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * The test for copy() method.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings("unchecked")
	public static boolean Example7() {
		try {
			String first = "first";
			String second = "second";
			StValueHolder<String> aValueHolder1 = new StValueHolder<String>(first);
			StValueHolder<String> aValueHolder2 = (StValueHolder<String>) aValueHolder1.copy();

			aValueHolder2.value_(second);

			return (aValueHolder1.value() == first) && (aValueHolder2.value() == second);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * A test for the dependency mechanism.
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example9() {
		try {
			StObject otherObject = new StObject() {
			};

			StObject transient1 = new StObject() {
				public void update_(DependentEvent evt) {
					System.out.println(evt.getAspect().toString());
				}
			};

			StObject transient2 = new StObject() {
				public void update_(DependentEvent evt) {
					System.out.println(evt.getAspect().toString());
				}
			};

			otherObject.addDependent_(transient1);
			otherObject.addDependent_(transient2);
			otherObject.changed_(StObject.$("something"));
			otherObject.removeDependent_(transient1);
			otherObject.removeDependent_(transient2);

			return true;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new StObjectTestExamples();
	}

}
