package jp.co.sra.smalltalk;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.TreeMap;

/**
 * StSortedCollectionTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1998/12/02 (by nisinaka)
 *  @updated   2008/04/14 (by nisinaka)
 *  @updated   2008/04/23 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StSortedCollectionTestExamples.java,v 9.10 2010/03/09 09:18:07 nisinaka Exp $
 */
public class StSortedCollectionTestExamples extends SmalltalkTestExamples {

	/**
	 * Example1: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static final boolean Example1() {
		StBlockClosure sortBlock = new StBlockClosure() {
			public Object value_value_(Object e1, Object e2) {
				return new Boolean(((Number) e1).intValue() <= ((Number) e2).intValue());
			}
		};

		StSortedCollection<Integer> collection = new StSortedCollection<Integer>(sortBlock);
		collection.add_(new Integer(0));
		collection.add_(new Integer(4));
		collection.add_(new Integer(5));
		collection.add_(new Integer(3));
		collection.add_(new Integer(2));
		collection.add_(new Integer(1));

		Object[] array = collection._asArray();
		for (int i = 0; i < array.length; i++) {
			System.out.println(i);

			if (((Number) array[i]).intValue() != i) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Example1: 
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static final boolean Example1g() {
		StBlockClosure2<Boolean, Integer, Integer> sortBlock = new StBlockClosure2<Boolean, Integer, Integer>() {
			public Boolean value(Integer e1, Integer e2) {
				return e1 <= e2;
			}
		};

		StSortedCollection<Integer> collection = new StSortedCollection<Integer>(sortBlock);
		collection.add_(new Integer(0));
		collection.add_(new Integer(4));
		collection.add_(new Integer(5));
		collection.add_(new Integer(3));
		collection.add_(new Integer(2));
		collection.add_(new Integer(1));

		Object[] array = collection._asArray();
		for (int i = 0; i < array.length; i++) {
			System.out.println(i);

			if (((Number) array[i]).intValue() != i) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Example2_double:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2_double() {
		int size = 100000;
		double[] values = new double[size];
		for (int i = 0; i < values.length; i++) {
			values[i] = Math.random();
		}

		// StSortedCollection
		long time = System.currentTimeMillis();
		StSortedCollection<Double> collection = new StSortedCollection<Double>(new StBlockClosure2<Boolean, Double, Double>() {
			public Boolean value(Double e1, Double e2) {
				return e1 <= e2;
			}
		});
		for (int i = 0; i < values.length; i++) {
			collection.add_(new Double(values[i]));
		}
		System.out.println("StSortedCollection          : elapsed " + (System.currentTimeMillis() - time));

		// TreeMap
		time = System.currentTimeMillis();
		TreeMap<Double, Object> aTreeMap = new TreeMap<Double, Object>(new Comparator<Double>() {
			public int compare(Double arg0, Double arg1) {
				double d0 = arg0.doubleValue();
				double d1 = arg1.doubleValue();
				return (d0 == d1) ? 0 : (d0 < d1) ? -1 : 1;
			}
		});
		for (int i = 0; i < values.length; i++) {
			aTreeMap.put(new Double(values[i]), null);
		}
		System.out.println("TreeMap                     : elapsed " + (System.currentTimeMillis() - time));

		// Collections.sort() with ArrayList
		time = System.currentTimeMillis();
		ArrayList<Double> aList = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++) {
			aList.add(new Double(values[i]));
		}
		Collections.sort(aList, new Comparator<Double>() {
			public int compare(Double arg0, Double arg1) {
				double d0 = arg0.doubleValue();
				double d1 = arg1.doubleValue();
				return (d0 == d1) ? 0 : (d0 < d1) ? -1 : 1;
			}
		});
		System.out.println("Collesions.sort() with List : elapsed " + (System.currentTimeMillis() - time));

		// Arrays.sort() with ArrayList
		time = System.currentTimeMillis();
		aList = new ArrayList<Double>();
		for (int i = 0; i < values.length; i++) {
			aList.add(new Double(values[i]));
		}
		Arrays.sort(aList.toArray(new Double[aList.size()]), new Comparator<Double>() {
			public int compare(Double arg0, Double arg1) {
				double d0 = arg0.doubleValue();
				double d1 = arg1.doubleValue();
				return (d0 == d1) ? 0 : (d0 < d1) ? -1 : 1;
			}
		});
		System.out.println("Arrays.sort() with List     : elapsed " + (System.currentTimeMillis() - time));

		// Arrays.sort() with double[]
		time = System.currentTimeMillis();
		double[] sortedValues = new double[values.length];
		for (int i = 0; i < values.length; i++) {
			sortedValues[i] = values[i];
		}
		Arrays.sort(values);
		System.out.println("Arrays.sort() with double[] : elapsed " + (System.currentTimeMillis() - time));

		return true;
	}

	/**
	 * Example3_String:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3_String() {
		int size = 100000;
		String[] values = new String[size];
		for (int i = 0; i < values.length; i++) {
			values[i] = String.valueOf(Math.random());
		}

		// StSortedCollection
		long time = System.currentTimeMillis();
		StSortedCollection<String> collection = new StSortedCollection<String>(new StBlockClosure2<Boolean, String, String>() {
			public Boolean value(String e1, String e2) {
				return new Boolean(e1.compareTo(e2) <= 0);
			}
		});
		for (int i = 0; i < values.length; i++) {
			collection.add_(values[i]);
		}
		System.out.println("StSortedCollection          : elapsed " + (System.currentTimeMillis() - time));

		// TreeMap
		time = System.currentTimeMillis();
		TreeMap<String, Object> aTreeMap = new TreeMap<String, Object>(new Comparator<String>() {
			public int compare(String arg0, String arg1) {
				return arg0.compareTo(arg1);
			}
		});
		for (int i = 0; i < values.length; i++) {
			aTreeMap.put(values[i], null);
		}
		System.out.println("TreeMap                     : elapsed " + (System.currentTimeMillis() - time));

		// Collections.sort() with ArrayList
		time = System.currentTimeMillis();
		ArrayList<String> aList = new ArrayList<String>();
		for (int i = 0; i < values.length; i++) {
			aList.add(values[i]);
		}
		Collections.sort(aList, new Comparator<String>() {
			public int compare(String arg0, String arg1) {
				return arg0.compareTo(arg1);
			}
		});
		System.out.println("Collesions.sort() with List : elapsed " + (System.currentTimeMillis() - time));

		// Arrays.sort() with ArrayList
		time = System.currentTimeMillis();
		aList = new ArrayList<String>();
		for (int i = 0; i < values.length; i++) {
			aList.add(values[i]);
		}
		Arrays.sort(aList.toArray(new String[aList.size()]), new Comparator<String>() {
			public int compare(String arg0, String arg1) {
				return arg0.compareTo(arg1);
			}
		});
		System.out.println("Arrays.sort() with List     : elapsed " + (System.currentTimeMillis() - time));

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		new StSortedCollectionTestExamples();
	}

}
