package jp.co.sra.smalltalk;

import java.io.IOException;
import java.io.Writer;

/**
 * StValueHolder class
 * 
 *  @author    nisinaka
 *  @created   1998/10/09 (by nisinaka)
 *  @updated   2008/04/22 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StValueHolder.java,v 9.11 2010/03/09 09:18:07 nisinaka Exp $
 */
public class StValueHolder<V> extends StValueModel<V> {

	protected V value = null;

	/**
	 * Create a new instance of <code>StValueHolder</code> and initialize it.
	 * 
	 * @category Instance creation
	 */
	public StValueHolder() {
		super();
	}

	/**
	 * Create a new instance of <code>StValueHolder</code> and initialize it with anObject.
	 * 
	 * @param anObject java.lang.Object
	 * @category Instance creation
	 */
	public StValueHolder(V anObject) {
		super();
		this.setValue_(anObject);
	}

	/**
	 * Create a new instance of <code>StValueHolder</code> and initialize it with anObject.
	 * 
	 * @param aClass java.lang.Class
	 * @param anObject java.lang.Object
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category Instance creation
	 * @deprecated since Jun701, use the constructor.
	 */
	@SuppressWarnings("unchecked")
	public static StValueHolder<?> With_(Class<? extends StValueHolder> aClass, Object anObject) {
		return _New(aClass, anObject);
	}

	/**
	 * Create a new instance of <code>StValueHolder</code> and initialize it with anObject.
	 * 
	 * @param anObject java.lang.Object
	 * @return jp.co.sra.smalltalk.StValueHolder
	 * @category Instance creation
	 * @deprecated since Jun701, use the constructor.
	 */
	public static StValueHolder<?> With_(Object anObject) {
		return With_(StValueHolder.class, anObject);
	}

	/**
	 * Answer the currently stored value.
	 * 
	 * @return java.lang.Object
	 * @see jp.co.sra.smalltalk.StValueModel#value()
	 * @category accessing
	 */
	@Override
	public V value() {
		return value;
	}

	/**
	 * Set the currently stored value, without notifying dependents.
	 * 
	 * @param newValue java.lang.Object
	 * @see jp.co.sra.smalltalk.StValueModel#setValue_(java.lang.Object)
	 * @category accessing
	 */
	@Override
	public void setValue_(V newValue) {
		value = newValue;
	}

	/**
	 * Assume 'value' is a Boolean and return it as boolean.
	 * 
	 * @return boolean
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public boolean _booleanValue() {
		return ((Boolean) value).booleanValue();
	}

	/**
	 * Assume 'value' is a kind of Number and return it as double.
	 * 
	 * @return double
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public double _doubleValue() {
		return ((Number) value).doubleValue();
	}

	/**
	 * Assume 'value' is a kind of Number and return it as float.
	 * 
	 * @return float
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public float _floatValue() {
		return ((Number) value).floatValue();
	}

	/**
	 * Assume 'value' is a kind of Number and return it as int.
	 * 
	 * @return int
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public int _intValue() {
		return ((Number) value).intValue();
	}

	/**
	 * Assume 'value' is a kind of Number and return it as long.
	 * 
	 * @return long
	 * @category accessing
	 * @deprecated since Jun701, use Generics.
	 */
	public long _longValue() {
		return ((Number) value).longValue();
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws java.io.IOException
	 * @see jp.co.sra.smalltalk.StObject#printOn_(java.io.Writer)
	 * @category printing
	 */
	@Override
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		if (this.value() == null) {
			aWriter.write("null");
		} else {
			aWriter.write(this.value().toString());
		}
	}

}
