package jp.co.sra.smalltalk;

/**
 * StValueHolderTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1998/10/xx (by nisinaka)
 *  @updated   2008/04/23 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StValueHolderTestExamples.java,v 9.11 2010/03/09 09:18:07 nisinaka Exp $
 */
public class StValueHolderTestExamples extends SmalltalkTestExamples {

	/**
	 * Example1:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings({ "unchecked", "deprecation" })
	public static boolean Example1() {
		final StValueHolder a = new StValueHolder(null);

		a.compute_(new StBlockClosure() {
			public Object value() {
				System.out.println("a = " + a.value().toString());
				return null;
			}
		});
		a.value_("Smalltalk");
		a.value_("Aoki");

		return true;
	}

	/**
	 * Example1g:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example1g() {
		final StValueHolder<String> a = new StValueHolder<String>();

		a.compute_(new StBlockClosure0<Void>() {
			public Void value() {
				System.out.println("a = " + a.value().toString());
				return null;
			}
		});
		a.value_("Smalltalk");
		a.value_("Aoki");

		return true;
	}

	/**
	 * Example2:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings("unchecked")
	public static boolean Example2() {
		final StValueHolder a = new StValueHolder(null);

		a.onChangeSend_to_("value", new StBlockClosure() {
			public Object value() {
				System.out.println("a = " + a.value().toString());

				return null;
			}
		});
		a.value_("Smalltalk");
		a.value_("Aoki");

		return true;
	}

	/**
	 * Example2g:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example2g() {
		final StValueHolder<String> a = new StValueHolder<String>();

		a.onChangeSend_to_("value", new StBlockClosure0<String>() {
			public String value() {
				System.out.println("a = " + a.value().toString());
				return null;
			}
		});
		a.value_("Smalltalk");
		a.value_("Aoki");

		return true;
	}

	/**
	 * Example3:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	@SuppressWarnings( { "unchecked", "deprecation" })
	public static boolean Example3() {
		StValueHolder a = new StValueHolder(1);

		a.compute_(new StBlockClosure() {
			public Object value_(Object v) {
				System.out.println("a = " + v.toString());

				return null;
			}
		});

		StValueHolder b = new StValueHolder(1);

		b.compute_(new StBlockClosure() {
			public Object value_(Object v) {
				System.out.println("b = " + v.toString());

				return null;
			}
		});

		StBlockClosure aBlock = new StBlockClosure() {
			public Object value_value_(Object va, Object vb) {
				return new Integer(((Integer) va).intValue() * ((Integer) vb).intValue());
			}
		};

		StValueHolder[] anArray = { a, b };
		StBlockValue ab = new StBlockValue(aBlock, anArray);

		ab.compute_(new StBlockClosure() {
			public Object value_(Object v) {
				System.out.println("ab = " + v.toString());

				return null;
			}
		});
		a.value_(10);
		b.value_(20);

		return true;
	}

	/**
	 * Example3:
	 * 
	 * @return boolean
	 * @category Examples
	 */
	public static boolean Example3g() {
		StValueHolder<Integer> a = new StValueHolder<Integer>(1);

		a.compute_(new StBlockClosure1<Integer, Integer>() {
			public Integer value(Integer v) {
				System.out.println("a = " + v.toString());
				return null;
			}
		});

		StValueHolder<Integer> b = new StValueHolder<Integer>(1);

		b.compute_(new StBlockClosure1<Integer, Integer>() {
			public Integer value(Integer v) {
				System.out.println("b = " + v.toString());
				return null;
			}
		});

		StBlockClosure2<Integer, Integer, Integer> aBlock = new StBlockClosure2<Integer, Integer, Integer>() {
			public Integer value(Integer va, Integer vb) {
				return va * vb;
			}
		};

		StBlockValue<Integer> ab = new StBlockValue<Integer>(aBlock, a, b);

		ab.compute_(new StBlockClosure1<Integer, Integer>() {
			public Integer value(Integer v) {
				System.out.println("ab = " + v.toString());
				return null;
			}
		});
		a.value_(10);
		b.value_(20);

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 * @category Main
	 */
	public static void main(String[] args) {
		new StValueHolderTestExamples();
	}

}
