package jp.co.sra.smalltalk.menu;

import java.util.ArrayList;

/**
 * StRadioButtonGroup class
 * 
 *  @author    nisinaka
 *  @created   2005/02/18 (by nisinaka)
 *  @updated   2008/04/14 (by nisinaka)
 *  @version   9.7
 *  @copyright 1999-2010 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2010 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StRadioButtonGroup.java,v 9.8 2010/03/09 09:18:08 nisinaka Exp $
 */
public class StRadioButtonGroup extends StAbstractMenuItem {

	protected static final StRadioButtonGroup DefaultGroup = new StRadioButtonGroup();

	protected ArrayList<StRadioButtonMenuItem> buttons;
	protected StRadioButtonMenuItem selectedButton;

	/**
	 * Create a new instance of StRadioButtonGroup and initialize it.
	 *
	 * @category Instance creation
	 */
	public StRadioButtonGroup() {
		buttons = new ArrayList<StRadioButtonMenuItem>();
		selectedButton = null;
	}

	/**
	 * Create a new instance of StRadioButtonGroup and initialize it.
	 *
	 * @param radioButtons jp.co.sra.smalltalk.menu.StRadioButtonMenuItem[]
	 * @category Instance creation
	 */
	public StRadioButtonGroup(StRadioButtonMenuItem[] radioButtons) {
		this();

		for (int i = 0; i < radioButtons.length; i++) {
			radioButtons[i].group_(this);
		}
	}

	/**
	 * Answer my currently selected button.
	 * 
	 * @return jp.co.sra.smalltalk.menu.StRadioButtonMenuItem
	 * @category accessing
	 */
	public StRadioButtonMenuItem selectedButton() {
		return selectedButton;
	}

	/**
	 * Add the specified StRadioButtonMenuItem to the receiver's group.
	 * 
	 * @param aRadioButtonMenuItem jp.co.sra.smalltalk.menu.StRadioButtonMenuItem
	 * @category adding
	 */
	public synchronized void add(StRadioButtonMenuItem aRadioButtonMenuItem) {
		buttons.add(aRadioButtonMenuItem);
	}

	/**
	 * Remove the specified StRadioButtonMenuItem to the receiver's group.
	 * 
	 * @param aRadioButtonMenuItem jp.co.sra.smalltalk.menu.StRadioButtonMenuItem
	 * @category removing
	 */
	public synchronized void remove(StRadioButtonMenuItem aRadioButtonMenuItem) {
		buttons.remove(aRadioButtonMenuItem);
	}

	/**
	 * Select the specified StRadioButtonMenuItem.
	 * 
	 * @param aRadioButtonMenuItem jp.co.sra.smalltalk.menu.StRadioButtonMenuItem
	 * @category selecting
	 */
	public synchronized void select(StRadioButtonMenuItem aRadioButtonMenuItem) {
		if (buttons.contains(aRadioButtonMenuItem) == false) {
			return;
		}
		if (selectedButton == aRadioButtonMenuItem) {
			return;
		}

		if (selectedButton != null) {
			selectedButton.beOff();
		}

		selectedButton = aRadioButtonMenuItem;

		if (selectedButton != null) {
			selectedButton.beOn();
		}
	}

	/**
	 * Answer my menu items.
	 * 
	 * @return java.util.ArrayList
	 * @category menu items
	 */
	public synchronized StMenuItem[] menuItems() {
		return buttons.toArray(new StMenuItem[buttons.size()]);
	}

}